{-# LANGUAGE FlexibleInstances #-}

module Data.Accum
	( Accum(..)
	, getAccum
	) where

import Data.String (IsString(..))

import Data.Bifunctor

data Accum a b = Accum a b
	deriving (Eq, Ord, Show)

instance Functor (Accum a) where
	fmap f (Accum a b) = Accum a (f b)

instance Bifunctor Accum where
	bimap f g (Accum a b) = Accum (f a) (g b)
	first f (Accum a b) = Accum (f a) b
	second g (Accum a b) = Accum a (g b)

instance Monoid a => Applicative (Accum a) where
	pure = Accum mempty
	(Accum a f) <*> (Accum a' b) = Accum (a `mappend` a') (f b)

instance Monoid a => Monad (Accum a) where
	return = Accum mempty
	(Accum a b) >>= f = case f b of
		Accum a' c -> Accum (a `mappend` a') c

instance (Semigroup a, Semigroup b) => Semigroup (Accum a b) where
	Accum a b <> Accum a' b' = Accum (a <> a') (b <> b')

instance (Monoid a, Monoid b) => Monoid (Accum a b) where
	mempty = Accum mempty mempty
{-
instance IsString a => IsString (Accum a b) where
	fromString s = Accum (fromString s) $ error "can't actually use this for values"
-}
instance IsString a => IsString (Accum [a] b) where
	fromString s = Accum (pure . fromString $ s) $ error "can't actually use this for values"


getAccum :: Accum a b -> a
getAccum (Accum a _) = a
