module Description.Body.Animal
	( speciesUsual
	, hasBodyTail
	, hasTail
	, hasWings
	, hasHorns
	, hasSheath
	, hasInternalBalls
	, hasSnout
	, hasMuzzle
	, hasFangs
	, hasTusks
	, hasTopEars
	, ridgedCockhead
	, animalCockSize
	, animalDickLocation
	, animalDickMaterial
	, animalMat

	, partName
	, speciesName
	, animalName

	, animalFootType
	, animalFootWord, animalFeetWord
	, animalHandWord, animalHandsWord
	) where

import Data.Char (toLower)

import Description.TfNode (Animal(..), Part(..))
import Description.Body.Types (DickLocation(..), FootType(..), DickSizeR(..))
import Description.Body.Types.Material (FullMat(..), MatAttr(..), Material(..))

speciesUsual :: Animal -> Part -> Int
speciesUsual a p = case (a, p) of
	(x, Legs)
		| x `elem` [Scorpion] -> 6
		| x `elem` [Wolf, Deer, Goat, Bull, Lion, Tiger, Unicorn, Boar, Dragon] -> 4
		| x `elem` [Human, Chicken, Bat] -> 2
		| otherwise -> 0
	(x, Arms)
		| x `elem` [Human] -> 2
		| otherwise -> 0
	(x, Wings)
		| hasWings x -> 2
		| otherwise -> 0
	(x, Tail)
		| hasTail x -> 1
		| otherwise -> 0
	(x, Horns)
		| x == Unicorn -> 1
		| hasHorns x -> 2
		| otherwise -> 0
	-- i don't forsee actually using these
	(_, Halo) -> 0
	(_, Eyes) -> 2
	_ -> 1

hasBodyTail :: Animal -> Bool
hasBodyTail a = case a of
	Snake -> True
	Fish -> True
	Dragon -> True
	_ -> False

hasTail :: Animal -> Bool
hasTail a = case a of
	Human -> False
	_ -> True

hasWings :: Animal -> Bool
hasWings a = case a of
	Bat -> True
	Chicken -> True
	Dragon -> True
	_ -> False

hasHorns :: Animal -> Bool
hasHorns a = case a of
	Deer -> True
	Goat -> True
	Bull -> True
	Unicorn -> True
	_ -> False

hasSheath :: Animal -> Bool
hasSheath a = case a of
	Deer -> True
	Goat -> True
	Bull -> True
	Unicorn -> True

	Wolf -> True
	Lion -> True
	Tiger -> True
	Boar -> True

	_ -> False

hasInternalBalls :: Animal -> Bool
hasInternalBalls a = case a of
	Snake -> True
	Fish -> True
	Scorpion -> True
	Spider -> True
	Chicken -> True

	_ -> False

hasMuzzle :: Animal -> Bool
hasMuzzle b = b `elem` [Wolf, Deer, Goat, Bull, Lion, Tiger, Dragon, Unicorn]

hasSnout :: Animal -> Bool
hasSnout b = b `elem` [Bat, Boar, Snake]

hasFangs :: Animal -> Bool
hasFangs x = x `elem` [Wolf, Lion, Tiger, Dragon, Snake]

hasTusks :: Animal -> Bool
hasTusks x = x `elem` [Boar]

hasTopEars :: Animal -> Bool
hasTopEars x = not $ x `elem` [Dragon, Scorpion, Spider, Chicken, Human, Fish, Snake]

partName :: Part -> String
partName p = toLower <$> show p

speciesName :: Animal -> String
speciesName a = toLower <$> show a

animalName :: Animal -> String
animalName a = toLower <$> show a

animalCockSize :: Animal -> DickSizeR
animalCockSize x = case x of
	Wolf -> MidThigh
	Deer -> MidThigh
	Goat -> MidThigh
	Bull -> ToKnee
	Lion -> Short
	Tiger -> Short
	Dragon -> ToKnee
	Unicorn -> ToKnee
	Scorpion -> Short
	Spider -> Short
	Chicken -> Short
	Dove -> Short
	Raven -> Short
	Bat -> Short
	Boar -> MidThigh
	Human -> Average
	Fish -> Short
	Snake -> Average
	Octopus -> MidThigh
	Mantis -> Short
	Snakeheaded -> MidThigh

ridgedCockhead :: Animal -> Bool
ridgedCockhead x = if x `elem` [Wolf, Deer, Bull, Bat]
	then False
	else True

animalDickMaterial :: Animal -> FullMat
animalDickMaterial a = case a of
	Goat -> FMat PalePink Flesh
	Unicorn -> FMat Ebony Flesh
	Wolf -> FMat BloodyRed Flesh
	Deer -> FMat RawRed Flesh
	Bull -> FMat RichPurple Flesh
	--Horse -> FMat MottledPinkAndBrown Flesh
	Lion -> FMat Pink Flesh
	Tiger -> FMat Pink Flesh
	Dragon -> FMat MottledPurple Flesh
	Scorpion -> FMat PurpleBlack Flesh
	Spider -> FMat PurpleBlack Flesh
	Chicken -> FMat Pink Flesh
	Bat -> FMat PurpleBlack Flesh
	Boar -> FMat Pink Flesh
	Human -> FMat RuddyRedBrown Skin
	Fish -> FMat BlotchyPurple Flesh
	Snake -> FMat BlotchyPurple Flesh

	Dove -> FMat Pink Flesh
	Raven -> FMat Pink Flesh
	Octopus -> error "mottled color-changing"
	Mantis -> error "mantis dick color"
	Snakeheaded -> FMat PatternedGreenYellow Scales

-- FIXME: this would hypothetically depend on the part; a rat tail isn't covered in fur; bird hands and feet aren't covered in feathers; bee wings aren't covered in chitin; etc
animalMat :: Maybe Animal -> FullMat
animalMat Nothing = FMat Nil Protoplasm
animalMat (Just a) = case a of
	Human -> FMat RuddyRedBrown Skin
	Wolf -> FMat CoarseGrey Fur
	Deer -> FMat DappledBrownAndTan Coat
	Goat -> FMat CoarseBrown Fur
	Bull -> FMat StiffGrey Coat
	--Horse -> FMat ShiningBrown Coat
	Lion -> FMat Golden Fur
	Tiger -> FMat StripedOrangeBlack Fur
	Dragon -> FMat GemlikeRed Scales
	Unicorn -> FMat ShiningSilver Coat
	Scorpion -> FMat ShiningBlack Chitin
	Spider -> FMat FuzzyBlack Chitin
	Chicken -> FMat White Feathers
	Bat -> FMat ShortGrey Fur
	Boar -> FMat BristlyBrown Coat
	Fish -> FMat ShiningBlue Scales
	Snake -> FMat PatternedGreenYellow Scales

	Dove -> FMat White Feathers
	Raven -> FMat Black Feathers
	Octopus -> FMat (error "octopus mat") (error "octopus mat")
	Mantis -> FMat (error "mantis mat") (error "mantis mat")
	Snakeheaded -> FMat PatternedGreenYellow Scales

animalFootType :: Animal -> FootType
animalFootType a = case a of
	Wolf -> Paws
	Deer -> ClovenHooves
	Goat -> ClovenHooves
	Bull -> ClovenHooves
	Lion -> Paws
	Tiger -> Paws
	Dragon -> Talons
	Unicorn -> Hooves
	Scorpion -> Talons -- ?
	Spider -> Talons -- ?
	Chicken -> Talons
	Bat -> Talons
	Boar -> ClovenHooves
	Human -> Feet
	-- these would not naturally have feet :V
	Fish -> Feet
	Snake -> Feet

	Octopus -> error "octopus feet"
	Dove -> Talons
	Raven -> Talons
	Mantis -> Talons -- ?
	Snakeheaded -> Feet

animalDickLocation :: Animal -> DickLocation
animalDickLocation a = case a of
	Human -> DickExternal
	Wolf -> DickSheathed
	Deer -> DickSheathed
	Goat -> DickSheathed
	Bull -> DickSheathed
	Lion -> DickSheathed
	Tiger -> DickSheathed
	Dragon -> DickVent
	Unicorn -> DickSheathed
	Scorpion -> DickVent
	Spider -> DickVent
	Chicken -> DickCloaca
	Bat -> DickSheathed
	Boar -> DickSheathed
	Fish -> DickVent
	Snake -> DickCloaca

	Octopus -> DickExternal
	Dove -> DickCloaca
	Raven -> DickCloaca
	Mantis -> error "mantis dick"
	Snakeheaded -> DickExternal

animalHandWord :: Animal -> String
animalHandWord x = case x of
	Wolf -> "paw"
	Lion -> "paw"
	Tiger -> "paw"
	Dragon -> "claw"
	Scorpion -> "talon" -- ?
	Spider -> "talon" -- ?
	Chicken -> "claw"
	Bat -> "claw"
	_ -> "hand"

animalHandsWord :: Animal -> String
animalHandsWord x = case x of
	Wolf -> "paws"
	Lion -> "paws"
	Tiger -> "paws"
	Dragon -> "claws"
	Scorpion -> "talons" -- ?
	Spider -> "talons" -- ?
	Chicken -> "claws"
	Bat -> "claws"
	_ -> "hands"

animalFootWord :: Animal -> String
animalFootWord x = case x of
	Wolf -> "paw"
	Deer -> "hoof"
	Goat -> "hoof"
	Bull -> "hoof"
	Lion -> "paw"
	Tiger -> "paw"
	Dragon -> "claw"
	Unicorn -> "hoof"
	Scorpion -> "talon" -- ?
	Spider -> "talon" -- ?
	Chicken -> "claw"
	Bat -> "claw"
	Boar -> "hoof"
	Human -> "foot"
	Fish -> "foot"
	Snake -> "foot"

	Dove -> "talon" -- ?
	Raven -> "talon"
	Octopus -> "tentacle" -- ?
	Mantis -> "talon" -- ?
	Snakeheaded -> error "snakeheaded foot"

animalFeetWord :: Animal -> String
animalFeetWord x = case x of
	Wolf -> "paws"
	Deer -> "hooves"
	Goat -> "hooves"
	Bull -> "hooves"
	Lion -> "paws"
	Tiger -> "paws"
	Dragon -> "claws"
	Unicorn -> "hooves"
	Scorpion -> "talons" -- ?
	Spider -> "talons" -- ?
	Chicken -> "claws"
	Bat -> "claws"
	Boar -> "hooves"
	Human -> "feet"
	Fish -> "feet"
	Snake -> "feet"

	Dove -> "talons" -- ?
	Raven -> "talons"
	Octopus -> "tentacles" -- ?
	Mantis -> "talons" -- ?
	Snakeheaded -> error "snakeheaded feet"
