module Description.Body.Sample
	( bodyMat
	, bodyMat_
	, bodyMat__
	, distinctMaterialSections
	, limbMaterials
	, majorLimbs
	, bodyCount
	, legOrBodySpecies
	, lowerBodySpecies
	, locoplan
	, hasLocotype
	, getBalls
	, getDick
	, ballProduction
	, BallProduction(..)
	, animalAss
	, armCountMatching
	, legCount
	, armCount
	, FootType(..)
	, footType
	, animalFootType
	, foot
	, feet
	, hand
	, hands
	, cumtype
	, hasMuzzle
	, hasSnout
	, hasFangs
	, hasTusks
	, hasTopEars
	, hasTail
	, hasHead
	, hasHorns
	, hasMouth
	, hasArms
	, hasLegs
	, cocksizeO
	, cocksizeR
	, DickSizeR(..)
	, DickSizeO(..)
	, hasKnot
	, hasClimaxAttr
	, hasDickAttr
	, hasFlare
	, hasMedial
	, hasBarbs
	, hasBaculum
	, hasDickType
	, ridgedCockhead
	, cockCount
	, manyCocks
	, oneCock
	, animalDickLocation
	, hasProstate
	, hasGSpot
	, hasBalls
	, hasExternalBalls
	, hasInternalBalls
	, hasDick
	, dickLocation
	, (?)
	, animalName

	, gender
	, Gender(He, She, It, They)

	, animalCockSize

	, Locatable(bodyCovering, bodySpecies)
	, MiscPart(..)

	, MouthType(..)
	, mouthType

	, HandType(..)
	, handType

	, FrontGenitalia(..)
	, frontGenitalia

	, DickAttr(..)
	, dickAttrs

	, ClimaxAttr(..)
	, climaxAttrs
	, firstOf

	, BallLocation(..)

	, hasAsshole
	, hasCloaca
	, hasNagaCloaca
	, primaryFuckableCloaca
	, hasCloacaDick

	, module TYPES
	) where

import Prelude hiding (head, tail)

import Data.Maybe (fromMaybe, catMaybes, mapMaybe)
import Data.String (IsString(..))
import Data.Set (Set)
import qualified Data.Set as Set

import Description.TfNode (Animal(..))
import qualified Description.TfNode as TF

import Utility.Misc (collate)
import Utility.Tuple (snd_, thd_)
import Description.Body.Types as TYPES hiding (ballLocation)
import qualified Description.Body.Types as TYPES (ballLocation)
-- import Description.Body.Types.Material
import Description.Pronoun (Gender(..))
import Description.Body.Animal (hasSnout, hasMuzzle, hasFangs, hasTusks, hasTopEars, ridgedCockhead, animalCockSize, animalDickLocation, animalFootType, animalFootWord, animalFeetWord, animalHandWord, animalHandsWord, animalName)

-- "Maybe" b/c we might not even have the locations in question, e.g., wings, tails, etc; if we have it we certainly have the covering
bodyMat :: BodyLocatable b => b -> BodyGlob -> Maybe BodyCovering
bodyMat l b = case loc l of
	Bas bas ->
		let
			lvals l = case l of
				Head -> fmap snd_ . head
				Chest -> pure . snd_ . chest
				Arms -> fmap snd_ . arms
				Legs -> fmap snd_ . legs
				Tail -> fmap snd_ . tail
				Wings -> fmap snd_ . wings
				Genitals -> fmap snd_ . genitalia
		in Solid <$> lvals bas (limbs . bodyplan $ b)
	Tra tra -> case lookup tra (transition . bodyplan $ b) of
		Just c -> Just c
		Nothing -> bodyMat (asBase tra) b

-- this is kind of a mess (BodyTransLocation can't be Locatable b/c Locatable's `bodyCovering` only returns a FullMat, not a BodyCovering), but then also MiscPart can't really be a BodyLocatable. or maybe it can? either way it's kind of a hassle.
bodyMat__ :: Locatable b => b -> BodyGlob -> BodyCovering
bodyMat__ l b = case bodyCovering l b of
	Just (_, mat, _) -> Solid mat
	Nothing -> Solid $ FMat Nil Protoplasm

bodyMat_ :: BodyLocatable b => b -> BodyGlob -> BodyCovering
bodyMat_ l b = fromMaybe (Solid $ FMat Nil Protoplasm) $ case loc l of
	Bas bas ->
		let
			lvals l = case l of
				Head -> fmap snd_ . head
				Chest -> pure . snd_ . chest
				Arms -> fmap snd_ . arms
				Legs -> fmap snd_ . legs
				Tail -> fmap snd_ . tail
				Wings -> fmap snd_ . wings
				Genitals -> fmap snd_ . genitalia
		in Solid <$> lvals bas (limbs . bodyplan $ b)
	Tra tra -> case lookup tra (transition . bodyplan $ b) of
		Just c -> Just c
		Nothing -> bodyMat (asBase tra) b

asBase :: BodyTransLocation -> BodyLocation
asBase Neck = Head
asBase (Waist Fore) = Chest
asBase (Waist Hind) = Chest
asBase Shoulders = Chest
asBase Crotch = Legs
asBase Back = Chest
asBase LowerBack = Legs

distinctMaterialSections :: BodyGlob -> [(FullMat, [BodyLocation])]
distinctMaterialSections b = collate $ fmap (\(a, b) -> (b, a)) $ limbMaterials b

limbMaterials :: BodyGlob -> [(BodyLocation, FullMat)]
limbMaterials b = catMaybes $ (\p b -> case bodyMat p b of
	Just (Solid fmat) -> Just (p, fmat)
	_ -> Nothing) <$> [Head, Chest, Arms, Legs, Tail, Wings, Genitals] <*> pure b

hasArms :: BodyGlob -> Bool
hasArms b = case locoplan b of
	Basic x
		| x `elem` [Biped, Centaur, Naga] -> True
	_ -> False

hasLegs :: BodyGlob -> Bool
hasLegs b = case locoplan b of
	Basic x | x `elem` [Slithering, Naga, Formless] -> False
	_ -> True

armCountMatching :: Int -> BodyGlob -> Bool
armCountMatching i b = i <= armCount b

hasHorns :: BodyGlob -> Bool
hasHorns b = case hornState . _fakedatas $ b of
	Nothing -> False
	Just horns -> hornCount horns > 0

hasHead :: BodyGlob -> Bool
hasHead b = case lookup TF.Head $ collate $ _unusuals b of
	Just us
		| Quantity Missing `elem` us -> False
	_ -> True

hasTail :: BodyGlob -> Bool
hasTail b = case tail . limbs . bodyplan $ b of
	Just (c, _, _) | c > 0 -> True
	_ -> False

legCount :: BodyGlob -> Int
legCount b = case legs . limbs . bodyplan $ b of
	Nothing -> 0
	Just (c, _, _) -> c

armCount :: BodyGlob -> Int
armCount b = case arms . limbs . bodyplan $ b of
	Nothing -> 0
	Just (c, _, _) -> c

hasMouth :: BodyGlob -> Bool
hasMouth = hasHead

getBalls :: BodyGlob -> Maybe BallState
getBalls body = case dickState . _fakedatas $ body of
	BallsOnly balls -> Just balls
	HasDick _ balls -> balls
	_ -> Nothing

getDick :: BodyGlob -> Maybe Dick
getDick body = case dickState . _fakedatas $ body of
	HasDick dick _ -> Just dick
	_ -> Nothing


{-
-- FIXME: listen i don't want to CoC this up and have "ball volume" but this is it for now. later i should have a 'production' value, somehow. for cum and pre? that would probably be overdoing it by a lot.
ballProduction :: BodyGlob -> Int
ballProduction b = case dickState . _fakedatas $ b of
	HasDick _ (Just balls) -> ballCount balls * ballSize balls
	BallsOnly balls -> ballCount balls * ballSize balls
	_ -> 0
-}

data BallProduction = ProstateOnly | Base | Quant Int
	deriving (Eq, Ord, Show, Read)

ballProduction :: BodyGlob -> Maybe BallProduction
ballProduction b = case dickState . _fakedatas $ b of
	HasDick _ Nothing -> Just ProstateOnly
	BallsOnly balls -> Just $ ballQuant balls
	HasDick _ (Just balls) -> Just $ ballQuant balls
	_ -> Nothing
	where
		ballQuant :: BallState -> BallProduction
		ballQuant balls = case (ballCount balls, ballSize balls) of
			(2, 1) -> Base
			(q, s) -> Quant $ (q * s) - 2 -- on a scale from 1 to uh 30

cumtype :: BodyGlob -> Maybe Fluid
cumtype b = case dickState . _fakedatas $ b of
	HasDick _ (Just b) -> Just $ ballFluid b
	BallsOnly b -> Just $ ballFluid b
	_ -> Nothing

locoplan :: BodyGlob -> LocomotionPlan
locoplan = planLocomotion . bodyplan

hasLocotype :: Locomotion -> BodyGlob -> Bool
hasLocotype l b = case locoplan b of
	Basic l' -> l == l'
	_ -> False

hasDickType :: DickLocation -> BodyGlob -> Bool
hasDickType dl b = case dickLocation b of
	Just dl' -> dl' == dl
	_ -> False

{-----
 - body-polling functions, to extract information from the raw tree. these functions don't have to be and probably shouldn't be in this file longterm.
 -}

(?) :: Maybe a -> a -> a
Nothing ? a = a
Just a ? _ = a

hasDick :: BodyGlob -> Bool
hasDick b = case dickState . _fakedatas $ b of
	HasDick {} -> True
	_ -> False

dickLocation :: BodyGlob -> Maybe DickLocation
dickLocation b = case dickState . _fakedatas $ b of
	HasDick dick _ -> Just $ dickPlacement dick
	_ -> Nothing

hasExternalBalls :: BodyGlob -> Bool
hasExternalBalls b = case dickState . _fakedatas $ b of
	HasDick _ (Just b) -> TYPES.ballLocation b == External
	BallsOnly b -> TYPES.ballLocation b == External
	_ -> False

hasInternalBalls :: BodyGlob -> Bool
hasInternalBalls b = hasBalls b && not (hasExternalBalls b)

hasBalls :: BodyGlob -> Bool
hasBalls b = case dickState . _fakedatas $ b of
	HasDick _ (Just _) -> True
	BallsOnly _ -> True
	_ -> False
{-
ballLocation :: BodyGlob -> Maybe BallLocation
ballLocation body = case dickState . _fakedatas $ body of
	HasDick _ b -> Just $ TYPES.ballLocation b
	BallsOnly b -> Just $ TYPES.ballLocation b
	_ -> Nothing
-}

hasProstate :: BodyGlob -> Bool
hasProstate b = hasBalls b || hasDick b

hasGSpot :: BodyGlob -> Bool
hasGSpot b = hasNagaCloaca b && not (hasProstate b)

hasNagaCloaca :: BodyGlob -> Bool
hasNagaCloaca b = (hasLocotype Naga b || hasLocotype Slithering b) && hasCloaca b

-- should use cloaca splices for anal sex
primaryFuckableCloaca :: BodyGlob -> Bool
primaryFuckableCloaca = hasNagaCloaca

hasCloacaDick :: BodyGlob -> Bool
hasCloacaDick b = case dickLocation b of
	Just DickExternal -> False
	Just DickSheathed -> False
	Just _ -> True
	_ -> False

data FrontGenitalia
	= DickPart Dick (Maybe BallState) -- or w/e
	| EmptyCloaca
--	| FrontNagaAsshole
	deriving (Eq, Ord, Show)

frontGenitalia :: BodyGlob -> Maybe FrontGenitalia
frontGenitalia b = case dickState . _fakedatas $ b of
	HasDick dick mballs -> Just $ DickPart dick mballs
	_ -> if hasCloaca b
		then Just EmptyCloaca
		else case locoplan b of
			Basic Naga -> Nothing --Just FrontNagaAsshole -- the thing is i think this is categorically unreachable: a naga without a dick will have a cloaca, and a naga with a dick will have an asshole, so there would be no code paths to return this.
			_ -> Nothing

cockCount :: BodyGlob -> Int
cockCount b = case dickState . _fakedatas $ b of
	HasDick dick _ -> dickCount dick
	_ -> 0

manyCocks :: BodyGlob -> Bool
manyCocks b = cockCount b > 1

oneCock :: BodyGlob -> Bool
oneCock = (== 1) . cockCount

hasKnot :: BodyGlob -> Bool
hasKnot b = case dickState . _fakedatas $ b of
	HasDick dick _ -> case dickKnot dick of
		Nothing -> False
		Just _ -> True
	_ -> False

hasClimaxAttr :: ClimaxAttr -> BodyGlob -> Bool
hasClimaxAttr a b = a `Set.member` climaxAttrs b

hasDickAttr :: DickAttr -> BodyGlob -> Bool
hasDickAttr a b = a `Set.member` dickAttrs b

hasFlare :: BodyGlob -> Bool
hasFlare b = Flaring `Set.member` climaxAttrs b

hasMedial :: BodyGlob -> Bool
hasMedial b = Medial `Set.member` dickAttrs b

hasBarbs :: BodyGlob -> Bool
hasBarbs b = Barbs `Set.member` dickAttrs b

hasBaculum :: BodyGlob -> Bool
hasBaculum b = case bodySpecies Genitals b of
	Just x -> animalBaculum x
	Nothing -> False
	where
		animalBaculum a = case a of
			Human -> False
			Deer -> False
			Unicorn -> False
			Boar -> False
			_ -> True

cocksizeR :: BodyGlob -> Maybe DickSizeR
cocksizeR b = case dickState . _fakedatas $ b of
	HasDick dick _ -> Just $ relativeSize dick
	_ -> Nothing

cocksizeO :: BodyGlob -> Maybe DickSizeO
cocksizeO b = case dickState . _fakedatas $ b of
	HasDick dick _ -> Just $ relativeToObjective $ relativeSize dick
	_ -> Nothing
	where
		relativeToObjective :: DickSizeR -> DickSizeO
		relativeToObjective = toEnum . (+ advance) . fromEnum
		advance = bodyHeightUnusual b

gender :: BodyGlob -> Gender
gender b = nounGender b

legOrBodySpecies :: Maybe NounSpec -> BodyPlan -> Maybe Animal
legOrBodySpecies ns plan = case legSpec of
	Just _ -> legSpec
	Nothing -> bodySpec
	where
		legSpec :: Maybe Animal
		legSpec = case planLocomotion plan of
			x | x `elem` [Basic Naga, Basic Slithering] -> thd_ =<< (tail $ limbs $ plan)
			_ -> thd_ =<< (legs $ limbs $ plan)
		bodySpec :: Maybe Animal
		bodySpec = thd_ $ chest $ limbs $ plan

lowerBodySpecies :: BodyGlob -> Maybe Animal
lowerBodySpecies b = case locoplan b of
	x
		| x `elem` [Basic Biped, Basic Quadruped, Basic Centaur, Basic Wyvern] -> thd_ =<< (legs $ limbs $ bodyplan b)
		| x `elem` [Basic Naga, Basic Slithering] -> thd_ =<< (tail $ limbs $ bodyplan b)
	_ -> Nothing

animalAss :: BodyGlob -> Maybe Animal
animalAss b = legOrBodySpecies (specNoun b) (bodyplan b)

footType :: BodyGlob -> FootType
footType b = case legOrBodySpecies (specNoun b) (bodyplan b) of
	Nothing -> Pseudopods
	Just a -> animalFootType a

foot :: IsString s => BodyGlob -> s
foot b = fromString $ case legOrBodySpecies (specNoun b) (bodyplan b) of
	Nothing -> "foot"
	Just x -> animalFootWord x

feet :: IsString s => BodyGlob -> s
feet b = fromString $ case legOrBodySpecies (specNoun b) (bodyplan b) of
	Nothing -> "feet"
	Just x -> animalFeetWord x

hand :: IsString s => BodyGlob -> s
hand b = fromString $ case bodySpecies Arms b of
	Nothing -> "hand"
	Just x -> animalHandWord x

hands :: IsString s => BodyGlob -> s
hands b = fromString $ case bodySpecies Arms b of
	Nothing -> "hands"
	Just x -> animalHandsWord x

class Show a => Locatable a where
	bodyCovering :: a -> BodyGlob -> Maybe (Int, FullMat, Maybe Animal)
	bodySpecies :: a -> BodyGlob -> Maybe Animal
	bodySpecies a body = (\(x, y, z) -> z) =<< bodyCovering a body

bodyCount :: Locatable a => a -> BodyGlob -> Int
bodyCount a body = case bodyCovering a body of
	Nothing -> 0
	Just (c, _, _) -> c

majorLimbs :: BodyGlob -> [BodyLocation]
majorLimbs body = limb `mapMaybe` [Head, Arms, Legs, Tail, Wings, Genitals]
	where
		limb l = ifNotZero (bodyCount l body) l
		ifNotZero n h = if n == 0 then Nothing else Just h

instance Locatable BodyLocation where
	bodyCovering Head = head . limbs . bodyplan
	bodyCovering Chest = Just . chest . limbs . bodyplan
	bodyCovering Arms = arms . limbs . bodyplan
	bodyCovering Legs = legs . limbs . bodyplan
	bodyCovering Tail = tail . limbs . bodyplan
	bodyCovering Wings = wings . limbs . bodyplan
	bodyCovering Genitals = genitalia . limbs . bodyplan

instance Locatable TF.Sex where
	bodyCovering TF.Cock = genitalia . limbs . bodyplan
	bodyCovering TF.Balls = genitalia . limbs . bodyplan
	-- i don't think this will ever come up, but a knot always counts as a wolf (unless there are no species tags at all)
	bodyCovering TF.Knot = (\body -> if hasKnot body
		then fmap (\(x, y, z) -> (x, y, const Wolf <$> z)) . genitalia . limbs . bodyplan $ body
		else Nothing)
	-- maybe these aren't precisely accurate
	bodyCovering TF.Mouth = head . limbs . bodyplan
	bodyCovering TF.Tongue = head . limbs . bodyplan
	-- breasts, flower, not rly supported yet
	bodyCovering _ = const Nothing

-- like nearly all of these are exclusively in the clayman scene
data MiscPart = Underbelly | Belly | Shaft | Cockhead | Thighs | Sheath | Ass | Teeth
	deriving (Eq, Ord, Enum, Bounded, Show, Read)

instance Locatable MiscPart where
	bodyCovering Underbelly body = fmap (\(x, y, z) -> (1, y, z)) $ (case locoplan body of
		Basic Naga -> tail . limbs . bodyplan $ body
		Basic Slithering -> tail . limbs . bodyplan $ body
		Basic Quadruped -> Just . chest . limbs . bodyplan $ body
		Basic Centaur -> legs . limbs . bodyplan $ body
		_ -> Nothing)
	bodyCovering Belly body = Just . chest . limbs . bodyplan $ body
	bodyCovering Thighs body = legs . limbs . bodyplan $ body
	bodyCovering Sheath body = if dickLocation body == Just DickSheathed
		then fmap (\(x, y, z) -> (1, y, z)) . genitalia . limbs . bodyplan $ body
		else Nothing
	bodyCovering Ass body = case locoplan body of
		Basic Naga -> tail . limbs . bodyplan $ body
		_ -> legs . limbs . bodyplan $ body
	-- the materials for these are gonna be wrong
	bodyCovering Shaft body = genitalia . limbs . bodyplan $ body
	bodyCovering Cockhead body = genitalia . limbs . bodyplan $ body
	bodyCovering Teeth body = head . limbs . bodyplan $ body

data MouthType = Mouth | Muzzle | Snout
	deriving (Eq, Ord, Enum, Bounded, Show, Read)

mouthType :: BodyGlob -> MouthType
mouthType b = case bodySpecies Head b of
	Just x | hasMuzzle x -> Muzzle
		| hasSnout x -> Snout
	_ -> Mouth

data HandType = Claw | Hand | Paw
	deriving (Eq, Ord, Enum, Bounded, Show, Read)

handType :: BodyGlob -> HandType
handType b = case bodySpecies Arms b of
	Nothing -> Hand
	Just x
		| x `elem` [Wolf, Lion, Tiger] -> Paw
		| x `elem` [Scorpion, Spider, Dragon] -> Claw
		{- other hand types conceptually:
			* a wyvern with a wing-hand
			* a not-quite-hoof hand w/ hard nails ([Deer, Goat, Bull, Horse, Unicorn, Boar])
			it's not clear if those would ever be useful
		-}
		| otherwise -> Hand


dickAttrs :: BodyGlob -> Set DickAttr
dickAttrs = (\d -> case d of
	HasDick dick _ -> dickAttr dick
	_ -> mempty) . dickState . _fakedatas

climaxAttrs :: BodyGlob -> Set ClimaxAttr
climaxAttrs = (\d -> case d of
	HasDick dick _ -> dickClimax dick
	_ -> mempty) . dickState . _fakedatas

firstOf :: Ord a => [a] -> (BodyGlob -> Set a) -> BodyGlob -> Maybe a
firstOf attrs f b = foldr (\n x -> case x of
	Just x -> Just x
	Nothing -> if n `Set.member` f b then Just n else Nothing) Nothing
	$ attrs

-- most snakes/nagas have cloacas but some have a separate asshole (if they don't have a cloacal dick)
hasAsshole :: BodyGlob -> Bool
hasAsshole b = case locoplan b of
	Basic Slithering -> case dickLocation b of
		Just DickExternal -> True
		Just DickSheathed -> True
		_ -> False
	Basic Naga -> case dickLocation b of
		Just DickExternal -> True
		Just DickSheathed -> True
		_ -> False
	_ -> True

-- FIXME: this is not, strictly-speaking, correct
hasCloaca :: BodyGlob -> Bool
hasCloaca b = case locoplan b of
	-- for slithering/nagas you either have an asshole (on the front) or cloaca
	Basic x | x `elem` [Slithering, Naga] -> not $ hasAsshole b
	-- for other bodytypes, if you have a cloaca dick then you have a cloaca (duh)
	_ -> case dickLocation b of
		Just DickVent -> True
		Just DickCloaca -> True
		Just _ -> False
		-- and if you don't have a dick then you have a cloaca if yr legs species would give you a cloaca
		Nothing -> case bodySpecies Legs b of
			Nothing -> False
			Just a -> animalDickLocation a `elem` [DickCloaca, DickVent]
