module Description.Body.Types
	( BodyGlob(..)

	, BodyPlan(..)
	, Locomotion(..)
	, LocomotionPlan(..)
	, LocomotionStrange(..)
	, SpeciesType(..)
	, NounSpec(..)

	, HeightVal(..)

	, module M
	, BasicUnusual(..)
	, Quantity(..)

	, FauxGlob(..)
	, SatyrType(..)
	, HeadType(..)

	, HornState(..)
	, HornType(..)

	, DickState(..)
	, Dick(..)
	, DickLocation(..)
	, DickAttr(..)
	, ClimaxAttr(..)
	, KnotState(..)
	, KnotAttr(..)
	, KnotUnsheathSize(..)
	, DickSizeR(..)
	, DickSizeO(..)
	, UnsheathSize(..)
	, BallState(..)
	, BallLocation(..)
	, Fluid(..)

	, FootType(..)
	) where

import Data.Set

import Description.Pronoun (Gender(..))
import Description.TfNode (Animal(..), Element)
import qualified Description.TfNode as TF (Part(..))

import Description.Body.Types.Material as M

-- this is all very preliminary and unfinished
-- this is also the datatype we're gonna be using in scenes, so we want a fairly robust set of querying functions, and we want to store data to make querying easy.
-- things to store: material types for each major bodypart
-- materialStore :: BodyLimb -> Material
data BodyGlob = BodyGlob
	{ pronounGender :: Gender -- usually YOU
	, nounGender :: Gender -- the actual pronoun yr looking for
	, _unusuals :: [(TF.Part, BasicUnusual)]
	, _fakedatas :: FauxGlob
	, bodyHeight :: HeightVal
	, bodyHeightUnusual :: Int

	, bodyplan :: BodyPlan
	, specNoun :: Maybe NounSpec
	}
	deriving (Show)

data BodyPlan = Plan
	{ planLocomotion :: LocomotionPlan
	, transition :: [(BodyTransLocation, BodyCovering)]
	, limbs :: Covering (Int, FullMat, Maybe Animal)
	}
	deriving (Eq, Ord, Show)


-- quadruped : centaur :: slither : naga
-- multi-legged bugs like centipedes or arachnids are still filed under "quadruped", and their associated "+ human torso" forms are filed under "centaur". birds and bats (and wyverns) are filed under "wyvern"; otherwise wings don't effect bodytype, so a dragon would just be 'quadruped'. this taxonomy is constructed by fiat without a lot of thought; if it turns out it's useful to divide things differently, well, do that.
data Locomotion = Quadruped | Biped | Centaur | Slithering | Naga | Wyvern | Formless | Complex
	deriving (Eq, Ord, Enum, Bounded, Show, Read)

data LocomotionPlan = Basic Locomotion | Spec LocomotionStrange
	deriving (Eq, Ord, Show)

data LocomotionStrange = Orb | FloatingHead | FloatingDick
	deriving (Eq, Ord, Show)

data SpeciesType = Protoplasmic | Certainly Animal
	deriving (Eq, Ord, Show)

 -- Gryphon Hippogryph Ugallu Shedu etc
data NounSpec
	= Satyr | Minotaur | Sileni Animal
	| SplitTaur Animal Animal
	| SplitNaga Animal Animal
	| Chimera
	| Polyhedra | Torus
	deriving (Eq, Ord, Show, Read)




data HeightVal
	= Floating -- orbs, floating heads, floating dicks
	| SmallAnimal -- 4-legged goat, wolf, deer, protoplasm; all snakes/worms; protoplasmic wyvern
	| HorsishAnimal -- 4-legged all other animals; bats and chickens
	| HumanHeight -- human bipeds
	| TallerThanHuman -- all furries
		-- TallerThanHuman is skipped in the [LARGE] procession
	| CentaurHeight -- all centaurs; dragons & wyverns; bipeds w/ a [LARGE] node
	| Ogre -- any centaur/dragon w/ a [LARGE] node; bipeds w/ a [LARGE]x2
	| Giant -- any centaur/dragon w/ a [LARGE]x2 node; bipeds w/ a [LARGE]x3
	| Titan -- any centaur/dragon w/ a [LARGE]x3 node
	deriving (Eq, Ord, Enum, Bounded, Show, Read)



data Quantity = Missing | Q Int
	deriving (Eq, Ord, Show, Read)




data BasicUnusual
	= Quantity Quantity
	| Species Animal
	deriving (Eq, Ord, Show)



data FauxGlob = FauxGlob
	{ satyrType :: SatyrType
	, bestial :: Bool
	, headStatus :: Set HeadType
	, ichorTongue :: Bool
	, haloType :: Maybe (Maybe Element)
	, dickState :: DickState
	, hornState :: Maybe HornState
	, bodyHorns :: Bool
	}
	deriving (Show)

data SatyrType = NoSatyr | BaseSatyr | IchorSatyr
	deriving (Eq, Ord, Enum, Bounded, Show)

data HeadType = Blendec | MetalHelmet | FlameHead | FacelessVoid | GeodeHead
	deriving (Eq, Ord, Enum, Bounded, Show)



data HornState = HornState
	{ hornType :: HornType
	, hornMaterial :: FullMat
	, hornCount :: Int
	, hornSize :: Int
	, hornMismatch :: Bool
	}
	deriving (Eq, Ord, Show, Read)

data HornType = Nubby | Tined | Curled | Prongs | Pointed | Tentacled
	deriving (Eq, Ord, Enum, Show, Read)

data DickState
	= NoDick
	| BallsOnly BallState
	| HasDick Dick (Maybe BallState)
	deriving (Eq, Ord, Show)

data Dick = Dick
	{ dickPlacement :: DickLocation
	, dickCount :: Int
	, relativeSize :: DickSizeR
--, objectiveSize :: DickSizeO -- this can be derived later, maybe?
	, dickAttr :: Set DickAttr
	, dickClimax :: Set ClimaxAttr
	, dickAnimal :: Maybe Animal
	, dickKnot :: Maybe KnotState
	, dickMaterial :: FullMat
	, unsheathSize :: Maybe UnsheathSize
	, replacingCloaca :: Bool
	}
	deriving (Eq, Ord, Show)

data DickAttr
	-- natural
	= Flared | Knotted | Barbs | Medial
	-- body-based
	| Armored
	-- gloss
	| Chained | Pierced | Stabbed'Sword | Stabbed'Arrows
	-- elemental
	| Ichorous | TentacleFringe | CrystalSpines
	| Infused'Fire | Infused'Frost | Infused'Electricity
	deriving (Eq, Ord, Show, Read)

-- ugh names
data ClimaxAttr = Flaring | Knotting | Twisting
	deriving (Eq, Ord, Enum, Bounded, Show, Read)

data KnotState = KnotState
	{ knotSize :: Int
	, knotCount :: Int
	, knotAttrs :: Set KnotAttr
	, knotUnsheath :: KnotUnsheathSize
	}
	deriving (Eq, Ord, Show)

data KnotAttr = ChainedKnot
	deriving (Eq, Ord, Enum, Bounded, Show)

data DickLocation = DickExternal | DickSheathed | DickCloaca | DickVent
	deriving (Eq, Ord, Enum, Bounded, Show, Read)

{- or pointed up:
 = Short / ToCrotch
 | Average / ToBellybutton
 | MidThigh / ToPecBottom
 | ToKnee / BetweenPecs
 | MidCalf / AgainstFace
 | ToFeet / PastHead
 | PastFeet / PastHeadX
 -}
-- the 'canonical' form is leg-based b/c of centaurs. sorry nagas + snakes.
data DickSizeR
	= Short | Average | MidThigh | ToKnee -- these are as far as 'natural'-sized dicks get
	| MidCalf | ToFeet | PastFeet -- these you get from [LARGE], x2, & x3 on dick
	-- | Dragging -- unreachable i think?
	deriving (Eq, Ord, Enum, Bounded, Show, Read)
data DickSizeO
	= Small'1_4 | Normal'5_7 | Large'8_12 | Huge'13_18 -- natural-sized dicks
	| Extreme'19_25 | Gigantic'26_39 | Mammoth'39_52 -- from [LARGE]
	| Titanic'53_64
	| Colossal'65_76 -- solidly person-sized
	| Cyclopean'77_88 -- w/ [LARGE]x3 on both dick and body, with a cock that's naturally `ToKnee` (bull, dragon, or unicorn)
	-- | Monstrous'89_100 -- solidly two people sized -- i think unreachable
	deriving (Eq, Ord, Enum, Bounded, Show, Read)

-- each [LARGE] on cock is +1 relative size +1 objective size.
-- each [LARGE] on body is +1 objective size.
-- animal nodes have a default size to them

-- this is only applicable for short, sheathed/internal dicks
data KnotUnsheathSize = FullyOut | KnotStuck
	deriving (Eq, Ord, Enum, Bounded, Show, Read)
-- this is only applicable for short, sheathed/internal dicks
data UnsheathSize = Prolapses | FullyUnsheaths | TipOut | FullyHidden
	deriving (Eq, Ord, Enum, Bounded, Show, Read)


-- come for dicks, drool for mouths, milk for breasts, lava for [FIRE], sap for [PLANT], ichor for [ICHOR]. ferrofluid for [METAL]. probably others.
data Fluid = Come | Lava | Ferrofluid | Ichor | Milk | Drool | Sap | GlowingBlack
	deriving (Eq, Ord, Enum, Bounded, Show, Read)

data BallLocation = External | Internal
	deriving (Eq, Ord, Enum, Bounded, Show, Read)

data BallState = BallState
	{ ballCount :: Int
	, ballSize :: Int
	, ballFluid :: Fluid
	, ballAnimal :: Maybe Animal
	, ballLocation :: BallLocation
	}
	deriving (Eq, Ord, Show)




data FootType = Feet | Hooves | ClovenHooves | Paws | Pseudopods | Talons
	deriving (Eq, Ord, Enum, Bounded, Show, Read)

