module Description.Build
  ( build
  , BodyGlob(..)
  , BodyPlan(..)
  , SpeciesType(..)
  , LocomotionPlan(..)
  , LocomotionStrange(..)
  , Locomotion(..)

  -- body polling functions
  , module SAMP
  , DickSizeR(..)
  , DickSizeO(..)
  , Fluid(..)
  , Material(..)
  ) where

import Prelude hiding (Fractional((/)), Word(..), empty, head, tail)

import Control.Arrow (second)
import Control.Monad
import Control.Lens ((.~))
import Data.Tree
import Data.Maybe
import Data.List ((\\))
import Data.Set (Set, member)
import qualified Data.Set as Set

-- the sems in description.construction conflict a bunch w/ tfnode names. it would be ideal to specify the fragments/sentences (which use sems a lot) in a different context than where we specify the qualifications for fragments/sentences (which use tfnodes a lot), b/c right now one of the two ends up GROTESQUELY BLOATED
import Description.Pronoun
import Description.TfNode (TfNode(..), Animal(..), isAnimal, getAnimal, isElement, getElement, isGloss, getGloss, Shape(..), Gloss(..), Weapon(..))
import qualified Description.TfNode as TF
import Description.Body.Sample (legOrBodySpecies, locoplan)
import qualified Description.Body.Sample as SAMP hiding (BodyTransLocation(..))

import Description.Body.Animal
import Description.Body.Types hiding (BodyLocation(..))

import Utility.Misc (collate)
import Utility.Monoid (guardC)
import Utility.Tree (hitNext, hitAny)
import Utility.Tuple (fst_, snd_, thd_, liftSnd)
import Data.Tuple (swap)

{-
build issues:
	* a flame head needs to unset the head species. probably??
	* transitional materials are calculated before elemental materials are applied, so if they're changed they end up outdated. (e.g., a fire satyr still ends up with a ruddyredbrown skin / coarsebrown fur waist transition)


bugs tofix:
> describe You $ Node (CR Animum) [Node (CP TF.Arms) [pure $ CS TF.More], pure $ CP TF.Head, Node (CP TF.Legs) [pure $ CA Goat], pure $ CP TF.Tail, pure $ CA Human]
	([],["You are four-armed satyr."])

> describe He $ Node (CR Animum) [pure $ CP TF.Arms, pure $ CP TF.Legs, pure $ CP TF.Tail, pure $ CP TF.Head, pure $ CA Human]
	([],["He is a one-tailed man."])

	(minimally: 1-count unusual limbs should drop the quantity modifier.
		"he is a tailed man."
	better: specific clause construction, somehow
		"he is a man with the {tail adjectives} tail of a {species} {, tail clause}" or smth)

sem resolution is a huge confusing mess and i don't really know how it works or how i want it to work; i need to try to figure that out / streamline it somewhat before adding in much more

SEM RESOLUTION: HOW SHOULD IT WORK??

	LANTERNS
	glowing tongues &c
	jackals

is
	you
	a / satyr <- "a" should use the quantity of "satyr", which would be the body sem

bubbles and boils around
	ichor
	the / red-hot / metal chain <- 'the' seems to always be here, like...
	the / red-hot / sword
	the / red-hot / swords
	etc

-}

hasDick :: DickState -> Bool
hasDick (HasDick _ _) = True
hasDick _ = False

build :: Gender -> Gender -> Tree TfNode -> BodyGlob
build n p r = BodyGlob p n
	(unusuals)
	(FauxGlob satyrType bestial headState hasLargeIchorTongue haloElement dickState hornState bodyHorns)
	(bodySize)
	bodySizeUnusual
	(bodyplan)
	(nounFragment)
	where
		BasicBody locomotion unusuals rawBodyplan nounFragment = buildBasicBody r

		(bodySize, bodySizeUnusual) = baseBodyHeight
			(planLocomotion $ bodyplan)
			((\(_,_,ma) -> ma) $ chest $ limbs $ bodyplan)
			(nounFragment)
			(size r)

		bodyplan :: BodyPlan
		bodyplan = rawBodyplan { limbs = merge <$> new <*> (limbs $ rawBodyplan) }
			where
				new = limbMatReplacements $ fmap snd_ $ limbs $ rawBodyplan
				merge :: Maybe b -> (a, b, c) -> (a, b, c)
				merge Nothing x = x
				merge (Just r) (a, _, c) = (a, r, c)
				-- if this is only `Nothing` (instead of `Just Nothing`) the <$>/<*> above will remove all the other limb data
				limbMatReplacements :: Covering FullMat -> Covering (Maybe FullMat)
				limbMatReplacements l = case catMaybes bodysets of
					[] -> case False of
						True -> error "" -- why did i write this code??
						_ -> nilCovering
					sets -> foldr bappend nilCovering sets
				nilCovering :: Covering (Maybe FullMat)
				nilCovering = Covering
					(Just Nothing)
					Nothing
					(Just Nothing)
					(Just Nothing)
					(Just Nothing)
					(Just Nothing)
					(Just Nothing)
				bappend :: Covering (Maybe FullMat) -> Covering (Maybe FullMat) -> Covering (Maybe FullMat)
				bappend
					(Covering ma b mc md me mf mg)
					(Covering ma' b' mc' md' me' mf' mg') =
						Covering (ma <//> ma') (b </> b') (mc <//> mc') (md <//> md') (me <//> me') (mf <//> mf') (mg <//> mg')
						where
							(</>) :: Maybe a -> Maybe a -> Maybe a
							Nothing </> (Just x) = Just x
							(Just x) </> _ = Just x
							_ </> _ = Nothing
							(<//>) :: Maybe (Maybe a) -> Maybe (Maybe a) -> Maybe (Maybe a)
							Nothing <//> (Just b) = Just b
							(Just a) <//> (Just b) = case (a, b) of
								(Nothing, Just b') -> Just (Just b')
								(Just a', _) -> Just (Just a')
								(_, _) -> Just Nothing
							(Just a) <//> Nothing = Just a
							_ <//> _ = Nothing
				updateBodyMattrs :: TfNode -> ((Int, FullMat, Maybe Animal) -> Maybe FullMat) -> Maybe (Covering (Maybe FullMat))
				updateBodyMattrs matchNode setAttr = case hitNext (== matchNode) r of
							Just _ -> Just $ setAttr <$> limbs rawBodyplan
							_ -> Nothing
				bodysets :: [Maybe (Covering (Maybe FullMat))]
				bodysets =
					-- full-body replacements
					[ case (bodyHorns, nounFragment) of
						(True, Just Polyhedra) -> Just $ Covering
							(Just Nothing)
							(Just $ FMat Nil Hoof)
							(Just Nothing)
							(Just Nothing)
							(Just Nothing)
							(Just Nothing)
							(Just Nothing)
						_ -> Nothing
					, guardC (FlameHead `member` headState)
						$ Covering
							(Just $ Just $ FMat BrilliantRed Flame)
							Nothing
							(Just Nothing)
							(Just Nothing)
							(Just Nothing)
							(Just Nothing)
							(Just Nothing)
					, guardC (MetalHelmet `member` headState)
						$ Covering
							(Just $ Just $ FMat RoughBlackGlyphEnscribed Armor)
							Nothing
							(Just Nothing)
							(Just Nothing)
							(Just Nothing)
							(Just Nothing)
							(Just Nothing)
					, guardC (FacelessVoid `member` headState)
						$ Covering
							(Just $ Just $ FMat StarryNebula Space)
							Nothing
							(Just Nothing)
							(Just Nothing)
							(Just Nothing)
							(Just Nothing)
							(Just Nothing)
					, updateBodyMattrs (CE TF.Ichor) $ \x -> case snd_ x of
							FMat _ Fur -> Just $ FMat OilyBlack Fur
							FMat _ Coat -> Just $ FMat OilyBlack Coat
							FMat _ Skin -> Just $ FMat PallidGrey Skin
							FMat _ Scales -> Just $ FMat DullGrey Scales
							FMat _ Protoplasm -> Just $ FMat OilyBlack Protoplasm
							x -> error $ "missing ichor mattr for " <> show x
					, updateBodyMattrs (CE TF.Fire) $ \x -> case snd_ x of
							FMat _ Fur -> Just $ FMat Fiery Fur
							FMat _ Coat -> Just $ FMat Fiery Coat
							FMat _ Skin -> Just $ FMat Fiery Skin
							FMat _ Scales -> Just $ FMat ScintilatingOrangeRed Scales
							FMat _ Protoplasm -> Just $ FMat BrilliantRed Flame
							x -> error $ "missing fire mattr for " <> show x
					, updateBodyMattrs (CE TF.Stellar) $ \x -> case snd_ x of
							FMat _ Skin -> Just $ FMat StarryNebula Skin
							FMat _ x | x `elem` [Coat, Fur, Scales] -> Just $ FMat BlackWithGlowingTips x
							x -> error $ "missing stellar mattr for " <> show x
					, let setMetalAttr x = Just $ FMat RoughBlackGlyphEnscribed Armor
						in case hitNext (== CE TF.Metal) r of
							Just _ -> Just $ setMetalAttr <$> limbs rawBodyplan
							_ -> Nothing
					]
		bestial = or
			[ isSatyr && fromMaybe False (do
					head_ <- hitAny (== CP TF.Head) r
					_ <- hitNext (== CA Goat) head_
					return True)
			, isMinotaur && fromMaybe False (do
					legs <- hitAny (== CP TF.Legs) r
					_ <- hitNext (== CA Bull) legs
					return True)
			]
			where
				isSatyr = nounFragment == Just Satyr
				isMinotaur = nounFragment == Just Minotaur
		haloElement = do
			head_ <- hitAny (== CP TF.Head) r
			halo <- hitAny (== CP TF.Halo) head_
			return $ getElement =<< fmap rootLabel (hitNext isElement halo)
		headState = Set.fromList $ catMaybes
			[ hasHeadNode (CE TF.Metal) MetalHelmet
			, hasHeadNode (CE TF.Ichor) Blendec
			, hasHeadNode (CE TF.Fire) FlameHead
			, hasHeadNode (CE TF.Stellar) FacelessVoid
			, hasHeadNode (CE TF.Gem) GeodeHead
			]
			where
				hasHeadNode n val = do
					head_ <- hitAny (== CP TF.Head) r
					_ <- hitNext (== n) head_
					return val
		hasLargeIchorTongue = fromMaybe False $ do
			head_ <- hitAny (== CP TF.Head) r
			mouth <- hitNext (== CX TF.Mouth) head_
			tongue <- hitNext (== CX TF.Tongue) mouth
			_ <- hitNext (== CE TF.Ichor) tongue
			_ <- hitNext (== CS TF.Large) tongue
			return True
		satyrType = case (isSatyr, hasIchorSatyr) of
			(True, True) -> IchorSatyr
			(True, False) -> BaseSatyr
			_ -> NoSatyr
			where
				isSatyr = nounFragment == Just Satyr
				hasIchorSatyr = fromMaybe False $ do
					_ <- hitNext (== CE TF.Ichor) r
					return $ True
		bodyHorns = case hitNext (== CP TF.Horns) r of
			Just _ -> True
			_ -> False
		hornState = let mhornNodes = do
					head <- hitNext (== CP TF.Head) r
					hitNext (== CP TF.Horns) head
			in
				case mhornNodes of
					Nothing -> Nothing
					Just hornNodes -> Just $ HornState actualHornType hornMaterial hornCount hornSize unusualHornType
						where
							effectiveHornAnimal = first [localAnimal, effectiveHeadAnimal]
							localAnimal = getAnimal =<< fmap rootLabel (hitNext isAnimal hornNodes)
							effectiveHeadAnimal = first $
								[ thd_ =<< (head $ limbs $ bodyplan)
								, thd_ $ chest $ limbs $ bodyplan
								]
							actualHornType = case getElement =<< (rootLabel <$> hitNext isElement hornNodes) of
								Just TF.Ichor -> Tentacled
								_ -> baseHornType
							baseHornType = case effectiveHornAnimal of
								Nothing -> Nubby
								Just Bull -> Prongs
								Just Deer -> Tined
								Just Goat -> Curled
								Just Unicorn -> Pointed
								Just _ -> Nubby
							hornMaterial = case getElement =<< (rootLabel <$> hitNext isElement hornNodes) of
								Just TF.Metal -> case baseHornType of
									Tined -> FMat Black WroughtIron
									_ -> FMat RoughBlackGlyphEnscribed Metal
								Just TF.Fire -> case baseHornType of
									Tined -> FMat Black BurntWood
									_ -> FMat LavaPatterned Rock
								Just TF.Stellar -> case baseHornType of
									Tined -> FMat BlackWithGlowingTips Horn
									_ -> FMat GlowingWhite Horn
								Just TF.Gem -> FMat TranslucentPurple Crystal
								Just TF.Ichor -> FMat OilyBlack Flesh
								_ -> case fmap snd_ $ head $ limbs $ bodyplan of
									Just (FMat _ Protoplasm) -> FMat Nil Protoplasm
									Just (FMat attr Flame) -> FMat attr Flame
									_ -> case effectiveHornAnimal of
										Just Unicorn -> FMat TranslucentGlowing Crystal
										_ -> FMat Brown Horn
							hornCount = quantity hornNodes * baseCount
							baseCount = if effectiveHornAnimal == Just Unicorn
									then 1
									else 2
							hornSize = size hornNodes
							unusualHornType = effectiveHornAnimal /= effectiveHeadAnimal
		dickState = case (hasDick, ballState) of
			(True, _) -> HasDick
				(Dick dickLocation dickCount dickSize attrs climax dickAnimal knotState materialData hardUnsheathState noCloaca)
				ballState
			(False, Nothing) -> NoDick
			(False, Just balls) -> BallsOnly balls
			where
				noCloaca = case locomotion of
					x | x `elem` [Slithering, Naga] -> case dickLocation of
						DickExternal -> True
						DickSheathed -> True
						_ -> False
					_ -> False

				-- TODO: actually use this in prose at all, to centralize unsheathing/size/knot lines
				hardUnsheathState = case (dickLocation, dickSize, dickCount) of
						(DickExternal, _, _) -> Nothing
						(t, _, n) | n >= 4 && t `elem` [DickVent, DickCloaca] -> Just Prolapses
						(_, Short, _) -> Just TipOut
						_ -> Just FullyUnsheaths

				cockCountModifier =
					if Set.member (CS More) dickNodes
						then 2
						else if Set.member (CS Morex3) dickNodes
							then 3
							else if Set.member (CS Morex4) dickNodes
								then 4
								else 1
				cockBaseCount = 
					if Set.member (CA Snake) dickNodes
						then 2
						else 1
				dickCount = cockBaseCount * cockCountModifier
				dickAnimal = first $
					[ getAnimal <=< listToMaybe $ Set.toList $ Set.filter isAnimal $ dickNodes
					, legOrBodySpecies (nounFragment) (bodyplan)
					]
				materialData = foldr ($) baseMaterialData $ fmap snd $ filter fst alterFuncs
				alterFuncs :: [(Bool, FullMat -> FullMat)]
				alterFuncs =
					[ (Ichorous `member` attrs
						, (attr .~ GreyBlack))
					, (Infused'Fire `member` attrs
						, (attr .~ GlowingRedHot))
					]
				baseMaterialData = case dickAnimal of
						Nothing -> FMat Nil Protoplasm
						Just a -> animalDickMaterial a
				dickSize = sizeBonus $ case dickAnimal of
						Nothing -> case locomotion of
							Centaur -> MidThigh
							_ -> Average
						Just s -> SAMP.animalCockSize s
					where
						sizeBonus = if CS TF.Large `member` dickNodes
							then a
							else if CS TF.Largex2 `member` dickNodes
							then a . a
							else if CS TF.Largex3 `member` dickNodes
							then a . a . a
							else id
							where
								a :: DickSizeR -> DickSizeR
								a = succ
				attrs = genDickAttrs dickNodes <> case fmap snd_ $ (if planLocomotion bodyplan `elem` [Basic Slithering, Basic Naga] then tail else legs) $ limbs $ bodyplan of
					Just (FMat _ Armor) -> Set.singleton Armored
					_ -> mempty
				climax = genClimaxAttrs dickNodes
				dickLocation = case dickAnimal of
					Nothing -> DickExternal
					Just a -> animalDickLocation a
				hasDick = fromMaybe False $ do
					_ <- hitNext (== CX TF.Cock) r
					return True
				dickNodes = (\s -> if Set.null $ Set.filter isAnimal s
					then case legOrBodySpecies (nounFragment) (bodyplan) of
						Just x -> Set.insert (CA x) s
						Nothing -> s
					else s)
					$ fromMaybe mempty $ do
						cock <- hitNext (== CX TF.Cock) r
						return $ case cock of
							(Node _ chs) -> Set.fromList . fmap rootLabel $ chs
				knotState = case knotNodes of
					Nothing -> case dickAnimal of
						Just Wolf -> Just $ KnotState 1 1 mempty FullyOut
						_ -> Nothing
					Just knot -> let
							knotSize = size knot + 1
							knotCount = quantity knot
							knotAttrs = case getGloss =<< rootLabel<$> hitNext isGloss knot of
								Just (Weapon Chain) -> Set.singleton ChainedKnot
								_ -> mempty
							hardKnotState = case (dickLocation, dickSize) of
								(DickSheathed, Short) -> case knotSize of
									n | n > 1 -> KnotStuck
									_ -> FullyOut
								_ -> FullyOut
						in
							Just $ KnotState knotSize knotCount knotAttrs hardKnotState
					where
						knotNodes = do
							cock <- hitNext (== CX TF.Cock) r
							knot <- hitNext (== CX TF.Knot) cock
							return $ knot
				ballState = makeBalls <$> mb
					where
						mb = hitNext (== CX TF.Balls) r
						makeBalls b = BallState count ballSize cumtype ballAnimal internal
							where
								ballModifier = quantity b
								count = 2 * ballModifier
								ballSize = size b + 1
								ballAnimal = first $
									[ (getAnimal =<<) $ rootLabel <$> hitNext isAnimal b
									, legOrBodySpecies (nounFragment) (bodyplan)
									]
								internal = if maybe False hasInternalBalls ballAnimal
									then Internal
									else External
								cumtype = case getElement =<< rootLabel <$> hitNext (isElement) b of
									Just TF.Ichor -> Ichor
									Just TF.Fire -> Lava
									Just TF.Metal -> Ferrofluid
									Just TF.Stellar -> GlowingBlack
									_ -> Come


baseBodyHeight :: LocomotionPlan -> Maybe Animal -> Maybe NounSpec -> Int -> (HeightVal, Int)
baseBodyHeight plan baseAnimal spec sizeBonus = (size sizeBonus $ baseHeight, sizeBonus)
	where
		baseHeight = case plan of
			Basic Centaur -> CentaurHeight
			Basic Quadruped -> case baseAnimal of
				Nothing -> SmallAnimal
				Just x | x `elem` [Wolf, Goat, Deer] -> SmallAnimal
					| x == Dragon -> CentaurHeight
					| otherwise -> HorsishAnimal
			Basic Biped -> case spec of
				Just Minotaur -> TallerThanHuman
				_ -> case baseAnimal of
					Just Human -> HumanHeight
					-- literally all the furries are taller than humans
					Just x -> TallerThanHuman
					_ -> HumanHeight
			Basic Slithering -> SmallAnimal
			Basic Naga -> HumanHeight
			-- w/e making this up
			Basic Wyvern -> case baseAnimal of
				Nothing -> SmallAnimal
				Just x | x `elem` [Bat, Chicken] -> HorsishAnimal
					| x == Dragon -> CentaurHeight
					| otherwise -> HorsishAnimal
			Basic Formless -> Floating
			Spec Orb -> Floating
			Spec FloatingDick -> Floating
			Spec FloatingHead -> Floating
			Basic Complex -> error "complex body"
		size x = case x of
			0 -> id
			1 -> a
			2 -> a . a
			3 -> a . a . a
			_ -> error "too big"
			where
				a h = case h of
					HumanHeight -> CentaurHeight
					_ -> succ h

quantity :: Tree TfNode -> Int
quantity v = case hitNext (\x -> x `elem` [CS More, CS Morex3, CS Morex4]) v of
	Nothing -> 1
	Just (Node (CS More) _) -> 2
	Just (Node (CS Morex3) _) -> 3
	Just (Node (CS Morex4) _) -> 4
	_ -> 1

size :: Tree TfNode -> Int
size v = case rootLabel <$> hitNext (\x -> x `elem` [CS Large, CS Largex2, CS Largex3]) v of
	Nothing -> 0
	Just (CS Large) -> 1
	Just (CS Largex2) -> 2
	Just (CS Largex3) -> 3
	_ -> 0

genDickAttrs :: Set TfNode -> Set DickAttr
genDickAttrs set =
	mconcat $ (\(f, attr) -> if any f set then Set.singleton attr else mempty) <$>
		[ ((\x -> x == CA Unicorn), Flared)
		, ((\x -> x == CA Unicorn), Medial)
		, ((\x -> x == CA Wolf || x == CX TF.Knot), Knotted)
		, ((\x -> x `elem` [CA Lion, CA Tiger]), Barbs)
		, (\x -> x == CG (TF.Weapon TF.Chain), Chained)
		, (\x -> x == CG (TF.Weapon TF.Sword), Stabbed'Sword)
		, (\x -> x == CG (TF.Weapon TF.Bow), Stabbed'Arrows)
		, (\x -> x == CE TF.Ichor, Ichorous)
		, (\x -> x == CE TF.Stellar, TentacleFringe)
		, (\x -> x == CE TF.Gem, CrystalSpines)
		, (\x -> x == CE TF.Fire, Infused'Fire)
		, (\x -> x == CE TF.Ice, Infused'Frost)
		, (\x -> x == CE TF.Storm, Infused'Electricity)
		, (\x -> x == CE TF.Metal, Pierced)
		]

genClimaxAttrs :: Set TfNode -> Set ClimaxAttr
genClimaxAttrs set =
	mconcat $ (\(f, attr) -> if any f set then Set.singleton attr else mempty) <$>
		[ ((\x -> x == CA Unicorn), Flaring)
		, ((`elem` [CA Bull, CA Boar]), Twisting)
		, ((\x -> x == CA Wolf || x == CX TF.Knot), Knotting)
		]

first :: [Maybe a] -> Maybe a
first = listToMaybe . catMaybes




data BasicBody = BasicBody
	{ locoType :: Locomotion
	, unusuals :: [(TF.Part, BasicUnusual)]

	, bodyshape :: BodyPlan
	, nounFragment :: Maybe NounSpec
	}
	deriving (Show)

buildBasicBody :: Tree TfNode -> BasicBody
buildBasicBody t = buildFragment . applyUnusuals $ body
	where
		extract (CA s) = Just s
		extract _ = Nothing
		(limbs', trans, locomotion) = readBody t
		body = BasicBody
			locomotion
			[]
			(Plan (Spec Orb) trans limbs')
			Nothing
		applyUnusuals, buildFragment :: BasicBody -> BasicBody
		applyUnusuals b = b { unusuals = unusuals }
			where
				foo :: (Covering (Int, FullMat, Maybe Animal) -> Maybe (Int, FullMat, Maybe Animal)) -> BasicBody -> (Int, Maybe TF.Animal)
				foo f = fromMaybe (0, Nothing) . fmap (\(c, _, a) -> (c, a)) . f . limbs . bodyshape
				limbCount :: [(TF.Part, (Int, Maybe TF.Animal))]
				limbCount = 
					[ (TF.Head, foo head b)
					, (TF.Arms, foo arms b)
					, (TF.Legs, foo legs b)
					, (TF.Tail, foo tail b)
					, (TF.Wings, foo wings b)
					]
				unusuals :: [(TF.Part, BasicUnusual)]
				unusuals = case first [(\(_, _, a) -> a) . chest . limbs . bodyshape $ b, fallbackSpecies] of
					Just bodySpec ->
						mapMaybe (uncurry (quantityUnusual bodySpec (locoType b))) limbCount
						<> speciesUnusual bodySpec (second snd <$> limbCount)
						<> hornless
					Nothing -> [] -- any species tags are unusual. no good way to figure out base limb quantities. let's just pretend this doesn't exist.
					where
						fallbackSpecies = case locoType b of
							Formless -> Nothing
							Biped -> Just Human
							Quadruped -> Just Wolf
							Slithering -> Just Snake
							Centaur -> Nothing
							Naga -> Nothing
							Wyvern -> Just Bat
							Complex -> Nothing
						hornless = maybeToList $ do
							head_ <- hitAny (== CP TF.Head) t
							let bodySpec = hitNext isAnimal t
							let headSpec = hitNext isAnimal head_
							let hornSpec = (getAnimal =<<) $ fmap rootLabel $ first $ [headSpec,bodySpec] :: Maybe Animal
							if fromMaybe False $ hasHorns <$> hornSpec
								then case hitNext (== CP TF.Horns) head_ of
									Nothing -> Just (TF.Horns, Quantity Missing)
									Just _ -> Nothing
								else Nothing
		buildFragment b = b { bodyshape = genShape, nounFragment = nounFragment, unusuals = unusuals b \\ toRemove }
			where
			emptyMatch _ = case t of
				Node _ [] -> True
				_ -> False
			partCount :: (Covering (Int, FullMat, Maybe Animal) -> Maybe (Int, FullMat, Maybe Animal)) -> BasicBody -> Int
			partCount p = fromMaybe 0 . fmap fst_ . p . limbs . bodyshape
			partSpec :: (Covering (Int, FullMat, Maybe Animal) -> Maybe (Int, FullMat, Maybe Animal)) -> BasicBody -> Maybe Animal
			partSpec p = thd_ <=< (p . limbs . bodyshape)
			partMatch p = p . limbs . bodyshape
			animalWormMatch b = and
				[ locoType b == Slithering
				, isJust $ partSpec (Just . chest) b
				]
			floatingHeadMatch b = and
				[ partCount head b >= 1
				, 0 == partCount arms b
				, 0 == partCount legs b
				, 0 == partCount tail b
				, 0 == partCount wings b
				]
			floatingDickMatch b = and
				[ 0 == partCount head b
				, 0 == partCount arms b
				, 0 == partCount legs b
				, 0 == partCount tail b
				, 0 == partCount wings b
				, partCount genitalia b >= 1
				]
			formlessMatch b = locoType b == Formless
			polyhedraMatch b = and
				[ formlessMatch b
				, 0 == partCount head b
				, 0 == partCount arms b
				, 0 == partCount legs b
				, 0 == partCount tail b
				, 0 == partCount wings b
				, 0 == partCount genitalia b
				, case hitNext (== CP TF.Horns) t of
					Just _ -> True
					_ -> False
				]
			torusMatch b = and
				[ formlessMatch b
				, 0 == partCount head b
				, 0 == partCount arms b
				, 0 == partCount legs b
				, 0 == partCount tail b
				, 0 == partCount wings b
				, 0 == partCount genitalia b
				, case hitNext (== CE TF.Metal) t of
					Just _ -> True
					_ -> False
				]
			-- keep in mind there are (should be) 'bestial' variants involving both head and legs of the species, for both satyr and minotaur
			satyrMatch b = and
				[ partSpec (Just . chest) b == Just Human
				, partSpec legs b == Just Goat
				, locoType b == Biped
				]
			minotaurMatch b = and
				[ partSpec (Just . chest) b == Just Human
				, partSpec head b == Just Bull
				, locoType b == Biped
				]
			sileniMatch b = and
				[ partSpec (Just . chest) b == Just Human
				, partSpec legs b == Just Unicorn
				, locoType b == Biped
				]
			centaurMatch b = and
				[ partSpec (Just . chest) b == Just Human
				, partSpec legs b == Just Unicorn
				, locoType b == Centaur
				]
			{-
			gryphonMatch b = and
				[ bodyB b == Just Bird -- (or eagle)
				, snd (legB b) == Just Lion
				, snd (headB b) == Just Bird || snd (headB b) == Nothing
				]
			-}
			splitTaur b = and
				[ isJust $ partSpec (Just . chest) b
				, isJust $ partSpec legs b
				, partSpec (Just . chest) b /= partSpec legs b
				, locoType b == Centaur
				]
			splitNaga b = and
				[ isJust $ partSpec (Just . chest) b
				, isJust $ partSpec tail b
				, partSpec (Just . chest) b /= partSpec tail b
				, locoType b == Naga
				]
			-- we only need 2 species flags instead of 3 b/c the body species is the + 1
			chimeraMatch b =
				let
					rest = filter (isSpeciesUnusual . snd) $ removeMerged (unusuals b) merged
					merged = mergeableUnusuals $ unusuals b
				in length rest + length merged >= 2
			genShape = Plan
				genSpec
				trans
				limbs'
			genSpec
				| emptyMatch b = Spec Orb
				| floatingDickMatch b = Spec FloatingDick
				| floatingHeadMatch b = Spec FloatingHead
				| otherwise = Basic (locoType b)
			(nounFragment, toRemove)
				| polyhedraMatch b = (Just Polyhedra, [])
				| torusMatch b = (Just Torus, [])
				| minotaurMatch b = (Just Minotaur,
					[(TF.Head, Species Bull)
					,(TF.Legs, Species Bull) -- legs and tail for the 'bestial' variant
					,(TF.Tail, Species Bull)
					])
				| satyrMatch b = (Just Satyr,
					[(TF.Legs, Species Goat)
					,(TF.Tail, Species Goat)
					])
				| sileniMatch b = case partSpec legs b of
					Just e -> (Just $ Sileni e,
						[(TF.Legs, Species e)
						,(TF.Tail, Species e)
						])
					_ -> (Nothing, [])
				| centaurMatch b = ((\spec -> SplitTaur Human spec) <$> partSpec legs b,
					[(TF.Legs, Species Unicorn)])
				| splitTaur b = case (,) <$> partSpec legs b <*> partSpec (Just . chest) b of
					Just (b, t) -> (Just $ SplitTaur t b,
						[(TF.Legs, Species b), (TF.Tail, Species b)])
					Nothing -> (Nothing, [])
				| splitNaga b = case (,) <$> partSpec tail b <*> partSpec (Just . chest) b of
					Just (b, t) -> (Just $ SplitNaga t b,
						[(TF.Tail, Species b)])
					Nothing -> (Nothing, [])
				| chimeraMatch b = (Just Chimera, [])
				-- slithering bodies that aren't Just Snake or Nothing get rendered as e.g., goat-worm, but they're generated up to this point using the e.g., goat defaults, so the tail and the lack of legs show up as unusuals
				| animalWormMatch b = (Nothing,
					[ (TF.Legs, Quantity Missing)
					] <> case partSpec tail b of
						Just s -> [(TF.Tail, Species s)]
						_ -> []
					)
				| floatingHeadMatch b = (Nothing, catMaybes
					[ Just (TF.Legs, Quantity Missing)
					, Just (TF.Arms, Quantity Missing)
					, Just (TF.Tail, Quantity Missing)
					, Just (TF.Wings, Quantity Missing)
					, case partSpec head b of
						Nothing -> Nothing
						Just a -> Just (TF.Head, Species a)
					-- floating heads have a special quantity article/noun; we don't want to also have an unusual adjective. that leads to "you're three floating three-headed heads"
					, Just (TF.Head, Quantity $ Q 2)
					, Just (TF.Head, Quantity $ Q 3)
					, Just (TF.Head, Quantity $ Q 4)
					])
				| floatingDickMatch b = (Nothing,
					[ (TF.Head, Quantity Missing)
					, (TF.Legs, Quantity Missing)
					, (TF.Arms, Quantity Missing)
					, (TF.Tail, Quantity Missing)
					, (TF.Wings, Quantity Missing)
					])
				-- formless bodies are just churning blobs, so they're not meaningfully "missing" anything
				| formlessMatch b = (Nothing,
					[ (TF.Head, Quantity Missing)
					, (TF.Arms, Quantity Missing)
					, (TF.Legs, Quantity Missing)
					, (TF.Tail, Quantity Missing)
					, (TF.Arms, Quantity (Q 2))
					, (TF.Legs, Quantity (Q 2))
					])
				| otherwise = (Nothing, [])


limbLocomotion :: Covering (Int, FullMat, Maybe Animal) -> Locomotion
limbLocomotion ls = case (heads_, arms_, legs_, tails_, bodytail, wings_) of
	(_, ac,  0, tc, bt,  _) | ac >= 2 && tc >= 1 && bt == True -> Naga
	(_, ac,  2,  _,  _,  _) | ac >= 2 -> Biped
	(_,  0, lc,  _,  _,  _) | lc >= 4 -> Quadruped
	(_,  0, lc,  _,  _, wc) | lc >= 2 && wc >= 2 -> Wyvern
	-- centaurs are also permitted to have wings.
	(_, ac, lc,  _,  _,  _) | ac >= 2 && lc >= 4 -> Centaur
	-- snakes are permitted to have wings.
	(_,  0,  0, tc, bt,  _) | tc >= 1 && bt == True -> Slithering
	(_, ac, lc,  _,  _,  0) | ac + lc <= 4 -> Formless
	-- the catch-all. try to avoid letting these exist by categorizing them.
	_ -> Complex
	where
		heads_ = fromMaybe 0 $ fst_ <$> head ls
		arms_ = fromMaybe 0 $ fst_ <$> arms ls
		legs_ = fromMaybe 0 $ fst_ <$> legs ls
		tails_ = fromMaybe 0 $ fst_ <$> tail ls
		bodytail = fromMaybe True $ fmap hasBodyTail $ thd_ =<< tail ls
		wings_ = fromMaybe 0 $ fst_ <$> wings ls

readBody :: Tree TfNode -> (Covering (Int, FullMat, Maybe Animal), [(BodyTransLocation, BodyCovering)], Locomotion)
readBody t = (limbs, trans, locomotion)
	where
		limbs :: Covering (Int, FullMat, Maybe Animal)
		limbs = readLimbs t
		locomotion = limbLocomotion limbs
		trans :: [(BodyTransLocation, BodyCovering)]
		trans =
			filter (\x -> case snd x of
				Trans (FMat _ a) (FMat _ b) -> a /= b
				_ -> False) .
			catMaybes $
				-- quadruped has matching 'shoulder' and 'waist' transitions b/c they both use 'legs'
				[ case locomotion of
					Quadruped -> liftSnd (Shoulders, Trans (mat $ chest limbs) <$> mat <$> legs limbs)
					_ -> liftSnd (Shoulders, Trans (mat $ chest limbs) <$> mat <$> arms limbs)
				-- naga/slithering uses the tail location b/c it has a waist but doesn't have legs
				, case locomotion of
					Naga -> liftSnd (Waist Fore, Trans (mat $ chest limbs) <$> mat <$> tail limbs)
					Slithering -> liftSnd (Waist Fore, Trans (mat $ chest limbs) <$> mat <$> tail limbs)
					-- there's a fore/hind distinction that comes up for centaurs; a quadruped might have hind legs mismatching the body, whereas a mismatching body/legs for centaurs would be more of a withers thing
					Quadruped -> liftSnd (Waist Hind, Trans (mat $ chest limbs) <$> mat <$> legs limbs)
					_ -> liftSnd (Waist Fore, Trans (mat $ chest limbs) <$> mat <$> legs limbs)
				, liftSnd (Crotch, Trans <$> (mat <$> legs limbs) <*> (mat <$> genitalia limbs))
				, liftSnd (Neck, Trans <$> (mat <$> head limbs) <*> pure (mat $ chest limbs))
				, liftSnd (Back, Trans (mat $ chest limbs) <$> mat <$> wings limbs)
				-- this was "ass" until i decided "lower back" made more sense
				-- b/c of the way these bodies are put together, they'll never have transitional asses
				, case locomotion of
					Naga -> Nothing
					Slithering -> Nothing
					_ -> liftSnd (LowerBack, Trans <$> (mat <$> legs limbs) <*> (mat <$> tail limbs))
				]
		mat :: (Int, FullMat, Maybe Animal) -> FullMat
		mat (_, m, _) = m
		count :: (Int, FullMat, Maybe Animal) -> Int
		count (c, _, _) = c

readLimbs :: Tree TfNode -> Covering (Int, FullMat, Maybe Animal)
readLimbs t = Covering
	(pick [limbAnimal TF.Head, Just rootAnimal])
	(rootAnimal)
	(pick [limbAnimal TF.Arms, Just rootAnimal])
	(pick [limbAnimal TF.Legs, Just rootAnimal])
	(pick [limbAnimal TF.Tail, limbAnimal TF.Legs, Just rootAnimal])
	(pick [limbAnimal TF.Wings, Just rootAnimal])
	-- the 'genitalia' slot is more like 'sheath or dick count'; if you have an internal dick it's 1 and has the material of yr sheath (or sheath analogue); if you have an external dick it's yr dick count.
	(case pick [limbAnimalX TF.Cock, limbAnimal TF.Legs, Just rootAnimal] of
		Just (q, m, Just a) -> case animalDickLocation a of
			DickExternal -> Just (q, m, Just a)
			_ -> Just (1, m, Just a)
		x -> x)
	where
		limbAnimal :: TF.Part -> Maybe (Int, FullMat, Maybe Animal)
		limbAnimal p = do
			h <- hitAny (CP p ==) t
			let manimal = (getAnimal . rootLabel) =<< hitNext isAnimal h
			let count = defaultCount p * quantity h
			return (count, animalMat manimal, manimal)
		limbAnimalX :: TF.Sex -> Maybe (Int, FullMat, Maybe Animal)
		limbAnimalX p = do
			h <- hitAny (CX p ==) t
			let manimal = (getAnimal . rootLabel) =<< hitNext isAnimal h
			let count = defaultCountX p * quantity h
			return (count, animalMat manimal, manimal)
		rootAnimal :: (Int, FullMat, Maybe Animal)
		rootAnimal = case (getAnimal . rootLabel) =<< hitNext isAnimal t of
			ma -> (1, animalMat ma, ma)
		{-
		if we straight-up don't have a given limb, return Nothing (corresponding to the first entry in the list being Nothing). otherwise, use the given part quantity, but if the part doesn't have an animal specified, check back up the hierarchy to see if there's one at a higher level it can default to
		-}
		pick :: [Maybe (Int, FullMat, Maybe Animal)] -> Maybe (Int, FullMat, Maybe Animal)
		pick [] = Nothing
		pick (Nothing:_) = Nothing
		pick (Just (c, m, Just a):_) = Just (c, m, Just a)
		pick (Just (c, _, Nothing):rs) =
			let (mat, animal) = next rs
			in Just (c, mat, animal)
			where
				next (Just (_, m, Nothing):rs) = next rs
				next (Just (_, m, Just a):_) = (m, Just a)
				next (Nothing:rs) = next rs
				-- this shouldn't even be reachable
				next [] = (animalMat Nothing, Nothing)


defaultCount :: TF.Part -> Int
defaultCount TF.Head = 1
defaultCount TF.Tail = 1
defaultCount _ = 2

defaultCountX :: TF.Sex -> Int
defaultCountX TF.Cock = 1
-- knot, breasts, etc
defaultCountX _ = 2

removeMerged :: [(TF.Part, BasicUnusual)] -> [(BasicUnusual, [TF.Part])] -> [(TF.Part, BasicUnusual)]
removeMerged unusuals merged = filter (not . uncurry inMerged) unusuals
	where
		inMerged :: TF.Part -> BasicUnusual -> Bool
		inMerged p u = case lookup u merged of
			Just vs -> p `elem` vs
			_ -> False


mergeableUnusuals :: [(TF.Part, BasicUnusual)] -> [(BasicUnusual, [TF.Part])]
mergeableUnusuals unusuals = filter canMerge . collate . fmap swap $ unusuals
	where
		canMerge :: (BasicUnusual, [TF.Part]) -> Bool
		canMerge (Quantity _, _) = False
		canMerge (_, [_]) = False
		{-
		{-this check demands uniqueness for parts, so if you got four arms + tiger arms + tiger head with this enabled it would be
			"yr a human w/ four tiger arms and the head of a tiger", rather than
			"yr a four-armed human w/ the head and arms of a tiger"
			neither is particularly good but i decided to go with the latter for the time being
		-}
		canMerge (Species _, ps) = all (\p -> case lookup p partKeyed of
			Just [_] -> True
			_ -> False) ps
		-}
		canMerge (Species _, _) = True
		partKeyed = collate unusuals


isSpeciesUnusual :: BasicUnusual -> Bool
isSpeciesUnusual (Species _) = True
isSpeciesUnusual _ = False


speciesUnusual :: TF.Animal -> [(TF.Part, Maybe TF.Animal)] -> [(TF.Part, BasicUnusual)]
speciesUnusual bodySpec = mapMaybe foo
	where
		foo :: (TF.Part, Maybe TF.Animal) -> Maybe (TF.Part, BasicUnusual)
		foo (_, Nothing) = Nothing
		foo (p, Just s) = if s == bodySpec
			then Nothing
			else Just (p, Species s)

quantityUnusual :: TF.Animal -> Locomotion -> TF.Part -> (Int, Maybe TF.Animal) -> Maybe (TF.Part, BasicUnusual)
quantityUnusual bodySpec l part (actual, mspec) =
	let
		expected = speciesUsual (fromMaybe bodySpec mspec) part
	in if actual /= expected
		-- furries aren't that weird (ignore a species-based arm or leg mismatch if we're walking upright, since we don't want to be shocked every time we realize we're a walking animal)
		then if or
				[ part == TF.Legs && actual == 2 && l == Biped
				, part == TF.Arms && actual == 2 && l == Biped
				-- centaurs too
				, part == TF.Arms && actual == 2 && l == Centaur
				-- nagas too
				, part == TF.Arms && actual == 2 && l == Naga
				-- nagas are not surprised by lacking legs
				, part == TF.Legs && actual == 0 && l == Naga
				]
			then Nothing
			else Just (part, if actual == 0 then Quantity Missing else Quantity (Q actual))
		else Nothing
