{-# LANGUAGE FlexibleInstances #-}
{-# LANGUAGE FlexibleContexts #-}
{-# LANGUAGE FunctionalDependencies #-}

module Description.Construction
  ( module P
  , convert

  , sentence

  , (<?>)
  , (/)
  , s
  , p
  , s'
  , s_
  , it's
  , _'s
  , _'re
  , _'ve
  , yr
  , a
  , a_
  , the
  , ing
  , ed
  , xe
  , xem
  , xyr
  , xe'
  , i
  , ij
  , ijDiscarding
  , ijReplace
  , ijReplaceJ
  , ijReq
  , req
  , ijReqJ
  , ijReqDiscarding
  , ijReqReplace
  , attempt
  , empty
  , comma
  , oxford
  , oneOnly
  , implicit
  , nil
  , errorSentence
  -- more?
  ) where

import Prelude hiding (Fractional((/)), Word(..), empty)

import Description.Parse.Types
import Description.Pronoun
import Description.Dictionary

import Description.Parse as P
import Description.Parse.Types as P




-- binds more weakly than / at 6
infix 5 <?>
(<?>) :: (Convertible word (Word String), Convertible sem Semantic) => sem -> word -> Fragment Semantic String
sem <?> w = Fragment (convert sem) $ convert w

class Convertible a r | a -> r where
	convert :: a -> r

-- this is basically a mappend/<> that also casts the type if applicable. it's mostly to make writing the description build code easier.
infixr 6 /
(/) :: (Convertible a r, Convertible b r, Monoid r) => a -> b -> r
a / b = convert a <> convert b

-- so we can connect sentences w/ /
instance Convertible String (Word String) where
	convert s = Lit (Stem s)

instance Convertible Punctuation (Word String) where
	convert s = Lit (Punc s)

instance Convertible Verb (Word String) where
	convert = V

instance Convertible Preposition (Word String) where
	convert = P

instance Convertible (Word String) (Word String) where
	convert = id

instance Convertible (Joiner (Word String) -> Word String) (Word String) where
	convert x = x empty

instance Convertible (Semantic -> Word String) (Word String) where
	convert x = x Body

-- so we can simplify basic ijs
{-
instance Convertible PartOfSpeech (Word String) where
	convert x = ij x empty
-}

-- so we can connect sems w/ /
instance Convertible PartOfSpeech Semantic where
	convert s = Part s

instance Convertible Semantic Semantic where
	convert = id

empty, comma, oxford :: Joiner (Word String)
empty = ([], [], [])
comma = ([Lit $ Punc Comma], [Lit $ Punc Comma], [Lit $ Punc Comma])
oxford = ([Lit $ Punc Comma], [Lit $ Stem "and"] , [Lit $ Punc Comma, Lit $ Stem "and"])

oneOnly :: Joiner (Word String)
oneOnly = ([], [Lit $ Stem "!!", Lit $ Error errMsg], [Lit $ Stem "!!", Lit $ Error errMsg])
	where
		errMsg = "got multiple hits on target expecting only one"

implicit :: Fragment Semantic String
implicit = None <?> (Lit Nil :: Word String)

nil :: Word String
nil = Lit Nil

errorSentence :: a -> String -> Sentence a Semantic String
errorSentence a err = S a (d (None <?> Lit (Error err))) []


s :: Word String
s = SubjPlural Nothing

p :: String -> String -> Word String
p sing plur = SubjNounPronoun (PlurFrag sing plur) Nothing

-- pluralize, but on a sems match rather than on the current part
s', s_ :: Semantic -> Word String
s' = SubjPlural . Just

s_ = s'

-- "it's" or "they're" or (he's, she's, etc) depending on quantity
-- i haven't used this anywhere yet
it's :: Semantic -> Word String
it's sem = xe sem / Contraction / "s" / s' sem

_'s :: Semantic -> Word String
_'s sem = Contraction / "s" / s' sem

-- FIXME: weirdly, this seems to (sometimes?) apply itself to the NEXT word, not the one before it. so `"foo" / _'re Whatever / "bar" / "baz"` will show up as "foo bar's baz". not sure what's up w/ that. workaround is to just insert a `nil` after it :V
_'re :: Semantic -> Word String
_'re sem = Contraction / (SubjNounPronoun (PlurFrag "s" "re") (Just sem) :: Word String)

_'ve :: Semantic -> Word String
_'ve sem = Contraction / (SubjNounPronoun (PlurFrag "s" "ve") (Just sem) :: Word String)

-- as in you're. it would be "'re" if that was a valid variable name
yr :: Word String
yr = Contraction / "s" / s' Body


-- i don't rly know how to systemize these. a/an is an article (or determiner or w/e) and sometimes plural things replace that w/ a determiner of their own ("you have a cock" vs. "you have two cocks"/"you have twin cocks"), so there's that aspect even before getting into the a vs. an resolution.
-- okay the solution is don't have it be magic. use `a` when the article you want to use is `a`. use "twin" or "two" or "the" or `number q` or whatever else when it's not.
a :: Word String
a = Lit AAn

a_ :: Semantic -> Word String
a_ = SubjSingular "a" . Just

the :: Word String
the = SubjSingular "the" Nothing

-- inflect last word as present participle (or maybe gerund?? for verbs?)
ing :: Word String
ing = Lit $ Inflect PresentParticiple

-- inflect last word as past participle
ed :: Word String
ed = Lit $ Inflect PastParticiple

xe :: Semantic -> Word String
xe = SubjNounPronoun E . Just

xem :: Semantic -> Word String
xem = SubjNounPronoun Em . Just

xyr :: Semantic -> Word String
xyr = SubjNounPronoun Eir . Just

xe' :: Semantic -> Word String
xe' = SubjNounPronoun E' . Just

ij :: SymSem a => a -> Joiner (Word String) -> Word String
ij = SubjInclude Optional Accumulate . asSem

i :: SymSem a => a -> Word String
i v = SubjInclude Optional Accumulate (asSem v) empty

ijDiscarding :: SymSem a => a -> Joiner (Word String) -> Word String
ijDiscarding = SubjInclude Optional Discard . asSem

ijReplace :: (SymSem a, SymSem b) => a -> b -> Word String
ijReplace a b = SubjInclude Optional (Replace $ asSem a) (asSem a <> asSem b) empty

ijReplaceJ :: (SymSem a, SymSem b) => a -> b -> Joiner (Word String) -> Word String
ijReplaceJ a b j = SubjInclude Optional (Replace $ asSem a) (asSem a <> asSem b) j

-- we're assuming a req is gonna turn up at most one result (so we don't have to care about the joiner). this might not be a reasonable assumption, but, w/e, we can fix that if it breaks
req, ijReq :: SymSem a => a -> Word String
ijReq a = SubjInclude Required Accumulate (asSem a) empty
req = ijReq

ijReqJ :: SymSem a => a -> Joiner (Word String) -> Word String
ijReqJ a = SubjInclude Required Accumulate (asSem a)

ijReqDiscarding :: SymSem a => a -> Word String
ijReqDiscarding a = SubjInclude Required Discard (asSem a) empty

ijReqReplace :: (SymSem a, SymSem b) => a -> b -> Word String
ijReqReplace a b = SubjInclude Required (Replace $ asSem a) (asSem a <> asSem b) empty

attempt :: Word a -> Word a -> Word a
attempt = WithFallback

sentence :: def -> [Fragment sem word] -> [[Fragment sem word]] -> Sentence def sem word
sentence ord bases fragments = S ord (mconcat $ d <$> bases) (mconcat fragments)
