module Description.Dictionary
	( inflect
	, Inflection(..)
	) where

-- inflection words operate on the prior word, which we might hope is a Verb or something other than a raw Right, but probably isn't. they should generate an error value (but not call `error`) when asked to inflect a word they don't know.
--                -ing                -ed              -s
data Inflection = PresentParticiple | PastParticiple | Plural
	deriving (Eq, Ord, Enum, Bounded, Show, Read)


-- these inflections combine present participle & gerund, and also past participle and something-i-don't-know-the-technical-name-of, i know, i know; it doesn't matter
inflect :: String -> Inflection -> Either String String
inflect w i = case (w, i) of
	("forms", PresentParticiple) -> Right "forming"
	("drools", PresentParticiple) -> Right "drooling"
	("illuminates", PresentParticiple) -> Right "illuminating"
	("sends", PresentParticiple) -> Right "sending"
	("drools", PastParticiple) -> Right "drooled"
	("stains", PastParticiple) -> Right "stained"
	("smears", PastParticiple) -> Right "smeared"
	("arms", PastParticiple) -> Right "armed"
	("legs", PastParticiple) -> Right "legged"
	("head", PastParticiple) -> Right "headed"
	("tail", PastParticiple) -> Right "tailed"
	-- plural nouns
	("tail", Plural) -> Right "tails"
	("head", Plural) -> Right "heads"
	("hemipene", Plural) -> Right "hemipenes"
	("helmet", Plural) -> Right "helmets"
	("cock", Plural) -> Right "cocks"
	("cockhead", Plural) -> Right "cockheads"
	("cocktip", Plural) -> Right "cocktips"
	("dick", Plural) -> Right "dicks"
	("length", Plural) -> Right "lengths"
	("shaft", Plural) -> Right "shafts"
	("ring", Plural) -> Right "rings"
	("bulge", Plural) -> Right "bulges"
	("mockery", Plural) -> Right "mockeries"
	("thing", Plural) -> Right "things"
	("underside", Plural) -> Right "undersides"
	("knot", Plural) -> Right "knots"
	("lump", Plural) -> Right "lumps"
	("growth", Plural) -> Right "growths"
	("flare", Plural) -> Right "flares"
	("band", Plural) -> Right "bands"
	("tip", Plural) -> Right "tips"
	("tendril", Plural) -> Right "tendrils"
	("pizzle", Plural) -> Right "pizzles"
	("spike", Plural) -> Right "spikes"
	("corkscrew", Plural) -> Right "corkscrews"
	("helix", Plural) -> Right "helixes"
	("pattern", Plural) -> Right "patterns"
	("streamer", Plural) -> Right "streamers"
	("pair", Plural) -> Right "pairs"
	("horn", Plural) -> Right "horns"
	("flame", Plural) -> Right "flames"
	("column", Plural) -> Right "columns"
	("branch", Plural) -> Right "branches"
	("plume", Plural) -> Right "plumes"
	("drip", Plural) -> Right "drips"
	("pillar", Plural) -> Right "pillars"
	("crown", Plural) -> Right "crowns"

	-- not sure if this is right
	("bind", PastParticiple) -> Right "bound"
	("binds", PastParticiple) -> Right "bound"
	("presses", PastParticiple) -> Right "pressed"
	("stabs", PastParticiple) -> Right "stabbed"
	("sheaths", PastParticiple) -> Right "sheathed"

	("that", Plural) -> Right "them" -- i'm uh pretty sure that's not right in all cases b/c english grammar is a mess, but that's right in the one case i might want to re-inflect "that" so far
	("s", Plural) -> Right "re" -- sure this is how we do contractions, sure, sure,,

	-- plural verbs
	(w, Plural) -> case w of
		"gets" -> Right "get"
		"moves" -> Right "move"
		"bulges" -> Right "bulge"
		"churns" -> Right "churn"
		-- these are both PERFECTLY LEGITIMATE SPELLINGS imo
		"unsheathes" -> Right "unsheath"
		"unsheaths" -> Right "unsheath"
		"inflates" -> Right "inflate"
		"softens" -> Right "soften"
		"pulls" -> Right "pull"
		"comes" -> Right "come"
		"spasms" -> Right "spasm"
		"narrows" -> Right "narrow"
		"slaps" -> Right "slap"
		"slides" -> Right "slide"
		"stirs" -> Right "stir"
		"smears" -> Right "smear"
		"scrapes" -> Right "scrape"
		"drags" -> Right "drag"
		"shudders" -> Right "shudder"
		"throbs" -> Right "throb"
		"swells" -> Right "swell"
		"arcs" -> Right "arc"
		"lurches" -> Right "lurch"
		"grinds" -> Right "grind"
		"finds" -> Right "find"
		"jerks" -> Right "jerk"
		"cannonades" -> Right "cannonade"
		"spills" -> Right "spill"
		"spits" -> Right "spit"
		"twitches" -> Right "twitch"
		"starts" -> Right "start"
		"pierces" -> Right "pierce"
		"flares" -> Right "flare"
		"pushes" -> Right "push"
		"leaks" -> Right "leak"
		"burns" -> Right "burn"
		"grows" -> Right "grow"
		"snaps" -> Right "snap"
		"hangs" -> Right "hang"

	-- plural category nouns 
	-- for pluralizing parts that already come out plural. this is a kludge
		"arms" -> Right "arms"

		"wolf" -> Right "wolves"
		"boar" -> Right "boars"
		"lion" -> Right "lions"
		"tiger" -> Right "tigers"
		"goat" -> Right "goats"
		"snake" -> Right "snakes"
		"deer" -> Right "deer"
		"stag" -> Right "stags"
		"doe" -> Right "does"
		_ -> Left $ "can't inflect \"" <> w <> "\" as " <> show i <> "."
	_ -> Left $ "can't inflect \"" <> w <> "\" as " <> show i <> "."
