{-# LANGUAGE DeriveGeneric #-}

module Description.TfNode
	( TfNode (..)
	, Animum (..)
	, Animal (..)
	, Part (..)
	, Sex (..)
	, Element (..)
	, Gloss (..)
	, Weapon (..)
	, Shape (..)
	, nodeLabel
	, isAnimal
	, getAnimal
	, isElement
	, getElement
	, isGloss
	, getGloss
	) where

import Data.Char (toLower)

import GHC.Generics (Generic)
import Control.DeepSeq (NFData)

data TfNode
	= CR Animum
	| CA Animal
	| CP Part
	| CE Element
	| CG Gloss
	| CS Shape
	| CX Sex
	deriving (Show, Eq, Ord, Generic)

instance NFData TfNode

data Animum = Animum
	deriving (Show, Read, Eq, Ord, Enum, Bounded, Generic)

instance NFData Animum

data Animal
	= Wolf | Deer | Goat | Bull | Lion | Tiger
	| Dragon | Unicorn
	| Scorpion | Spider
	| Dove | Raven | Chicken
	| Bat
	| Boar | Human
	| Fish
	| Octopus
	| Mantis
	| Snake | Snakeheaded
	deriving (Show, Read, Eq, Ord, Enum, Bounded, Generic)

instance NFData Animal

data Part
	= Arms | Legs
	| Head | Eyes | Teeth
	| Tail | Wings | Horns | Halo
	| Tentacle
	-- | Hooves
	deriving (Show, Read, Eq, Ord, Enum, Bounded, Generic)

instance NFData Part

data Sex
	= Mouth | Tongue
	| Breasts
	| Cock | Knot | Balls
	| Flower
	deriving (Show, Read, Eq, Ord, Enum, Bounded, Generic)

instance NFData Sex

-- add Poison; maybe move Bone and Obsidian up to here
-- sun and night should become subnodes of stellar. like... sun moon void, basically?
data Element
	= Fire | Ash | Magma
	| Stellar | Sun | Moon | Stars | Void
	| Poison
	| Ichor | Infested
	| Ice | Night | Storm | Gem | Metal | Static | Plant
	deriving (Show, Read, Eq, Ord, Enum, Bounded, Generic)

instance NFData Element

-- add Fat maybe? idk i have no clue how to do build stuff
data Shape
	= More | Morex3 | Morex4 | Less
	| Large | Largex2 | Largex3
	| Asymmetrical
	deriving (Show, Read, Eq, Ord, Enum, Bounded, Generic)

instance NFData Shape

data Gloss
	= Bone | Obsidian | Spines | Veins | Pustules
	| Weapon Weapon
	deriving (Show, Read, Eq, Ord, Generic)

instance NFData Gloss

data Weapon = Sword | Spear | Bow | Chain
	deriving (Show, Read, Eq, Ord, Enum, Bounded, Generic)

instance NFData Weapon

nodeLabel :: TfNode -> String
nodeLabel x = case x of
	CR r -> toLower <$> show r
	CA a -> toLower <$> show a
	CP p -> toLower <$> show p
	CS s -> toLower <$> show s
	CX x -> toLower <$> show x
	CE e -> toLower <$> show e
	CG (Weapon w) -> toLower <$> show w
	CG g -> toLower <$> show g

isElement :: TfNode -> Bool
isElement (CE _) = True
isElement _ = False

getElement :: TfNode -> Maybe Element
getElement (CE e) = Just e
getElement _ = Nothing

isAnimal :: TfNode -> Bool
isAnimal (CA _) = True
isAnimal _ = False

getAnimal :: TfNode -> Maybe Animal
getAnimal (CA a) = Just a
getAnimal _ = Nothing

isGloss :: TfNode -> Bool
isGloss (CG _) = True
isGloss _ = False

getGloss :: TfNode -> Maybe Gloss
getGloss (CG a) = Just a
getGloss _ = Nothing
