
import Codec.Picture
import Utility.Rand

import Sites.Hell.Generation.MapData
import Sites.Hell.Generation.DemoMap

main = do
  seed <- return 127683913759384829 -- randomIO
  print $ seed
  let world = (\(a, _, _) -> a) $ evalRand (mkStdGen seed) $ demo (128, 128)
  writePng "map.png"
    $ generateImage' tileColor world 4

-- | rescale an image function
generateImage' :: (a -> PixelRGB8) -> Map a () -> Int -> Image PixelRGB8
generateImage' f world scale =
  let
    x = _mapWidth world
    y = _mapHeight world
    g xi yi = displayMap f world (xi `div` scale) (yi `div` scale)
  in generateImage g (x * scale) (y * scale)

displayMap :: (a -> PixelRGB8) -> Map a () -> Int -> Int -> PixelRGB8
displayMap f (Map xb yb cs _) x y = if inBounds xb yb x y
  then f (cs !! y !! x)
  else error $ mconcat ["displayMap: out of bound coord (", show x, ", ", show y, ")"]

inBounds :: (Ord a, Num a) => a -> a -> a -> a -> Bool
inBounds xb yb x y = x >= 0 && x < xb && y >= 0 && y < yb

tileColor :: Tile -> PixelRGB8
tileColor t = case t of
  Void -> PixelRGB8 255 255 255
  DeepWater -> PixelRGB8 0 8 48
  ShallowWater -> PixelRGB8 16 32 72
  Sandbar -> PixelRGB8 128 98 80
  Field -> PixelRGB8 44 80 64
  FieldBlock -> PixelRGB8 36 72 56
  Rocks -> PixelRGB8 128 128 98
  RefineryGround -> PixelRGB8 72 44 32
  RefineryPlating -> PixelRGB8 72 64 44
  RefineryPath -> PixelRGB8 94 72 64
  RefineryBlock -> PixelRGB8 64 56 36
  PathL -> PixelRGB8 255 0 0
  PathR -> PixelRGB8 255 0 0
  BridgeL -> PixelRGB8 0 255 0
  BridgeR -> PixelRGB8 0 255 0
