{-# LANGUAGE DeriveDataTypeable #-}

module Server.Error
	( HellError(..)
	, HellServer
	, except
	, asExcept
	, spUnwrapExceptT
	, module Ex
	, logLn
	, module L
	) where

import Data.Typeable

import Control.Monad.Trans
import Control.Monad.Except as Ex (MonadError(..), ExceptT(..), runExceptT)
import Control.Exception as Ex (SomeException, Exception)
import Control.Exception.Lifted as Ex (catch)

import System.Log.Logger as L (logM, Priority(..))

import Happstack.Server (ServerPart, ServerPartT, mapServerPartT)
import Happstack.Server.Internal.Monads (UnWebT, runServerPartT, mapServerPartT', ununWebT)
import Happstack.Server.Types (Request)

data HellError = NoError | HellError [String]
	deriving (Eq, Ord, Show, Read, Typeable)

instance Exception HellError

instance Semigroup HellError where
	a <> NoError = a
	NoError <> b = b
	HellError a <> HellError b = HellError $ a <> b

instance Monoid HellError where
	mempty = NoError

type HellServer = ServerPartT (ExceptT HellError IO)

except :: (err -> ServerPart a) -> ServerPartT (ExceptT err IO) a -> ServerPart a
except handler = mapServerPartT' (spUnwrapExceptT handler)

-- this is for handling errors internally to a route, and then just wrapping the result up so its type matches the hell-default error type
asExcept :: ServerPartT IO a -> ServerPartT (ExceptT err IO) a
asExcept = mapServerPartT lift

spUnwrapExceptT:: Monad m => (e -> ServerPartT m a) -> Request -> UnWebT (ExceptT e m) a -> UnWebT m a
spUnwrapExceptT handler rq x = do
	err <- runExceptT x
	case err of
		Left e -> ununWebT $ runServerPartT (handler e) rq
		Right a -> return a

logLn :: Priority -> String -> IO ()
logLn = logM "Happstack.Server.AccessLog.Combined"
