{-# LANGUAGE OverloadedStrings #-}
{-# LANGUAGE DeriveGeneric #-}
{-# LANGUAGE StandaloneDeriving #-}

module Server.Layout
	( minimal
	, errorPage
	, stylesheet
	, script
	, link
	, rss
	, module HI
	) where

import Text.Blaze.Html as HI (Html, toHtml, toValue, ToMarkup(..), (!))
import qualified Text.Blaze.Html5 as H
import qualified Text.Blaze.Html5.Attributes as A

import GHC.Generics (Generic)
import Control.DeepSeq (NFData)
import Happstack.Server (Response(..))
import Happstack.Server.Types (RsFlags(..), HeaderPair(..), Length(..))

deriving instance Generic Response
deriving instance Generic RsFlags
deriving instance Generic HeaderPair
deriving instance Generic Length


instance NFData Response where
instance NFData RsFlags where
instance NFData HeaderPair where
instance NFData Length where

minimal :: (ToMarkup headers, ToMarkup contents) => String -> headers -> contents -> Html
minimal title headers contents = do
	H.docType
	toMarkup headers
	H.title $ preEscapedToMarkup title
	toMarkup contents

errorPage :: ToMarkup a => Int -> a -> Html
errorPage errCount errMsg =
	minimal "ERROR"
		(do
			stylesheet "/static/css/main.css"
			stylesheet "/static/css/error.css")
		(do
			H.h1 $ toHtml $ "ERROR " ++ show errCount ++ " ERROR"
			H.div ! A.id "main" $ do
				H.p $ toHtml errMsg)

-- todo: media?
stylesheet :: String -> Html
stylesheet url =
	H.link ! A.type_ "text/css" ! A.rel "stylesheet" ! A.href (toValue url)

script :: String -> Html
script url =
	H.script ! A.type_ "text/javascript" ! A.src (toValue url) $ return ()

link :: (H.ToValue a, ToMarkup b) => a -> b -> Html
link url contents = H.a ! A.href (toValue url) $ toHtml contents

rss :: H.ToValue a => a -> Html
rss url = H.link ! A.rel "alternate" ! A.type_ "application/rss+xml" ! A.href (toValue url)

