{-# LANGUAGE ViewPatterns, FlexibleInstances, GeneralizedNewtypeDeriving #-}
{-# LANGUAGE FlexibleContexts #-}
{-# LANGUAGE MultiParamTypeClasses #-}
{-# LANGUAGE FunctionalDependencies #-}
{-# LANGUAGE OverloadedStrings #-}
{-# LANGUAGE DeriveGeneric #-}
{-# LANGUAGE UndecidableInstances #-}

module Sites.Hell.Events.GameEvent
	( module MP
	, module C
	, module UM
	, module X
	, Encounter (..)
	, Event (..)
	, Revise (..)
	, Opponent (..)
	, Stats (..)
	, AI (..)
	, ChallengeAction (..)
	, Alchemy (..)
	, Item (..)
	, IIndex (..)
	, GlobalFlag(..)
	, NodeAction(..)

	, EncGlob(..)
	, body
	, mental
	, mind
	, Mental(..)
	, EncounterEncoding (..)
	, withData
	, withSubdata
	, extractRaws

	, readLocal
	, nBranch
	, bodyFail
	, GeneratorRandom (..)
	, genRandom

	, module Accum
	, module P
	, c
	, ProseItem(..)
	, EventReel
	, Markup
	, strong
	, em
	, br
	, quote

	, guard'

	, s
	, plur
	, cloaca
	, (/)

	, (//)
	, Convertible(..)

	, triMap
	, sceneDisplay
	, MarkedScene
	, SceneBloc
	, Scene(..)

	, SetMod(..)
	, Modify(..)
	, Val(..)
	, Cond(..)
	, Op(..)
	, include
	, Scope(..)

	, number
	, materialWord
	, partAdjective
	, cockPlural

	, teeth
	, muzzle
	, side
	, sides
	, hips
	, hand
	, hands
	, feet
	, foot
	, groan
	, groans
	, groaning
	, growl
	, bellow
	, sheath
	, cock
	, forelegs
	, hind
	, cockhead
	, unsheathed
	, unsheathing
	, animalAss
	, fingertips
	, finger
	, fingers
	, ankles
	, fingernail
	, thumb

	, empty
	) where

import Prelude hiding ((/), length)

import Control.Monad as MP (MonadPlus(..))
import Data.Char as C (toLower)
import Data.Vector (length, (!))
import Utility.Misc as UM (safeToEnum)
import Utility.Rand as X
import Control.Lens (Lens, Lens')

import GHC.Generics (Generic)
import Control.DeepSeq (NFData)

import Data.Maybe
import Data.String (IsString(..))
import Data.List (nub)
import Data.Tree (Tree)
import Data.Text (Text, unpack, pack)
import qualified Data.Text as T (head, null)
import Data.Foldable (foldl')
import qualified Data.HashMap.Strict as H (lookup, insert)
import Control.Arrow (first)

import Control.Lens (_2, _3, (%~), (^.))
import Server.JSON

import Data.Convertible
import Description.TfNode hiding (Part(..))
import Description.Dictionary
import Description.Build (BodyGlob)
import Description.Body.Sample (cockCount, legCount, handType, HandType(..), MiscPart(..))
import qualified Description.Body.Sample as Samp
import Description.Construction as P (Punctuation(..))

import Data.Accum as Accum

import Utility.Tuple (snd_)

-- extra stuff for the sampling functions
import Description.Build (LocomotionPlan(..), Locomotion(..))
import Description.Body.Sample (locoplan, dickLocation, hasMuzzle, hasSnout, animalName, bodySpecies, bodyMat_, Locatable(bodySpecies, bodyCovering), footType, FootType(..), hasFangs, hasTusks)
import Description.Body.Types (DickLocation(..), Material(..), BodyTransLocation(..), Hind(..))
import Description.Body.Types.Material (BodyCovering(..), FullMat(..), BodyLocation(..))

data Encounter = Encounter (Int, Int) Value
	deriving (Eq, Show)

instance ToJSON Encounter where
	toJSON (Encounter (x, y) ev) =
		object
			[ "loc" .= object ["x" .= x, "y" .= y]
			, "ev" .= ev
			]
instance FromJSON Encounter where
	parseJSON (Object vs) = Encounter <$> decodeWith pairCoord vs "loc" <*> vs .: "ev"
	parseJSON _ = mzero

data Item = I IIndex Int
	deriving (Eq, Ord, Generic, Show, Read)
-- just like on the js side, this should take both items and tfnodes. so like...
--- data Alchemy = Alchemy ((Either [Item] [TfNode]) (Either [Item] [TfNode])
data Alchemy = A [Either Item TfNode] (Either Item TfNode)
	deriving (Eq, Ord, Generic, Show)

instance NFData IIndex
instance NFData Item
instance NFData Alchemy

data Opponent = Opponent
	{ oppName :: String
	, challengeType :: String
	, oppStats :: Stats
	, oppAI :: AI
	}
	deriving (Eq, Ord, Generic, Show, Read)

data Stats = Stats
	{ str :: Int -- strength
	, spd :: Int -- speed
	, def :: Int -- defense
	, vit :: Int -- vitality
	, int :: Int -- intelligence
	, cha :: Int -- charisma
	, res :: Int -- resistance
	, atn :: Int -- attune
	}
	deriving (Eq, Ord, Generic, Show, Read)

data AI = RandomAI [[ChallengeAction]]
	deriving (Eq, Ord, Generic, Show, Read)

data ChallengeAction = Wait | Block | Parry | Reflect | Hit | Cast | Charge | Heal
	deriving (Eq, Ord, Enum, Bounded, Generic, Show, Read)

instance NFData Opponent
instance NFData Stats
instance NFData AI
instance NFData ChallengeAction

instance ToJSON Opponent where
	toJSON (Opponent name kind stats ai) = object
		[ "name" .= name
		, "stats" .= stats
		, "ai" .= ai
		, "kind" .= kind
		]

instance ToJSON Stats where
	toJSON (Stats str spd def vit int cha res atn) = object
		[ "str" .= str
		, "spd" .= spd
		, "def" .= def
		, "vit" .= vit
		, "int_" .= int
		, "cha" .= cha
		, "res" .= res
		, "atn" .= atn
		]

instance ToJSON AI where
	toJSON (RandomAI cs) = object
		[ "type" .= ("random" :: String)
		, "actions" .= cs
		]

instance ToJSON ChallengeAction where
	toJSON x = case x of
		Wait -> "wait"
		Block -> "block"
		Parry -> "parry"
		Reflect -> "reflect"
		Cast -> "cast"
		Charge -> "charge"
		Hit -> "hit"
		Heal -> "heal"

data Scope l = Local l | Global GlobalFlag
	deriving (Eq, Ord, Generic, Show, Read)

instance Functor Scope where
	fmap _ (Global g) = Global g
	fmap f (Local l) = Local $ f l

-- TODO: add a hr rule?
data Event e a l
	= Narration [String]
	| Narration2 EventReel
	| Dialog String String [String]
	| Dialog2 String String EventReel
	| Dialog3 String EventReel EventReel
	| ItemModify [Item]
	| NodeGive TfNode
	| PushNode NodeAction [TfNode]
	| Choices [(String, a)]
	| Choices2 [(String, a, Maybe (Cond l))]
	| Challenge Opponent [[Item]]
	| FauxChoice String
	| Jump a
	| HarvestNode (Scope l) String [[Item]]
	-- | Challenge {AI TYPE/VALUES HERE} {THEN DROPS HERE}
	| Shop String [Alchemy]
	| BodyEditor
	| Close
	| DeactivateSelf
	| InlineBranch (Cond l) [Event e a l] [Event e a l]
	| SetFlag (Scope l) SetMod
	| UnsetFlag (Scope l)
	| SetEncounterData e
	| PushEncounterData Revise e (Scope l)
	| LoadAllSubData (Scope l) [String]
	| PopToSubData (Scope l) [String]
	| PeekLoadEncounterData (Scope l)
	| Branch [(Cond l, a)] a -- like a choice, but using a variable table lookup. the final `a` is the fallthrough if none of the conditions pass
	deriving (Show)

data NodeAction = CreatePath | ForceRemove TfNode | Overwriting TfNode
	deriving (Eq, Ord, Show, Generic)

instance NFData NodeAction

instance ToJSON NodeAction where
	toJSON a = case a of
		ForceRemove n -> object $ ["replace" .= n]
		CreatePath -> "createPath"
		Overwriting n -> object $ ["overwrite" .= n]

data Revise = Fix | Change
	deriving (Eq, Ord, Enum, Bounded, Show, Read)

-- strictly `Scene l a e =>` i think? maybe? but due to all the () use i can't actually say that V:
triMap :: ToJSON e => (e -> f) -> (a -> b) -> (l -> m) -> Event e a l -> Event f b m
triMap ef af lf e = case e of
	Choices cs -> Choices $ (\(s, a) -> (s, af a)) <$> cs
	Choices2 cs -> Choices2 $ (\(s, a, mcl) -> (s, af a, (fmap lf) <$> mcl)) <$> cs
	Jump a -> Jump $ af a
	InlineBranch cl e e' -> InlineBranch (lf <$> cl) (triMap ef af lf <$> e) (triMap ef af lf <$> e')
	SetFlag sl m -> SetFlag (lf <$> sl) m
	UnsetFlag sl -> UnsetFlag (lf <$> sl)
	SetEncounterData e -> SetEncounterData $ ef e
	-- the 'fix' branches break the functor law, fyi
	PushEncounterData Change e sl -> PushEncounterData Change (ef e) (lf <$> sl)
	PushEncounterData Fix e sl -> SetFlag (lf <$> sl) $ Set $ VE $ toElem e
	LoadAllSubData sl s -> LoadAllSubData (lf <$> sl) s
	PopToSubData sl s -> PopToSubData (lf <$> sl) s
	PeekLoadEncounterData sl -> PeekLoadEncounterData (lf <$> sl)
	Branch cs a -> Branch ((\(cl, a') -> (lf <$> cl, af a')) <$> cs) (af a)
	HarvestNode l s is -> HarvestNode (lf <$> l) s is

	Narration s -> Narration s
	Narration2 e -> Narration2 e
	Dialog i n s -> Dialog i n s
	Dialog2 i n e -> Dialog2 i n e
	Dialog3 i n e -> Dialog3 i n e
	ItemModify is -> ItemModify is
	NodeGive tf -> NodeGive tf
	PushNode a path -> PushNode a path
	Challenge o is -> Challenge o is
	FauxChoice s -> FauxChoice s
	Shop s as -> Shop s as
	BodyEditor -> BodyEditor
	Close -> Close
	DeactivateSelf -> DeactivateSelf

data GlobalFlag
	= TurnsElapsed
	| MetSonOfHamarTabal
	| MetClayMan
	| MetGanzerWorker
	| MetFlameDemon
	| MetLesserHellhound
	| MetDryad
	| MetGargoyle
	| MetSobekSheut
	| LookingForDryadForFarmer
--	| DryadsFound
	| DryadFoundData
	| VisitedHellGate
	| HoundKnightLeave
	| MetHoundKnight
	| VoidWormQuest
	| BarracksActive
	deriving (Eq, Ord, Enum, Bounded, Generic, Show, Read)

globaldisplay :: GlobalFlag -> String
globaldisplay = ('&' :) . fmap toLower . show

data SetMod = Set Val | Mod Modify
	deriving (Eq, Ord, Generic, Show, Read)

instance NFData SetMod

instance ToJSON SetMod where
	toJSON (Set v) = toJSON v
	toJSON (Mod m) = toJSON m

instance Num SetMod where
	(+) = numOpSet (+)
	(-) = numOpSet (-)
	(*) = numOpSet (*)
	fromInteger i = Set (fromIntegral i)
	abs (Set x) = Set (abs x)
	abs _ = error "can't use numeric ops on non-number vals"
	signum (Set x) = Set (signum x)
	signum _ = error "can't use numeric ops on non-number vals"

numOpSet :: (Int -> Int -> Int) -> SetMod -> SetMod -> SetMod
numOpSet f (Set a) (Set b) = Set (numOp f a b)
numOpSet _ _ _ = error "can't use numeric ops on non-number vals"


data Modify = Increment | Decrement
	deriving (Eq, Ord, Generic, Show, Read)

instance NFData Modify

instance ToJSON Modify where
	toJSON Increment = object ["mod" .= ("inc" :: String)]
	toJSON Decrement = object ["mod" .= ("dec" :: String)]


data LocationID = LID Int
	deriving (Eq, Ord, Generic, Show, Read)

instance NFData LocationID

instance ToJSON LocationID where
	toJSON (LID id) = object ["id" .= id]

-- on the client side elem will be stored in arrays, but i think we only ever want to place them in / read them from one at a time
data Val = VB Bool | VS String | VN Int | VA [Val] | VE Elem | VID LocationID
	deriving (Eq, Ord, Generic, Show, Read)

instance NFData Val

instance Num Val where
	(+) = numOp (+)
	(-) = numOp (-)
	(*) = numOp (*)
	fromInteger i = VN (fromIntegral i)
	abs (VN x) = VN (abs x)
	abs _ = error "can't use numeric ops on non-number vals"
	signum (VN x) = VN (signum x)
	signum _ = error "can't use numeric ops on non-number vals"

data Elem = E String [Val] [Val]
	deriving (Eq, Ord, Generic, Show, Read)

instance NFData Elem

class ToElem e where
	toElem :: e -> Elem

instance ToJSON e => ToElem e where
	toElem v = case fromJSON . toJSON $ v of
		Success e -> e
		Error msg -> error msg

numOp :: (Int -> Int -> Int) -> Val -> Val -> Val
numOp f (VN a) (VN b) = VN (f a b)
numOp _ _ _ = error "can't use numeric ops on non-number vals"

instance ToJSON Val where
	toJSON (VB b) = toJSON b
	toJSON (VS s) = toJSON s
	toJSON (VN n) = toJSON n
	toJSON (VE es) = toJSON es
	toJSON (VA arr) = toJSON arr
	toJSON (VID lid) = toJSON lid

instance FromJSON Val where
	parseJSON b@(Bool _) = VB <$> parseJSON b
	parseJSON s@(String _) = VS <$> parseJSON s
	parseJSON n@(Number _) = VN <$> parseJSON n
	parseJSON a@(Array _) = VA <$> parseJSON a
	parseJSON o@(Object _) = VE <$> parseJSON o
	parseJSON v = typeMismatch "Val" v

instance ToJSON Elem where
	toJSON (E name vals subs) = object
		[ "name" .= name
		, "data" .= toJSON vals
		, "subdata" .= toJSON subs
		]

instance FromJSON Elem where
	parseJSON (Object vs) = E
		<$> (vs .: "name")
		<*> (vs .: "data")
		<*> (vs .:? "subdata" .!= [])
	parseJSON e = error $ "bad elem decode on " ++ show e

data Cond l
	= C (Scope l) Op
	| IC [Item]
	| VAnd (Cond l) (Cond l)
	| VOr (Cond l) (Cond l)
	deriving (Show)

instance Functor Cond where
	fmap f (C s op) = C (fmap f s) op
	fmap _ (IC is) = IC is
	fmap f (VAnd a b) = VAnd (fmap f a) (fmap f b)
	fmap f (VOr a b) = VOr (fmap f a) (fmap f b)

data Cond' = C' String Op | IC' [Item] | VAnd' Cond' Cond' | VOr' Cond' Cond'
	deriving (Eq, Ord, Generic, Show, Read)

instance NFData Cond'

instance ToJSON Cond' where
	toJSON (C' flag op) = object ["flag" .= flag, "cond" .= op]
	toJSON (IC' items) = object ["flag" .= ("__hasitem" :: String), "items" .= items]
	toJSON (VAnd' a b) = object ["a" .= a, "b" .= b]
	toJSON (VOr' c d) = object ["c" .= c, "d" .= d]

data Op = IsSet | NotSet | Eq Val | Neq Val | Lt Val | Gt Val | Contains String
	deriving (Eq, Ord, Generic, Show, Read)

instance NFData Op

instance ToJSON Op where
	toJSON IsSet = object ["op" .= ("set" :: String)]
	toJSON NotSet = object ["op" .= ("notSet" :: String)]
	toJSON (Eq v) = object ["op" .= ("eq" :: String), "val" .= v]
	toJSON (Neq v) = object ["op" .= ("neq" :: String), "val" .= v]
	toJSON (Lt v) = object ["op" .= ("lt" :: String), "val" .= v]
	toJSON (Gt v) = object ["op" .= ("gt" :: String), "val" .= v]
	toJSON (Contains s) = object ["op" .= ("peek" :: String), "val" .= s]

-- make sure to keep this synced up with the enum in /static/js/item.js
data IIndex
	= DUMMY

	| LODESTONE
	| ORRERY_MECHANISM

	| RIVER_WATER
	| ETERNAL_FLAME
	| SCINTILLATING_CRYSTAL
	| MACHINE_PART

	| ASH_BLOSSOM
	| GNARLED_ROOT
	| BUDDING_BRANCH

	| SOUL_SKIN

	| SOULSTONE_ORE
	| RAW_FLAMECRYSTAL

	| DEMONS_BONE

	| GRAVEN_TABLET
	| ACCRETION_DISC

	| SUN_STONE
	| MOON_STONE
	| CHAOS_CRYSTAL
	| GLOW_CRYSTAL

	| LIONS_MANE -- lion
	| MANDRAKE -- human
	| GOATSBEARD -- goat
	| ADDERS_FORK -- snake
	| OX_EYE -- bull
	| CATTAIL -- tiger
	| HARTS_TONGUE -- deer
	| TONGUE_OF_DOG -- wolf
	| HORSE_HOOF -- unicorn
	| PIGS_TAIL -- boar

	| ASH
	| RUST

	deriving (Eq, Ord, Enum, Bounded, Generic, Show, Read)

data ProseItem
	= Raw Text
	| Punc Punctuation
	| BodyBranch (BodyGlob -> EventReel)
	| Markup (Markup ProseItem)
	| MarkupBlock (Markup EventReel)
	| Include Text
	| VarBranch Cond' ProseItem ProseItem

instance Show ProseItem where
	show (Raw t) = "Raw " <> show t
	show (Punc p) = "Punc " <> show p
	show (BodyBranch _) = "BodyBranch <function>"
	show (Markup m) = "Markup " <> show m
	show (MarkupBlock m) = "Markup " <> show m
	show (Include i) = "Include " <> show i
	show (VarBranch s t f) = "VarBranch " <> show s <> " " <> show t <> " " <> show f

data EncGlob b m = EncGlob
	{ _body :: b
	, _mind :: Mental m
	}
	deriving (Eq, Ord, Show, Read)

class (ToJSON b, ToJSON m, FromJSON b, FromJSON m) => EncounterEncoding b m | b -> m, m -> b where
	encEncode :: EncGlob b m -> Value
	encDecode :: Value -> Parser (EncGlob b m)

withData :: String -> Value -> Value
withData name data_ = object ["name" .= name, "data" .= data_]

withSubdata :: Value -> Value -> Value
withSubdata v subdata = case v of
	Object vs -> Object $ H.insert "subdata" subdata $ vs
	_ -> "bad subdata object"

extractRaws :: Value -> Maybe (String, [Value], Maybe [Value])
extractRaws (Object vs) = case (,,)
	<$> (fromJSON <$> H.lookup "name" vs)
	<*> (fromJSON <$> H.lookup "data" vs)
	<*> pure (fromJSON <$> H.lookup "subdata" vs) of -- this seems dumb
		Just (Success n, Success d, sd) -> case sd of
			Just (Success vs) -> Just (n, d, Just vs)
			Nothing -> Just (n, d, Nothing)
			_ -> Nothing
		_ -> Nothing
extractRaws _ = Nothing

body :: Lens (EncGlob b m) (EncGlob c m) b c
body f enc = (\b -> enc { _body = b }) <$> f (_body enc)

mental :: Lens' (EncGlob b m) (Mental m)
mental f enc = (\m -> enc { _mind = m }) <$> f (_mind enc)

mind :: Lens (EncGlob b m) (EncGlob b n) m n
mind f enc = (\m -> enc { _mind = (_mind enc) { _mental = m } })
	<$> f (_mental . _mind $ enc)

instance EncounterEncoding b m => ToJSON (EncGlob b m) where
	toJSON = encEncode

instance EncounterEncoding b m => FromJSON (EncGlob b m) where
	parseJSON = encDecode


data Mental a = Mental
	{ _name :: String
	, _nameRevealed :: Bool
	, _mental :: a
	}
	deriving (Eq, Ord, Show, Read)

__mental :: Lens (Mental a) (Mental b) a b
__mental f mental = (\m -> mental { _mental = m }) <$> f (_mental mental)

instance ToJSON a => ToJSON (Mental a) where
	toJSON (Mental n r v) = toJSON $ [toJSON n, toJSON r, toJSON v]
instance FromJSON a => FromJSON (Mental a) where
	parseJSON (Array vs) = if length vs == 3
		then case Mental
			<$> fromJSON (vs ! 0)
			<*> fromJSON (vs ! 1)
			<*> fromJSON (vs ! 2) of
				Success v -> return v
				_ -> mzero
		else mzero
	parseJSON _ = mzero

type EventReel = Accum [ProseItem] ()

include :: Scene i vars mob => vars -> Accum [ProseItem] ()
include = (\x -> Accum [x] ()) . Include . pack . vardisplay

instance IsString ProseItem where
	fromString s = Raw . pack $ s

-- ugh
-- instance IsString [ProseItem] where
-- 	fromString s = [fromString s]
{-
instance Convertible ProseItem Punctuation where
	safeConvert = Right . Punc

instance Convertible ProseItem (BodyGlob -> [ProseItem]) where
	safeConvert = Right . Branch

instance Convertible ProseItem (Markup ProseItem) where
	safeConvert = Right . Markup

instance Convertible ProseItem String where
	safeConvert = Right . Raw . pack

instance Convertible b a => Convertible (Accum b ()) a where
	safeConvert = fmap (\x -> Accum x ()) . safeConvert
-}
instance Convertible a a where
	safeConvert = Right

-- ughhhh
instance Convertible Punctuation (Accum [ProseItem] ()) where
	safeConvert = Right . (\x -> Accum x ()) . pure . Punc

instance Convertible (BodyGlob -> [ProseItem]) (Accum [ProseItem] ()) where
	safeConvert = Right . (\x -> Accum x ()) . pure . BodyBranch . ((\x -> Accum x ()) .)

instance Convertible (BodyGlob -> String) (Accum [ProseItem] ()) where
	safeConvert = Right . (\x -> Accum x ()) . pure . BodyBranch . ((\x -> Accum x ()) .) . fmap (pure . (convert :: String -> ProseItem))


instance Convertible (BodyGlob -> Accum [ProseItem] ()) (Accum [ProseItem] ()) where
	safeConvert = Right . (\x -> Accum x ()) . pure . BodyBranch

instance Convertible (Markup ProseItem) (Accum [ProseItem] ()) where
	safeConvert = Right . (\x -> Accum x ()) . pure . Markup

instance Convertible (Markup (Accum [ProseItem] ())) (Accum [ProseItem] ()) where
	safeConvert = Right . (\x -> Accum x ()) . pure . MarkupBlock

instance Convertible String (Accum [ProseItem] ()) where
	safeConvert = Right . (\x -> Accum x ()) . pure . Raw . pack

instance Convertible String ProseItem where
	safeConvert = Right . Raw . pack

c :: Convertible a (Accum [ProseItem] ()) => a -> Accum [ProseItem] ()
c = convert
{-
infixl 1 //
(//) :: (Monoid b, Convertible b a) => b -> a -> b
base // new = base <> convert new
-}
infixl 6 /
(/) :: a -> (a -> b -> Text) -> b -> [ProseItem]
(/) a f b = [Raw $ f a b]

s :: Text -> BodyGlob -> Text
s t b = if cockCount b == 1 then t else pluralize t
	where
		pluralize t = case inflect (unpack t) Plural of
			Right ts -> pack ts
			Left e -> error $ e

plur :: Text -> Text -> BodyGlob -> Text
plur plur_ sing b = if cockCount b == 1 then sing else plur_

cloaca :: a -> a -> BodyGlob -> a
cloaca cl a b = if Samp.hasAsshole b then a else cl

infixl 5 //
(//) :: Monad m => m a -> m b -> m b
(//) = (>>)
{- (//) :: Convertible (Accum [ProseItem] ()) b => Accum [ProseItem] a -> b -> Accum [ProseItem] ()
(//) a b = a >> c b -}

data Tag = Strong | Em | Br | Quote
	deriving (Eq, Ord, Enum, Bounded, Generic, Show)

instance NFData Tag

data Markup a = M Tag a
	deriving (Eq, Ord, Generic, Show)

instance NFData a => NFData (Markup a)

strong :: ProseItem -> Markup ProseItem
strong = M Strong

em :: ProseItem -> Markup ProseItem
em = M Em

quote :: a -> Markup a
quote = M Quote

br :: Markup ProseItem
br = M Br (Raw mempty)

parseProse :: BodyGlob -> [ProseItem] -> [[StringOrMarkup]]
parseProse body = go . (debranch =<<)
	where
		go :: [ProseItem] -> [[StringOrMarkup]]
		go = (\(ps, final) -> reverse $ reverse final : ps)
			. foldl' parseItem ([], mempty)

		-- debranch before we fold through since otherwise the punctuation can get mangled
		debranch :: ProseItem -> [ProseItem]
		debranch (BodyBranch f) = debranch =<< (getAccum $ f body)
		debranch i = [i]

		parseItem :: ([[StringOrMarkup]], [StringOrMarkup]) -> ProseItem -> ([[StringOrMarkup]], [StringOrMarkup])
		parseItem (ps, cur) (Raw t) = (ps, if null cur
			then [S $ unpack t]
			else if shouldSpace t
				then cur `append` S (' ' : unpack t)
				else cur `append` S (unpack t))
		parseItem (ps, cur) (Punc Paragraph) = (reverse cur : ps, mempty)
		parseItem (ps, cur) (Punc p) = (ps, cur `append` S pt)
			where pt = case p of
				Join -> ""
				Hyphen -> "-"
				Contraction -> "'"
				Comma -> ","
				Colon -> ":"
				Semicolon -> ";"
				Emdash -> " --"
				Period -> "."
				Bang -> "!"
				Paragraph -> ""
		parseItem (ps, cur) (Markup m) = (ps, cur `append` markup m)
		parseItem (ps, cur) (MarkupBlock m) = (reverse cur : ps, [markup' m])
		parseItem (ps, cur) (Include l) = (ps, cur `append` L (unpack l))
		-- la la always space a branch, because it would be a hassle to try to figure out when you really need to. plus, like, don't do subword variable branches?
		parseItem (ps, cur) (VarBranch sc t f) = (ps, cur `append` S " " `append` V sc (concat $ parseProse body [t]) (concat $ parseProse body [f]))
		parseItem _ (BodyBranch _) = error "unremoved branch in prose segment"

		shouldSpace :: Text -> Bool
		shouldSpace x | T.null x = False
		shouldSpace x | T.head x `elem` [' ', '-', '\'', ',', ':', ';', '.', '!'] = False
		shouldSpace _ = True

		append :: [StringOrMarkup] -> StringOrMarkup -> [StringOrMarkup]
		append ((S s'):f) (S s) = (S $ s' <> s) : f
		append r n = n : r

		markup :: Markup ProseItem -> StringOrMarkup
		markup (M t p) = M' $ M t $ case p of
			Raw s -> if shouldSpace s
				then [[S $ ' ' : unpack s]]
				else [[S $ unpack s]]
			_ -> error "bad markup contents"
		markup' :: Markup EventReel -> StringOrMarkup
		markup' (M t p) = M' $ M t $ vals
			where
				vals :: [[StringOrMarkup]]
				vals = parseProse body $ getAccum p

type MarkedScene i vars mob = BodyGlob -> mob -> [Event mob i vars]
type SceneBloc = (String, [RawEvent])

-- this fundep means `i` and `vars` must be uniquely paired together, so no events sharing the same vartable. that's... mostly just to get rid of an ambigious type error when using `vardisplay` otherwise.
-- tojson since 1. it does actually have to be tojson to do encoding elsewhere, and 2. since we use toelem when encoding encounter data, which requires tojson
class ToJSON mob => Scene i vars mob | i -> mob vars, vars -> i mob, mob -> i vars where
	link :: i -> MarkedScene i vars mob
	display :: i -> String
	vardisplay :: vars -> String
	varread :: String -> Maybe vars
	varread _ = Nothing
	fauxTree :: mob -> Maybe (Tree String)
	fauxTree _ = Nothing

-- do we actually need this at all ever?
readLocal :: Scene i v mob => Value -> Maybe [(v, Val)]
readLocal (Object obj) = case H.lookup "local" obj of
	Nothing -> Nothing
	Just a -> case fromJSON a of
		Success b -> mapM (\(k, v) -> (,) <$> varread k <*> pure v) b
		Error _ -> Nothing
	where
		r :: Result a -> Maybe a
		r (Success a) = Just a
		r (Error _) = Nothing
readLocal _ = Nothing



data RawEvent
	= Narration' [[StringOrMarkup]]
	| Dialog' String [StringOrMarkup] [[StringOrMarkup]]
	| ItemModify' [Item]
	| NodeGive' TfNode
	| PushNode' NodeAction [TfNode]
	| Choices' [(String, String, Maybe Cond')]
	| Challenge' Opponent [[Item]]
	| FauxChoice' String
	| Jump' String
	| HarvestNode' String String [[Item]]
	-- | Combat' ???
	| Shop' String [Alchemy]
	| BodyEditor'
	| SetFlag' String (Maybe SetMod)
	| SetEncounterData' Elem
	| PeekLoadEncounterData' String
	| LoadAllSubData' String [String]
	| PopToSubData' String [String]
	| InlineBranch' Cond' [RawEvent] [RawEvent]
	| Branch' [(Cond', String)] String
	| Close'
	| DeactivateSelf'
	deriving (Eq, Ord, Generic, Show)


instance NFData RawEvent

sceneDisplay :: (Scene i vars mob, Eq i) => BodyGlob -> mob -> MarkedScene i vars mob -> [SceneBloc]
sceneDisplay b m root = snd $ expandSceneFlow b m baseIndices baseTable
	where
		baseTable = pure ("_activate", displayEvent b m <$> root b m)
		baseIndices = flowTargets $ root b m

-- there are two things we want to do w/ each event / event index we get: one is turn it into [RawEvent], the other is find the choices and get those events. but we don't want to get events we've already got. so we want to accumulate a [SceneBloc] map, basically, and pass that through while we attach new keys to it
expandSceneFlow :: (Scene i vars mob, Eq i) => BodyGlob -> mob -> [i] -> [SceneBloc] -> ([i], [SceneBloc])
expandSceneFlow _ _ [] scenekey = ([], scenekey)
expandSceneFlow b m is scenekey = let
		(is', scenekey') = first concat $ unzip $ foo `mapMaybe` nub is
	in
		expandSceneFlow b m is' (scenekey ++ scenekey')
	where
		-- foo :: Scene i mob => i -> Maybe ([i], SceneBloc)
		foo i = case lookup (display i) scenekey of
			Just _ -> Nothing
			Nothing -> let
					evs = link i b m
				in
					Just (flowTargets evs, (display i, displayEvent b m <$> evs))

flowTargets :: [Event e i a] -> [i]
flowTargets = foldr extractTarget []
	where
		extractTarget :: Event e i a -> [i] -> [i]
		extractTarget (Choices chs) vs = fmap snd chs ++ vs
		extractTarget (Choices2 chs) vs = fmap (^. _2) chs ++ vs
		extractTarget (Jump t) vs = t : vs
		-- we're not doing (can't do w/ the current state) a reachability check
		extractTarget (Branch conds f) vs = f : fmap snd conds ++ vs
		extractTarget _ vs = vs

displayEvent :: Scene i vars mob => BodyGlob -> mob -> Event mob i vars -> RawEvent
displayEvent _ _ (Narration ls) = Narration' $ fmap (pure . S) $ ls
displayEvent b _ (Narration2 g) = Narration' $ parseProse b $ getAccum g
displayEvent _ _ (Dialog i n ls) = Dialog' i [S n] $ fmap (pure . S) $ ls
displayEvent b _ (Dialog2 i n g) = Dialog' i [S n] $ parseProse b $ getAccum g
displayEvent b _ (Dialog3 i n g) = Dialog' i (concat $ parseProse b $ getAccum n) $ parseProse b $ getAccum g
displayEvent _ _ (ItemModify is) = ItemModify' is
displayEvent _ _ (NodeGive tf) = NodeGive' tf
displayEvent _ _ (PushNode a path) = PushNode' a path
displayEvent _ _ (Choices vs) = Choices' $ fmap ((\(a, b) -> (a, b, Nothing)) . (_2 %~ display)) vs
displayEvent _ _ (Choices2 vs) = Choices' $ fmap ((_3 %~ fmap concrete) . (_2 %~ display)) vs
displayEvent _ _ (Challenge opponent drops) = Challenge' opponent drops
displayEvent _ _ (FauxChoice c) = FauxChoice' c
displayEvent _ _ (Jump t) = Jump' $ display t
displayEvent _ _ (HarvestNode l b is) = HarvestNode' (scopedisplay l) b is
displayEvent _ _ (Shop n as) = Shop' n as
displayEvent _ _ BodyEditor = BodyEditor'
displayEvent _ _ Close = Close'
displayEvent _ _ DeactivateSelf = DeactivateSelf'

displayEvent b m (InlineBranch cond t f) =
	InlineBranch' (concrete cond) (displayEvent b m <$> t) (displayEvent b m <$> f)
displayEvent _ _ (SetFlag s val) = SetFlag' (scopedisplay s) (Just val)
displayEvent _ _ (UnsetFlag s) = SetFlag' (scopedisplay s) Nothing
displayEvent _ _ (SetEncounterData m') = SetEncounterData' $ toElem m'
displayEvent _ _ (PushEncounterData _ m' s) = SetFlag' (scopedisplay s) $ Just $ Set $ VE $ toElem m'
displayEvent _ _ (PeekLoadEncounterData s) = PeekLoadEncounterData' (scopedisplay s)
displayEvent _ _ (LoadAllSubData s encs) = LoadAllSubData' (scopedisplay s) encs
displayEvent _ _ (PopToSubData s encs) = PopToSubData' (scopedisplay s) encs
displayEvent _ _ (Branch conds fallback) = Branch' (fmap (\(cond, branch) -> (concrete cond, display branch)) conds) (display fallback)

scopedisplay :: Scene i l mob => Scope l -> String
scopedisplay (Local l) = vardisplay l
scopedisplay (Global g) = globaldisplay g

nBranch :: (Scene i l mob, Convertible a ProseItem, Convertible b ProseItem) => (Cond l) -> a -> b -> EventReel
nBranch cond t f = (\x -> Accum [x] ()) $ VarBranch (concrete cond) (convert t) (convert f)

concrete :: Scene i l mob => Cond l -> Cond'
concrete (C s v) = C' (scopedisplay s) v
concrete (IC items) = IC' items
concrete (VAnd a b) = VAnd' (concrete a) (concrete b)
concrete (VOr a b) = VOr' (concrete a) (concrete b)

instance ToJSON IIndex where
	toJSON = toJSON . fromEnum

instance FromJSON IIndex where
	parseJSON (Number n) = case safeToEnum . round $ n of
		Just n' -> return n'
		_ -> mzero
	parseJSON _ = mzero

instance ToJSON Item where
	toJSON (I item count) = object ["item" .= item, "count" .= count]

instance ToJSON Alchemy where
	toJSON (A is node) = object
		[ "cost" .= fmap (either toJSON toJSON) is
		, "sale" .= either toJSON toJSON node
		]

choicelist :: (String, String, Maybe Cond') -> Value
choicelist (t, ev, Nothing) = object ["t" .= t, "v" .= ev]
choicelist (t, ev, Just cond) = object
	[ "t" .= t
	, "v" .= ev
	, "cond" .= cond
	]

instance ToJSON RawEvent where
	toJSON e = object $ case e of
		Narration' texts ->
			[ "type" .= ("narration" :: String)
			, "texts" .= texts
			]
		Dialog' img name texts ->
			[ "type" .= ("dialog" :: String)
			, "img" .= img
			, "name" .= name
			, "texts" .= texts
			]
		ItemModify' items ->
			[ "type" .= ("itemModify" :: String)
			, "items" .= items
			]
		NodeGive' node ->
			[ "type" .= ("nodeGive" :: String)
			, "node" .= node
			]
		PushNode' action path ->
			[ "type" .= ("pushNode" :: String)
			, "action" .= action
			, "path" .= path
			]
		Choices' choices ->
			[ "type" .= ("choices" :: String)
			, "choices" .= (choicelist <$> choices)
			]
		FauxChoice' choice ->
			[ "type" .= ("fauxChoice" :: String)
			, "choice" .= choice
			]
		Challenge' opponent drops ->
			[ "type" .= ("challenge" :: String)
			, "opponent" .= opponent
			, "drops" .= drops
			]
		HarvestNode' flag blurb items ->
			[ "type" .= ("harvestNode" :: String)
			, "text" .= blurb
			, "flag" .= flag
			, "items" .= items
			]
		-- set variable named `flag` to value `val`
		SetFlag' flag mval -> case mval of
			Just val ->
				[ "type" .= ("setFlag" :: String)
				, "flag" .= flag
				, "val" .= val
				]
			Nothing ->
				[ "type" .= ("setFlag" :: String)
				, "flag" .= flag
				, "val" .= Null
				]
		SetEncounterData' encdata ->
			[ "type" .= ("setEncounterData" :: String)
			, "enc" .= encdata
			]
		PeekLoadEncounterData' flag ->
			[ "type" .= ("peekLoadEncounterData" :: String)
			, "flag" .= flag
			]
		LoadAllSubData' flag allowed ->
			[ "type" .= ("loadAllSubData" :: String)
			, "flag" .= flag
			, "allowed" .= allowed
			]
		PopToSubData' flag allowed ->
			[ "type" .= ("popToSubData" :: String)
			, "flag" .= flag
			, "allowed" .= allowed
			]
		Jump' t ->
			[ "type" .= ("jump" :: String)
			, "target" .= t
			]
		InlineBranch' cond t f ->
			[ "type" .= ("inlineBranch" :: String)
			, "cond" .= cond
			, "pass" .= t
			, "fail" .= f
			]
		Branch' conds default_ ->
			[ "type" .= ("branch" :: String)
			, "conds" .= fmap (\(c, b) -> object ["cond" .= c, "branch" .= b]) conds
			, "default_" .= default_
			]
		Shop' verb sales ->
			[ "type" .= ("shop" :: String)
			, "verb" .= verb
			, "sales" .= sales
			]
		Close' ->
			[ "type" .= ("close" :: String)
			]
		BodyEditor' ->
			[ "type" .= ("bodyEditor" :: String)
			]
		DeactivateSelf' ->
			[ "type" .= ("deactivateSelf" :: String)
			]

-- TODO: this needs to be renamed something like 'eventAPIData', it's getting more and more inaccurate as i keep piling stuff into the JSON format
data StringOrMarkup
	= S String
	| M' (Markup [[StringOrMarkup]])
	| L String
	| V Cond' [StringOrMarkup] [StringOrMarkup]
	deriving (Eq, Ord, Generic, Show)

instance NFData StringOrMarkup

instance ToJSON StringOrMarkup where
	toJSON (S s) = toJSON s
	toJSON (M' (M t s)) = case t of
		Br -> object ["t" .= ts]
		_ -> object ["t" .= ts, "s" .= s]
		where
			ts :: String
			ts = case t of
				Strong -> "b"
				Em -> "i"
				Br -> "br"
				Quote -> "q"
	toJSON (L l) = object ["l" .= l]
	toJSON (V cond t f) = object
		[ "cond" .= cond
		, "t" .= t
		, "f" .= f
		]
{-
instance FromJSON StringOrMarkup where
	parseJSON (String s) = S s
	parseJSON (Object vs) = M' ...
-}

decodeWith :: (Value -> Parser a) -> Object -> Text -> Parser a
decodeWith dec obj key = case H.lookup key obj of
	Nothing -> fail $ "key " ++ show key ++ " not present"
	Just v -> dec v

pairCoord :: (FromJSON a, FromJSON b) => Value -> Parser (a, b)
pairCoord (Object vs) = (,) <$> vs .: "x" <*> vs .: "y"
pairCoord _ = mzero

bodyFail :: (Show a, Show b) => a -> b -> c
bodyFail s f = error $ unwords ["no handle in", show s, "for body w/", show f]

-- this is a newtype-instance for all bounded/enum types, to make randomly generating basic monsters easier
newtype GeneratorRandom a = GeneratorRandom { getGenRand :: a }
	deriving (Enum, Bounded)

instance (Bounded a, Enum a) => Random (GeneratorRandom a) where
	randomR (GeneratorRandom lo, GeneratorRandom hi) g =
		first toEnum . randomR (fromEnum lo, fromEnum hi) $ g
	random = randomR (minBound, maxBound)

genRandom :: (Bounded a, Enum a, RandomGen g) => Rand g a
genRandom = getGenRand <$> liftRand random



guard' :: MonadPlus m => Bool -> a -> m a
guard' False _ = mzero
guard' True a = return a


--- SAMPLING STUFF HERE

hand :: BodyGlob -> Accum [ProseItem] ()
hand = Samp.hand

hands :: BodyGlob -> Accum [ProseItem] ()
hands = Samp.hands

feet :: BodyGlob -> Accum [ProseItem] ()
feet = Samp.feet

foot :: BodyGlob -> Accum [ProseItem] ()
foot = Samp.foot

forelegs :: BodyGlob -> Accum [ProseItem] ()
forelegs b = case legCount b of
	4 -> case fromMaybe Samp.Feet $ fmap Samp.animalFootType $ bodySpecies Legs b of
		Samp.Paws -> "forepaws"
		_ -> "forelegs"
	2 -> "legs"
	_ -> "foremost legs"

hind :: BodyGlob -> Accum [ProseItem] ()
hind b = case legCount b of
	2 -> empty
	4 -> "hind"
	_ -> "hindmost"

{-
case bodySpecies Legs b of
	Just Unicorn -> "fat horse ass"
	Just Horse -> "fat horse ass"
	Just Bull -> "huge, meaty bull ass"
	_ -> "fat ass"
-}
animalAss :: BodyGlob -> Accum [ProseItem] ()
animalAss b = case bodySpecies Legs b of
	Just Unicorn -> partAdjective Ass // "horse ass"
	--Just Horse -> partAdjective Ass // "horse ass"
	Just Bull -> "thick, meaty bull ass"
	Just Human -> partAdjective Ass // "ass"
	Just x -> partAdjective Ass // c (animalName x) // "ass"
	_ -> partAdjective Ass // "ass"

-- FIXME: make this do something!
{-
partadjective legs: {meaty/hairy/scaly/etc}
partadjective ass: {fuzzy}
legs: {potential adjective: hairy, goatish, w/e}
chest: 'hairy' if you got that hair flag
cockhead: sharp, flat, gnarled

partAdjective :: Part -> BodyGlob -> b
partAdjective p b = case partSpecies p b of
	Just Human -> ""
	Just Goat -> "hairy"
	Just Bull -> "coarsely-haired"
	Just Snake -> "scaled"
	_ -> ""

for balls:
	low-hanging

-}
partAdjective :: (Locatable a, Show a) => a -> Accum [ProseItem] ()
partAdjective p = case show p of
	"Cockhead" -> wrapper $ \b -> case bodySpecies p b of
		Nothing -> "needlelike"
		Just a -> case a of
			Wolf -> "pointed"
			Deer -> "gnarled"
			Goat -> "squirming"
			Bull -> "rumpled"
			Lion -> "barbed"
			Tiger -> "barbed"
			Dragon -> "dagger-shaped"
			Unicorn -> "flat" -- or 'broad'?
			Boar -> "corkscrewing"
			Human -> "fat"
			Snake -> "spiny" -- or 'ridged'?
			a -> mempty --error $ show a
	"Ass" -> wrapper $ \b -> case bodySpecies p b of
		Just Unicorn -> "fat"
		Just Bull -> "thick, meaty"
		Just Goat -> "hairy"
		Just Wolf -> "hairy"
		_ -> case snd_ <$> bodyCovering p b of
			Just (FMat _ Protoplasm) -> "protoplasmic"
			Just (FMat _ Scales) -> "scaled"
			_ -> "fat"
	"Underbelly" -> wrapper $ \b -> case bodyMat_ Chest b of
			Solid (FMat _ x) -> case x of
				_ | x `elem` [Fur, Coat] -> "hairy"
					| x `elem` [Skin, Scales] -> "smooth"
					| otherwise -> mempty
			_ -> mempty
	"Shaft" -> wrapper $ \b -> case bodySpecies p b of
		Just Bull -> "gnarled"
		Just Snake -> "ridged"
		Just Tiger -> "barbed"
		Just Lion -> "barbed"
		Nothing -> case snd_ <$> bodyCovering p b of
			Just (FMat _ Protoplasm) -> "rubbery"
			_ -> mempty
		_ -> mempty
	_ -> return () -- error "partAdjective"
	where
		wrapper = (`Accum` ()) . pure . BodyBranch

-- output: the name of the material of the part
materialWord :: Locatable a => a -> BodyGlob -> Accum [ProseItem] ()
materialWord a body = case Samp.bodyCovering a body of
	Nothing -> error $ "missing part (" <> show a <> ")"
	Just (_, FMat _ mat, _) -> c $ Samp.matword mat


-- fake body-sampling functions here:
-- replaced by bodyHeight???
bodySize :: BodyGlob -> Int
bodySize _ = 0

muzzle :: BodyGlob -> Accum [ProseItem] ()
muzzle b = case bodySpecies Head b of
	Just x | hasMuzzle x -> "muzzle"
		| hasSnout x -> "snout"
	_ -> "mouth"

groan :: BodyGlob -> Accum [ProseItem] ()
groan b = case bodySpecies Head b of
	Just Unicorn -> "whinny"
	--Just Horse -> "whinny"
	Just Bull -> "low"
	Just Goat -> "bleat"
	Just Wolf -> "howl"
	Just Snake -> "hiss"
	-- TODO: the rest of the species here
	Just Lion -> "roar"
	_ -> "groan"

groans :: BodyGlob -> Accum [ProseItem] ()
groans b = case bodySpecies Head b of
	Just Unicorn -> "whinnies"
	--Just Horse -> "whinnies"
	Just Bull -> "lows"
	Just Goat -> "bleats"
	Just Wolf -> "howls"
	Just Snake -> "hisses"
	-- TODO: the rest of the species here
	Just Lion -> "roars"
	_ -> "groans"

groaning :: BodyGlob -> Accum [ProseItem] ()
groaning b = case bodySpecies Head b of
	Just Unicorn -> "whinnying"
	--Just Horse -> "whinnying"
	Just Bull -> "lowing"
	Just Goat -> "bleating"
	Just Wolf -> "howling"
	Just Snake -> "hissing"
	-- TODO: the rest of the species here
	Just Lion -> "roaring"
	_ -> "groaning"

growl :: BodyGlob -> Accum [ProseItem] ()
growl b = case bodySpecies Head b of
	Just Unicorn -> "snort"
	--Just Horse -> "snort"
	Just Bull -> "snort"
	Just Snake -> "hiss"
	_ -> "growl"

bellow :: BodyGlob -> Accum [ProseItem] ()
bellow b = case bodySpecies Head b of
	Just Lion -> "roar"
	Just Tiger -> "roar"
	Just Snake -> "yowl"
	Just Wolf -> "howl"
	Just Unicorn -> "snort and huff"
	_ -> "bellow"

cockhead :: BodyGlob -> Accum [ProseItem] ()
cockhead b = case bodySpecies Cock b of
	Just Wolf -> "sharp" // c ("cocktip"/s)
	--Just Horse -> "flat" // c ("cockhead"/s)
	Just Human -> "fat" // c ("cockhead"/s)
	Just Bull -> "gnarled" // c ("cockhead"/s)
	_ -> c ("cockhead"/s)




fingertips :: BodyGlob -> Accum [ProseItem] ()
fingertips b = case handType b of
	Claw -> "clawtip"
	Hand -> "fingertips"
	Paw -> "pawtips"
	-- _ -> "fingertips"

fingernail :: BodyGlob -> Accum [ProseItem] ()
fingernail b = case handType b of
	Claw -> "tip"
	Hand -> "fingernail"
	Paw -> "clawtip"

-- TODO: hm i feel like 'hooftip' or something like it should be here
finger :: BodyGlob -> Accum [ProseItem] ()
finger b = case handType b of
	Claw -> "claw"
	Hand -> "finger"
	_ -> "finger"

fingers :: BodyGlob -> Accum [ProseItem] ()
fingers b = case handType b of
	Claw -> "claws"
	Hand -> "fingers"
	_ -> "fingers"

thumb :: BodyGlob -> Accum [ProseItem] ()
thumb b = case handType b of
	Claw -> "spur"
	Hand -> "thumb"
	Paw -> "dewclaw"

stomach :: BodyGlob -> Accum [ProseItem] ()
stomach b = case locoplan b of
	Basic Slithering -> "underbelly"
	_ -> "stomach"

teeth :: BodyGlob -> Accum [ProseItem] ()
teeth b = case bodySpecies Head b of
	Just x | hasFangs x -> "fangs"
		| hasTusks x -> "tusks"
	_ -> "teeth"

ankles :: BodyGlob -> Accum [ProseItem] ()
ankles b = case footType b of
	Hooves -> "hocks"
	ClovenHooves -> "hocks"
	_ -> "ankles"

-- here in side/sides/hips we see three very different ways of trying to figure this out :V
side :: BodyGlob -> Accum [ProseItem] ()
side b = case bodyMat_ (Waist Fore) b of
	Solid (FMat _ Skin) -> "side"
	Solid (FMat _ Fur) -> "flank"
	Solid (FMat _ Coat) -> "flank"
	_ -> "side"

sides :: BodyGlob -> Accum [ProseItem] ()
sides b = case locoplan b of
	Basic Quadruped -> "flanks"
	Basic Centaur -> "flanks"
	_ -> case footType b of
		Hooves -> "flanks"
		ClovenHooves -> "flanks"
		_ -> "sides"

hips :: BodyGlob -> Accum [ProseItem] ()
hips b = case locoplan b of
	Basic Slithering -> "sides"
	Basic Naga -> "hips"
	_ -> case bodySpecies Legs b of
		Just Human -> "hips"
		Just Goat -> "haunches"
		Just Bull -> "haunches"
		Just Unicorn -> "flanks"
		--Just Horse -> "flanks"
		Just Wolf -> "flanks"
		_ -> "hips"

unsheathed :: BodyGlob -> Accum [ProseItem] ()
unsheathed b = case dickLocation b of
	Just DickExternal -> mempty
	Nothing -> mempty
	_ -> "unsheathed"

unsheathing :: BodyGlob -> Accum [ProseItem] ()
unsheathing b = case dickLocation b of
	Just DickExternal -> mempty
	Nothing -> mempty
	_ -> "unsheathing"

sheath :: BodyGlob -> Accum [ProseItem] ()
sheath b = (c :: String -> Accum [ProseItem] ()) $ case dickLocation b of
	Just DickSheathed -> "sheath"
	Just DickCloaca -> "cloaca"
	Just DickVent -> "vent"
	Just DickExternal -> "foreskin" -- uh
	Nothing -> if Samp.hasCloaca b
		then "cloaca"
		else "nub" -- does this ever get used? IT SHOULDN'T

cock :: BodyGlob -> Accum [ProseItem] ()
cock b = if cockCount b == 2 then "twin" // c ("cock"/s) else case bodySpecies Cock b of
	--Just Horse -> "equine" // c ("cock"/s)
	Just Unicorn -> "equine" // c ("cock"/s)
	_ -> c ("cock"/s)

number :: Integral a => a -> Accum [ProseItem] ()
number x | x >= 0 && x < 10 = ones !! fromIntegral x
	| otherwise = error "number"
	where
		ones = ["zero", "one", "two", "three", "four", "five", "six", "seven", "eight", "nine"]

cockPlural :: String -> String -> BodyGlob -> Accum [ProseItem] ()
cockPlural sing plur b = c $ if cockCount b == 1 then sing else plur

empty :: Accum [ProseItem] ()
empty = mempty
