{-# LANGUAGE MultiParamTypeClasses #-}
module Sites.Hell.Events.Hubur.Amduscias
	( Amduscias(..)
	, deck
	, scene

	, name
	, bits
	, variants
	) where

import Prelude hiding ((/), (^))

import Server.JSON
import Sites.Hell.Events.Hubur

import Sites.Hell.Events.SceneUtils hiding ((/))
import Sites.Hell.Events.Construction ((/))

name :: String
name = "amduscias"

data Amduscias = Amduscias
	{ occupation :: Occupation
	, decoration :: Decoration
	}
	deriving (Eq, Ord, Show, Read)

data Occupation = Scholar | Bard
	deriving (Eq, Ord, Enum, Bounded, Show, Read)

data Decoration = Tattoo | Chain | Leather
	deriving (Eq, Ord, Enum, Bounded, Show, Read)

instance Random Amduscias where
	randomR _ = random
	random = getRand $ Amduscias
		<$> genRandom
		<*> genRandom

instance ToJSON Amduscias where
	toJSON (Amduscias o d) = object
		[ pack "name" .= name
		, pack "data" .= [fromEnum o, fromEnum d]
		]

bits :: Value -> Value
bits v = object
	[ pack "name" .= name
	, pack "data" .= [v, Null]
	]

variants :: [(MapZone, Occupation)]
variants =
	[ (FactoryStacks, Scholar)
	, (FactoryPulpit, Bard)
	]

instance FromJSON Amduscias where
	parseJSON (Object vs) = case (parse (.: pack "name") vs, parse (.: pack "data") vs) of
			(Success pName, Success [o, d])
				| pName == name ->
					case Amduscias
						<$> safeToEnum o
						<*> safeToEnum d of
							Nothing -> mzero
							Just a -> return a
			_ -> mzero
	parseJSON _ = mzero

deck :: RandomGen g => Rand g (Enumerable Amduscias)
deck = do
	decoration <- liftRand $ shuffle [Tattoo, Chain, Leather]
	return $ Amduscias
		<$> pure Scholar
		<*> from decoration

data Index = Dummy
	deriving (Eq, Ord, Show, Read)

instance Scene Index () Amduscias where
	link f = case f of
		_ -> error "bad link"

	display = fmap toLower . show
	vardisplay = fmap toLower . show

scene :: Encounter Index () Amduscias
scene b a = describe b a -- <> error "bad scene"

--- actual event start

describe :: Encounter Index () Amduscias
describe _ a =
	[ Narration2 $ "It's an Amduscias"
		/ case occupation a of
			Scholar -> "scholar"
			Bard -> "bard"
		/ ", floating about. He's a well-muscled unicorn-man, having grey-brown fur and amber eyes"
		/ case decoration a of
			Tattoo -> ", with black tattoos visible through his short coat: tightly-packed script, in curving and hooked lines across his body"
			Chain -> ". There's a length of silvery chain wrapped in an intricate interlocking pattern across his shoulders and chest, winding down to loop between his fingers. The chains make a muted jingling when he shifts"
			Leather -> ". He's wearing a heavy leather harness, with bands and straps across his biceps and stomach"
		/ ". But at the waist he's wearing a mess of shredded skirts that ultimately open into nothing, just billowing around his body as he levitates about. His skirt is gathered up into tight rolls of wrapped fabric across his abs and hips, and then spills open into layered folds, hanging to what would be about mid-calf. Rarely his skirt catches as if pressed against his hips or thighs, outlining illusory flesh."
		/ Paragraph / case occupation a of
			Scholar -> "He's reading a book, and a half-dozen other tomes orbit around him in an interlocking configuration."
			Bard -> "He's playing a tune on a trumpet, with a half-dozen other brass instruments floating in an orbit around him."
	]
