{-# LANGUAGE MultiParamTypeClasses #-}

module Sites.Hell.Events.Hubur.Barracks
	( Barracks(..)
	, deck
	, scene

	, name
	, bits
	, variants
	) where

import Prelude hiding ((/))

import Control.Applicative

import Sites.Hell.Events.Construction ((/))
import Sites.Hell.Events.SceneUtils
import Server.JSON

import Sites.Hell.Events.Hubur
import Sites.Hell.Events.Hubur.MinotaurMiner (MinotaurMiner(..))
import Sites.Hell.Events.Hubur.SlimeGuy (SlimeGuy(..))

name :: String
name = "miningbarracks"

data Barracks
	= EmptyBarracks
	| ActiveBarracks (Maybe Participant) (Maybe Participant)
	deriving (Eq, Ord, Show, Read)

data Participant = Slime SlimeGuy | Minotaur MinotaurMiner
	deriving (Eq, Ord, Show, Read)

instance ToJSON Participant where
	toJSON (Slime s) = toJSON s
	toJSON (Minotaur m) = toJSON m

instance Random Barracks where
	randomR _ = random
	random = getRand $ pure EmptyBarracks

deck :: RandomGen g => Rand g (Enumerable Barracks)
deck = return $ pure $ EmptyBarracks

instance ToJSON Barracks where
	toJSON EmptyBarracks = object
		[ pack "name" .= name
		, pack "data" .= ([0] :: [Int])
		]
	toJSON (ActiveBarracks Nothing Nothing) = object
		[ pack "name" .= name
		, pack "data" .= ([1] :: [Int])
		, pack "subdata" .= ([] :: [Int])
		]
	toJSON (ActiveBarracks (Just x) Nothing) = object
		[ pack "name" .= name
		, pack "data" .= ([1] :: [Int])
		, pack "subdata" .= [toJSON x]
		]
	toJSON (ActiveBarracks (Just x) (Just y)) = object
		[ pack "name" .= name
		, pack "data" .= ([1] :: [Int])
		, pack "subdata" .= [toJSON x, toJSON y]
		]
	toJSON _ = error "bad barracks"

bits :: Value -> Value
bits _ = object
	[ pack "name" .= name
	, pack "data" .= [ Null ]
	]

variants :: [(MapZone, ())]
variants =
	[ (FactoryMineTunnels, ())
	]

instance FromJSON Barracks where
	parseJSON (Object vs) = case
		( parse (.: pack "name") vs
		, parse (.: pack "data") vs :: Result [Int]
		, parse (.: pack "subdata") vs :: Result [Value]) of
			(Success pName, Success [0], Error _)
				| pName == name -> return EmptyBarracks
			(Success pName, Success [1], Success vs)
				| pName == name -> let
						getPart = parse $ \v -> (Slime <$> parseJSON v)
							<|> (Minotaur <$> parseJSON v)
					in case getPart `mapM` vs of
						Success (a:b:_) -> return $ ActiveBarracks (Just a) (Just b)
						Success (a:_) -> return $ ActiveBarracks (Just a) Nothing
						Success [] -> return $ ActiveBarracks Nothing Nothing
						_ -> mzero
			_ -> mzero
	parseJSON _ = mzero

data Index
	= Inside | Leave
	| ActivatePrompt | UnableToActivate | Activate
	deriving (Eq, Ord, Show, Read)

data Var = DummyVar
	deriving (Eq, Ord, Show, Read)

instance Scene Index Var Barracks where
	link f = case f of
		Inside -> inside
		Leave -> leave
		UnableToActivate -> \_ _ -> []
		ActivatePrompt -> activatePrompt
		Activate -> activate

	display = fmap toLower . show
	vardisplay = fmap toLower . show

scene :: Encounter Index Var Barracks
scene b m =
	[ Narration2 $
		c "There's a squat concrete building, built partially into the uneven floor of the tunnel. It's got some black-paned windows and a dingy light, flickering above a nondescript metal door."
	, Choices
		[ ("try the door", Inside)
		, ("keep going", Leave)
		]
	]

inside :: Encounter Index Var Barracks
inside b EmptyBarracks =
	[ Narration2 $
		"It's slightly more homely inside. The lights don't flicker, and there are enough of them to light the place sufficiently. It's one big room, with a few alcoves to break it up slightly. The floor is poured concrete slabs. There are stretches of bunkbeds across half the room, and opposite that a room sectioned off with a U-shaped metal countertop; something like a kitchenette."
		/ Paragraph / "It seems like nobody's here right now."
	, Branch
		[ ( {-C (Global VoidWormQuest) (Gt 99) `VAnd` -} C (Global BarracksActive) (Neq 1), ActivatePrompt)
		]
		UnableToActivate
	]
inside b (ActiveBarracks Nothing Nothing) =
	[ Narration2 $
		"It's slightly more homely inside. The lights don't flicker, and there are enough of them to light the place sufficiently. It's one big room, with a few alcoves to break it up slightly. The floor is poured concrete slabs. There are stretches of bunkbeds across half the room, and opposite that a room sectioned off with a U-shaped metal countertop; something like a kitchenette."
		/ Paragraph / "It seems like nobody's here right now."
	]
inside b _ = error "no inside written yet"

activatePrompt :: Encounter Index Var Barracks
activatePrompt _ EmptyBarracks =
	[ Narration2 $ c "It might come in handy to remember this place for later."
	, Choices
		[ ("keep it in mind", Activate)
		, ("nevermind", Leave)
		]
	]
activatePrompt _ _ = error "bad activatePrompt"

activate :: Encounter Index Var Barracks
activate _ EmptyBarracks =
	[ Narration2 $ c "Remembered."
	, SetFlag (Global BarracksActive) (Set 1)
	, SetEncounterData $ ActiveBarracks Nothing Nothing
	]
activate _ _ = error "bad activate"

leave :: Encounter Index Var Barracks
leave _ _ =
	[ Narration2 $ c "You leave." ]

{-
minotaur miner getting gangbanged by slimes??? like in the miner barracks or w/e

like maybe give minotaurs & slimes 'top' and 'bottom' generation flags. & their specific scenes in the gangbang depend on those flags. (i think for the purposes here, 'vers' counts as 'whatever doesn't match the other person')
	x minotaur bottom / slime top: minotaur fucked by slime (pc can fuck afterwards)
	x minotaur top / slime bottom: minotaur fucks slime (pc can spitroast)
	x minotaur bottom / slime bottom: slime fisting minotaur dick & then shot back out (pc can help)
	x minotaur top / slime top: they jerk each other off (and then u get fucked)
	... except ofc:

	* minotaur top / minotaur bottom: minotaurs fuck / pc sandwiched b/t them
	* minotaur top / minotaur top: u get fucked
	* minotaur bottom / minotaur bottom: they fist each other

	* slime top / slime top: they fuse together and fuck you as a big archslime tentacle monster
	* slime top / slime bottom: slimes fuck / pc sandwiched b/t them
	* slime bottom / slime bottom: slime cockvored into other slime and then shot out 

	... AND THEN

	* dragonoid bottom / dragonoid bottom: mutual knotting w/ makeouts
	* dragonoid top / dragonoid bottom: one dragonoid fucks the other
	* dragonoid top / dragonoid top: u get four dicks shoved into yr ass

	* dragonoid bottom / minotaur top: dragonoid sprawls back while minotaur feeds the whole length of his dick up into his slit
	* dragonoid bottom / minotaur bottom: mutual fisting, all groaning w/ their dicks mashed together across their chests
	* dragonoid top / minotaur top: u get fucked
	* dragonoid top / minotaur bottom: dragonoid knots minotaur & cumflates him

	* dragonoid bottom / slime top: slime fists dragonoid's slit fist-deep
	* dragonoid bottom / slime bottom: u fuck both of them while they make out??
	* dragonoid top / slime top: u get fucked
	* dragonoid top / slime bottom: dragonoid fucks slime

(miner barracks headed by a flame demon or son of hamar tabal or smth)
	like sure okay, a {barracks head} encounter that refers to another monster type, but then 1. the sex scenes would either be awkwardly reused or i'd have to write two separate new ones, and either way it would give this specific encounter a lot of like, faux-uniqueness.

	...like even w/ the busted map generation system i have now, i can still place specific encounters in specific rooms. at doors even! kind of.
		like honestly that would be a v. v. basic setup: have some fixed "barracks room" event that sticks around, and then be able to 'send' encounters there, so it's like, talking to the big minotaur who's all "yeah im going to the barracks to get gangbanged" and then you can talk to slimes like "hey maybe check out the barracks", and then the scene you get is based on that counter of slimes sent there (& then afterwards all the events reactivate)

-}


{-
* minotaur bottom / slime top: minotaur fucked by slime (pc can fuck afterwards)

you arrive when they've already gotten to business. {the minotaur} is sprawled back across the low counter, with {the slime} pouring all down his front, goopy layers of slime soaking his ruff and smearing across his skin. {the slime} pulls back from the kiss they were in, leaving glimmering threads of spit stretching between them until the minotaur reaches up and catches them, smears them across the furled mess of his face with a hiss and a plume of steam. the slime has been grinding his dick against the minotaur's ass, hooves up in the air -- he pushes the minotaur's feet down, practically folding him double, putting the huge muscled globes of his ass cheeks on display as he reaches into his ass, slime fingers meeting no resistance as he easily shoves two, three fingers inside, no problem. the minotaur's dick pulses, unsheathed and stiffening all up his chest, nearly level with his upturned face. the underside ripples, heavy molten-glass pre bulging his flesh and then erupting from the tip, spilling in heavy cords across his chest and head as the slime fingerfucks him, an eager low bursting from his mouth.

the slime pulls back, not entirely to give you a good look of the minotaur's spread cheeks, his asshole a dark knot of flesh between his cheeks, pouted and spread so much you can see the ruddy glow of his inner flesh, red-hot. the slime slides his cock between the minotaur's cheeks, glazing him with sloppy ooze, and then shoves inside, all the way, hips meeting the minotaur's with a wet _crack_, little droplets of slime bursting from the slime's body from the impact, showering forward across the minotaur's body, mostly going up in plumes of steam. the minotaur outright _moos_, like a bull, and his cock sprays ruddily-glowing glass all across his face, cooling across his shoulders but bubbling into a froth where it's splattered across his churning molten-lava-gyre face.

the minotaur raises one hand, bracing it palm-spread across the wall behind him as the slime starts to thrust, the sharply-defined muscles of his chest and side on display, his black hide wet with sweat and slime. his body rocks with each thrust, skidding slightly, and the slime wraps both hands around his waist, gripping his tight as he starts hammering away, thrusts a mess of squelching crackles. the minotaur's cock lurches, painting his front in streamers of pre, pouring down his chest and over his sides in thick, heavy streams, spilling onto the counter and lighting it on fire where the molten glass pools.

the slime wraps a hand around the minotaur's cock, bending down to stroke its full length, hips still smacking his ass as he thrusts hard and fast, and it only takes a few strokes before the minotaur bellows and erupts, spurts of molten lava spraying from his cock in heavy droplets, splattering across the wall behind and piling up, forming sluggish runnels. the minotaur works his hips, fucking himself on the slime's cock just as much as the slime is fucking him, snorting and bellowing with each cord of lava spraying from his cock, hosing himself down in burning lines until he's drenched, skin and hide and craggy rock alike all glazed in white-hot lava, slowly leeching color and heat as it hardens.

the slime groans, coming inside, panting sharply before he makes a few sharp thrusts and then hilts inside with a groan, only rutting shallowly as he spills his load inside. the slime smirks at you as you {step} in, pulling out with a final sloppy squirt, leaving the minotaur's hole gaping, the sludgy mess of come already starting to boil into a froth as it spills from his splayed hole.

> you want a turn?

he says, and smacks the minotaur's ass cheek -- the minotaur lets out a sharp bleat -- for emphasis, leaving a slimy handprint behind

{then the pc can fuck him, again condensed}


~> CENTAUR

your cock has fully unsheathed as you've watched, hard and drooling a messy puddle on the floor. your hooves clop loudly on the floor as you walk across the room, each step slapping your cock against the stiff fur of your underbelly and sending a splatter of pre gushing in all directions.

the slime steps aside as you reach the minotaur, hand idly tugging on his half-hard cock as he watches. you rear up, forelegs clattering across the counter before they find purchase, and your cock slaps across his body with a meaty _crack_. the minotaur groans, huffing, and slumps his shoulders down, his used ass rising up to meet your cock. your broad, flat cockhead nestles between his cheeks, shoved against the sloppy, gaping pucker of his asshole, sluggishly drooling the frothy mess of the slime's load. you brace your forelegs on the counter, arms pressed against the wall, and look down at the minotaur -- his head shoved up against the wall in anticipation, bleating. you plunge forward, cock ramming into his ass, displacing a swampy mess of slime as you sink all the way to your medial ring in a single thrust.

the minotaur lets out a sharp, bleating moo, slowly transforming into a rolling groan as his ass clenches around your shaft, pulsing as his muscles spasm. you jerk forward, the ridge of your medial ring slapping against his hole, pulling back fractionally webbed with the goopy runnels of the slime's load. ooze smears down your shaft, slurping into your sheath, and you jerk forward again, the rest of your cock plunging in, balls smacking against the minotaur's thighs, sheath bunched up between his muscled cheeks.

the minotaur moos again, and you fuck him hard and fast -- his body convulses, loud _cracks_ as he sprays his load across the counter, cock pinned against his chest. his hole is slick and open, drooling the mess of the slime's load as well as your own pre. between that and the show they put on, it doesn't take much for you to come. you groan, hips bucking, plunging the final few inches of your cock back and forth through the minotaur's slopping, gaping ring, and then you blow, cock trembling and shooting off, flooding his ass with a slimy, churning mass of come. the minotaur just groans, panting, his ass squelching, sharp squirts of mingled come gushing out around your pumping shaft as you unload in him, pulse after pulse flooding him and slowly working its way out, streaming in thick cords down your shaft until it bursts around his spasming asshole.

you pull out with a sloppy squelch, cock flopping to one side, spraying the final few gushes all across his thighs and hooves, and his ass is a gaping, broken-wide mess, his guts all glowing with a ruddy heat and now completely slathered in churned-up layers of come. the slime steps closer -- hand on his cock -- and sprays a second load all across the minotaur's ass, adding a fresh layer of stringy slime to the mess slowly oozing from his wrecked hole, and then runs his hands over the minotaur's calves, catching up the mess of your combined loads and feeding it back into his broken hole.
-}




-- * minotaur top / slime bottom: minotaur fucks slime (pc can spitroast)

{-
you arrive when they've already gotten to business. {the slime} is pinned against the wall, the minotaur leaning against him, squat fingers shoving into his asshole, cock unsheathed and spanning the space between them, drooling molten pre that hisses and spits every time it drools down across the slime's back. the minotaur takes a step back, hooves clopping loud on the floor, and aims his cock straight at the slime's asshole, then steps forward, shoving half of it straight into him. the slime groans, loud and long, not stopping until the minotaur takes the other step, sheathing his entire cock in a single brutal thrust. you can see it, through the {orange/purple} of the slime's body: cock a solid, unbending line all the way up, from the slime's ass to where the tip rests inside his rib cage. it spurts, a mess of molten pre bursting out, churning the slime around it into a foaming, buzzing mess, and the slime raises a hand, pressing it against his chest, panting.

the minotaur digs his fingers into the slime's hip, strong enough that his flesh starts to warp and twist like gelatin, and starts thrusting, jerking back and slamming forward, carving a broad line of opaque frothy mess through the center of the slime's body. loose slime pours down from his broken hole, spurting out with each withdraw; the minotaur's cock emerges shining and dripping, absolutely slathered in shed slime.

the slime groans, his own cock a heavy arch, drooling a thick cord of slime from the tip, swinging with each thrust. the slime's gut churns and bulges, the mess of molten pre slowly sluicing down the minotaur's thrusting cock, slowly forming a heavy swell across his stomach, more and more building as the minotaur hilts his cock again and again into the slime's yielding body.

the minotaur looks over at you, and smacks the slime's ass, sending a shower of slime droplets splattering in all directions.

> You want to join in? He's still got a hole free.

The slime groans in agreement, hands both braced against the wall.

-> fuck his mouth
-> just watch

---

-> just watch

The minotaur shrugs.

> Have it your way.

But he twists a little, tugging the slime a half-step over, changing the angle so you have full view of each thrust. He hilts again with a smack, and now you can see it in profile: the arch of the minotaur's cock, still spurting out pre in thick, heavy globs, and spilling down his shaft in a frothing mess to pile up in the slime's increasingly distended gut. He steps forward, bracing a heavy hoof a half-step up the sloping wall, using that as leverage to drag the slime back, hips pinned against his. His cock twitches, dragging through the viscous, churned-up slime on the interior of the slime's body, cockhead smacking against the slime's spine, erupting in a sharp spume of molten pre that nearly reaches the slime's neck.

The slime sags down, one arm braced against the wall, the other catching his sagging gut. The minotaur snorts and starts thrusting again, smacking his hips against the slime's ass, each thrust spraying pre and slime all across his front, spilling from the slime's gaping hole in frothing globs, dripping down the low, swaying span of the minotaur's balls.

The minotaur bellows when he comes, his cock thudding hard enough it smacks against the slime's bones, and through the {murkily-/translucent} flesh of the slime's body you can watch the minotaur's cock shudder and twist, cockhead and shaft spiralling as his grisled flesh clenches and pulls. His load is even hotter and heavier, thick spurts of molten-lava come splattering across the slime's bones, drooling down to add to the huge, pregnant-looking weight churning inside, stretching the slime's flesh like elastic. The minotaur comes and comes, bleating and mooing, hands twisted so tightly around the slime's hips they look submerged in his flesh, cock still shallowly thrusting. Shot after shot of come floods him in a boiling, burning wash, the incandescent glow of it shining through the slime's body as it piles up in his bloated gut.

The minotaur slides back, cock emerging with a wet slurp, boiling tar spilling down his shaft, steaming from the heat. The slime all but collapses, the mass of molten come in his gut nearly looking like it's going to bud off of him, a messy churning sphere settling above his hips. The minotaur paints him in his load, hand squelching as he strokes himself off, milking out spurt after spurt of his load, splattering all up and down the slime's back, bubbling and steaming where it hits his flesh and flash-cools, crusting over with a wrinkled black coating.

-}




-- * minotaur top / slime top: they jerk each other off (and then u get fucked??)

{-
You arrive when they've already gotten down to business. They're pressed against each other, making out -- well, the slime's face deformed by a giant tongue-tentacle, smearing a mass of boiling froth up and down across the minotaur's furled-lava face -- with their cocks sandwiched between them, rutting up and down through the messy channel between them, slathered in frothy foaming bubbles of churned-up ooze. They're jerking each other off, hands wrapped halfway around their cocks, humping against each other in jerky motions.

The slime's hand pumps up and down the minotaur's giant cock, smearing it thickly with ooze, making it glisten, every weird flange and ridge along its gristled length gleaming wetly. The thick tube of his urethra is bloated and visibly pulsing, burbling thick glassy pre from the tip, smearing across the slime's jaw. The slime's cock is jammed up along the minotaur's lopsided cock, with the minotaur's knobbled fingers catching at the tip, shoving it tighter against his shaft. The slime's rutting back and forth as he pants, gushing splatters of acid pre that he just keeps smearing along the minotaur's shaft, up and down, in a frothing, squelching mess. Their pre mingles together into a thick creamy froth, opaque dollops encrusted with glassy bubbles splattering all over the minotaur's chest and belly, winding in cords between they spiky tufts of his haunches.

The slime leans in, panting sharply, moaning as his tongue wraps around and around the minotaur's head until it's nearly cocooned in a rippling sea of slime. His cock jumps, fucking the grisly line of the minotaur's cock, and he groans deep, cock erupting -- brilliant acid-yellow come gushing out of him and just getting lost in the stringy mess corded between them, pulse after pulse frothing up as it splatters all up across the minotaur's chest. But his hands clench harder where they're wrapped around the minotaur's cock, digging in, and the minotaur bleats and bellows, hands wrapping around the slime's hips, grinding his cock up and down the slime's chest. His cock shudders, lurching back and forth trapped between their bodies, burbling pre at weirder and weirder angles as his cockhead shudders and twists, spiraling around in a twisting hoop. A shot of molten-lava come sprays from his cock, fountaining up into the air and splattering all across the both of them in glowing lines, hissing and sending up plumes of acrid smoke where it hits the slime, the red heat being quenched by the slime's body. But the minotaur keeps coming, huffing and groaning, gushing come all down the slime's front, shrouding the both of them in a mess of black, acrid smoke.

They peel apart eventually, the both of them panting, come and froth stretched out between them in enormous, heavy cords, smearing down their skin.
-}


-- * minotaur bottom / slime bottom: slime fisting minotaur dick & then shot back out (pc can help)
{-
You arrive when they've already gotten down to business. The minotaur is pressed against the wall, breathing hard, with the slime kneeling in front of him, the minotaur's cock fully, hugely erect and pointed out nearly horizontal, drenched in thick slime. The slime's stroking him off, one hand squeezing the tip, the other working the shaft. The minotaur is bleating unevenly, huffs and grunts, skin sheened with sweat, his lava glow fluctuating rapidly.

And... the reason for that is because the slime has shoved a finger or two down into the minotaur's cock. The slime's hand isn't just wrapped around the minotaur's cockhead; he's got his thumb notched against the dimple of his urethra and a finger shoved straight down inside, turning the little divot into a broad circle -- visible through the slime's murky flesh -- clenching and spasming as the slime works his finger around. You can see the long fat line of the minotaur's urethra bulge, flooded with slime or backed up with his own heavy pre or both, the tube warped two, three fingers thick, nearly snakelike as it squirms.

Every little motion of the slime's hand makes the minotaur's entire body shudder, hips jerking forward or back, neither seeming like conscious action so much as some reflexive spasm from overstimulated flesh. And still the slime is trying to stuff more inside: you can see his fingers creep over the gnarl of the minotaur's cockhead, working back and forth over the splayed opening of his urethra. The minotaur wails as the slime shoves in a second finger, and his hands splayed across the wall twitch so hard he ends up with two fistfuls of crumpled metal sheeting.

The slime eagerly feeds his second finger into the minotaur's cock, his urethra a lopsided oval, fingers plunging back and forth, finger-fucking the minotaur's cockhead in a series of gummy squelches. The minotaur whines, shuddering, and his cock lurches again -- and the slime just takes that as an opportunity to shove another finger into his spasming urethra, his churning fingers making sloppy squelches as he shoves sloppy slime deeper down into the minotaur's bloated dick. You can see the slime foam up inside: his urethra bulges further, the tube visibly bloating in uneven blobs, flesh going from spongy and squishy to being drawn out into a thick, taut half-circle, practically hanging off the underside of the minotaur's cock.

Three fingers clearly isn't enough for the slime, though. He finger-fucks the minotaur's cock more and more roughly, jerking out to let some of his foaming slime spurt out with the force of an orgasm before shoving his fingers back inside. The minotaur's cockhead warps and bloats: partly from the bulk of half the slime's hand shoved inside, but partly simply from the abuse, flesh bruised and swollen, bulging out into lumpy swells as the slime shoves harder and harder.

They both groan when the slime fits his entire fist him: the slime an eager, orgasmic whine; the minotaur an exhausted bellow. Even that's not enough: the slime punches down, shoving most of his forearm into the minotaur's cock, looking now almost like a straw: the thick walls of his cock thinner than the slime stretching it out, the massive gaping hole of his urethra plainly visible through the flesh of the slime's elbow.

It's clear the minotaur can't take much more of it, though: he tips his head back -- horns gouging tracks in the wall behind him -- and bellows. His cock ripples, balls jerking hard, and his abs contract into huge blocks. His cock lurches, jerking against the slime's arm, and a lopsided bulge bloats out along the underside of his cock, urethra swollen up grotesquely. His cock surges again, the bloat digging further up his shaft, becoming taut. The slime pushes back -- arm visibly straining against the pressurized force, sinking bicep deep -- and between the two forces the minotaur's cock warps further, urethra swelling fatter and fatter until finally it gapes wide enough for a spume of frothing ooze to erupt out around the slime's arm, plastering him in a layer of half-molten lava, fracturing into glassy shards as it cools instantly.

The slime uses his other hand to squeeze the minotaur's cock tight around his bicep, and he gropes around inside the minotaur's dick, probing fingers dimpling his flesh from the inside. The minotaur bellows, thrashing back and forth, and his cock remains fixed in place, anchored by the bulk of the slime shoved inside him.

The slime rides out each shot of the minotaur's climax, rolling into each thrust, arm shoved nearly shoulder deep before his questing fingers find what he's looking for: he grabs hold and yanks. His arm spills from the minotaur's wrecked dick with an absolute waterfall of slime, and his fist is wrapped around the tip of what looks like the most massive sound imaginable: a giant pillar of lava-come, half-molten and still flexible as the slime yanks more and more from the minotaur's gaping urethra, cooling in the open air to a shining purple-black. The minotaur howls brokenly, hips jerking -- fucking his cock on the massive sound of his own cooling come -- as he spews more and more come into his blocked-up pipes, the root of his cock visibly bulging all the way back to his taint as his internal passages bloat. He sags, hooves sliding across the floor as he falls to his knees, stomach hollowing, sweat pouring down his chest.

The slime keeps the sound in until the minotaur's done, and only slowly peels it out, letting the half-molten rock twist and bend around inside before he pulls it out as a smooth, unbent pillar; by the time the lumpy tip bursts past the minotaur's abused cock lips the thing is nearly twice the length of his already absurdly-long cock. The minotaur just sags to the side, entire body trembling from exertion, the slime squashing slightly to hold up his weight.
-}
