{-# LANGUAGE MultiParamTypeClasses #-}

module Sites.Hell.Events.Hubur.Cave
	( Cave
	, mineralDeck
	, asakkuDeck
	, rockDemonDeck
	, scene
	, name
	, bits
	, variants
	) where

import Prelude hiding ((/))
import Server.JSON
import Sites.Hell.Events.Construction ((/))

import Sites.Hell.Events.Hubur.Shadow (Shadow(..))
import qualified Sites.Hell.Events.Hubur.Shadow as Shadow
import Sites.Hell.Events.Hubur.Asakku (Asakku(..))
import qualified Sites.Hell.Events.Hubur.Asakku as Asakku
import Sites.Hell.Events.Hubur.RockDemon (RockDemon(..))
import qualified Sites.Hell.Events.Hubur.RockDemon as RockDemon

import Sites.Hell.Events.SceneUtils
import Sites.Hell.Events.Hubur

name :: String
name = "cave"

data Cave
	= CaveProgress
		{ light :: Light
		, exterior :: CaveExterior
		, opts :: [CaveOption]
		, term :: CaveTerminal
		}
	| CaveSolved
		{ light :: Light
		, exterior :: CaveExterior
		, term :: CaveTerminal
		}
		deriving (Show)

data Light = Candles | Lanterns | Dark
	deriving (Eq, Ord, Enum, Bounded, Show, Read)

data CaveSplash = Tunnel | Gallery | Stream | Narrow
	deriving (Eq, Ord, Enum, Bounded, Show, Read)

data CaveAction = GutteringLight | FoundCache IIndex | WildMonster Opponent | NoAction
	deriving (Eq, Ord, Show, Read)

data CaveOption = CO CaveSplash CaveAction
	deriving (Eq, Ord, Show, Read)

data CaveTerminal = TAsakku Asakku | TRock RockDemon | TShadow Shadow
	| TMineralSpawn IIndex
	deriving (Show)

data CaveExterior = Blank | PlantSpawn IIndex
	deriving (Eq, Ord, Show, Read)

{-
instance Random Cave where
	randomR _ = random
	random = getRand $ Cave
		<$> genRandom

-}

caveActions :: [CaveAction]
caveActions =
	[ WildMonster skull
	, WildMonster beast
	, WildMonster beast
	, FoundCache ORRERY_MECHANISM
	, FoundCache SUN_STONE
	, FoundCache MOON_STONE
	, FoundCache GLOW_CRYSTAL
	, GutteringLight
	, NoAction
	]

instance ToJSON CaveOption where
	toJSON (CO splash act) = toJSON $ (s * 10 + a :: Int)
		where
			s = case splash of
				Tunnel -> 1
				Gallery -> 2
				Stream -> 3
				Narrow -> 4
			a = case act of
				NoAction -> 0
				GutteringLight -> 1
				FoundCache GLOW_CRYSTAL -> 2
				FoundCache MOON_STONE -> 3
				FoundCache SUN_STONE -> 4
				FoundCache ORRERY_MECHANISM -> 5
				WildMonster enc
					| enc == skull -> 6
					| enc == beast -> 7
				_ -> error $ "bad action (" ++ show act ++ ")"

instance FromJSON CaveOption where
	parseJSON n'@(Number _) = parser <$> parseJSON n'
		where
			parser :: Int -> CaveOption
			parser n = CO s a
				where
					si = n `div` 10
					s = case si of
						1 -> Tunnel
						2 -> Gallery
						3 -> Stream
						4 -> Narrow
						_ -> error $ "bad opt (" ++ show n ++ ")"
					ai = n `mod` 10
					a = case ai of
						0 -> NoAction
						1 -> GutteringLight
						2 -> FoundCache GLOW_CRYSTAL
						3 -> FoundCache MOON_STONE
						4 -> FoundCache SUN_STONE
						5 -> FoundCache ORRERY_MECHANISM
						6 -> WildMonster skull
						7 -> WildMonster beast
						_ -> error $ "bad opt (" ++ show n ++ ")"
	parseJSON _ = mzero

skull :: Opponent
skull = Opponent "phantasmal flaming skull" "combat"
	(Stats 1 1 1 5 4 1 1 3)
	(RandomAI
		[ [Wait, Block, Wait, Wait, Wait, Hit]
		, [Block, Wait, Block, Wait, Hit, Wait]
		, [Block, Block, Wait, Charge, Cast, Wait]
		])

beast :: Opponent
beast = Opponent "feral cave-beast demon" "combat"
	(Stats 4 2 1 6 2 1 1 3)
	(RandomAI
		[ [Wait, Block, Hit, Wait]
		, [Wait, Block, Hit, Hit]
		, [Wait, Block, Hit, Wait, Wait]
		, [Block, Hit]
		, [Wait]
		])

instance ToJSON Cave where
	toJSON (CaveProgress l ext opts t) = object
		[ pack "name" .= name
		, pack "data" .=
			[toJSON $ (0 :: Int), toJSON $ fromEnumExt ext, toJSON $ fromEnumTerminal t, toJSON $ fromEnum l, toJSON $ toJSON <$> opts]
		, pack "subdata" .= case t of
			TShadow s -> [toJSON s]
			TAsakku a -> [toJSON a]
			TRock r -> [toJSON r]
			TMineralSpawn i -> [toJSON i]
		]
	toJSON (CaveSolved l ext t) = object
		[ pack "name" .= name
		, pack "data" .= [1, fromEnumExt ext, fromEnumTerminal t, fromEnum l]
		, pack "subdata" .= case t of
			TShadow s -> [toJSON s]
			TAsakku a -> [toJSON a]
			TRock r -> [toJSON r]
			TMineralSpawn i -> [toJSON i]
		]

-- wow this is a mess. this is i think the first time i'm explicitly storing some non-int values in the data value, and so i don't really have the framework to do that elegantly yet.
instance FromJSON Cave where
	parseJSON (Object vs) = case
		( parse (.: pack "name") vs
		, parse (.: pack "data") vs
		, parse (.: pack "subdata") vs :: Result [Value]) of
			(Success pName, Success [z, ext', ti', l', opts], Success [tv])
				| pName == name && fromJSON z == Success (0 :: Int)  -> let
						d :: Value -> Int
						d v = case fromJSON v of
							Success v' -> v'
							_ -> error "bad decode"
						ext = d ext'
						ti = d ti'
						l = d l'
					in case CaveProgress
						<$> safeToEnum l
						<*> toEnumExt ext
						<*> mparse (parseJSON) opts
						<*> mparse (toEnumTerminal ti) tv of
							Nothing -> error $ unwords
								[ show (safeToEnum l :: Maybe Light)
								, show l
								, show (mparse (toEnumTerminal ti) tv :: Maybe CaveTerminal)
								, show ti
								, show tv
								]
							Just h -> return h
			(Success pName, Success [z, ext', ti', l'], Success [tv])
				| pName == name && fromJSON z == Success (1 :: Int) -> let
						d :: Value -> Int
						d v = case fromJSON v of
							Success v' -> v'
							_ -> error "bad decode"
						ext = d ext'
						ti = d ti'
						l = d l'
					in case CaveSolved
						<$> safeToEnum l
						<*> toEnumExt ext
						<*> mparse (toEnumTerminal ti) tv of
							Nothing -> mzero
							Just h -> return h
			_ -> mzero
	parseJSON _ = mzero

bits :: Value -> Value
bits _ = object
	[ pack "name" .= name
	, pack "data" .= ([] :: [Int]) -- FIXME: probably not right
	]

variants :: [(MapZone, ())]
variants =
	[ (Badland, ())
	]

data Index = LeaveEarly | Leave
	| FirstIn
	| SecondIn
	| ThirdIn
	| FourthIn
	| End
	| AIndex Asakku.Index
	| SIndex Shadow.Index
	| RIndex RockDemon.Index
	deriving (Eq, Ord, Show, Read)

data Var = ExteriorLastHarvested | InteriorLastHarvested
	| AsAsakku () | AsRock () | AsShadow ()
	deriving (Show, Read)

instance Scene Index Var Cave where
	link x = case x of
		LeaveEarly -> leaveEarly
		Leave -> leave
		FirstIn -> (\_ _ -> [Narration2 $ c "You enter the cave."]) <> depth 1
		SecondIn -> (\_ _ -> [Narration2 $ c "You head deeper into the cave."]) <> depth 2
		ThirdIn -> (\_ _ -> [Narration2 $ c "You head deeper into the cave."]) <> depth 3
		FourthIn -> (\_ _ -> [Narration2 $ c "You head deeper into the cave."]) <> depth 4
		End -> end
		AIndex ai -> \b c -> case term c of
			TAsakku a -> asAsakku c <$> link ai b a
			_ -> error "bad cave scene"
		SIndex si -> \b c -> case term c of
			TShadow s -> asShadow c <$> link si b s
			_ -> error "bad cave scene"
		RIndex ri -> \b c -> case term c of
			TRock r -> asRock c <$> link ri b r
			_ -> error "bad cave scene"
	display = fmap toLower . show
	vardisplay = fmap toLower . show

asAsakku :: Cave -> Event Asakku Asakku.Index () -> Event Cave Index Var
asAsakku c = triMap (\a -> c { term = TAsakku a }) AIndex AsAsakku

asShadow :: Cave -> Event Shadow Shadow.Index () -> Event Cave Index Var
asShadow c = triMap (\a -> c { term = TShadow a }) SIndex AsShadow

asRock :: Cave -> Event RockDemon RockDemon.Index () -> Event Cave Index Var
asRock c = triMap (\a -> c { term = TRock a }) RIndex AsRock

fromEnumTerminal :: CaveTerminal -> Int
fromEnumTerminal t = case t of
	TShadow _ -> 0
	TAsakku _ -> 1
	TRock _ -> 2
	TMineralSpawn _ -> 3

toEnumTerminal :: Int -> Value -> Parser CaveTerminal
toEnumTerminal 0 v = TShadow <$> parseJSON v
toEnumTerminal 1 v = TAsakku <$> parseJSON v
toEnumTerminal 2 v = TRock <$> parseJSON v
toEnumTerminal 3 v = TMineralSpawn <$> parseJSON v
toEnumTerminal _ _ = mzero

fromEnumExt :: CaveExterior -> Int
fromEnumExt Blank = 0
fromEnumExt (PlantSpawn i) = fromEnum i

toEnumExt :: Int -> Maybe CaveExterior
toEnumExt 0 = Just Blank
toEnumExt n = PlantSpawn <$> safeToEnum n

deck :: RandomGen g => Rand g (Enumerable Cave)
deck = mineralDeck

mineralDeck :: RandomGen g => Rand g (Enumerable Cave)
mineralDeck = do
	-- light <- liftRand $ shuffle [Candles, Lanterns, Dark]
	exterior <- liftRand $ shuffle $ plants <> (const Blank <$> plants) -- 50% chance of having an exterior node
	options <- do
		splash <- liftRand $ shuffle [Tunnel, Gallery, Stream, Narrow]
		action <- liftRand $ shuffle caveActions
		return $ CO <$> from splash <*> from action
	mineral <- liftRand $ shuffle minerals
	return $ CaveProgress
		<$> pure Dark -- mine-type caves are always dark; inhabited ones are always lit up
		<*> from exterior
		<*> listRangeOf (2, 4) options
		<*> from mineral

asakkuDeck :: RandomGen g => Asakku -> Rand g (Enumerable Cave)
asakkuDeck asakku = do
	light <- liftRand $ shuffle [Candles, Lanterns]
	exterior <- liftRand $ shuffle $ plants <> (const Blank <$> plants) -- 50% chance of having an exterior node
	options <- do
		splash <- liftRand $ shuffle [Tunnel, Gallery, Stream, Narrow]
		action <- liftRand $ shuffle caveActions
		return $ CO <$> from splash <*> from action
	return $ CaveProgress
		<$> from light
		<*> from exterior
		<*> listRangeOf (2, 4) options
		<*> pure (TAsakku asakku)

rockDemonDeck :: RandomGen g => RockDemon -> Rand g (Enumerable Cave)
rockDemonDeck rock = do
	light <- liftRand $ shuffle [Candles, Lanterns]
	exterior <- liftRand $ shuffle $ plants <> (const Blank <$> plants) -- 50% chance of having an exterior node
	options <- do
		splash <- liftRand $ shuffle [Tunnel, Gallery, Stream, Narrow]
		action <- liftRand $ shuffle caveActions
		return $ CO <$> from splash <*> from action
	return $ CaveProgress
		<$> from light
		<*> from exterior
		<*> listRangeOf (2, 4) options
		<*> pure (TRock rock)

randomItem :: IIndex -> [[Item]]
randomItem i = (\x -> [I i x]) <$> [1,2,2,3]

plantResources :: [IIndex]
plantResources =
	[ LIONS_MANE -- lion
	, MANDRAKE -- human
	, GOATSBEARD -- goat
	, ADDERS_FORK -- snake
	, OX_EYE -- bull
	, CATTAIL -- tiger
	, HARTS_TONGUE -- deer
	, TONGUE_OF_DOG -- wolf
	, HORSE_HOOF -- unicorn
	, PIGS_TAIL -- boar
	]

mineralResources :: [IIndex]
mineralResources =
	[ SOULSTONE_ORE -- ???
	, LODESTONE -- metal
	, RAW_FLAMECRYSTAL -- fire
	]

minerals :: [CaveTerminal]
minerals = TMineralSpawn <$> mineralResources

plants :: [CaveExterior]
plants = PlantSpawn <$> plantResources



scene :: Encounter Index Var Cave
scene = approach

approach :: Encounter Index Var Cave
approach _ cave =
	[ Narration2 $
		"In the craggy wasteland, you come across a cave entrace."
		/ case light cave of
			Dark -> c "The interior is pitch black, yawning open to an unlit abyss."
			x -> case x of
					Lanterns -> "There are lanterns just within the arch, bathing the scene in eerie purple light."
					Candles -> "The walls of the cave are covered in candles, melted together into mounds. They burn with an eerie purple light."
					Dark -> error "unreachable"
				/ "The candle-flames waver in the wind, sending shadows rippling across the rock."
	] <>
	case exterior cave of
			Blank -> []
			PlantSpawn i ->
				let message = case i of
					LIONS_MANE -> "A host of thick, golden grasses grow outside the cave."
					MANDRAKE -> "Before the cave, the bulbs of some lumpy root show above the soil."
					GOATSBEARD -> "A long, tufted plant grows in the cracks of the cave opening."
					ADDERS_FORK -> "A low-lying plant spreads hangs from the rocks."
					OX_EYE -> "A cluster of wildflowers bloom before the cave opening."
					CATTAIL -> "A bunch of tufted cattails grows in a shallow puddle in the lee of the cave opening."
					HARTS_TONGUE -> "A bush with clusters of sodden, limp leaves grows beside the rocks."
					TONGUE_OF_DOG -> "Besides the cave entrance, red flowers grow in clusters."
					HORSE_HOOF -> "A dense, waxy growth sprouts from the soil."
					PIGS_TAIL -> "Curly wild grasses sprout beside the cave opening."
					x -> error $ "unhandled resource node (" ++ show x ++ ")"
				in pure $ HarvestNode (Local ExteriorLastHarvested) message $ randomItem i
	<>
	[ Choices
		[ ("enter", FirstIn)
		, ("leave", LeaveEarly)
		]
	]

depth :: Int -> Encounter Index Var Cave
depth i b cave = case drop (i-1) $ opts cave of
	(CO splash act):rs ->
		(Narration2 $ c $ caveSplash splash)
		: caveAction act
		<> [ SetEncounterData $ cave { opts = drop i $ opts cave }
			, Choices
				[ ("head deeper", if i == length (opts cave) 
					then End
					else case i of
						1 -> SecondIn
						2 -> ThirdIn
						3 -> FourthIn
						x -> error $ "bad depth in (" ++ show x ++ ")")
				, ("turn back", Leave)
				]
			]
	[] -> [ Jump End ] -- error $ "bad cave depth (" ++ show i ++ ")"

end :: Encounter Index Var Cave
end b cave = case term cave of
	TAsakku a ->
		[ Narration2 $ c "You reach the bottom of the cave."
		] <> (asAsakku cave <$> Asakku.scene b a)
	TRock r ->
		[ Narration2 $ c "You reach the bottom of the cave."
		] <> (asRock cave <$> RockDemon.scene b r)
	TShadow s ->
		[ Narration2 $ c "You reach the bottom of the cave."
		] <> (asShadow cave <$> Shadow.scene b s)
	TMineralSpawn i ->
		[ Narration2 $ "You reach the bottom of the cave."
			/ Paragraph / "The cave opens up into a vast chamber, walls glimmering with seams of rough ore."
		, let message = case i of
				SOULSTONE_ORE -> "The seam is of soulstone, bubbling up to the surface in bulbous chunks."
				LODESTONE -> "The seam is of raw lodestone, exterting a strange gravitational pull as you reach closer."
				RAW_FLAMECRYSTAL -> "The seam is of burning flamecrystal, sending wisps of flame up the walls."
				x -> error $ "unhandled resource node (" ++ show x ++ ")"
			in HarvestNode (Local InteriorLastHarvested) message $ randomItem i
		, FauxChoice "head back up"
		, Jump Leave
		]

caveSplash :: CaveSplash -> String
caveSplash s = case s of
	Tunnel -> "The cave winds down a narrow tunnel, and the temperature drops. The walls shimmer with moisture."
	Gallery -> "The cave rapidly opens up into a vast, echoing galley, with exits in all directions. There's a low noise, continually; some distant sound distorted into nothingness."
	Stream -> "The passage dips lower, and a trickle of water from the far wall becomes a stream, running still and silent along your path."
	Narrow -> "The passage thins, more and more, until it's oppressively claustrophobic. You barely manage to squeeze through a narrow gap."

caveAction :: CaveAction -> [Event Cave Index Var]
caveAction a = case a of
	WildMonster enc
		| enc == skull ->
			[ Narration2 $ c "Suddenly, a jabbering skull bursts out, trailing a plume of phantasmal light!"
			, Challenge enc []
			]
		| enc == beast ->
			[ Narration2 $ c "There's a rumble from above, and then a feral cave-beast demon leaps down at you, its malformed claws skittering on the rough ground!"
			, Challenge enc []
			]
	FoundCache i
		| i == ORRERY_MECHANISM ->
			[ Narration2 $ c "Strewn across the rough cave floor are some ancient gears, made of some metal that soaks up the light."
			, ItemModify [I i 2]
			]
		| i == SUN_STONE ->
			[ Narration2 $ c "The cave is flooded with light, from a glowing orb suspended in the middle of the air."
			, ItemModify [I i 1]
			]
		| i == MOON_STONE ->
			[ Narration2 $ c "There's a strange smooth stone lying on the cave floor, amidst rough gravel."
			, ItemModify [I i 1]
			]
		| i == GLOW_CRYSTAL ->
			[ Narration2 $ c "Small glowing crystals are embedded in the cave walls, making the passage glitter like there are faint stars illuminating you."
			, ItemModify [I i 2]
			]
	GutteringLight ->
		[ Narration2 $ c "Your light gutters and fades, leaving you in pitch darkness."
		]
	NoAction ->
		[ Narration2 $ c "The cave echoes all around you."
		]
	a -> error $ "bad action (" ++ show a ++ ")"

leaveEarly :: Encounter Index Var Cave
leaveEarly _ _ =
	[ Narration2 $ c "You turn away from the cave."
	]

leave :: Encounter Index Var Cave
leave _ _ =
	[ Narration2 $ c "You make your way back, til you reach the surface."
	]

{-

	* random item
	* random respawning resource (like cave plants / ore / crystals)
	* subroom containing one of the above but requiring a lore/search challenge to find
	* random combat challenge w/ some wild demon-monster
	* random movement challenge to navigate across pits or climb a wall or w/e
	* locked door needing a key or a lore/wits challenge to pick
	* various varieties of branching path (fork, pit, etc)


data CaveEncounter = CaveItem x | CaveFight x | CaveResourceSink x
data CaveTerminalEncounter = CaveResourceNode resource | CaveLockedDoor npc


Narration2 "In a pile on the cave floor, you find..."
GiveItem ...

Narration2 "A wild monster shrieks out of the darkness!"
Combat

	phantasmic shrieking skull

	feral cave-beast demon


-- or like, have a choice bar that explicitly states a requirement?
Narration2 "Your light dims and gutters..."
if (pc has items x)
	then
		[ Choices
			[ burn x item
			, turn back
			]
		]
	else
		[ Narration "(if you had x item you could spend it to proceed)"
		, Choices
			[ turn back
			]
		]

uhhh and then a local caveprogress/cavesolved variable that determines where you start at w/ the cave??
-}
