{-# LANGUAGE MultiParamTypeClasses #-}
module Sites.Hell.Events.Hubur.CedarTree
	( CedarTree(..)
	, dryadDeck
	, natureSpiritDeck
	, unicornDeck

	, scene

	, name
	, bits
	, variants
	) where

import Prelude hiding ((/), (^))

import Sites.Hell.Events.SceneUtils hiding ((/))
import Sites.Hell.Events.Construction ((/))
import Server.JSON

import Sites.Hell.Events.Hubur
import Sites.Hell.Events.Hubur.Dryad (Dryad)
import qualified Sites.Hell.Events.Hubur.Dryad as Dryad
import Sites.Hell.Events.Hubur.Unicorn (Unicorn)
import qualified Sites.Hell.Events.Hubur.Unicorn as Unicorn
import Sites.Hell.Events.Hubur.NatureSpirit (NatureSpirit)
import qualified Sites.Hell.Events.Hubur.NatureSpirit as Nature

name :: String
name = "cedartree"

data CedarTree = CDryad Dryad | CNature NatureSpirit | CUnicorn Unicorn
	deriving (Eq, Ord, Show, Read)

instance ToJSON CedarTree where
	toJSON (CDryad d) = object
		[ pack "name" .= name
		, pack "data" .= [0 :: Int]
		, pack "subdata" .= [toJSON d]
		]
	toJSON (CNature n) = object
		[ pack "name" .= name
		, pack "data" .= [1 :: Int]
		, pack "subdata" .= [toJSON n]
		]
	toJSON (CUnicorn u) = object
		[ pack "name" .= name
		, pack "data" .= [2 :: Int]
		, pack "subdata" .= [toJSON u]
		]

bits :: Value -> Value
bits _ = object
	[ pack "name" .= name
	, pack "data" .= [Null]
	]

variants :: [(MapZone, ())]
variants =
	[ (Glade, ())
	]

instance FromJSON CedarTree where
	parseJSON (Object vs) = case
		( parse (.: pack "name") vs
		, parse (.: pack "data") vs  :: Result [Int]
		, parse (.: pack "subdata") vs
		) of
			(Success pName, Success [i], Success [v])
				| pName == name -> case (case i of
					0 -> CDryad <$> mparse parseJSON v
					1 -> CNature <$> mparse parseJSON v
					2 -> CUnicorn <$> mparse parseJSON v
					_ -> Nothing) of
						Just d -> return d
						_ -> mzero
			_ -> mzero
	parseJSON _ = mzero

dryadDeck :: RandomGen g => Dryad -> Rand g (Enumerable CedarTree)
dryadDeck d = return . pure $ CDryad d

unicornDeck :: RandomGen g => Unicorn -> Rand g (Enumerable CedarTree)
unicornDeck u = return . pure $ CUnicorn u

natureSpiritDeck :: RandomGen g => NatureSpirit -> Rand g (Enumerable CedarTree)
natureSpiritDeck n = return . pure $ CNature n


data Index = DIndex Dryad.Index | UIndex Unicorn.Index | NIndex Nature.Index
	deriving (Eq, Ord, Show, Read)

data Var = AsDryad Dryad.Vars | AsUnicorn () | AsNature ()
	deriving (Eq, Ord, Show, Read)

instance Scene Index Var CedarTree where
	-- link :: Index -> BodyGlob -> CedarTree -> [Event CedarTree Index Var]
	link f = case f of
		DIndex di -> \b c -> case c of
			CDryad d -> asDryad <$> link di b d
			_ -> error "bad cedartree scene"
		UIndex ui -> \b c -> case c of
			CUnicorn u -> asUni <$> link ui b u
			_ -> error "bad cedartree scene"
		NIndex ni -> \b c -> case c of
			CNature n -> asNature <$> link ni b n
			_ -> error "bad cedartree scene"
	display = fmap toLower . show
	vardisplay = fmap toLower . show

-- issue: we describe the encounter and then the scene also describes the encounter, again. we need to separate scene from like, first interactive step
scene :: Encounter Index Var CedarTree
scene b c = describe b c <> case c of
	CDryad d -> asDryad <$> Dryad.openingChoices b d
	CNature n -> [] -- asNature <$> Nature.scene b n
	CUnicorn u -> [] -- asUni <$> Unicorn.scene b u

asDryad :: Event Dryad Dryad.Index Dryad.Vars -> Event CedarTree Index Var
asDryad = triMap CDryad DIndex AsDryad

asNature :: Event NatureSpirit Nature.Index () -> Event CedarTree Index Var
asNature = triMap CNature NIndex AsNature

asUni :: Event Unicorn Unicorn.Index () -> Event CedarTree Index Var
asUni = triMap CUnicorn UIndex AsUnicorn

describe :: Encounter Index Var CedarTree
describe b c =
	[ Narration2 $ "In the midst of the forest, you come across a towering cedar tree."
		/ case c of
			CDryad d -> "Shadowed in the canopy of the tree, in a mossy hollow between two gnarled humps of exposed roots, is a dryad:"
			CNature n -> "Shadowed in the canopy of the tree is a nature spirit:"
			CUnicorn u -> "Shadowed in the canopy of the tree is a unicorn:"
	] <> case c of
		CDryad d -> asDryad <$> (Dryad.describe d)
		CNature n -> asNature <$> (Nature.describe n)
		CUnicorn u -> asUni <$> (Unicorn.describe u)
