{-# LANGUAGE MultiParamTypeClasses #-}

module Sites.Hell.Events.Hubur.CrystalGolem
	( CrystalGolem(..)
	, deck
	, scene

	, name
	, bits
	, variants
	) where

import Prelude hiding ((/))

import Sites.Hell.Events.Construction ((/))
import Sites.Hell.Events.SceneUtils

import Sites.Hell.Events.Hubur

name :: String
name = "crystalgolem"

data CrystalGolem = CrystalGolem
	deriving (Eq, Ord, Show, Read)

instance Random CrystalGolem where
	randomR _ = random
	random = getRand $ pure CrystalGolem

instance ToJSON CrystalGolem where
	toJSON CrystalGolem = object
		[ pack "name" .= name
		, pack "data" .= ([] :: [Int])
		]

bits :: Value -> Value
bits _ = object
	[ pack "name" .= name
	, pack "data" .= ([] :: [Int])
	]

variants :: [(MapZone, ())]
variants =
	[ (FactoryMineTunnels, ())
	]

instance FromJSON CrystalGolem where
	parseJSON (Object vs) = case (parse (.: pack "name") vs, parse (.: pack "data") vs :: Result [Int]) of
			(Success pName, Success [])
				| pName == name ->
					case pure CrystalGolem of
							Nothing -> mzero
							Just a -> return a
			_ -> mzero
	parseJSON _ = mzero

deck :: RandomGen g => Rand g (Enumerable CrystalGolem)
deck = return $ pure $ CrystalGolem

data Index = Dummy
	deriving (Eq, Ord, Show, Read)

instance Scene Index () CrystalGolem where
	link f = case f of
		_ -> error "dummy index"

	display = fmap toLower . show
	vardisplay = fmap toLower . show

scene :: Encounter Index () CrystalGolem
scene _ c =
	[ description c
	-- , error "bad scene"
	]

description :: CrystalGolem -> Event CrystalGolem Index ()
description w = Narration2 $
	"There's a massive crystal golem, laboring."
	/ Paragraph / "It's made from black crystal, roughly humanoid, and it grinds and cracks as it moves. Its hands and feet are more like pegs, thinning as they extend from its bulk. What it does the lifting with is something like a ribcage and something like a mouth, that tears open across its chest. Under its surface there are flecks of red that catch the light, channeled together in something like a skeleton. The crystal becomes nearly-opaque only a few inches deep."
	/ Paragraph / "It's impossible to tell if it can think or talk; if it can, it's not in any hurry to tell you."
