{-# LANGUAGE ViewPatterns #-}
{-# LANGUAGE MultiParamTypeClasses #-}
{-# LANGUAGE FlexibleContexts #-}

module Sites.Hell.Events.Hubur.Dryad
	( Dryad(..)
	, Plant(..)
	, Shop(..)

	, Vars
	, Index

	, describe
	, openingChoices

	, deck
	, scene

	, name
	, bits
	, variants
	) where

import Prelude hiding ((/), (^))

import Control.Arrow
import Control.Monad
import Data.Maybe (catMaybes, fromMaybe)
--import Data.Set (member)

import Server.JSON
import Sites.Hell.Events.SceneUtils hiding ((/))
import Sites.Hell.Events.Construction ((/), (^))

import Sites.Hell.Events.Hubur

import Sites.Hell.Events.GameEvent (cockhead, cock, materialWord)
import Description.TfNode (Animal(..), Sex(Balls), Shape(More), TfNode(..))
import Description.Body.Sample hiding (feet, hands, hand, (?))

name :: String
name = "dryad"

data Dryad = Dryad
	{ plant :: Plant
	, shop :: Shop
	}
	deriving (Eq, Ord, Show, Read)
data Plant = Tree | Rose
	deriving (Eq, Ord, Enum, Bounded, Show, Read)
data Shop = NoShop | IsShop
	deriving (Eq, Ord, Enum, Bounded, Show, Read)

instance Random Plant where
	randomR (lo, hi) g = first toEnum . randomR (fromEnum lo, fromEnum hi) $ g
	random = randomR (minBound, maxBound)
instance Random Shop where
	randomR (lo, hi) g = first toEnum . randomR (fromEnum lo, fromEnum hi) $ g
	random = randomR (minBound, maxBound)
instance Random Dryad where
	randomR _ = random
	random = getRand $ Dryad
		<$> liftRand random
		<*> liftRand random

instance ToJSON Dryad where
	toJSON (Dryad p s) = object
		[ pack "name" .= name
		, pack "data" .= [fromEnum p, fromEnum s]
		]

bits :: Value -> Value
bits _ = object
	[ pack "name" .= name
	, pack "data" .= [Null, Null]
	]

variants :: [(MapZone, ())]
variants =
	[ (Glade, ())
	]

instance FromJSON Dryad where
	parseJSON (Object vs) = case (parse (.: pack "name") vs, parse (.: pack "data") vs) of
			(Success pName, Success [p, s])
				| pName == name ->
				case Dryad
					<$> safeToEnum p
					<*> safeToEnum s of
						Nothing -> mzero
						Just h -> return h
			_ -> mzero
	parseJSON _ = mzero

deck :: RandomGen g => Rand g (Enumerable Dryad)
deck = do
	plant <- liftRand $ shuffle [Tree, Rose]
	shop <- liftRand $ shuffle [NoShop, IsShop]
	return $ Dryad <$> from plant <*> from shop

data Index = OpeningChoices | Talk TalkOption | Leave | FarmVisit | ShopPrompt | FuckIntro | Voyeur | Blow | Frot
	deriving (Eq, Ord, Show, Read)

data TalkOption
	= Intro
	| SonOfHamarTabal
	| FlameDemon
	| SobekSheut
	| GanzerWorker
	| ClayMan
	| LesserHellhound
	| HellhoundKnight
	| Gargoyle
	| Dryads
	deriving (Eq, Ord, Enum, Bounded, Show, Read)

data Vars = Met
	deriving (Eq, Ord, Enum, Bounded, Show, Read)

instance Scene Index Vars Dryad where
	link x = case x of
		OpeningChoices -> openingChoices
		Talk opt -> talk opt
		ShopPrompt -> shopPrompt
		Leave -> leave
		FarmVisit -> farmVisit
		FuckIntro -> fuckIntro
		Voyeur -> voyeur
		Blow -> blow
		Frot -> frot
	display = fmap toLower . show
	vardisplay = fmap toLower . show

describe :: Dryad -> [Event Dryad Index Vars]
describe = description


description :: Dryad -> [Event Dryad Index Vars]
description d =
	[ SetFlag (Global MetDryad) 1
	, Narration2 $ c "He is roughly human-shaped, with arms and legs, a head, broad shoulders. " // (case plant d of
			Tree -> "He is made from living wood: his core brilliant green, the pale, bright color of new growth. Across his shoulders, down his legs, the color fades as his bark thickens, and it swells at his hands and feet: thick and knotted, giving him immense, gnarled paws with stumpy fingers. His face is clear enough, jaw sharp, nose wide, brows heavy. His eyes are a solid gleaming green; his pointed ears dragged out into long, twisted branches, buds of leaves growing at the tips. He has a thick mass of greenery for hair, cascading over his shoulders, the coiled vines and broad fernlike leaves framing his face like a mane."
			Rose -> "He is like a rose: his head, if he has one, is concealed within a massive blossom of black rose petals; behind the folded petals there is a slightest glimpse of glowing yellow eyes. His skin is brilliant green, the bright color of flower stems. At his core, down his chest, it lightens to a pale, milky green. Thorns erupt from his skin anywhere body hair would be expected: thick spikes coating the backs of his forearms, spreading across his chest, and covering his legs.")
		// (case shop d of
			NoShop -> mempty
			IsShop -> c Paragraph // "There's an array of alchemical junk spread out on the ground around him: bags of blackened pollen, crushed petals, split seeds, vials of marshy water.")
	, InlineBranch (C (Local Met) IsSet)
		[dialog d $ "Hey, welcome back."
			/ Paragraph / "Back for anything in particular?"]
		[]
	, SetFlag (Local Met) 1
	]

scene :: MarkedScene Index Vars Dryad
scene b d =
	[ Narration2 $
		nBranch (C (Local Met) IsSet)
			("You return to the dryad's glade." :: String)
			("In the marshy swamp shallows you stumble across the glade of a dryad. There's a big tree with long willowy branches in the center of a clearing, and the dryad is sat in a mossy hollow between two immense gnarled roots." :: String)
	] <> description d <> openingChoices b d

openingChoices :: MarkedScene Index Vars Dryad
openingChoices _ d = case shop d of
	NoShop ->
		[ Choices2
			[ ("hey you wanna fuck", FuckIntro, Nothing)
			, ("hey so there's this farm...", FarmVisit, Just $ C (Global LookingForDryadForFarmer) (Gt 0))
			, ("hey let's talk", Talk Intro, Nothing)
			, ("k bye", Leave, Nothing)
			]
		]
	IsShop ->
		[ Choices2
			[ ("hey you wanna fuck", FuckIntro, Nothing)
			, ("what're you working on?", ShopPrompt, Nothing)
			, ("hey so there's this farm...", FarmVisit, Just $ C (Global LookingForDryadForFarmer) (Gt 0))
			, ("hey let's talk", Talk Intro, Nothing)
			, ("k bye", Leave, Nothing)
			]
		]


talk :: TalkOption -> MarkedScene Index Vars Dryad
talk t _ d = case t of
	Intro ->
		[ Choices2
			[ ("...the sons of hamar-tabal", Talk SonOfHamarTabal, Just $ C (Global MetSonOfHamarTabal) IsSet)
			, ("...the ganzer workers", Talk GanzerWorker, Just $ C (Global MetGanzerWorker) IsSet)
			, ("...the dryads", Talk Dryads, Just $ C (Global MetDryad) IsSet)
			, ("...the clay men", Talk ClayMan, Just $ C (Global MetClayMan) IsSet)
			, ("...the flame demons", Talk FlameDemon, Just $ C (Global MetFlameDemon) IsSet)
			, ("...the lesser hellhounds", Talk LesserHellhound, Just $ C (Global MetLesserHellhound) IsSet)
			, ("...the hellhound knights", Talk HellhoundKnight, Just $ C (Global MetHoundKnight) IsSet)
			, ("...the gargoyles", Talk Gargoyle, Just $ C (Global MetGargoyle) IsSet)
			, ("...the sobek sheuts", Talk SobekSheut, Just $ C (Global MetSobekSheut) IsSet)
			, ("<==", OpeningChoices, Nothing)
			]
		]
	SonOfHamarTabal ->
		[ dialog d $ "There are some out here farming, they're pretty nice. Those ones could always use a dryad or two helping them out. But I guess most of them stick to the factory."
		, Choices
			[ ("<==", Talk Intro)
			]
		]
	FlameDemon ->
		[ dialog d $ "They're factory types; I don't see much of them out here. Unless they're trying to analyze elemental distributions in flora, or whatever. They're big into alchemy."
		, Choices
			[ ("<==", Talk Intro)
			]
		]
	SobekSheut ->
		[ dialog d $ "They get all high off the river water and look for people to fuck. It's tiring. I mean... I guess their thing is like, reproduction? Trying to get lucky with the gates of Guf. But honestly I think the zombies are less creepy about it. Souls are tacky and overrated."
		, Narration2 $ c "He pauses."
		, dialog d $ "Uh, no offense."
		, FauxChoice "nah its cool"
		, Choices
			[ ("<==", Talk Intro)
			]
		]
	GanzerWorker ->
		[ dialog d $ "Oh, I don't see them that often. Well, not since Ganzer fell, but that was... that was a" / em "while" / "ago. Good to see they've all mellowed out since then."
		, Choices
			[ ("<==", Talk Intro)
			]
		]
	ClayMan ->
		[ dialog d $ "They're fine."
			/ Paragraph / "They kinda get off on, like, us sending down roots all through their clay. And it's like, look, I'm just eating here. But whatever, it's not a big deal; they're nice enough."
		, Choices
			[ ("<==", Talk Intro)
			]
		]
	LesserHellhound ->
		[ dialog d $ "They're always yapping and chasing each other through the marsh. Loud and annoying is what they are."
		, Choices
			[ ("<==", Talk Intro)
			]
		]
	HellhoundKnight ->
		[ dialog d $ "So they're down from the mountains again? They just like stirring shit up with Cerberus, though they'd never admit it. They're happy to think of us as part of the background. And I'm plenty happy to let them."
		, Choices
			[ ("<==", Talk Intro)
			]
		]
	Gargoyle ->
		[ dialog d $ "The what? I guess I've seen some crawling around on the factory walls. I don't go over that way much."
		, Choices
			[ ("<==", Talk Intro)
			]
		]
	Dryads ->
		[ dialog d $ "Yeah?"
			/ Paragraph / "I mean, I'm fine."
		, Choices
			[ ("<==", Talk Intro)
			]
		]


leave :: MarkedScene Index Vars Dryad
leave _ _ =
	[ Narration
		[ "You leave."
		]
	]

farmVisit :: MarkedScene Index Vars Dryad
farmVisit _ d =
	[ Narration ["You tell the dryad about the nearby farm."]
	, dialog d $ "Sure, why not. It's been a while since I stretched my legs anyway."
	--, SetFlag (Global DryadsFound) (Mod Increment)
	, PushEncounterData Fix d (Global DryadFoundData)
	, SetFlag (Global LookingForDryadForFarmer) (Mod Decrement)
	, DeactivateSelf
	]

shopPrompt :: MarkedScene Index Vars Dryad
shopPrompt _ d =
	[ dialog d $ "Oh, I'm just doing some refining. Extracting the power of the wood, you know, dryad shit."
	, Narration ["The dryad gestures at the spread junk."]
	, dialog d $ "I could probably make you some stuff, if you want."
	, Shop "refine"
		[ A
			[ Left $ I GNARLED_ROOT 2
			] $ Right $ CX Cock
		, A
			[ Left $ I BUDDING_BRANCH 2
			] $ Right $ CX Balls
		, A
			[ Left $ I GNARLED_ROOT 1
			, Left $ I BUDDING_BRANCH 1
			] $ Right $ CX Knot
		, A
			[ Left $ I SOUL_SKIN 3
			] $ Right $ CS More
		]
	, Narration ["The dryad looks a little abashed."]
	, dialog d $ "You can probably tell where my mind's been lately, huh?"
	, Choices
		[ ("<==", OpeningChoices)
		]
	]

dialog :: Convertible a EventReel => Dryad -> a -> Event Dryad Index Vars
dialog _ = Dialog2 "/static/img/icon.png" "Dryad" . convert

-- TODO: floating dick scene
fuckIntro :: MarkedScene Index Vars Dryad
fuckIntro b d =
	-- "{short conversational exchange probably commenting on yr body/bodytype}"
	[ dialog d $ case locoplan b of
		Spec FloatingHead -> "You're eager, aren't you?"
			/ Paragraph / "All it took was getting eyes for you to start leering, didn't it."
		Spec FloatingDick -> c "Lemme guess, you wanna top. Call it a hunch, 'cause you decided to make yourself a living dildo."
		Spec Orb -> c "You must think you're a real hot-shit floating orb, huh?"
		Basic Formless -> c "You must think you're a real hot-shit floating orb, huh?"
		Basic Biped -> c "Sure, why not."
		Basic Quadruped -> c "Sure, why not."
		Basic Centaur -> c "Sure, why not."
		Basic Naga -> c "Y'know, I'm not really into snakes, but I guess you'll do."
		Basic Slithering -> c "Y'know, I'm not really into snakes, but I guess you'll do."
		f -> bodyFail FuckIntro f
	, Narration2 $ "He sprawls back on a bed of peat and moss, hands drifting down his chest -- tweaking his left nipple,"
		/ case plant d of
			Rose -> "palms "
			Tree -> "gnarled paws "
		/ "scratching down "
		/ case plant d of
			Rose -> "through the broad thorns fanned across his pecs, down "
			Tree -> mempty
		/ "to the thick spar of his cock, his flesh there gone"
		/ case plant d of
			Rose -> "white-green"
			Tree -> "dusky green"
		/ ". He strokes his cock, more tugging on it than anything hurried, slowly coaxing himself to full erection. When his foreskin draws back, his cockhead is bright green. He groans, head lolling back, strokes gaining purpose as he starts jerking off in earnest. His pre is opaque, like milkweed sap, a runny white fluid steadily drooling down his shaft, over his"
		/ case plant d of
			Rose -> "fingers"
			Tree -> "wooden paws"
		/ ". It smells, sharp and astringent, like the crushed leaves of a bitter herb."
	] <>
	case locoplan b of
		Spec FloatingHead ->
			[ Dialog "/static/img/icon.png" "Dryad"
				[ "You go for the floating head look because you really like using your mouth?"]
			, Narration
				[ "He opens his eyes to look at you and gives his dick a coaxing tug."]
			, Dialog "/static/img/icon.png" "Dryad"
				[ "Because I wanna try that out."]
			, Choices
				[ ("just watch", Voyeur)
				, ("blow him", Blow)
				]
			]
		x
			| x `elem` [Spec Orb, Spec FloatingDick, Basic Formless, Basic Complex] ->
				[ Dialog "/static/img/icon.png" "Dryad"
					[ "Too bad you don't got anything you could put to use."
					]
				, Narration
					[ "He tugs on his nipple, showily shuddering in pleasure."
					]
				, Dialog "/static/img/icon.png" "Dryad"
					[ "Guess you'll just have to watch."
					]
				, Choices
					[ ("yeah sure", Voyeur)
					]
				]
		_ ->
			[ Narration
				[ "He opens his eyes to look at you, lips pulling back in a lopsided grin."
				]
			, Dialog "/static/img/icon.png" "Dryad"
				[ "You gonna just watch?"
				]
			, Choices $ catMaybes
				[ pure ("yeah just watch", Voyeur)
				, guardC (hasMouth b) ("blow him", Blow)
				, guardC (hasDick b) ("straddle him + frot", Frot)
				]
			]

voyeur :: MarkedScene Index Vars Dryad
voyeur b d =
	[ Narration2 $ "He leers at you for a second before letting his head fall back down, looking up, eyes closed, jerking off like you're not even there, though he spreads his legs a little to give you a better view. He tugs his shaft, smearing milky pre over his" / case plant d of
			Tree -> "wooden flesh"
			Rose -> "pliant flesh"
		/ ", thumb smearing across the green-shoot skin of his cockhead until it's shiny. The flesh of his sac furrows, huge fruit-pit nuts pulling up tight against the base of his shaft, only for him to reach down with his other hand and tug them down, " / case plant d of
			Tree -> "thick paws grubbing at "
			Rose -> "fingers working "
		/ "his sac, "/ case plant d of
			Tree -> mempty
			Rose -> "rubbing his fingers across the short, spiny thorns all across his flesh, "
		/ "tugging in time with his strokes. He dribbles pre all down his shaft, letting it spatter in milky droplets across his stomach, forming rivulets along his chiseled muscles."
		/ Paragraph / "He lets his shaft drop, cock lolling across his stomach, steadily leaking as it comes to rest in the divot of his hip, spilling across his thigh. He uses his hand to scrape up his chest, smearing pre in sloppy finger-painted lines up across his chest, " / case plant d of
			Rose -> "working it into his fan of thorns before "
			Tree -> mempty
		/ "going back to milk out more fluid, back and forth until there's a solid line of wobbly smears up his stomach and chest, milky droplets running like sweat down his sides" / case plant d of
			Rose -> ", nubby spines surfacing like boulders in a river"
			Tree -> mempty
		/ "."
		/ Paragraph / "He groans and braces his feet, arching up, hand on his balls reaching back to play with his hole -- " / case plant d of
			Rose -> "dry fingers just spreading his cheeks, showing the green-pale flesh of his clenching hole, fingers tapping against it, the knuckle of his thumb still jogging against his balls, "
			Tree -> "grinding the rough bark of his clublike hand between his cheeks, stubby fingers digging their dull claws against his asshole as he grinds down, fat balls flopping down over the mass of dense wood and "
		/ "churning in his heavy sac, cock still twitching and leaking, even untouched. He humps up, cock flopping up, fat cockhead hitting his stomach with a loud" / em "slap" / ", working back and forth in shuddering spurts as he rolls his hips, fucking the air. His abs clench, stomach hollowing, cock sliding up and down over a sloppy trough of glazed flesh, " / case plant d of
			Rose ->"fingers"
			Tree -> "blunt claws"
		/ "digging against his hole, other hand splayed across his chest, " / case plant d of
			Rose -> "nails"
			Tree -> "claws"
		/ "scraping across his nipple, twisting so hard he bruises green-brown, sharp crushed-leaf-smelling sap glistening in the half-circle rings his nails leave."
		/ Paragraph / "He groans again, hand snapping down to his cock again, mouth open, breathing fast as he stops playing around. He fucks his hand, hips jerking up, grunting and groaning until he comes with a loud cry, cock flaring in his hand, green veins standing out from his" / case plant d of
			Tree -> "dark"
			Rose -> "pale"
		/ "flesh as he finally erupts: tarry, translucent green sap spewing from his cock, so thick it spurts up his chest and sticks there without hardly dripping at all. He groans louder, hand moving fast, squirt after squirt of sap spraying across his chest, splattering down in layers that each stand out from each other, thick beads of glossy green sap only slowly bleeding together into a thick ribbon of come up and down his chest."
		/ Paragraph / "He collapses back down onto the bed of moss, breathing hard, skin sheened with sweet-smelling sweat, come still drooling from his cock, practically gluing his hand down with its tar resin stickiness, final pulses just smearing across the back of his hand, spilling down onto his stomach."
		/ Paragraph / "He opens his eyes, looking over at you with a smirk."
	, Dialog "/static/img/icon.png" "Dryad"
		[ "Hope you enjoyed the show."
		]
	]

blow :: MarkedScene Index Vars Dryad
blow b d =
	[ Narration2 $
		case locoplan b of
			Spec FloatingHead -> do
				"You float on over between his thighs, and he reaches out when you get close and pulls you the rest of the way, pressing your" / muzzle / "against his cockhead, rubbing your head up and down against his shaft like you're a toy. You lap along the length of his cock before he draws you to the tip, his fat cockhead dribbling pre over your lips."
			Basic x | x `elem` [Biped, Slithering, Naga] ->
				hasLocotype Biped
					? "You get down between his legs, "
					?: "You slither closer, moss damp across your underbelly, "
				/ hasLocotype Slithering
					? ("until you touch his leg and wrap around it, coiling up his calf and thigh"
						/ case plant d of
							Rose -> ", thorns spiky "
								/ case bodyMat_ Tail b of
									Solid (FMat _ Bone) -> c "against your bones"
									_ -> "under your" / materialWord Tail b
								/ ","
							Tree -> mempty
						/ "until your" / muzzle / "bumps against his cockhead. He releases his cock, letting you coil around it"
						/ case bodySpecies Head b of
							Just Snake -> ", tongue flitting across his slit"
							Just Wolf -> ", slavering tongue lapping at his slit"
							_ -> ", tongue swiping across his slit"
						/ ".")
					?: (hand / "sliding up his inner thigh until you're grinding your knuckles against the root of his cock, under his balls. He releases his cock, letting it slap up against his abs with a smack, as you bring your other" / hand / "up, wrapping around his shaft to stroke it, smearing his milkweed pre along his shaft until it glistens dully.")
			Basic Quadruped -> "You" / case bodySpecies Legs b of
					Just Wolf -> "pad over"
					Just Unicorn -> "trot over"
					_ -> "walk over"
				/ "between his spread thighs and lap your tongue up his" / case plant d of
					Rose -> "thorn-stubbled"
					Tree -> mempty
				/ "thighs. He groans, body shuddering under you. His cock twitches and lolls to the side; he reaches out and guides it to your" / muzzle / ", smearing his dripping cockhead across your upper lip before you open wider, tongue" / case bodySpecies Head b of
					Just Wolf -> "lolling"
					Just Snake -> "flitting"
					_ -> "licking"
				/ "out to catch his cockhead and pull it into your mouth."
			Basic Centaur -> "You" / case bodySpecies Legs b of
					Just Wolf -> "pad over"
					Just Unicorn -> "trot over"
					_ -> "walk over"
				/ ", close, and fold your legs beneath you as you lean in, " / hands / "stroking up his"
				/ case plant d of
					Rose -> "thorn-stubbled"
					Tree -> mempty
				/ "thighs to wrap around his dick. He groans, hips rising up, cock sliding through your grasp, and you bend down, pressing your face against his hip and sliding up to lap at the head of his cock."
			f -> bodyFail Blow f
		/ Paragraph / "The taste is bitter, sharp on your tongue as you lean in and suck his cockhead into your mouth, fat flesh pulsing against your tongue. You drool down his shaft, thinning his tacky pre with spit, and start bobbing up and down in short motions"
		/ hasArms ? (", one" / hand / "tugging his balls, the other stroking his shaft; your"
			/ case bodyMat_ Arms b of
				Solid (FMat _ Fur) -> "hairy palm"
				Solid (FMat _ Scales) -> "smooth palm"
				_ -> "palm"
			/ "becoming slick with pre and spit")
		/ ". He groans, "
		-- TODO: check for material, so if yr a skeleton or w/e that doesn't come up. that's solidly t3 tho
		/ case bodySpecies Head b of
			Just x | hasTopEars x -> "hand pressing down between your ears"
			Just Human -> "hand tangling in your hair"
			_ -> "hand cupping the top of your head"
		/ ", and thrusts up to meet your mouth, building up a slow rhythm. His abs clench; all you can see of his body. His shaft sloppily plunges into your" / muzzle / ","
		/ case mouthType b of
			Mouth -> c "his hand on your head pulling you down, easing a fraction of an inch deeper with each thrust -- his fat cockhead first butting against the back of your mouth and then, with a slight hitch, sliding down your throat"
			_ -> "his entire shaft sliding into your" / muzzle / " -- his fat cockhead butting against the back of your mouth at the deepest point"
		/ ". The dryad groans, hand on your head slowly pulling you down until"
		/ case bodySpecies Head b of -- TODO: change to a mouthtype check?
			Just Wolf -> "your wet nose"
			Just Snake -> "the tip of your snout"
			_ -> "your nose"
		/ "presses against"
		/ case plant d of
			Rose -> "his thornbush pubes"
			Tree -> "the crest of his pelvis"
		/ ", his cock shuddering and twitching"
		/ case mouthType b of
			Mouth -> "against the clenching walls of your throat"
			_ -> "in the back of your mouth"
		/ "."
		/ Paragraph / "He pulls you off slowly, and not all the way, keeping his cockhead"
		/ case mouthType b of
			Mouth -> "in your throat"
			_ -> "drooling across your tongue"
		/ "as he thrusts, hips smacking your face in a slobbery mess of drool and pre, forming thick cords hanging"
		/ case mouthType b of
			Mouth -> c "from your chin"
			_ -> "down the underside of your" / muzzle
		/ ", drooling all across his thighs. His other hand pulls across his chest, fingers tugging at his nipples as he fucks your face, cock shuddering and gushing lines of stringy pre down your throat."
		/ Paragraph / "He groans louder, thrusts picking up, cock tensing, the ridges of his cockhead"
		/ case mouthType b of
			Mouth -> "digging into the flesh of your throat"
			_ -> "grinding against the back of your mouth"
		/ ", his breath huffing as he strains forward, pulling your head down and humping the final few inches of his cock back and forth, pants growing louder and louder until he lets out a sharp cry, cock throbbing. His come spills hot and thick"
		/ case mouthType b of
			Mouth -> "down your throat and you pull back, the next spurts flooding across your tongue"
			_ -> "in the back of your mouth, clogging your throat and spilling down across your tongue"
		/ ", thick and bitter-tasting. You swallow the rest of his load, green sap drooling from the corners of your mouth"
		/ hasArms ? ", stroking him again"
		/ "as he spills the final spurts into your open mouth, piling in a sticky heap across your tongue."
	] <> blowAftermath b d

blowAftermath :: MarkedScene Index Vars Dryad
blowAftermath b d =
	[ Narration2 $ "He cradles your head for a second before pushing you aside, his cock leaving a messy smear across"
		/ case mouthType b of
			Mouth -> c "your cheek"
			_ -> "the side of your" / muzzle
		/ "as it spills from your lips."
	, dialog d $ "Hey, good job."
		/ Paragraph / "Uh, seeya later."
	, Narration2 $ "You shakily "
		/ case locoplan b of
			Basic Slithering -> c "slither upright"
			Basic Naga -> c "slither upright"
			_ -> "get to your" / feet
		/ ", and he gives you a short lazy wave as you leave."
	]

frot :: MarkedScene Index Vars Dryad
frot b d = case locoplan b of
	x
		| x `elem` [Basic Biped, Basic Slithering, Basic Naga] ->
			frotBipedNagaSlither b d <> frotAftermath b d
		| x `elem` [Basic Quadruped, Basic Centaur] ->
			frotQuadCentaur b d <> frotAftermath b d
	f -> bodyFail Frot f
	-- TODO: and then a short exchange before you go

frotBipedNagaSlither :: MarkedScene Index Vars Dryad
frotBipedNagaSlither b d =
	[ Narration2 $ case locoplan b of
			Basic Biped -> "You straddle him, "
			_ -> "You slither up, "
		/ case locoplan b of
			Basic Slithering -> c "looping around his moss bed before you rise up. You coil around his arm, underbelly scales sliding smoothly across his cock."
			_ -> case dickLocation b of
				Just DickExternal -> "your" / cock / "splaying half-hard across the line of his muscled hip. "
				Just DickSheathed -> "your" / cock / "lolling, limp, from your sheath and spilling across the line of his muscled hip."
				_ -> "your vent flushed and bulging, " / cock / " peeking out, " / "its tip"^ p "their tips" / "digging into the line of his muscled hip."
		/ case locoplan b of
			Basic Slithering -> case dickLocation b of
				Just x | x `elem` [DickSheathed, DickExternal] ->
					let cockSheath = if x == DickSheathed then c "sheath" else c cock
					in "Your" / cockSheath / (if x == DickSheathed then c "hangs" else c $ Hangs^s) / "heavily from your streamlined body; it's impossible to miss its rough drag as it slides up the dryad's thigh."
						/ case x of
							DickSheathed -> "Your" / cock / Lolls^s / ", limp, across the line of his muscled hip."
							_ -> mempty
				_ -> "Your cloaca, flushed, drags roughly against the glade's ground, lips dragging up against his thigh before you grind against his cock. Your" / cock / Bulges^s / "outward, everting from inside you in little spurts. "
			_ -> c "His hands and yours wrap around your lengths, knuckles butting against each other; you don't stay half-hard for long. "
		/ "Your"
		/ oneCock ? "cock grinds against his, " ?: "shafts frame his, "
		/ hasArms ? "your hands squeezing them together; "
		/ case firstOf [Barbs, Medial, Knotted] dickAttrs b of
			Just Barbs -> c "your spongy spines rasp up and down his shaft as you grind against him"
			Just Medial -> "your gristly medial" / Ring^s / Bears^s / "down against his shaft"
			Just Knotted -> "your" / Knot^s / "just-barely" / Crests^s / "out, grinding against the base of his shaft"
			_ -> mempty
		/ ". His pre is milky, thicker than water, and your"
		/ case locoplan b of
			Basic Slithering -> "slow thrusts"
			_ -> "palms"
		/ "smear it up and down your"
		/ manyCocks ? (number (cockCount b +1))
		/ "shafts until they"
		/ oneCock ? "both" ?: "all"
		/ "shine dully."
		/ oneCock
			? "His fingers curl against your cockheads, squeezing them together, and his shallow thrusts butt them together, faint slick noises coming from the motion. "
			?: ("He reaches down, hand cupping all" / (number $ cockCount b + 1) / "shafts, squeezing gently to pin his cock between yours. ")
		/ Paragraph
		/ hasHead ? (case locoplan b of
			Basic Slithering -> "You twist your body across his, looping in a chunky curve around his arm, around his chest, "
				/ case bodySpecies Head b of
					Just Snake -> "and taste his face, tongue flitting out to lap across his cheek"
					Just Wolf -> "and drool all across his face, tongue slobbering over his cheek"
					_ -> "and press your face to his"
				/ ". He groans and raises his other arm to press against your neck, drawing you down in an ungainly kiss"
				/ case bodySpecies Head b of
					Just x | hasFangs x -> ", his tongue lapping across your fangs"
						| hasTusks x -> ", his tongue lapping across your tusks"
					_ -> mempty
				/ "."
			_ -> c "His other hand tugs on your shoulder, pulling you down on top of him, and he kisses you open-mouthed, hand sliding down your back.")
		/ case locoplan b of
			Basic Slithering -> c "He rolls his hips, cocks sliding through his fist, his thick, milky pre smearing with yours to make a frothy slurry, slopping across his hand and spilling across his stomach, lubing your motions as you grind against each other. "
			_ -> "He rolls his hips, your cocks pinned between your stomachs, "
				/ case bodyMat_ Chest b of
					Solid (FMat _ x) -> case x of
						x | x `elem` [Skin, Scales] -> "your pre and his smearing across your bellies, lubing your motions as you grind against each other."
							| x `elem` [Fur, Coat] -> "your pre and his soaking your fur to your stomach, grinding it deeper down in your coat as you rut together."
							| otherwise -> "sliding up and down as you rut against each other."
					_ -> "sliding up and down as you rut against each other."
		/ "He groans into your mouth, cock twitching against yours, leaking pre in sloppy gushes; his next thrust arches him an inch off the mossy ground, cock sliding against yours and sending a shocky burst of pleasure up your "
		/ case locoplan b of
			Basic Slithering -> "body"
			_ -> "spine"
		/ "."
		/ Paragraph
		/ "You build up a rhythm, "
		/ case locoplan b of
				Basic Slithering -> case dickLocation b of
					Just x | x `elem` [DickSheathed, DickExternal] -> "bodies grinding together, "
					_ -> "your cloaca rutting against his balls, "
				_ -> "hips grinding against together, "
		/ "your" / Cock^s / "sliding along his, combined pre making slick lube. "
		/ "He's panting fast, groaning into your mouth"
		/ case mouthType b of
			Mouth -> mempty
			_ -> "and across your" / muzzle
		/ case locoplan b of
			Basic Slithering -> c "and he grabs for your body, pinning your slowly-churning curls to his crotch, letting him hump against you with more force, "
			_ -> "as he laces both arms"
				/ case bodyMat Neck b of
					Just (Solid (FMat _ Fur)) -> "around your hairy shoulders"
					Just (Solid (FMat _ Skin)) -> "around your shoulders"
					Nothing -> "over your shoulders" -- headless
					_ -> "over your shoulders"
		/ ", excess fluid spilling down his sides in shining rivulets. His entire body goes taut, stomach and thighs straining, and he makes a few short, sharp bucks, moan swallowed up in your mouth."
		/ Paragraph
		/ "His cock trembles, twitching against yours, and then he erupts in messy streamers of thick come, green sap; the slight catch of the sticky fluid is enough to bring you off. You blow a half-second later, " / Cock^s / "sliding through the sticky mess, adding your own load to the mess squelching between your bodies"
		/ case bodyMat_ Chest b of
			Solid (FMat _ Fur) -> ", clumping your fur into heavy, dripping tassels"
			Solid (FMat _ Coat) -> ", matting down your coat to your heaving sides"
			_ -> mempty
		/ ". You thrust forward, grinding your spurting " / Cock^s / "across his belly, the both of you drenched in come as you shoot off again and again, his green-sap come gone murky and translucent, stretching out in taffylike globs between your chests."
		/ Paragraph
		/ case locoplan b of
			Basic Slithering -> "You spill to the side, lurching your" / Cock^s / "against his in a final thrust, "
			_ -> do
				"You sit back, giving your" / Cock^s / "a final tug to coax the last of your load out, "
		/ case cockCount b of
			1 -> "firing in a slashing line across his chest, "
			2 -> "firing in twin lines up his chest, "
			_ -> "firing in a mess of spurting lines across his chest, "
		/ "adding a fresh coat to the smeared froth all over his front; he gives you a tired smile, tugging on his dick, a final dribble of green sap smearing across his cockhead."
	]

frotQuadCentaur :: MarkedScene Index Vars Dryad
frotQuadCentaur b d =
	[ Narration2 $ "You step forward, " / feet / "bracketing his hips, and then again, standing over him. Your cock slaps against his stomach "
		/ case fromMaybe Normal'5_7 $ cocksizeO b of
			-- we're not actually committing to handling cock sizes at this stage
			-- Non -> {}
			-- Small'1_4 -> "when you're standing over him."
			-- Normal'5_7 -> {}
			-- Large'8_12 -> "as you approach, flat cockhead dragging up nearly to his pecs."
			-- Extreme'13_18 -> {}
			-- Gargantuan'19_26 -> "before your forelegs even approach his hips, flat cockhead dragging up his pecs and smacking him in the face when you finally stand above him."
			-- _ -> "{before you're even close / etc}"
			_ -> "as you approach, " / Cockhead^s / "dragging up nearly to his pecs"
				/ (\b -> bodySpecies Cock b == Just Goat) ? ", your lashing pizzle squirming like a fat rubbery worm over his chest"
				/ "."
		/ case locoplan b of
			Basic Centaur -> "You reach down and pull his head up, pressing his face against your" / Cockhead^s / ", just past your" / forelegs / "."
			_ -> "You press your face against his, your immense cock sandwiched between your mouths, your drool spilling over your" / Cockhead^s / "into his open, panting mouth."
		/ "He grinds his cock against the underside of your shaft, using both hands to pump your mismatched shafts together. Your pre drizzles down your shaft, making his strokes slurp. You" / groan / ", jerking forward, and your cock cannonades up"
		/ case plant d of
			Tree -> "his chest"
			Rose -> "through the fanned thorns across his chest"
		/ ", slapping him in the face,"
		/ case bodySpecies Cock b of
			--Just Horse -> "cockhole spreading as you squirt him full-on in the face with a messy blast of pre" // c Comma
			Just Unicorn -> "cockhole spreading as you squirt him full-on in the face with a messy blast of pre,"
			Just Goat -> "your pizzle spraying sludgy lines of pre all over,"
			_ -> "your pre"
		/ "running in liquid rivulets across his skin."
		/ case locoplan b of
			Basic Centaur -> "You rub his face against your cockhead, keeping his head pinned down as you plaster his face in gush after gush of pre."
			_ -> case bodySpecies Head b of
				Just Wolf -> "You drag your tongue through the mess and feed it into his mouth, slobber and pre flooding his mouth and spilling down his lips."
				Just Goat -> "You bleat against his face, tongue lolling out to drag through the mess, feeding it into his mouth and overflowing down his lips."
				_ -> mempty
		/ "He moans, the sound vibrating through your body."
		/ Paragraph / "He hunches up, rubbing his cock"
			/ oneCock ? "along your sloppy shaft" ?: "between your sloppy shafts"
			/ ", mouth open wide as you splatter pre in bursts across his face, until he's glazed and shiny, thick cords of pre oozing down his neck and"
			/ case plant d of
				Rose -> "winding their way through the broad, nubby spikes fanning"
				Tree -> mempty
			/ "across his pecs."
		/ Paragraph
		/ case bodySpecies Cock b of
			Just Unicorn -> c "He works his hands back and forth over your medial ring, gripping the spongy flesh"
			_ -> c (hasKnot
				? ("He strokes down your cock to the base, hands cupping the rise of your knot"
					/ case dickLocation b of
						Just DickExternal -> mempty
						Just _ -> ", still lodged in your" / sheath
						f -> bodyFail Frot f)
				?: "He strokes your shaft up and down")
		/ ", his cockhead bumping against the underside, adding his milky pre to the sloppy mess drizzling all down your cocks."
		/ Paragraph
		/ hasKnot ? ("He reaches back to squeeze your growing knot, coaxing it just-so out of your" / sheath / ", fingers hooking behind it and dragging you forward, the sharp shock of pleasure making you growl and gush another squirt of pre across his body.")
		/ Paragraph
		/ "You snort as you grind down against him, "/ partAdjective Underbelly / "underbelly pressed against his chest, your cocks grinding increasingly sloppily between you."
		/ case bodySpecies Cock b of
			Just Unicorn -> c "The rim of your flare slams against his jaw with each thrust,"
			Just Wolf -> c "Your sharp cocktip slaps against his jaw with each thrust,"
			Just Goat -> c "Your writhing pizzle squirms across his face, "
			Just Bull -> c "Your gnarled cockhead ruts against his jaw, "
			_ -> "Your" / cockhead / "presses against his face, "
		/ "pelting him with runny pre until he's slathered so thick it's nearly opaque. You" / groan / "," / Cock^s / "spasming"
		/ case firstOf [Flared] dickAttrs b of
			Just _ -> ", the gristled lump behind your cockhead swelling as your flare clenches and spreads out"
			_ -> mempty
		/ case firstOf [Knotted] dickAttrs b of
			Just _ -> (", your" / Knot^s / "throbbing hard and swelling, " / "its"^p "their" / "aching bulk growing heavier and harder with each beat")
			_ -> mempty
		/ ". A solid rope of come slaps his face with enough force that it rebounds, splattering off his glazed face like a rock being skipped over water. "
		/ case locoplan b of
			Basic Centaur -> "You grip his head with both hands, pinning him in place between your forelegs as you paint him in your load. "
			_ -> "He squeezes your shaft, humping your spurting cock as you paint him in your load. "
		/ "He opens his mouth to moan, noise cut off in a gurgle as you flood his mouth, and he swallows messily, come drooling out over his lips."
		/ case bodySpecies Cock b of
			Just Unicorn -> "He presses his lips to your flared cockhead, just under your hole,"
			Just Wolf -> "He sucks your sharp cocktip into his mouth, "
			Just Goat -> "He sloppily sucks on your pizzle, obscene wet sounds coming from his mouth as he tries to fit more of your cockhead into his mouth, "
			Just Bull -> "He sloppily sucks on the rim of your gnarled cockhead, huffing, "
			Just Snake -> "He drags his face along the stiff lines of flesh webbed along your cock, "
			_ -> "He sucks along your spurting cockhead, "
		/ "his moan vibrating up through your flesh. His hips jerk, cock sliding along yours, and he adds his load to the mess slathered across his body, muscles spasming weakly as he comes."
		/ Paragraph / "He collapses back, the rest of your load dribbling down across his chest, your" / Cock^s / "still weakly pulsing, him and the greenery beneath absolutely saturated in come. You step back slowly, softening" / Cock^s / "dragging down his chest as a heavy weight before lolling to the side with a final twitch, stretching out messy cords of come between the two of you."
	]

frotAftermath :: MarkedScene Index Vars Dryad
frotAftermath b d =
	[ Narration2 $ "He breathes hard, panting in the aftermath, sloppy lines of come spilling down his chest and sides. He reaches down, catching a thick smear of your combined loads on his "
		/ case plant d of
			Rose -> "finger "
			Tree -> "paw "
		/ "and brings it to his mouth, lapping it up."
	, dialog d $ "Hey, good."
	, Narration2 $ "He lets his hand fall with a thump, bonelessly lolling his head until he's staring straight up"
		/ case plant d of
			Tree -> ", eyes glassy after his orgasm"
			Rose -> ""
		/ "."
	, dialog d $ "Good job."
		/ Paragraph / "Uh, seeya later."
	, FauxChoice "yeah bye"
	, Narration ["He gives a boneless wave as you go."]
	]
