{-# LANGUAGE MultiParamTypeClasses #-}
{-# LANGUAGE FlexibleContexts #-}

module Sites.Hell.Events.Hubur.FarmerHamarTabal
	( FarmerHamarTabal
	, deck
	, scene

	, name
	, bits
	, variants
	) where

import Prelude hiding ((/), (^))

import Sites.Hell.Events.SceneUtils hiding ((/))
import Sites.Hell.Events.Construction ((/), (^))
import Sites.Hell.Events.Hubur

import Description.TfNode (Animal(..))

import Sites.Hell.Events.Hubur.HamarTabal (HamarTabal(..), description, Location(..), Clothes(..), Hair(..), Piercings(..), Tone(..), Horns(..))
import Sites.Hell.Events.Hubur.Dryad (Dryad(..), Plant(..))

name :: String
name = "farmer"

data FarmerHamarTabal = FarmerHamarTabal HamarTabal DryadStatus
	deriving (Eq, Ord, Show)

data DryadStatus = NoDryad | HasDryad Dryad
	deriving (Eq, Ord, Show, Read)

instance Random FarmerHamarTabal where
	randomR _ = random
	random = getRand $ do
		d <- deck
		roll d

instance ToJSON FarmerHamarTabal where
	toJSON (FarmerHamarTabal farmer dstate) = case dstate of
		NoDryad -> object
			[ pack "name" .= name
			, pack "data" .= ([] :: [Int])
			, pack "subdata" .= [toJSON farmer]
			]
		HasDryad dryad -> object
			[ pack "name" .= name
			, pack "data" .= ([] :: [Int])
			, pack "subdata" .= [toJSON farmer, toJSON dryad]
			]

-- idk if this would need subdata? like it's not used for anything right now, so, probably not.
bits :: Value -> Value
bits _ = object
	[ pack "name" .= name
	, pack "data" .= ([] :: [Int])
	]

variants :: [(MapZone, ())]
variants =
	[ (Field, ())
	]

instance FromJSON FarmerHamarTabal where
	parseJSON (Object vs) = case
		( parse (.: pack "name") vs
		, parse (.: pack "data") vs :: Result [Int]
		, parse (.: pack "subdata") vs) of
			(Success pName, Success [], c) | pName == name -> case c of
				Error _ -> mzero
				Success [farmer] -> FarmerHamarTabal
					<$> parseJSON farmer
					<*> pure NoDryad
				Success [farmer, dryad] -> FarmerHamarTabal
					<$> parseJSON farmer
					<*> (HasDryad <$> parseJSON dryad)
				Success _ -> mzero
			_ -> mzero
	parseJSON _ = mzero

deck :: RandomGen g => Rand g (Enumerable FarmerHamarTabal)
deck = do
	hamar <- hamarTabalDeck
	return $ FarmerHamarTabal
		<$> hamar
		<*> pure NoDryad

hamarTabalDeck :: RandomGen g => Rand g (Enumerable HamarTabal)
hamarTabalDeck = do
	hair <- liftRand $ shuffle [Shaved, Dreads]
	pierced <- liftRand $ shuffle [IsPierced, NotPierced]
	tone <- liftRand $ shuffle [Veiny, Hairy, Muscled]
	horns <- liftRand $ shuffle [NoHorns, Horns]
	return $ HamarTabal
		<$> pure Skirt
		<*> from hair
		<*> from pierced
		<*> from tone
		<*> from horns
		<*> pure Farm


data Index
	= FirstVisit
	| Talk | Leave
	| FarmQuestion | LeaveRequest
	| Propose | Reminder
	| DryadSuccess
	| RevisitAfterDryad | JoinIn | LeaveEm
	| FuckHim | SuckOffDryad
	| FuckHimHarder | Share
	deriving (Eq, Ord, Enum, Bounded, Show, Read)

data Vars = DryadQuest
	deriving (Eq, Ord, Enum, Bounded, Show, Read)

instance Scene Index Vars FarmerHamarTabal where
	link f = case f of
		FirstVisit -> firstVisit
		Talk -> talk
		Leave -> leave
		FarmQuestion -> farmQuestion
		LeaveRequest -> leaveRequest
		Propose -> propose
		Reminder -> reminder
		DryadSuccess -> dryadSuccess
		RevisitAfterDryad -> revisitAfterDryad
		JoinIn -> joinIn
		LeaveEm -> leaveEm
		FuckHim -> fuckHim
		SuckOffDryad -> suckOffDryad
		FuckHimHarder -> fuckHimHarder
		Share -> share
	display = fmap toLower . show
	vardisplay = fmap toLower . show

{-
InlineBranch
	(C (Local DryadQuest) (Eq 0)
		`VAnd` C (Global DryadFoundData) (Contains "dryad"))
	[ PopToSubData (Global DryadFoundData) ["dryad"]
	]
	[]


InlineBranch' (VAnd' (C' "dryadquest" (Eq (VN 0))) (C' "&dryadfounddata" (Contains "dryad"))) [PopToSubData' "&dryadfounddata" ["dryad"]] []

{"fail":[],"cond":{"a":{"cond":{"op":"eq","val":0},"flag":"dryadquest"},"b":{"cond":{"op":"peek","val":"dryad"},"flag":"&dryadfounddata"}},"type":"inlineBranch","pass":[{"flag":"&dryadfounddata","allowed":["dryad"],"type":"popToSubData"}]}


-}

dialog :: Convertible ev EventReel => FarmerHamarTabal -> ev -> Event m a b
dialog _ = Dialog2 "/static/img/icon.png" "Son of Hamar-Tabal" . convert


scene :: Encounter Index Vars FarmerHamarTabal
scene _ (FarmerHamarTabal _ NoDryad) =
	[ Jump FirstVisit
	]

scene _ (FarmerHamarTabal _ (HasDryad _)) =
	[ Branch
		[ (C (Local DryadQuest) (Gt 0), RevisitAfterDryad)
		] DryadSuccess
	]

firstVisit :: Encounter Index Vars FarmerHamarTabal
firstVisit _ (FarmerHamarTabal farmer _) =
	[ Narration2 $ c "The floodplain is a mess of shallow hills and shallower valleys, all half-flooded with silt. But over the next hill, in the valley below, there's a small farm: circular clay building, ramshackle wooden fences, a field of green shoots, a small orchard."
	, description farmer
	, Choices
		[ ("go talk to him", Talk)
		, ("leave", Leave)
		]
	]

dryadSuccess :: Encounter Index Vars FarmerHamarTabal
dryadSuccess _ f =
	[ SetFlag (Local DryadQuest) 1
	--, SetFlag (Global DryadsFound) (Mod Decrement)
	, Narration2 $
		"Looks like the dryad beat you here. He's talking with the son of Hamar-Tabal. Actually, judging by his posture it's less 'talking' and more 'flirting'."
		/ Paragraph / "They turn away from their conversation to talk to you. The dryad waves from the hip."
	, dialog f $ "Hey, thanks. He'll help with the orchard for sure."
		/ Paragraph / "Oh hey, there's this; it might be useful for you."
	, ItemModify
		[ I BUDDING_BRANCH 2
		, I GNARLED_ROOT 2
		]
	]

talk :: Encounter Index Vars FarmerHamarTabal
talk _ f =
	[ Narration2 $ c "You approach. The son of Hamar-Tabal notices you and straightens, hands falling off the strap of the plow."
	, dialog f $ "Hey there, traveller. What brings you here?"
	, Choices
		[ ("so whats w/ the farm", FarmQuestion)
		, ("uh nothing/well bye", LeaveRequest)
		]
	]

leave :: Encounter Index Vars FarmerHamarTabal
leave _ _ =
	[ Narration2 $ c "Sure, farms are boring."
	]

leaveRequest :: Encounter Index Vars FarmerHamarTabal
leaveRequest b f =
	[ Branch
		[ (C (Local DryadQuest) (Eq 0), Reminder)
		] Propose
	]

reminder :: Encounter Index Vars FarmerHamarTabal
reminder _ f =
	[ dialog f $ "Well, see you."
		/ Paragraph / "Keep an eye out for a dryad, if you would."
	]

propose :: Encounter Index Vars FarmerHamarTabal
propose _ f =
	[ dialog f $ "Well, I'll be seeing you."
		/ Paragraph / "Oh, since you're wandering around -- if you happen to see a dryad around, tell 'em to come over here. I could always use some help with the plants."
	, SetFlag (Local DryadQuest) 0
	, SetFlag (Global LookingForDryadForFarmer) (Mod Increment)
	]

farmQuestion :: Encounter Index Vars FarmerHamarTabal
farmQuestion _ f =
	[ dialog f $
		"You really are a fresh one, aren't you? Haven't seen any hellhounds yet, have you? The royal ones; not Cerberus' hounds."
		/ Paragraph / "This is hell, lost soul. Whatever world you were incarnated in -- there was ground, air, space. A world made from gross matter, obeying physical rules. You remember it well, even without your memory; it's writ into the structure of your soul."
		/ Paragraph / "Here in hell, there are no rules. But the hellhounds, and the Annunaki who rule beyond them, have made a declaration: that the world should be shaped in a way that makes" / em "sense" / ". Something comprehensible, for the poor souls interred here, so that they may see and comprehend the world."
		/ Paragraph / "Hence: farms. Bodies, food, growth."
	, FauxChoice "so it's not real?"
	, dialog f $ "That's quite the question coming from a blob of soul dreaming of being something bigger. Nothing exists just as itself."
	, Choices [("well okay", Talk)]
	]

revisitAfterDryad :: Encounter Index Vars FarmerHamarTabal
revisitAfterDryad b (FarmerHamarTabal _ NoDryad) = error "bad scene RevisitAfterDryad"
revisitAfterDryad b (FarmerHamarTabal _ (HasDryad dryad)) =
	[ Narration2 $ "You head back to the farm."
		/ Paragraph / "The son of Hamar-Tabal seems to be otherwise occupied -- they're in the orchard, and the dryad has been shoved up against the trunk of a tree, one hand lifted up to reach back around the trunk, the other threaded into the son of Hamar-Tabal's hair, guiding him back and forth along his cock."
		/ Paragraph / "The son of Hamar-Tabal slurps loudly as he bobs down, shoulders jerking when he swallows. When he pulls back the dryad's cock is visibly gleaming with spit, bright rivulets spilling down his shaft and slowly dripping down the furrowed flesh of his balls. The dryad moans and arches forward, arm tugging gently forward, and the son of Hamar-Tabal sinks down, lips hitting the dryad's"
		/ case plant dryad of
			Rose -> "thorned"
			Tree -> mempty
		/ "crotch with a wet slap. They've been at this for a while."
	] <> (case (locoplan b, hasDick b) of
		(Basic Formless, _) ->
			[ Choices
				[ ("nah leave em to it", LeaveEm)
				]
			]
		(Basic _, True) ->
			[ Choices
				[ ("get closer & join in", JoinIn)
				, ("nah leave em to it", LeaveEm)
				]
			]
		_ ->
			[ Choices
				[ ("nah leave em to it", LeaveEm)
				]
			])

leaveEm :: Encounter Index Vars FarmerHamarTabal
leaveEm _ _ =
	[ Narration2 $ open "You decide to leave them to their fucking."
	]

joinIn :: Encounter Index Vars FarmerHamarTabal
joinIn b _ =
	[ Narration2 $ "You take a step closer."
		/ Paragraph / "The dryad looks up from watching the son of Hamar-Tabal blow him and gives you a cockeyed grin. You don't know when the son of Hamar-Tabal realizes you're there -- maybe his tail-head saw you -- but all he does is spread his legs when you get close, not distracted for a minute from sucking on the dryad's dick."
		/ case locoplan b of
			Basic x | x `elem` [Biped, Naga, Centaur] -> "He moans when you grab his ass though, squirming when you spread his hairy cheeks, to reveal his hole: flesh dusky grey-purple, already clenching and parting in anticipation."
			-- FIXME: hasHead etc
			_ -> "He moans when you nose against his cheeks though, lapping between them to press against his hole: flush dusky grey-purple, already clenching and parting in anticipation."
	, Choices
		[ ("fuck the son of hamar-tabal", FuckHim)
		, ("join him in sucking off the dryad", SuckOffDryad)
		]
	]

fuckHim :: Encounter Index Vars FarmerHamarTabal
fuckHim b (FarmerHamarTabal _ NoDryad) = error "bad scene FuckHim"
fuckHim b (FarmerHamarTabal farmer (HasDryad dryad)) =
	[ Narration2 $ (case locoplan b of
		Basic x
			| x `elem` [Quadruped, Centaur] ->
				"You mount him in a rush, " / feet / "scraping his sides. Your" / Cock^s / "is"^p "are" / "already"
				/ (case dickLocation b of
					Just DickExternal -> c "sticking out hard from between your hind legs"
					Just _ -> "jutting out from your" / sheath
					f -> bodyFail FuckHim f)
				/ ", drooling, and so you hunch forward," / Cock^s / "skidding across his hairy ass and sending an electric pulse of pleasure up your spine -- you thrust once, twice, before your" / manyCocks ? "left" / "cock catches on his clenching asshole and shoves inside. You slam in,"
				/ manyCocks
					? "other cock skewing up his cheek to dribble pre across his back as you sink"
					?: "sinking"
				/ (case (hasKnot b, hasExternalBalls b) of
					(True, _) -> "knot-deep"
					(False, True) -> "to the balls"
					(False, False) -> "to the root")
				/ "in a single thrust"
				/ case firstOf [Medial] dickAttrs b of
					Just Medial -> ", medial ring crashing against his hole for a fraction of a second before it punches inside"
					_ -> mempty
				/ ". The force of your thrust shoves the son of Hamar-Tabal's shoulders against the dryad's thighs, nose mashed flat against the"
				/ case plant dryad of
					Tree -> "muscled"
					Rose -> "thorny"
				/ "arch of the dryad's crotch."
				/ manyCocks ? "You thrust once, then draw back, and your second cock drags down his back to line up right against his already-occupied hole, cockhead digging into the furrowed ring. You shove it inside too, all the way down; the son of Hamar-Tabal's bellow burbles out around the dryad's cock."
			| x `elem` [Biped, Naga] ->
				"You" / (case x of
					Biped -> "kneel"
					Naga -> "slither up"
					f -> bodyFail FuckHim f)
				/ "behind him,"
				/ case dickLocation b of
					Just DickExternal -> "tugging your" / Cock^s / "to full hardness"
					Just _ -> "rubbing your" / sheath / "til your" / Cock^s / Spills^s / "out, half-hard"
					f -> bodyFail FuckHim f
				/ ", and press" / manyCocks ? "one" ?: "your" / "cockhead against his hole: he groans, muffled around the dryad's cock, and his hole spasms and opens, practically sucking you inside."
				/ manyCocks
					? ("You tug back, cockhead pulling from his hole with a soft" / em "pop" / ", and then shove your other cockhead in. Then, slowly, you feed the first one in alongside the other, both at once stretching his asshole into a broad oval. You push inside with a lurch and slide all the way in")
					?: "You slide in easily, all the way down to the hilt"
				/ (case (hasCloaca b, hasExternalBalls b) of
					(False, True) -> "; your balls hit his ass with a smack"
					(True, _) -> ", until your flushed vent grinds against the cheeks of his ass"
					_ -> mempty)
				/ "."
		Basic Slithering ->
			"You coil all around his chest and thighs," / Cock^s / "rutting up against his ass, drooling more as you pull yourself tight. You slide forward, cockhead digging against his asshole, popping inside when he flexes his asshole open."
			/ manyCocks ? "Your other cockhead bears down against his already-occupied hole, spurting pre, and lurches inside a half-second later."
		Basic Formless ->
			"You float up behind him," / Cock^s / "sliding across his ass, and then rotate into place," / manyCocks ? "left" / "cockhead bearing down against his flush asshole and popping in" / manyCocks ? ", followed by the right a half-second later" / "."
		f -> bodyFail FuckHim f)
		/ Paragraph
		/ case locoplan b of
			Basic Centaur ->
				"You practically pin the son of Hamar-Tabal beneath you, the entire length of your"
				/ case bodySpecies Cock b of
					Just a -> animalName a
					_ -> mempty
				/ Cock^s / "shoved inside him, underbelly pressed against his back. You're face-to-face with the dryad, and he raises his other arm and tugs you forward, lazily kissing you"
				/ case plant dryad of
					Rose -> "at the center of his rose-petal head"
					Tree -> mempty
				/ "while you both hilt into the son of Hamar-Tabal below, his groans muffled into sloppy sputters and gags."
			Basic Slithering ->
				"You wrap all around the son of Hamar-Tabal and the dryad too, looping between him and the tree. You pull the dryad's hips forward and the son of Hamar-Tabal's head down in time with your thrusts, coils squeezing around the dryad's waist and over the back of the son of Hamar-Tabal's neck, tight enough that he sputters and coughs, face pinned down against the dryad's"
				/ case plant dryad of
					Tree -> "muscular"
					Rose -> "thorny"
				/ "crotch."
			Basic Formless ->
				"You thrust by convolving," / Cock^s / "driving in piston-like and then pulling entirely free, half-submerging in your body before" / "it smears"^p "they smear" / "across his hole and" / Shoves^s / "back in."
			Basic x | x `elem` [Naga, Biped, Quadruped, Wyvern] ->
				"The two of you find a rhythm with the son of Hamar-Tabal pinned between you,"
				/ hasExternalBalls
					? "your balls steadily clapping against his ass"
					?: ("steadily fucking him with"
						/ (manyCocks ? "both cocks" ?: "the whole length of your cock"))
				/ "as the dryad hilts in his mouth, your thrusts making him sputter and cough as the motion stirs the dryad's cock around in his throat."
			f -> bodyFail FuckHim f
	] <> (case locoplan b of
		Basic Centaur -> [Jump FuckHimHarder]
		Basic Formless -> [Jump FuckHimHarder]
		_ -> if hasMouth b
			then [Choices
				[("fuck him harder", FuckHimHarder)
				,("lean in and share the dryad's cock", Share)
				]]
			else [Jump FuckHimHarder])

data Path = DidShare | DidFuck
	deriving (Eq, Ord, Show, Read)

fuckHimHarder :: Encounter Index Vars FarmerHamarTabal
fuckHimHarder b f =
	[ Narration2 $ open "You ram him harder, each thrust jolting his body forward, smearing his face against the dryad's stomach, the bulge of the dryad's cock in his throat jumping up and down as he gurgles."
		/ Paragraph
		/ (case locoplan b of
			Basic Centaur -> c "The dryad grins, draping his arms over your shoulders, and arches up, pinning the son of Hamar-Tabal's head right between your forelegs. he kisses you again, tongue lapping at your lips."
			Basic Quadruped -> c "The dryad grins, hand lifting off of the son of Hamar-Tabal's head for a second to scratch"
				/ case bodySpecies Head b of
					Just a | a `elem` [Snake, Human] -> "across the top of your head"
					_ -> "between your ears"
				/ ", urging you on as your thrusts slam him deeper down onto the dryad's cock."
			Basic Slithering -> "You're coiled all up and down their bodies, tying them together -- wrapped right around one of the son of Hamar-Tabal's legs," / Cock^s / "pounding into his hole, and then you're looped in a spiral all up his stomach and chest and shoulders, up across the dryad's waist. His hand wraps around your neck, lifting your head up, and the dryad kisses you sloppily, mouth open, tongue lapping"
				/ (case mouthType b of
					Mouth -> c "over your lips"
					_ -> "along your" / muzzle)
				/ "into your mouth, spit spattering down across the son of Hamar-Tabal's back."
			Basic x | x `elem` [Biped, Naga] -> c "The dryad grins, raising his other hand to tug you forward, lips meeting in a sloppy kiss over the son of Hamar-Tabal's back. The dryad laps at your lips, tongue flitting inside your mouth, the both of you groaning and huffing as you pound the son of Hamar-Tabal."
			f -> bodyFail FuckHimHarder f)
		/ Paragraph / "The dryad fucks the son of Hamar-Tabal's face in short sharp jerks. A frothy mess of spit and pre gurgles from the son of Hamar-Tabal's mouth and nose as you brutally spitroast him, your"
		/ case bodySpecies Cock b of
			Just a -> animalName a
			_ -> mempty
		/ "cock punching back and forth through his puffy, bruised asshole, each thrust sending messy squirts of pre erupting out" / "along"^p "between" / "your pumping" / Shaft^s / "."
	]
	<> fuckClimax DidFuck b f

share :: Encounter Index Vars FarmerHamarTabal
share b f = suckOffInterject b f <> fuckClimax DidShare b f

suckOffDryad :: Encounter Index Vars FarmerHamarTabal
suckOffDryad b f = suckOffInterject b f <> climaxClose DidShare b f <> closerDialog b f

suckOffInterject :: Encounter Index Vars FarmerHamarTabal
suckOffInterject b f =
	[ Narration2 $ "You lean forward"
		/ case mouthType b of
			Mouth -> c "over the son of Hamar-Tabal's shoulders"
			_ -> "," / muzzle / "butting against the son of Hamar-Tabal's shoulders"
		/ "and"
		/ case bodySpecies Head b of
			Just Unicorn -> "nibbling across the dryad's shaft with your lips"
			_ -> "lap at the dryad's cock"
		/ ", adding your spit to the mess dully shining across it. The dryad cups your head with his other hand, letting the two of you trade off, slurping and drooling on his cock."
		/ case mouthType b of
			Mouth -> c "Your lips spread around the side of the dryad's cock, the son of Hamar-Tabal laving the other side, and you kiss wetly, tongues smearing over the dryad's flesh, catching and sharing spurts of pre that drench your faces."
			_ -> "You shove the dryad's cock to the side of your" / muzzle / "and let your tongue loll out down his shaft, shoving it into the son of Hamar-Tabal's open mouth, drool and pre pouring from your mouth and overflowing his, spilling in stringy lines down to the ground."
		/ Paragraph / "The dryad groans and tugs you both down, pressing your face under the heavy, hot arch of his cock so you can drool across his balls, while the son of Hamar-Tabal suckles on his cockhead. His hips jerk, stomach clenching, and he sprays his load all over the son of Hamar-Tabal's face, making him rear back for a second before he catches the dryad's cockhead in his mouth, cheeks bulging from the sheer volume of his load. You slide up the dryad's shaft and the son of Hamar-Tabal pulls off with a wet slurp, a mess of come spilling from his open mouth for a half-second before"
		/ case mouthType b of
			Mouth -> c "his lips crash against yours"
			_ -> "his lips press against your" / muzzle
		/ "and he sloppily kisses you, pushing a slimy mess of bitter come into your mouth. You make out around the dryad's spurting cock, thick cords of come spurting across both of your faces and drooling from your lips"
		/ case bodyMat_ Head b of
			Solid (FMat _ Fur) -> ", matting your fur into thick clumps"
			Solid (FMat _ Coat) -> ", flattening your coat in disheveled clumps"
			_ -> mempty
		/ "."
	]

fuckClimax :: Path -> Encounter Index Vars FarmerHamarTabal
fuckClimax path b f =
	[ Narration2 $ "You come only a half-second later, hips smacking the son of Hamar-Tabal's ass hard before you" / groan
		/ case locoplan b of
			Basic Centaur -> "into the dryad's mouth"
			_ -> case path of
				DidShare -> "into his mouth"
				_ -> mempty
		/ case firstOf [Knotting] climaxAttrs b of
			Just Knotting -> manyCocks
				? ", one knot and then the other slamming against his clenching asshole and sloppily slurping inside,"
				?: ", turgid knot slamming past his clenching asshole,"
			_ -> mempty
		/ "and erupt"
		/ case firstOf [Flaring, Twisting, Knotting] climaxAttrs b of
			Just Knotting -> "," / Knot^s / "swelling with each pulse"
			Just Flaring -> "flare"^p "twin flares" / "shuddering and then kicking open with enough force to knock the wind from the son of Hamar-Tabal's lungs"
			Just Twisting -> "," / Cock^s / "lurching hard as" / "it twists into a fat spiral"^p "they twist and curve into fat spirals"
			_ -> mempty
		/ ". Your load drenches him, short thrusts rutting back and forth, smearing your load up and down the walls of his ass until"
		/ case firstOf [Knotting] climaxAttrs b of
			Just Knotting -> "you flood his ass entirely," / Cock^s / "just churning up the slimy sea of come flooding inside him as the wet weight of pressure against your still-swelling" / Knot^s / Grows^s
			Nothing -> "slimy squirts erupt out around the base of your" / Cock^s
			_ -> mempty
		/ "."
	] <> climaxClose DidFuck b f
	<> [Narration2 $ case firstOf [Knotting] climaxAttrs b of
			Just Knotting -> "Before the time it takes for your" / Knot^s / "to go down, the tree has already borne fruit, heavy purple things like oversized plums. The dryad raises a hand and one falls straight into his palm, and he splits the skin and feeds heavy, juicy wedges of fruit flesh to you and the son of Hamar-Tabal." / Paragraph
			_ -> mempty
		/ "You pull out"
		/ case firstOf [Knotting] climaxAttrs b of
			Just Knotting ->", eventually,"
			_ -> mempty
		/ "with a thick, sloppy" / em "slurp" / ", leaving his asshole cratered, rim flushed and bulging, hole drooling a stream of come all down his thighs. He wobbles, sitting back on his haunches, just increasing the thick flow of come, face and ass all drenched, dripping with yours and the dryad's splattered loads."
		]
	<> closerDialog b f


climaxClose :: Path -> Encounter Index Vars FarmerHamarTabal
climaxClose path b f =
	[ Narration2 $ "The dryad finishes across the both of your faces,"
		/ case path of
			DidShare -> "come smearing with your drool to coat the son of Hamar-Tabal's face in a sloppy sheen, bubbling up at the corners of his mouth"
			DidFuck -> "drenching the both of you in thick, bitter-tasting come as you unload inside the son of Hamar-Tabal's ass"
		/ ", and in that moment you realize the tree he's been leaning against has burst into flower, pink-purple clusters of flowers weighing down the branches, sending thick drifts of pollen cascading down over you, painting your sweaty, come-soaked bodies with a layer of gritty pollen."
	]

closerDialog :: Encounter Index Vars FarmerHamarTabal
closerDialog b f =
	[ Narration ["The son of Hamar-Tabal pants raggedly, leaning against the dryad's thigh as he recovers."]
	, Dialog2 "/static/img/icon.png" "Son of Hamar-Tabal" $ open "Yeah, this is exactly the thing I was missing,"
	, Narration ["he says up to the dryad, as he scrapes his fingers across his stubbled cheeks, catching grimy blobs of come and shoving them in his mouth."]
	, Dialog2 "/static/img/icon.png" "Son of Hamar-Tabal" $ open "You weren't half bad either,"
	, Narration ["he says to you."]
	, Dialog2 "/static/img/icon.png" "Dryad" $ open "Glad I could help,"
	, Narration
		[ "the dryad says, and smacks the son of Hamar-Tabal across the face with his cock."
		, "Yeah, you get the feeling these two are gonna be fucking a lot."
		]
	]
