{-# LANGUAGE MultiParamTypeClasses #-}

module Sites.Hell.Events.Hubur.FeralHound
	( FeralHound(..)
	, deck
	, scene

	, name
	, bits
	, variants
	) where

import Prelude hiding ((/))

import Sites.Hell.Events.Construction ((/))
import Sites.Hell.Events.SceneUtils

import Sites.Hell.Events.Hubur

name :: String
name = "feralhound"

data FeralHound = FeralHound
	deriving (Eq, Ord, Show, Read)

instance Random FeralHound where
	randomR _ = random
	random = getRand $ pure FeralHound

instance ToJSON FeralHound where
	toJSON FeralHound = object
		[ pack "name" .= name
		, pack "data" .= ([] :: [Int])
		]

bits :: Value -> Value
bits _ = object
	[ pack "name" .= name
	, pack "data" .= ([] :: [Int])
	]

variants :: [(MapZone, ())]
variants =
	[ (Badland, ())
	]
instance FromJSON FeralHound where
	parseJSON (Object vs) = case (parse (.: pack "name") vs, parse (.: pack "data") vs :: Result [Int]) of
			(Success pName, Success [])
				| pName == name ->
					case pure FeralHound of
							Nothing -> mzero
							Just a -> return a
			_ -> mzero
	parseJSON _ = mzero

deck :: RandomGen g => Rand g (Enumerable FeralHound)
deck = return $ pure $ FeralHound

data Index = Dummy
	deriving (Eq, Ord, Show, Read)

instance Scene Index () FeralHound where
	link f = case f of
		_ -> error "dummy index"

	display = fmap toLower . show
	vardisplay = fmap toLower . show

scene :: Encounter Index () FeralHound
scene _ c =
	[ description c
	-- , error "bad scene"
	]

description :: FeralHound -> Event FeralHound Index ()
description w = Narration2 $
	"It's a feral hellhound."
	/ Paragraph / "There's a shape there, but your eye slides off it. It's an opening into a realm with too many angles, one that unfolds into vista after vista, spreading out to infinity. There's a border to the opening, but it's amorphous, beating like a heart, ringing like a skull. It stretches out everywhere, in all directions. There's a blossoming of parts: eyes and teeth and tongues, spilling out into fleshy trees, forming heaving caves like the inside of of a collapsing lung, burrowing in the ground like roots, and yet as far as it spans it never seems to reach anywhere."
	/ Paragraph / "It's difficult to parse what you're looking at."
