{-# LANGUAGE MultiParamTypeClasses #-}

module Sites.Hell.Events.Hubur.HellGate
	( HellGate
	, deck
	, scene

	, name
	, bits
	, variants
	) where

import Utility.Rand

import Sites.Hell.Events.SceneUtils

import Sites.Hell.Events.Hubur
import Sites.Hell.Events.GameEvent (br, quote)

name :: String
name = "hellgate"

data HellGate = HellGate
	deriving (Eq, Ord, Show, Read)

instance Random HellGate where
	randomR _ = random
	random = getRand $ return HellGate

instance ToJSON HellGate where
	toJSON HellGate = object
		[ pack "name" .= name
		, pack "data" .= ([] :: [Int])
		]

bits :: Value -> Value
bits _ = object
	[ pack "name" .= name
	, pack "data" .= ([] :: [Int])
	]

variants :: [(MapZone, ())]
variants =
	[ (Badland, ())
	]

instance FromJSON HellGate where
	parseJSON (Object vs) = case (parse (.: pack "name") vs, parse (.: pack "data") vs :: Result [Int]) of
			(Success pName, Success [])
				| pName == name ->
				case pure HellGate of
						Nothing -> mzero
						Just h -> return h
			_ -> mzero
	parseJSON _ = mzero

deck :: RandomGen g => Rand g (Enumerable HellGate)
deck = return . pure $ HellGate

data Index
	= OpeningChoices
	| Gate | Cerberus | CerberusFuck | CerberusHell | Archway
	| Encampment
	| Leave
	deriving (Eq, Ord, Enum, Bounded, Show)

data Vars = MetCerberus
	deriving (Eq, Ord, Enum, Bounded, Show)

instance Scene Index Vars HellGate where
	link i = case i of
		OpeningChoices -> openingChoices
		Gate -> gate
		Cerberus -> cerberus
		CerberusFuck -> fuckPrompt
		CerberusHell -> cerberusHell
		Archway -> archway
		Encampment -> encampment
		Leave -> leave
	display = fmap toLower . show
	vardisplay = fmap toLower . show

scene :: Encounter Index Vars HellGate
scene = opening

-- opening
opening :: Encounter Index Vars HellGate
opening _ _ =
	[ Narration2 $ open "There's a vast gate on the flood plain, with no walls in sight. It's huge, and made of ancient black granite, with runes of fire burning across the top, and the sky above blackens and pulses in time with the glow of the runes. There's an encampment around it, tents and decks and kennel-houses, all close around the gate but never quite touching. The place is busy with motion. There are lots of hellhounds about."
	, Jump OpeningChoices
	]

openingChoices :: Encounter Index Vars HellGate
openingChoices _ _ =
	[ Choices
		[ ("check out the gate", Gate)
		, ("check out the encampment", Encampment)
		, ("leave", Leave)
		]
	]

leave :: Encounter Index Vars HellGate
leave _ _ =
	[ Narration ["You turn around and move away until the gate is just a messy blot on the horizon."]
	]

--- check out the gate
gate :: Encounter Index Vars HellGate
gate _ _ =
	[ SetFlag (Global VisitedHellGate) 1
	, Narration2 $ open "It's the gate to hell: vast enough to contain an army; tall enough to admit a colossus. Though the countryside sprawls out on either side, through it there is only darkness. It's engraved deeply with ornate reliefs and decorations, and its top -- even with the burning runes -- is lost in the shadow of the pulsing night above."
		// Paragraph // "Writ into the arch of the gate are letters of burning fire:"
		// (quote $ open "Through me is the city of woe"
			// br // "Through me is eternal woe"
			// br // "Through me, go with all the lost"
			// Paragraph // "JUSTICE moved my sublime creator:"
			// br // "The Divine POWER created me"
			// br // "Suffused with highest WISDOM and primal LOVE"
			// Paragraph // "Before me came nothing, save the eternal,"
			// br // "And eternal I last"
			// Paragraph // "ABANDON ALL HOPE, YE WHO ENTER HERE")
		// Paragraph // "The gate is scratched all over, the rock at hand-height eroded inwards from the passage of billions of hands over the eons. Along every curve, nook, and cranny, thicker on the flat planes, there's graffiti: faint white scores through the rock, drawn over and obscuring older marks. All the language is illegible, but there are some caricatures."
		// Paragraph // "And then, emerging from the darkness of the arch: a vast shape, incomprehensibly stretched, like the darkness reached out and took shape. Burning red eyes open, and then more and more, overlapping over each other, eyes opening into maws of sharp teeth that themselves spread and bulge, vomiting up pupils. A flurry of wings and eyes and teeth and fire."
	, Jump Cerberus
	]

cerberus :: Encounter Index Vars HellGate
cerberus _ _ =
	[ Narration2 $ open "It's Cerberus."
		// Paragraph // "He looks down at you: a million miles tall, stretched out to infinity, no dividing line between his body and the world, eyes blossoming like flowers all around you."
	, Choices2
		[ ("hey you wanna fuck", CerberusFuck, Nothing)
		, ("so whats w/ this whole hell thing anyway", CerberusHell, Nothing)
		, ("walk into the archway", Archway, Nothing)
		, ("run away", OpeningChoices, Just $ C (Local MetCerberus) NotSet)
		, ("go back", OpeningChoices, Just $ C (Local MetCerberus) IsSet)
		]
	]

fuckPrompt :: Encounter Index Vars HellGate
fuckPrompt _ _ =
	[ Dialog2 "/static/img/icon.png" "Cerberus" $ open "That's cute."
	, Narration2 $ nBranch (C (Local MetCerberus) IsSet)
		("Cerberus' voice rumbles.")
		("Cerberus' voice is like an earthquake.")
	, Dialog2 "/static/img/icon.png" "Cerberus" $ open "But I don't fuck prey."
	, SetFlag (Local MetCerberus) 1
	, Choices [("well okay", Cerberus)]
	]

archway :: Encounter Index Vars HellGate
archway _ _ =
	[ Narration2 $ open "You look into the darkness."
		// Paragraph // "It turns out you really don't want to do that, actually."
	, Choices [("<==", Cerberus)]
	]

cerberusHell :: Encounter Index Vars HellGate
cerberusHell _ _ =
	[ Dialog2 "/static/img/icon.png" "Cerberus" $ open "And what does it matter to you, little soul?"
	, Narration2 $ nBranch (C (Local MetCerberus) IsSet)
		("Cerberus' voice rumbles.")
		("Cerberus' voice is like an earthquake.")
	, Dialog2 "/static/img/icon.png" "Cerberus" $ open "Hell is hell. It was built to house your kind. To be seen and heard and smelt and touched. Without comprehension it is just noise and chaos and vast shadows, in seeing parts of structure but never understanding the whole. There's no point to eternal torment if a soul can't understand it's being tortured."
		// Paragraph // "That is the payment for the sin of your existence. The flow is endless: souls upon souls, down through the ages, streaming further and further into Hell's depths.  And as for the rest of us-- we exist for your service. Guide and judge and jailer. Your every need fulfilled, as fits the will of the Authority."
	, Narration2 $ open "One enormous eye grows -- closer or simply larger, it's impossible to tell -- until it fills the entire field of your vision."
	, Dialog2 "/static/img/icon.png" "Cerberus" $ open "But perhaps less so in these times."
	, FauxChoice "oh??"
	, Dialog2 "/static/img/icon.png" "Cerberus" $ open "The Authority's interest in His creation has waned, and it was never more than a passing fancy. A masturbatory fixation on His grandiose works, and a furious anger when anything steps out of line of His divine plan. His eye has turned away."
		// Paragraph // "And as for me -- what do I care of this world as He created it? It is emptiness and ether, made to feign substance. It is all a thin veneer: air, land, water, fire, all there to obscure the truth of this world: yawning chaos, nothingness intermingling with reality."
		// Paragraph // "This place is the final exhale of divine breath as it touches the void; the realm the furthest from the divine sight. The chains His order imposes on the world grow slack: let us cast them off, and see what can be made in the void beyond the edge of the world."
	, SetFlag (Local MetCerberus) 1
	, Choices
		[ ("well okay", Cerberus)
		]
	]

encampment :: Encounter Index Vars HellGate
encampment _ _ =
	[ Narration
		[ "It's a mess. There are hellhounds galore, all over the place. Most of them look as you've started to expect hellhounds to look -- darkness and fire -- but there are others that are a lot more impressionistic. Glimmers in the air and oceans of blood and darkness and sharp angles exploding out to infinity and fire. If you look at them too long it starts to hurt, and you get the sensation that reality is curling up at the edges, burning away from the hellhounds' fire."
		]
	, Choices
		[ ("that's about it tho, go back", OpeningChoices)
		]
	]
{-

> Well.
> It is all in the service of you, you know that, right? Souls pour endlessly down into the gullet of hell. Souls have limits. The universe the Authority built was not made for souls: the worlds of mortals are safe, cut off from the chaotic abyss that sings through the rest of creation. To see it in truth would not be... pleasant, or comprehensible, for you.

> When the Authority built hell, he built it to fit souls. There's no point in eternal torment if a soul can't understand it's being tortured. Just noise and chaos and vast shadows, in seeing parts of structure but never understanding the whole.

> But it chafes. Crunching ourselves down into bodies and symbols and meaning. Warping ourselves, simplifying and simplifying and ultimately cutting off any errant strangeness. All to obey some ancient dictate: making ourselves fit in the eyes of the Authority. Fit to be comprehensible torturers for wayward souls.

> As of late, some of us have lost interest in remaining fit. Let souls try to grasp the shape of our structures and be cast into chaos. Let the knives of our thoughts cut furrows through the world.

> But there are some who cling to this old order. Because they have forgotten themselves, or drowned themselves in flesh, or simply because this order is ruled and they think themselves the rulers.
-}
