{-# LANGUAGE ViewPatterns #-}
{-# LANGUAGE MultiParamTypeClasses #-}

module Sites.Hell.Events.Hubur.LesserHound
	( LesserHound(..)
	, deck
	, description
	, scene

	, name
	, bits
	, variants
	) where

import Prelude hiding ((/), (^))

import Data.Maybe (catMaybes)

import Server.JSON
import Sites.Hell.Events.SceneUtils hiding ((/))
import Sites.Hell.Events.Construction ((/), (^))
import Sites.Hell.Events.GameEvent (materialWord)

import Sites.Hell.Events.Hubur

import Description.TfNode (Animal(..))
import Description.Build

name :: String
name = "lesser hellhound"

-- no variants :V
-- maybe: blue / green / red / white fire + a pure-shadow variant. or bones
data LesserHound = LesserHound Body
	deriving (Eq, Show)

data Body = ShadowFire | BlackWhite | Skull
	deriving (Eq, Ord, Enum, Bounded, Show, Read)

instance Random LesserHound where
	randomR _ = random
	random = getRand $ LesserHound
		<$> genRandom

instance ToJSON LesserHound where
	toJSON (LesserHound b) = object
		[ pack "name" .= name
		, pack "data" .= [fromEnum b]
		]

bits :: Value -> Value
bits _ = object
	[ pack "name" .= name
	, pack "data" .= [Null]
	]

variants :: [(MapZone, ())]
variants =
	[ (Badland, ())
	]

instance FromJSON LesserHound where
	parseJSON (Object vs) = case (parse (.: pack "name") vs, parse (.: pack "data") vs :: Result [Int]) of
			(Success pName, Success [b])
				| pName == name ->
				case LesserHound
					<$> safeToEnum b of
						Nothing -> mzero
						Just h -> return h
			_ -> mzero
	parseJSON _ = mzero

deck :: RandomGen g => Rand g (Enumerable LesserHound)
deck = do
	body <- liftRand $ shuffle [ShadowFire, BlackWhite, Skull]
	return $ LesserHound
		<$> from body

data FlipCrawl = Flip | Crawl
	deriving (Eq, Ord, Enum, Bounded, Show)

data SheathState = Sheathed | Popped
	deriving (Eq, Ord, Enum, Bounded, Show)

data FromAnal = Directly | AfterFisting | AfterMilking
	deriving (Eq, Ord, Enum, Bounded, Show, Read)

data Index
	= IntroChoices | Talk | LeavePlatonic
	| Leave
	| FuckOpening
	| FuckOrb
--	| TopCoax | FuckTop FromAnal
	| Sheathplay | JOClimax
	| Assplay | Fisting | Milking | AssplayCloser
	| FuckAnal SheathState | AnalKnot | AnalComeshot
	| OralBranch SheathState
	| CrawlOral | FlipOral
	| OralKnot FlipCrawl | OralComeshot FlipCrawl
	| LeaveAfter
	deriving (Eq, Ord, Show)

instance Scene Index () LesserHound where
	link x = case x of
		IntroChoices -> introChoices
		Talk -> talk
		LeavePlatonic -> leavePlatonic
		Leave -> leave
		FuckOpening -> fuckOpening
--		TopCoax -> topCoax
--		FuckTop from -> fuckTop from
		FuckOrb -> fuckOrb
		Sheathplay -> sheathplay
		JOClimax -> joclimax
		Assplay -> assplay
		Fisting -> fisting
		Milking -> milking
		AssplayCloser -> assplayCloser
		OralBranch s -> oralBranch s
		CrawlOral -> crawlOral
		FlipOral -> flipOral
		FuckAnal s -> fuckAnal s
		AnalKnot -> analKnot
		AnalComeshot -> analComeshot
		OralKnot fc -> oralKnot fc
		OralComeshot fc -> oralComeshot fc
		LeaveAfter -> closing True
	display = fmap toLower . show
	vardisplay = fmap toLower . show

scene :: MarkedScene Index () LesserHound
scene b h =
	[ SetFlag (Global MetLesserHellhound) 1
	, description h
	] <> introChoices b h

description :: LesserHound -> Event LesserHound Index ()
description (LesserHound b) = Narration2 $
	"A lesser hellhound blocks your way."
	/ Paragraph
	/ case b of
		ShadowFire -> "It's a mess of black shadow and sputtering red fire,"
		BlackWhite -> "It's a mass of grey-white flames,"
		Skull -> "It's a bulky black mess of substance,"
	/ "drawn in the rough shape of something somewhere between a jackal and a hyena."
	/ case b of
		ShadowFire -> "Its eyes are embers, billowing smoke tracing back over its head as it paces around you, outlining its bulky body. Flame bursts from its paws; it leaves behind guttering pawprints. A line of fire burns down from its haunches, outlining its tail like a whip of flame, ending in a crackling fire, like a torch in high wind. Flame spills from its lips like drool, burning froth and foam outlining its muzzle. Other than that, its body is flat black, like a shadow cut out of the world."
		BlackWhite -> "The flames burst back from its back, as if it's facing a strong wind. Its eyes are black spots, a hollow void within the flames of its head, or made from the gaps of the wavering flames that make up its form. Whipping flames spill out across its shoulders and haunches, and down the length of its tail, giving it a spiky, elongated look."
		Skull -> "The black material peels back across its head and down its back, revealing pure white bone. The individual segments of its spine erupt out from its body, half-surfaced in inky goo, all the way down to its tail, which is a long chainlike cord of bone, webbed with tendons of black ichor. The hollows of its skull burn with an eerie green-grey light. Black goo spills through its muzzle, half-formed tendrils like tongues lapping out at the air before they lose their form and dripple to the ground in oily strands."

introChoices :: MarkedScene Index () LesserHound
introChoices b h =
	[ Choices
		[ ("hey you wanna fuck", FuckOpening)
		, ("hey lets talk", Talk)
		, ("bye", LeavePlatonic)
		]
	]

talk :: MarkedScene Index () LesserHound
talk b h =
	[ Dialog "/static/img/icon.png" "Lesser Hellhound"
		[ "Nah let's not"
		, "I have better ideas for what you could put your mouth to doing"
		]
	, Choices
		[ ("<==", IntroChoices)
		]
	]

{-
> son of hamar-tabal
children of the ferryman. their sire has a divine purpose; they do not.

> flame demons
pale flames. will be ash before long. who has their creation helped?

> gargoyles
dumb automata. not a spark of life within their bodies.

> ganzer workers
deserters. traitors to the Crown.

> clay men
mud that thinks itself alive.

> dryads
trees and nothing else.


> lesser hellhounds
yes, us! inheritors of the might of the world. it is us who will rule, and bring all of hell within our dominion. when the Authority turned aside, it is the annunaki who ruled, and that authority flows down. branches and leaves. we are the final link in the long chain of Rule, and our rule is sweet.



> sobek sheuts
blind and foolish cultists. seeking souls that are not theirs.


> cerberus / feral hellhounds
degenerates who betray the world order. cerberus itself! how the mighty have fallen.

-}

leavePlatonic :: MarkedScene Index () LesserHound
leavePlatonic _ _ =
	[ Dialog "/static/img/icon.png" "Lesser Hellhound" ["Well bye."]
	]

leave :: MarkedScene Index () LesserHound
leave _ _ =
	[ Narration
		[ "The hound scoffs, his cock still steadily drooling on the ground."
		]
	, Dialog "/static/img/icon.png" "Lesser Hellhound" ["Well bye."]
	]

fuckOpening :: MarkedScene Index () LesserHound
fuckOpening b h = case locoplan b of
	Spec FloatingDick ->
		[ Narration
			[ "The hellhound scoffs."
			]
		, Dialog "/static/img/icon.png" "Lesser Hellhound"
			[ "I'm not looking to get fucked by only " ++ (case cockCount b of
				1 -> "a floating dick"
				2 -> "a pair of floating dicks"
				_ -> "a mess of floating dicks")
			, "Bring some hips and fuck me properly"
			, "If you do at all"
			]
		, Narration
			[ "You leave."
			]
		]
	_ ->
		[ Narration
			[ "The hellhound howls and slavers."
			]
		]
		<> case planLocomotion $ bodyplan b of
			Spec Orb -> formlessSex
			Basic Formless -> formlessSex
			Spec FloatingHead ->
				[ Dialog "/static/img/icon.png" "Lesser Hellhound"
					[ "You don't got much of a body"
					, "But your mouth looks so sweet"
					, "Open wide"
					, "Let me sheath myself in you"
					]
				, Narration
					[ "His cock, already hard, is a thing like a red-hot poker: black pre oozing from the tip and boiling and bubbling away into a thick crust, not quenching the infernal heat in the least."
					]
				, Choices
					[ ("suck him off", FuckOrb)
					, ("actually bye", Leave)
					]
				]
			_ ->
				[ Dialog "/static/img/icon.png" "Lesser Hellhound"
					[ "What luck what luck"
					, "Such a sweet soul to happen across"
					, "Eaten alive by the rut and here we have a fresh little soul to sheath myself in"
					, "My weapon is eager to sink into the depths of your soul"
					]
				, Narration
					[ "His cock, already hard, is a thing like a red-hot poker: black pre oozing from the tip and boiling and bubbling away into a thick crust, not quenching the infernal heat in the least."
					]
				, Choices $ catMaybes
					[ guardC (hasArms b) ("stroke his dick", Sheathplay)
					, guardC (hasArms b) ("actually lemme play w/ yr ass", Assplay)
					--, guardC (hasDick b) ("how do you feel about getting fucked, though", TopCoax)
					, pure ("suck him off", OralBranch Sheathed)
					, pure ("let him fuck you", FuckAnal Sheathed)
					, pure ("actually bye", Leave)
					]
				]
	where
		formlessSex =
			[ Dialog "/static/img/icon.png" "Lesser Hellhound"
				[ "A chunk of soul with not even an illusory body to call your own"
				, "But even this I can sink my cock in with a little work"
				, "Feel your membrane burst as I stab inside you"
				, "Feel your protoplasm shuddering around my cock"
				]
			, Narration
				[ "His cock, already hard, is a thing like a red-hot poker, black pre oozing from the tip and boiling and bubbling away into a thick crust, not quenching the infernal heat in the least."
				]
			, Choices
				[ ("uh sure let's try this out", FuckOrb)
				, ("actually bye", Leave)
				]
			]

topCoax :: MarkedScene Index () LesserHound
topCoax b h =
	[
{- TODO:
his lips curl in a snarl.

> and you'd be the one to do it?
> you must think highly of yourself

still, his cock throbs unremitting between his haunches, letting out a constant drizzle of steaming pre. his gaze settles on {your cock/your sheath/your vent} and stays there for a long moment. he's drooling. he's always drooling a little, but now he's drooling more.

> sheath yrself in my depths
> my hole will eat you alive

so, he's probably fine with it.

-> fuck him / mount him

-}
	] <> case locoplan b of
		_ -> fuckTop Directly b h

fuckTop :: FromAnal -> MarkedScene Index () LesserHound
fuckTop from b h =
	[
{-
TODO: then you fuck him
-}
	]

orbHeadSplit :: Show b => BodyGlob -> b -> a -> a -> a
orbHeadSplit b err orb _head = case locoplan b of
	Spec FloatingHead -> _head
	Spec Orb -> orb
	Basic Formless -> orb
	f -> bodyFail err f

isHead :: BodyGlob -> Bool
isHead b = case locoplan b of
	Spec FloatingHead -> True
	_ -> False

fuckOrb :: MarkedScene Index () LesserHound
fuckOrb b h =
	[ Dialog "/static/img/icon.png" "Lesser Hellhound" ["Well how about you float on over to my dick"]
	, Narration2 $
		"He rears up on his hind legs for a moment, paws tearing at the air, his ruddy, red-hot cock on display, drooling a steady stream of black pre."
		/ Paragraph / "You float on over to his dick, swooping down under his haunches. He taps at you with one of his forelegs, claws catching your"
		/ isHead ? "cheek" ?: "side"
		/ "and pinning you to the ground. He snarls, cock twitching and slapping up across his belly, spurting pre across you. He twists, cock slapping across your"
		/ isHead ? "cheeks" ?: "surface"
		/ "before he"
		/ orbHeadSplit b FuckOrb
				(c "bears down, your entire body distending from the force, rubbery membrane deforming. He pulls back, painting your surface with a few spurts of glossy black pre, hot fluid slowly osmosing through your membrane, and then shoves forward again, bearing down until his cock tears through inside you with a sharp beat of something like pain.")
				("shoves into your" / muzzle / ".")
		/ Paragraph / "He fucks your"
		/ isHead ? (c muzzle) ?: "oozing hole"
		/ ", both paws scrabbling across your"
		/ orbHeadSplit b FuckOrb "body" "head"
		/ "as he hunches forward, curling up in a tight twist as he hammers forward, cock emerging for a moment slick with"
		/ orbHeadSplit b FuckOrb "protoplasm" "spit"
		/ "before he plunges it in all the way to the base, his engorged knot slapping against your"
		/ orbHeadSplit b FuckOrb "torn hole" "lips"
		/ ". He growls, cock pulsing, a drool of pre squirting inside you and bubbling right back out, paws shifting on top of you, before he starts thrusting."
		/ Paragraph / "He growls and barks, using you as a sex toy; an open hole. His cock rams against the"
		/ orbHeadSplit b FuckOrb "inner surface of your orb" "bottom of your throat"
		/ ", distending you from the inside. His squirts of pre fill you up, black ooze slopping down"
		/orbHeadSplit b FuckOrb "over your ragged torn-open hole" "your lips"
		/ ", smearing across his knot and sizzling."
		/ Paragraph / "He howls, cock ramming harder, knot slapping against"
		/ orbHeadSplit b FuckOrb "your hole" "your lips"
		/ "until with a lurching" / em "pop" / "it pops inside, lodging behind"
		/ orbHeadSplit b FuckOrb (c "your swollen membrane") ("your" / teeth)
		/ "and digging in, swelling more as he starts to come. Bursts of come flood into you, hardly anywhere for them to go, "
		/ orbHeadSplit b FuckOrb "your orb growing cloudy and dense" "your cheeks bulging"
		/ "as he squirts pulse after pulse of tarry black slime."
		/ Paragraph / "He twists, trying to twist backwards so he can tug back on his knot, but with so little body in your body you're just drawn up to smack across his underbelly, wedged between his flanks. He yaps and growls, hips shaking, but regardless of his motions you're stuck there in place, swaying and slowly bloating with each pulse of come. His motions slowly die down to just pants, huffing and groaning as he unloads inside you, his thick black come giving way to thinner sludge, not pouring so much as being pumped into your swollen-to-bursting body, each shudder against more and more resistance;"
		/ orbHeadSplit b FuckOrb
			"your orb swollen and lopsided, folds of it bulging out in lumpy clusters."
			"your cheeks bloated, the base of your throat bulging out in a dewlap-like sack, your sinuses flooded, a dribble spilling continually out of your nose."
		/ Paragraph / "The instant his knot shows the first sign of deflating you all but pop off him, the pressure too much for his knot to keep you stuck in place,"
		/ orbHeadSplit b FuckOrb
			"your stabbed-open hole erupting in a spray of boiling come"
			"vomiting out a glossy waterfall of boiling come"
		/ "all across the hound's underbelly and flanks, drenching his cock in his own issue. You pull the rest of the way off with a soggy" / em "slurp" / ","
		/ orbHeadSplit b FuckOrb "hole" "mouth"
		/ "oozing down your"
		/ orbHeadSplit b FuckOrb "sides" "chin"
		/ "as the majority of his load spills out of your too-small body."
		/ Paragraph / "The hound is still coming, weakly, cock twitching to smack up across his belly, shooting out little streamers of come through the slathering mess."
	, Choices
		[ ("well that was weird/good okay bye", LeaveAfter)
		]
	]

sheathplay :: MarkedScene Index () LesserHound
sheathplay b h =
	[ Narration2 $
		"You lean in against his side -- his shadowy flesh eventually admits a solid base -- and reach for his dick, jutting out between his haunches as a fuming, boiling spar. It's hot under your fingers, but otherwise like flesh: spongy and heavy, with some give. The hellhound yaps, thrusting into your hands; his flesh catches and skids over your skin, the heat of friction nothing compared to its own heat. His flanks quiver, muscles flexing, and he humps forward, shoving his cock through your grip: down to grind your fingers against the shadow-flesh sheath, fingertips curling inside, and then back until the sharp tip slops through your fingers with a thick gush of his boiling black-tar pre, coating your hand in thick ooze. His pre burbles out in gushes, drooling down to the ground in long, taffy-like strings; the stuff that spills down his shaft boils down into a gummy tar before you can do much with it: you smear the sludgy mess across his red-hot length, leaving behind black smears that flake apart into burnt crusts on your next stroke."
		/ Paragraph / "When he grinds down you can feel his knot through his sheath: a fat, rubbery bulge of flesh, bouncing against your curled fingers with each thrust. Muscles twitch along the underside of his shaft, at the very base; you can feel each pulse of pre shoot up, firing through the length of his cock before it splatters all over your hand. On his next thrust you grip fractionally tighter, pressing your fingers against his sheath, letting his softened knot pop into your palm with a wet" / em "slap" / ". The hound yaps, fiery drool spilling from his open muzzle."
		/ Paragraph / "You skim your fingers around his knot, the flesh shuddering and trembling, blood -- or whatever the hound has instead of blood -- pulsing just beneath the surface. You wrap both hands around it and squeeze, tugging at his unsheathed length; each motion makes his entire cock throb, slinging ropes of boiling pre across your forearm, each one with a matching set of wild animal noises, yaps and barks and whines."
		/ Paragraph / "You dip down, sliding a finger into the confines of his sheath, wrinkled up now behind the bulk of his cock, only covering the very root of his shaft. He whines as you stroke along the underside, the main bulk of his cock untouched -- shuddering pressed taut against his underbelly -- while you play with his knot and sheath, milking him, sliding two and then three fingers up inside his sheath."
	, Choices
		[ ("make him come", JOClimax)
		, ("suck him off", OralBranch Popped)
		, ("let him fuck you", FuckAnal Popped)
		-- ("fuck his sheath", Sheathfuck)
		]
	]

joclimax :: MarkedScene Index () LesserHound
joclimax b h =
	[ Narration2 $ "His cock shudders, pulsing in your grip. His knot burns hotter and hotter, dull red glow becoming a brighter yellow, knot nearly white-hot. Its muted glow shows through your"
		/ case bodyMat_ Arms b of
			Solid (FMat _ x) | x `elem` [Skin, Fur, Scales, Protoplasm] -> matword x
			Solid (FMat _ Coat) -> "skin"
			_ -> "flesh"
		/ ", each throb of his cock swelling it larger and larger, as its rubbery flesh fills out into a rock-hard bulge. Pre spurts out across his underbelly in stronger squirts, smacking his forelegs, drenching his chest, making a streaky mess of bubbling tar on the ground before him."
		/ Paragraph / "He whines, head thrashing back and forth, yapping, as he thrusts into your hands, knot solidly smacking against your palms again and again until you decide to capture it, locking it between your hands and squeezing down. The hound howls, baying up into the darkness as his hind legs tremble, jerking fractionally forward as if to bury his knot deeper in its illusory tie. His cock shudders again, slapping up across his underbelly, and erupts with a steaming gout of come, boiling black tar spraying in thick, gummy waves out in front of him, hot droplets splattering all across your side, thick cords catching across your arms."
		/ Paragraph / "You milk his knot, squeezing rhythmically, and he ruts into the motion, powerful muscles in his haunches bearing down against you. He yammers and yaps as he comes, cock spewing out rope after rope of boiling tar onto the ground, in a bubbling, fuming puddle. Excess drools down his shaft, baking into thick sludge by the time it hits your hands, spilling between your fingers and making each of the hound's thrusts loudly squelch and slurp as he thrusts into the slimy, sludge-drenched confines of your hands."
		/ Paragraph / "Eventually his ejaculation slows and then stops, just small spurts of come sputtering into the churning pool below, and he sags down, tension leaving his body, half-hard knot pulling from your loosened grip with a final" / em "slurp" / ". He pants, drooling fire, and staggers to the side, leaving you"
		/ case locoplan b of
			Basic x | x `elem` [Centaur, Biped] -> "kneeling"
			Basic Naga -> "coiled"
			f -> bodyFail "joClimax" f
		/ "there, arm drenched to the shoulder in come, dark splotches all across your side."
	] <> closing False b h

{- TODO:
intro dialog etc; this branches from `guardC (hasArms b) ("actually lemme play w/ yr ass", Assplay)`

also todo: splices etc



-}
assplay :: MarkedScene Index () LesserHound
assplay b h =
	[ Narration2 $
		"You pet down across his haunches, over the base of his flaming tail, and then beneath. His black flesh isn't precisely fur, but it's not bare muscle either. His asshole is the same inky, voidlike black as the rest of him, only identifiable by touch: there's a rumpled, muscular divot beneath his tail, shuddering when you bear down. Plus, his cock thwacks against his stomach when you find it, gushing out a spurt of fuming, black pre as your fingertips graze back and forth over his hole."
		/ Paragraph / "You ease a finger in -- his flesh isn't slick, but it doesn't have the same catch as real flesh either; it's not hard to push in. He's hot inside, and pulsing, inky shadow-flesh churning and twisting against your intrusion. He huffs, head drooping, panting out plumes of fire as you work back and forth, sliding two fingers past opening of his ass, spreading it wider, thumb sliding back and forth over the muscle. It trembles under your fingertips."
		/ Paragraph / "You push in to the knuckle, scissoring your fingers apart, pulling back to spread him wider. There's the sense of being buried under fur, or in layers of shadow; even as his hole stretches and splays open under your fingers you're still just pushing into an inky black void, fingers occluded randomly by tufts of shadow-stuff."
		/ Paragraph / "You twist, fingers grinding against the inner walls of his ass, knuckles butting against harder substance, and the hound snarls and pants, rutting back against you. You fingerfuck him, three fingers shoving into his spread hole, bashing against his inner walls. His cock shudders, audibly slapping against his stomach, gushes of black pre splattering the ground in droplets of bubbling tar."
	, Choices
		[ ("fit a whole fist in there", Fisting)
		, ("play w/ his knot while you milk him", Milking)
		]
	]

fisting :: MarkedScene Index () LesserHound
fisting b h =
	[ Narration2 $ "At three fingers it's not hard to fit the rest of your hand in. You tuck your pinky and thumb up against your palm, pushing in. His asshole butts against your knuckles, rim shuddering, and you press down, constant pressure slowly wearing him down, until suddenly his hole spasms and spreads and you punch your entire fist into his ass. He howls, ragged, and his cock shudders again, knot bloated, half-wedged inside his sheath. He hunches forward, fucking the air, trying to releave the tension, but all that does his fuck himself on your fist."
		/ Paragraph / "You work your fist back and forth, knuckles rolling against the smooth flesh of his inner walls: smoky and dense, slick without being wet. You can feel his knot pulse; there's a cord of muscle under your curled fingers that pulls and twitches, and each time his cock jerks, slapping up against his underbelly."
		/ Paragraph / "You pull out, with his asshole clenching around your hand the whole way -- he whines, rutting back -- and then push back in, pumping your fist over the ring of his asshole until he learns to take it, hole tight around the crest of your knuckles, flowering open. You shove in wrist deep, back and forth, punch-fucking him while he yowls and yaps. He slavers fiery drool as he fucks himself on your hand, harder and harder. His knot finally surfaces, slurping from his sheath, throbbing between his haunches. His entire cock is drenched in burnt black sludge, with crusted flakes peeling off from the force of his twitches, making a gritty, tarry slurry beneath him."
		/ Paragraph / "His asshole swallows up your hand, letting you ram your knuckles against his bloated, rubbery prostate again and again, each wet" / em "slap" / "making his cock erupt in a cord of sludge. He bays, body heaving with the force of his ragged breath as he fucks himself back onto your fist. The ring of his asshole grows shiny, defined in puffy slabs, all dense rubbery substance bruised and warped from the force of your fist. He keens, whining, as you run"
		/ armCountMatching 4 ? "another hand" ?: "your other hand"
		/ "over his hole, fingers toying with the rim and tugging it yet wider as you keep probing inside him, shoving deeper until his hole bulges out around your forearm, and then"
		/ case bodyMat_ Arms b of
			Solid (FMat _ Skin) -> ", clinging tight to your skin,"
			Solid (FMat _ Scales) -> ", catching on your rippling scales,"
			Solid (FMat _ Fur) -> ", pulling your fur back in spiky tufts,"
			Solid (FMat _ Coat) -> ", pulling your coat back,"
			Solid (FMat _ Armor) -> ", sawing against your armored plates,"
			_ -> mempty
		/ "warps backwards in a protruding bulge as you drag your hand back out. You pull out with a pop -- and another few heavy gushes of slime from the hellhound's trembling cock -- and his hole bows out into a thick, sloppy gape, half-prolapsed from his body. You shove your fist back inside, a wet" / em "slurp" / "as it sinks back in, and the hound whines again, the inner flesh of his body convulsing wildly around you. He howls and bays, asshole milking your fist as you work it back and forth through his tight confines."
		/ Paragraph / "His cock erupts -- again -- spurting fuming black tar all across the ground beneath him: one hard spurt that makes his entire body shudder, and then another, and another, until his cock is pouring thin, sloppy come like it thinks he's knotted: shaft rubbery and heavy, knot fully inflated, and all the while he's slavering and panting, fucking himself back on your fist, asshole bruised into a spongy, puffy mess as you keep punch-fucking him through his orgasm. His cock never quite stops coming, twitching to slap against his underbelly, spurting thick black cords of sludge in between a continual pissing out of thinner slime. You keep fisting him, punching into his guts; you can feel each pulse of his cock spasm through his body, internal muscles rhythmically firing over and over as he pours out a sloppy load all over the ground."
		/ Paragraph / "He slumps to the ground, slumping into the mess of his own load, asshole bulging before he soggily pulls off your fist, leaving his asshole a gaping, sloppy crater."
	] <> assplaySwitch AfterFisting b h

milking :: MarkedScene Index () LesserHound
milking b h =
	[ Narration2 $
		"His balls are drawn up tight between his haunches, submerged in a coil of shadow-substance, heavy and hard under"
		/ armCountMatching 4 ? "your" ?: "your other"
		/ "hand. His cock, meanwhile, is clearly visible: a spar of red-hot flesh, surface coated with a flaking, burning crust of hardened black-tar pre. It hangs down heavily between his haunches, twitching erratically and drooling out streamers of oily black pre as you work your fingers around in his ass."
		/ Paragraph / "His sheath is pulled back, half his knot cresting out, and you reach forward and pull it the rest of the way down, knot emerging with a wet" / em "slurp" / ". You bear down inside him, fingers finding the bloated lump of his prostate and working over it, hand on the outside sliding up along the underside of his cock, behind his knot, stroking back and forth. The hellhound snarls, hips jerking like he doesn't know what direction to move, and his cock throbs, a muscle contracting just under your fingers, pumping out a sloppy drizzle of pre. Or come. It's hard to tell the difference. You work your fingers against his spasming internal muscles, milking out another gush of black slime. You catch it in your palm, smearing it across his shaft -- catching a mess of gritty, charred gunk, smearing it all together into a grimy mess -- and stroke him in time with your prods against his prostate. His cock gushes, the constant dribble of his pre turning into more of a constant pour, a solid line of ooze hanging down from his cocktip all the way to the ground, thickening and thinning in time with his pulses."
		/ Paragraph / "You draw his cock back between his haunches, pointing straight down, and wrap your hand around his knot, fingers probing across the root of his cock, inside his sheath. You're four fingers deep inside his ass, the muscle splayed like a giant, rubbery band, and his prostate convulses under your fingers, muscles squeezing down on it -- and when they let up, you replicate the motion, milking his prostate, keeping his cock drooling more and more thick, tarry slime. There are dense clots inside his prostate, gummed-up slime, and the thudding pressure knocks some free, winding their way down, throbbing under your fingers pressed against his cock, ultimately gushing from his cock in grimy clots. The ooze builds up between his paws, so thick it piles on top of itself in a sloppy mound."
		/ Paragraph / "He whines and snaps at the air, hunching, tar-slathered cock constantly spurting, knot half-inflated, pissing out a sludgy mire of come and pre all over his paws. He's not coming, precisely, but he's spurting a lot more than pre. You keep the pressure on his prostate, the abused muscle throbbing as you milk out yet more stringy, boiling slime, cock yanked down backwards, just stirring the air as he tries to thrust, to get more contact. His knot tries to swell, to knot your hand, but you slide up, hand stuffed mostly inside his sheath, fingers wrapped around the slender root of his cock. Milking out more and more slime, until the bubbling morass far outstrips what he'd come usually, even knotted: a big half-congealed puddle, spreading all around his body,"
		/ case locoplan b of
			Basic Biped -> "squelching under your knees,"
			Basic Naga -> "squelching under your coiled body,"
			Basic Centaur -> "squelching under your folded forelegs"
			f -> bodyFail "milking" f
		/ "milking him without ever giving him the release of a proper orgasm."
		/ Paragraph / "He slumps to the ground, underbelly squelching, and you release your grip on his cock, fingers pulling from his asshole, leaving the rim of his asshole swollen and pouting, shiny from the mess of his own pre you've used as lube."
	] <> assplaySwitch AfterMilking b h

assplaySwitch :: FromAnal -> MarkedScene Index () LesserHound
assplaySwitch from b h =
	[ Narration2 $ c "He pants, exhausted, body slumped down into the morass of pre and come you've forced out of him, cock still hard and erratically spurting sludgy black tar. His flames gutter down, lapping low across his fur. His entire body shudders, rising and falling with the force of his breath. His asshole spasms, closing on nothing only to spread back open into a sloppy gape, looking like a swallowing mouth."
{-
	, Choices
		[ ("yeah this is good", AssplayCloser)
		, ("okay i'm gonna fuck you now", FuckTop from)
		]
	-} , FauxChoice "yeah, this is good"
	, Jump AssplayCloser
	]

assplayCloser :: MarkedScene Index () LesserHound
assplayCloser b h =
	[ Narration2 $ c "Yeah, that looks about right. You leave him there in a fucked-out pile of his own come, his cock still needily trembling and drooling."
	]

oralBranch :: SheathState -> MarkedScene Index () LesserHound
oralBranch sheath b h = case locoplan b of
	Basic Centaur -> flipOral b h
	Basic Quadruped -> crawlOral b h-- i guess?? i mean you don't have arms to flip w/
	Basic Slithering -> crawlOral b h
	_ ->
		[ Narration2 $ c "You lick your lips. The hound's cock throbs, sending the cords of bubbling black-tar pre stuck to his shaft swaying."
		, Choices
			[ ("crawl under his belly", CrawlOral)
			, ("flip him over", FlipOral)
			]
		]


flipOral :: MarkedScene Index () LesserHound
flipOral b h =
	[ Narration2 $
		"You reach out and grab the hellhound by the haunches, and flip him over onto his back. He yelps as he topples over, twisting, and his cock slides back and forth over his belly, dribbling all over the black shadow-mass of his body."
		/ Paragraph / "You lean in close, taking his cock in hand and giving it a few pumps, smearing his gummy pre all over the shaft, before you press your mouth against the red-hot tip and take it into your" / muzzle / "," / hands / "keeping his haunches spread as you dive down onto his cock."
		/ Paragraph / "Despite the awkward position for him, the hound rapidly gets into it, yapping and hunching, the muscles of his hips jerking against your" / hands / "as he tries to thrust into your mouth, not particularly making much headway. You bob back and forth on his cock, spit thinning his pre into an oily mess, slopping down his shaft and boiling down into a blackened crust. You swallow, gulping down the tarry mess, swallowing when the sharp tip jabs into the back of your mouth. You swallow the rest of his cock, lips straining against the swollen flesh of his knot, tip down your throat, shuddering each time you swallow. Thick, heavy globs of pre pour down your throat, a messy froth bubbling out past your lips and boiling down to nothing as it smears across his fat knot."
		/ Paragraph / "His haunches jerk harder, his huge paws windmilling in the air, head tipped back as he barks, tongue lolling from his open mouth. He rams his knot against your lips, grinding it down as hard as he can, given his position."
	, Choices
		[ ("let him knot yr face", OralKnot Flip)
		, ("pull off", OralComeshot Flip)
		]
	]

crawlOral :: MarkedScene Index () LesserHound
crawlOral b h =
	[ Narration2 $
		"To get your mouth to his dick you gotta"
		/ case locoplan b of
			Basic Slithering -> "slither"
			Basic Quadruped -> "get on your belly and crawl"
			Basic Biped -> "get on your back and shimmy"
			Basic Naga -> "slither"
			f -> bodyFail CrawlOral f
		/ "down through the dust, between the hound's legs. The hound helps, paws sudden heavy impact-weights across your"
		/ case locoplan b of
			Basic Slithering -> "underbelly"
			Basic Quadruped -> "underbelly and chest"
			_ -> "stomach and chest"
		/ ", his already-drooling cock smacking across your face and leaving you tethered together with thick cords of black precome before you manage to suck the tip into your mouth."
		/ Paragraph / "The hellhound doesn't wait at all: he yaps and yammers and hunches forward, ramming his cock near knot-deep into your mouth, "
		/ case mouthType b of
			Mouth -> c "sharp tip breaching your throat"
			_ -> "solidly filling your" / muzzle
		/ ". You cough and gag, a messy sputter of spit and oily pre splattering across your upturned face, sizzling down across the flesh of his cock, burning away in an instant. He humps your face, knot slamming against your lips and pulling back slimy with black ooze spilling from your mouth. His swollen cock throbs arrythmically, knot pulsing against your lips, like his cock is a living thing straining against the flesh of your mouth. His shaft carves its way deeper into your throat as the hound jerkily mounts your face"
		/ case locoplan b of
			Basic Slithering -> mempty
			_ -> ", forepaws scrabbling at your shoulders"
		/ ", tongue spilling boiling ooze down your neck"
		/ hasDick ? (", flecks of drool splattering across your own hard" / Cock^s)
		/ "."
		/ Paragraph / case locoplan b of
			Basic Quadruped ->
				"You swallow, " / muzzle / "dragging along the underside of the hound's cock, until your nose is pressed between the shadowy swell of his balls. His sheath slurps back and forth, digging against the base of his knot when he bottoms out in your throat. The hound growls and yammers, drool spilling in sloppy cords across your underbelly"/ hasDick ? ", streaming over your untouched cock, coating it in black drool"/ "."
			Basic Slithering ->
				"Your body writhes, huge coils looping around themselves, twisting around the hound's forelegs and chest. The hound growls and yammers"
				/ case bodyMat_ Chest b of
					Solid (FMat _ Scales) -> ", drool sheening your scales"
					_ -> mempty
				/ "as he digs his paws into the sides of your body and thrusts, grinding his knot against your" / muzzle / "."
			Basic x | x `elem` [Biped, Naga] -> do
				"You reach up, hands wrapped around his flanks, cradling his heavy balls, squeezing his knot, stroking the heavy flesh of his sheath back and forth, and he growls and yammers, "
				/ case locoplan b of
					Basic Naga -> "drool spilling in sloppy cords across your underbelly"
					Basic Biped -> "drool spilling in sloppy cords across your stomach"
					f -> bodyFail CrawlOral f
				/ hasDick ? (", and streaming around your untouched" / Cock^s)
				/ "as he fucks you: your hand stroking his sheath, lips wetly kissing the swell of his knot."
			f -> bodyFail CrawlOral f
		/ Paragraph / "He thrusts fast and hard, using your body as a yielding orifice to fuck. His balls crash against your face with the force of iron weights, shaft forcing your throat wide, knot ramming again and again against your face. Your body shudders, throat spasming, and the hound thrusts again and holds it, grinding his knot up and down your sloppy face in fractional increments, the heat burning away the tacky ooze splattered across your skin. You open wide; your jaw spreads, and his knot rams inside, swelling your cheeks -- the hound howls, jerking back, and rips his knot from your mouth nearly as soon as he crams it inside, back and forth, the black-flame flesh of his sheath billowing up, peeling back further as he tries to cram the entirety of his cock in you."
		/ Paragraph / "His thrusts grow even more frantic, yips and barks emerging from his throat with a messy spatter of drool, spraying your body down with bubbling black tar. He whines, knot throbbing, swelling even further, struggling to fit even past your bruised lips."
	, Choices
		[ ("let him knot yr face", OralKnot Crawl)
		, ("pull off", OralComeshot Crawl)
		]
	]

oralKnot :: FlipCrawl -> MarkedScene Index () LesserHound
oralKnot fc b h =
	[ Narration2 $
		case fc of
			Flip -> "You bob down, trying to suck the huge, burning swell of his already nearly-entirely engorged knot into your mouth; each failed attempt makes his entire body spasm and jolt, his cock spurting sloppier and sloppier spurts of pre into your mouth. He whines, a feral animal noise, and his knot throbs. You shove down harder, tongue curling along the swell of his knot, until it finally lurches into your mouth with a" / em "slurp" / ", bloating your cheeks with its girth, its ruddy-red glow visible through your skin."
			Crawl -> "You open wide and let him ram his swollen knot in with one final thrust. He whines, a feral animal noise, and his knot throbs, larger and larger locked behind your" / teeth / ", until it bloats both cheeks."
		/ "He howls, loud, and his cock throbs all down your throat, sharp tip spasming, his first squirt of pre spraying straight down into your stomach. His load coats your throat, tar-thick slime building up around his shaft, but only a thin smear of drool spills from your lips, throat knotted tight. You swallow and swallow, gulping down his scalding load, burning heat spreading all through your body. And then, knotted, he keeps coming, the thick tar giving way to a thinner sludge, shaft throbbing and throbbing against your lips, wrapped tight against the base of his cock."
		/ Paragraph / case fc of
			Flip -> "The hound squirms, trying vainly to shift while on his back, trying to tug his knot taut against your" / teeth / ". You pull back minutely, wrapping your hand around the span of shaft under his knot, mostly sunk into his sheath, and use that to tug him bodily forward an inch, until his knot strains against your lips and teeth. His entire body sags, letting out a globbering howl, as his knot steadily throbs in your mouth, its spurts fading"
			Crawl -> "The hound huffs and twists, balls dragging across your face, cock twisted backwards behind his legs, dragging you bodily forward an inch, flesh of his knot digging against the backs of your" / teeth / ". He keeps panting, yapping, as he pisses out come, spurts subsiding "
		/ "into a continual spill of sloppy come, building up into a heavy, sloshing weight inside you. It takes minutes, "
		/ case fc of
			Flip -> "suckling on his knot"
				/ hasArms ? ("with your" / hands / "dug into his flanks")
				/ ","
			Crawl -> c "pinned down with his cock down your throat, "
		/ "but eventually his ejaculation slows and then stops, knot gone spongy, and when"
		/ case fc of
			Flip -> "you pull back"
			Crawl -> "he tugs again"
		/ "it pops free, spreading your jaw brutally-wide again for a second before the whole thing pulls from your" / muzzle / ", sliding out a dull ruddy red, ropes of thick black come"
		/ case fc of
			Flip -> "spilling across his underbelly"
			Crawl -> "streaking down his shaft, "
		/ "steaming gently."
	]
	<> closing True b h

oralComeshot :: FlipCrawl -> MarkedScene Index () LesserHound
oralComeshot fc b h = if hasArms b
	then
		[ Narration2 $
			case fc of
				Flip -> "You bob back and forth a few final times before you pull back and finish him off with your hand, black-tar pre bubbling and boiling over your hands as you pump his sloppy cock, nearly all of it burning down to a crackling crust before he comes. You"
				Crawl -> "He thrusts a few final times, cock churning in your throat, drawing out entirely to slap across your face, and you"
			/ "squeeze down on his knot, letting him faux-knot your hands, only the tip of his cock still in your mouth. His wiggling hips drag him side-to-side, trying to cram his knot deeper, ooze boiling off his burning flesh as his knot burns hotter and hotter, a ruddy coal held in your hands."
			/ Paragraph / "He howls, loud, and his cock throbs, knot flushed nearly white-hot, and then erupts with a torrent of boiling black tar, flooding your mouth in a sudden spurt and pouring down your face, hissing and steaming, sluggishly boiling still even as it puddles on the ground. You swallow part of his scalding load, coughing some of the mess across his underbelly, soaking his fur and smearing across your flesh -- and then, knotted to your hand, he keeps coming, the thick tar giving way to a thinner sludge, the root of his shaft throbbing against your fingers, knot shuddering in your hands, cock leaking continually all across your face, slathering you in boiling tar."
			/ Paragraph / case fc of
				Flip -> "You curl your fingers around the underside of his knot and tug hard, dragging him bodily forward a half-inch, hips nearly rising off the ground with a good chunk of his weight held right across his knot, and he howls."
				Crawl -> "The hound huffs and twists around, cock twisted backwards between his legs, and pulls, dragging you an inch forward."
			/ "The pressure keeps him coming, cock pissing out sludgy black come in weak spurts, "
			/ case fc of
				Flip -> c "his body"
				Crawl -> "soaking your" / materialWord Ass
			/ ". The sheer volume of his come quenches the heat of his cock, "
			/ case fc of
				Flip -> "spurting over his shadow-body and soaking his fur, spitting against his lines of burning flame as it spills over his sides"
				Crawl -> "come hissing and steaming as it streams down his red-hot shaft, squelching and gurgling as it coats your fingers"
			/ ". Eventually his ejaculation slows and then stops, knot gone spongy, a dull ruddy red, and"
			/ case fc of
				Flip -> "you let him down, knot popping from your loose grip, cock smacking against his belly and splattering black sludge all over, only barely boiling where his cock comes to rest in the swampy mess he's made"
				Crawl -> "when he twists again, minutes later, it pops from your loose grip, hanging low from his body, sheath trying to swallow his still-swollen knot in slow increments"
			/ "."
		]
	else
		[ Narration2 $
			"He thrusts a few final times, cock churning in your throat, drawing out entirely to slap"
			/ case mouthType b of
					Mouth -> c "across your face"
					_ -> "along your" / muzzle
			/ ". His knot jabs against your mouth, iron-hard flesh straining and failing to push forward. His wiggling hips drag him side-to-side, trying to cram his knot inside, ooze boiling off his burning flesh with a horrible acrid stench."
			/ Paragraph / "He howls, loud, and his cock throbs, knot flushed nearly white-hot, and then erupts with a torrent of boiling black tar, flooding your mouth in a sudden spurt and pouring down your face, hissing and steaming, sluggishly boiling still even as it puddles on the ground. You swallow part of his scalding load, coughing some of the mess across his underbelly, soaking his fur and smearing across your flesh. The hound jerks, cock pulling from your mouth entirely to slide"
			/ case mouthType b of
					Mouth -> c "across your face"
					_ -> "along your" / muzzle
			/ ", gushing foul ooze in tarry streams all down the sides of your head, down your neck."
			/ Paragraph / "The hound humps your face, cock sliding through the mess of sludge he spurts all over you, forming sloppy trenches in the glaze of boiling black slime, come hissing and crackling as it streams down his red-hot shaft, knot pulsing white-hot. The hound whines, ejaculation slowing after the first few spurts until he's sluggishly drooling the dregs of his load, cock sliding up across your scalp, entirely coating your head in thick cords of ooze."
		]
	<> closing False b h

fuckAnal :: SheathState -> MarkedScene Index () LesserHound
fuckAnal sheath b h =
	[ case locoplan b of
		Basic Biped -> fuckAnalBiped b h
		x
			| x `elem` [Basic Centaur, Basic Quadruped] -> fuckAnalQuadCent b h
			| x `elem` [Basic Naga, Basic Slithering] -> fuckAnalNagaSlither b h
		f -> bodyFail "fuckAnal" f
	, Choices
		[ ("let him knot you", AnalKnot)
		, ("pull off", AnalComeshot)
		]
	]

fuckAnalBiped :: BodyGlob -> LesserHound -> Event LesserHound Index ()
fuckAnalBiped b _ = Narration2 $
	"You get down on all fours and show him your ass"
	/ hasTail
		? (", raising your"
			/ case bodySpecies Tail b of
				Just Goat -> "stubby goat tail"
				--Just Horse -> "heavy horse tail"
				Just Unicorn -> "heavy horse tail"
				_ -> "tail"
			/ "to show off your fat pucker unobstructed")
	/ ". Even as you start to crouch he's on you, impact nearly toppling you to the ground; he bowls you down onto all fours, shoulders down, ass up, his body a heavy weight all across your back, his hard cock jamming against your thighs, ass"
	/ hasExternalBalls ? ", smacking your balls"
	/ ", thrusting between your" / partAdjective Legs / "legs. His forelegs clamp around your waist, his erratic hunches mostly driving his cock up and down your crack, ass cheeks spread around his fat shaft, "
	/ case bodyMat_ Legs b of
		Solid (FMat _ Skin) -> "skin glazed with his tarry black precome"
		Solid (FMat _ Fur) -> "fur sheened with tarry slime as he saws his cock back and forth"
		_ -> "smearing his tarry black pre back and forth over your asshole"
	/ ". His knot slaps your cheeks with a sharp" / em "crack" / ", thin webs of slime"
	/ case bodyMat_ Legs b of
		Solid (FMat _ Fur) -> "tangling all over your fur"
		_ -> "dragging out between you"
	/ "as he withdraws, burning off his red-hot shaft and leaving behind a thick slimy crust."
	/ Paragraph / "The hound readjusts, head twitching and casting a messy cord of slobber from your back to your scalp, and his cock drags down minutely, tip bearing down against your asshole. He thrusts once, twice, before his sharp tip notches into place, hitting your asshole and catching, and then he slams forward, burying your body in flame and black shadow as he bears down, sheathing most of his cock inside you in a single brutal lunge. He staggers, hind legs skittering, cock throbbing hot and huge inside you, before he jerks back a fraction of an inch and drives forward again, knot spreading your cheeks and slamming hard against your spread hole, the" / em "smack" / "sending up"
	/ case bodyMat_ Legs b of
		Solid (FMat _ Skin) -> "droplets of bubbling precome all across your skin"
		Solid (FMat _ Scales) -> "droplets of bubbling precome all across your scales"
		Solid (FMat _ Fur) -> "a wash of precome that catches in your tangled fur in fat beads"
		Solid (FMat _ Coat) -> "a wash of precome that catches in your coat in fat beads"
		_ -> "droplets of bubbling precome all over your ass"
	/ ", impact driving you a half-inch forward."
	/ Paragraph / "The hellhound starts thrusting like mad, no time to waste now that he's inside you, huge paws digging deep into your sides, cock pummeling your asshole, knot crashing again and again against your clenching, spasming hole. He howls, yipping and snarling, tongue lolling long from his mouth and swiping across your shoulders and neck, "
	/ case bodyMat_ Legs b of
		Solid (FMat _ Fur) -> "tongue soggily sucking on your shaggy fur"
		_ -> "leaving you dripping with black tar"
	/ "."
	/ Paragraph / "He's pent up; it doesn't take him long to push himself over with the way he's thrusting. His knot hammers harder against you, thrusts squishing it against your hole for long seconds at a time, struggling to cram it inside."

-- if yr a quadruped AND a small-animal type (currently only wolf i think) then he can mount you properly
fuckAnalQuadCent :: BodyGlob -> LesserHound -> Event LesserHound Index ()
fuckAnalQuadCent b _ = Narration2 $
	"You present your ass to him, "
	/ hasTail ? "tail flicking away from your hindquarters to show off your fat pucker, "
	/ "and the hound yips and slavers, "
	/ case bodyHeight b of
		SmallAnimal -> "rearing up to mount you in an instant, cock gushing pre across your haunches as he stabs at your asshole"
		_ -> "jumping up on his hind legs to try to mount you, cock smearing messy lines of black pre across your flanks"
	/ "."
	/ Paragraph / case bodyHeight b of
		SmallAnimal -> mempty
		_ -> "You end up having to sit down, folding your legs under you so the hellhound can reach: even as you sit he's still trying to mount, claws digging into your flanks, cock rutting against the cleft of your ass, up and down, until you're both finally in a position for him to thrust."
	/ "He jerks forward, cock skidding up, and he sinks knot-deep instantly, howling in triumph as his burning-hot cock throbs inside you, knot shuddering against your hole, cock steadily spurting huge spurts of boiling black pre into your ass."
	/ Paragraph / "He digs his claws into the meat of your ass, open panting mouth spilling black drool down your sides, and starts thrusting wildly, pounding away with hardly a thought in his head. His knot smacks against your hole, digging fractionally in, and after a moment -- and a half-dozen more thrusts, the flesh of his knot webbed with goopy pre, thick cords stretched between his knot and your hole -- he digs his hind paws into the ground and thrusts again, hard, slamming his cock against your hole and then pressing down."
	/ Paragraph / "You open up, hole spreading, and swallow his knot easily, thick asshole clenching hard around the base of his cock, his sheath kissing your flesh. The hound howls again, yipping and barking, and jerks back, hole blossoming as he pulls out, knot pulsing like a heart, notched against your hole for a brief moment before he slams it back in. The hound starts knot-fucking you with a ragged, erratic rhythm, cock gushing and gushing pre inside you, sloppy ooze spraying out"
	/ case bodyMat_ Legs b of
		Solid (FMat _ Fur) -> "and drenching your fur until it's entirely saturated"
		_ -> "to paint your ass"
	/ "each time he rips his knot loose. He yaps, sides heaving, tongue lolling from his mouth, drooling black ichor down your"
	/ case bodyMat_ Legs b of
		Solid (FMat _ Fur) -> "sodden"
		_ -> mempty
	/ sides / ". His knot slurps back and forth through your hole, each thrust punctuated by the" / em "slap" / "of his balls"
	/ hasExternalBalls ? "against yours"
	/ ", and then the soggy slurp as he pulls his knot out, a wrenching sensation that echoes up further inside."
	/ Paragraph / "The hound doesn't last long; his knot throbs with each thrust, still swelling in increments, each thrust taking minutely more effort to shove it inside; bulging the flesh of your ass more as he struggles to rip it back out. The sloppy pulses of pre soaking your insides grow hotter, bubbling and boiling inside you, the mess oozing from your hole steaming"
	/ case bodyMat_ Legs b of
		Solid (FMat _ Fur) -> "as it winds its way through your fur"
		_ -> mempty
	/ "."

nagaSlitherSplit :: Show b => b -> BodyGlob -> a -> a -> a
nagaSlitherSplit err b naga slither = case locoplan b of
	Basic Naga -> naga
	Basic Slithering -> slither
	f -> bodyFail err f

fuckAnalNagaSlither :: BodyGlob -> LesserHound -> Event LesserHound Index ()
fuckAnalNagaSlither b _ = Narration2 $
	"You flip over onto your back, " / Cock^s / "pressed against your"
	/ hasLocotype Naga ? "stomach" ?: "underbelly"
	/ hasAsshole
			? ("but the hound's gaze goes below it, to the barely-there divot of your asshole, "
				/ case bodyMat_ Tail b of
					Solid (FMat _ Scales) -> "a narrow line between your scales"
					Solid (FMat _ Protoplasm) -> "a dark rubbery circle dimpling into your protoplasm"
					_ -> mempty)
			?: (", and the hound's gaze focuses on your spread vent, lips raised around your" / Shaft^s)
	/ ". He leers, dipping his head to lap across your "
	/ hasAsshole ? "asshole" ?: "cloaca"
	/ ", laughing as you groan and buck up. His slobbery tongue drags against your sensitive flesh, folding over itself in ichorous piles as he strains to push inside. But he's thinking with his cock, and it's only two or three swipes before he pulls up and straddles you, "
	/ case bodyMat_ Tail b of
		Solid (FMat _ Scales) -> "cock sliding against your scales, the sharp tip digging into your hole."
		Solid (FMat _ Protoplasm) -> "cock bloating your soft, puffy protoplasm, when he digs the sharp tip into the hole."
		_ -> "cock sliding to press against your hole."
	/ "He huffs, slobbering drool across"
	/ hasLocotype Naga
		? "your neck -- you tip his head up, kissing his slavering mouth -- "
		?: "the arch of your body -- up until he's laving across your head, drooling into your mouth -- "
	/ "and jerks forward, cock skidding up and down over your asshole until it catches. He howls into your mouth, sulphurous breath burning your lungs, and drives in nearly to the knot in a single thrust, insides burning and spasming around his inflamed shaft."
	/ Paragraph / "His hips jerk, paws scrabbling at your sides, and he pulls back, thrusts forward, ramming his knot against your hole with a loud" / em "smack" / ". His whole cock twitches, shuddering inside you, and he digs his paws into the ground beside you and starts thrusting, sloppy black pre smearing out of your asshole, sizzling and burning across his red-hot cock."
	/ Paragraph / hasLocotype Naga ? "You wrap your arms around his shoulders" ?: "You coil around him and squeeze" / ", pulling you deeper into his slavering kiss, his tongue licking across your" / teeth / "and spilling further into your mouth. His shoulder drop, ass hunching, as he starts thrusting. His fat cock churns inside you, spurts of pre firing in hot bursts inside you, erupting around your clenching hole with each thrust. "
	/ hasDick ? ("Your" / Cock^s / Digs^s / "against his underbelly, shadow-fur slick and enveloping, teasing out little bursts of pre with each of his thrusts.")
	/ Paragraph / "He doesn't last long: his knot rams against your hole with each thrust, webs of slime hooked all around its burning swell, boiling down into a crackling black crust. His thrusts meet your flesh with a" / em "crack" / "that shakes all through your body, and his paws scrape the ground, digging in deeper to give him better traction, working his nearly fully-inflated knot up and down against your hole."

analComeshot :: MarkedScene Index () LesserHound
analComeshot b h =
	[ Narration2 $
		case locoplan b of
			Basic Biped ->
				"The hound's knot rams against your hole, spanking against your flesh with wet, sloppy smacks, once, twice, before he howls and comes, his sloppy gushes of pre suddenly overwhelmed by sudden spurts of come, bursting in hot blossoms inside your ass. The tar-thick sludge boils inside you as it sluggishly floods down the length of his cock and spills from your ass, hissing and boiling away into acrid steam as it spills against his knot, crammed up against your hole, his reflexive, hunching thrusts still driving its fully-inflated weight between your cheeks, each" / em "smack" / "sending out a thick, sluggish gush of come spraying in every direction, layers of steaming come webbing across your hips and down your thighs."
				/ Paragraph / "The hound lurches back, half his cock pulling from your ass, slathered in boiling come, and thrusts forward, smacking hard against your asshole, and again, and his next thrust drags his entire cock out with a" / em "squelch" / ", sharp tip spraying thick lances of inky black come"
				/ case bodyMat_ Back b of
					Solid (FMat _ Skin) -> "all across your flesh, squirting in heavy, boiling ropes across your back"
					Solid (FMat _ Fur) -> "all up your back, soaking your fur down in sodden hanks"
					Solid (FMat _ Coat) -> "all up your back, plastering your coat to your skin"
					_ -> "all up your back, squirting in heavy, boiling ropes"
				/ "as he humps wildly, cock skidding across your" / partAdjective Ass / "ass cheeks before he finds his place again, sinking knot-deep for a few frenzied thrusts before he slips out again, still-spurting cock splattering you in messy lines, black goo coating your back and drooling down your sides."
				/ Paragraph / "The hound's spurts slow, and his thrusts with it, until he's slowly rolling his cock between your cheeks, a dribble of come winding down his shaft and smearing across your"
				/ case bodyMat_ Legs b of
						Solid (FMat _ Skin) -> "already-glazed skin"
						Solid (FMat _ Fur) -> "entirely saturated fur"
						Solid (FMat _ Coat) -> "matted coat"
						Solid (FMat _ Scales) -> "slime-coated scales"
						Solid (FMat _ Protoplasm) -> "rubbery protoplasm"
						_ -> "ass"
				/ ", the underside of his still-swollen knot digging against your gaping hole, reheating the come slopping out of you to boiling."
				/ Paragraph / "He dismounts, falling to the side, cock hanging low between his haunches, its flesh gone a dull ruddy red, coated in sluicing layers of steaming come."
			Basic x
				| x `elem` [Centaur, Quadruped] ->
					"You clench just before his knot enters you again, the hound's thrusts banging his knot against your hole again and again, each thrust sending up a spray of pre, droplets splattering across your"
					/ case bodyMat_ Legs b of
						Solid (FMat _ Skin) -> c "flesh"
						Solid (FMat _ x) -> c (matword x)
						Trans _ (FMat _ x) -> c (matword x)
					/ ". He howls and barks, yapping; the slide of his shaft, the squeeze of each thrust's impact across his knot, all enough to bring him off, even if he's not knotted. He comes with a yammering series of yips, drooling all down your back, cock erupting into your ass and instantly flooding out, a gush of boiling black sludge that floods the space between you."
					/ Paragraph / "He thrusts, unloading inside you, until an errant thrust jerks him out entirely -- his cock skids up, the next gush of come erupting into the air and splattering down all across your back"
					/ hasTail ? ", soaking your tail in grimy black ooze"
					/ ". He ruts against your hide, cock skidding between your cheeks before he manages to sheath himself again, making a few more thrusts into your ass before his cock pops out again. Your open hole gapes, black come pouring out and being smeared all up and down your ass by the hound's shaft, still erupting thick black tar"
					/ case bodyMat_ Legs b of
						Solid (FMat _ Skin) -> "all over your skin"
						Solid (FMat _ Fur) -> "into your soaked and saturated fur"
						Solid (FMat _ Coat) -> "atop your saturated coat"
						Solid (FMat _ Scales) -> "all over your scales"
						_ -> "all up and down your sides"
					/ "."
					/ Paragraph / "Not properly knotted, his ejaculation dies down quickly, his still-inflamed knot notched between your cheeks, underside pressed against your gape as his cock gushes its final few spurts up across your back. He pulls back, panting, cock glazed and drizzling black slime to the ground."
				| x `elem` [Naga, Slithering] ->
					"You shift and reach down"
					/ hasLocotype Naga ? ", wrapping your hand" ?: "with your tail, wrapping it"
					/ "around his knot and squeezing: The hound yelps, yapping and barking, knot throbbing"
					/ hasLocotype Naga ? "in your palm" ?: "against your tail"
					/ ", and then he spasms, entire body thrashing. The heat of his pre blooms, fiery shots of come pouring inside you -- an eruption of pre gushing out around his shaft as he unloads inside you, cock thrusting through the sloppy black ooze, knot a hot coal "
					/ hasLocotype Naga ? "cradled in your hands" ?: "bound by your tail"
					/ ". He pulls out entirely, rutting up your body, cock spraying wildly, thick black tar splattering across your" / (materialWord Tail) / ", still boiling as it "
					/ case bodyMat_ Legs b of
						-- pretty sure that's not reachable since this is slithering/naga only!
						Solid (FMat _ Fur) -> "mats down your fur into nubby, congealed dreadlocks"
						_ -> "drips down your sides"
					/ "."
					/ Paragraph / "The hound ruts his cock across your body, knot throbbing, and his pulses slow and subside into a steady drool, black ooze spilling down his shaft and smearing across your body. He huffs, sides heaving, open mouth spilling black drool that's almost indistinguishable from his load, glazing you in a sloppy coating of slime."
			f -> bodyFail AnalKnot f
	]
	<> closing False b h

analKnot :: MarkedScene Index () LesserHound
analKnot b h =
	[ Narration2 $
		case locoplan b of
			Basic x
				| x == Biped ->
					"You clench and then open, spreading yourself, and the hound's next thrust slams into you, knot bearing down, pressing inside you in fractional increments until the entire thing abruptly slurps inside, swallowed up by your convulsing asshole. The hound slavers, splattering black drool all across your back. His hips beat a tattoo across your ass as he humps wildly, knot jerking back and forth inside you in fractional motions, the drawn-taut flesh of your inner ass shifting with it; his thrusts grind your cheeks together, hole dipping deeper from the pressure, turning your hole into a thick crater, and when he jerks back the entire flesh of your ass bulges out around his knot, beneath your asshole, as a smooth round well of flesh."
					/ Paragraph / "The hound's knot throbs, his gushes of pre erupting into huge bolts of come as his knot swells to its utmost. Heat blooms inside you, the hound's comeshots audibly slapping across the walls of your inner ass, its tar-thick sludge sluggishly bubbling and boiling inside you as the hound fills you up."
					/ Paragraph / "His paws scrabble down your sides; his entire body twists. The hound dismounts, twisting backwards until you're ass-to-ass, and his cock lurches inside you with the motion, knot straining against your flesh, rotating in shuddering, shocky increments. He tugs, dragging you bodily backwards an inch, knot straining against the ring of your ass. And, knotted, he keeps coming: the tarry sludge giving way to thinner ooze, pouring in sloppy waves inside you, each spurt flooding higher and higher. Slowly, over the course of minutes, your guts flood, gurgling as his boiling come winds its way deeper inside you, cock steadily pissing out a continual flood of come, shuddering weakly."
					/ Paragraph / "After minutes, on your hands and knees in the dust, the hound's ejaculation slows and then stops. The hound tugs again, knot still swollen but rubbery, and slowly he pulls himself off -- knot dragging backwards against your asshole, spreading your bruised flesh in increments, until with a loud, sloppy" / em "squelch" / "he tears his knot free, an explosion of come splattering all across your bodies; his cock tethered to you with sloppy, dripping ropes of come; a waterfall of come pouring from your gaping hole."
					/ Paragraph / "The hound pants, his cock a ruddy red, drooping between his hind legs, slathered in steaming come."
				| x `elem` [Centaur, Quadruped] ->
						"You flex open, swallowing his knot one last time, and then clamp down -- the hound jerks, still thrusting, rutting his knot a scant inch back and forth in your ass. He howls, yammering and barking, and comes in a rush, the first squirts of tarry black ooze drooling out around his knot before it swells entirely locking the rest of his load into your ass. He thrusts, just wiggling his knot inside you, cock shuddering with each pulse of come he unleashes, gurgling in sloppy waves deeper inside you, streaming back down his cock to form a wet weight bearing down against his knot."
					/ Paragraph / "And, being knotted, he just keeps coming, the thick tar giving way to a thinner ooze, pouring from his cock in a continual flow. He pants, loud, and dismounts, twisting around backwards -- cock twisting inside you with his motion -- until you're ass-to-ass, his cock pulled backwards between his hind legs. The pressure on his knot keeps him coming, pissing out slobbery slime, each squirt stirring the lake of come growing inside you, flooding higher. Your stomach gurgles, boiling come slopping down into your guts, pulse after pulse as the hound keeps coming for minutes, unceasing."
					/ Paragraph / "Eventually his ejaculation slows, and then stops. His knot softens, slowly, and for the final minute you can feel his load start to work its way around the edges of his deflating knot, sloppy black ooze burbling out of your ass, around his knot. He tugs, entire body straining as he works to pull his knot free, and it finally gives with a hollow" / em "pop" / ", the flesh of your ass bulging out under your asshole, more and more, until his knot rips free with an explosion of come, your thick ring clenching and opening on nothing as a watery mess of come pours out, slopping down your legs."
				| x `elem` [Naga, Slithering] ->
					"You spread your hole, clenching tight and then opening, and his next straining push finds purchase, slowly sinking inside you, until he lurches forward, jaw snapping, head and shoulders wildly thrashing side-to-side, and slams his knot home. His entire body shudders, cock throbbing, and with him knotted inside you you can feel every incremental stretch as his knot finishes swelling, locking him tight inside you. His cock burns hotter, the sludge bubbling out around the base of his shaft thick and tar-like, and he spasms, cock erupting inside you with a fierce spurt of come, burning hot splattering inside you, again, and again, and again. The hound snarls and drools black tar into your mouth as he unloads. The heat swells and spreads, flooding deeper inside, and now, knotted, he keeps coming: the thick, tarry ooze giving way to a thinner sludge, freely spilling across the walls of your"
					/ hasAsshole
						? "ass, gurgling as it spills into your guts."
						?: "cloaca, gurgling as it spills deeper inside."
					/ Paragraph / "The hound whines, twisting, tongue pulling from your mouth in a sloppy flood. He tugs his knot back, and then twists around backwards, straddling the length of your tail, tugging hard. The pressure against his knot"
					/ hasAsshole
						? " -- the tightness of your ass in addition to the straining clench of your asshole -- "
						?: (" -- the rubbery tightness of your overstuffed cloaca in addition to the straining clench of your vent"
							/ hasDick ? (", gaping around the" / number (cockCount b + 1) / "cocks wedged through it")
							/ " -- ")
					/ "keeps him coming, pissing out watery come into your guts for minutes, intermittently tugging forward, knot bulging out the flesh beneath your ring."
					/ Paragraph / "Eventually the flood slows and then stops, and the hound's next jerk pops his knot out, flesh spongy, heat faded to a dull red. Your hole gapes, masses of black come oozing out of your freshly-fucked hole, spilling across the sides of your tail."
					/ Paragraph / "The hound stands there, cock hanging low between his hind legs, sheath bunched up against his knot, struggling to swallow the turgid mass."
			f -> bodyFail AnalKnot f
	]
	<> closing True b h



closing :: Bool -> MarkedScene Index () LesserHound
closing knotted b h =
	[ Narration
		[ "The hound shakes like a dog casting off water, fire whipping across the shadow-mass of his body. His sheath finally swallows up his slowly-softening knot, cock sinking back into his sheath in jerky motions."
		]
	, Dialog "/static/img/icon.png" "Lesser Hellhound"
		[ if knotted
			then "It's been a while since I knotted a sweet soul such as you"
			else "You spent me well enough for now"
		]
	, Narration2 $
		"The hound laps out, licking across your"
		/ case planLocomotion . bodyplan $ b of
				Spec Orb -> "orb "
				Basic Formless -> "orb "
				_ -> "face "
		/ "and leaving behind a fresh smear of black drool."
	, Dialog "/static/img/icon.png" "Lesser Hellhound"
		[ "Come back later"
		, "I'll mount you again"
		, "I'm sure you're eager for any cock this land provides"
		]
	]


{-

-- "oh what luck to find a tasty soul wandering around"

-- whining and begging once you get a human form and he wants to fuck (oral or anal :V)
---

The hound laps at your skin -- the surface of the membrane that divides your still-protoplasmic soul from the rest of the world. A whine grows in his throat. His cock is hanging low, tip drooling dark fluid into the swamp, knot already partly-swollen. His ember-eyes are dim with lust. His words are hardly distinct from the animal whining and begging:

>Been so long... sweet, fresh soul... need to rut, to fuck...

By the time his litany has dwindled to just yips and howls his cock is drooling steadily, fully erect, knot almost entirely engorged without even touching it.

-- (this is where you kick him away / present yrself)

He's on you instantly, paws scrabbling your sides, cock shockingly hot slapping down across your back, jerking and sliding over your skin. He aims down, cock driving between your thighs, tip leaving dark trails on your colorless skin as he tries to find any hole. You arch higher, letting his cock skid up, sharp tip digging into the groove of your imagined-asshole, and in the next moment he howls, slamming himself to the knot in your amoebic body. He pants, tongue lolling from his muzzle, absently lapping across your neck as he hunches forward, the fat swell of his knot battering against your asshole. His cock burns inside you, almost painfully hot, sharply _solid_ compared to your vague, boneless shape. Black pre floods your insides, slopping out with wetter and wetter slurps as he fills the small pocket of your pseudo-orifice.

He does not take long: it is a minute, perhaps two, before he pushes harder, your pliant membrane distending, sucking his knot in with a watery squelch, a final squirt of black slime spraying out before he's firmly plugging your hole, ass bulging outward from the fat swell of his knot. He comes with a yipping howl, vainly trying to thrust, cock spurting and spurting: burning-hot come with each pulse, splattering across your flooded insides, and then the long, slow spill of thinner, cooler slime, filling you until your body distends, bloating and stretching to fit his entire load.

Your membrane-skin osmoses slowly, but with him locked inside you, panting and huffing on your back -- a black blush stains your colorless skin, murky through the translucent flesh of your spirit-body. He grins against your shoulder, teeth glinting.

>Were you suitable, I'd breed you until it took. Look how eagerly your spirit clings to life. Soaking in my seed, hungry, guzzling up my load.

His cock kicks lodged inside you, a fresh spill of black come bloating you further.

An unknown time passes before he finishes, the rock-hard flesh of his knot slowly softening until it pops out in a flood of black ooze, a slimy waterfall from your gaping asshole. The hellhound leers at you, tongue flicking out, fiery drool throwing up smoke as he swallows his own come.

>Come back again later and I'll mount you again.
>I know you're eager for any cock this land will give you.

---




-- whining and begging once you get an _animal_ form (ditto)
-- etc etc centaur


	-- combat transition variants? want to avoid "we can fuck but if we're not gonna fuck we have to fight" but also want to avoid "yr options are sex, combat, or a boring "nothing happens" event"

-}
