{-# LANGUAGE MultiParamTypeClasses #-}
{-# LANGUAGE FlexibleContexts #-}

module Sites.Hell.Events.Hubur.Nightmare
	( Nightmare(..)
	, deck
	, scene
	, description
	, introTalk

	, name
	, bits
	, variants
	) where

import Prelude hiding ((/), (^))

import Sites.Hell.Events.Construction ((/), (^))
import Sites.Hell.Events.SceneUtils
import Description.TfNode (Animal(..))

import Sites.Hell.Events.Hubur

name :: String
name = "nightmare"

data Nightmare = Nightmare
	deriving (Eq, Ord, Show, Read)

instance Random Nightmare where
	randomR _ = random
	random = getRand $ pure Nightmare

instance ToJSON Nightmare where
	toJSON Nightmare = object
		[ pack "name" .= name
		, pack "data" .= ([] :: [Int])
		]

bits :: Value -> Value
bits _ = object
	[ pack "name" .= name
	, pack "data" .= ([] :: [Int])
	]

variants :: [(MapZone, ())]
variants =
	[ (Badland, ())
	]

instance FromJSON Nightmare where
	parseJSON (Object vs) = case (parse (.: pack "name") vs, parse (.: pack "data") vs :: Result [Int]) of
			(Success pName, Success [])
				| pName == name ->
					case pure Nightmare of
							Nothing -> mzero
							Just a -> return a
			_ -> mzero
	parseJSON _ = mzero

deck :: RandomGen g => Rand g (Enumerable Nightmare)
deck = return $ pure $ Nightmare

data Index = IntroTalk | JustWandering
	| WannaFuck | Sure | No
	| Unsheaths FromCent | Kneel | Mounted
	| SuckOff | KeepPlaying | FingerCockslit | FrotShaft
	deriving (Eq, Ord, Show, Read)

data From = FromSound | FromHJ
	deriving (Eq, Ord, Enum, Bounded, Show, Read)

data FromCent = UseMouth | UseAss
	deriving (Eq, Ord, Enum, Bounded, Show, Read)

instance Scene Index () Nightmare where
	link f = case f of
		IntroTalk -> introTalk
		JustWandering -> justWandering
		No -> no
		WannaFuck -> wannaFuck
		Sure -> sure
		Kneel -> kneel
		Unsheaths f -> unsheaths f
		Mounted -> mounted
		FrotShaft -> frotShaft
		FingerCockslit -> fingerCockslit
		KeepPlaying -> keepPlaying
		SuckOff -> suckOff

	display = fmap toLower . show
	vardisplay = fmap toLower . show

{-

in standing stones -> There's a nightmare among the standing stones. The horse kind.
on obelisk -> There's a nightmare waiting before the obelisk base. The horse kind.
-}

dialog :: Convertible a EventReel => Nightmare -> a -> Event Nightmare Index ()
dialog _ = Dialog2 "/static/img/icon.png" "Nightmare" . convert

scene :: Encounter Index () Nightmare
scene b n =
	[ Narration2 $ c "A nightmare roars across the dry badlands. The horse kind."
	, description n
	-- , error "bad scene"
	] <> introTalk b n

description :: Nightmare -> Event Nightmare Index ()
description n = Narration2 $
		c "It's a giant, muscular stallion, with a pitch-black coat and a mane of rippling fire. It's about 19 hands high. Its mane spills across its head and down its neck, billowing back at times in an imperceptible wind; its tail is fiery as well. It's shod with red-hot iron, and it leaves flaming hoofprints in its wake. Its eyes are burning red, with flames billowing up from them, streaming across its face."

introTalk :: Encounter Index () Nightmare
introTalk b n =
	[ Narration2 $ c "It notices your approach and turns to face you; its burning eyes stare into you."
	, dialog n $
		"A living soul, out in the badlands?"
		/ Paragraph / "What brings you out here?"
	, Choices
		[ ("just wandering", JustWandering)
		, ("hey you wanna fuck", WannaFuck)
		]
	]

justWandering :: Encounter Index () Nightmare
justWandering b n =
	[ dialog n $ "In that case, keep an eye out. Who knows what could befall a stray soul out in the wilds."
	]

wannaFuck :: Encounter Index () Nightmare
wannaFuck b n =
	case locoplan b of
		Spec Orb -> rejection b n
		Spec FloatingHead -> rejection b n
		Spec FloatingDick -> rejection b n
		Basic x | x `elem` [Biped, Naga, Quadruped, Centaur, Slithering] ->
			[ dialog n $ "You know, I could use some help with something I've been wanting to try. Something like a fertility ritual."
				/ Paragraph / "Your part would be the " / em "receptacle" / "."
			, Choices
				[ ("yeah sounds good", Sure)
				, ("on second thought no", No)
				]
			]
		Basic _ -> rejection b n

no :: Encounter Index () Nightmare
no b n =
	[ dialog n $ "Well, suit yourself then."
	, Narration2 $ c "You leave."
	]

rejection :: Encounter Index () Nightmare
rejection b n =
	[ Narration2 $ c "The nightmare looks over you."
	, dialog n $ "You're going to have to try a little harder than" / em "that" / ". There's hardly any of you to properly fuck."
		/ Paragraph / "Come back when you have a body that can satisfy me."
	]

sure :: Encounter Index () Nightmare
sure b n =
	[ dialog n $ "Well."
		/ Paragraph / case locoplan b of
			Basic Biped -> c "You can start by getting on your knees."
			Basic Naga -> c "You can start by getting on your belly."
			Basic Quadruped -> c "You can start by presenting yourself for mounting."
			Basic Centaur -> "You can start by presenting yourself for mounting."
				/ Paragraph / "Unless, of course, you want to use your mouth."
			Basic Slithering -> c "Let's see just how much you can swallow."
			x -> bodyFail Sure x
	, case locoplan b of
		Basic x | x `elem` [Biped, Naga, Slithering] -> Jump Kneel
		Basic Centaur -> Choices
			[ ("show your ass", Mounted)
			, ("use your mouth", Kneel)
			]
		Basic Quadruped -> Jump Mounted
		f -> bodyFail Sure f
	]

kneel :: Encounter Index () Nightmare
kneel b n =
	[ Narration2 $ case locoplan b of
			Basic Centaur -> "You step closer to the nightmare's giant body, and then let your legs buckle, sinking down so your lower body is lying on the ground. You slide a hand down the nightmare's side, its black coat smooth under your palm. You have to lean further down, reaching under its belly, your upper back sliding against the scratchy, stiff hairs of its underbelly."
				/ Paragraph / "Its balls are enormous, sliding against its inner flanks, and its sheath sways, drooping lower, loose flesh spreading as bloats wider. Muscles twtich across its underbelly, cock bulking its sheath before any part of it touches the open air."
			Basic x | x `elem` [Biped, Naga, Slithering] -> case x of
					Biped -> "You drop to your knees."
					Naga -> "You drop down to the ground, belly scraping across the dust."
					Slithering -> "You slither between the nightmare's forelegs."
					f -> bodyFail Sure f
				/ "From here, you can see between its flanks, across its glossy, short-furred underbelly. Its balls are enormous, sliding against its inner flanks, and its sheath sways back and forth"
				/ case x of
					Slithering -> "as you pull yourself erect, moving closer"
					_ -> ", loose flesh bunching and spreading apart with each step it makes towards you"
				/ "."
				/ Paragraph / "The meat of its cock shifts; muscles twitch across its underbelly, and its sheath bloats fatter, cock sliding down bulking its sheath before any part of it touches the open air."
			f -> bodyFail Sure f
		/ case locoplan b of
			Basic x | x `elem` [Slithering, Centaur] -> "Its bloated sheath bulges, sheath slowly parting to reveal the enormous platter of its flat cockhead, sluggishly cresting out from its bunched sheathflesh."
			_ -> "The nightmare stops just before you, head nearly between its front legs, its nose exhaling smoky breath in a haze around you. You stare at its sheath, loose folds of flesh parting enough to reveal the enormous platter of its flat cockhead, sluggishly cresting out from its bunched sheathflesh."
		/ case locoplan b of
			Basic Biped -> "You drop even lower, on all fours, crawling between the nightmare's legs to get you closer to its cock."
				/ hasDick ? ("Your own" / unsheathed / "cock bobs beneath you, sending out little frissions of pleasure when it hits your stomach.")
			Basic Naga -> "You squirm between the nightmare's legs to get closer"
				/ hasDick ? (", your own" / unsheathed / "cock digging painfully into the ground")
				/ "."
			Basic Centaur -> c "You open wide, leaning in, letting your tongue press across the tip, face-to-tip against it."
			Basic Slithering -> c "You open wide, leaning in, letting your tongue flit across the tip, pressed face-to-tip against it."
			f -> bodyFail Sure f
	, Jump $ Unsheaths UseMouth
	]

mounted :: Encounter Index () Nightmare
mounted b n =
	[ Narration2 $ "You turn around and spread your legs" / hasTail ? "lifting your tail out of the way" / ", so the nightmare has a good target for your ass. You clench, asshole pursing and then spreading, a frission of eager nervousness running up your spine. The rocky dust crunches under the nightmare's hooves as it approaches, and you look back, watching it step closer and closer."
	, Jump $ Unsheaths UseAss
	]

unsheaths :: FromCent -> Encounter Index () Nightmare
unsheaths fc b n =
	[ Narration2 $ "Its cock spills out of its wrinkled, leathery sheath as an arching curve of shining black flesh"
		/ case locoplan b of
			Basic Centaur | fc == UseMouth -> ", and you sway backwards with it, catching stray dribbles of pre across your" / muzzle
			Basic Slithering -> ", and you sway backwards with it, catching stray dribbles of pre across your" / muzzle
			_ -> mempty
		/ ". It hangs between its haunches, so long and heavy the tip presses against the ground."
		/ hasArms ? "It's massively thick, fatter than one of your arms, the flat-capped head just as thick as the shaft and nearly the size of your head."
			?: "It's thigh-thick, massively huge, and the flat-capped head is just as thick as the rest of the shaft, only defined by the gnarled ridge a few inches down the shaft."
		/ "A shuddering pulse goes through its length every second, slowly filling it out, bringing it to full hardness"
		/ case locoplan b of
			Basic Centaur | fc == UseMouth -> " -- pointing it straight at your awaiting face"
			Basic x | x `elem` [Biped, Naga] -> " -- pointing it straight at your awaiting face"
			_ -> mempty
		/ ". It's mottled near the tip, a silvery grey-black that dapples down the shaft nearly to its fat medial ring. Beneath that, its flesh is thicker, more leathery, and riddled with long corded veins that grow and swell larger towards the base, eventually vanishing into its swollen sheath."
		/ Paragraph / "There's a smear across the crown, orange-red dust pale against its flesh, slowly darkening as pre drools from the tip and spreads across its cockhead. The nightmare's underbelly shudders, muscles clenching, and its cock jerks up"
		/ case locoplan b of
			Basic Centaur | fc == UseMouth -> ", glancing across your front before it"
			Basic x | x `elem` [Biped, Naga] -> ", glancing across your front before it"
			_ -> "and"
		/ "slaps its stomach with a" / em "crack" / ", tip forcing out a glimmering string of pre from the impact before the shaft slings down, cockhead dragging across the ground again and picking up a thicker coating of dust."
	] <> case locoplan b of
			Basic Centaur -> case fc of
				UseMouth -> clean b n
				UseAss -> centQuadMounted b n
			Basic Quadruped -> centQuadMounted b n
			Basic x | x `elem` [Biped, Naga, Slithering] -> clean b n
			f -> bodyFail Sure f

clean :: Encounter Index () Nightmare
clean b n =
	[ dialog n $ c "Clean it,"
	, Narration2 $ "the nightmare says, and you"
		/ hasArms
			? "lean in, hands reaching out to wrap around its monolithic shaft, mashing its dense, rubbery cockhead against your face."
			?: "coil in closer, winding the bulk of your body between its forelegs as you flit your tongue across the dusty, slimy tip."
		/ "It's hot"
		/ hasArms
			? "to the touch"
			?: (case bodyMat_ Head b of
				Solid (FMat _ Flame) -> mempty -- lol
				Solid (FMat _ x) -> "against your" / matword x
				_ -> c "to the touch")
		/ ", pulsing under the surface, and it gives a strong lurch and spews a mess of hot, runny pre"
		/ case bodyMat_ Head b of
			Solid (FMat _ x) | x `elem` [Skin, Scales, Protoplasm] -> "all across your"
				/ case mouthType b of
					Mouth -> c "face, drooling down your chin"
					_ -> muzzle / ", drooling down your jaw"
				/ "and dripping to the ground below"
			Solid (FMat _ x) | x `elem` [Coat, Fur] -> "all across your" / muzzle b / ", catching in your" / matword x / "and slowly soaking down to your skin"
			_ -> c "all across your face"
		/ "."
		/ case bodySpecies Head b of
			Just Snake -> "You" / groan / " and lick across its cocktip, tongue tasting dry dust and rich, rank pre in equal measure. You swallow, drinking down a mess of dust and pre, and slowly lap at the dark smears of dirt, washed away slowly by your tongue and its constantly-flowing pre."
			_ -> "You" / groan / ", mouth flooded with pouring lines of pre, and lap across its dusty cockhead, gulping down a mess of dust and pre, licking over its flesh until it's unmarred, shining now with spit and pre."
	, case locoplan b of
		Basic Slithering -> Jump SuckOff
		Basic Centaur -> Jump SuckOff
		Basic x | x `elem` [Biped, Naga] -> Choices
			[ ("try to swallow it", SuckOff)
			, ("keep playing with it", KeepPlaying)
			]
		f -> bodyFail "clean" f
	]

-- no quadrupeds, but this is a branch allowed for bipeds, nagas, & centaurs, and the only branch for slithering
suckOff :: Encounter Index () Nightmare
suckOff b n =
	[ Narration2 $ "It's huge, but you think you can take it. The ridge of its cockhead shudders, pulsing with the nightmare's heartbeat, and you lick along the slimy crest of its cockhead, over the flat tip to the round, bulging cockslit. Pre splatters across your face and drools down"
		/ case locoplan b of
			Basic Slithering -> "your body"
			_ -> "your neck"
		/ "as you size up the nightmare's cock."
		/ Paragraph / case bodySpecies Head b of
			Just Snake -> c "You open wide, unhinging your jaw, and lurch forward, engulfing the nightmare's cockhead in a single motion. Your face distends, warping around the enormous swell, and you lurch forward again, gulping to settle its cock deeper down your throat. Its cockhead jabs against the back of your mouth, gushing pre down your throat, and you swallow again, throat straining as you force its cock deeper."
			_ -> "You run your" / case mouthType b of
					Mouth -> c "lips"
					_ -> c muzzle
				/ "over the knobbled rim, and open wide -- not enough to take it in, not yet -- and let part of the side slurp into your mouth. Its flesh trembles against your tongue, pre spurting out"
				/ case locoplan b of
					Basic Slithering -> "down your body"
					_ -> "over your shoulder"
				/ ", and you slide your lips across the slimy rim, open wide, wider, to try to take more in."
				/ Paragraph / "Your lips ache, bruised from the stretch as you keep trying to swallow the rim, breath billowing across the flat of its cockhead, threads of glistening slime webbing between its shining flesh and your lips. The bottom of the cockhead slurps into your mouth, bulging cockhole smearing across your nose before it slurps over your lips, spurting rank pre into your mouth as you snuffle across its cockhead, trying to wrap your lips all the way around."
				/ Paragraph / "The nightmare snorts, legs shifting above you, and it pushes forward, the rubbery flesh of its cockhead grinding against your face. The ridge along the edge scrapes over your lips, more of the bottom half of its cockhead pushing into your mouth, each fraction spreading your jaw wider and wider. You gurgle, throat flooded with pre, cockhead pulsing in your mouth, and the nightmare whinnies and rears up, hips slamming forward. Your jaw pops from its socket with a sting of pain, hanging open, and the nightmare plunges its cock into your mouth, slamming against the back of your throat before forcing itself deeper. You gag and gurgle, cords of slime oozing out around your straining lips, the massive pillar of its cock sunk into your throat, so much more of its shaft left out in the open. You go crosseyed staring at its medial ring."
		/ Paragraph / "The nightmare ruts forward, satiny flesh of its shaft spilling"
		/ case mouthType b of
			Mouth -> "over your lips"
			Muzzle -> "into your muzzle"
			Snout -> "into your open maw"
		/ "as it sinks its cock down your throat. Its medial ring smacks against your lips, halting its progress, and pulls back webbed with slime; a mess of spit and pre spills"
		/ case locoplan b of
			Basic Slithering -> "down across your body, and into your coils"
			_ -> "down your chin, coating your chest in hot runnels of slime"
		/ ". Then the nightmare pushes in again, more insistently: its medial ring slurps over your bruised lips, over your tongue, and shoves its way down your throat."
		/ Paragraph / "The nightmare's thrusts press your head between its flanks, the veiny flesh of the root of its shaft plunging into your mouth with a gurgle. You gag, spitting slime against its underbelly and sheath, and its thrusts smear it all across your fance, drenching you in layers of ooze, forming thick, flabby cords that hang from your chin and jiggle when it ruts forward, until its titanic balls smack against your chest and smear the cords of slime into a slick paste,"
		/ case bodyMat_ Head b of
			Solid (FMat _ Skin) -> c "drenching your skin"
			Solid (FMat _ Scales) -> c "sheening across your scales"
			Solid (FMat _ x) | x `elem` [Fur, Coat] -> "matting your" / matword x / "to your skin"
			_ -> c "coating your head in slime"
		/ "."
		/ Paragraph / "The nightmare holds you there, bracketed in by its hooves,"
		/ case locoplan b of
			Basic Slithering -> "your coils wound around its forelegs, its underbelly shifting above you"
			_ -> "belly scraping across your back"
		/ "as its hips rock, plunging to the hilt into your broken mouth and then pulling back, ripping its medial ring up through your throat until it pops out into the back of your mouth, covered in thick, slick phlegm, all mingled together with its rank, spicy pre. Its cockhead shudders"
		/ case locoplan b of
			Basic Slithering -> "down the length of your body"
			_ -> "in your stomach"
		/ ", beating like a second heart, gushing out spurts of pre that slosh and gurgle within you, filling your stomach to aching."
		/ hasLocotype Slithering ? "Your snake-body is warped over its cock, sheathing its flesh, flesh drawn so tight in places its gnarled veins, or medial ring, are barely visible through your skin."
		/ Paragraph / "The only sign the nightmare gives of getting close is a slight increase in breath, its snorting exhales billowing out clouds of steam. And then, suddenly, when it ruts forward again, grinding your face against its sheath, its flare shudders, flexing hard and expanding. The flesh of your throat warps -- more -- and bulges, the crest of its flare"
		/ case locoplan b of
			Basic Slithering -> "bloating your body"
			_ -> "inside your ribcage, its head digging against the walls of your stomach"
		/ "as it erupts in heavy, hot ropes of come. All you can do is gurgle, come piling up inside you, all trapped behind the nightmare's flared cockhead. Gush after gush spurts into your stomach, forming a dense weight, bloating out your stomach after only a few gushes. And the nightmare's orgasm continues, body unmoving, cock spurting again and again until your gut sags down beneath you,"
		/ case bodyMat_ Chest b of
			Solid (FMat _ Scales) -> "scales"
			Solid (FMat _ x) | x `elem` [Skin, Fur, Coat] -> "skin"
			_ -> "flesh"
		/ "drawn taut, aching with the weight of it."
		/ Paragraph / "The nightmare appears content to keep going, cock shooting unendingly inside you, but eventually its spurts slow and then subside, flare softening until it relaxes back against the beast's shaft. Your overtaxed throat spasms, come racing up along the softening shaft, and spews from your mouth in a thick, sloppy slurry as the nightmare slowly withdraws, shaft slurping from your mouth amid a waterfall of its own come. Slime paints across its underbelly and shaft, drooling down its heavy balls, subsiding for a moment as its medial ring rams against the inside of your lips, before it yanks free with a fresh explosion of come. Its cock pulls free with a" / em "slurp" / ", flat tip dragging against your" / teeth / "for a moment before it pulls free, smacking against"
		/ case mouthType b of
			Mouth -> c "your cheek"
			_ -> "the side of your" / muzzle
		/ "before flopping, half-hard and completely slathered in come, down between its haunches."
		/ Paragraph / "You gurgle, head hanging down, wetly gagging as you cough up thick globs of come, pouring from"
		/ case mouthType b of
			Mouth -> "your mouth and nose"
			Muzzle -> "your open muzzle and nose"
			Snout -> "your open mouth"
		/ "in a sloppy stream. You cough, each swallow bringing up more come, with your bloated stomach churning from the sheer volume."
	] <> nightmareCloser b n

keepPlaying :: Encounter Index () Nightmare
keepPlaying b n =
	[ Narration2 $ "Face-to-face with its cockhead, it's far too huge to fit in your mouth, so instead you lap across the flat head, hands stroking around the rim. The nightmare snorts above you, and its cock shudders, lurching so strongly it rips itself from your loose grip and smacks across your face, rebounding back to drape heavily over your shoulder, drooling pre in uneven bursts down your back."
		/ Paragraph / "You press your face to the side of its shaft, just under the ridge of its cockhead, to the lumpy, gnarled mess of muscle that will -- soon -- pull its cockhead out into a giant flare. It smells, spicy and rich, and you mash your face against it, pre coating your skin until it's all you can smell. You lap across its flesh, catching fat dollops of pre drooling down the underside of its weighty shaft, and gulp in down in hungry swallows. Your grind your face down, open-mouthed and groaning, wetly snuffling across its pre-streaked flesh."
		/ Paragraph / "You wrap your hands as far around the nightmare's cock as you can manage, stroking its thick, heavy flesh, like soft leather. Its skin dimples minutely under your" / fingers / ", and you can feel the internal muscles spasming under your" / fingertips / "a moment before the latest gush of pre splatters all across your jaw and neck, drooling down your chest in messy streamers."
		/ Paragraph / "The nightmare takes a step forward, hooves shifting beside you,"
		/ case locoplan b of
			Basic Biped -> "and you bowl backwards, moving with its cock, ending up on your back"
			Basic Naga -> "and you slide back with the motion, rolling over onto your back with your tail curled around the nightmare's hooves"
			f -> bodyFail "clean" f
		/ ", staring up at the mammoth cock. The flare shudders, and pre gushes out past your head, ending with a dribble of thick, slimy dregs that spatter down all across your face and chest. You groan, wrapping around its fat cock with both arms, dragging it down fractionally so that its gushes of pre splatter across your" / muzzle / ", hot and liquid and reeking of musk. You kiss its cockhead, tongue dragging across the bulging circle of its cockslit. It gushes into your mouth, making you sputter and gasp, the spray"
		/ case bodyMat_ Head b of
			Solid (FMat _ x) | x `elem` [Skin, Scales] -> "splattering all across your" / matword x
				| x `elem` [Fur, Coat] -> "enough to soak your" / matword x / "to your skin as it pours from your open" / muzzle / "in slimy streams"
			Solid (FMat _ Flame) -> c "hissing and spitting as it crackles across your fiery body"
			_ -> c "splattering all over your face"
		/ "."
	, Choices
		[ ("finger cockslit", FingerCockslit)
		, ("frot shaft", FrotShaft)
		]
	]

fingerCockslit :: Encounter Index () Nightmare
fingerCockslit b n =
	[ Narration2 $ "You open wide, tongue sliding over the bulge of its round cockhole, testing the rubbery flesh. Its hole shudders and opens with each pulse of pre -- spraying into your mouth and instantly overflowing it, tastebuds saturated with the spicy taste -- and when you push your tongue harder it shoves into the hole itself, probing against the inner flesh of the nightmare's cock."
		/ Paragraph / "The nightmare whinnies, cock shuddering, and the next gush of pre sprays out around your tongue, drenching your face in runny cords. You grasp the ridge of its cockhead with"
		/ armCountMatching 4
			? "four hands"
			?: "both hands"
		/ ", fingers curled over the fat rim, thumbs spread across the crown, digging in to the dense, rubbery flesh"
		/ armCountMatching 6 ? ", using the rest to rub across its shaft"
		/ ". The nightmare's cockhole pouts, circular bulge spreading into an oval, the dimpling, shuddering opening itself splaying wide in a perpetual spread, revealing an inch or two of its inner canal, constantly streaming a drizzle of pre. Its cock shudders again, inner muscles pulsing, and you get to see the passage spread itself, a lightly-curving corridor visible for a moment before the latest gush of pre sprays out all across your face."
		/ Paragraph / "You catch some of the mess of pre on your fingers, and slide them closer, over its bulging cocktip. Its flesh is slick, slippery, and with your fingers dripping with its preseed the contact is nearly frictionless. Your thumbtip digs into the side of its cockhole, animal muscles fluttering under your touch, and your thumb slips inside neatly. The nightmare's cockhole bulges, pre spilling out around your thumb, backing up minutely, and you slide"
		/ armCountMatching 4
			? "another"
			?: "your other"
		/ "hand across its flat cockhead, notching"
		/ armCountMatching 4
			? "your"
			?: "your other"
		/ "thumb in place beside the first. The nightmare's cock is so huge even that goes in easy, the backed-up pre erupting out in an enormous splattering gush when you shove"
		/ armCountMatching 4 ? "your" ?: "your other"
		/ "thumb in and then peel its cockslit wide; your thumbs dig into opposite sides of its walls and spread it into a massive open gape."
		/ Paragraph / "The flesh inside is smooth and glossy, slightly rumpled near the tip where you're playing with it, and its black flesh has a purple tinge, rawer along its inner flesh. The nightmare's flesh is dense, heavy, clenching back against your thumbs, but it's also" / em "rubbery" / ", stretching more as you dig in harder, until its cockhole is spread into an enormous open canal as wide as a splayed hand, with a constant streamer of pre pouring down the middle in pulses, splattering in rank waves all across your chest."
		/ Paragraph / "You smear"
		/ armCountMatching 4 ? "one of your free hands" ?: "your left hand"
		/ "across its cockhead, index finger digging into its mounded cockhole for a moment before you curl it over the ridge and reach in, sliding finger after finger into the nightmare's cock until they're all sliding along its inner flesh, palm rolling back and forth over the ridge at the tip. You push your entire hand into the nightmare's cock, easing your other hand back only to get some resistance, letting its gaping tunnel collapse into something that fits snugly around your hand as you shove it in deeper, plunging your hand, wrist, forearm into the nightmare's cock."
		/ case bodyMat_ Arms b of
			Solid (FMat _ x) | x `elem` [Skin, Scales] -> "Its rubbery inner flesh slides across your" / matword x / ", pulsing, slick and gushing with pre as you stroke your hand back and forth within its shaft."
				| x `elem` [Fur, Coat] -> "Your" / matword x / "drags across its skin, damp and sodden. You grind your hand in further, hair pressing flat to your skin when you sink in, only to catch and rasp across its inner walls when you pull back, adding heat and friction to the sloppy mess."
			_ -> c "Its rubbery inner flesh slides across your hand as you stroke back and forth within its shaft."
		/ "You feel across its shaft with your other"
		/ armCountMatching 4 ? "hands" ?: "hand"
		/ ", digging in -- you can feel the pressure against your hand within the shaft, and the bulge it makes along the tip of the nightmare's shaft, but its cock is so titanic already that an arm shoved halfway in is hardly visible to the eye."
		/ Paragraph / "You pump your hand back and forth, stroking it off from the inside. You ball your fist, letting pre build up behind it, spurting through your knuckles, and then open your hand, letting pre erupt out across your front in enormous, watery gushes, and then you do it again and again, plunging your arm deeper into its monstrously-huge shaft each time, until with a wet" / em "smack" / "your shoulder rams against the distended tip, your entire arm buried in its shaft, and still only just-barely reaching to where its shaft vanishes into its sheath. Its cock pulses around you, muscle tremors hardly felt across its bare flesh picked up perfectly with your arm buried inside it."
		/ Paragraph / "The nightmare ruts forward, mashing its cockhead against your shoulder, and pulls back, its already-bulging cocklips bloating even further for a moment before its cock slides off your arm. You punch-fuck its cock, ramming in and pulling back, having to brace"
		/ armCountMatching 4 ? "an opposing" ?: "your other"
		/ "arm against the nightmare's body to keep at it"
		/ armCountMatching 4 ? ", using your other hands to keep stroking its shaft, playing with the rim of its flare"
		/ ". The nightmare whinnies, hips working, and its cock pulses around your arm -- at first rippling, and then stronger and stronger clenches. A knot of muscle spasms around your bicep, forming a thick, clenching ring, and its flare snaps open. Its cock pulses, shooting -- and backing up against your fist, the initial blast pressurized into a painfully sharp eruption of come squirting out around your arm, lancing across your front, and then the rest of the spurt sluggishly drooling down along the length of your arm, spilling down its flare. Its cock shudders again, further backing it up, come spraying in all directions, a sloppy reservioir building up near the base of its cock, hot and liquid immersing your hand. The nightmare shoots again and again, come constantly gushing from its tip, pressure rising and falling in time with its shots. You work your arm through the tight vise of its flaring muscles, each push forward milking out a sloppy gush of backed-up come, followed by a hollow, gurgling sucking noise when you pull back out. Come pours across your body, churned up by your pumping arm into a frothing, foamy mess."
	] <> nightmareOrgasm FromSound b n

frotShaft :: Encounter Index () Nightmare
frotShaft b n =
	[ Narration2 $ "You pump its shaft in your arms,"
		/ armCountMatching 4 ? "needing four hands to wrap all around its titanic girth"
			?: "too fat to even fit both hands around"
		/ ", and arch your body up to press against the dripping, drooling underside of its shaft"
		/ hasDick ? (", your untouched" / Cock^s / "sliding against it, making you moan in pleasure as you rut against the nightmare's titanic cock")
		/ "."
		/ Paragraph / "You rut against the nightmare's cock with your entire body, face pressed against the tip, drenched in runny slime, arms pumping the shaft"
		/ hasDick ? (case cocksizeO b of
				Just x -> ", your own" / partAdjective Cock / Cock^s / (case x of
					Small'1_4 -> c "grinding against its sheath, at the very base"
					Normal'5_7 -> c "sliding against the root of its vein-gnarled shaft"
					Large'8_12 -> "rutting against the base," / Tip^s / "sliding up to grind against the fat, rubbery ring of its medial ring"
					Huge'13_18 -> "slopping against its shaft, " / Tip^s / "dragging over the fat, rubbery ring of its medial ring"
					Extreme'19_25 -> "nearly matching its own monstrous shaft," / partAdjective Cockhead / Tip^s / "sliding along its slimy length, squirting pre across its cockhead and adding to the sloppy drool pouring back across your chest"
					Gigantic'26_39 -> "just as huge as its own length, their combined weight nearly pinning you to the ground as you rut your cocks together, your" / partAdjective Cockhead / Tip^s / "smacking against the ridge of the nightmare's cockhead."
					_ -> c "overwhelming even its own monstrous shaft, pinning you to the ground under the combined bulk of your gigantic cocks")
				f -> bodyFail "frotShaft" f)
		/ ". Its cock pulses, pulling up against the nightmare's chest, the motion so strong that it nearly rips itself from your grasp each time."
		/ Paragraph / "The nightmare's breath comes faster, clouds of steam billowing from its nostrils in two sharp lines, and it ruts against you, thrusting its cock through your sloppy, dripping grasp. You slide your hands back, stroking the gnarled, veiny base beneath its medial ring, squeezing the spongy ring itself, sliding your hands all the way up to the thick crown, fingers dug into the gristly rim, stroking its cock base to tip as you writhe beneath it, body smeared in goopy pre, dust churning into thick, sludgy mud beneath you."
		-- maybe put pc orgasm here
		/ Paragraph / "The ripple of its orgasm races through its cock, muscles spasming under the surface. Its underbelly shudders, cock bobbing up and down, pulsing once, twice, before the third tremor comes with a fountaining spurt of thick come, spraying all across your face and chest in a hot stream, flooding your open mouth with a gurgle. Its flare spreads, turning its enormous cockhead into something ludicrous, an enormous platter of slick flesh, raised rim gnarled and gristly, surface fluttering back and forth with each gush of come. The nightmare snorts and whinnies, panting now, pawing at the ground with one flaming hoof as it hoses you down with pulse after pulse of come."
	] <> nightmareOrgasm FromHJ b n

nightmareOrgasm :: From -> Encounter Index () Nightmare
nightmareOrgasm from b n =
	[ Narration2 $ "Its load is thick, squelching and slimy as it oozes down your body, but there's so much of it that it simply pours down to the ground in liquid streams. You groan, rutting against its shaft, your strokes making a wet," / em "slorp" / "-ing noise as you"
		/ case from of
			FromHJ -> "pump along"
			FromSound -> "piston inside"
		/ "its drenched shaft, its flesh slick and frictionless under your fingers."
		/ case from of
			FromHJ -> "You drag its shaft down lower, aiming the spraying cockhead all across your chest -- a sharp blast knocks the breath from your lungs, and a fat web of come erupts out in all directions from the splash."
			FromSound -> "You twist your arm back and forth inside its shaft, come gurgling down inside, bursting out in spurts and gushes."
		/ "Fat droplets"
		/ case bodyMat_ Chest b of
			Solid (FMat _ Skin) -> c "splatter all across your skin, pinprick specks of heat on the rare parts of your skin that aren't already entirely soaked"
			Solid (FMat _ Scales) -> c "splatter all across your scales, their heat soaking down into your flesh"
			Solid (FMat _ x) | x `elem` [Coat, Fur] -> "splatter all across your sodden" / matword x / ", sticking it solidly to your skin"
			_ -> c "splatter all across your chest"
		/ "."
		/ Paragraph / "You keep pumping its shaft, writhing in the slimy mess of come until you're entirely soaked. The nightmare's pulses subside, hosing squirts turning into a long, continual drool, thicker ooze"
		/ case from of
			FromHJ -> "draining out from its mammoth cock as you massage along the underside, wringing out as much come as possible,"
			FromSound -> "clinging to your arm and making its way out in thick, rubbery globs, drooling from your shoulder down, and"
		/ "smearing all over your sodden body."
		/ Paragraph / "The nightmare's cock softens, slowly, and you collapse to the ground, sinking into the churned-up morass of come beneath you,"
		/ case from of
			FromHJ -> "the nightmare's shaft a heavy weight across your chest, still sluggishly oozing come. You lap at the tip, catching a mouthful of thick, tarry dregs, before the nightmare pulls away -- cock smearing all across your disheveled body before it finally pulls away entirely, leaving you half-immersed in the puddle of its spilled load."
			FromSound -> "the dense flesh wrapped taut around your arm slowly becoming more pliant as its shaft droops, until it's only kept straight by your arm shoved inside it. You pull your arm out, slowly, letting backed-up come spurt from the tips in sloppy bursts until finally you pull your wrist free from the clenching tip and the shaft smacks down across your chest, the flushed, gaping tip still slowly oozing come."
		/ Paragraph / "You sluggishly pull yourself upright, wiping come from your eyes, more come sluicing down your chest, back a mess of thick, slimy mud. The nightmare is beside you, looking about as smug as a horse can look. Its cock is still unsheathed, hanging limp between its flanks, and still drooling from the tip."
		/ case locoplan b of
			Basic Biped -> "You push yourself to your feet, and take a few tottering steps, come streaming down your body and soaking the dust under your feet."
			Basic Naga -> "You slide out from the sodden pile of come, tail leaving a furrowed trail of come-drenched dirt behind you."
			f -> bodyFail "frotShaft" f
	] <> nightmareCloser b n

centQuadMounted :: Encounter Index () Nightmare
centQuadMounted b n =
	[ Narration2 $ "The nightmare walks closer, cock slinging lines of pre from its crown as it bobs up and down, until the nightmare is so close you can feel its breath across your ass."
		/ Paragraph / "It rears up, hooves milling at the air, letting out a trumpeting whinny as it lurches forward, weight crashing down across your back. Its cock feels like a burning brand, mashed against your flanks, and the nightmare ruts, breath bursting in streams across your back, trying to mash its cock into place."
		/ case bodyMat__ Ass b of
			Solid (FMat _ x) | x `elem` [Fur, Coat] -> "The fat, flat head knocks against your ass, skidding up through your" / matword x / "and smearing it with pre."
			_ -> c "The fat, flat head knocks against your ass, slimy with a smearing of pre."
		/ "You can feel each shift and shudder ripple through its length, the rubbery flesh trembling against your skin. A sloppy gush of pre erupts from the tip, where it's mashed against your skin, and it explodes out in all directions, squirting up across your body and giving the nightmare's weighty cock the slickness to smear to the side, notching in the cleft of your asshole with a" / em "slurp" / ". Its heat soaks into you, tip nestling up and down across your asshole, before it slams forward again and plunges its cock inside, slamming forward with brutally powerful thrusts once, twice, three times, before its flanks meet your ass, its weighty balls"
		/ hasExternalBalls ? "slapping against your own"
			?: "smashing against your haunches"
		/ ", the entire length of its shaft sunk into your guts. Its sloppy sheathflesh grinds against your burning asshole, slopping back and forth in wrinkled folds as it shifts its weight. Your body shudders, asshole spasming around the fat length, guts shuddering and burning from the rough entry."
		/ Paragraph / "The nightmare's body bears down on you, its stomach pressed against your back, its forelegs wrapped around your body, digging into your sides. Its hips shift, letting the root of its cock spill out from your still-spasming asshole before it ruts forward, slamming it back inside. It snorts, steam billowing from its nostrils all across your back."
		/ Paragraph / "Its medial ring, not felt at all on the initial plunge, throbs deep inside you, feeling impossibly huge. It catches on your guts, making them lurch and twist with each thrust. The fat, flat crown batters against internal walls deep inside you"
		/ case locoplan b of
			Basic Quadruped -> ", knocking your breath from your lungs when the nightmare pushes yet deeper"
			Basic Centaur -> ""
			f -> bodyFail "centQuadMounted" f
		/ ". Its hooves grappling your sides, hind legs clomping and shifting on the rocky ground as it spreads its legs, getting more leverage for its next thrust -- the implacable force of its initial thrust nothing compared to the force it brings to the next thrust: each thrust plunges its cock to the root inside your body, sheath smacking the rim of your spread hole, lurid squelches coming from inside you as its thrusts churn up the mess of pre pouring down your inner walls."
		/ Paragraph / "Your body bows forward from the force of it,"
		/ case footType b of
			Hooves -> "and your hooves scrape against the rocky ground"
			ClovenHooves -> "and your hooves scrape against the rocky ground"
			Paws -> "paws gripping on the rocky ground"
			Talons -> "talons digging into the rocky ground"
			Pseudopods -> "pseudopods slurping forward across the dusty ground"
			Feet -> "feet gripping against the rocky ground"
		/ "as you struggle to stay upright against its punishing thrusts. You spread your legs to take the brunt of it, rutting back to take each thrust, the nightmare's cock plunging lewdly into your spread ass with a wet" / em "crack"/ "each time it hilts, its fat balls"
		/ hasExternalBalls ? "smashing against your own with a frission of pain each time"
			?: "smacking against your flanks each time"
		/ ". Pre gurgles in your guts, gushing inside you so deep you can't feel it directly, just a growing weight and heat. Spurts sloshing down your guts to coat the nightmare's shaft and ease its brutal thrusts, building up against your spasming, aching asshole. The nightmare makes another hard thrust, dragging your asshole out, and its pre erupts out from your already-flooded ass in an sloppy gush, splattering down your flanks in runny lines, gushing from your gaped asshole. Then its angle shifts minutely, and your asshoel clenches tight, back to pre slowly building up inside you in rippling, sloshing streams."
		/ case locoplan b of
			Basic Centaur -> Paragraph / "Its head butts against your upper back, and you twist back, one arm slung over its shoulders. It raises its head, eyes frenzied, muzzle open and drooling across your side. It lurches forward and up, nose sliding across your face, followed by its long, slick tongue"
				/ case bodyMat_ Head b of
					Solid (FMat _ Skin) -> c "dragging over your skin"
					Solid (FMat _ Scales) -> c "dragging the wrong way across your scales"
					Solid (FMat _ x) | x `elem` [Fur, Coat] -> "soaking your" / matword x / "to your skin across your" / muzzle
					_ -> mempty
				/ ", shoving its way into your panting mouth, down your throat. Its hot breath envelops your head as it huffs and pants, sloppily kissing you as it pummels your guts."
			_ -> mempty
		/ hasDick ? (Paragraph / "Heat blossoms in your guts, sparking through to"
			/ hasBalls ? "your balls" ?: "your cock"
			/ "as it keeps plunging inside you."
			/ hasProstate ? ("Its thrusts slide across your prostate, stabbing deeper inside you, and each one sends a frission of pleasure through you, making your cock drool pre in thick lines, oozing down between your" / feet / ".")
			/ "An internal muscle spasms, not like an orgasm but not unlike it either, and your cock gushes a single spurt of come, splattering against your forelegs. Its thrusts grind against internal walls inside you, making obscene slurping noises as it grinds its sheath against your ass, and your cock spurts again, painting the dusty ground in another rope of your come. Every few thrusts its cockhead smacks against some trembling, flexing muscle, and your cock lets out an accompanying spurt of come, again and again as you rut back into its thrusts, until the ground below you is soaked in come, more spurting from your cock in uneven gushes as the nightmare fucks it out of you."
			)
		/ Paragraph / "Its thrusts get sharper and even shorter, hardly a sliver of its shaft pulling from your ass, pushing its cock deep-and-deeper in your ass, sheath peeling back against the absolute root of its shaft, balls heavy cannonballs draped over your flanks and jostling. It snorts"
		/ hasLocotype Centaur ? "across your face, tongue drooling down your throat"
		/ "and jerks forward, hips flexing, entire body lurching, as its cock twitches hard inside you."
		/ Paragraph / "The rim of its cockhead spreads in shudders for a moment, testing your insides, before its entire cock jerks upward and its flare snaps open. Muscles beneath the tip clench hard and spread its cockhead into a broad, brutal flare, digging hard into the walls of your guts. It erupts in immense geysers, spurts gurgling through your guts as it comes in gush after gush, filling you with its issue, all plugged inside you from the aching spread of its flare. Your guts spasm, pried open, stinging with each gush. Its issue builds up inside you, in a churning sea of heavy, hot fluid, slopping and cracking against itself in waves, building up and up until it spills over into some deeper passage, gurgling deeper inside you in hot waves. The nightmare's flanks tremble atop you, cock shuddering with the force of its eruption, pumping pulse after pulse of come into a reservoir inside you that grows into a hot, aching weight bloating out"
		/ hasLocotype Centaur ? "your lower body"
			?: "your gut"
		/ "in a heavy, pregnant-looking gut"
		/ case bodyMat__ Underbelly b of
			Solid (FMat _ m) -> case m of
				Coat -> ", fur drawn sparse over your widely-stretched skin"
				Fur -> ", fur drawn sparse over your widely-stretched skin"
				Skin -> ", skin widely seamed in ragged stretch marks"
				Scales -> "with your scales gapped apart, revealing stretched skin between them"
				_ -> mempty
			_ -> mempty
		/ "."
		/ hasDick ? "Your cock skews out along the taut curve of your stomach, digging into your bloated flesh. The tip constantly drizzles a slimy mixture of come and pre onto the ground below."
		/ Paragraph / "You pant, gasping for breath"
		/ hasLocotype Centaur ? "around the nightmare's squirming tongue, still shoved down your throat"
		/ ", entire body shuddering and quaking from the prolonged force of the nightmare's climax. Come thunders into you, sloshing and slurping in your grotesquely-bloated guts, building up more and more behind the shuddering plug of its flare. Your guts warp, spreading minutely more, and a squirt of overheated come erupts back over the lip of its flare, sluicing down your ass in a hot, burning line. You" / groan / ", guts convulsing again, another gush of come spraying out -- its load too huge to even be contained behind its immense flare. Come leaks out, following the slimy, slickened paths of prior squirts, until each new shot into your guts is met with a sharp backwards pulse, drooling back out. Streamers of frothing, bubbly come pour from your gaping asshole, squelching between your tightly-pressed-together bodies. Come drools from your flanks"
		/ hasExternalBalls ? ", streaming down your balls to drip in stringy lines from their undersides"
		/ ", coating your" / feet / "and turning the dusty ground beneath you into dense, slimy mud as the nightmare's unending orgasm fills you to overflowing."
		/ Paragraph / "Eventually the nightmare's pulses stop, and its flare shudders again before sluggishly peeling away from your brutalized anal walls. The instant it does, the entire mess of backed-up come floods down, battering against your broken asshole in a wave, turning the spurts of come drooling from your ass into a gushing waterfall. The nightmare pulls out, cock sliding free in the midst of the torrent of come, leaving you tottering on your" / feet / ", legs buckling, asshole a ruined wreck, rim bruised into rubbery slabs, hole gaping open. Your guts gurgle as the nightmare's load starts draining out of your used body."
		/ Paragraph / "The nightmare stands next to you, its half-hard cock slathered in come, streaming down its shaft and drooling to the ground in syrupy strands."
	] <> nightmareCloser b n

nightmareCloser :: Encounter Index () Nightmare
nightmareCloser b n =
	[ dialog n $ "That was good enough for a first try,"
	, Narration2 $ c "the nightmare says, to your disheveled and dripping body."
	]
