{-# LANGUAGE MultiParamTypeClasses #-}

module Sites.Hell.Events.Hubur.OldBones
	( OldBones(..)
	, deck
	, scene

	, name
	, bits
	, variants
	) where

import Sites.Hell.Events.SceneUtils hiding (Material(..))
import Server.JSON

import Sites.Hell.Events.Hubur

name :: String
name = "oldbones"

data OldBones = Bone | Ash | AshBlossom
	deriving (Eq, Ord, Enum, Bounded, Show, Read)

instance Random OldBones where
	randomR _ = random
	random = (`runRand` genRandom)

instance ToJSON OldBones where
	toJSON d = object
		[ pack "name" .= name
		, pack "data" .= [fromEnum d]
		]

bits :: Value -> Value
bits _ = object
	[ pack "name" .= name
	, pack "data" .= [Null]
	]

variants :: [(MapZone, ())]
variants =
	[ (Badland, ())
	]

instance FromJSON OldBones where
	parseJSON (Object vs) = case (parse (.: pack "name") vs, parse (.: pack "data") vs) of
			(Success pName, Success [d])
				| pName == name ->
				case safeToEnum d of
						Nothing -> mzero
						Just h -> return h
			_ -> mzero
	parseJSON _ = mzero

deck :: RandomGen g => Rand g (Enumerable OldBones)
deck = do
	opts <- liftRand $ shuffle [Bone, Bone, Ash, AshBlossom]
	return $ from opts

instance Scene () () OldBones where
	link _ = error "link"
	display = fmap toLower . show
	vardisplay = fmap toLower . show

scene :: Encounter () () OldBones
scene _ d = case d of
	Bone ->
		[ Narration
			[ "In the midst of the blasted wasteland, there's a tangle of old bones, slowly crumbling to dust."
			]
		, ItemModify
			[I DEMONS_BONE 2]
		, DeactivateSelf
		]
	Ash ->
		[ Narration
			[ "In the midst of the blasted wasteland, there's a tangle of bones, left exposed for eons. They crumble apart at your touch."
			]
		, ItemModify
			[I ASH 2]
		, DeactivateSelf
		]
	AshBlossom->
		[ Narration
			["You come across a thicket of dark, spiky vines growing through a dessicated corpse. Its flowers are in bloom, dark and dusty, and you pluck two."]
		, ItemModify
			[I ASH_BLOSSOM 2]
		, DeactivateSelf
		]
