{-# LANGUAGE MultiParamTypeClasses #-}

module Sites.Hell.Events.Hubur.Opening
	( Opening
	, scene
	) where

import Sites.Hell.Events.SceneUtils

data Opening = Opening

instance Random Opening where
	randomR _ = random
	random = getRand $ pure Opening

instance ToJSON Opening where
	toJSON Opening = object
		[ (pack "name") .= "opening"
		, (pack "data") .= ([] :: [Int])
		]

instance FromJSON Opening where
	parseJSON (Object vs) = case (parse (.: pack "name") vs, parse (.: pack "data") vs :: Result [Int]) of
			(Success name, Success [])
				| name == "opening" -> return Opening
			_ -> mzero
	parseJSON _ = mzero

data Index = ChoicePrompt | Where | Who | What | Now
	deriving (Eq, Ord, Enum, Bounded, Show, Read)

data Vars = BirdName | AskedWhere | AskedWho | AskedWhat
	deriving (Eq, Ord, Enum, Bounded, Show, Read)

instance Scene Index Vars Opening where
	link f = case f of
		ChoicePrompt -> choicePrompt
		Where -> where_
		Who -> who
		What -> what
		Now -> now
	display = fmap toLower . show
	vardisplay = fmap toLower . show

scene :: Encounter Index Vars Opening
scene = opening

opening :: Encounter Index Vars Opening
opening _ _ =
	[ ItemModify [I RIVER_WATER 99]
	, Narration2 $ open
		"Vision returns slowly."
		// Paragraph // "You wake on a floodplain: black water and thorny underbrush, and a pulsing red sky above."
		// Paragraph // "It's impossible to tell how much time passes as you pull yourself together."
		// Paragraph // "Eventually, there's a sound from above: flapping. Something like a bird descends. A bird-shaped tear in space, where the space revealed behind it is buzzing static. Its voice, when it speaks, is a rhythmic modulation of noise, barely comprehensible."
	, SetFlag (Local BirdName) $ Set (VS "Bird?")
	, Dialog3 "/static/img/icon.png" (include BirdName) $ open
		"Well, if it isn't a little lost soul!"
	, Narration2 $ open
		"Its feathers erupt in a puff of static as it folds its wings beneath it and hops closer."
	, Dialog3 "/static/img/icon.png" (include BirdName) $ open
		"You could get in a lot of trouble around here, you know. All sorts of undesirables lurking about, just waiting to snatch you up."
	, Jump ChoicePrompt
	]

choicePrompt :: Encounter Index Vars Opening
choicePrompt _ _ =
	[ Choices2
		[ ("where am i?", Where, Just $ C (Local AskedWhere) NotSet)
		, ("what happened to me?", What, Just $ C (Local AskedWhat) NotSet)
		, ("who are you?", Who, Just $ C (Local AskedWho) NotSet)
		-- can only leave after naming yrself
		, ("so...? what now?", Now, Just $ C (Local AskedWhat) IsSet)
		]
	]

where_ :: Encounter Index Vars Opening
where_ _ _ =
	[ SetFlag (Local AskedWhere) 1
	, Dialog3 "/static/img/icon.png" (include BirdName) $ open
		"Oh? Hasn't it caught on? You're in Hell! Hell, the terminus of the world, where all souls wash up eventually. Hell, where the dead dwell for eternity. The ending, the land of no escape."
	, Dialog3 "/static/img/icon.png" (include BirdName) $ open
		"You, specifically: you've washed up on the shores of mother Hubur, the river that flows into Hell. You're on the flood plain now, not that far from the ruins of the old fortress Ganzer."
		// Paragraph // "Not that that's particularly useful for you to know."
	, Jump ChoicePrompt
	]

what :: Encounter Index Vars Opening
what _ _ =
	[ SetFlag (Local AskedWhat) 1
	, Dialog3 "/static/img/icon.png" (include BirdName) $ open
		"Why, you're dead! Shuffled off the mortal coil, given up the ghost, etc. Given the living world its last, and headed out into the rest of Creation."
		// Paragraph // "You'd think there'd be a psychopomp around to give you the tour, but the best you'll get is me, I'm afraid."
	, Dialog3 "/static/img/icon.png" (include BirdName) $ open
		"Don't worry about the memory loss -- all your thoughts were in your old brain, and that's something you've no longer got with you. It doesn't matter here, anyway. If you were a pauper or a prince, a slave or a master... in death all souls are the same, and judged accordingly."
	, Dialog3 "/static/img/icon.png" (include BirdName) $ open
		"But you might need to think up one thing: the Authority is positively perversely obsessed with names. Names for this, names for that. Names to place things into the world, or cast them out into the darkness."
	-- NAME BOX HERE
	, Narration2 $ open
		"__nameboxprompt"
	-- NAME BOX COMMIT CALLBACK HERE
	, FauxChoice "__nameboxcallback"
	, Dialog3 "/static/img/icon.png" (include BirdName) $ open
		"That's as good a name as one could expect, I suppose."
	, Jump ChoicePrompt
	]

who :: Encounter Index Vars Opening
who _ _ =
	[ SetFlag (Local AskedWho) 1
	, InlineBranch (C (Local AskedWhat) IsSet)
		[ Dialog3 "/static/img/icon.png" (include BirdName) $ open
			"What did I just say? There are advantages to being nameless, here."
		]
		[ Dialog3 "/static/img/icon.png" (include BirdName) $ open
			"What an unwise question. It pays to be nameless here."
		]
	, SetFlag (Local BirdName) $ Set (VS "L?")
	, Dialog3 "/static/img/icon.png" (include BirdName) $ open
		"If you must, you can call me" // em "L" // "."
	, Jump ChoicePrompt
	]

now :: Encounter Index Vars Opening
now _ _ =
	[ Dialog3 "/static/img/icon.png" (include BirdName) $ open
		"So! Well," // em "I" // "have certain errands I ought to be running, and" // em "you" // ", presumably, are about to go off and get used to your eternity yearning for salvation."
		// Paragraph // "Perhaps we'll meet again. But likely not."
		// Paragraph // "Fare thee well."
	, Narration2 $ open
		"The bird hops and flaps its wings, taking aloft. It vanishes into the sky, and the din of its presence fades."
		// Paragraph // "So..."
	, FauxChoice "nothing to do but go"
	, Narration2 $ open
		"You set out."
	]
