{-# LANGUAGE MultiParamTypeClasses #-}

module Sites.Hell.Events.Hubur.RandomDrop
	( RandomDrop(..)
	, deck
	, scene
	) where

import Sites.Hell.Events.SceneUtils hiding (Material(..))

{-
	| LODESTONE
	| ORRERY_MECHANISM

	| RIVER_WATER
	| ETERNAL_FLAME
	| SCINTILLATING_CRYSTAL
	| MACHINE_PART

	| ASH_BLOSSOM
	| GNARLED_ROOT
	| BUDDING_BRANCH

	| SOUL_SKIN
-}
data RandomDrop = Flowers | Parts | Crystals | Skin | Flame | Water
	deriving (Eq, Ord, Enum, Bounded, Show, Read)

instance Random RandomDrop where
	randomR _ = random
	random = (`runRand` genRandom)

instance ToJSON RandomDrop where
	toJSON d = object
		[ (pack "name") .= "drop"
		, (pack "data") .= [fromEnum d]
		]

instance FromJSON RandomDrop where
	parseJSON (Object vs) = case (parse (.: pack "name") vs, parse (.: pack "data") vs) of
			(Success name, Success [d])
				| name == ("drop" :: String) ->
				case safeToEnum d of
						Nothing -> mzero
						Just h -> return h
			_ -> mzero
	parseJSON _ = mzero

deck :: RandomGen g => Rand g (Enumerable RandomDrop)
deck = do
	opts <- liftRand $ shuffle [Flowers, Parts, Crystals, Skin, Flame, Water]
	return $ from opts

instance Scene () () RandomDrop where
	link _ = error "link"
	display = fmap toLower . show
	vardisplay = fmap toLower . show

scene :: Encounter () () RandomDrop
scene _ d = case d of
	Flowers ->
		[ Narration
			[ "Some ash flowers are blossoming in the marsh. You pick them."
			]
		, ItemModify
			[I ASH_BLOSSOM 2]
		]
	Parts ->
		[ Narration 
			[ "Sloshing through the shallow waters, you find some broken machine parts half-buried in the riverside muck."
			]
		, ItemModify
			[I MACHINE_PART 2]
		]
	Crystals ->
		[ Narration
			["A patch of chunky crystals have congealed in a slowly-revolving river eddy."]
		, ItemModify
			[I SCINTILLATING_CRYSTAL 2]
		]
	Skin ->
		[ Narration
			["While along the bank of the river, you spot a strange, colorless thing out in the sluggish flow, twisting on a jutting branch."]
		, ItemModify
			[I SOUL_SKIN 2]
		]
	Flame ->
		[ Narration
			["Part of the river has caught aflame, burning a ruddy red-brown, and tendrils creep up onto the bank."]
		, ItemModify
			[I ETERNAL_FLAME 2]
		]
	Water ->
		[ Narration
			["The river flows unctuously, lapping low and wet at the banks. The ground is soggy and damp, water bubbling up in hollows like the ground is perpetually in the process of liquefying."]
		, ItemModify
			[I RIVER_WATER 2]
		]
