{-# LANGUAGE ViewPatterns #-}
{-# LANGUAGE MultiParamTypeClasses #-}

module Sites.Hell.Events.Hubur.RockDemon
	( RockDemon(..)

	, Index

	, deck
	, scene

	, name
	, bits
	, variants
	) where

import Prelude hiding ((/))

import Server.JSON
import Utility.Rand

import Sites.Hell.Events.SceneUtils
import Sites.Hell.Events.Construction ((/))

import Sites.Hell.Events.Hubur

name :: String
name = "rock demon"

data RockDemon = RockDemon
	deriving (Eq, Show)

instance Random RockDemon where
	randomR _ = random
	random = getRand $ return RockDemon

instance ToJSON RockDemon where
	toJSON RockDemon = object
		[ pack "name" .= name
		, pack "data" .= ([] :: [Int])
		]

bits :: Value -> Value
bits _ = object
	[ pack "name" .= name
	, pack "data" .= ([] :: [Int])
	]

variants :: [(MapZone, ())]
variants =
	[ (Badland, ())
	]

instance FromJSON RockDemon where
	parseJSON (Object vs) = case (parse (.: pack "name") vs, parse (.: pack "data") vs :: Result [Int]) of
			(Success pName, Success [])
				| pName == name ->
				case pure RockDemon of
						Nothing -> mzero
						Just h -> return h
			_ -> mzero
	parseJSON _ = mzero

deck :: RandomGen g => Rand g (Enumerable RockDemon)
deck = return $ pure $ RockDemon

data Index = Dummy
	deriving (Eq, Ord, Show, Read)

instance Scene Index () RockDemon where
	link x = case x of
		Dummy -> error "unfinished scene instance"
	display = fmap toLower . show
	vardisplay = fmap toLower . show


description :: RockDemon -> Event RockDemon Index ()
description RockDemon = Narration2 $
	"There's an enormous rock demon at the bottom of the cave."
	/ Paragraph / "It's humanoid, and top-heavy, with four massive arms that it drags, scraping, across the ground. Its hands and feet are shaped into thick black talons, and they send up showers of sparks on the rocky ground. The backs of its arms, its shoulders, its head and back, the backs of its legs, all are coated in an eruption of rocky slabs; across its chest and inner arms the rock thins into rough scutes, revealing leathery grey skin the same exact color as the rock itself. Its head is made from or covered in rock, and light from its four eyes shines out from deep slashes in its rocky helmet."
	/ Paragraph / "It moves with a constant grinding, crunching noise."

scene :: a -> RockDemon -> [Event RockDemon Index ()]
scene _ s =
	[description s]
