{-# LANGUAGE MultiParamTypeClasses #-}
module Sites.Hell.Events.Hubur.Shadow
	( Shadow(..)

	, Index
	
	, deck
	, scene

	, name
	, bits
	, variants
	) where

import Prelude hiding ((/), (^))

import Sites.Hell.Events.SceneUtils hiding ((/))
import Sites.Hell.Events.Construction ((/))

import Sites.Hell.Events.Hubur

name :: String
name = "shadow"

data Shadow = Shadow
	deriving (Eq, Ord, Show, Read)

instance Random Shadow where
	randomR _ = random
	random = getRand $ pure Shadow

instance ToJSON Shadow where
	toJSON Shadow = object
		[ pack "name" .= name
		, pack "data" .= ([] :: [Int])
		]

bits :: Value -> Value
bits _ = object
	[ pack "name" .= name
	, pack "data" .= ([] :: [Int])
	]

variants :: [(MapZone, ())]
variants =
	[ (FactoryResearch, ())
	, (FactoryStacks, ())
	]

instance FromJSON Shadow where
	parseJSON (Object vs) = case (parse (.: pack "name") vs, parse (.: pack "data") vs :: Result [Int]) of
			(Success pName, Success [])
				| pName == name -> return Shadow
			_ -> mzero
	parseJSON _ = mzero

deck :: RandomGen g => Deck g Shadow
deck = return . pure $ Shadow

data Index = Dummy
	deriving (Eq, Ord, Show, Read)

instance Scene Index () Shadow where
	link f = case f of
		_ -> error "bad link"

	display = fmap toLower . show
	vardisplay = fmap toLower . show

scene :: Encounter Index () Shadow
scene b s = describe b s -- <> error "bad scene"

--- actual event start

describe :: Encounter Index () Shadow
describe _ s =
	[ Narration2 $ "There's a shadow here. It has cut off its owner, if it ever had one, and now stands on its own two feet. It's a black silhouette from every angle, an opening into a realm of darkness cut out of the world."
		/ Paragraph / "It's dimly humanoid, or like a humanoid-shaped thing in a nightmare: proportions all wrong, hands that are gigantic talons, feet growing into monstrous paws. Spikes erupt from its body like the claws of some strange beast, reaching out from the darkness; some look like they speared it through, jutting through its arms or legs or head."
	]
