{-# LANGUAGE MultiParamTypeClasses #-}
module Sites.Hell.Events.Hubur.ShadowMage
	( ShadowMage(..)
	, deck
	, scene

	, name
	, bits
	, variants
	) where

import Prelude hiding ((/), (^))

import Server.JSON
import Sites.Hell.Events.SceneUtils hiding ((/))
import Sites.Hell.Events.Construction ((/))

import Sites.Hell.Events.Hubur

name :: String
name = "shadowmage"

data ShadowMage = ShadowMage
	{ burn :: Burn
	}
	deriving (Eq, Ord, Show, Read)

data Burn = Side
	deriving (Eq, Ord, Enum, Bounded, Show, Read)

instance Random ShadowMage where
	randomR _ = random
	random = getRand $ ShadowMage
		<$> genRandom

instance ToJSON ShadowMage where
	toJSON (ShadowMage b) = object
		[ pack "name" .= name
		, pack "data" .= [fromEnum b]
		]

bits :: Value -> Value
bits _ = object
	[ pack "name" .= name
	, pack "data" .= [Null]
	]

variants :: [(MapZone, ())]
variants =
	[ (FactoryResearch, ())
	, (FactoryStacks, ())
	]
instance FromJSON ShadowMage where
	parseJSON (Object vs) = case (parse (.: pack "name") vs, parse (.: pack "data") vs) of
			(Success pName, Success [b])
				| pName == name -> case ShadowMage
					<$> safeToEnum b of
						Nothing -> mzero
						Just s -> return s
			_ -> mzero
	parseJSON _ = mzero

deck :: RandomGen g => Deck g ShadowMage
deck = return . pure $ ShadowMage Side

data Index = Dummy
	deriving (Eq, Ord, Show, Read)

instance Scene Index () ShadowMage where
	link f = case f of
		_ -> error "bad link"

	display = fmap toLower . show
	vardisplay = fmap toLower . show

scene :: Encounter Index () ShadowMage
scene b s = describe b s -- <> error "bad scene"

--- actual event start

describe :: Encounter Index () ShadowMage
describe _ s =
	[ Narration2 $ "There's a shadow mage here."
		/ Paragraph / "He's ghoulishly pale, white skin webbed with dark veins, blue-black blood clear under his skin. His shadow has wrapped itself around his body and sunk in teeth. It scours him like a burn, covering him in splashes and streaks, coating nearly the entire left half of his body. Where it feeds it has grown substance and form, teeth and eyes and tongues in grotesque relief, full of zigzag curves and whorling circles, dots of red eyes in lines and curves within the deepest folds of shadow. His shadow-burn splashes across his face, and his left eye has become a burning circle of pure red."
	]
