{-# LANGUAGE MultiParamTypeClasses #-}

module Sites.Hell.Events.Hubur.Skull
	( Skull
	, emptyDeck
	, zombieDeck
	, scene

	, name
	, bits
	, variants
	) where

import Sites.Hell.Events.SceneUtils

import Server.JSON
import Sites.Hell.Events.Hubur
import Sites.Hell.Events.Hubur.Zombie (Zombie)
import qualified Sites.Hell.Events.Hubur.Zombie as Zombie

name :: String
name = "skull"

data Skull = Skull
	{ occupant :: Maybe Occupant
	}
	deriving (Eq, Ord, Show, Read)

data Occupant = OZombie Zombie
	deriving (Eq, Ord, Show, Read)

instance ToJSON Occupant where
	toJSON (OZombie z) = toJSON z

instance FromJSON Occupant where
	parseJSON o@(Object vs) = OZombie <$> parseJSON o
	parseJSON _ = mzero

instance ToJSON Skull where
	toJSON (Skull Nothing) = object
		[ pack "name" .= name
		, pack "data" .= ([] :: [Int])
		]
	toJSON (Skull (Just o)) = object
		[ pack "name" .= name
		, pack "data" .= ([] :: [Int])
		, pack "subdata" .= [toJSON o]
		]

bits :: Value -> Value
bits _ = object
	[ pack "name" .= name
	, pack "data" .= ([] :: [Int])
	]

variants :: [(MapZone, ())]
variants =
	[ (SwampZombie, ())
	]

instance FromJSON Skull where
	parseJSON (Object vs) = case (parse (.: pack "name") vs, parse (.: pack "data") vs :: Result [Int], parse (.: pack "subdata") vs) of
			(Success pName, Success [], v)
				| pName == name ->
				case v of
					Error _ -> return $ Skull Nothing
					Success [] -> return $ Skull Nothing
					Success [o] -> case Skull . Just <$> mparse parseJSON o of
						Nothing -> mzero
						Just h -> return h
					_ -> mzero
			_ -> mzero
	parseJSON _ = mzero

emptyDeck :: RandomGen g => Rand g (Enumerable Skull)
emptyDeck = return . pure $ Skull Nothing

zombieDeck :: RandomGen g => Zombie -> Rand g (Enumerable Skull)
zombieDeck z = return . pure $ Skull $ Just $ OZombie z

data Index = ZI Zombie.Index
	deriving (Eq, Ord, Show, Read)

data Vars = ZV ()
	deriving (Eq, Ord, Show, Read)

instance Scene Index Vars Skull where
	link x = case x of
		ZI i -> \b s -> case occupant s of
			Just (OZombie z) -> asZombie <$> link i b z
			_ -> error "bad skull instance"
	display = fmap toLower . show
	vardisplay = fmap toLower . show

asZombie :: Event Zombie Zombie.Index () -> Event Skull Index Vars
asZombie = triMap (\z -> Skull $ Just $ OZombie z) ZI ZV

scene :: Encounter Index Vars Skull
scene = opening

opening :: Encounter Index Vars Skull
opening b s =
	case occupant s of
		Nothing ->
			[ Narration2 $ c "There's a vast dragon skull lying across the swamp. A half-dozen people could easily fit through one of its hollow eye sockets. Earth has piled up around it, and the interior is a half-flooded grotto."
			]
		Just (OZombie z) ->
			asZombie <$> Zombie.scene b z
