{-# LANGUAGE MultiParamTypeClasses #-}
{-# LANGUAGE FlexibleContexts #-}

module Sites.Hell.Events.Hubur.SlimeGuy
	( SlimeGuy(..)
	, SlimeMaterial(..)
	, Pref(..)
	, Hair(..)
	, deck
	, scene

	, name
	, bits
	, variants
	) where

import Prelude hiding ((/), (^))

import Data.Maybe (catMaybes)
import Control.Monad

import Server.JSON
import Sites.Hell.Events.SceneUtils hiding ((/))
import Sites.Hell.Events.Construction ((/), (^))

import Sites.Hell.Events.Hubur

import Description.Body.Types (ballLocation)
import Description.TfNode (Animal(..))


{-
TODO:

	* blocking shunts all through forceBlow & willingBlow, probably: nagas, centaurs, quadrupeds, slithering.


TO ADD:
	* hook the 'next time' yeah/maybe/no scenes up to flags that lead to a different intro
	* expand the variety of the next time prompts & have them show up in more of the closers


-}

name :: String
name = "slimeguy"

data SlimeGuy = SlimeGuy
	{ material :: SlimeMaterial
	, sexPref :: Pref
	, hairStyle :: Hair
	, ornament :: Ornament
	}
	deriving (Eq, Ord, Show, Read)

data SlimeMaterial = Dissolve | Refine
	deriving (Eq, Ord, Enum, Bounded, Show, Read)

data Pref = Top | Bottom | Switch
	deriving (Eq, Ord, Enum, Bounded, Show, Read)

data Hair = None | BraidsShort | BraidsLong
	deriving (Eq, Ord, Enum, Bounded, Show, Read)

data Ornament = Bare | Rings
	deriving (Eq, Ord, Enum, Bounded, Show, Read)

instance Random SlimeGuy where
	randomR _ = random
	random = getRand $ SlimeGuy
		<$> genRandom <*> genRandom <*> genRandom <*> genRandom

deck :: RandomGen g => Rand g (Enumerable SlimeGuy)
deck = do
	material <- liftRand $ shuffle [Dissolve, Refine]
	pref <- liftRand $ shuffle [Top, Bottom, Switch]
	hair <- liftRand $ shuffle [None, BraidsShort, BraidsLong]
	ornament <- liftRand $ shuffle [Bare, Rings]
	return $ SlimeGuy
		<$> from material
		<*> from pref
		<*> from hair
		<*> from ornament

instance ToJSON SlimeGuy where
	toJSON (SlimeGuy m p h o) = object
		[ pack "name" .= name
		, pack "data" .= [fromEnum m, fromEnum p, fromEnum h, fromEnum o]
		]

bits :: Value -> Value
bits _ = object
	[ pack "name" .= name
	, pack "data" .= [Null, Null, Null, Null]
	]

variants :: [(MapZone, ())]
variants =
	-- processing version?
	[ (FactoryProcessing, ())
	, (FactoryProcessingPlant, ())
	-- dissolving version?
	, (FactoryMineTunnels, ())
	, (FactoryQuarry, ())
	]

instance FromJSON SlimeGuy where
	parseJSON (Object vs) = case (parse (.: pack "name") vs, parse (.: pack "data") vs) of
			(Success pName, Success [m, p, h, o])
				| pName == name ->
				case SlimeGuy
					<$> safeToEnum m
					<*> safeToEnum p
					<*> safeToEnum h
					<*> safeToEnum o of
						Nothing -> mzero
						Just enc -> return enc
			_ -> mzero
	parseJSON _ = mzero


data Index
	= Leave
	| WannaFuck
	| SexBackdown
	| EaseUp
	| SexChoose Mood
	| TopChoices Mood | BottomChoices Mood
	-- he tops:
	| BlowHim Mood | WillingBlow Mood | ForceBlow
	| GetFucked Mood | FuckBasic | Fisted Mood
	-- you top:
	| TopThroat | TopAss | FinishThroat | FinishFace From | FinishAss From
	-- some close stuff
	| NextTimeAccede Move | NextTimeConsider Move | NextTimeReject Move
	deriving (Eq, Ord, Show, Read)

data Move = WreckAss | YouTop
	deriving (Eq, Ord, Enum, Bounded, Show, Read)

data Mood = Easy | Sadist
	deriving (Eq, Ord, Enum, Bounded, Show, Read)

data How = Directly | Swap
	deriving (Eq, Ord, Enum, Bounded, Show, Read)

data From = FromBJ | FromFuck
	deriving (Eq, Ord, Enum, Bounded, Show, Read)

instance Scene Index () SlimeGuy where
	link f = case f of
		Leave -> leave
		WannaFuck -> fuckPrompt
		SexBackdown -> sexBackdown
		SexChoose mood -> sexChoose mood
		BottomChoices mood -> bottomChoices mood Swap
		TopChoices mood -> topChoices mood Swap
		EaseUp -> easeup
		BlowHim mood -> blowHim mood
		WillingBlow mood -> willingBlow mood
		ForceBlow -> forceBlow
		GetFucked mood -> bottomAss mood
		Fisted mood -> fisted mood
		FuckBasic -> fuckBasic
		TopThroat -> topThroat
		TopAss -> topAss
		FinishThroat -> finishThroat
		FinishFace from -> finishFace from
		FinishAss from -> finishAss from
		NextTimeAccede opt -> nextTimeAccede opt
		NextTimeConsider opt -> nextTimeConsider opt
		NextTimeReject opt -> nextTimeReject opt
	display = fmap toLower . show
	vardisplay = fmap toLower . show



scene :: Encounter Index () SlimeGuy
scene b s =
	[ Narration2 $ "It's a big, thick-bodied slime guy,"
		/ case material s of
			Dissolve -> "stirring up a vat of ore chunks with his arms, slowly dissolving the rock away from the minerals."
			Refine -> "reaching through a swampy mess of ore slurry, chunks of ore visibly drifting up his arms to his torso."
		/ describe s
	, Choices
		[ ("hey yr hot wanna fuck around", WannaFuck)
		, ("nah leave him to his work", Leave)
		]
	]

leave :: Encounter Index () SlimeGuy
leave b s =
	[ Narration2 $ c "You keep moving along the mine tunnel."
	]

describe :: SlimeGuy -> EventReel
describe s = "He's"
	/ case material s of
		Dissolve -> "a cloudy, burnt-orange color, body murkily translucent, with slowly-drifting clouds of more opaque substance inside him. A thick mantle of frothy bubbles spreads across his broad, bulky shoulders, covering his shoulderblades, and sluggishly drools down his back."
		Refine -> "a glowing purple, leanly-defined body crisply translucent all the way through. His skin grows more opaque down his arms, hands nearly-opaque and glowing brighter than the rest of him."
	/ case ornament s of
		Bare -> mempty
		Rings -> "He's got a mess of rings across his hands -- mostly gold, since the ones that aren't are blackened and corroded. Some of them are askew, half-sunk inside his body."
	/ Paragraph / "He's got about half a skeleton: skull, with a loose jawbone; most of a ribcage, a few ribs free-floating; spine dissolving into chunks; half a pelvis. The slime he's made from seems thick, dense enough to keep the bones in basically the right place."
	/ case material s of
		Dissolve -> "His bones are blackened and corroded, and thickly-encrusted with mineral formations, forming spurs and spikes. On the skull, they've been tended into big horns that jut out from the front, burst through the slime's skin, and arch overhead."
		Refine -> "Mixed in with his bones are chunks of half-processed ore, slowly breaking apart and mineralizing. The bones are encrusted with square, rainbow-colored crystals, shining dimly through the slime's skin. On the skull, they've been tended into an ornate crest, like a crown, that twists around his head like a pair of tightly-curled horns."
	/ case hairStyle s of
		None -> mempty
		BraidsShort -> Paragraph / "He's got something like hair; in between the horns and across his head, he's got bunched-up cords of slime that jut out in thick bundles, each ending in a splayed tuft. It looks like short, stiff braids. Or like gigantic rubbery earthworms."
		BraidsLong -> Paragraph / "He's got something like hair; in between the horns and across his head, down in a shaggy mess all across his shoulders are a bunch of thick, rubbery cords of slime, like broad, heavy braids. Or like gigantic earthworms. When he moves, they don't move quite like hair would; they're dimly prehensile, sliding against each other and ending up with strings of thin slime caught along their edges."

-- the refine one would have a cock w/ like, internal cockrings? basically reusing the design from that one commission of sean's. they vibrate.


{-
scene count: ~12


start:
	description, setting intro (etching ore or dissolving it or fishing it out of ore slurry, smelting it inside their bodies, etc)
	-> hey you wanna fuck
		see above w/ minotaurs; dialog stuff for top/bottom}
		yea sure lemme just whip up a cock here}
		& the slime grows a cock and it's a lot more like structurally solid than a clay man's cock would be. so it can be bigger.}
		"hey you wanna blow me or you just want me to shove it in yr ass"}

		-> blow him
			you suck on his cock & it's like continually oozing weird burning slime both from the tip and just generally everywhere}
			then he comes down yr throat}

		-> hey fuck me
			~> BIPED, NAGA, SLITHERING
			yr like lying on a bed of ore chunks (while he lifts yr legs up if applicable) and fucks you}
			then he floods yr guts w/ weird acid cum}
			~> QUADRUPED, CENTAUR
			he like half-engulfs yr haunches and fucks you w/ slime continually spurting from yr ass and oozing down yr legs}


		-> fuck his throat
			~> BIPED, NAGA
			he gets 'on his knees' in front of you and you grab his horns and fuck his throat, etc}
			-> cum down his throat
			-> finish on his face (like one line of pulling out + jump to 'finish face' scene)
			-> finish in his ass (like one line of penetration + jump to 'finish ass' scene)
			~> CENTAUR, QUADRUPED
			you kinda shove him down to hands and knees and fuck his face. small (quad) animals can use their paws/hooves on his back but big animals can't; so there would be a little split here}
			anyway then you come down his throat}
			~> SLITHERING
			loop coils around his head and pull him down into yr cloaca or w/e you got}


		-> fuck his ass
			~> BIPED, NAGA, SLITHERING
			pin him against the ore pile w/ yr arms / coils}
			then fuck him}
				~> SLITHERING
				come in his ass}
				~> BIPED, NAGA
					-> finish in his ass (jump to 'finish ass' scene)
					-> finish on his face (like one line of pulling out + jump to 'finish face' scene)
			~> CENTAUR, QUADRUPED
				get him down on his elbows and then mount him}
				anyway then you come in his ass}

	-> FINISH ASS
		blah blah cumflation, etc, w/e}
	-> FINISH FACE
		blah blah facial, w/e}
	then a final little bit where you can see yr cum churning around inside}
-}



dialog :: Convertible a EventReel => SlimeGuy -> a -> Event SlimeGuy Index ()
dialog s = Dialog2 "/static/img/icon.png" (case material s of
	Dissolve -> "Dissolving Slime"
	Refine -> "Refining Slime") . convert

fuckPrompt :: Encounter Index () SlimeGuy
fuckPrompt b s = case locoplan b of
	x | x `elem` [Spec Orb, Spec FloatingHead, Spec FloatingDick, Basic Formless, Basic Complex] ->
		[ dialog s $ "Really. That's what you're opening with."
			/ Paragraph / "Not that it's that important; I'm not into"
			/ case x of
				Spec Orb -> "floating orbs"
				_ -> "disconnected limbs"
			/ ", so you're gonna have to go somewhere else to get off."
		, FauxChoice "well fine"
		, dialog s $ "Fine."
		, Narration2 $ c "He waves."
		, dialog s $ "Seeya."
		]
	_ ->
		[ dialog s $ "Really. That's what you're opening with."
		, Narration2 $ c "He pulls back, arms warping and smearing back together through the rocky slurry. He ends up with a chunk of rock in his hand, acid froth bubbling up from its pitted surface: he squeezes down on it, and it shatters, flecks of dissolving rock forming a foaming trail down his arm."
		, dialog s $ "You wanna think about that again?"
		, Choices
			[ ("wow nevermind", SexBackdown)
			, ("yeah i'm still down even tho yr made from dissolving acid slime", SexChoose Easy)
			, ("look i'm down SPECIFICALLY BECAUSE yr made from dissolving acid slime", SexChoose Sadist)
			]
		]

sexBackdown :: Encounter Index () SlimeGuy
sexBackdown b s =
	[ Narration2 $ c "The slime guy shrugs."
	, dialog s $ "Not my fault you don't got what it takes."
	, Narration2 $ c "He turns back to his work."
	, dialog s $ "Be seeing you."
	, FauxChoice "yeah seeya"
	, Narration2 $ c "You leave."
	]

sexChoose :: Mood -> Encounter Index () SlimeGuy
sexChoose mood b s =
	[ Narration2 $ c "He huffs out a short laugh."
	, dialog s $ case mood of
		Easy -> "You're down to risk it, huh?"
		Sadist -> "You don't say. That's the kinda shit you're into, huh?"
	] <> case sexPref s of
		Top -> topChoices mood Directly b s
		Bottom -> bottomChoices mood Directly b s
		Switch ->
			[ dialog s $ "So what'll it be?"
			, Narration2 $ c "The slime licks his hand sloppily, leaving a line of drool slowly spilling over his lips and down his chin. He wraps his hand around his cock, tugging on it idly. It shudders, throbbing in his loose grip, slowly inflating."
			, dialog s $ case mood of
				Easy -> c "You wanna fuck me, or should I fuck you?"
				Sadist -> "You wanna try your luck sticking your dick in me,"
					/ case locoplan b of
						Basic x | x `elem` [Slithering, Naga] -> "or are you gonna lay back"
						_ -> "or are you gonna get down on your knees"
					/ "and let me fuck you up?"
			, Choices $ catMaybes
				[ guardC (hasHead b) ("i wanna suck you off", BlowHim mood)
				, Just ("how abt you fuck me", GetFucked mood)
				, guardC (hasDick b) ("c'mon suck me off", TopThroat)
				, guardC (hasDick b) ("i wanna fuck you", TopAss)
				]
			]

bottomChoices :: Mood -> How -> Encounter Index () SlimeGuy
bottomChoices mood how b s =
	case how of
		Directly -> mempty
		Swap ->
			[ dialog s $ "Oh, so that's how it is."
			]
	<>
	[ Narration2 $ "The slime reaches down to tug on his cock, the surface-layer of slime smearing around, frothing up, stretching between his fingers in thin cords when he draws back. His crotch bulges, the thick tassel of flesh there bulging and shuddering like it's being forcefully inflated, slime surging into it to fill it out."
		/ case material s of
			Refine -> "Thick pellets of crystal flow through his belly, sliding into place along his cock like internal piercings."
			Dissolve -> mempty
		/ "He cocks his hips, cock swollen fat and heavy, half-hard, slowly thickening in jerks and twitches."
	, dialog s $ case mood of
		Easy -> "So what're you after? You wanna try sucking on it?"
		Sadist -> "So you want me to fuck your face or just shove it up your ass?"
	, Choices $ catMaybes $ case mood of
		Easy ->
			[ guardC (hasHead b) ("yeah i wanna suck you off", BlowHim Easy)
			, Just ("how abt you fuck me", GetFucked Easy)
			, case how of
				Directly -> guardC (hasDick b) ("actually i was kinda thinking i'd fuck you", TopChoices mood)
				Swap -> Nothing
			]
		Sadist ->
			[ case how of
				Directly -> Just ("man how about you ease up a bit", EaseUp)
				Swap -> Nothing
			, guardC (hasHead b) ("yeah cmon fuck my face", BlowHim Sadist)
			, Just ("i want you to shove it in my ass", GetFucked Sadist)
			, case how of
				Directly -> guardC (hasDick b) ("actually i was kinda thinking i'd fuck you", TopChoices mood)
				Swap -> Nothing
			]
	]

topChoices :: Mood -> How -> Encounter Index () SlimeGuy
topChoices mood how b s =
	case how of
		Directly -> mempty
		Swap ->
			[ dialog s $ "Oh, so that's how it is."
			]
	<>
	[ Narration2 $
		(case how of
			Directly -> "The slime"
			Swap -> "He")
		/ "reaches up and presses his hand to his mouth, tongue licking from wrist all the way to his fingers, which he shoves into his mouth and sloppily sucks on, letting spit drool all the way down to his elbow. It's not the most practical gesture, since he's all shiny with slime everywhere already."
	, case mood of
		Sadist -> dialog s $ "So, what, you wanna feel some slime burning up your cock?"
		Easy -> dialog s $ "So you want me to suck on it? Or you just wanna try shoving it in?"
	, Choices $ catMaybes
		[ guardC (hasDick b) ("yeah c'mon suck me off", TopThroat)
		, guardC (hasDick b) ("yeah i wanna fuck you", TopAss)
		, case how of
			Directly -> guardC (hasDick b) ("actually i was kinda thinking you'd fuck me", BottomChoices mood)
			Swap -> Nothing
		]
	]

easeup :: Encounter Index () SlimeGuy
easeup b s =
	[ Narration2 $ c "The slime snorts."
	, dialog s $ "Well sorry for" / em "assuming" / "."
		/ Paragraph / "But what'd you expect to get talking like that?"
	, dialog s $ "So, c'mon, what're you after? You wanna try sucking on it or what?"
	, Choices $ catMaybes
		[ guardC (hasHead b) ("yeah i wanna suck you off", BlowHim Easy)
		, Just ("how abt you fuck me", GetFucked Easy)
		]
	]


topThroat :: Encounter Index () SlimeGuy
topThroat b s =
	[ dialog s $ "You're the boss, man."
	, Narration2 $ "The slime drops down to his knees"
		/ case locoplan b of
			Basic Slithering -> "and runs a hand down your side"
			Basic x | x `elem` [Quadruped, Centaur] -> "in front of you"
			Basic x | x `elem` [Biped, Naga] -> ", level with your cock"
			f -> bodyFail "topThroat" f
		/ ". He swallows, throat jerking, and then licks his lips, tongue flitting out from the hollow of his mouth for a second to smear cords of slime over his lips."
	]
	<> case locoplan b of
		Basic Slithering -> topThroatSlithering b s
		Basic x | x `elem` [Centaur, Quadruped] -> topThroatAnimal b s
		Basic x | x `elem` [Biped, Naga] ->
			[ Narration2 $ "You reach down and grab his horns,"
				/ case material s of
					Dissolve -> "burnt-carbon mess rough and craggy against your palms,"
					Refine -> "burnished metal seeming a fraction away from dissolving like mercury in your palms,"
				/ "and drag him fractionally closer, pressing his lips across your shaft. He mouths across your dick wetly, drooling slime down in cords, and you angle his head with one hand and feed him your dick with the other, shoving the entire half-hard length with nearly no resistance all the way down his throat. You can still see it through him, a" / partAdjective Shaft / "shaft spanning down into his chest, limned with a froth of slime. His slime-body wraps around your cock and pumps and squeezes in anatomically-impossible ways. Your cock pulses, a fat bubble of pre forming around your cockhead, murky white slowly expanding like a balloon as you drool out more pre." -- TODO: {white}, should branch on come color; also {down into his chest}, should branch on cock length
				/ Paragraph / "You grip around his horns and fuck his throat, tugging him back and thrusting forward, his slime-flesh distending when his face smacks against your crotch, only really stopping when his teeth press against your"
				/ case bodyMat_ (Waist Fore) b of
					Solid (FMat _ x) -> matword x
					Trans (FMat _ x) _ -> matword x
				/ "."
				/ Paragraph / "You pump back and forth, the slimy tunnel of his throat gulping and squeezing around your shaft, the teeth in his unhinged jaw sometimes digging against your cock. He leans into it, sinking onto your cock just as much as you're pulling him down onto it. Your cock pounds into his open mouth with a sick wet slurp, gurgling and popping; thick froth erupts from his mouth each time he bottoms out on your cock, spraying in foamy lines all across your hips"
				/ case bodyMat_ (Waist Fore) b of
					Solid (FMat _ Fur) -> ", tanging in your shaggy fur and matting it down into thick wet hanks"
					Solid (FMat _ Skin) -> mempty
					Solid (FMat _ Coat) -> ", smearing into your stiff coat"
					Solid (FMat _ Scales) -> mempty
					_ -> mempty
				/ "."
				/ Paragraph / "Your cock twitches, visibly"
				/ case firstOf [Flaring] climaxAttrs b of
					Just Flaring -> "flaring"
					_ -> "shuddering"
				/ "within the slime's body, and heat ripples up and down through you. The small pocket of pre has turned into a sloppy, churning mess, visibly rippling and warping inside him as your cock stabs through it each time, churning slime and pre together in a slurry."
			, Choices
				[ ("cum down his throat", FinishThroat)
				, ("finish on his face", FinishFace FromBJ)
				, ("finish in his ass", FinishAss FromBJ)
				]
			]
		f -> bodyFail "topThroat" f

topThroatSlithering :: Encounter Index () SlimeGuy
topThroatSlithering b s_ =
	[ Narration2 $ "You coil around his head first, ignoring the burn that spreads all across your underbelly at the touch of his slime, and grind his head down against"
		/ case dickLocation b of
			Just DickExternal -> c "your hardening cock"
			Just DickSheathed -> c "your sheath"
			Just _ -> "your spread" / sheath
			f -> bodyFail "topThroatSlithering" f
		/ ", coils sliding back and forth as"
		/ case dickLocation b of
			Just DickExternal -> "he wetly mouths across your" / Shaft^s / ", coating" / "it"^p "them" / "with slick, burning slime before he settles down lower, sucking" / case cockCount b of
					1 -> "the head"
					2 -> "one head and then the other into his mouth"
					_ -> "one head into his mouth, and then another and another, lips warping into rubbery swells as he crams all your cocks into his stretched-wide mouth"
			Just DickSheathed -> "he sloppily kisses your wrinkled sheath, drooling burning slime down inside it as your" / Cock^s / Unsheathes^s / "directly into his open mouth"
			Just _ -> "he wetly drags his lips over your spread lips, tongue darting inside your" / sheath / "as your" / Cock^s / Unsheathes^s / "directly into his mouth"
			f -> bodyFail "topThroatSlithering" f
		/ case cockCount b of
			1 -> mempty
			2 -> c "cheeks bulging from your twin dicks"
			q -> "cheeks bulging, the structure of his jaw and throat destroyed by the girth of your" / number q / "cocks all shoved into his mouth at once"
		/ ". You rut against him, and he effortlessly takes"
		/ case cockCount b of
			1 -> "your cock"
			2 -> "both cocks"
			_ -> "all your cocks"
		/ "down his throat, rubbery and half-hard from the sting as"
		/ oneCock
			? "it swells his throat"
			?: "they stretch his throat into a bloated mess"
		/ ". He gurgles, drooling and spitting up slime as you start up a rhythm. You're mostly sprawled backwards over top his head and shoulders, coils tightening across his shoulders as you slurp back and forth, letting his drool spill down across your underbelly in burning cords."
		/ Paragraph / "You hilt your" / unsheathed / Cock^s / "all the way down his throat," / Tip^s / "visibly stirring through the slime around his ribcage. Pre squirts out into his slime, forming first a translucent smear and then, as you keep rutting, a fat bubble of cloudy slime, warping and distending against the pressure of his internal movements of slime. He gags again, frothing slime spilling from his mouth and nose"
		/ case bodyMat_ Chest b of
			Solid (FMat _ Protoplasm) -> c ", soaking your protoplasm; osmosing inside you with a burning pang"
			Solid (FMat _ x)
				| x `elem` [Fur, Coat] -> ", drenching your" / matword x
				| x `elem` [Scales, Skin] -> ", spilling across your" / matword x
			_ -> mempty
		/ ", and your" / Cock^s / case firstOf [Flaring] climaxAttrs b of
			Just Flaring -> c $ Flares^s
			_ -> c $ Twitches^s
		/ "down his throat,"
		/ case cockCount b of
			1 -> mempty
			2 -> "both"
			_ -> "all of them"
		/ "gushing pre in thick squirts."
	, Choices
		[ ("cum down his throat", FinishThroat)
		, ("finish on his face", FinishFace FromBJ)
		, ("finish in his ass", FinishAss FromBJ)
		]
	]

topThroatAnimal :: Encounter Index () SlimeGuy
topThroatAnimal b s_ =
	[ Narration2 $ case bodyHeight b of
			SmallAnimal -> "You hop up onto your hind legs," / feet / "planted on his shoulders. The slime of his body shifts under your" / feet / ", but doesn't buckle. He leans in, sending your" / feet / "skittering down his back, until he's crouched under you, face pressed against your"
				/ case dickLocation b of
					Just DickSheathed -> "unsheathing"
					_ -> mempty
				/ Cock^s / "."
			_ -> "You take a step over him, and then another, so that he's pressed beneath your underbelly." / case dickLocation b of
					Just DickSheathed -> "Your fat sheath knocks"
					Just DickExternal -> "Your hanging cock knocks"
					Just _ -> "Your fat vent presses"
					f -> bodyFail "topThroatAnimal" f
				/ "against the side of his face"
				/ case dickLocation b of
					Just DickExternal -> mempty
					Just _ -> ", cock dropping out in a slimy mess"
					f -> bodyFail "topThroatAnimal" f
				/ "."
		/ Paragraph / case dickLocation b of
			Just DickExternal -> c "Your" / Cock^s / Hangs^s / "in a low arc, rubbery and half-hard; the slime dips down along the" / Shaft^s
			Just x -> "Your" / Cock^s / Spills^s / "out from" / case x of
					DickSheathed -> c $ "its sheath"^p "their sheath"
					_ -> c "your vent"
			f -> bodyFail "topThroatAnimal" f
		/ ", dragging down over the slime's cheek and digging into his neck; he dips down with" / "it"^p "them"
		/ ", mouth open and suckling on the side of" / manyCocks ? "one of" / "your sloppy" / Shaft^s / ". The tip shudders when his lips wrap around it, the" / partAdjective Cockhead / "head pulsing and flooding his mouth with a mess of pre; he sputters and sprays most of it back across your shaft"
		/ case bodyMat_ (Waist Hind) b of
			Solid (FMat _ Coat) -> ", catching in the stiff coat of your underbelly"
			Solid (FMat _ Fur) -> ", catching in the curling fur of your underbelly"
			_ -> mempty
		/ "."
		/ case bodyHeight b of
		SmallAnimal -> "Your" / feet / "slide down as he lowers himself, and you brace against his lower back and shove your cock into his spasming throat."
		_ -> "You hunch forward, your" / "body effortlessly cramming more of your cock down into his spasming throat." -- {strong/powerful/rangy}
		/ Paragraph / "He gurgles wetly, slime throat milking your shaft even as he coughs up froth, streaking down the rest of your shaft. He slurps loudly, making sloppy wet gurgles and splatters as you thrust in; your cockhead drives down his throat and into his chest, leaving a fucked-open channel inside him when you draw back, slick and lubed with a foaming, frothing mixture of pre and slime that squirts up around your shaft when you thrust forward again, erupting from the slime's mouth in a spray that paints all across your" / hips / "and underbelly."
		/ Paragraph / "You fuck his throat, each thrust ending with the slime's entire body convulsing in a sickening gag, his always-slimy face completely slathered in a mess of frothing ooze, thick cords hanging down from his chin, down the underside of your shaft. You push forward," / groaning / ", and shove your cock into him down to the root"
		/ case dickLocation b of
			Just DickExternal -> c ", grinding his face against your crotch."
			Just d -> ", until his face smacks against your" / sheath / "with a slimy" / em "plap" / ", his lips pressed against it in an obscene kiss. He gags and sputters, spitting up sloppy ooze directly into it until"
				/ case d of
					DickSheathed -> c "the wrinkled flesh bloats and bulges. When you pull back the sloppy connection breaks, shaft wrenching out of his throat, and the ooze squirts back out from your sheath, painting thick, stringy lines across his face."
					_ -> "your bloated" / sheath / "parts wider, flooded with his spit, and with an internal contraction you spray the oozing mess of pre and slime back across his face."
			f -> bodyFail "topThroatAnimal" f
		/ Paragraph / "You buck harder, getting closer, the slime guy's body rippling from the force of each impact"
		/ case dickLocation b of
			Just DickExternal -> mempty
			Just _ -> "," / sheath / "mashing against his face"
			f -> bodyFail "topThroatAnimal" f
		/ hasExternalBalls ? ", balls smacking hard against his jaw"
		/ ". His throat is flooded, stomach sloshing with your pre, and the spasmodic clench of his slime curling around your cock grows to be too much, and you" / groan / "up at the sky as you erupt. Your cock pulses hard, come cannonading out in torrents, flooding his stomach to bulging in pulse after pulse, surging up his fucked-open throat along your still-thrusting shaft to spray from his mouth, splattering the ground beneath you in thick puddles of come. You keep thrusting, slower, churning your cock around in the morass of come inside him." -- was {bay}, not groan
		/ Paragraph / case bodyHeight b of
			SmallAnimal -> "You"
			_ -> "When you step back, half your shaft spilling back up his throat, you"
		/ "can see his bloated belly through him, pressing down against the ground, a big compressed bubble of churning come."
		/ case bodyHeight b of
			SmallAnimal -> "You step back, half your shaft spilling back up his throat, and he gurgles and gags, come pouring from his mouth."
			_ -> "Come spills up his throat, pumped up by his choking, and he gurgles and gags, come pouring from his mouth."
		/ "The only change when you dismount entirely -- cockhead pulling from his mouth with a" / em "pop" / "-- is that your load isn't even partially plugged up inside him; he retches wetly, puking up a mess of come, frothing at the edges, dazed and nearly facedown in the slimy mess."
		] <> throatfuckAftermath b s_

throatfuckAftermath :: Encounter Index () SlimeGuy
throatfuckAftermath b s =
	[ Narration2 $ c "The slime stays there heaving for a while before he sits back -- bloated gut still visibly churning with your load -- and wipes his mouth with the back of his hand."
	, dialog s $ "Fuck."
		/ Paragraph / "Yeah, wow, y'did good."
	, Narration2 $ c "He coughs and spits again, voice wrecked."
	, dialog s $ "I'm gonna, uh, need to take a bit to recover. See you then."
	, FauxChoice "yea seeya"
	, Narration2 $ c "You leave. You look back after a while to see that he's shakily gotten to his feet, leaning against the wall. He catches you looking and grins."
	]

wreckassAftermath :: Encounter Index () SlimeGuy
wreckassAftermath b s =
	[ Narration2 $ "The slime stays there for a bit, your load drooling from his ass in a sloppy mess, coursing down his trembling thighs. He spreads a hand across his bloated gut, the churning orb of come inside him slopping to one side and then the other as he gets to his feet."
		/ Paragraph / "He sags heavily against the wall:"
	, dialog s $ "Yeah, fuck, y'did good."
	, Narration2 $ c "Come spills from his wrecked asshole, dribbling and spurting as he clenches, utterly drenching his legs in a foaming, frothing mess."
	, dialog s $ "I'm gonna, uh, need to take a bit to recover. See you then."
	, FauxChoice "yea seeya"
	, Narration2 $ c "You leave. You look back after a while to see that he's still sprawled against the wall, legs spread, a finger sloppily milking your load out of his ass. He catches you looking and grins."
	]

finishThroat :: Encounter Index () SlimeGuy
finishThroat b s =
	[ Narration2 $ hasArms
			? "You grip his horns tighter,"
			?: "You wrap tighter around his head,"
		/ "leaning forward and rutting against his face -- him leaning back, your"
		/ hasLegs ? "thighs" ?: "underbelly"
		/ "pressed tight against his chest as you fuck the final few inches of your cock back and forth. He gurgles, froth bubbling up his throat and spewing across"
		/ case locoplan b of
			Basic Biped -> "your hips and stomach"
			Basic Naga -> "your stomach and underbelly"
			Basic Slithering -> "your underbelly, down across your tail"
			f -> bodyFail "finishThroat" f
		/ ", face warped around the girth of your fat cock as you hammer it down again and again, each thrust scraping your"
		/ case firstOf [Flaring] climaxAttrs b of
			Just Flaring -> "half-flared"
			_ -> mempty
		/ "cockhead against the inner gristle of his body, sending a hot spark of heat shooting up through your cock."
		/ Paragraph / "Your cock twitches"
		/ case firstOf [Flaring, Twisting] climaxAttrs b of
			Just Flaring -> ", flare kicking hard,"
			Just Twisting -> ", your cockhead making its first sideways lurch,"
			_ -> mempty
		/ "and you spurt a slimy gush of pre, smearing and diffusing through the slime's now-bloated gut. Heat sings up your shaft, pleasure burning hot and hard across your body, and you come with a" / groan / ", still sloppily facefucking him even as your cock shudders and erupts, not breaking your pace at all."
		/ Paragraph / "He gurgles and chokes, spraying a froth of slime and come all up across"
		/ case locoplan b of
			Basic Naga -> c "your stomach and down your underbelly"
			Basic Biped -> c "your stomach and all over your legs"
			Basic Slithering -> c "your body"
			Basic _ -> "all over your" / forelegs
			f -> bodyFail "finishThroat" f
		/ ". His throat spasms around your pounding cock, each thrust bringing up a sloppy, liquid mess of slime churned together with come, fountaining up through his fucked-open throat around your shaft. You unload in his gut, it seems like half of your load getting spat up across your cock, until he's so drenched it's hard to tell his features from just more loose slime."
		/ Paragraph / "You flood his stomach until it's bloated, a big opaque marble of come stirring around in his gut, and then you stay in his throat until you've gone half-hard, just keeping him plugged up until you can pull out without him immediately puking up more of your load. He laps at the heavy, spongy flesh of your shaft, mostly just smearing its coating of frothing slime around."
	] <> throatfuckAftermath b s


topAss :: Encounter Index () SlimeGuy
topAss b s_ =
	[ dialog s_ $ "Sure, lets go for it."
	, Narration2 $ c "The slime stretches, arms raised above his head, on his tip-toes -- threads of slime glimmerering down when he spreads his interlaced fingers apart."
	, dialog s_ $ "C'mon, rip up my ass."
	, Narration2 $ c "He gets down on his knees: fat, firm ass on display, looking back at you."
	] <> case locoplan b of
		Basic x | x `elem` [Biped, Naga, Slithering] ->
			[ Narration2 $ "You"
				/ hasLocotype Biped
					? "get down on your knees behind him"
					?: "slither down lower to line yourself up"
				/ "and shove in."
				/ hasArms
					? ("You test him out with a" / finger / "or two before, spreading his fat cheeks and shoving into the hot, slimy mess that's his ass.")
					?: ("You coil up close to him, body a sinuous trail over the floor," / Cockhead^s / "nudging between his cheeks before you push inside.")
				/ "Your"
				/ hasArms
					? (fingers / " -- and" / Cock^s / " --")
					?: (Cock^s)
				/ " spread him open easy, yielding, and you thrust in, fitting"
				/ case cockCount b of
					1 -> "your entire cock inside him"
					2 -> "both your cocks in him"
					_ -> "all your cocks in him"
				/ "in a single thrust"
				/ case cockCount b of
					q | q >= 3 -> ", distending his body into a warped and rippling mass, his thick slime stirring and squirming around your cocks as they all sink inside, in a pinned-together bunch"
					_ -> mempty
				/ "."
				/ Paragraph / "He groans, rutting back against you, asshole clenching around the base of your" / Shaft^s / ", his guts milking" / "it"^p"them" / "like fingers and tongues are sliding up all around your" / Length^s / ". You can still see your" / Cock^s / "through him, tinged from the"
				/ case material s_ of
					Dissolve -> "yellow"
					Refine -> "purple"
				/ "slime of his body."
				/ Paragraph / "You thrust, "
				/ hasLocotype Slithering ? "underbelly clapping against" ?: "hips hitting"
				/ "his ass and sending his flesh jiggling"
				/ hasExternalBalls ? ", your balls smacking his cheeks hard"
				/ ". You build up a rhythm, fucking him over the piles of ore, his own cock drooling acid slime all over them, sending up fuming plumes as it soaks in. His body is fucked-open, a fat tunnel spread open inside him, slick and slimy, filled with frothing ooze and your mingled pre, squelching and slurping as you rut forward."
				/ case cockCount b of
					1 -> mempty
					_ -> "Your cocks bloat his gut, the heads smearing around across the thicker membrane of his stomach, dragging it out into peaks and arcs, making his entire body distend when you thrust in to the hilt,"
						/ case dickLocation b of
							Just DickExternal -> c $ hasExternalBalls ? "balls smacking against his cratered ass" ?: "crotch hitting his cheeks with a smack"
							Just _ -> sheath / "smacking against his cratered ass"
							f -> bodyFail "topAss" f
						/ ", nearly picking his entire body up by the force of your thrusts."
				/ "You gush pre into him, forming a wobbling bubble in his bloated belly, bursting and sluicing down your" / Shaft^s / "every time your" / Cockhead^s / Pierces^s / "into it. Your" / Cock^s / "visibly" / Shudders^s / ", flexing and flaring, and he groans, facedown on the rocky ore, fucking himself on your" / Cock^s / "just as much as you're fucking him."
			, Choices
				[ ("finish in his ass", FinishAss FromFuck)
				, ("finish on his face", FinishFace FromFuck)
				]
			]
		Basic x | x `elem` [Quadruped, Centaur] ->
			[ Narration2 $ "You mount him hard, driving him down to his elbows on the ground with a gurgling" / em "whoof" / ". Your"
				/ case dickLocation b of
					Just DickExternal -> c "half-hard cock"
					Just _ -> c sheath
					f -> bodyFail "topAss" f
				/ "grinds against him for a few seconds, sliding up and down the curve of his back beneath you before you start to"
				/ case dickLocation b of
					Just DickSheathed -> "unsheath"
					Just DickExternal -> "stiffen up"
					Just _ -> "spill out"
					f -> bodyFail "topAss" f
				/ ", the rubbery arc of your cock smearing across him, keeping him pinned down."
			-- (this should maybe have a variant if you got a baculum since yr cock can't be lolling THAT much)
				/ Paragraph / "You rut up and down his sides, cock spilling over his hips and reaching up nearly to his shoulderblades, sometimes dipping down low enough to catch over the soft flesh of his cheeks. You" / growl / "and draw back, using one" / foot / "in the center of his back to pin him to the ground while you pull your cock back and line it up, tip pointed right between his cheeks. You dig in, legs jerking, hips working minutely to keep your cock sinking further"
				/ case firstOf [Flared] dickAttrs b of
					Just Flared -> ", your broad tip spreading his cheeks tight around your thick cockhead"
					_ -> mempty
				/ ". His cheeks clench around your cockhead, practically sucking you in, and you" / groan / "and plow forward, cramming your cockhead and the first section of your shaft into his ass. His body jolts against your" / foot / ", sliding a fractional distance forward, only noticeable from the slime coating he leaves on everything he's pressed against."
				/ Paragraph / "He groans and huffs, pinned there and half-impaled on your cock, your length visible through his body: sliding through the opening of his half-destroyed pelvis, warping the slime-substance of his stomach. The slime squeezes and clenches around your shaft and cockhead, like tongues or nubby fingers, gripping and stroking your cock internally. There's a fat span of your shaft hanging out between you, drooping down in a weighty arch before it hits his ass cheeks, warped where they're spread around your shaft. You bear down on his back, pressing down harder to stop him from sliding forward as you push more of your cock in, sliding in sudden jerks forward as you plunge deeper inside,"
				/ case bodySpecies Genitals b of
					Just a | ridgedCockhead a -> partAdjective Cockhead / "cockhead tearing through his guts"
					_ -> partAdjective Cockhead / "cockhead piercing through his solid slime-body"
				/ "and shoving directly between his ribs before you bottom out with a wet" / em "slap"
				/ case dickLocation b of
					Just DickExternal -> mempty
					Just _ -> "," / sheath / "kissing his ass"
					f -> bodyFail "topAss" f
				/ hasExternalBalls ? ", balls coming to rest against his cheeks with a subdued slap"
				/ "."
				/ Paragraph / "The slime groans, breathless, sliding a pinned hand to feel across his chest and stomach, to push down against his skin. Your cock shudders inside him, its outline becoming blurred and hazy as the slime starts to froth up around your flesh, pre washing a mess of foaming bubbles down lower. You step off his back and he springs upward, entire body tugged slightly up solely from the arch of your cock planted inside him, skewering him from ass to chest. He braces his arms against the wall in preparation for the thrust you're already making: cock sliding back, leaving a fucked-open hole inside him, before slamming forward again, hilting with a" / em "smack" / "."
				/ Paragraph / "You fuck him hard, your rhythm fast and merciless, each thrust splattering slime across your" / hips / ". His slime forms a churned-up, stringy mess sliding all along your shaft, tugging back against your thrusts when you withdraw. Your cock shudders, pleasure sparking"
				/ hasExternalBalls ? "behind your balls" / "at the root of your cock"
				/ "and streaming up through your body, and you start pounding harder as you come close."
				/ Paragraph / "You come with a trumpeting bellow," -- was {trumpeting bellow}
				/ case x of
					Quadruped -> "head tipped up,"
					Centaur -> "head tipped back,"
					f -> bodyFail "topAss" f
				/ hasExternalBalls ? "your balls lurching hard as you unload"
				/ "." / comeInAssFinish b s_
			] <> wreckassAftermath b s_
		f -> bodyFail "topAss" f

finishAss :: From -> Encounter Index () SlimeGuy
finishAss from b s_ =
	[ Narration2 $ case from of
			FromBJ -> "You pull out with a wet" / em "slurp" / ", leaving his mouth hanging open, froth bubbling up from his heaving throat, before you shove him to the side,"
				/ hasArms
					? ("fingers prying into his fat ass so you can line up your" / Cock^s / "and slam" / "it"^p "them" / "in to the hilt even as you start to come.")
					?: (Cock^s / "smearing over the curve of his fat ass and slamming inside even as you start to come.")
			FromFuck -> c "You hammer him hard, thrusts smacking his ass as you get closer."
		/ comeInAssFinish b s_
	] <> wreckassAftermath b s_

comeInAssFinish :: BodyGlob -> SlimeGuy -> EventReel
comeInAssFinish b s_ = "Your load erupts into the slime with a thick gush, forming a warped pocket of churning ooze, bulging and wobbling as you keep thrusting. Your" / hips / "smack heavily against his ass"
	/ hasExternalBalls ? ", balls cracking against his skin"
	/ "with each thrust, again and again as you make short sharp thrusts. You jam the final few inches of your" / Cock^s / "back and forth, his slime-body lewdly slurping and gurgling with each thrust. A churning slurry of slime and come"
	/ oneCock ? "wreaths your shaft" ?: "spills between your shafts"
	/ ", spurting down along the fucked-open tunnel of his ass to erupt into the open air as a foaming mess, splattering in a stringy mess all across your"
	/ case locoplan b of
		Basic Biped -> "chest and thighs"
		Basic Naga -> "chest and underbelly"
		Basic Slithering -> "coils"
		Basic x
			| x `elem` [Centaur, Quadruped] -> "haunches and underbelly"
		f -> bodyFail "comeInAssFinish" f
	/ ", pouring down over the slime's folded legs until it's almost impossible to discern the slime of his body from the rest of it. Your load is a messy opaque glob within his ribcage, surface spinning and warping from the force of each shot inside."
	/ Paragraph / "You"
	/ hasLegs ? "take a step back" ?: "spill backwards"
	/ ", "
	/ oneCock
		? "half your cock dragging out from his wrecked ass"
		?: "your cocks slurping out from his wrecked ass halfway"
	/ " -- even his slime-body bruised, asshole a big crater that yawns open wide as your" / Cock^s / Spills^s / "out, drenched and drooling with come -- and then"
	/ hasLegs ? "take another step" ?: "pull back again"
	/ ","
	/ case firstOf [Flaring, Knotting] climaxAttrs b of
		Just x -> "yanking" / case cockCount b of
				1 -> "the entire thing"
				2 -> "both cocks"
				_ -> "all the rest of your cocks"
			/ "out with a lurch," / case x of
				Flaring -> "flat" / Cockhead^s
				Knotting -> c $ Knot^s
				f -> bodyFail "comeInAssFinish" f
			/ "catching on the lip of his asshole and pulling an enormous puffily-ringed swell of slime-flesh out to hang from his ass, drenched opaque with come"
		Nothing -> "pulling out" / case cockCount b of
				1 -> "your entire cock"
				2 -> "both cocks"
				_ -> "all your cocks"
			/ "with a sick, heavy" / em "slurp"
	/ ". Your" / Cock^s / Shudders^s / ", still spitting come up across his back, into the gape of his ass, as he sprawls there and shudders, gasping for breath."

finishFace :: From -> Encounter Index () SlimeGuy
finishFace from b s =
	[ Narration2 $ case from of
			FromBJ -> "You pull out from his throat with a" / em "slurp" / ", and he retches up a foaming mess of pre"
				/ case bodySpecies Genitals b of
					Just a | not (ridgedCockhead a) -> "as you pull your" / partAdjective Cockhead / "cockhead out"
					_ -> "along with your" / partAdjective Cockhead / "cockhead"
				/ ", wetly coughing as you"
				/ hasArms
					? "wrap hold of your cock and stroke it"
					?: "rut your cock across his foam-covered face"
				/ ". Your shaft is drenched with slime, drooling down to"
				/ case (dickLocation b, hasExternalBalls b) of
					(Just DickExternal, True) -> c "your balls"
					(Just _, True) -> "your" / sheath / "and balls"
					(Just _, False) -> "your" / sheath
					_ -> c "the base of your cock"
				/ ", thick globs of it splattering down across his chest as you"
				/ hasArms
					? "jerk yourself off"
					?: "fuck across his face"
				/ "."
			FromFuck -> "You tear out of his ass, leaving it gaping and cratered, spilling slime and pre as you flip him over, cock slathered in a heavy frothing slurry, dripping the mess all across his chest as you"
				/ hasArms
					? "start stroking"
					?: "send it crashing down across his face, rutting furiously"
				/ ", only an inch away from coming already."
		/ "You erupt all across his face, your first shot lancing out and cracking hard across his cheek, the squirt rebounding to spray droplets all over. And then you shoot and shoot and shoot,"
		/ hasArms
			? "hands pumping your shaft hard, milking your cockhead,"
			?: "smearing through the grimy cords of your load splattered all across his face,"
		/ "head thrown back as you"
		/ case bodySpecies Head b of
			Just Human -> c "groan"
			_ -> "groan and" / groan
		/ ", utterly drenching the slime in your load. You rut back and forth,"
		/ hasArms
			? "fucking your hand,"
			?: "obscene squelches coming from your cocks driving channels over his drenched face,"
		/ "come squirting out across his drenched and disheveled face, pouring into his open and flooded mouth."
		/ Paragraph / "Come piles up across him in slimy puddles, sluicing down his skin and pouring down onto the half-dissolved rocks, spilling across his sides in slimy rivers. He groans, muffled sometimes with his mouth pressed against the underside of your cock, and dimly you can feel him come too, his cock shuddering and shooting off, spraying acid-slime come all across his stomach and chest, mingling with your load in a fuming mess."
	] <> facialCloser from b s

facialCloser :: From -> Encounter Index () SlimeGuy
facialCloser from b s =
	[ Narration2 $ c "He sputters, mouth open wide, come pooling along his jawbone."
	, dialog s $ "Fuck."
	, Narration2 $ case from of
		FromBJ -> "His voice is hoarse; the sound comes out murky, and he coughs afterward, spitting up a mess of loose slime and pre."
		FromFuck -> mempty
		/ "He swallows and swipes through the mess drooling down his face, sucking on his fingers."
	, dialog s $ "Man, you come a lot."
	, Narration2 $ c "He drags a thumb up the side of his cheek, slime and come overflowing his finger and mostly splattering to the ground; he shoves his thumb into his mouth anyway."
	, dialog s $ "'s hot."
	, Narration2 $ case from of
			FromFuck -> "He staggers to his feet, not without a wince and a shudder, a hand reaching back to gingerly press against his ass, a mess of pre and churned-up slime still spilling down his legs. More"
			FromBJ -> "He gets up with a lean stretch, and more"
		/ "of your load starts to spill down his body, forming messy streaks through his slime coating."
	, dialog s $ "Well that was great and all, but I gotta get back to work sooner or later."
	, Narration2 $ c "He shrugs, rolling his shoulders, and more cords of come spill down, splattering to the ground."
	, dialog s $ "Come back if you wanna fuck me up s'more."
	]


blowHim :: Mood -> Encounter Index () SlimeGuy
blowHim mood b s =
	[ Narration2 $ c "He gestures at his cock, rubbery and fat, shuddering slightly from his internal flow of slime."
	, dialog s $ case mood of
		Easy -> "Well, get at it."
		Sadist -> case locoplan b of
			Basic Biped -> "Well, get on your fuckin' knees."
			Basic x
				| x `elem` [Centaur, Quadruped] -> "Well, fuckin' kneel down."
				| x `elem` [Naga, Slithering] -> "Well, get the fuck down there."
			f -> bodyFail "blowHim" f
	, Choices $ case mood of
		Easy ->
			[ (down, WillingBlow Easy)
			]
		Sadist ->
			[ (down, WillingBlow Sadist)
			, ("how abt you make me", ForceBlow)
			]
	]
	where
		down = case locoplan b of
			Basic Biped -> "get on yr knees"
			Basic x | x `elem` [Centaur, Quadruped] -> "kneel down"
			_ -> "get down"


forceBlow :: Encounter Index () SlimeGuy
forceBlow b s =
	[ Narration2 $ c "He grins at you -- slime lips peeling back to show the burnt-black and overgrown teeth of his skull, already always on display."
	, dialog s $ "That's how you wanna play it, huh?"
		/ Paragraph / "That's cool, I can go with that."
	, Narration2 $ "Then he steps in and backhands you across the face. The impact is sharp and hard, snapping your head to the side, and he follows it up by stepping in and grabbing your"
		/ case (hasHorns b, bodySpecies Head b) of
			(True, _) -> c "horns"
			(False, Just Human) -> c "hair"
			(False, Just x) -> c muzzle
			(False, Nothing) -> c "head"
		/ ", dragging you closer in. He twists you around, pinning one arm between your bodies and grabbing your other one, so that you're pressed tight together, the"
		/ case bodyMat_ Chest b of
			Solid (FMat _ Coat) -> "hair"
			Solid (FMat _ x) -> matword x
			_ -> "flesh"
		/ "all across your front prickling and itching. Hot fumes, sour-smelling, billow out from his body, rippling up through the air. He grins down at you and then spits right in your face, stinging pain trickling"
		/ case mouthType b of
			Mouth -> c "across the bridge of your nose"
			_ -> "across your" / muzzle
		/ "to dribble down your jaw."
		/ Paragraph / "The slime burns, acidic, foaming up slightly everywhere it touches flesh; even your breath does it, exhale sending a patch of his neck blistering up into frothy patches that slowly slide down over his shoulder."
		/ Paragraph / "He uses his free hand to grab you by the throat, fingers digging in as he bears down. He's actually really strong, for somebody made entirely out of slime. You could break out of his grip if you" / em "really wanted to" / ", but you don't, so you let him shove you down to your knees, right in front of his half-hard cock."
		/ Paragraph / "He keeps one hand splayed across your neck. He uses his other hand to grab his cock and slap it across your face, droplets of acid pre stinging everywhere they land. His cock is" / em "heavy" / ", dense and weighty when he rests it across your face, leaking burning pre all down your face."
		/ Paragraph / "You're aware you're open-mouthed and panting, the bubbling froth of his acid slime loud in your ears as it sluices down your flushed face, painfully sour when it spills into your mouth. He drags his cock over your face, smearing slime up and down, thick cords of it spilling down your neck and across your chest, the itching turning into a burning as it soaks in."
		/ Paragraph / "He shoves two fingers into your mouth, keeping it pried open as he shoves his cock inside, sharp sour taste bursting across your tongue." / filmyBj
		/ Paragraph / "He lets you sloppily suck on his cockhead for a minute, drooling out frothing, hissing globs of acid, but he" / sharedHeadGrab b s
	, dialog s $ "C'mon, actually suck it."
	, Narration2 $ sharedHeadGrab2 / "You gag, and his cock shudders in your throat -- he" / hasHorns ? "grips your horns tighter and shoves your head" ?: "grabs your head and shoves it" / "down deeper."
		/ Paragraph / "He groans, leaning over you -- mouth open, a thick cord of spit forming from his lower lip and stretching lower in a fat bead. He spits messily, wiping his face with the back of his hand, and his drool cracks across your face, smearing all over your face. He jerks his hips forward, slapping your face into his crotch, and spits again, a wet slap across your head as he smears his spit"
		/ case bodyMat_ Head b of
			Solid (FMat _ x) | x `elem` [Fur, Coat] -> "into your" / matword x
				| x `elem` [Skin, Scales] -> "across your" / matword x
			_ -> c "over your skin"
		/ ", working it in with his slimy fingers."
		/ Paragraph / "His cock throbs in your throat, pulsing steadily, and he drags your spit-and-tear-slathered face back, letting his cockhead pop back up into your mouth with a lurch, and your wet gags finally produce, coughing out his cockhead with a wet gush of slime, bursting up from your abused throat and splattering down the length of his cock, webbing in cords across his thighs. More ooze burbles over your lips, and he lets you heave for a while, jabbing his cock just far back enough to keep you gagging: eyes watering, nose clogged with acid slime, a mess continually spilling over your bruised lips."
		/ Paragraph / "Then his grip on the back of your head tightens, a fraction-of-a-second motion before he rams your head back down, shaft plunging back down into your throat with a warbling squawk, and your face smacks against his crotch, the ooze you just vomited up smearing across your face, tethering you to him with finger-thick cords of frothy, bubbling slime."
		/ Paragraph / "He holds you down for a while, hips jerking forward minutely with each sick warbling gag you make, stirring his cock in your throat, before he starts to thrust again, humming in pleasure each time you gag and vomit up the latest batch of acid pre burning your throat, using his other hand to smear it back across your face in thick planes: the cords between you slurping and squelching as he hammers your face down against his crotch, stray strings of slime splattering in every direction; pulling back as thick filmy planes spun all across your slime-slathered face."
	, dialog s $ "Getting close."
		/ Paragraph / "Gonna come down your tight little throat."
	, Narration2 $ "Then he just holds you in place, both hands wrapped around your head, fucking the final inch of his cock back and forth. His cock shudders with each sick, warbling noise that comes from your aching, fucked-open throat."
		/ Paragraph / "It's almost impossible to tell when he actually comes: his cock jerks and pulses, the spill of his load just gushing into the dense froth of ooze already clogging your throat, only sluggishly dripping down the back of your throat in a sour line. You gag more, stomach hollowing, spitting up his load across his crotch, and he catches the smears off his body and feeds them over your lips, fingers stroking his shaft as he pokes globs of come into your tongue, sour and burning."
	, dialog s $ "Doing good."
	, Narration2 $ "he says, keeping his oozing cock lodged in your throat, even as you heave and gag, coughing up frothing ooze, face flushed and sweaty, tears streaming down your face. He keeps going, grip shifting on your" / hasHorns ? "horns" ?: "head" / ", just idly fucking your throat with his softening, rubbery cock. You gag continuously, sick wet coughs smearing gummy lines of phlegm all over his shaft and hips, and he just keeps at it."
		/ Paragraph / em "Then" / "he pulls out, limp cock thickly corded with viscous, clumpy clots of come, more and more burbling up your throat and spilling down your jaw until he finally pulls you all the way off, leaving your mouth yawning open with tendrils of slime still anchored deep down in your throat, spanning all across his cock, raking their way up when he tugs his cock to the side and slaps it across your face, grinding it down to work his come into your skin."
	, dialog s $ "Good" / case gender b of
				He -> "boy"
				She -> "girl"
				-- lolll
				It -> "job"
				They -> "job"
				f -> bodyFail "forceBlow" f
			/ "."
	, Narration2 $ "The slime pats your head, touch smearing slime more into your soaked skin."
		/ Paragraph / "He milks his cock, fingers tugging at his rubbery length, and squeezes out a final drool of slime, dribbling across your forehead and spilling down over your eyes in a burning wash before he steps back, leaving you panting wetly, your front slathered in a gross mess of frothy acid come."
		-- / Paragraph / "{You think you came in your pants.}"
	, dialog s $ "You good?"
	, Narration2 $ c "The slime guy looks down at your disheveled form, his lips still pulled back in a lopsided smirk. He leans in, extending a hand."
	, FauxChoice "let him give you a hand up"
	, Narration2 $ "The touch just burns your" / hand / "more, but whatever. You stand up, swaying a little on your" / feet / ". The slime guy tips your head up, running his fingers over the chapped, flaking flesh across your neck, the touch just working more acid into the burns. You hiss in pain and he lets out a little laugh, but doesn't stop doing it."
	, dialog s $ "You look awfully fucked-up,"
	, Narration2 $ c "is what he says."
	, dialog s $ "So I guess I did a good job, huh?"
	] <> bjCloser b s

filmyBj :: String
filmyBj = "He's got a kind of rubbery, filmy coating all over his body, a skin membrane, but it's pinned back behind the bloated ridge of his cockhead, foreskin-like, in wrinkled folds, and raw acid slime drools from the tip. You lick across his cockhead, having to struggle to swallow even the spit-diluted solution. His cock shudders, slime coursing under his skin, slowly hardening in your mouth, a continual drip of foaming, stinging pre spilling across your tongue and down your throat in a burning trail."

bjCloser :: Encounter Index () SlimeGuy
bjCloser b s = case sexPref s of
	Top ->
		[ Narration2 $ c "He doesn't really give you time to respond, though:"
		, dialog s $ "Hey, how about next time you lemme fuck up your ass?"
		, Choices
			[ ("yeah sure", NextTimeAccede WreckAss)
			, ("i'll think abt it", NextTimeConsider WreckAss)
			, ("nah", NextTimeReject WreckAss)
			]
		]
	Bottom ->
		[ Narration2 $ c "He doesn't really give you time to respond, though:"
		, dialog s $ "Hey, how about next time you fuck" / em "me" / "?"
		, Choices
			[ ("yeah sure", NextTimeAccede YouTop)
			, ("i'll think abt it", NextTimeConsider YouTop)
			, ("nah", NextTimeReject YouTop)
			]
		]
	Switch ->
		[ FauxChoice "yea real good"
		, Narration2 $ c "Your voice is all fucked-up. He laughs."
		, dialog s $ "Gonna lemme kiss it better?"
		, Narration2 $ c "He leers, probably already fantasizing about shoving his burning tongue down your throat."
		, dialog s $ "Too bad I gotta get back to work sometime. Wanna see just how busted up I can get you, but that'll haveta be some other time."
			/ Paragraph / "Oh well, next time, right?"
		, FauxChoice "yea next time"
		, dialog s $ "See you then."
		]

nextTimeAccede :: Move -> Encounter Index () SlimeGuy
nextTimeAccede _ b s =
	[ Narration2 $ c "Your voice is all fucked-up. He laughs."
	, dialog s $ "Hey, nice."
	, Narration2 $ c "He socks you in the shoulder, lightly."
	, dialog s $ "See you then, yeah?"
	]

nextTimeConsider :: Move -> Encounter Index () SlimeGuy
nextTimeConsider YouTop b s =
	[ dialog s $ "Hah, hey, don't push yourself."
		/ Paragraph / "Some people are just dedicated bottoms."
	, Narration2 $ c "He shrugs."
	, dialog s $ "Don't worry, I'm happy to keep fucking you up as long as you want me."
	]
nextTimeConsider WreckAss b s =
	[ dialog s $ "Hah, hey, don't push yourself."
		/ Paragraph / "Maybe just a finger. Or a fist."
	, Narration2 $ c "He shrugs."
	, dialog s $ "Don't worry, I'm happy to just fuck up your throat."
	]

nextTimeReject :: Move -> Encounter Index () SlimeGuy
nextTimeReject YouTop b s =
	[ Narration2 $ c "He laughs."
	, dialog s $ "A dedicated bottom, huh."
		/ Paragraph / "Don't worry about it; I'm happy to fuck you up as much as you like."
	]
nextTimeReject WreckAss b s =
	[ Narration2 $ c "He laughs."
	, dialog s $ "Only wanna get throatfucked, huh?"
	, Narration2 $ c "He leers, groping his cock, still drooling with spit and slime."
	, dialog s $ "Don't worry, I'm happy to do that as much as you like."
	]

sharedHeadGrab :: BodyGlob -> SlimeGuy -> EventReel
sharedHeadGrab b s =
	"gets tired of that after a while though: he"
		/ case (hasHorns b, bodySpecies Head b) of
			(True, _) -> "grabs your horns"
			(False, Just Human) -> "grabs your hair"
			(False, Just a) | hasTopEars a -> "grabs the scruff between your ears"
			_ -> "grabs your head"
		/ "and drags you up, bringing you face-to-face with his cockhead, smearing your lips with fresh acid slime."

sharedHeadGrab2 :: String
sharedHeadGrab2 = "He smacks it against your lips, splattering droplets of pre that immediately start frothing up the second they touch your skin, drenching your lips and teeth in sour acid slime. You lap around the tip, tugging it into your mouth, and he thrusts forward, beating his cockhead against the back of your throat. Fully hard, it's still slightly rubbery, and it warps easily enough to burst down your throat. Your mouth is flooded with acid, throat burning, and it foams and froths up all over, spilling down your chin in a grimy slurry. His cock is a long line of buzzing, burning heat in your throat, steadily leaking frothy pre."



willingBlow :: Mood -> Encounter Index () SlimeGuy
willingBlow mood b s =
	[ Narration2 $ "You get down in front of him," / muzzle / "pressing against his cock, lolling thick but still limp across the side of your face. The slime burns, acidic, foaming up slightly everywhere it touches flesh; even your breath does it, exhale sending a patch of his thigh blistering up into frothy bubbles that sluggishly spill down his leg."
		/ Paragraph / "It's not precisely a surprise that his cockhead tastes sour -- acid, after all -- but" / em "how" / "sour is a surprise." / filmyBj
		/ Paragraph / "You lap down his shaft, leaving the tip to drizzle foaming pre across" / case mouthType b of
			Mouth -> c "your cheek"
			_ -> "the side of your" / muzzle
		/ "as you spread your lips over the side of his shaft, until you're pressed against the fold of his thigh, his balls -- vaguely defined lumps -- pressing against your jaw."
		/ case material s of
			Refine -> "The crystal pellets inside his shaft buzz under your tongue, minute electric pulses running through the slime and sending them vibrating, a hot electric spreading across your mouth as you drag your tongue up and down the underside of his cock."
			Dissolve -> mempty
		/ Paragraph / "The slime" / sharedHeadGrab b s
	, dialog s $ "C'mon, actually suck it."
	, Narration2 $ sharedHeadGrab2 / "He grins down at you -- slime lips peeling back to show"
		/ case material s of
			Dissolve -> "the burnt, blackened teeth of his skull"
			Refine -> "the glittering, mineral-encrusted teeth of his skull"
		/ ", already always on display -- and grinds your head against his crotch, cock throbbing in your throat."
		/ Paragraph / "He thrusts a little, experimentally at first -- and before your sloppy gags lube up your throat with phlegm and acid slime -- and then faster, wrapping both hands around your head as he grinds forward, shoving his heavy, rubbery cock back and forth. Your throat warps under the assault, the slime's brutally-thick cock leaving a burning ache in your throat even beyond the acid."
	, dialog s $ "Getting close."
		/ Paragraph / "Gonna come down yr tight lil throat."
	, Narration2 $ "His cock throbs and thickens in your throat in time, until you're warbling and coughing from the sheer thickness. The slime makes a few more short thrusts, grinning down at your flushed, slime-coated face and his cock shudders in your throat before it erupts, oozing cords of come down your throat, forming a burning pit in your stomach. He tips his head back, keeping your head clamped in place as he ruts back and forth, just dragging the fat crest of his cock back and forth through your burning throat, gushing out a mass of slime that clings and piles up in your throat, until it feels like you're flooded from throat to stomach, his cock just stirring up and down through the gummy mess."
		/ Paragraph / "He pulls out, more just letting go of your head and letting your gags spit it out along with a messy gush of come. You cough and sputter, coughing up more come, and you wipe your bruised lips with the back of your hand, smearing a long line of acid come across your arm."
	, dialog s $ "Good" / case gender b of
			He -> "boy"
			She -> "girl"
			-- :VVVV
			It -> "job"
			They -> "job"
			f -> bodyFail "willingBlow" f
		/ "."
	, Narration2 $ "The slime pats your head, touch smearing slime more into your soaked"
		/ case bodyMat_ Head b of
			Solid (FMat _ x) -> matword x
			_ -> "skin"
		/ "."
	, dialog s $ "I guess that worked out, huh?"
	] <> bjCloser b s


bottomAss :: Mood -> Encounter Index () SlimeGuy
bottomAss mood b s_ =
	case mood of
		Easy ->
			[ dialog s_ $ "Sure, we can do that."
				/ case locoplan b of
					Basic Slithering -> Paragraph / "Lemme see your underbelly, c'mon, flip over."
					_ -> mempty
			, Narration2 $ case locoplan b of
				Basic Slithering -> "You slither up across the rocky ore pile before you squirm and flip over, the" / case bodyMat_ Tail b of
						Solid (FMat _ Scales) -> "delicate scales"
						Solid (FMat _ Protoplasm) -> "sagging curves of protoplasm"
						Solid (FMat _ Skin) -> "soft skin"
						Solid (FMat _ Fur) -> "thin fur"
						Solid (FMat _ Metal) -> "hard metal"
						_ -> "flesh"
					/ "of your underbelly on display as he leans in."
				Basic x -> "He jerks his head to the side, to the sizzling, steaming heap of rock slurry."
					/ case x of
						_ | x `elem` [Centaur, Quadruped] -> "He guides you forward until you're nearly astride the rock pile and he's got a good angle on your ass."
						_ -> mempty
				f -> bodyFail "bottomAss" f
			]
		Sadist -> case locoplan b of
			Basic Slithering ->
				[ dialog s_ $ "Yeah, I can do that."
					/ Paragraph / "C'mon, lemme see your" / hasNagaCloaca ? "cloaca" ?: "ass" / "."
				, Narration2 $ c "He says, and then bodily flips you over, flopping you backwards in a startled heap all across the rocky ore pile."
				]
			Basic x
				| x `elem` [Biped, Naga] ->
					[ dialog s_ $ "Yeah, I can do that."
					, Narration2 $ c "He shoves you back, sending you sprawling across the pile of ore. Rocks dig into your back, smaller pebbles skittering down the pile from the motion."
					, dialog s_ $ "Gimmie yr" / hasNagaCloaca ? "cloaca" ?: "ass" / "."
					, Narration2 $ "You sprawl"
						/ hasLocotype Biped ? "on your stomach" ?: "back"
						/ "on the rocky pile, and the acid immediately starts itching and burning all across your"
						/ hasLocotype Biped ? "front" ?: "back"
						/ "."
					]
				| x `elem` [Centaur, Quadruped] ->
					[ Narration2 $ c "He slaps a hand across your side, trailing back to your ass."
					, dialog s_ $ "Gimmie your ass."
					, Narration2 $ c "His fingers probe across your flanks, digging between your cheeks, and he keeps guiding you on until you're astride the rock pile, legs spread."
					]
			f -> bodyFail "bottomAss" f
	<>
	[ Narration2 $ case locoplan b of
		Basic x | x `elem` [Naga, Slithering] -> "He presses his hands across your" / hasLocotype Naga ? "stomach" ?: "underbelly" / "and slides them down, the slick ooze residue his touch leaves behind rapidly frothing up into a bubbling, stinging lather."
			/ case dickLocation b of
				Just DickSheathed -> "He slides a finger around the rim of your sheath before shoving two inside, fingers smearing burning slime" / case cockCount b of
						1 -> "all around the cap of your cockhead"
						2 -> "around the tips of your twin cocks"
						_ -> "over the flesh of your cockheads, all packed together"
					/ ", before he pulls out with a slurp and slides an inch down, shoving at your asshole."
				Just DickExternal -> "He knocks his knuckles against your" / Cock^s / "on his way down, wrist bopping against your taut balls, mostly ignoring" / "it"^p "them" / "as he reaches down lower and shoves two fingers into your asshole."
				_ -> "He smears his fingers over the crest of your vent, fingers stroking up and down its lips before he spreads them minutely and shoves two fingers in," / case cockCount b of
						1 -> c "sliding his fingers in past the head of your still-sheathed cock"
						2 -> c "curling his fingers around your twin cocks and sliding deeper along their shafts"
						0 -> c "rooting around inside you"
						q -> "digging against the packed-tight mass of your" / number q / "cocks, having to spread them apart to push deeper"
					/ "."
			/ "He grins as you jerk against him, hand spread over your side as he shoves another finger into your"
			/ hasCloaca ? "cloaca" ?: "ass"
			/ ", spreading the"
			/ hasCloaca ? "lips" ?: "ring"
			/ "wider as he starts to fingerfuck you, his acid slime buzzing and frothing across your inner walls, a stinging heat blooming inside as the ooze burns deeper. He shoves his fingers in to the knuckle, letting a frothy squirt erupt out of you, splattering all across his hand as he works more and more into your spread"
			/ hasCloaca ? "cloaca" ?: "asshole"
			/ "."
			/ Paragraph / hasNagaCloaca &*& hasDick
				? ("Your" / Cock^s / Shudders^s / " against his probing fingers, somehow managing to flush and harden even coated in burning ooze. The slime grins down at you and curls a finger around your cock, tugging -- manually yanking" / "it"^p "them" / "from your cloaca,"
				/ case (cockCount b, bodySpecies Genitals b) of
					(q, Just Snake) | q > 1 -> "one hemipene after the other"
					_ -> mempty
				/ "until"
				/ "it's"^p "they're" / "splayed across your underbelly, limp and heavy, shuddering as he returns to fucking your slit, fingers sliding"
				/ case dickLocation b of
					Just d | d `elem` [DickExternal, DickSheathed] -> hasExternalBalls
							? "across your balls"
							?: "across the root of your cock"
						/ "and plunging back into your cloaca"
					Just _ -> "down" / "your"^p "one of your" / Shaft^s / "all the way back inside your" / sheath
					f -> bodyFail "bottomAss" f
				/ ".")
		Basic x | x `elem` [Biped, Centaur, Quadruped] -> "The slime presses his hands against"
			/ case x of
				Biped -> c "the backs of your thighs"
				_ -> "your" / hips
			/ "and trails them up to your ass, leaving a trail of itching, burning slime. His fingers slide between your cheeks, pressing against the tight knot of your asshole, digging in. His slime froths up into foaming globs, almost instantly burning."
			/ Paragraph / "He spreads a hand across your hip"
			/ case locoplan b of
				Basic x | x `elem` [Centaur, Quadruped] -> hasTail ? ", tugging your tail aside"
				_ -> mempty
			/ ", the other probing deeper into your ass. There's a sharp shock of pain as the acid dribbles all down inside at his touch. His fingers twitch, rubbery and dense as he fingers your ass, each motion slick and slippery but also scoring a line through your ever-more-inflamed skin."
			/ Paragraph / "He leans in, slime pressing against your thighs all wet and burning, and his fingers glide across the burnt, stinging flesh of your spread asshole, pushing in and out."
		f -> bodyFail "bottomAss" f
		/ Paragraph / "He pulls back"
			/ hasNagaCloaca
				? (hasDick
					? (", thumb pressed against the" / case cockCount b of
							1 -> "underside of your cock"
							_ -> "conjoined swell where your shafts come together"
						/ ", keeping your lips spread uncomfortably wide")
					?: ", thumb just-so parting your flushed lips")
				?: ", thumb notched against the loose, sloppy hole of your asshole"
			/ ", and just smears his fingers over your"
			/ case bodyMat_ LowerBack b of
				Solid (FMat _ x) -> c $ matword x
				Trans (FMat _ x) _ -> matword x / ", under your tail"
			/ ". He grinds down against you, his cock a sudden hot heavy wedge rolling over your hip, his body pressed against your side, leaving a burning sting everywhere you touch."
		/ Paragraph / "His thumb rolls up and down, digging into the already-swollen skin, bloated and inflamed, parting your"
		/ hasLocotype Naga &*& hasCloaca ? "lips" ?: "hole"
		/ "further, tugging until it hurts."
	, dialog s_ $ "Y'got a nice hole, don't you?"
		/ Paragraph / "How hard d'you want me to ruin it?"
		/ Paragraph / "You wanna try to take my whole hand?"
	, Choices
		[ ("cmon just shove yr dick in me already", FuckBasic)
		, ("yeah fist me", Fisted mood)
		]
	]

fuckBasic :: Encounter Index () SlimeGuy
fuckBasic b s =
	[ dialog s $ "Oh yeah? You just wanna take my load?"
		/ Paragraph / "You asked for it."
	, Narration2 $ "The slime rolls his hips, fat cock lurching up to loll across"
		/ case locoplan b of
			Basic Biped -> "your ass"
			Basic x
				| x `elem` [Naga, Slithering] -> "the span of your underbelly"
				| x `elem` [Quadruped, Centaur] -> "your flanks"
			f -> bodyFail "fuckBasic" f
		/ ". He's got a kind of rubbery, filmy coating all over his body, a skin membrane, but it's pinned back behind the bloated ridge of his cockhead, foreskin-like, in wrinkled folds, and raw acid slime drools from the tip. It's grossly fat, nearly as thick as one of his wrists. It oozes pre continually, forming a messy stream of bubbling, fuming slime that soaks down your side, itching and burning."
		/ Paragraph / case locoplan b of
			Basic x | x `elem` [Biped, Naga, Slithering] -> case x of
					Biped -> c "The slime lines up behind you, palm grinding the root of his cock against your hole as the fat shaft shudders, spanning up to the small of your back. He's leaving heavier smears of slime now that he's aroused and dripping, spilling slimy ooze all over your ass in wet droplets that leave a wake of itching and burning. His fingers dig into your ass, bending his cock around to follow so that his slobbery cockhead is between his fingers, pressed right against your spread hole. He shoves in, ramming forward, burying the entire length of his cock into you in a single thrust."
					_ -> "The slime straddles your underbelly, rutting his cock"
						/ hasDick ? "against yours" ?: "against your spread lips" -- if you don't have a dick then you have a cloaca by default
						/ "for a second before he tugs it down into place, cockhead drooling burning slime into"
						/ hasCloaca ? "your vent" ?: "your gaping asshole"
						/ ", fingers keeping you spread so he can fill you up, the thick weight of his pre slowly piling up inside you. Then he uses his fingers to aim his cockhead down and shoves in, ramming forward, burying the entire length of his cock into you in a single thrust."
				/ Paragraph / "He groans, eager, and thrusts fast, hands gripping your"
				/ hasLocotype Biped ? "ass" ?: "sides"
				/ "as he starts fucking you hard, no warm-up. His burning slime makes good lube: thick and stringy, slurping and squelching. You" / groan / ","
				/ hasNagaCloaca ? "cloaca" ?: "ass"
				/ "feeling like it's on fire, the burning heat spreading in sharp stabbing jolts inside you, acid gurgling as it smears over your inner walls. Slime gushes from your inflamed"
				/ hasNagaCloaca ? "slit" ?: "ass"
				/ "with each thrust, squirting out in ropes when the slime slams in. His cock throbs; slick, slimy flesh nevertheless tearing and scraping all along your insides, the ridge of his cockhead forcing a gummy, semisolid mass of ooze back and forth."
				/ Paragraph / case x of
					Slithering -> "You lurch and wrap part of your upper body over his shoulders, and he responds by groaning and driving in harder, the loop of your body pressed all up his stomach and chest. He pulls you closer with one hand, settling your weight across his back, his other hand still braced on your side, fingers just-barely smearing across" / hasCloaca ? "your gaping cloaca" ?: "your fucked-up asshole" / "."
					_ -> "He leans in, chest pressing against yours, the wet slime cool and slick for a second before the burning starts, and presses his lips to yours in a sloppy kiss, drooling burning slime into your mouth as he hunches forward, rhythmic slurping noises coming from his gummy cock, slime squirting from your" / hasCloaca ? "cloaca" ?: "ass" / ", his lips smearing over your" / muzzle / "."
				/ Paragraph / "His entire body starts to drool, decohering slightly as his thrusts pick up the pace, each impact making a wet" / em "smack" / "and spraying acid droplets all over your body, burning wherever they land. He groans, the sound gurgling like his throat is full of water, and his cock throbs, flexing as he comes. His spurts become a continual pouring, part of his acid body just gushing inside in wet streams, flooding you with its weight. His thick, tarry load gurgles out, piling up against the inside of your"
				/ hasNagaCloaca ? "cloaca" ?: "ass"
				/ "for a second; your"
				/ hasNagaCloaca ? "lips are" ?: "ring is"
				/ "so inflamed"
				/ hasNagaCloaca ? "they" ?: "it"
				/ "nearly"
				/ hasNagaCloaca ? "seal" ?: "seals"
				/ "his cock in; your skin bloats for a moment, pressure building, before the volume becomes too much and his load spurts out, pouring in foaming tracks"
				/ hasLocotype Biped ? "down your thighs" ?: "over your sides" 
				/ "."
			Basic x | x `elem` [Quadruped, Centaur] -> "He lines up behind your haunches, cock dragging across your flanks, leaving heavier smears of slime now that he's aroused and dripping. He dips down, pressing a sloppy kiss to your asshole, tongue flitting across your hole, spitting a wet mess of slime all across your puckered hole before he rises up, fingers still probing at your hole before his cockhead presses against it. He's slick and slimy, the substance of his body dense and solid but still warping and compressing in ways utterly unlike flesh. His cock spurts, another mess of slime gurgling across your ass, spilling in a frothy mess into your spread hole. He jerks forward, cock smearing across the fat furl of your asshole before it pushes all the way through, cockhead sinking inside, keeping the next gush of hot pre spilling across the inner walls of your ass."
				/ Paragraph / "The slime's entire body is oozing and dripping nearly as much as his cock is. He groans, hands grabbing at your flanks, digging into the meat of your hips as he jerks forward again, hips slapping against your ass and forcing a slimy eruption of ooze from your ass, splattering back across his thighs, smearing between you. Each thrust bursts inside you, a heavy gush of slime painting in ropy cords across your insides, rapidly filling and then flooding out; each thrust forces a wet mess of slime from your ass, splattering across your hindquarters, drooling in slimy ropes down your haunches."
			f -> bodyFail "fuckBasic" f
		/ Paragraph / "The slime stays hilted in you for a while, just stirring his cock through the mess of come even after he's gone half-hard, and when he pulls out his cock comes out corded with ropes of come, spilling out over your destroyed"
		/ hasNagaCloaca ? "cloaca lips" ?: "anal ring"
		/ "."
		/ Paragraph / "He feels at your flesh, fingers digging into their dense, pulpy weight."
	, dialog s $ "Yeah, you make a pretty good cumdump."
	, Narration2 $ c "He slaps your side, sending up a spray of slime."
	, dialog s $ "Good job."
	] <> fuckCloser b s


fisted :: Mood -> Encounter Index () SlimeGuy
fisted mood b s_ =
	[ Narration2 $ "The slime huffs out a short laugh, but while he's doing it he's also shoving his fingers in you, deeper, twisting them around. His gel-body is solid but it's not" / em "that" / "solid; under the burn and bubbling froth you can feel the crest of his hand warp as he tries to work it inside your"
		/ hasNagaCloaca ? "spread cloaca" ?: "gaping hole" / "."
		/ Paragraph / "He pulls back, fingers sliding out with a gush of froth, and plays his fingers around"
		/ hasNagaCloaca ? "your swollen rim" ?: "your flushed lips"
		/ ", working a finger or two back and forth inside"
		/ hasNagaCloaca ? (case cockCount b of
			1 -> ", sliding them up against the embedded shaft of your cock"
			2 -> ", digging down in the divot between your shafts"
			_ -> ", digging between your shafts")
		/ ". He taps his fingers against your inner walls, each touch a shock of burning, slime frothing up in a foaming mess inside you and spilling in a froth"
		/ case locoplan b of
			Basic Biped -> c "down your thighs"
			Basic x
				| x `elem` [Naga, Slithering] -> c "across your underbelly"
				| x `elem` [Quadruped, Centaur] -> "down your hind legs" / hasExternalBalls ? ", burning across your hanging balls"
			f -> bodyFail "fisted" f
		/ "."
	] <> case mood of
		Easy -> 
			[ dialog s_ $ "Shouldn't be that hard to get you open."
			, Narration2 $ "He pushes in a little more, gel warping, and the crest of his hand slops inside with a wash of goo, your"
				/ hasNagaCloaca ? "inner cloaca" ?: "asshole"
				/ "reflexively clenching hard around it."
			]
		Sadist ->
			[ Narration2 $ c "He leans in close, hissing in your ear:"
			, dialog s_ $ "I'm gonna wreck you hard, I hope you know."
			, Narration2 $ "And he punctuates it with a twist, slopping the crest of his hand into your"
				/ hasNagaCloaca ? "cloaca" ?: "ass"
				/ "and laughing softly when your entire body jerks, muscles clenching hard around his hand."
			]
	<> [ Narration2 $ "He twists his hand around slowly, more sliding in until he's got his entire hand, curled up into a fist, lodged"
		/ hasNagaCloaca ? "past your enflamed lips" ?: "inside your ass"
		/ ". He works it around a little, knuckles digging into your flesh. He knocks against"
		/ hasProstate ? "your prostate" ?: "your inner walls"
		/ case locoplan b of
			Basic x | x `elem` [Naga, Slithering] -> "from the front"
			_ -> mempty
		/ "and you" / groan / "lowly; he laughs a little again and presses down" -- was {groan/croon}
		/ hasDick
			? ", grinning as your cock shudders"
			?: (", grinning as your" / hasNagaCloaca ? "cloaca" ?: "asshole" / "convulses around him")
		/ "."
		/ Paragraph / case hairStyle s_ of
			BraidsLong -> "He swipes his hanging braids out of his eyes (not that he actually has eyes) and"
			_ -> "He"
		/ "leans in, other hand steadying himself on your"
		/ case locoplan b of
			Basic Slithering -> c "coils"
			Basic x
				| x `elem` [Biped, Naga] -> c "hip"
				| x `elem` [Centaur, Quadruped] -> c hips
			f -> bodyFail "fisted" f
		/ ". He looks down at your spread"
		/ hasNagaCloaca ? "lips" ?: "ass"
		/ ", flexing against his wrist, frothy burbles of slime slurping out when your body convulses and clamps down, sloppy sucking noises coming out of your wrecked hole."
	] <> case mood of
		Easy ->
			[ dialog s_ $ "Easy does it."
			, Narration2 $ "He grinds down again, pushing the crest of his knuckles against"
				/ hasProstate ? "your prostate" ?: "your g-spot"
				/ "and just leaving it there, a constant pressure sending out frissions of pleasure, even as the burning ache of his hand shoved up"
				/ hasNagaCloaca ? "inside your cloaca" ?: "your ass"
				/ "grows and grows."
				/ hasDick ? ("Your" / Cock^s / Leaks^s / case cockCount b of
						1 -> ", drooling a mixture of pre and come"
						_ -> "in messy cords of mingled pre and come"
					/ ", twitching weakly as" / "it"^p "they" / Spits^s / "out more"
					/ case locoplan b of
						Basic x
							| x `elem` [Biped, Naga, Slithering] -> c "all across your stomach"
							| x `elem` [Centaur, Quadruped] -> ", drooling from the" / Tip^s / "down to the ground"
						f -> bodyFail "fisted" f
					/ ".")
				/ Paragraph / "He brings you off slow, just slick, burning pressure all across your insides, slime spurting out"
				/ hasNagaCloaca ? "between your spasming lips" ?: "of your sucking hole"
				/ "as you get closer and closer to coming in agonizing increments, tension cresting and cresting until with a shaky gasp"
				/ hasDick
					? ("your" / Cock^s / Twitches^s / "and" / Starts^s / "pumping, drooling out come in streamers"
						/ case locoplan b of
							Basic Slithering -> "that drizzle across your looped coils and spill down across the rocky floor"
							Basic x | x `elem` [Biped, Naga] -> "that spurt across your stomach and drip down your sides"
								| x `elem` [Centaur, Quadruped] -> "that splatter across the rocky floor"
							f -> bodyFail "fisted" f
						/ ", more and more and more, an ejaculation that lasts for minutes with his fingers burrowing into" / hasProstate ? "your prostate" ?: "your g-spot" / ", leaving your inner flesh scorched and raw.")
					?: (hasNagaCloaca
						? ("you come," / sheath / "clamping down around his hand, pouring slimy juices out across his hand in slow, rippling pulses, making lewd sucking noises as he digs his fingers in against your shuddering walls, making your pulsing" / sheath / "squirt all up his arm and over his chest.")
						?: "you come, shaking.")
				/ Paragraph / "His hand slips out with a muffled" / em "slurp" / ", followed by a squirt of slime that becomes a continual spill; the frothing mess drools out over your slack lips, all swollen and bloated from the use. He works his fingers across your bloated"
				/ hasNagaCloaca ? "opening" ?: "ring"
				/ ","
				/ case bodyMat_ (Waist Hind) b of
					Solid (FMat _ Skin) -> c "smearing the sluggishly-spurting slime all over your flesh"
					Solid (FMat _ m)
						| m `elem` [Fur, Coat] -> c "grinding your sluggishly-spurting slime into your coat"
						| otherwise -> "smearing the sluggishly-spurting slime all over your" / matword m
					Trans (FMat _ t) (FMat _ b) -> "smearing the sluggishly spurting slime up across your" / matword t / "and painting it down across your" / matword b
				/ "."
			]
		Sadist ->
			[ dialog s_ $ "I think you can take a lil more, huh?"
				/ Paragraph / "More importantly" / em "I" / "wanna get up in those guts, right."
				/ Paragraph / "So open up."
			, Narration2 $ "He grinds in, hand uncurling, fingers jabbing deeper"
				/ hasNagaCloaca &*& hasDick ? (", sliding up along the root of your" / Cock^s)
				/ ", and his wrist pushes in past your"
				/ hasNagaCloaca ? "lips" ?: "ring"
				/ ". He claws at your guts with his rubbery fingers, dragging burning lines across your inner flesh, shifting up and down as he shoves the first bit of his forearm in. Even with the burning and frothing his slime is still slick, and so he slides in, slime spilling from"
				/ hasNagaCloaca ? "between your lips" ?: "your hole"
				/ "in blurts, slathering his arm in a thicker layer of goop."
				/ Paragraph / "He draws back and punches in, ramming deep, groaning encouragingly at your full-body shudder. He does it again, fingers clawing at your guts, shoveling heavy masses of slime deeper up inside you. He jerks back, pulling out entirely, and the motion leaves your"
				/ hasNagaCloaca ? "opening" ?: "hole"
				/ "gaping: flush lips, bruised and battered, thick cords of slime spilling down your raw inner flesh and burbling out over his fingers. Then he punches back in, starting up a brutal rhythm: punching back and forth, a squirt of frothing slime erupting from"
				/ hasNagaCloaca ? "between your lips" ?: "your hole"
				/ "with each motion, splattering across his chest as he roots around, deeper and deeper."
				/ Paragraph / "You come with an agonized" / groan
				/ (if hasNagaCloaca b && dickLocation b == Nothing
					then c ", squirting hard around his forearm, your juices mixed in with his slime"
					else (case ballLocation <$> getBalls b of
							Nothing -> mempty
							Just External -> "balls lurching and pulling up abruptly, hard enough that the" / em "smack" / "of impact hurts"
							Just Internal -> c "your balls spasm and squeeze, internal muscles working them as you erupt"
						/ ". Your" / Cock^s / "shudder,"
						/ case locoplan b of
							Basic x | x `elem` [Centaur, Quadruped, Slithering] -> "slapping up across your underbelly"
								| x `elem` [Naga, Biped] -> "smacking across your stomach"
							f -> bodyFail "fisted" f
						/ "and erupt, spraying a mess of come"
						/ case locoplan b of
							Basic x | x `elem` [Centaur, Quadruped] -> c "across your underbelly, thick ropes smacking hard against your legs and splattering on the ground in front of you"
								| x `elem` [Naga, Biped, Slithering] -> "all up your body"
									/ case bodyMat_ Chest b of
										Solid (FMat _ Fur) -> ", catching in the fur of your chest,"
										Solid (FMat _ Coat) -> ", catching across your stiff coat,"
										Solid (FMat _ Scales) -> ", running in rivulets between your scales,"
										Solid (FMat _ Skin) -> ", drenching your already-sodden skin,"
										_ -> mempty
									/ "and splattering across your face, salty and hot when some dribbles into your mouth"
							f -> bodyFail "fisted" f))
				/ "."
				/ Paragraph / "He works his arm around, elbow-deep up inside"
				/ hasNagaCloaca ? "your battered cloaca" ?: "your ass"
				/ ", the slack, spread muscle gaping, a rubbery band of flesh spilling out from inside, half-prolapsed and caught around his forearm, pulsing as you shudder out your orgasm."
			, dialog s_ $ "See? I knew you liked it."
			, Narration2 $ hasProstate
				? ("He pulls back and rams his hand against your prostate,"
					-- / "{kneading its swollen mass, }" -- TODO: for bigger prostates (which i guess would be shared w/ bigger balls?)
					/ "milking out another few agonizing squirts of come, along with your shuddering" / groans / ", with each impact.")
				/ "He tugs, and his hand pops out without a catch, your spasming muscles practically spitting it out."
				/ hasNagaCloaca
					? "He runs his fingers over your lips, what started out as slim folds of flesh now grotesquely inflamed, giant puffy slabs too big to fit inside your body." -- was {, nearly entirely hiding your nubby cock/s in their bloated folds}, for small dicks
					?: "He runs his fingers over your asshole, what started out as a tight pucker now a bruised, gaping mess, ring bloated out between your cheeks, flesh bruised and swollen into thick, meaty slabs, all shiny and taut."
			, dialog s_ $ "Yeah, got a nice hole. Got it real fucked up."
			, Narration2 $ "He digs in again, folding his hand around your bruised, prolapsed lips, and squeezes, skin bloating out between his fingers. You hiss,"
				/ hasTail ? "tail shaking,"
				/ "and squirm away from him, and he laughs and smacks"
				/ hasNagaCloaca ? "across your grotesquely-swollen vent" ?: "your ass"
				/ ", leaving a sloppy handprint of slime in the middle of the mess."
			]
	<> [ dialog s_ $ "You took it real good. Good job. You good?"
		, FauxChoice "yea im cool"
		, dialog s_ $ "Hey, good."
	] <> fuckCloser b s_

fuckCloser :: Encounter Index () SlimeGuy
fuckCloser b s_ =
	[ Narration2 $ "He leans in to press a burning kiss across your"
		/ case locoplan b of
			Basic Slithering -> "side"
			Basic x | x `elem` [Biped, Naga] -> "shoulder"
				| x `elem` [Quadruped, Centaur] -> "flanks"
			f -> bodyFail "fisted" f
		/ ", and then steps back, watching the muscles of your"
		/ case locoplan b of
			Basic Biped -> "stomach and thighs"
			Basic x
				| x `elem` [Naga, Slithering] -> "underbelly"
				| x `elem` [Centaur, Quadruped] -> "underbelly and flanks"
			f -> bodyFail "fisted" f
		/ "shudder, sluggishly pushing out a sloppy trail of ooze."
		/ case locoplan b of
			Basic Biped -> c "He reaches out and helps you back up onto your feet."
			Basic Naga -> "He helps you sit up, the clench of your core sending another slimy gush of ooze spurting from your" / hasCloaca ? "cloaca" ?: "asshole" / "."
			Basic Slithering -> "He helps you flop back over, body sliding as you shimmy before twisting, letting out an involuntary" / groan / "when the bruised meat of your" / hasCloaca ? "blown cloaca" ?: "blown asshole" / "grinds down in puffy folds against the ground, pulsing with your heartbeat."
			_ -> mempty
		, dialog s_ $ "Well, y'know, I oughtta get back to work at some point."
			/ Paragraph / "You know where to find me if you want more."
		, Narration2 $ "He almost absently swipes his hand through the sloppy tracks of slime spilling from your" / hasNagaCloaca ? "vent" ?: "asshole" / ", catching a tarry handful, and turns back to the ore slurry, drizzling it over the half-dissolved ores."
			/ Paragraph / "It takes you a bit to get going, but you manage."
		]
