{-# LANGUAGE MultiParamTypeClasses #-}

module Sites.Hell.Events.Hubur.SuitedDemon
	( SuitedDemon(..)
	, deck
	, scene

	, name
	, bits
	, variants
	) where

import Prelude hiding ((/))

import Server.JSON

import Sites.Hell.Events.Construction ((/))
import Sites.Hell.Events.SceneUtils

import Sites.Hell.Events.Hubur

name :: String
name = "suiteddemon"

data SuitedDemon = SuitedDemon
	{ garland :: Garland
	}
	deriving (Eq, Ord, Show, Read)

data Garland = Garland | NoGarland
	deriving (Eq, Ord, Enum, Bounded, Show, Read)

instance Random SuitedDemon where
	randomR _ = random
	random = getRand $ SuitedDemon
		<$> genRandom

instance ToJSON SuitedDemon where
	toJSON (SuitedDemon g) = object
		[ pack "name" .= name
		, pack "data" .= [fromEnum g]
		]

bits :: Value -> Value
bits _ = object
	[ pack "name" .= name
	, pack "data" .= [Null]
	]

variants :: [(MapZone, ())]
variants =
	[ (FactoryProcessingPlant, ())
	]

instance FromJSON SuitedDemon where
	parseJSON (Object vs) = case (parse (.: pack "name") vs, parse (.: pack "data") vs) of
			(Success pName, Success [g])
				| pName == name ->
					case SuitedDemon
						<$> safeToEnum g of
							Nothing -> mzero
							Just a -> return a
			_ -> mzero
	parseJSON _ = mzero

deck :: RandomGen g => Rand g (Enumerable SuitedDemon)
deck = do
	garland <- liftRand $ shuffle [Garland, NoGarland]
	return $ SuitedDemon
		<$> from garland

data Index = Dummy
	deriving (Eq, Ord, Show, Read)

instance Scene Index () SuitedDemon where
	link f = case f of
		_ -> error "dummy index"

	display = fmap toLower . show
	vardisplay = fmap toLower . show

scene :: Encounter Index () SuitedDemon
scene _ c =
	[ description c
	-- , error "bad scene"
	]

description :: SuitedDemon -> Event SuitedDemon Index ()
description d =
	Narration2 $
		"A demon encased in a heavy, rubbery suit. It's not entirely clear if the suit is part of it."
		/ Paragraph / "It's demonic in shape: legs with an elongated ankle, like an animal's, with a leathery, reptilian clawed paw for a foot; a twining spade-tipped tail; and hands that end in sharp, shining claws -- but all sheathed in the clinging material of its suit, outlining every curve of its musculature."
		/ case garland d of
			Garland -> "Its sole article of clothing, if it could be said to be that over the suit, is a necklace of fist-sized pearls of fire, in a cord hanging from its neck."
			NoGarland -> mempty
		/ "Its head is obscured: the suit has a helmet, and the upper half of it is clear and bubble-like, but the inside is full of a dense red gas, constantly fuming, and nothing can be seen within."
		/ Paragraph / "The suit itself is off-white, marred with ash and black streaks, with parts of the material etched in blotches as if from acid. There are"
		/ case garland d of
			Garland -> "more"
			NoGarland -> mempty
		/ "pearls of fire embedded forcefully into the suit -- there are scars and burn marks surrounding each one -- and the only reason why they couldn't be interpreted as more damage is the regularity of their placement: three around the mound of their shoulders, connected in a circle, and a line down the inside of each arm, ending with a fat, burning lozenge in the palm of their hands; and then a more complex geometric design in loops and circles down their chest, with pearls on each pectoral, the outside of the hips, the crotch, and then down their legs to the ankle."
