{-# LANGUAGE MultiParamTypeClasses #-}
module Sites.Hell.Events.Hubur.SwampThicket
	( SwampThicket(..)
	, deck
	, scene

	, name
	, bits
	, variants
	) where

import Prelude hiding ((/), (^))

import Sites.Hell.Events.SceneUtils hiding ((/))
-- import Sites.Hell.Events.Construction ((/))
import Server.JSON

import Sites.Hell.Events.Hubur

name :: String
name = "thicket"

data Plants = Reeds | Briars | DeadTree | Fen
	deriving (Eq, Ord, Enum, Bounded, Show, Read)

data SwampThicket = SwampThicket
	{ plants :: Plants
	}
	deriving (Eq, Ord, Show, Read)

instance ToJSON SwampThicket where
	toJSON (SwampThicket p) = object
		[ pack "name" .= name
		, pack "data" .= [fromEnum p]
		]

bits :: Value -> Value
bits _ = object
	[ pack "name" .= name
	, pack "data" .= [Null]
	]

variants :: [(MapZone, ())]
variants =
	[ (SwampSobek, ())
	, (SwampZombie, ())
	]

instance FromJSON SwampThicket where
	parseJSON (Object vs) = case (parse (.: pack "name") vs, parse (.: pack "data") vs :: Result [Int]) of
			(Success pName, Success [p])
				| pName == name -> case
					SwampThicket
						<$> safeToEnum p of
							Just x -> return x
							Nothing -> mzero
			_ -> mzero
	parseJSON _ = mzero

deck :: RandomGen g => Rand g (Enumerable SwampThicket)
deck = do
	plants <- liftRand $ shuffle [Reeds, Briars, DeadTree, Fen]
	return $ SwampThicket <$> from plants

data Index = Dummy
	deriving (Eq, Ord, Show, Read)

data Var = LastHarvested
	deriving (Eq, Ord, Show, Read)

instance Scene Index Var SwampThicket where
	link f = case f of
		_ -> error "bad link"

	display = fmap toLower . show
	vardisplay = fmap toLower . show

scene :: Encounter Index Var SwampThicket
scene b z = describe b z -- <> error "bad scene"

--- actual event start

describe :: Encounter Index Var SwampThicket
describe _ s =
	[ Narration2 $ case plants s of
		Reeds -> c "Stiff growths of reeds erupt overhead, forming a mazy thicket of watery paths between overgrown bamboo islands. Within the maze, the plants deafen the constant hum of hell's heart, and their fronds blot out the twisting sky above. The only sign that you're still in hell is the unearthly shade of the water, glowing faintly from unknown sources."
		Briars -> c "The swamp is overgrown in a thicket of dense briars. Huge black stalks arch overhead, punctuated with thorns the length of a hand, and each with dozens of offshoots of finer stalks, knotted together in an impenetrable wall of thorns."
		DeadTree -> c "An old tree towers above you, its many gnarled limbs bare of any leaves. It grew half atop a boulder, on a sliver of the only dry land visible in the swamp. A mess of grasses sprout around its base, and creeper vines wind up its trunk, with hanging mosses still dangling from its lower limbs."
		Fen -> c "The swamp spreads out around you as a vast, flooded plain. Cattails and long grasses poke out above the water in ragged clumps, outlining the edges of sunken islands."
	, HarvestNode (Local LastHarvested) "There's a clump of herbs growing here."
			(pure <$>
				[ I LIONS_MANE 2 -- lion
				, I MANDRAKE 2 -- human
				, I GOATSBEARD 2 -- goat
				, I ADDERS_FORK 2 -- snake
				, I OX_EYE 2 -- bull
				, I CATTAIL 2 -- tiger
				, I HARTS_TONGUE 2 -- deer
				, I TONGUE_OF_DOG 2 -- wolf
				, I HORSE_HOOF 2 -- unicorn
				, I PIGS_TAIL 2 -- boar
				])
	]
