{-# LANGUAGE MultiParamTypeClasses #-}
module Sites.Hell.Events.Hubur.UmuDemon
	( UmuDemon(..)
	, deck
	, scene

	, name
	, bits
	, variants
	) where

import Prelude hiding ((/), (^))

import Data.Maybe

import Sites.Hell.Events.SceneUtils hiding ((/))
import Sites.Hell.Events.Construction ((/))
import Server.JSON

import Sites.Hell.Events.Hubur

import Data.Tree
import Description.TfNode (Animal(..), TfNode(..), Shape(..), nodeLabel)
import qualified Description.TfNode as TF (Part(..), Sex(..), Element(..))

name :: String
name = "umu-demon"

data UmuDemon = UmuDemon
	{ day :: Day
	}
	deriving (Eq, Ord, Show, Read)

data Day = Sunlight | Storm | Night | Clouds | Wind | Frost | Snow
	deriving (Eq, Ord, Enum, Bounded, Show, Read)

instance Random UmuDemon where
	randomR _ = random
	random = getRand $ UmuDemon
		<$> genRandom

instance ToJSON UmuDemon where
	toJSON (UmuDemon d) = object
		[ pack "name" .= name
		, pack "data" .= [fromEnum d]
		]

bits :: Value -> Value
bits _ = object
	[ pack "name" .= name
	, pack "data" .= [Null]
	]

variants :: [(MapZone, ())]
variants =
	[ (Field, ())
	, (Glade, ())
	]
instance FromJSON UmuDemon where
	parseJSON (Object vs) = case (parse (.: pack "name") vs, parse (.: pack "data") vs :: Result [Int]) of
			(Success pName, Success [d])
				| pName == name ->
					case UmuDemon
						<$> safeToEnum d of
							Nothing -> mzero
							Just a -> return a
			_ -> mzero
	parseJSON _ = mzero

genFauxTree :: UmuDemon -> Tree (Either String TfNode)
genFauxTree (UmuDemon d) = Node (Left $ "UMU-DEMON") $ catMaybes
	[ pure $ pure $ Right $ CP TF.Arms
	, pure $ Node (Right $ CP TF.Head) $ catMaybes
		[ pure $ pure $ Right $ CA Lion
		, pure $ pure $ Right $ CP TF.Horns
		]
	, pure $ Node (Right $ CP TF.Legs)
		[ Node (Left "feet")
			[
			]
		]
	, pure $ pure $ Left "wings"
	, pure $ pure $ Right $ CP TF.Tail
	, pure $ Node (Right $ CX TF.Cock) $ catMaybes
		[ pure $ pure $ Right $ CS Large
		, pure $ pure $ Right $ CA Lion
		]
	, pure $ Node (Right $ CX TF.Balls) $ catMaybes
		[ pure $ pure $ Right $ CS Large
		, pure $ pure $ Right $ CA Lion
		]
	, pure $ pure $ buildNode
	, pure $ pure $ elemNode
	]
	where
		buildNode = case d of
			Sunlight -> Left "muscular"
			Storm -> Left "muscular"
			Night -> Left "lean"
			Clouds -> Left "fat"
			Wind -> Left "lean"
			Frost -> Left "lean"
			Snow -> Left "fat"
		elemNode = case d of
			Sunlight -> Right $ CE TF.Sun
			Storm -> Right $ CE TF.Storm
			Night -> Right $ CE TF.Night
			Clouds -> Left "cloud" -- storm>>cloud
			Wind -> Left "wind" -- storm>>wind
			Frost -> Right $ CE TF.Ice
			Snow -> Left "snow" -- ice>>snow

deck :: RandomGen g => Rand g (Enumerable UmuDemon)
deck = do
	day <- liftRand $ shuffle [Sunlight, Storm, Night, Clouds, Wind, Frost, Snow]
	return $ UmuDemon
		<$> from day

data Index = Dummy
	deriving (Eq, Ord, Show, Read)

instance Scene Index () UmuDemon where
	link f = case f of
		_ -> error "bad link"

	display = fmap toLower . show
	vardisplay = fmap toLower . show
	fauxTree = Just . fmap (either id nodeLabel) . genFauxTree

scene :: Encounter Index () UmuDemon
scene b z = describe b z -- <> error "bad scene"

--- actual event start

describe :: Encounter Index () UmuDemon
describe _ u =
	[ Narration2 $ case day u of
			Storm -> "An ugallu-demon."
			_ -> "An umu-demon."
		/ "A demon of days; they are lion-headed and bipedal, with vast wings and clawing talons for feet. This one is a" / case day u of
			Sunlight -> "day of brilliant sunlight"
			Storm -> "crackling thunderstorm"
			Night -> "dark night"
			Clouds -> "misty, cloud-wreathed day"
			Wind -> "billowing windy day"
			Frost -> "bitter cold frost"
			Snow -> "cold, snowy day"
		/ ":" / case day u of
			Sunlight -> "its body is blotted out by its own brilliant radiance, rendering it into an eye-searing inverse silhouette"
			Storm -> "it's bursting with blue-white electricity, streaming in rivulets down its body"
			Night -> "its body is unlit, ignoring the light of hell in favor of a dim, impossibly-distant illumination only faintly playing across its brow"
			Clouds -> "its mane is an immense, heavy stormcloud, and rivulets of rain stream down its hide"
			Wind -> "its mane streams out behind it in a constant wind"
			Frost -> "its stiff fur is frozen in place, with complex spirals of frost painted across its body"
			Snow -> "it's wreathed in white, fur heavy shelves of snow piled up across its body"
		/ "."
		/ Paragraph / case day u of
			Sunlight -> "It's impossible to discern any of its features, only the shape of its body: big, muscular, with large paws and thick talons. Its wings flap dizzingly, leaving behind afterimages both in your eyes and in the world, hazing and reflecting its light in all directions."
			Storm -> "It's hugely muscular, with thickly-vascular skin visible through its sparse greyish fur. Its mane wreaths its head in a fuzzy halo. Its fur covers its entire body, gathering in thicker tufts across its chest and shoulders, but painted beneath that, in thick lines across its muscular hide, are vivid tattoos in an electric blue, glowing with the crackle of its lightning. Its wings keep it aloft, spinning out electrified dust devils with each flap."
			Night -> "It's lean and rangy, with long claws and gaunt talons. Its fur is an inky black. Its mane hangs down across its head and shoulders in heavy, lank strands, almost seeming to drool blackness down its chest. There are tiny flecks of paint across its hide, glowing like stars. Its wings seem almost decorative; it floats serenely, wings fluttering gently."
			Clouds -> "It's heavy and broad, thickly-muscled under layers of fat, with its thick wreaths of grey stormcloud-fur adding further body. Rain streams down its skin, dripping from its talons, and soaking its grey fur to near-invisibility, highlighting its tattooed skin: grey-brown skin, with white-blue markings, in complex patterns of glyphs across its chest and back, spiralling in curves down its arms and legs."
			Wind -> "It's leanly muscled, with fur that grows out into thick shaggy outcroppings, all perpetually pressed flat to its skin or streaming out behind it. Its mane is a mess of curls, spilling down its back, with stray hairs fluttering in its constant onrushing wind."
			Frost -> "It's leanly-muscled, with thick flanks. Its mane is frozen in a messy sunburst, frozen like immense icicles haloing its head. Its skin is a pale blue-white, visible through its light grey fur, and both fur and skin alike sparkle with a layer of eternal frost, thicker and thinner in places, forming fat wedges of ice framing its shoulders and hips."
			Snow -> "It's heavy and round, but hovers in the air weightlessly, with its wings hardly needing to flex at all. Its thick fur is napped together into heavy white dreads: mane draped down across its shoulders, shorter fur across its shoulders and down the center of its chest, over its belly, forming thinner, nubbier dreads. Snow collects across its dreads, making them look smooth and featureless, like they've been daubed in mud, and each movement sends stray flakes spilling out in bursts, catching on its frozen hide and making it glitter with cold."
	]
