{-# LANGUAGE MultiParamTypeClasses #-}

module Sites.Hell.Events.Hubur.VoidSnake
	( VoidWorm(..)
	, deck
	, scene

	, name
	, bits
	, variants
	) where
{-
OKAY SO: the quest would have a RANDOM PARTNER, w/ i guess later on the potential to slot in a known npc partner of the same type.

so this should afford:
* flame demon
* son of hamar-tabal
* minotaur miner

-}
import Prelude hiding ((/))

import Sites.Hell.Events.Construction ((/))
import Sites.Hell.Events.SceneUtils

import Sites.Hell.Events.Hubur

import Sites.Hell.Events.Hubur.MinotaurMiner (MinotaurMiner)
import Sites.Hell.Events.Hubur.HamarTabal (HamarTabal)
import Sites.Hell.Events.Hubur.FlameDemon (FlameDemon)

name :: String
name = "voidworm"

data VoidWorm = VoidWorm
	deriving (Eq, Ord, Show, Read)

instance Random VoidWorm where
	randomR _ = random
	random = getRand $ pure VoidWorm

instance ToJSON VoidWorm where
	toJSON VoidWorm = object
		[ pack "name" .= name
		, pack "data" .= ([] :: [Int])
		]

bits :: Value -> Value
bits _ = object
	[ pack "name" .= name
	, pack "data" .= ([] :: [Int])
	]

variants :: [(MapZone, ())]
variants =
	[ (FactoryMineTunnels, ())
	]
instance FromJSON VoidWorm where
	parseJSON (Object vs) = case (parse (.: pack "name") vs, parse (.: pack "data") vs :: Result [Int]) of
			(Success pName, Success [])
				| pName == name ->
					case pure VoidWorm of
							Nothing -> mzero
							Just a -> return a
			_ -> mzero
	parseJSON _ = mzero

deck :: RandomGen g => Rand g (Enumerable VoidWorm)
deck = return $ pure $ VoidWorm

data Index = DummyIndex
	deriving (Eq, Ord, Show, Read)

data Var = DummyVar
	deriving (Eq, Ord, Show, Read)

instance Scene Index Var VoidWorm where
	link f = case f of
		_ -> error "dummy index"

	display = fmap toLower . show
	vardisplay = fmap toLower . show

scene :: Encounter Index Var VoidWorm
scene _ c =
	[ description c
	-- , error "bad scene"
	]

description :: VoidWorm -> Event VoidWorm Index Var
description w = Narration2 $
	"It's a void worm."
	/ Paragraph / "It's an enormous snake, looking like a tear in the fabric of the world. It's glowing brightly, but no light you can see by: void body all lit up with unlight, leaving burning afterimages on your eyes. It moves fast, in stutters and stops, slithering across the ground and through the air, not so much floating as cutting through space, unaffected by gravity."
	/ Paragraph / "Its head is almost octopoid in construction: a broad, beaklike mouth, and a messy fringe of horns that spill back and out from its head, horns that squirm and writhe wildly but only when it moves. It's got seven slit eyes, all burning with unlight."

-- etc
data VoidSnakeQuestNpc
	= Flame FlameDemon
	| SonOfHamarTabal HamarTabal
	| Minotaur MinotaurMiner


-- which means making the specific scenes work right for all of the variations for all the npcs (i mean, this totals like. 11. at absolute max. obviously three major ones, and some of the minor ones are like "this flame demon has a goatee")

{-
There's something of a tableu: there's a big, solid looking demon satyr talking to a {son of hamar-tabal / goat-headed flame demon / lava minotaur} who's all strapped down with what your mind immediately completes as "adventuring gear":
	FlameDemon -> belts girdled around their waist and hips; a sword on one side and cylindrical scroll containers knocking on the other; an assortment of mismatched armor up across his sides and over his shoulders
	Minotaur -> a heavy handaxe with the same cracked-lava patterning as the minotaur's skin; a heavy black bag over his shoulder, made from something that can withstand his heat; an assortment of mismatched armor covering his legs
	HamarTabal -> a length of chain wound round his forearm and over his knuckles; thick pads of kevlar-looking protectors on his calves and arms; what looks like extra-heavy duty hazmat pants
.

[DEMON SATYR] ready to go?

[???] just about.

[DEMON SATYR] looks like the main incursion is a ways down. they've probably set up a nest. again.

[???] yeah, i'll handle it.

The demon satyr notices you first, and looks over:

[DEMON SATYR] Hey there, stranger.
[DEMON SATYR] Probably shouldn't go any further: the mine's shut down. An infestation of void snakes. Not much to do until they're cleared out.

-> void snakes?

[DEMON SATYR] Big squirmy things, fly through the air. They can do a lot of damage to a soul like yourself.

-> sure okay w/e

[???] I was just about to head out to deal with them. again!
-}


{-

so what's w/ the void snakes?

flame demon:
	probably bits of unwoven soul shards

	so sometimes souls grow tired of the world and leave into the void. in hopes of finding... something. and--

	i'll skip the heavy soul science, but you probably need a bit of a basic lesson. when the Authority created the universe, it did so by intermingling its substance with nothingness. by taking the thing that Is -- the authority -- and mixing it with the thing that Isn't, in different aspects and combinations. so we all are; everything that is exists because it is some aspect of the Authority.

	even the void: perhaps it is nothingness itself, the sole thing in the universe that exists beyond the will of the Authority. But more likely the way we see it, the way we understand and can interact with it... that too is mediated by the Authority. we all only see with the Authority's eyes.

	but we can split a soul, and split and split and split, and tease out different flavors. different aspects of the divine oneness, in different proportions. so many of them are beyond us, still unknown.

	so souls who want to leave creation go to the void, and the void tears them up, and the void snakes are, probably, what results. an exotic tone, one stable enough to remain conscious. and one interested in returning to creation. by soul measure, they're only an infinitesimal fraction of the souls that go missing; who knows what else exists out there.

	nobody's ever actually seen the transformation though. there have been some attempts to chart the void, to find some -- void snake breeding grounds, but... they haven't gone well. we think there must be some energies emanated by the deep void, ones we haven't found yet. ones that we might not be able to find.

minotaur miner:
	some breath of the void

	they live out there, in the crushing dark and frozen quiet of the void. and they look at creation and yearn for fire. so they come here. and we do our best to oblige them. they're strange things, but we can still speak.

son of hamar-tabal:
	> hell if i know. nobody really knows for sure.

	> they got these really knobbly cocks, though, like braided rope.

	he gestures, interlacing his fingers together.

	> i mean, they also like smashing things up, so that's a bit of a hassle. but...

	> that's pretty glib i guess but what kind of answer did you want? they're void worms. they live in the void. they're kind of a fucking hassle.

-}


{-
void snake aftermath:

(this would be like... you win the Big Fight w/ the void snakes and that warps you back to the start of the area and resets the encounters. but if you go all the way back, you end up w/ the whoever was helping you loitering around w/ the void snakes, and you can be like

-> ...so yr just hanging around here?

> well, i mean, it's kind of... we're kind of used to the void snakes, you know?
> they're not actually a serious threat precisely.
> so, uh. really a lot of that squirming and thrashing and drooling ichorous venom stuff is basically flirting.

-> flirting.

> yeah so uh. hey d'you wanna fuck?
> well. get fucked. by a bunch of void snakes. {and then me // and then maybe fuck me}

-> sure absolutely
-> nah

---


~> BIPED, QUADRUPED, NAGA
{snakes loop around the both of you and mash you together so yr making out w/ yr cocks pinned against each other}
	~> BIPED, QUADRUPED
	{snakes take turns fucking you, but yr both completely surrounded in and covered by void snakes so it's just this mass of weird unglowing flesh sometimes w/ a prehensile dick or two shoved into yr mouth and ass and rubbing all over yr body}
	{sloppier and sloppier as you get fucked until there's actually a churning surface, like, chunky black come}
	~> NAGA
	{snakes trade off fucking yr ass and letting yr companion fuck yr ass (since front-facing cloaca hole etc)}
~> CENTAUR
{snakes loop around you and it's like the above only yr cock enters into it less since it's all hanging behind you / maybe yr companion fucks b/t yr legs sometimes}
~> SLITHERING
{??? fucked like a slinky from both sides. wrapped around yr companion. speared through w/ a prehensile snake dick and then shoved inside yr companions ass. etc???}


---

The void snakes, always circling in their stuttering rhythm, coalesce. The don't come as one, but instead as a churning, intermingling mass, passing through the rock and through each other to squirm closer, until they're pressed tight around the two of you.

But of all of them, one is the first to touch. A void snake glides closer, twisting its serpentine body around, burning eyes blinking in a winking rhythm. It peers down at you, body held erect, showing off the pulsing waves of unlight that spill across its underbelly, seeming to radiate out from a core and wind through the wormlike conduits that make up its body.

Also, it's got a dick.

There's a divot near where its body twists away from you, and as you watch the divot spreads and opens, spitting out what looks like a complex woven knotwork, ridge by ridge. It's a bunch of worms, wrapped together in a broad span, all coated in slime. It's much wider than it is thick, diamond-shaped, with a bunch of folds and tips jutting out along the sides like stubby horns. Its cock coruscates with unlight, black on black, brighter and brighter up the shaft until the braid comes apart at the tip, forming a four-pronged tip, fat cords splayed like stubby fingers. The entire thing squirms, prehensile, black slime bubbling up from the slats between the cords, spilling down from its floating body and spilling across the cavern floor.

There are others just as close, unsheathed also, and {the satyr} sags back, sprawling across the sluggishly-churning nest of void snakes. His cock is already out of his sheath, fat and heavy, drooling a pearlescent line of pre.

And then like a whirling storm bursting, the cyclone of void snakes contracts and smears all across the two of you in a frenzy: blotting out all light, rendering your world into one of touch and presure. Their bodies are damp, hot, with a faint striation of flesh. And their cocks are like that, only moreso: squirming prehensile things, dripping with ichor, boilingly hot, almost frictionless as they smear across your skin and leave behind streaks of dark ichor.

~> BIPED, NAGA
	You're buffeted by their motions, tugged every-which-way: drawn from your seat and pulled into their churning mass, surrounded on all sides by their alien flesh.
~> CENTAUR, QUADRUPED
	You're tugged up off your {feet}, legs splayed. A snake slithers beneath you, cock dragging all across your underbelly, and more follow: spreading your hind legs, squirming around your legs and pulling them all in all different directions: you're pulled into their churning mass, surrounded on all sides by their alien flesh.


-> squirm

You're in a realm of wet heat and sliding pressure. Seams between (or possibly through) the snakes shed flashes of light across their coils, and their unlight-glowing cocks fail to illuminate anything yet still manage to distinguish their surroundings. They bear down, scales scraping over your skin, pressure grinding down.

Abruptly, like a curtain being drawn away, {the satyr} is in front of you, a snake between the two of you squirming aside and then you're plastered together, pressed chest-to-chest against another body. He's panting, huffing, and he grasps for your arms, legs, the both of you clinging together as the mass of snakes bears down. He gasps, shuddering, and his body jolts forward against you -- and in a second you realize why, as a void snake lines up its cock against your ass and shoves inside, matching the pressure as you're both fucked. It's impossible to line up the cocks shoving inside you with the snake doing it: it's just so much slithering flesh cording around you, all hunching and squirming and pulsing. The splayed cockheads lap across your ass, each one digging against your hole, before they lurch inside, one after the other -- for a moment two of them are shoved inside, the other two squirming over your cheeks, before they all settle inside as a squishy, grisled mass, tugging back and forth across the walls of your ass like a cat's tail trying to lash.

{the satyr} groans, stomach digging into yours as the void snake fucking him drives deeper, and you kiss messily, panting and gasping with each thrust. His lips drag over yours, the both of you jostled too much to line anything up, just dragging your mouths over shoulders, necks, jaws, nipping and biting between breathless groans.

You can feel the motions of his fuck: the snake behind him coiling and rutting, cock rising and falling across his belly, pressing the broad cockhead against you through {the satyr's} flesh. his own cock drools, spurting raggedly, pre lost instantly in the slick glide of snakeskin all around you.

Flesh all around you churns, and the squirming cockhead pressed half-inside your ass suddenly drives deeper, knocking the breath from your lungs as you gasp against {the satyr's} face, all your focus suddenly on the squirming, pumping mass of flesh sliding over the curve of your ass, sinking into your stretched hole. The snake lurches, coils bunching behind you, and it ruts forward and back, mashing your body puppetlike against {the satyr's}. It's almost like the two snakes are fucking each other, stroking each other off with their cocks, and your bodies simply happen to be flesh in the way. Their thrusts prod and coil, mashing your bloated bellies together, lurching backwards only to ram forward again.

The two of you hold on to each other as well as you can. More cocks press against you: smearing across your side, thrusting into the space between your chests, drenching you both with oily precome. The endless churning of the mass of snakes smears it around, slowly coating the both of you in thick, slimy whorls of black pre.

A cock ruts against the side of your head, slurping down to press between you, big ropy cords of muscle sawing across your face, long enough that it can loop in a lazy figure-eight around both yours and {the satyr's} heads, mashing the both of you up against its bloated slit, smearing its oozing flesh across your faces. You open your mouth, fat knobs along the side of its cock sawing over your cheek and bursting into your mouth, already slathered in the mingled ooze. More wrap around your arms and legs, big bands of ridged flesh from a half-dozen cocks cording across your body, just using you as a surface to rut against.

The writhing mass around you hisses and croons, flesh heavy dragging across your sides, everything so slick now it's impossible to tell which of the slimy, twisting protrusions bearing down against you are cocks and which are the rest of a snake. Flesh wedges everywhere it can: cockheads digging into your elbows, pressing between your thighs, rutting along the groove where your bodies press together, over the small of your back. You're slathered in pre, and your gasping breaths end up blowing frothy bubbles from the big planes of pre stretched out all around you.

Suddenly, abruptly, a cock erupts, gushing a thick mess of chunky slime all over your sides, and that seems to egg more of the snakes on. The cock fucking you jerks forward, thick knobs of folded flesh dragging against your asshole, slurping through; another spray of come gushes all over it, lubing the length further as it burrows inside. Hissing overcomes the din of slurping, and you're mashed forward, pressed tight against {the satyr's chest} as the cock inside you shudders, twisting hard enough to knock the breath from your lungs, and erupts: its load is burning hot, chunky and gritty, and it gushes into you in slimy pulses, so thick it seems to fold over itself as it pours deeper into your overfilled ass.

Cocks push into your mouth, slide along your face, and the cock in your ass pulls out -- still spraying all up your back -- and is replaced instantly by another, with a third fighting it, rutting all up and down your back to get its turn. You gurgle, acrid void-come spilling into your mouth, burning down your throat, and you cough and sputter, spitting the frothing mess into {the satyr's} open mouth, sloppily kissing even with your mouths drooling black come, the cocks lodged in your bloated stomachs churning and fighting each other. Spurt after spurt of black come sprays all over you, sudden hoselike squirts coating you, piling up and streaming down your legs, thickly glazing every inch of your body in come. Another cock erupts inside you, or in the satyr, all over the both of you, and you feel it the same, the satyr's gut bloating with come, rippling and bulging against yours.

Both of your bodies are entirely slathered in come and ooze, cords drenching your side and drooling from your spread-eagled arms and legs, dense and heavy as another cock erupts across your face or up your side, forming thick, slimy mats that spill down your body. Huge pockets of ooze collect between you and the snakes, piling up in gravity-defying clusters -- or perhaps at some point you and the satyr were toppled over, on your side or upside-down now; it's impossible to tell.

You lose track of the number of snakes to shove into your ass: some diving deep, fucking you roughly, come bursting deeper within you; others smearing already-spurting cocks over your gaping and wrecked cheeks, only shallowly dipping inside. Your ass aches, ring blown wide, come continually oozing out, spurting more when a snake mashes its cock inside and thrusts roughly, each jerk sending an eruption gushing out across your bodies.

The ooze piles up and up, until at some point you recognize it's not just patches and lines clinging to you, but a dense, swampy weight all of its own: half your body is submerged in a morass of come, black chunky ooze flooding the cavern floor and building up, more flooding into your ass and spilling out, adding to the mess.

It goes on forever, impossible to tell if snakes get tired out and leave -- if void snakes are even a thing that gets tired -- but eventually, dimly, dizzily, your tired mund catches more slivers of light through the coils, more and more until the world around you resolves again: the cavern drenched top to bottom in ooze, stalactites of come hanging from the ceiling and dripping down to the floor. The low hollow of the cavern has been entirely flooded, forming a broad, nearly waist-deep pool of void snake come: a thick black ooze, chunky and gritty, threaded through and sparking with unlight. It churns on its own, convolving in sludgy waves, lapping at your sides -- for you're submerged in it, nearly floating on your back.

There's only a few snakes left, only one tangled around you, cock lodged deep into your stomach as it hunches and squirms. Your hole is a wreck: gaping, prolapsed, broken into a crater that the snake pummels further. Its cock seems like nothing but knobs and hooks, tugging and tearing at your flesh, scoring across your insides. It comes, maybe; the pool of come comes up past your ass, and its cock forces a mess of caked-on come into you with each thrust; it's impossible to say if the slobbery pressure building inside you comes from its cock erupting or simply the motion of its fucking.

But the snake pulls out, pulls away, leaving you busted and broken wide, covered head to toe in oily smears, absolutely soaking every inch of skin. You're half-submerged, and when its cock slithers out of you your hole remains broken and gaping, more ichor sluggishly burbling inside, filling whatever little space was left open. The snake ripples into the air, cock still thickly-corded with slime, webbed all across your thighs and belly. As it peels away, the tarry residue stretches and eventually snaps, splashing into the morass with a muffled _crack_.
-}


-- you top:

{-
{The satyr} surfaces, crawling out of the churning morass with his entire body streaming with ichor, so thickly coated no part of him is visible: just layer on layer of chunky black slime. He settles back, sprawling across the rock, thick ripples of come lapping up across his thighs. He raises his legs, hands reaching back to play with his hole: he's wrecked and gaping, bruised and half-prolapsed in huge, rubbery folds. His fingers dig in to his flesh, smearing through the tacky mess of slime, only for more to ooze from his overfilled ass, slurping lewdly down his fat cheeks.

-> go fuck him
-> nah let him play w/ himself

---

-> go fuck him

You wade over, stirring the slime, and he tips his head toward you -- face relatively cleaned-off; all his features visible, just with tarry black swipes of come all over his cheeks, soaking his hair.

> Pretty good, huh?

His fingers probe into his ass, his cock lurching up his stomach as he does it. He pants, wetly, still catching his breath but not entirely spent yet.

> Think you can go one more time?

He splays his ass open wide, fingers hooking into his bruised rim and digging deep, pulling his hole open until it shows off his inner guts, flesh clenching and squeezing, huge globs of black come still gurgling out from deeper inside, sluicing down the walls of his ass.

You wade into place, your cock dragging up his hairy thigh and pressing against his bloated lips, tip burying itself into a lumpy fold, and he wraps his fingers around your cockhead, slathering it in shed come, before guiding it into his hole, rim not even touching your cockhead as you shove inside. He's absolutely flooded, guts gurgling and squelching as you drive in, thick slimy void snake come squirting down along your shaft as you sink to the root effortlessly, his wrecked ass offering up no resistance whatsoever.

He pants hoarsely, one hand reaching up from his ass to scrape along his chest, leaving behind a visible strip of {hairy blue-black flesh} as all the murky slime piles up across his fingers, and then he shoves them in his mouth, panting for breath even as he greedily swallows the slimy mouthful of half-coagulated come. He arches into your thrust, each one forcing out a slimy eruption of black come that splatters all over your thighs and up your chest, just soaking into the mess you're already slathered in.

It doesn't take long before you come, grunting and groaning when you erupt. Your load smears out along your shaft, white threads in the chunky black ooze, slowly mingling to a grimy grey. You pull out with a wet _slorp_, his asshole simply cratered, gaping wide as the mingled loads inside him steadily pour out into the swampy mess.

It's a few more minutes before you both manage to get the energy to wade up out of the mess and just sprawl on the ground, void come slowly puddling beneath you, coagulating into huge gummy globs all over your skin, drooling from both of your broken holes.

{some dialog here}
-}


-- companion tops:

{-
{The satyr} surfaces with a gasp, entire body streaming with ichor, so thickly coated no part of him is visible: just layer on layer of chunky black slime. He wades towards you, stirring the slime.

> Pretty good, huh?

He scrapes a hand across his helmet, the dim glow of his sigils burning through in patches, and shakes the gummy handful of slime down, spilling in clots. He gropes his crotch -- slime squelching up between his fingers, cock itself hardly visible.

> Think you can take one more?

-> yea go for it
-> nah i think we're done here

---

-> yea go for it

He sprawls into you, chests smacking together, and shoves you onto your back, legs raised. Thick black ooze, chunky and hot, slowly spills inward across your chest, and he smears a hand down, lining his cock up with yours for a second and tugging on them both with sloppy, syrupy strokes, before he reaches lower to play with your ass. You ass is below the water line, and his hairy knuckles grind against the rubbery flesh of your hole, back and forth, forcing sloppy squirts of come out, only for it to come slurping back inside when his fingers withdraw.

His cock knocks against your thigh, rubbery and heavy, and he guides it into place, working his half-hard length into your ass with his fingers, just using it to stir the mass of come inside you around. He hardens slowly, the heavy mass shuddering and bloating inside you as he thrusts. Come squirts in and out along his shaft, thick chunks of black come smearing across his length and forming a dense paste, adding further drag to his thrusts. Your hole pulses, broken and gaping, his touch dragging back and forth, all pressure digging and tugging at the bruised and sensitive rim of your ass.

His hands grip your {thighs/haunches} more firmly, ooze squelching, and he thrusts faster, cock hard enough to thrust properly. Even so, it's more of the same: the thick mass of black come you're immersed in churning and slurping, huge strings slimy across the underside of your thighs, jiggling between where they're tethered to his chest.

He comes with an exhausted groan, cock practically bruised from overuse when he finally comes, the gush of his load impossible to discern from the sluggish churn of void snake come still slopping around inside you. He thrusts sharper, panting, and collapses to the side, cock pulling out from you with a _slurp_, leaving your hole spread and gaping for more of the ooze pooled around you to flood inside and gurgle back out.

It's a few more minutes before you both manage to get the energy to wade up out of the mess and just sprawl on the ground, void come slowly puddling beneath you, coagulating into huge gummy globs all over your skin, drooling from both of your broken holes.

{some dialog here}
-}
