{-# LANGUAGE MultiParamTypeClasses #-}

module Sites.Hell.Events.Hubur.Worms
	( Worms
	, deck
	, scene
	) where

import Sites.Hell.Events.SceneUtils

data Worms = Worms
	deriving (Eq, Ord, Enum, Bounded, Show, Read)

instance Random Worms where
	randomR _ = random
	random = getRand $ return Worms

instance ToJSON Worms where
	toJSON d = object
		[ pack "name" .= "worms"
		, pack "data" .= ([] :: [Int])
		]

instance FromJSON Worms where
	parseJSON (Object vs) = case (parse (.: pack "name") vs, parse (.: pack "data") vs :: Result [Int]) of
			(Success name, Success [])
				| name == ("worms" :: String) ->
				case pure Worms of
						Nothing -> mzero
						Just h -> return h
			_ -> mzero
	parseJSON _ = mzero

deck :: RandomGen g => Rand g (Enumerable Worms)
deck = return . pure $ Worms

instance Scene () () Worms where
	link _ = error "link"
	display = fmap toLower . show
	vardisplay = fmap toLower . show

scene :: Encounter () () Worms
scene = worms

worms :: Encounter () () Worms
worms _ _ =
	[ Narration
		[ "The ground here is pitted with sunken, craterlike holes, and as you move closer a buzz starts beneath you. A heavy, sluggish mass of pallid worms spills from a half-dozen holes and slowly writhe your way."
		]
	, Challenge
		(Opponent "a pile of writhing worms" "fight"
			(Stats
				{ str = 4, def = 4, spd = 2, vit = 7
				, int = 2, res = 6, cha = 2, atn = 2 })
			(RandomAI
				[ [Wait, Wait, Hit]
				, [Hit, Wait, Hit]
				, [Hit, Block]
				, [Wait, Block]
				]
			)
		)
		[]
	, Narration
		[ "The remaining worms squirm away, back into their holes."
		]
	, DeactivateSelf
	]
