{-# LANGUAGE MultiParamTypeClasses #-}
{-# LANGUAGE FlexibleContexts #-}

module Sites.Hell.Events.Hubur.Zombie
	( Zombie(..)
	, Index(..)
	, skullDeck
	, wildDeck
	, scene

	, name
	, bits
	, variants
	) where

import Prelude hiding ((/), (^))

import Sites.Hell.Events.SceneUtils hiding ((/))
import Sites.Hell.Events.Construction ((/))

import Description.TfNode
import Server.JSON
import Sites.Hell.Events.Hubur

name :: String
name = "zombie"

data Zombie = Zombie
	{ variant :: Variant
	}
	deriving (Eq, Ord, Show, Read)

data Variant = Base | Swordsman | Cook
	deriving (Eq, Ord, Enum, Bounded, Show, Read)

instance ToJSON Zombie where
	toJSON (Zombie v) = object
		[ pack "name" .= name
		, pack "data" .= [fromEnum v]
		]

bits :: Value -> Value
bits _ = object
	[ pack "name" .= name
	, pack "data" .= [Null]
	]

variants :: [(MapZone, ())]
variants =
	[ (SwampZombie, ())
	]

instance FromJSON Zombie where
	parseJSON (Object vs) = case (parse (.: pack "name") vs, parse (.: pack "data") vs) of
			(Success pName, Success [v])
				| pName == name -> case Zombie <$> safeToEnum v of
					Just z -> return z
					_ -> mzero
			_ -> mzero
	parseJSON _ = mzero

skullDeck :: RandomGen g => Rand g (Enumerable Zombie)
skullDeck = do
	variant <- liftRand $ shuffle [Swordsman, Cook]
	return $ Zombie <$> from variant

wildDeck :: RandomGen g => Rand g (Enumerable Zombie)
wildDeck = return $ pure $ Zombie Base

data Index = ZChallenge | ZShop | LetsFight | TurnDown
	deriving (Eq, Ord, Show, Read)

instance Scene Index () Zombie where
	link f = case f of
		ZChallenge -> challenge
		ZShop -> shop
		LetsFight -> letsFight
		TurnDown -> turnDown

	display = fmap toLower . show
	vardisplay = fmap toLower . show


dialog :: Convertible a EventReel => Zombie -> a -> Event Zombie Index ()
dialog z = Dialog2 "/static/img/icon.png" (case variant z of
	Base -> "Zombie"
	Swordsman -> "Zombie Swordsman"
	Cook -> "Zombie Cook") . convert

scene :: Encounter Index () Zombie
scene b z = describe b z -- <> error "bad scene"

--- actual event start

describe :: Encounter Index () Zombie
describe _ z =
	[ Narration2 $ case variant z of
		Base -> "You come across a zombie. It's a thing that was at one point a dead body. Its muscles are overgrown, and it moves with a shambling, inhuman gait. Its skin is a faded grey, and streaked with ash and silt."
			/ Paragraph / "There's a squirming thing wrapped around its head, all leathery purple-black flesh: a central mass that covers its face and then long octopus-like tentacles that fan out across its shoulders and back."
			/ Paragraph / "Its tentacles trail down its body, clamped tight to the skin like grotesquely-gigantic veins. They bite deep under the skin across its shoulders, chest, and back, the skin around them piled up into dark, oozing bruises. Its grey skin blossoms into vivid bruises where the bones are close to the surface: purple, green, and yellow tinges down its spine, at its elbows and knees. Glowing green fluid pulses in its veins, feeding its muscles with unnatural energy. A single loose tentacle extends from its face like an enormous proboscis, tapering in wormlike rings to a squirming tip, slavering with shed ichor."
		_ -> "There's a vast dragon skull lying across the swamp. A half-dozen people could easily fit through one of its hollow eye sockets. Earth has piled up around it, and the interior is a half-flooded grotto."
			/ Paragraph / case variant z of
				Swordsman -> c "There's a figure leaning against the old yellowed bone: It's a zombie swordsman. Its muscles are overgrown, and its grey skin is streaked with ash and silt in painted lines. It's wearing a rough kilt of dark fabric, wrapped around its waist, and into a looser loop it's carrying a sheathed sword."
				Cook -> "Steam streams from the dragon's maw. There's an immense stone cauldron perched on its ancient jawbone, and the water around it boils, lit by some glowing element beneath the surface."
					/ Paragraph / "There's a zombie bent over the lip of the cauldron, wearing only an oily apron across its front, stirring the greenish goop within with an oak ladle nearly as big as its own body."
				_ -> error "unreachable"
	] <> case variant z of
		Swordsman -> [Jump ZChallenge]
		Cook -> [Jump ZShop]
		Base -> []

shop :: Encounter Index () Zombie
shop _ z =
	[ dialog z $ "If you have the ingredients, I could add them in."
	, Shop "cook"
		[ A [Left $ I LIONS_MANE 3]
			$ Right $ CA Lion
		, A [Left $ I MANDRAKE 3]
			$ Right $ CA Human
		, A [Left $ I GOATSBEARD 3]
			$ Right $ CA Goat
		, A [Left $ I ADDERS_FORK 3]
			$ Right $ CA Snake
		, A [Left $ I OX_EYE 3]
			$ Right $ CA Bull
		, A [Left $ I CATTAIL 3]
			$ Right $ CA Tiger
		, A [Left $ I HARTS_TONGUE 3]
			$ Right $ CA Deer
		, A [Left $ I TONGUE_OF_DOG 3]
			$ Right $ CA Wolf
		, A [Left $ I HORSE_HOOF 3]
			$ Right $ CA Unicorn
		, A [Left $ I PIGS_TAIL 3]
			$ Right $ CA Boar
		]
	, BodyEditor
	]

challenge :: Encounter Index () Zombie
challenge _ z =
	[ Narration2 $ c "The parasite-thing on its head shudders, pulsing wetly, and it speaks:"
	, dialog z $ "A soul out wandering the fields of Hell?"
		/ Paragraph / "What a venturesome soul you are."
	, Narration2 $ c "It gestures with a tentacle to its sword."
	, dialog z $ c "If you're here, we might as well fight. My sword has been aching for blood."
	, Choices
		[ ("yeah lets fight", LetsFight)
		, ("maybe lets not", TurnDown)
		]
	]

turnDown :: Encounter Index () Zombie
turnDown _ z =
	[ Narration2 $ c "It shrugs, and its tentacles writhe in approximation."
	, dialog z $ c "Serve yourself. I'll be here, though."
	]

letsFight :: Encounter Index () Zombie
letsFight _ z =
	[ Narration2 $ c "The zombie draws its sword: long and curved, and made from an iridescent black metal."
	, dialog z $ c "Prepare yourself."
	, Challenge
		(Opponent "Zombie Swordsman" "fight"
			(Stats
				{ str = 10, def = 10, spd = 10, vit = 16
				, int = 10, res = 10, cha = 10, atn = 10 })
			(RandomAI
				[ [Charge, Hit]
				, [Block, Block]
				, [Cast]
				, [Charge, Parry]
				, [Charge, Charge, Charge, Hit]
				, [Heal, Wait]
				, [Charge, Heal, Wait]
				, [Parry, Hit, Hit]
				, [Wait, Wait, Wait, Wait]
				]
			)
		)
		[]
	, Narration2 $ c "It sheaths its sword with a click, what blood you shed already absorbed into the blade."
	, dialog z $ c "Well then. Congratulations."
	]
