{-# LANGUAGE MultiParamTypeClasses #-}
module Sites.Hell.Events.Hubur.ZombieHierophant
	( ZombieHierophant(..)
	, deck
	, scene

	, name
	, bits
	, variants
	) where

import Prelude hiding ((/), (^))

import Description.TfNode
import qualified Description.Body.Types as Body (Fluid(Ichor))

import Sites.Hell.Events.SceneUtils hiding ((/))
import Sites.Hell.Events.Construction ((/))

import Sites.Hell.Events.Hubur

name :: String
name = "zombiehierophant"

data ZombieHierophant = ZombieHierophant
	deriving (Eq, Ord, Show, Read)

instance Random ZombieHierophant where
	randomR _ = random
	random = getRand $ pure ZombieHierophant

instance ToJSON ZombieHierophant where
	toJSON ZombieHierophant = object
		[ pack "name" .= name
		, pack "data" .= ([] :: [Int])
		]

bits :: Value -> Value
bits _ = object
	[ pack "name" .= name
	, pack "data" .= ([] :: [Int])
	]

variants :: [(MapZone, ())]
variants =
	[ (SwampZombie, ())
	]

instance FromJSON ZombieHierophant where
	parseJSON (Object vs) = case (parse (.: pack "name") vs, parse (.: pack "data") vs :: Result [Int]) of
			(Success pName, Success [])
				| pName == name -> return ZombieHierophant
			_ -> mzero
	parseJSON _ = mzero

deck :: RandomGen g => Rand g (Enumerable ZombieHierophant)
deck = return . pure $ ZombieHierophant

data Index = Leave | SexOpen | Sex | Infest
	deriving (Eq, Ord, Show, Read)

instance Scene Index () ZombieHierophant where
	link f = case f of
		Leave -> leave
		SexOpen -> sexOpen
		Sex -> sex
		Infest -> infest

	display = fmap toLower . show
	vardisplay = fmap toLower . show

scene :: Encounter Index () ZombieHierophant
scene b z = describe b z
	<> [ Choices
		[ ("test sex scene", SexOpen)
		, ("leave", Leave)
		]
	] -- <> error "bad scene"

--- actual event start

describe :: Encounter Index () ZombieHierophant
describe _ z =
	[ Narration2 $ "It's a zombie hierophant. It's bigger even than a regular zombie, body swollen with muscle to grotesque proportions; it shambles around gorilla-like, using its knuckles, its arms oversized even compared to the rest of its giant body."
		/ Paragraph / "Its skin is grey and ichorous, and its flesh is thickly webbed with bulging veins, glowing green, giving it a sickly illumination. Sunk into the flesh of its meaty back are big glass tubes filled with concentrated green ooze, six of them in pairs of three, below its shoulderblades going down its back. They whirr and gurgle sometimes, pumping more ooze into its flesh."
		/ Paragraph / "Like a regular zombie, it's got a squirming tentacle-thing covering its head, a mess of shuddering purple-black flesh. But where a regular zombie has only a dozen-or-so tentacles, all clamped to its flesh, this has dozens upon dozens, so much so that its head resembles something like a sea urchin: a mess of tentacles sticking stiffly out in all directions, waving like fronds. They ooze tarry black slime, sheening down its neck and chest, coating its flesh in streams."
	]

leave :: Encounter Index () ZombieHierophant
leave _ _ =
	[ Narration2 $ c "You leave."
	]

sexOpen :: Encounter Index () ZombieHierophant
sexOpen b z =
	case cumtype b of
		Just Body.Ichor ->
			[ Choices
				[ ("try the infestation branch", Infest)
				, ("regular sex scene", Sex)
				]
			]
		_ -> [Jump Sex]

sex :: Encounter Index () ZombieHierophant
sex b z =
	[ Narration2 $ c "{insert sex scene here}"
	]

infest :: Encounter Index () ZombieHierophant
infest b z =
	[ Narration2 $ c "{infestation sex scene here}"
	, PushNode (Overwriting (CE Ichor)) [CR Animum, CX Balls, CE Infested]
	, Narration2 $ c "{post-infestation closer}"
	]
