module Sites.Hell.Events.Piercings
	( PiercingType (..)
	, PiercingLocation (..)
	, fromEnumPiercings
	, toEnumPiercings

	, piercingOf
	, piercingsOfType
	) where

import Data.Ratio
import Data.Maybe

import Data.Enumerable

data PiercingType = Ring | Padlock | Stud | Tag | Chain | Hook | Cross
	deriving (Eq, Ord, Enum, Bounded, Show)

data PiercingLocation
	= Glans
	| Frenum
	| Balls
	| Taint
	| SheathLip
	| SheathUnderside

	| Septum
	| Tongue
	| Eyebrow
	| Earlobe
	| EarShell

	| Nipple
	deriving (Eq, Ord, Enum, Bounded, Show)

piercingOf :: [a] -> Ratio Int -> Enumerable (Maybe a)
piercingOf ts chance = from (replicate total Nothing)
		<> from (replicate' (numerator chance) (Just <$> ts))
	where
		total = length ts * (denominator chance - numerator chance)
		replicate' 0 _ = []
		replicate' i vs = vs <> replicate' (i-1) vs

piercingsOfType :: [PiercingType] -> [PiercingLocation] -> Ratio Int -> Enumerable [(PiercingLocation, PiercingType)]
piercingsOfType ts ls chanceEach = fmap catMaybes $ foldr mergePs (pure []) selectedPs
	where
		p :: PiercingLocation -> Enumerable (Maybe (PiercingLocation, PiercingType))
		p loc = fmap ((,) loc) <$> piercingOf ts chanceEach
		acc :: Enumerable (Maybe a) -> Enumerable ([a] -> [a])
		acc = fmap (\v -> case v of
			Nothing -> id
			Just x -> (x :))
		selectedPs :: [Enumerable (Maybe (PiercingLocation, PiercingType))]
		selectedPs = p <$> ls
		mergePs :: Enumerable a -> Enumerable [a] -> Enumerable [a]
		mergePs new rest = (:) <$> new <*> rest

toEnumPiercings :: Int -> (PiercingLocation, PiercingType)
toEnumPiercings i = (l, t)
	where
		l = toEnum $ i `mod` (fromEnum (maxBound :: PiercingLocation) + 1)
		t = toEnum $ i `div` (fromEnum (maxBound :: PiercingLocation) + 1)

fromEnumPiercings :: (PiercingLocation, PiercingType) -> Int
fromEnumPiercings (l, t) = (fromEnum t * (fromEnum (maxBound :: PiercingLocation) + 1)) + fromEnum l


