{-# LANGUAGE OverloadedStrings #-}

module Sites.Hell.Routes
	( route
	) where

import Data.Maybe (fromMaybe)

import System.FilePath.Posix

-- happstack!
import Happstack.Server
-- for the ToMessage instance for RSS
import Server.JSON ()
import Server.Error

-- web-routes
import Web.Routes (RouteT, showURL)

import Text.RSS (RSS(..), ChannelElem(..), Item)
import Network.URI (parseURI)
import qualified Data.Text as T (unpack)

import Sites.Hell.Sitemap (SiteRoute(..))

import Sites.Hell.Routes.Splash (splashPage, newsPostPage, newsFeed)
-- import Sites.Hell.Routes.DBCheck (dbCheckPage)

import Sites.Hell.Routes.Info (infoPage, todoPage, goalPage)
import Sites.Hell.Routes.Population (populationAPI)
import Sites.Hell.Routes.Event (eventAPI)
import Sites.Hell.Routes.Description (descriptionAPI, descriptionFreeformEditor)

route :: String -> SiteRoute -> RouteT SiteRoute HellServer Response
route base url = case url of
	Splash mpage -> toResponse <$> splashPage mpage
	NewsPost id_ -> toResponse <$> newsPostPage id_
	NewsFeed -> toResponse <$> (hellGameRSS =<< newsFeed)

	-- DBCheck -> dbCheckPage
	AdminNewsPost -> error ":("

	Info -> toResponse <$> infoPage
	Todo -> toResponse <$> todoPage
	Goal -> toResponse <$> goalPage
	Game -> serveFile (asContentType "text/html")
		$ joinPath [base, "static", "hell.html"]
	PopulationGenerator -> toResponse <$> populationAPI
	EventParser -> toResponse <$> eventAPI
	Description -> toResponse <$> descriptionAPI
	DescriptionTest -> toResponse <$> descriptionFreeformEditor

hellGameRSS :: [Item] -> RouteT SiteRoute HellServer RSS
hellGameRSS is = do
	url <- fmap (fromMaybe (error "bad uri parse") . parseURI . T.unpack) $ showURL (Splash Nothing)
	return $ RSS "HELL GAME" url "hell game development reel"
		[ Language "en-us"
		, Copyright "© xax 2014-2016"
		, TTL 1440 -- etc. we might want to extract the most recent item date to make that the update date? or have the date-generated header, w/e
		]
		is
