{-# LANGUAGE ViewPatterns #-}
{-# LANGUAGE OverloadedStrings #-}

module Sites.Hell.Routes.Description
	( descriptionAPI
	, descriptionFreeformEditor
	) where

import Control.Monad
import Data.Monoid ((<>))

import Data.ByteString.Char8 (pack)

import Web.Routes (RouteT)
-- for the `HasRqData` instance for `RouteT`
import Web.Routes.Happstack ()
import Server.Error (HellServer)
import Happstack.Server (look)
import qualified Text.Blaze.Html5 as H
import qualified Text.Blaze.Html5.Attributes as A

import Description.Pronoun
import Server.JSON
import Server.Layout
import Sites.Hell.Sitemap (SiteRoute(..))

import Description.Describe (describe, stub)

descriptionAPI :: RouteT SiteRoute HellServer Value
descriptionAPI = do
	d <- liftM (decodeStrict . pack) $ look "data"
	n <- liftM pronounDecode $ look "ngender"
	p <- liftM pronounDecode $ look "pgender"
	case (d, (,) <$> n <*> p) of
		(Just (parse jsonToTree -> Success d'), Just (n, p)) ->
			let (errs, desc) = describe n p d'
			in
				return . object
					$ [ "description" .= desc
					  , "stub" .= stub n d'
					  , "errors" .= (showError . Missing <$> errs)
					  ]
		-- TODO: Text.JSON's decodestrict returns an error message in this case; Data.Aeson has some other function for that & just returns Maybe. maybe call that instead
		(Nothing, _) ->
			return . object
				$ [ ("description" .= ([] :: [String]))
				  , ("errors" .= (showError <$> [Invalid "??"]))
				]
		(_, Nothing) ->
			return . object
				$ [ ("description" .= ([] :: [String]))
				  , ("errors" .= (showError <$> [Invalid "invalid pronoun"]))
				]
		_ -> -- well-formed JSON but an invalid body tree
			return . object
				$ [ ("description" .= ([] :: [String]))
				  , ("errors" .= (showError <$> [Invalid "invalid body"]))
				]

pronounDecode :: String -> Maybe Gender
pronounDecode c = case c of
	"he" -> Just He
	"she" -> Just She
	"they" -> Just They
	"it" -> Just It
	"you" -> Just You
	_ -> Nothing

{-
description builder todos:

* more state for nodes???
	* animum levels
	* gloss/element/shape levels
		* color gloss lvls look like "white/black"

-}
descriptionFreeformEditor :: RouteT SiteRoute HellServer Html
descriptionFreeformEditor = return $ do
	H.docType
	H.title "description builder test"
	stylesheet "/static/css/main.css"
	stylesheet "/static/css/desceditor.css"
	stylesheet "/static/css/bodyparts.css"

	script "/static/js/jquery-1.10.2.js"
	script "/static/js/underscore-1.8.2.js"
	script "/static/js/common.js"
	script "/static/js/body.js"
	script "/static/js/editor.js"
	script "/static/js/descriptionEditor.js"
	H.h1 "DESCRIPTION TEST"
	H.section ! A.class_ "frame" $ do
		H.header ! A.id "desc_id" $ "description"
		H.div ! A.class_ "inner" $ do
			H.fieldset ! A.id "pronoun" $ do
				H.span ! A.class_ "pcheck" $ do
					H.input
						! A.name "y"
						! A.type_ "checkbox"
						! A.id "you"
						! A.value "you"
						! A.checked ""
					H.label
						! A.for "you"
						$ "you"
				H.span ! A.class_ "pradios" $
					sequence_ $
						(\x -> do
							(if x == "it"
								then (! A.checked "")
								else id)
								$ H.input
									! A.name "p"
									! A.type_ "radio"
									! A.id (toValue x)
									! A.value (toValue x)
							H.label
								! A.for (toValue x)
								$ toHtml x)
							<$> (["it", "he", "she", "they"] :: [String])
			H.div ! A.id "descriptionText" $
				"nothing yet"
	H.div ! A.id "wide" $ do
		H.div ! A.class_ "left" $ do
			H.section ! A.class_ "frame" $ do
				H.header "body"
				H.ul ! A.class_ "inner" ! A.id "builder" $ return ()
		H.div ! A.class_ "right" $ do
			H.section ! A.class_ "frame" $ do
				H.header "parts"
				H.div ! A.class_ "inner" ! A.id "parts" $
					"..."

data ProseError = Missing String | Invalid String | Incoherent String
	deriving (Eq, Ord, Show, Read)

showError :: ProseError -> String
showError p = case p of
	Missing s -> "Missing: " <> s
	Invalid s -> "Invalid: " <> s
	Incoherent s -> "Incoherent: not even sure where to start with \"" <> s <> "\""
