{-# LANGUAGE ViewPatterns #-}
{-# LANGUAGE OverloadedStrings #-}

module Sites.Hell.Routes.Event
	( eventAPI
	) where

import Control.Monad
import Control.Monad.Trans (lift)
import Control.DeepSeq

import Data.ByteString.Char8 (pack)
import qualified Data.Text as T (pack)
import Data.Tree
import Data.Monoid ((<>))

import Server.Error (HellServer, logLn, Priority(..))
import Happstack.Server (look)
import Web.Routes (RouteT)
-- for the HasRqData instance for RouteT
import Web.Routes.Happstack ()

import Server.JSON
import Server.Error (SomeException, catch)
import Sites.Hell.Sitemap (SiteRoute(..))

import Description.Pronoun
import Description.Build (build, BodyGlob)
import Sites.Hell.Events.GameEvent
import qualified Sites.Hell.Events.Hubur.LesserHound as LesserHound
import qualified Sites.Hell.Events.Hubur.SobekSheut as SobekSheut
import qualified Sites.Hell.Events.Hubur.Sahagin as Sahagin
import qualified Sites.Hell.Events.Hubur.Dryad as Dryad
import qualified Sites.Hell.Events.Hubur.GanzerSoldier as GanzerSoldier
import qualified Sites.Hell.Events.Hubur.Gargoyle as Gargoyle
import qualified Sites.Hell.Events.Hubur.FlameDemon as FlameDemon
import qualified Sites.Hell.Events.Hubur.ClayMan as ClayMan
import qualified Sites.Hell.Events.Hubur.HamarTabal as HamarTabal

import qualified Sites.Hell.Events.Hubur.MechanicShop as MechanicShop
import qualified Sites.Hell.Events.Hubur.FlareStack as FlareStack

import qualified Sites.Hell.Events.Hubur.Opening as Opening
import qualified Sites.Hell.Events.Hubur.FarmerHamarTabal as FarmerHamarTabal
import qualified Sites.Hell.Events.Hubur.RandomDrop as RandomDrop
import qualified Sites.Hell.Events.Hubur.HellGate as HellGate
import qualified Sites.Hell.Events.Hubur.Worms as Worms
import qualified Sites.Hell.Events.Hubur.Skull as Skull
import qualified Sites.Hell.Events.Hubur.Cave as Cave
import qualified Sites.Hell.Events.Hubur.Obelisk as Obelisk
import qualified Sites.Hell.Events.Hubur.StandingStones as StandingStones
import qualified Sites.Hell.Events.Hubur.CedarTree as CedarTree
import qualified Sites.Hell.Events.Hubur.SwampThicket as SwampThicket

import qualified Sites.Hell.Events.Hubur.Bubbles as Bubbles
import qualified Sites.Hell.Events.Hubur.OldBones as OldBones
import qualified Sites.Hell.Events.Hubur.TreasurePile as TreasurePile

import qualified Sites.Hell.Events.Hubur.HoundKnight as HoundKnight
import qualified Sites.Hell.Events.Hubur.MinotaurMiner as MinotaurMiner
import qualified Sites.Hell.Events.Hubur.SlimeGuy as SlimeGuy
import qualified Sites.Hell.Events.Hubur.CrystalDragonoid as CrystalDragonoid

import qualified Sites.Hell.Events.Hubur.Barracks as Barracks
import qualified Sites.Hell.Events.Hubur.VoidSnake as VoidSnake

import qualified Sites.Hell.Events.Hubur.Amduscias as Amduscias
import qualified Sites.Hell.Events.Hubur.CrystalGolem as CrystalGolem
import qualified Sites.Hell.Events.Hubur.FeralHound as FeralHound
import qualified Sites.Hell.Events.Hubur.Nightmare as Nightmare
import qualified Sites.Hell.Events.Hubur.PaperRook as PaperRook
import qualified Sites.Hell.Events.Hubur.Shadow as Shadow
import qualified Sites.Hell.Events.Hubur.ShadowMage as ShadowMage
import qualified Sites.Hell.Events.Hubur.SuitedDemon as SuitedDemon
import qualified Sites.Hell.Events.Hubur.TarpitDemon as TarpitDemon
import qualified Sites.Hell.Events.Hubur.Unicorn as Unicorn
import qualified Sites.Hell.Events.Hubur.WillOWisp as WillOWisp
import qualified Sites.Hell.Events.Hubur.ZombieHierophant as ZombieHierophant
import qualified Sites.Hell.Events.Hubur.Zombie as Zombie
import qualified Sites.Hell.Events.Hubur.RockDemon as RockDemon

import qualified Sites.Hell.Events.Hubur.Asakku as Asakku
import qualified Sites.Hell.Events.Hubur.NatureSpirit as NatureSpirit
import qualified Sites.Hell.Events.Hubur.Humbaba as Humbaba
import qualified Sites.Hell.Events.Hubur.UmuDemon as UmuDemon


buildScene :: (Scene i vars mob, Eq i, FromJSON mob) => BodyGlob -> MarkedScene i vars mob -> Value -> Maybe ([SceneBloc], Maybe (Tree String))
buildScene pcBody scene obj = case fromJSON obj of
	Success enc -> Just (sceneDisplay pcBody enc scene, fauxTree enc)
	_ -> Nothing

firstParse :: [p -> Maybe a] -> a -> p -> a
firstParse (f:rs) err p = case f p of
	Just a -> a
	Nothing -> firstParse rs err p
firstParse [] err _ = err

scenes :: BodyGlob -> Value -> ([SceneBloc], Maybe (Tree String))
scenes pcBody = firstParse
	[ buildScene pcBody FlameDemon.scene
	, buildScene pcBody HamarTabal.scene
	, buildScene pcBody ClayMan.scene
	, buildScene pcBody LesserHound.scene
	, buildScene pcBody SobekSheut.scene
	, buildScene pcBody Sahagin.scene
	, buildScene pcBody Dryad.scene
	, buildScene pcBody GanzerSoldier.scene
	, buildScene pcBody MechanicShop.scene
	, buildScene pcBody FlareStack.scene
	, buildScene pcBody Gargoyle.scene

	, buildScene pcBody Opening.scene -- do we really need this?
	, buildScene pcBody FarmerHamarTabal.scene
	, buildScene pcBody RandomDrop.scene
	, buildScene pcBody HellGate.scene
	, buildScene pcBody Worms.scene
	, buildScene pcBody Skull.scene
	, buildScene pcBody Cave.scene
	, buildScene pcBody Barracks.scene
	, buildScene pcBody Obelisk.scene
	, buildScene pcBody StandingStones.scene
	, buildScene pcBody CedarTree.scene
	, buildScene pcBody SwampThicket.scene

	, buildScene pcBody Bubbles.scene
	, buildScene pcBody OldBones.scene
	, buildScene pcBody TreasurePile.scene

	, buildScene pcBody HoundKnight.scene

	, buildScene pcBody MinotaurMiner.scene
	, buildScene pcBody SlimeGuy.scene
	, buildScene pcBody CrystalDragonoid.scene

	, buildScene pcBody Amduscias.scene
	, buildScene pcBody Cave.scene
	, buildScene pcBody CrystalGolem.scene
	, buildScene pcBody FeralHound.scene
	, buildScene pcBody Nightmare.scene
	, buildScene pcBody PaperRook.scene
	, buildScene pcBody Shadow.scene
	, buildScene pcBody ShadowMage.scene
	, buildScene pcBody SuitedDemon.scene
	, buildScene pcBody TarpitDemon.scene
	, buildScene pcBody Unicorn.scene
	, buildScene pcBody WillOWisp.scene
	, buildScene pcBody ZombieHierophant.scene
	, buildScene pcBody Zombie.scene
	, buildScene pcBody VoidSnake.scene
	, buildScene pcBody RockDemon.scene

	, buildScene pcBody Asakku.scene
	, buildScene pcBody NatureSpirit.scene
	, buildScene pcBody Humbaba.scene
	, buildScene pcBody UmuDemon.scene
	] $ ([], Nothing)

eventAPI :: RouteT SiteRoute HellServer Value
eventAPI = do
	body <- liftM (decodeStrict . pack) . look $ "body"
	ngender' <- liftM (fromJSON . String . T.pack) . look $ "noun" -- we can't decode, since it's not a complete JSON object, just a string, and strings aren't valid top-level JSON values
	event <- liftM (decodeStrict . pack) . look $ "event"
	let evs = case (ngender', body, event) of
		(Success ngender, Just (parse jsonToTree -> Success b), Just (Encounter _ (Object ev))) -> case scenes (build ngender You b) (Object ev) of
				([], Nothing) -> Error "invalid or unhandled event"
				sceneBlocs -> pure $ sceneBlocs
		_ -> Error "can't make heads or tails of that request, friend"
	-- we need to evaluate evs to force any potential scene exception to generate so we can catch it. and we need to evaluate it all the way, since a failing scene is gonna appear as an error somewhere in the guts of the event list, & won't necessarily show up the instant we evaluate evs to WHNF
	result <- lift $ ((return $!! evs) `catch` catchError)
	return . object $ case result of
		Success (evs', evBody) ->
			[ "events" .= evs'
			, "title" .= ("EVENT" :: String)
			] <> case evBody of
				Nothing -> []
				Just b -> pure $ "body" .= b
		Error err -> ["error" .= err]
	where
		-- generateScene :: Pronoun -> Tree TfNode -> HashMap Text Value -> MarkedScene i mob -> Result [SceneBloc]
		generateScene ngender b ev scene =
			sceneDisplay (build ngender You b)
				<$> parse parseJSON (Object ev)
				<*> pure scene

-- the hope is displaying these errors won't EXPOSE INTERNAL SURFACE too much, since like... they're only body parse errors.
catchError :: SomeException -> HellServer (Result a)
catchError ex = do
	lift . lift $ logLn WARNING $ concat
		[ "*** error on next line: "
		, show ex]
	return $ Error $ show ex
