{-# LANGUAGE OverloadedStrings #-}

module Sites.Hell.Routes.Population
	( populationAPI
	) where

{- TO ADD:

badland
	obelisk

shallows
	bubbles
	vortex

swamp
	ganzer fortress (1)

glade
	unicorn glade

factory-quarry

factory-tarpit
	tarpit throne (1)

factory-tunnels
	mining barracks


factory-processing
	flare stack

factory-plant
	{ooze vats}

factory-grotto
	crystal(?) throne (1)


factory-research
	alchemy shop

factory-stacks

factory-pulpit
	flame throne (1)

-}

import Control.Monad
import Data.Maybe (listToMaybe)
import Data.Enumerable

-- happstack
import Server.Error (HellServer)
import Happstack.Server (look{-, badRequest-})

-- web-routes
import Web.Routes (RouteT)
-- for the HasRqData instance for RouteT
import Web.Routes.Happstack ()

import Utility.Rand
import Server.JSON
import Sites.Hell.Sitemap (SiteRoute(..))

import Data.Text (pack, Text)

import qualified Sites.Hell.Events.Hubur as Hubur

import qualified Sites.Hell.Events.Hubur.ClayMan as ClayMan (name, deck, bits, variants)
import qualified Sites.Hell.Events.Hubur.Dryad as Dryad (name, deck, bits, variants)
import qualified Sites.Hell.Events.Hubur.FlameDemon as FlameDemon
	(name, deck, bits, variants, mineName, mineDeck, mineBits, mineVariants)
import qualified Sites.Hell.Events.Hubur.GanzerSoldier as GanzerSoldier (name, deck, bits, variants)
import qualified Sites.Hell.Events.Hubur.Gargoyle as Gargoyle (name, deck, bits, variants)
import qualified Sites.Hell.Events.Hubur.HamarTabal as HamarTabal (name, factoryDeck, bits, variants)
import qualified Sites.Hell.Events.Hubur.LesserHound as LesserHound (name, deck, bits, variants)
import qualified Sites.Hell.Events.Hubur.Sahagin as Sahagin (name, deck, bits, variants)
import qualified Sites.Hell.Events.Hubur.SobekSheut as SobekSheut (name, deck, bits, variants)
import qualified Sites.Hell.Events.Hubur.Unicorn as Unicorn (name, deck, bits, variants)
import qualified Sites.Hell.Events.Hubur.HoundKnight as HoundKnight (name, deck, bits, variants)
import qualified Sites.Hell.Events.Hubur.FeralHound as FeralHound (name, deck, bits, variants)
import qualified Sites.Hell.Events.Hubur.Nightmare as Nightmare (name, deck, bits, variants)
import qualified Sites.Hell.Events.Hubur.Zombie as Zombie (name, wildDeck, skullDeck, bits, variants)
import qualified Sites.Hell.Events.Hubur.ZombieHierophant as ZombieHierophant (name, deck, bits, variants)
import qualified Sites.Hell.Events.Hubur.Amduscias as Amduscias (name, deck, bits, variants)
import qualified Sites.Hell.Events.Hubur.Shadow as Shadow (name, deck, bits, variants)
import qualified Sites.Hell.Events.Hubur.ShadowMage as ShadowMage (name, deck, bits, variants)
import qualified Sites.Hell.Events.Hubur.PaperRook as PaperRook (name, deck, bits, variants)
import qualified Sites.Hell.Events.Hubur.TarpitDemon as TarpitDemon (name, deck, bits, variants)
import qualified Sites.Hell.Events.Hubur.CrystalGolem as CrystalGolem (name, deck, bits, variants)
import qualified Sites.Hell.Events.Hubur.SuitedDemon as SuitedDemon (name, deck, bits, variants)
import qualified Sites.Hell.Events.Hubur.WillOWisp as WillOWisp (name, deck, bits, variants)
import qualified Sites.Hell.Events.Hubur.VoidSnake as VoidWorm (name, deck, bits, variants)
import qualified Sites.Hell.Events.Hubur.MinotaurMiner as MinotaurMiner (name, deck, bits, variants)
import qualified Sites.Hell.Events.Hubur.SlimeGuy as SlimeGuy (name, deck, bits, variants)
import qualified Sites.Hell.Events.Hubur.CrystalDragonoid as CrystalDragonoid (name, deck, bits, variants)

import qualified Sites.Hell.Events.Hubur.Humbaba as Humbaba (name, deck, bits, variants)
import qualified Sites.Hell.Events.Hubur.UmuDemon as UmuDemon (name, deck, bits, variants)
import qualified Sites.Hell.Events.Hubur.NatureSpirit as NatureSpirit (name, deck, bits, variants)
import qualified Sites.Hell.Events.Hubur.Asakku as Asakku (name, deck, bits, variants)
import qualified Sites.Hell.Events.Hubur.RockDemon as RockDemon (deck)

-- import qualified Sites.Hell.Events.Hubur.RandomDrop as RandomDrop
-- import qualified Sites.Hell.Events.Hubur.Worms as Worms

-- import qualified Sites.Hell.Events.Hubur.AlchemyShop as AlchemyShop
-- import qualified Sites.Hell.Events.Hubur.FlareStack as FlareStack
import qualified Sites.Hell.Events.Hubur.Barracks as Barracks
import qualified Sites.Hell.Events.Hubur.MechanicShop as MechanicShop
import qualified Sites.Hell.Events.Hubur.FlareStack as FlareStack
import qualified Sites.Hell.Events.Hubur.Cave as Cave
import qualified Sites.Hell.Events.Hubur.FarmerHamarTabal as Farm
import qualified Sites.Hell.Events.Hubur.HellGate as HellGate
import qualified Sites.Hell.Events.Hubur.Skull as Skull
import qualified Sites.Hell.Events.Hubur.Obelisk as Obelisk
import qualified Sites.Hell.Events.Hubur.StandingStones as StandingStones
import qualified Sites.Hell.Events.Hubur.CedarTree as CedarTree
import qualified Sites.Hell.Events.Hubur.SwampThicket as SwampThicket

import qualified Sites.Hell.Events.Hubur.Bubbles as Bubbles
import qualified Sites.Hell.Events.Hubur.OldBones as OldBones
import qualified Sites.Hell.Events.Hubur.TreasurePile as TreasurePile

mread :: Read a => String -> Maybe a
mread = (\(a', rs) -> if null rs then Just a' else Nothing) <=< listToMaybe . reads

populationAPI :: RouteT SiteRoute HellServer Value
populationAPI = do
	s <- liftM mread . look $ "seed"
	loc <- look $ "map"
	case (s, loc) of
		(Just seed, "hubur") -> return $ huburPopulation seed
		(Nothing, _) -> return . object $ ["error" .= ("bad seed" :: String)]
		_ -> return . object $ ["error" .= ("bad map" :: String)]


varList :: Enum l => [(Hubur.MapZone, l)] -> Value
varList vars = object $ (\(z, l) -> pack (Hubur.mapLabel z) .= fromEnum l) <$> vars

locationTemplate :: Enum l => String -> (Value -> Value) -> [(Hubur.MapZone, l)] -> (Text, Value)
locationTemplate name bits variants = pack name .= object
		[ "zones" .= (Hubur.mapLabel . fst <$> variants)
		, "variants" .= (bits . varList $ variants)
		]

{- this would have to return something like...

{ "factory-plant": [enc, enc, ...]
, "swamp": [enc, enc, ...]
, ...
}

except JUST that doesn't handle the moving case: when `enc` can fit into many zone

{ pop:
	[ {enc: encdata
		, where:
			["factory-plant", "factory-processing", "factory-quarry", "factory-tunnels"
			]
		}
	]
, switch:
	[ { template:
			{ name: "son of hamar-tabal"
			-- a null value is one that is untouched by movement; a key-index is a data value that needs to be set to a specific key if the encounter is populated into that region.
			, data:
				[ null
				, null
				, null
				, {"factory-processing": 1, "factory-plant": 2, "factory-quarry": 0, "factory-tunnels": 3}
				]
			}
	]


and the build of that would look like...
-}


{-
okay:
X. attach a number to every generated encounter, and get the client-side parser recording and storing IDs

2. locations that take existing encounter data (anything that takes part in a `populationSplit` call) should instead... be generated after their component encounters, and be handed a list of ids?
	that would involve changing their constructors to be dumber and not type-safe, though (since they could be getting any id, not necessarily an id matching their expected type). maybe still have the constructor take the full type, but change its output. basically outside of generation, encounters shouldn't know about ids at all, so we shouldn't totally revise encounter handling to deal with ids.

3. somehow attach a 'used' tag to encounters that were used in the above, and make the client side 1. record the full population list and 2. set the same 'used' tag on encounters placed on the map

4. the locations like the machine shop or farm that generate their own NPCs should instead take an npc from the reels as above

* in related but distinct changes, encounters should transmit their own names and icons. this is for various reasons (different tree styles, will-o-wisp changes, display text showing encounter type / NPC name after visit)

on the client side:
* blah blah when sending an event it should replace npc ids with the npc data, and after an event runs it should replace the npc data with an altered version
* map generation should have limits for number of npcs in a given zone, to prevent the solid-packed setups we have now
-}
huburPopulation :: Int -> Value
huburPopulation seed = evalRand (mkStdGen seed) $ do
	-- moving, respawning encounters
	dryads <- populationRaw popCount Dryad.deck -- :: [Dryad]
	unicorns <- populationRaw popCount Unicorn.deck -- :: [Unicorn]
	natureSpirits <- populationRaw popCount NatureSpirit.deck -- :: [NatureSpirit]

	(cedarD, dryads') <- populationSplit 7 CedarTree.dryadDeck dryads
	(cedarU, unicorns') <- populationSplit 7 CedarTree.unicornDeck unicorns
	(cedarN, natureSpirits') <- populationSplit 7 CedarTree.natureSpiritDeck natureSpirits


	asakku <- populationRaw popCount Asakku.deck
	rockDemons <- populationRaw popCount RockDemon.deck
	(caveA, asakku') <- populationSplit 8 Cave.asakkuDeck asakku
	(caveR, _) <- populationSplit 8 Cave.rockDemonDeck rockDemons

	skullZombies <- populationRaw popCount Zombie.skullDeck
	(skullZ, _) <- populationSplit 12 Skull.zombieDeck skullZombies

	pops <- fmap join . sequence $
		-- hubur general
		[ population popCount ClayMan.deck
		, population popCount UmuDemon.deck
		, pure $ toJSON <$> asakku'
--		, pure $ toJSON <$> rockDemons' -- rock demons only show up in caves
--		, population popCount RandomDrop.deck
--		, population popCount Worms.deck

		-- glade
		, pure $ toJSON <$> dryads'
		, pure $ toJSON <$> unicorns'
		, population popCount Humbaba.deck
		, pure $ toJSON <$> natureSpirits'
		-- badland
		, population popCount LesserHound.deck
		, population popCount HoundKnight.deck
		, population popCount FeralHound.deck
		, population popCount Nightmare.deck
		-- shallows
		, population popCount Sahagin.deck
		-- swamp
		, population popCount WillOWisp.deck
		, population popCount SobekSheut.deck
		, population popCount Zombie.wildDeck
		, population popCount ZombieHierophant.deck

		-- factory general
		, population popCount GanzerSoldier.deck
		, population popCount Gargoyle.deck
		, population popCount HamarTabal.factoryDeck
		-- factory-tunnels (/factory-quarry, for some)
		, population popCount MinotaurMiner.deck
		, population popCount SlimeGuy.deck
		, population popCount CrystalDragonoid.deck
		, population popCount FlameDemon.mineDeck
		, population popCount CrystalGolem.deck
		, population popCount VoidWorm.deck
		-- factory-tarpit
		, population popCount TarpitDemon.deck
		-- factory-stacks (/factory-research, for some)
		, population popCount FlameDemon.deck
		, population popCount Amduscias.deck
		, population popCount Shadow.deck
		, population popCount ShadowMage.deck
		, population popCount PaperRook.deck
		-- factory-plant (/factory-processing, for some)
		, population popCount SuitedDemon.deck
		-- there are no location-tagged mud/sand sections, so this is technically an 'encounter'
		, population 48 TreasurePile.deck
		-- likewise, we don't want vanishing events to use up a location slot
		, population 24 OldBones.deck
		]
	-- fixed locations
	locs <- fmap join . sequence $
		-- factory-processing locations
		[ population 24 MechanicShop.deck
		, population 24 FlareStack.deck
		-- factory-research
		-- , population locCount AlchemyShop.deck
		-- factory-tunnels
		, population 7 Barracks.deck
		-- fields locations
		, population 24 Farm.deck
		, population 24 StandingStones.deck
		-- swamp locations
		, population 4 Skull.emptyDeck
		, pure skullZ
		, population 24 SwampThicket.deck
		-- badland locations
		, populationRoll 24 Cave.mineralDeck
		, pure caveA
		, pure caveR
		, population 24 Obelisk.deck
		, population 1 HellGate.deck
		-- glade locations
		, pure cedarD
		, pure cedarU
		, pure cedarN
		-- misc. treasure
		, population 24 Bubbles.deck
		]
	let idPops = uncurry toIdTagged <$> zip [0..] pops
	let idLocs = uncurry toIdTagged <$> zip [(length pops)..] locs
	return $ object
		[ "pop" .= idPops
		, "loc" .= idLocs
		, "zoneTemplate" .= object
			[ locationTemplate "pcLocation" (\_ -> Null)
				[ (Hubur.SwampSobek, ())
				, (Hubur.Field, ())
				]

			, locationTemplate ClayMan.name ClayMan.bits ClayMan.variants
			, locationTemplate UmuDemon.name UmuDemon.bits UmuDemon.variants
			, locationTemplate Asakku.name Asakku.bits Asakku.variants
			--, locationTemplate RockDemon.name RockDemon.bits RockDemon.variants
			, locationTemplate Dryad.name Dryad.bits Dryad.variants
			, locationTemplate NatureSpirit.name NatureSpirit.bits NatureSpirit.variants
			-- uHHH
			, locationTemplate FlameDemon.name FlameDemon.bits FlameDemon.variants
			, locationTemplate FlameDemon.mineName FlameDemon.mineBits FlameDemon.mineVariants

			, locationTemplate GanzerSoldier.name GanzerSoldier.bits GanzerSoldier.variants
			, locationTemplate Gargoyle.name Gargoyle.bits Gargoyle.variants
			, locationTemplate HamarTabal.name HamarTabal.bits HamarTabal.variants
			, locationTemplate LesserHound.name LesserHound.bits LesserHound.variants
			, locationTemplate Sahagin.name Sahagin.bits Sahagin.variants
			, locationTemplate SobekSheut.name SobekSheut.bits SobekSheut.variants
			, locationTemplate Unicorn.name Unicorn.bits Unicorn.variants
			, locationTemplate Humbaba.name Humbaba.bits Humbaba.variants
			, locationTemplate HoundKnight.name HoundKnight.bits HoundKnight.variants
			, locationTemplate FeralHound.name FeralHound.bits FeralHound.variants
			, locationTemplate Nightmare.name Nightmare.bits Nightmare.variants
			, locationTemplate Zombie.name Zombie.bits Zombie.variants
			, locationTemplate ZombieHierophant.name ZombieHierophant.bits ZombieHierophant.variants
			, locationTemplate Amduscias.name Amduscias.bits Amduscias.variants
			, locationTemplate Shadow.name Shadow.bits Shadow.variants
			, locationTemplate ShadowMage.name ShadowMage.bits ShadowMage.variants
			, locationTemplate PaperRook.name PaperRook.bits PaperRook.variants
			, locationTemplate TarpitDemon.name TarpitDemon.bits TarpitDemon.variants
			, locationTemplate CrystalGolem.name CrystalGolem.bits CrystalGolem.variants
			, locationTemplate SuitedDemon.name SuitedDemon.bits SuitedDemon.variants
			, locationTemplate WillOWisp.name WillOWisp.bits WillOWisp.variants
			, locationTemplate VoidWorm.name VoidWorm.bits VoidWorm.variants
			, locationTemplate MinotaurMiner.name MinotaurMiner.bits MinotaurMiner.variants
			, locationTemplate SlimeGuy.name SlimeGuy.bits SlimeGuy.variants
			, locationTemplate CrystalDragonoid.name CrystalDragonoid.bits CrystalDragonoid.variants

			, locationTemplate MechanicShop.name MechanicShop.bits MechanicShop.variants
			, locationTemplate Barracks.name Barracks.bits Barracks.variants
			, locationTemplate Farm.name Farm.bits Farm.variants

			, locationTemplate Skull.name Skull.bits Skull.variants
			, locationTemplate Cave.name Cave.bits Cave.variants
			, locationTemplate HellGate.name HellGate.bits HellGate.variants
			, locationTemplate Obelisk.name Obelisk.bits Obelisk.variants
			, locationTemplate StandingStones.name StandingStones.bits StandingStones.variants
			, locationTemplate CedarTree.name CedarTree.bits CedarTree.variants
			, locationTemplate SwampThicket.name SwampThicket.bits SwampThicket.variants
{-
alchemy shop (factory refining)
-}
			, locationTemplate FlareStack.name FlareStack.bits FlareStack.variants

			, locationTemplate Bubbles.name Bubbles.bits Bubbles.variants
			, locationTemplate TreasurePile.name TreasurePile.bits TreasurePile.variants
			, locationTemplate OldBones.name OldBones.bits OldBones.variants
			]
		]
	where
		toIdTagged :: ToJSON a => Int -> a -> Value
		toIdTagged id obj = object ["id" .= id, "event" .= obj]
		popCount = 66
		locCount = 7

population :: (RandomGen g, ToJSON a) => Int -> Rand g (Enumerable a) -> Rand g [Value]
population = populationEnum
populationEnum :: (RandomGen g, ToJSON a) => Int -> Rand g (Enumerable a) -> Rand g [Value]
populationEnum count rdeck = do
	deck <- rdeck
	return $ toJSON <$> (take count . cycle $ enumerate deck)
populationRoll :: (RandomGen g, ToJSON a) => Int -> Rand g (Enumerable a) -> Rand g [Value]
populationRoll count rdeck = fmap toJSON <$> rollX count rdeck
populationRaw :: (RandomGen g, ToJSON a) => Int -> Rand g (Enumerable a) -> Rand g [a]
populationRaw count rdeck = do
	deck <- rdeck
	return $ (take count . cycle $ enumerate deck)
populationSplit :: (RandomGen g, ToJSON a)
	=> Int
	-> (s -> Rand g (Enumerable a))
	-> [s]
	-> Rand g ([Value],[s])
populationSplit c deck pops =
	(,)
		<$> wrapped'
		<*> pure (drop c pops)
	where
		{-
		-- wrapped :: RandomGen g => Rand g [Value]
		wrapped = fmap (fmap toJSON . take c . cycle . enumerate . mconcat)
			. traverse deck
				$ pops
		-}
		-- this can reroll the same value, but it will at least get a better span of the available options, if they're large
		-- wrapped' :: RandomGen g => Rand g [Value]
		wrapped' = fmap (fmap toJSON) . rollX c . fmap mconcat . traverse deck $ pops
rollX :: RandomGen g => Int -> Rand g (Enumerable a) -> Rand g [a]
rollX i renum = (\enum -> replicateM i (roll enum)) =<< renum
