{-# LANGUAGE OverloadedStrings #-}
{-# LANGUAGE TypeFamilies #-}

module Sites.Hell.Routes.Splash
	( splashPage
	, newsPostPage
	, newsFeed
	) where

import Control.Monad
--import Control.Monad.Trans

import Web.Routes (RouteT, showURL)
import Server.Error (HellServer)
import Text.Blaze.Html.Renderer.String (renderHtml)
import qualified Text.Blaze.Html.Renderer.Text as Text (renderHtml)
import qualified Text.Blaze.Html5 as H
import qualified Text.Blaze.Html5.Attributes as A

import Sites.Hell.Sitemap (SiteRoute(..))
import Server.Layout

import Data.Foldable (foldl')
import Data.Maybe (fromMaybe, maybeToList)
import Data.Time.Clock (UTCTime(..))
--import Data.Time.Calendar (Day(..))
import Data.Time (formatTime, defaultTimeLocale, hoursToTimeZone, localTimeToUTC)
import Data.Text (Text, strip, toLower)
import qualified Data.Text as Text (null, lines, filter)
import Data.Text.Lazy (toStrict)

import qualified Text.RSS as RSS (Item, ItemElem(..))

-- reform stuff
import Happstack.Server (Input)
import Text.Reform
import Text.Reform.Happstack (happstackEitherForm, happstackViewForm)
import Text.Reform.Blaze.Text
 -- this has `instance Happstack m => Happstack (RouteT url m)`
import Web.Routes.Happstack ()

-- import Server.Database
import Data.Convertible.Base (ConvertResult, ConvertError(..))

data Newspost = Newspost
	{ newsPostId :: Int
	, whenPosted :: UTCTime
	, hilight :: Bool
	, commentCount :: Int
	, contents :: Html
	}

data Comment = Comment
	{ commentId :: Int
	, commentPosted :: Either String UTCTime
	, hidden :: Bool
	, byAdmin :: Bool
	, author :: Text
	, body :: Text
	}
	deriving (Show)

splashPage :: Maybe Int -> RouteT SiteRoute HellServer Html
splashPage mpage = do
	newsPosts <- pure $ Right [] -- withHellDB $ splashDB mpage 10
	newsPosts' <- either (pure . return) (mapM $ newsItem True) newsPosts
	splash newsPosts' mpage 10

newsPostPage :: Int -> RouteT SiteRoute HellServer Html
newsPostPage id_ =
		if id_ <= 0
		then mzero
		else do
			mpost <- pure Nothing -- withHellDB $ newsPostDB id_
			case mpost of
				Nothing -> mzero
				Just (newspost, comments) -> do
					-- check for form posts
					action <- showURL $ NewsPost id_
					formResult <- happstackEitherForm (form action) "comment" commentForm
					(commentPostOutput, fauxComment) <- case formResult of
						Left _ -> do
							(return (Nothing, Nothing) :: RouteT SiteRoute HellServer (Maybe Html, Maybe Comment))
						Right comment@(CommentSubmit author _ text) -> do
							worked <- pure $ Left (return ()) -- withHellDB $ insertCommentDB comment (newsPostId newspost)
							return $ case worked of
								Left errHtml -> (Just errHtml, Nothing)
								Right fauxComment -> (Just (H.p ! A.class_ "notice" $ "comment posted"), Just fauxComment)
					let newspost' = case fauxComment of
						Nothing -> newspost
						Just _ -> newspost {commentCount = commentCount newspost + 1}
					formHtml <- happstackViewForm (form action) "comment" commentForm
					-- output
					newsHtml <- newsItem False newspost'
					withHeader $ do
						case commentPostOutput of
							Just html -> html
							_ -> mempty
						H.ul ! A.id "news" $ do
							H.li $ newsHtml
						H.ol ! A.id "comments" $
							mapM_ commentItem (comments <> maybeToList fauxComment)
						H.hr
						H.div ! A.id "comment" $ do
							H.h3 "leave a new comment"
							formHtml

newsFeed :: RouteT SiteRoute HellServer [RSS.Item]
newsFeed = do
	newsposts <- pure $ Right [] -- withHellDB $ splashDB Nothing 20
	let newsposts' = case newsposts of
		Left _ -> []
		Right vs -> vs
	return $ newsItemRSS <$> newsposts'

data AppError
	= AppCFE (CommonFormError [Input])
	deriving Show

instance FormError AppError where
	type ErrorInputType AppError = [Input]
	commonFormError = AppCFE

type HellForm a = Form (RouteT SiteRoute HellServer) [Input] AppError Html () a

data CommentSubmit = CommentSubmit Text Text Text

commentForm :: HellForm CommentSubmit
commentForm = CommentSubmit
	<$> label ("name" :: Html) ++> inputText "" <++ br
	<*> label ("what's the name of this game? (this is to stop spambots)" :: Html) ++> inputText "" <++ br
	<*> label ("comment" :: Html) ++> textarea 80 40 "" <++ br
	<* view ("No HTML allowed in comments; plain text only." >> H.br)
	<* inputSubmit "post comment"

withHeader :: Html -> RouteT SiteRoute HellServer Html
withHeader content = do
	splashUrl <- showURL (Splash Nothing)
	gameUrl <- showURL Game
	infoUrl <- showURL Info
	todoUrl <- showURL Todo
	goalUrl <- showURL Goal
	rssUrl <- showURL NewsFeed
	return $ minimal "HELL GAME"
		(do
			stylesheet "/static/css/main.css"
			H.link ! A.rel "alternate" ! A.type_ "application/rss+xml" ! A.href (toValue rssUrl)
		)
		(do
			H.h1 ! A.id "main" $
				link splashUrl $ do
					H.span ! A.id "hell" $ "HELL"
					H.span ! A.id "game" $ "GAME"
			H.nav $ do
				H.ul $ do
					H.li ! A.id "play" $ link gameUrl ("PLAY" :: Html)
					H.li $ link infoUrl ("more info" :: Html)
					H.li $ link goalUrl ("project goals" :: Html)
					--H.li $ link todoUrl ("code TODO" :: Html)
					H.li ! A.class_ "patreon" $ link ("http://patreon.com/xax" :: String) ("patreon" :: Html)
			H.div ! A.id "main" $ content
			)

splash :: [Html] -> Maybe Int -> Int -> RouteT SiteRoute HellServer Html
splash newsItems mpage count = do
	let (prevUrl_, nextUrl_) = case (mpage, length newsItems == count) of
		(Nothing, True) -> (Nothing, Just $ Splash $ Just 2)
		(Nothing, False) -> (Nothing, Nothing)
		(Just 1, True) -> (Nothing, Just $ Splash $ Just 2)
		(Just 1, False) -> (Nothing, Nothing)
		(Just 2, True) -> (Just $ Splash Nothing, Just $ Splash $ Just 3)
		(Just 2, False) -> (Just $ Splash Nothing, Nothing)
		(Just x, True) -> (Just $ Splash $ Just (x - 1), Just $ Splash $ Just (x + 1))
		(Just x, False) -> (Just $ Splash $ Just (x - 1), Nothing)
	prevUrl <- case showURL <$> prevUrl_ of
		Nothing -> return Nothing
		Just x -> Just <$> x
	nextUrl <- case showURL <$> nextUrl_ of
		Nothing -> return Nothing
		Just x -> Just <$> x
	let prevHtml = fromMaybe (return ())
		$ (\x -> H.li (link x (H.preEscapedToHtml ("&larr; newer newsposts" :: String))))
			<$> prevUrl
	let nextHtml = fromMaybe (return ())
		$ (\x -> H.li (link x (H.preEscapedToHtml ("older newsposts &rarr;" :: String))))
			<$> nextUrl
	withHeader $ do
		H.ul ! A.id "news" $ do
			sequence_ . fmap H.li $ newsItems
		H.ul ! A.id "nav" $ prevHtml >> nextHtml

newsItem :: Bool -> Newspost -> RouteT SiteRoute HellServer Html
newsItem linkComments (Newspost id_ date _ commentCount content) = do
	postURL <- showURL $ NewsPost id_
	let newCommentURL = postURL <> "#comment"
	let readCommentURL = postURL <> "#commenttop"
	return $ do
		H.h2 $ toHtml $ formatTime defaultTimeLocale "%B %e, %Y" date
		H.div ! A.class_ "content" $ do
			content
			H.div ! A.class_ "commentanchor" $ if commentCount == 0
					then do
						if linkComments then link newCommentURL ("leave a comment" :: Html) else return ()
					else do
						let linkText = show commentCount <> (if commentCount == 1
								then " comment"
								else " comments")
						if linkComments
							then do
								link readCommentURL $ toHtml linkText
								link newCommentURL ("leave a comment" :: Html)
							else H.span ! A.id "commenttop" $ toHtml linkText

newsItemRSS :: Newspost -> RSS.Item
newsItemRSS (Newspost _ date _ _ content) =
	[ RSS.Title $ formatTime defaultTimeLocale "%B %e, %Y" date
	, RSS.Description $ renderHtml content
	, RSS.PubDate date
	]

commentItem :: Comment -> Html
commentItem (Comment id_ when _ byAdmin author content) = do
	let foo = if byAdmin then (! A.class_ "admin comment") else (! A.class_ "comment")
	foo H.li $ do
		H.div ! A.class_ "commentbar" $ do
			H.span ! A.class_ "name" $ toHtml author
			H.span ! A.class_ "date" $ toHtml $ either id (formatTime defaultTimeLocale "%l:%M%P, %B %e, %Y") when -- 12:46am, January 1, 2015
		H.div ! A.class_ "commentbody" $ H.preEscapedToHtml content

{-
newsPostDB :: IConnection conn => Int -> conn -> IO (Maybe (Newspost, [Comment]))
newsPostDB id_ conn = do
	q <- prepare conn "SELECT newspost.id, newspost.date, newspost.hilight, newspost.body, COUNT (newspost_comment.id) FROM newspost LEFT JOIN newspost_comment ON (newspost.id = newspost_comment.newspostid) AND newspost_comment.hidden = 'false' WHERE newspost.id = ? GROUP BY newspost.id, newspost.date, newspost.hilight, newspost.body;"
	execute q [iToSql id_]
	res <- liftM (fmap newsParse) $ fetchRow q
	case res of
		Nothing -> return Nothing
		Just (Right newspost) -> do
			q' <- prepare conn "SELECT newspost_comment.id, newspost_comment.date, newspost_comment.hilight, newspost_comment.hidden, newspost_comment.author, newspost_comment.body FROM newspost_comment WHERE newspost_comment.newspostid = ? AND newspost_comment.hidden = false ORDER BY newspost_comment.date ASC;"
			execute q' [iToSql $ newsPostId newspost]
			res <- liftM (mapM commentParse) $ fetchAllRows' q'
			case res of
				Right comments -> return $ Just (newspost, comments)
				_ -> return Nothing
		_ -> return Nothing

insertCommentDB :: IConnection conn => CommentSubmit -> Int -> conn -> IO (Either Html Comment)
insertCommentDB (CommentSubmit author auth text) newspost conn = if spam
	then return $ Left $ H.p ! A.class_ "error" $ "comment not posted; spam-detection question not answered correctly (hint: look up, cmon) (you should be able to hit 'back' to get your comment)"
	else do
		let authorName = if Text.null author then "anonymous" else author
		-- escape the text before we put it into the db
		let escapedText = escapeHtml text
		q <- prepare conn "INSERT INTO newspost_comment(newspostid, author, body) VALUES (?, ?, ?);"
		r <- execute q [iToSql newspost, toSql authorName, toSql escapedText]
		return $ if r == 1
			then Right $ Comment 0 (Left "now") False False authorName escapedText
			else Left $ H.p ! A.class_ "error" $ "comment not posted; db issues"
	where
		spam = authStr /= "hellgame" && authStr /= "hellgate"
		authStr = toLower $ Text.filter (\c -> not $ c `elem` [' ', '.', '?', '!']) $ auth
-}

escapeHtml :: Text -> Text
escapeHtml text =
		  toStrict
		. Text.renderHtml -- convert it to html and then turn that html into text
		. ($ mempty)
		. fst
		. foldl' commentFold (H.p, True) -- fold through it using a continuation to determine where the next line should go (in an existing paragraph or as a new paragraph)
		. fmap strip -- remove leading/trailing whitespace
		. Text.lines -- split into paragraphs
			$ text
	where
		commentFold :: (Html -> Html, Bool) -> Text -> (Html -> Html, Bool)
		commentFold (cont, newparagraph) n
			| Text.null n = if newparagraph
				then (cont, True)
				else (\a -> cont mempty >> H.p a, True)
			| otherwise = if newparagraph
				then (cont . (toHtml n >>), False)
				else (cont . (H.br >> toHtml n >>), False)

{-
splashDB :: IConnection conn => Maybe Int -> Int -> conn -> IO (Either Html [Newspost])
splashDB mpage limit conn = do
	q <- prepare conn "SELECT newspost.id, newspost.date, newspost.hilight, newspost.body, COUNT (newspost_comment.id) FROM newspost LEFT JOIN newspost_comment ON (newspost.id = newspost_comment.newspostid) AND newspost_comment.hidden = 'false' GROUP BY newspost.id, newspost.date, newspost.hilight, newspost.body ORDER BY date DESC LIMIT ? OFFSET ?;"
	let page = max 0 $ ((fromMaybe 1 mpage) - 1) * limit
	execute q [iToSql limit, iToSql page]
	res <- fetchAllRows' q
	return $ case Right [] {- mapM newsParse res -} of
		Right []
			| page > 1 -> Left $ do
				H.p "No news posts this far back."
			| otherwise -> Left $ do
				H.p "No news posts yet."
		Right ps -> Right ps
		-- some kinda db error, but also, not an _actual_ db error? like this would only happen if we have a malformed db schema somehow
		Left f -> Left $ do
			H.p $ "Malformed db schema?" -- >> toHtml (show f)

newsParse :: [SqlValue] -> ConvertResult Newspost
newsParse [id_, date, hilight, html, commentCount] = Newspost
	<$> safeFromSql id_
	<*> fmap (localTimeToUTC $ hoursToTimeZone 0) (safeFromSql date)
	<*> safeFromSql hilight
	<*> fmap (fromMaybe 0) (safeFromSql commentCount) -- will be NULL if no comments
	<*> fmap H.preEscapedToHtml (safeFromSql html :: ConvertResult String)
newsParse f = Left $ ConvertError (show f) "[SqlValue]" "Newspost" "invalid fields"

commentParse :: [SqlValue] -> ConvertResult Comment
commentParse [id_, date, hilight, hidden, author, contents] = Comment
	<$> safeFromSql id_
	<*> fmap (Right . localTimeToUTC (hoursToTimeZone 0)) (safeFromSql date)
	<*> safeFromSql hidden
	<*> safeFromSql hilight
	<*> safeFromSql author
	<*> safeFromSql contents
commentParse f = Left $ ConvertError (show f) "[SqlValue]" "Comment" "invalid fields"
-}
