{-# LANGUAGE DeriveDataTypeable #-}
{-# LANGUAGE TypeOperators #-}
{-# LANGUAGE TemplateHaskell #-}
{-# LANGUAGE OverloadedStrings #-}

module Sites.Index.Site
	( site
	) where

import Prelude hiding ((.))

import System.FilePath.Posix (joinPath)

import Control.Category ((.))
import Control.Monad
import Data.Data (Typeable)
import Data.String (IsString(fromString))

import Happstack.Server (ServerPartT, ServerPart, Response, serveFile, asContentType, toResponse)

-- web-routes-boomerang & boomerang
import Web.Routes (RouteT, runRouteT, setDefault)
import Web.Routes.Happstack (implSite)
import Web.Routes.Boomerang (Router, boomerangSite, lit, (:-))
import Text.Boomerang.TH (makeBoomerangs)

import qualified Text.Blaze.Html5 as H

data SiteRoute = Index | Robots | Fail
	deriving (Eq, Ord, Typeable)

$(makeBoomerangs ''SiteRoute)

sitemap :: Router () (SiteRoute :- ())
sitemap = mconcat
	[ rIndex
	, rRobots . lit "robots.txt"
	, rFail . lit "fail"
	]

route :: String -> SiteRoute -> RouteT SiteRoute (ServerPartT IO) Response
route base url = case url of
	Index -> fmap toResponse $ return $ H.docTypeHtml (return ())
	Robots -> serveFile (asContentType "text/plain")
		$ joinPath [base, "robots.txt"]
	Fail -> error "fail"

site :: String -> String -> Int -> String -> ServerPart Response
site base host port basePath = msum
	[ implSite (fromString siteString) (fromString basePath)
		$ setDefault Index $ boomerangSite (runRouteT $ route base) sitemap
	]
	where
		siteString = if port == 80
			then concat ["http://", host]
			else concat ["http://", host, ":", show port]
