module Utility.Misc
	( mapWithParents
	, collate
	, swapCollate
	, within
	, safeToEnum
	, mapAccumLM
	) where

import Data.List (sortBy, groupBy)
import Data.Tree (Tree (..))
import Data.Function (on)
import Data.Ord (comparing)
import Data.Tuple (swap)

-- should maybe move this into like, Utility.Tree
mapWithParents :: ([a] -> b) -> Tree a -> Tree b
mapWithParents = mapWithParents' []
	where
		mapWithParents' :: [a] -> ([a] -> b) -> Tree a -> Tree b
		mapWithParents' ps f (Node x chs) = Node (f $ x:ps) $ mapWithParents' (x:ps) f <$> chs

collate :: (Ord a) => [(a, b)] -> [(a, [b])]
collate =
	  fmap
		(\x -> case x of
			[] -> error "empty collate group"
			((a, _):_) -> (a, fmap snd x))
	. groupBy ((==) `on` fst)
	. sortBy (comparing fst)

swapCollate :: Ord b => [(a, b)] -> [(b, [a])]
swapCollate = collate . fmap swap

within :: Ord a => a -> a -> a -> Bool
within lo hi v = lo <= v && v <= hi

safeToEnum :: (Enum a, Bounded a) => Int -> Maybe a
safeToEnum = enumIfBetween minBound maxBound
	where
		enumIfBetween :: (Enum a) => a -> a -> Int -> Maybe a
		enumIfBetween a z x =
			let
				a' = fromEnum a
				z' = fromEnum z
			in
				if a' <= x && x <= z'
					then Just $ toEnum x
					else Nothing

mapAccumLM :: Monad m => (a -> b -> m (a, c)) -> a -> [b] -> m (a, [c])
mapAccumLM f = go
	where
		-- go :: a -> [b] -> m (a, [c])
		go zero [] = return (zero, [])
		go zero (x:xs) = do
			(one, c) <- f zero x
			(aleph, cs) <- go one xs
			return (aleph, c:cs)
