"use strict";
var Body = (function () {

	var Nodes = (
		{ UNKNOWN: 0
		, ROOT:    1 // just "animum"
		, LIMB:    2 // limbs: arms, legs, head, tail, wings
		, SHAPE:   3 // shape modifiers: more, less, larger
		, ANIMAL:  4 // species: human, boar, wolf, deer
		, SEX:     5 // genitalia: cock, vagina, breasts, balls
		, PART:    6 // accent parts: horns, haloes, eyes
		, ELEMENT: 7 // classical element: fire, water, metal, night, gem
		, GLOSS:   8 // misc. attributes: glossy, ichorous, veiny, spiny (also i think including color and weapon)
		, DETAIL:  9 // minor, part-specific details: hairstyle, build, feet, hands
		})
		, typeclasses = [];
	typeclasses[Nodes.ROOT] = ["R", "root"];
	typeclasses[Nodes.LIMB] = ["L", "limb"];
	typeclasses[Nodes.PART] = ["P", "part"];
	typeclasses[Nodes.SHAPE] = ["S", "shape"];
	typeclasses[Nodes.SEX] = ["X", "sex"];
	typeclasses[Nodes.ELEMENT] = ["E", "element"];
	typeclasses[Nodes.GLOSS] = ["G", "gloss"];
	typeclasses[Nodes.ANIMAL] = ["A", "animal"];
	typeclasses[Nodes.DETAIL] = ["D", "detail"];
	typeclasses[Nodes.UNKNOWN] = ["?", "unknown"];

	function arrayContains (arr, v) {
		return arr.indexOf (v) !== -1;
	}

	function containsPath (path, body) {
		if (path.length === 0) {
			return true;
		}
		var check = false;
		if (typeof path[0] === "string") { // node check
			check = body.value == path[0];
		} else { // type check
			check = body.type == path[0];
		}
		return check && body.children.map (function (ch) {
			return containsPath (path.slice(1), ch);
		}).reduce (function (a, b) { return a || b; }, false);
	}

	function DetailKind (name, permitted, activeWhen) {
		this.name = name;
		this.permitted = permitted;
		this.active = activeWhen;
		return this;
	}
	DetailKind.prototype =
		{ permits: function (node) {
				return arrayContains (this.permitted, node);
			}
		, typeclass: function () {
				return typeclasses [Nodes.DETAIL][1];
			}
		, title: function () {
				return document.createTextNode (this.name + "\u00bb");
			}
		}

	function NodeKind () {
		this.permitted = [];
		this.forbidden = [];
		this.type = -1;
		Array.prototype.slice.call(arguments).map (function (arg) {
			var name;
			if (typeof arg === "string") {
				if (arg.substr (0,1) === "!") {
					name = arg.substr(1);
					this.forbidden.push (name);
					this.type = typeOf (name);
				} else {
					this.permitted.push (arg);
					this.type = typeOf (arg);
				}
			} else {
				this.type = arg|0;
			}
		}.bind(this));
		return this;
	}
	NodeKind.prototype =
		{ permits: function (node) {
				if (typeof node === "string") {
					return (typeOf (node) === this.type
						&& (arrayContains (this.permitted, node) // explicitly allowed
							|| (this.permitted.length === 0 && this.forbidden.length === 0) // allows all of type
							|| (this.forbidden.length > 0 && !arrayContains (this.forbidden, node)))); // using forbidden values & not explicitly forbidden
				} else {
					return this.type === node;
				}
			}
		, typeclass: function () {
				return typeclasses [this.type][1];
			}
		};

	function availableDetails (body, parents) {
		var pString = parents.map(function (n) {return n.value;}).reduce (function (a, b) {return a + "/" + b; });
		switch (pString) {
			case "animum":
				return (
					[ new DetailKind ("build", ["muscular", "lean", "fat"], function () {
						return true; })
					, new DetailKind ("body hair", ["hairless", "hairy"], function () {
						return true; })
					]);
			case "animum/head":
				return (
					[ new DetailKind ("hairstyle", ["shaggy", "topknot", "dreadlocks"]
						, function (b) {
							return containsPath (["animum", "head", "human"]) ||
								(containsPath (["animum", "human"]) && !containsPath (["animum", "head", Nodes.ANIMAL]));
							})
					, new DetailKind ("facial hair", ["stubble", "goatee"], function () {
							return true; })
					]);
			default:
				return [];
		}
	}

	function permittedKids (parents) {
		var pString = parents.map(function (n) {return n.value;}).reduce (function (a, b) {return a + "/" + b; });
		//console.log (pString);
		switch (pString) {
			case "animum":
				return [ {n: new NodeKind (Nodes.LIMB), limit: 5}
					, {n: new NodeKind (Nodes.PART), limit: 5}
					, {n: new NodeKind ("!snakehead"), limit: 1}
					, {n: new NodeKind ("cock", "balls", "breasts", "flower", "bellymouth", "mouth"), limit: 5}
					, {n: new NodeKind ("large", "largex2", "largex3", "small"), limit: 1}
					, {n: new NodeKind (Nodes.ELEMENT), limit: 1}
					, {n: new NodeKind (Nodes.GLOSS), limit: 1}
					];
			case "animum/legs":
				return [ {n: new NodeKind ("more", "morex3", "morex4", "asymmetrical"), limit: 2}
					, {n: new NodeKind (Nodes.ANIMAL), limit: 1}
					];
			case "animum/arms":
				return [ {n: new NodeKind ("more", "morex3", "morex4", "asymmetrical"), limit: 2}
					, {n: new NodeKind (Nodes.ANIMAL), limit: 1}
					];
			case "animum/head":
				return [ {n: new NodeKind ("horns", "halo", "eyes"), limit: 3}
					, {n: new NodeKind ("more", "morex3", "morex4", "asymmetrical"), limit: 1}
					, {n: new NodeKind (Nodes.ANIMAL), limit: 1}
					, {n: new NodeKind (Nodes.ELEMENT), limit: 1}
					, {n: new NodeKind ("mouth"), limit: 1}
					];
			case "animum/head/mouth":
				return [ {n: new NodeKind ("more", "morex3", "morex4", "asymmetrical"), limit: 1}
					, {n: new NodeKind ("tongue", "cock"), limit: 1}
					, {n: new NodeKind ("teeth"), limit: 1}
					, {n: new NodeKind (Nodes.ELEMENT), limit: 1}
					];
			case "animum/head/mouth/tongue":
				return [ {n: new NodeKind (Nodes.SHAPE), limit: 3}
					, {n: new NodeKind (Nodes.ELEMENT), limit: 1}
					];
			case "animum/head/mouth/teeth":
				return [ {n: new NodeKind (Nodes.SHAPE), limit: 3}
					, {n: new NodeKind (Nodes.GLOSS), limit: 1}
					];
			case "animum/head/horns":
				return [ {n: new NodeKind (Nodes.SHAPE), limit: 3}
					, {n: new NodeKind (Nodes.ELEMENT), limit: 1}
					, {n: new NodeKind ("goat", "bull", "unicorn", "deer"), limit: 1}
					];
			case "animum/head/halo":
				return [ {n: new NodeKind (Nodes.ELEMENT), limit: 1}
					];
			case "animum/tail":
				return [ {n: new NodeKind ("!human"), limit: 1}
					, {n: new NodeKind ("cock"), limit: 1}
					];
			case "animum/wings":
				return [ {n: new NodeKind (Nodes.SHAPE), limit: 3}
					, {n: new NodeKind ("dragon", "chicken", "bat"), limit: 1}
					];
			case "animum/head/mouth/cock":
			case "animum/tail/cock":
			case "animum/cock":
				return [ {n: new NodeKind ("knot"), limit: 1}
					, {n: new NodeKind (Nodes.SHAPE), limit: 3}
					, {n: new NodeKind (Nodes.ANIMAL), limit: 1}
					, {n: new NodeKind (Nodes.ELEMENT), limit: 1}
					, {n: new NodeKind (Nodes.GLOSS), limit: 1}
					];
			case "animum/head/mouth/cock/knot":
			case "animum/tail/cock/knot":
			case "animum/cock/knot":
				return [ {n: new NodeKind (Nodes.SHAPE), limit: 3}
					, {n: new NodeKind (Nodes.GLOSS), limit: 1}
					];
			case "animum/balls":
				return [ {n: new NodeKind (Nodes.SHAPE), limit: 3}
					, {n: new NodeKind (Nodes.ANIMAL), limit: 1}
					, {n: new NodeKind (Nodes.ELEMENT), limit: 1}
					, {n: new NodeKind (Nodes.GLOSS), limit: 1}
					];
			default:
				return [];
		}
	}

	function subname (value) {
		switch (value) {
			case "morex3":
				return "x3";
			case "morex4":
				return "x4";
			case "largex2":
				return "x2";
			case "largex3":
				return "x3";
			case "snakehead":
				return "head";
			case "bellymouth":
				return "belly";

			case "piercing-chain":
				return "chain";
			case "piercing-tag":
				return "tag";
			case "piercing-hook":
				return "hook";
			case "piercing-stud":
				return "stud";
			case "piercing-ring":
				return "ring";
			case "piercing-cross":
				return "cross";
			case "piercing-padlock":
				return "padlock";
			default:
				return value;
		}
	}
	function superclass (value) {
		switch (value) {
			case "sun":
			case "moon":
			case "stars":
			case "void":
				return "stellar";
			case "sword":
			case "spear":
			case "bow":
			case "chain":
				return "weapon";
			case "ash":
			case "magma":
				return "fire";
			case "thorn":
				return "plant";
			case "more":
			case "morex3":
			case "morex4":
			case "less":
				return "more";
			case "large":
			case "largex2":
			case "largex3":
			case "small":
				return "large";
			case "snakehead":
				return "snake";
			//case "mouth": // mouth not being classed to 'mouth' is what keeps mouth and bellymouth from overlapping. this is kind of a special case, but like. crotch mouth and belly mouth at once should be options.
			case "bellymouth":
				return "mouth";
			case "ichor":
			case "infested":
				return "ichor";

			case "piercing-chain":
			case "piercing-tag":
			case "piercing-hook":
			case "piercing-stud":
			case "piercing-ring":
			case "piercing-cross":
			case "piercing-padlock":
				return "piercing";
			default:
				return null;
		}
	}
	function typeOf (value) {
		switch (value) {
			case "animum":
			case Nodes.ROOT:
				return Nodes.ROOT;
			case "arms":
			case "legs":
			case "head":
			case "tail":
			case "wings":
			case Nodes.LIMB:
				return Nodes.LIMB;
			case "horns":
			case "halo":
			case "eyes":
			case "tentacle":
			case "teeth":
			case Nodes.PART:
				return Nodes.PART;
			case "more":
			case "morex3":
			case "morex4":
			case "less":
			case "large":
			case "largex2":
			case "largex3":
			case "small":
			case "asymmetrical":
			case Nodes.SHAPE:
				return Nodes.SHAPE;
			case "cock":
			case "knot":
			case "balls":
			case "flower":
			case "breasts":
			case "mouth":
			case "bellymouth":
			case "tongue":
			case Nodes.SEX:
				return Nodes.SEX;
			case "fire":
			case "ash":
			case "magma":

			case "metal":

			case "ichor":
			case "infested":

			case "ice":
			case "storm":
			case "gem":
			case "metal":
			case "static":
			case "poison":
			case "bone":

			case "plant":
			case "thorn":

			case "stellar":
			case "sun":
			case "moon":
			case "stars":
			case "void":

			case Nodes.ELEMENT:
				return Nodes.ELEMENT;
			case "shine":
			case "grace":
			case "spines":
			case "veins":

			case "sword":
			case "spear":
			case "bow":
			case "chain":

			case Nodes.GLOSS:
				return Nodes.GLOSS;
			case "wolf":
			case "deer":
			case "goat":
			case "bull":
			case "lion":
			case "tiger":
			case "dragon":
			case "unicorn":
			case "scorpion":
			case "chicken":
			case "bat":
			case "boar":
			case "human":
			case "fish":

			case "snake":
			case "snakehead":

			case "octopus":
			case "mantis":
			case Nodes.ANIMAL:
				return Nodes.ANIMAL;
			default:
				return Nodes.UNKNOWN;
		}
	}

	// this could i think get rid of the parents req, if we made sure to call `rebuildAllowedChildren` (via addChild or changeValue) when attaching nodes.
	function Node (root, parents) {
		var chsAllowed, this_;
		if (parents) {
			parents.push ({value: root});
			chsAllowed = permittedKids (parents);
			chsAllowed.sort (function (a, b) {
				return a.n.type - b.n.type;
			});
		} else {
			chsAllowed = [];
		}
		this.parent = null;
		this.value = root;
		this.type = typeOf (root);
		this.permitted = null;
		this.level = 1;
		this_ = this;
		this.children = [];
		this.childrenTemplate = chsAllowed;
		// manually add one open element slot to [ANIMUM]
		if (root === "animum") {
			var n = new Node ("", []);
			n.type = Nodes.ELEMENT;
			n.permitted = this.childrenTemplate.filter (function (c) {
				return c.n.type === Nodes.ELEMENT;
			})[0].n;
			this.addChild (n);

			availableDetails (this, [this]).map (function (d) {
				var dNode = new Node ("", []);
				dNode.type = Nodes.DETAIL;
				dNode.permitted = d;
				this.addChild(dNode);
			}.bind(this));
		}
		/*
		_.map (chsAllowed, function (c) {
			var n = new Node ("", []);
			n.type = c.type;
			n.permitted = c;
			this_.addChild (n);
		});
		*/
	}
	Node.prototype =
		{ addChild: function (child) {
				child.parent = this;
				this.children.push (child);

				child.rebuildAllowedChildren ();
			}
		, replaceChild: function (childValue, newValue) {
			var oldNode;
			var chs = this.children.filter (function (c) {
				return c.value === childValue;
			});
			if (chs.length <= 0) {
				return null;
			}
			oldNode = new Node (chs[0].value, []); // we really just need to know 1. if we did replace a node and 2. what that node's name is, rather than actually needing to know e.g., if the old node had children.
			chs[0].changeValue (new Node (newValue, []));
			return oldNode;
		}
		, attach: function (child) {
				// look for a blank child that can accept the arg value. returns true if the body tree was updated w/ the new child and false otherwise.
				var i = 0;
				var c = null;
				for (i = 0; i < this.children.length; i++) {
					c = this.children[i];
					if (c.value === "" && c.permits (child.value)) {
						c.changeValue (child);
						return true;
					}
				}
				return false;
			}
		, rebuildAllowedChildren: function () {
			// explicitly regenerate the permitted children list
			var parents = [];
			var o = this;
			var root_;
			while (o) {
				root_ = o;
				parents.push (o);
				o = o.parent;
			}
			parents.reverse();
			//console.log (parents);
			//console.log (permittedKids (parents));
			this.childrenTemplate = permittedKids (parents);
			availableDetails (root_, parents).map (function (d) {
				var dNode = new Node ("", []);
				dNode.type = Nodes.DETAIL;
				dNode.permitted = d;
				this.addChild(dNode);
			}.bind(this));
		}
		// is the given node type allowed to replace this node
		, permits: function (p) {
			return this.permitted && this.permitted.permits(p);
		}
		// is the potential sibling node `v` permitted to exist in addition to this
		// node (e.g., does it overlap as in the case of [ARMS] / [ARMS] or
		// [COLOR]foo / [COLOR]bar
		, overlaps: function (v) {
				return this.value === v || (superclass (this.value) !== null && superclass (this.value) === superclass (v));
			}
		, changeValue: function (n) {
				var this_ = this;
				this.value = n.value;
				this.level = n.level;
				this.children = n.children;
				// lord do i hate javascript. objects are pointers until they aren't
				// (for reference what happens w/o this line is that the parents can drift out of date b/c somewhere in this maze of code i'm apparently copying a part object rather than modifying it, and that remains there as the old parent. and, for reasons still unknown to me, the parent ends up with a null parent value itself, even though that... shouldn't happen.)
				_.map (this.children, function (c) { c.parent = this_; });

				this.rebuildAllowedChildren ();
			}
		, clear: function () {
				this.value = "";
				this.level = 1;
				this.children = [];
			}
		// strip down the node data to only the information needed for server communication
		, asNecessary: function () {
			function f (n) {
				return (
					{ value: n.value
					, children: _.filter (_.map (n.children, f), function (c) { return c.value !== ""; })
					});
			}
			return f (this);
		}
		// savedata needs to also save open slots (aka slots with a value of ""), since those need to be restored on load
		, asSavedata: function () {
				function f (n) {
					var r = (
						{ value: n.value
						});
					if (n.value === "") {
						r.type = n.type;
					}
					r.children = n.children.map (f);
					return r;
				}
				return f (this);
			}
		, followOrAddPath (path) {
			var addedNode;
			if (path.length === 0) {
				return this;
			}
			console.log ("followOrAddPath: " + this.value + " got " + path.reduce (function (a, b) {return a + "," + b;}));
			var chs = this.children.filter (function (c) {
				return c.value === path[0];
			});
			if (chs.length === 0) {
				addedNode = new Node (path[0], []);
				this.addChild (addedNode);
				return addedNode.followPath (path.slice (1));
			}
			return chs[0].followPath (path.slice (1));
		}
		, followPath (path) {
			if (path.length === 0) {
				return this;
			}
			console.log ("followPath: " + this.value + " got " + path.reduce (function (a, b) {return a + "," + b;}));
			var chs = this.children.filter (function (c) {
				return c.value === path[0];
			});
			if (chs.length === 0) {
				return null;
			}
			return chs[0].followPath (path.slice (1));
		}
		, atMaxChildren: function () {
				var maxKids = this.childrenTemplate.reduce (function (acc, ch) {
					return acc + ch.limit;
				}, 0);
				var kids = this.children.length;
				return kids >= maxKids;
			}
		, typeclass: function () {
				return typeclasses [this.type][1];
			}
		, names: function () {
				var supername, sub, fullname;
				supername = superclass (this.value);
				sub = subname (this.value);
				if (supername !== null && supername !== this.value) {
					fullname = { name: supername.toUpperCase() , sub: sub};
				} else if (this.value !== "") {
					fullname = { name: this.value.toUpperCase() , sub: null};
				} else {
					fullname = { name: null , sub: null };
				}
				return fullname;
			}
		, nameVal: function () {
				var supername, sub, fullname;
				supername = superclass (this.value);
				sub = subname (this.value);
				if (supername !== null && supername !== this.value) {
					fullname = "[" + supername.toUpperCase() + "]\u00bb" + sub;
				} else if (this.value !== "") {
					fullname = "[" + this.value.toUpperCase() + "]";
				} else {
					fullname = null;
				}
				return fullname;
			}
		, asButton: function (callback, inCall, outCall) {
			var
				  button = document.createElement ("button")
				, this_ = this
				;
			button.classList.add ("bodypart");
			button.classList.add (typeclasses[this.type][1]);
			button.appendChild (this.asStatic (false));
			button.dataset.node = this.value;
			button.dataset.type = this.type;
			if (callback) {
				button.addEventListener ('click', _.partial (callback, this_), false);
			}
			if (inCall) {
				button.addEventListener ('mouseover', _.partial (inCall, this_), false);
			}
			if (outCall) {
				button.addEventListener ('mouseout', _.partial (outCall, this_), false);
			}
			return button;
		}
		, asFieldset: function (callback, additional, expandFunc) {
			var fieldset = document.createElement ("fieldset"), filled, blank;
			filled = document.createElement ("fieldset");
			filled.classList.add ("filled");
			blank = document.createElement ("fieldset");
			blank.classList.add ("blank");
			_.map (this.children, function (n) {
				if (n.root === "") {
					blank.appendChild (n.asButton (callback, additional, expandFunc));
				} else {
					filled.appendChild (n.asButton (callback, additional, expandFunc));
				}
				fieldset.appendChild (n.asButton(callback, additional, expandFunc));
			});
			return fieldset;
		}
		, asTree: function () {
			var c, v;
			v  = document.createElement ("li");
			v.appendChild (this.asStatic(false));
			if (this.children.length > 0) {
				c = document.createElement ("ul");
				_.map (this.children, function (ch) {
					if (ch.value !== "") {
						c.appendChild (ch.asTree ());
					}
				});
				if (c.firstChild) {
					v.appendChild (c);
				}
			}
			return v;
		}
		, asStatic: function (drawChildren) {
			var c, v, name, lvl;
			v = document.createElement ("span");
			v.classList.add ("node");
			v.classList.add (typeclasses[this.type][1]);
			if (this.value === "") {
				v.classList.add ("blank");
			} else {
				try {
					v.classList.add (this.value);
				} catch (e) {
					console.log ("bad node value: " + this.value);
					throw e;
				}
				name = this.names();
				v.appendChild (spanLabel ("br", "["));
				v.appendChild (spanLabel ("t", name.name));
				v.appendChild (spanLabel ("br", "]"));
				if (name.sub) {
					v.appendChild (document.createTextNode ("\u00bb"));
					v.appendChild (spanLabel ("sub", name.sub));
				}
			}
			if (this.children.length > 0 && drawChildren) {
				c = document.createElement ("div");
				c.appendChild (v);
				_.map (this.children, function (d) {
					c.appendChild (d.asStatic(drawChildren));
				});
				return c;
			}
			return v;
		}
	};

	return (
		{ Part: Node
		, typeOf: typeOf
		, NodeKind: NodeKind
		});

}());
