
// buckets are containers for equivalent (whatever that means) values. a bucket is created with a certain equality predicate, and if any item in a bucket matches, then the new item can fit in the existing bucket.
function Bucket (pred) {
	this.pred = pred;
	this.in = [];
}
Bucket.prototype =
	{ fits: function (obj) {
			for (var i = 0; i < this.in.length; i++) {
				if (this.pred(this.in[i], obj)) {
					return true;
				}
			}
			return false;
		}
	, add: function (obj) {
			this.in.push (obj);
		}
	, merge: function (buckets) {
			var b = new Bucket(this.pred);
			this.in.map (function (o) {
				b.add (o);
			});
			buckets.map (function (bucket) {
				bucket.in.map (function (o) {
					b.add (o);
				});
			});
			return b;
		}
	, contents: function () {
			return this.in;
		}
	};

// given an equality predicate and a bunch of values, return buckets of distinct groups of values that are not transitively equal. (e.g., a set of distinct color gradients; a set of non-overlapping landscapes; etc)
function bucket (pred, vals) {
	return vals.reduce (function (buckets, val) {
		var part = partition (function (b) {
			return b.fits (val);
		}, buckets);
		var fit = part.yes;
		var rest = part.no;
		var b = null;
		switch (fit.length) {
			case 0: // value does not fit into any existing bucket
				b = new Bucket(pred);
				b.add (val);
				break;
			case 1: // value fits in exactly one existing bucket
				fit[0].add (val);
				break;
			default: // value fits in more than one existing bucket, and by transitivity, all matching buckets must now be merged
				b = fit[0].merge (fit.slice(1));
				b.add (val);
				buckets = rest;
				break;
		}
		if (b) {
			buckets.push (b);
		}
		return buckets;
	}, []);
}
