var Challenge = (function(){
	"use strict";

	var frame = null;
	function setFrame (f) {
		frame = f;
	}

	function blocSlug (n) {
		switch (n) {
			case "hit":
				return "hit!";
			case "search":
				return "search!";
			case "cast":
				return "cast!";
			default:
				return n;
		}
	}

	function unknownAct () {
		var w = document.createElement ("wrap");
		var a = document.createElement ("span");
		w.classList.add ("wrap");
		a.appendChild (document.createTextNode ("??"));
		a.classList.add ("unknown");
		w.appendChild (a);
		return w;
	}

	var stances = new Enum ("NORMAL", "CHARGE_1", "CHARGE_2", "CHARGE_3", "CHARGE_FULL", "DISRUPTED");

	function Bloc (acts) {
		this.chargeState = stances.NORMAL();
		this.acts = [];
		this.actsStrings = [];
		this.length = 0;
		if (acts !== undefined && acts.length > 0) {
			this.addActions (acts);
		}
		return this;
	}
	Bloc.prototype =
		{ addAction: function (next) {
				// are we currently disrupted?
				if (this.chargeState === stances.DISRUPTED()) {
					if (next === "wait" || next === "block") {
						this.chargeState = stances.NORMAL();
					} else {
						this.acts.push ({s: this.chargeState, a: next});
						this.actsStrings.push (next);
						this.length++;
						return;
					}
				}
				// how is the charge gauge?
				if (next === "charge") {
					if (this.chargeState >= 0 && this.chargeState < 4) { this.chargeState += 1;}
				} else if (next === "wait" || next === "block") {
					this.chargeState = 0;
				} else {
					if (this.chargeState > 0 && this.chargeState < 4) { this.chargeState += 1;}
				}
				// making the action
				this.acts.push ({s: this.chargeState, a: next});
				this.actsStrings.push (next);
				this.length++;
				// did we release charge (FIXME: we should be checking if the current state is a charge state, first)
				if (next !== "charge") { this.chargeState = stances.NORMAL(); }
			}
		, addActions: function (acts) {
				_.reduce (acts, function (b_, next) {
					b_.addAction (next);
					return b_;
				}, this);
			}
		, getDisruptedFrom: function (i) {
				while (i < this.acts.length) {
					if (this.acts[i].a === "wait" || this.acts[i].a === "block") {
						return;
					}
					this.acts[i].s = stances.DISRUPTED();
					i++;
				}
				// if we hit the end of the reel without returning, we're still disrupted up to the present moment, so new actions would continue to be disrupted
				this.chargeState = stances.DISRUPTED();
			}
		, trimFront: function () {
				// remove the first item and forget it
				this.acts.shift();
				this.actsStrings.shift();
				this.length--;
			}
		, printActsRaw: function () {
				var as = [];
				var a;
				var w;
				var i = 0;
				for (i = 0; i < this.acts.length; i++) {
					w = document.createElement ("span");
					w.classList.add ("wrap");
					a = document.createElement ("span");
					a.appendChild (document.createTextNode (blocSlug (this.acts[i].a)));
					if (this.acts[i].a === "??") { a.classList.add ("unknown"); }
					else { a.classList.add (this.acts[i].a); }
					switch (this.acts[i].s) {
						case 1:
							a.classList.add ("c1");
							break;
						case 2:
							a.classList.add ("c2");
							break;
						case 3:
							a.classList.add ("c3");
							break;
						case 4:
							a.classList.add ("cfull");
							break;
						case 5: // disrupted
							a.classList.add ("disrupted");
							break;
						default:
							break;
					}
					w.appendChild (a);
					as.push (w);
				}
				return as;
			}
		, print: function () {
				var as = this.printActsRaw();
				var b = document.createElement ("div");
				var i = 0;
				b.classList.add ("bloc");
				for (i = 0; i < as.length; i++) {
					b.appendChild (as[i]);
				}
				return b;
			}
		, printInput: function (encounter, props) {
				var op = this.printActsRaw();
				var this_ = this;
				var button = document.createElement ("button");
				button.classList.add ("bloc");
				_.map (op, function (a) { button.appendChild (a); });
				button.addEventListener ("click", function () {
					encounter.commitPcActions (this_.actsStrings, props);
				});
				return button;
			}
		};

	function Action (props, acts) {
		this.props = props;
		this.acts = acts;
		this.actBloc = new Bloc (acts);

		return this;
	}
	Action.prototype =
		{ print: function (encounter) {
				var li = document.createElement ("li");
				var label = document.createElement ("label");
				var button = this.actBloc.printInput (encounter, this.props);
				button.id = "act-" + this.props.name;
				label.appendChild (document.createTextNode (this.props.name));
				label.htmlFor = "act-" + this.props.name;
				li.appendChild (label);
				li.appendChild (button);
				return li;
			}
		};

	// change @display@ to show @value@, in a scroll (assuming 'display''s text is numeric and 'value' is a number)
	function alterValueUnsafe (display, value, callback) {
		var cur = display.textContent|0; // $(display).html(),
		var set = cur;
		var increment = cur < value ? 1 : -1;
		var delay = 750 / Math.abs(cur - value);
		while (delay < 25) {
			delay *= 2;
			increment *= 2;
		}

		function repeat () {
			if ((increment < 0 && set + increment < value)
				|| (increment > 0 && set + increment > value)) {
				cur = value|0;
				set = value|0;
				setText (display, cur); //$(display).html(cur);
			} else {
				cur += increment|0;
				set += increment|0;
				setText (display, cur); //$(display).html(cur);
			}
			if (callback) { callback (display, cur); }
			if (set != value) {
				setTimeout (repeat, delay);
			}
		}
		repeat ();
	}

	function hpDisplayUpdate (max, dom, val) {
		function healthPercent (cur, max) {
			var percent = cur / max;
			if (cur === max) { return "perfect"; }
			if (percent >= 0.75) { return "reasonable"; }
			if (percent >= 0.50) { return "caution"; }
			if (percent >= 0.25) { return "danger"; }
			if (cur === 0) { return "end"; }
			return "danger";
		}
		var c = healthPercent (val|0, max|0);
		dom.classList.remove ("perfect");
		dom.classList.remove ("reasonable");
		dom.classList.remove ("caution");
		dom.classList.remove ("danger");
		dom.classList.remove ("end");
		dom.classList.add (c);
	}

	function resolve (self, target, s, t) {
		var targetDamage = 0;
		var selfDamage = 0;
		var disrupted = 0;

		// that final "1" would maybe be like (1 + {skill level} * 0.15), so higher level skills have some kinda damage increase
		var basePhysDamage = Math.max (1, (s.str / t.def) * 3 * 1);
		var baseMagDamage = Math.max (1, (s.int_ / t.res) * 3 * 1);
		console.log ("basePhysDamage: " + basePhysDamage + "; baseMagDamage: " + baseMagDamage);

		var hasBlock = target.a === "block" ? 0.5 : 1;
		var chargeMultipliers = [1, 3, 6, 9, 12];
		var charge = (self.s > 0 && self.s <= 4) ? chargeMultipliers[self.s-1] : 1;

		// q: are these actions actually gonna be magically mutated elsewhere due to spooky javascript object action at a distance? and followup q: if they are, is that going to negatively impact anything?
		if (self.s === stances.DISRUPTED()) { self.a = "wait"; }
		if (target.s === stances.DISRUPTED()) {target.a = "wait"; }

		if (self.a === "hit" || self.a === "cast") {
			targetDamage = self.a === "hit" ? basePhysDamage : baseMagDamage;
			targetDamage = (targetDamage * charge) * hasBlock;
			if (target.a === "charge") {
				disrupted = 1;
			}
			if (target.a === "parry" && self.a === "hit") {
				selfDamage = targetDamage * 0.66;
				targetDamage = targetDamage * 0.34;
			} else if (target.a === "reflect" && self.a === "cast") {
				selfDamage = targetDamage * 0.66;
				targetDamage = targetDamage * 0.34;
			}
			if (target.a === "dazed") {
				targetDamage = targetDamage * 2;
			}
		}
		// eat shit parry-haver etc
		if (self.a === "disarm" && target.a === "parry") {
			targetDamage = (basePhysDamage * charge);
		}
		// heal shouldn't involve the target's res (so it's not just magic damage) but it probably shouldn't just be "int"
		if (self.a === "heal") {
			selfDamage = (s.int_ * 2) * charge * -1;
		}
		targetDamage = Math.floor (targetDamage);
		selfDamage = Math.ceil (selfDamage);
		return (
			{ targetDamage: targetDamage
			, targetDisrupted: disrupted
			, selfDamage: selfDamage
			});
	}

	function matchup (oppo, pc, oppoStats, pcStats) {
		var r1 = resolve (oppo, pc, oppoStats, pcStats);
		var r2 = resolve (pc, oppo, pcStats, oppoStats);

		var totalOppo = r1.selfDamage + r2.targetDamage;
		var totalPc = r1.targetDamage + r2.selfDamage;
		var oppoDisrupted = r2.targetDisrupted;
		var pcDisrupted = r1.targetDisrupted;

		return (
			{ pcDam: totalPc
			, pcDamType:
				pc.a === "block"   ? "blocked" :
				totalPc < 0        ? "healed" :
				oppo.a === "parry" ? "parried" :
				null
			, pcDis: pcDisrupted
			, opDam: totalOppo
			, opDamType:
				oppo.a === "block" ? "blocked" :
				totalOppo < 0      ? "healed" :
				pc.a === "parry"   ? "parried" :
				null
			, opDis: oppoDisrupted
			});
	}

	function turnResult (dam, damtype) {
		var r = document.createElement("span");
		r.classList.add ("r");
		if (dam === 0) { return r; }
		r.classList.add ("damage");
		if (damtype) { r.classList.add (damtype); }
		r.appendChild (document.createTextNode (dam));
		return r;
	}

	function setClassUniquely (kids, final_, className) {
		var i;
		for (i = 0; i < final_; i++) {
			kids[i].classList.remove (className);
		}
		kids[final_].classList.add (className);
	}


	function Encounter () {
		this.opponentActs = new Bloc();
		this.queued = 0;
		this.opponentAI = null;
		this.pcActs = new Bloc();
		this.lastPcActionLength = 0;
		this.pcActionsAvailable = [];
		this.pcActionsTaken = {};
		this.now = 0;
		this.turnsForgotten = 0;
		this.running = 1;
		this.pcStats = {};
		this.opponentStats = {};
		this.opponentStats.hp = {cur: 0, max: 0};
		this.opponentGutter = frame.querySelector ("#monstro_gutter .slide");
		this.pcGutter = frame.querySelector ("#pc_gutter .slide");

		this.victory = null;
		this.failure = null;

		this.initialize();
		return this;
	}
	Encounter.prototype =
		{ initialize: function () {
				// clear any old HTML state
				clearDOM (this.opponentGutter);
				clearDOM (this.pcGutter);

				// activate input buttons
				var inputs = frame.querySelectorAll ("#action_list button");
				_.map (inputs, function (i) { i.disabled = false; });

				// redisplay the ui, if applicable
				var ui = frame.querySelector ("#uiPane");
				ui.style.display = "block";
			}
		, begin: function () {
				var this_ = this;
				var actionList = frame.querySelector ("ul#action_list");
				clearDOM (actionList);
				_.map (this.pcActionsAvailable, function (a) { actionList.appendChild (a.print (this_)); });
			}
		, queueOpponentActions: function (acts) {
				var cur = this.opponentActs.length;
				var newActs;
				var visible;
				var i = 0;
				this.opponentActs.addActions (acts);
				newActs = this.opponentActs.printActsRaw().slice(cur);
				visible = cur;
				for (i = 0; i < newActs.length; i++) {
					if (visible <= 0) {
						this.opponentGutter.appendChild (newActs[i]);
					} else {
						this.opponentGutter.appendChild (unknownAct());
					}
					visible++;
					this.queued++;
				}
			}
		, setOpponent: function (opponent) {
				var name = frame.querySelector ("#monstro_stats .name");
				var hp = frame.querySelector ("#monstro_stats .hp");
				console.log (opponent);
				clearDOM (name);
				name.appendChild (document.createTextNode (opponent.name));
				this.opponentAI = opponent.ai;
				this.opponentStats = opponent.stats;
				clearDOM (hp);
				hp.appendChild (document.createTextNode (opponent.stats.hp.cur));

				hpDisplayUpdate (this.opponentStats.hp.max, hp, this.opponentStats.hp.cur);
				this.fillQueue();
			}
		, setPcActions: function (acts) {
				this.pcActionsAvailable = acts;
			}
		, setOwnStats: function (name_, stats) {
				var name = frame.querySelector ("#pc_stats .name");
				var hp = frame.querySelector ("#pc_stats .hp");
				this.pcStats = stats;

				clearDOM (name);
				name.appendChild (document.createTextNode (name_));

				clearDOM (hp);
				hp.appendChild (document.createTextNode (this.pcStats.hp.cur));
				hpDisplayUpdate (this.pcStats.hp.max, hp, this.pcStats.hp.cur);
			}
		, end: function () {
				var inputs;
				var this_ = this;
				this.running = 0;

				inputs = frame.querySelectorAll ("#action_list button");
				_.map (inputs, function (i) { i.disabled = true; });

				setTimeout (function () {
					// or just wipe everything
					this_.resultScreen();
				}, 2750);
			}
		, damagePC: function (dam) {
				// don't do more damage then there's hp remaining / don't heal more than max health
				var hp = Math.min (Math.max (0, this.pcStats.hp.cur - dam), this.pcStats.hp.max);
				alterValueUnsafe (frame.querySelector("#pc_stats .hp"), hp, _.partial (hpDisplayUpdate, this.pcStats.hp.max));
				this.pcStats.hp.cur = hp;
				if (this.pcStats.hp.cur <= 0) {
					this.end ();
				}
				return (this.pcStats.hp.cur <= 0);
			}
		, damageOpponent: function (dam, cancelWin) {
				// see above
				var hp = Math.min (Math.max (0, this.opponentStats.hp.cur - dam), this.opponentStats.hp.max);
				alterValueUnsafe (frame.querySelector("#monstro_stats .hp"), hp, _.partial (hpDisplayUpdate, this.opponentStats.hp.max));
				this.opponentStats.hp.cur = hp;
				if (this.opponentStats.hp.cur <= 0 && !cancelWin) {
					this.end ();
				}
			}
		, fillQueue: function () {
				var op;
				while (this.queued < 7) {
					op = this.opponentAI();
					this.queueOpponentActions (op);
				}
			}
		, commitPcActions: function (acts, actprops) {
				var cur = this.pcActs.length;
				var this_ = this;
				this.lastPcActionLength = acts.length;
				this.pcActs.addActions (acts);
				// record what actions the pc took, for tracking/xp purposes
				if (this.pcActionsTaken[actprops.name] !== undefined) {
					this.pcActionsTaken[actprops.name] += 1;
				} else {
					this.pcActionsTaken[actprops.name] = 1;
				}
				var newActs = this.pcActs.printActsRaw().slice(cur);
				var i = 0;
				/* add all the new actions even before running, because if the action state changes (e.g., gets disrupted) there are ways to change the bloc so they will generate differently and there are ways to change the currently-existing DOM objects, but there's no way to change the arguments of `newActs` which are abruptly out of date.
				*/
				for (i = 0; i < newActs.length; i++) {
					this.pcGutter.appendChild (newActs[i]);
				}
				for (i = 0; this.running && i < newActs.length; i++) {
					this.runNow();
				}
				setClassUniquely (this.pcGutter.childNodes, this.pcActs.length-1, "now");
				setClassUniquely (this.opponentGutter.childNodes, this.pcActs.length-1, "now");
				setTimeout (function () {this_.forgetPast();}, 500);
			}
		, runNow: function () {
				// run the current matchup
				var i = this.now - this.turnsForgotten;
				var turnAction;
				var j;
				// see above spooky object action at a distance; we'd maybe want to copy these rather than use them directly. maybe?
				turnAction = matchup (this.opponentActs.acts[i], this.pcActs.acts[i], this.opponentStats, this.pcStats);

				this.opponentGutter.childNodes[i].appendChild (turnResult (turnAction.opDam, turnAction.opDamType));
				this.pcGutter.childNodes[i].appendChild (turnResult (turnAction.pcDam, turnAction.pcDamType));

				if (turnAction.pcDis) {
					this.pcActs.getDisruptedFrom(i);
					for (j = i; j < this.pcGutter.childNodes.length; j++) {
						this.pcGutter.childNodes[j].firstChild.classList.add ("disrupted");
					}
				}
				if (turnAction.opDis) {
					this.opponentActs.getDisruptedFrom(i);
					for (j = i; j < this.opponentGutter.childNodes.length; j++) {
						// don't strikeout the ?? blocks
						if (this.opponentGutter.childNodes[j].firstChild.classList.contains ("unknown")) {
							break;
						}
						this.opponentGutter.childNodes[j].firstChild.classList.add ("disrupted");
					}
				}
				var playerDead = 0;
				if (turnAction.pcDam !== 0) { playerDead = this.damagePC (turnAction.pcDam); }
				if (turnAction.opDam !== 0) { this.damageOpponent (turnAction.opDam, playerDead); }
				// advance the 'now' index
				this.now++;
				this.queued--;
				// reveal one more opponent action
				var cindex = this.now - this.turnsForgotten;
				var next = this.opponentActs.printActsRaw().slice(cindex, cindex+1);
				var flip = this.opponentGutter.childNodes[cindex];
				flip.parentNode.insertBefore (next[0], flip);
				flip.parentNode.removeChild (flip);
				// fill the action buffer back up / clear history if necessary
				this.fillQueue();
			}
		, forgetPast: function () {
				// 1. slide the gutters to the right(?) by however many units we're gonna delete * 5.6em over the course of a half-second
				// 2. cut off the old units and reset the positioning to 0 immediately
				var toRemove = Math.max (0, (this.now - this.turnsForgotten) - (this.lastPcActionLength + 1));
				var length = (-5 * toRemove) + "em";
				var this_ = this;
				this.opponentGutter.classList.add ("sliding");
				this.opponentGutter.style.left = length;
				this.pcGutter.classList.add ("sliding");
				this.pcGutter.style.left = length;
				setTimeout (function () {
					this_.pcGutter.classList.remove ("sliding");
					this_.pcGutter.style.left = "0";
					this_.opponentGutter.classList.remove ("sliding");
					this_.opponentGutter.style.left = "0";
					while (this_.now - this_.turnsForgotten > this_.lastPcActionLength + 1) {
						this_.forget();
					}
				}, 500);
			}
		, forget: function () {
				this.turnsForgotten++;
				this.pcGutter.removeChild (this.pcGutter.firstChild);
				this.opponentGutter.removeChild (this.opponentGutter.firstChild);
				this.pcActs.trimFront();
				this.opponentActs.trimFront();
			}
		, victoryCallback: function (f) {
				this.victory = f;
			}
		, failureCallback: function (f) {
				this.failure = f;
			}
		, resultScreen: function () {
				var challengeNode = frame;
				var ui = frame.querySelector ("#uiPane");
				ui.style.display = "none";

				var screen_ = document.createElement ("div");
				screen_.classList.add ("results");

				if (this.pcStats.hp.cur <= 0) {
					if (this.failure) {
						this.failure (this, screen_);
					}
				} else {
					if (this.victory) {
						this.victory(this, screen_);
					}
				}
				challengeNode.appendChild (screen_);
			}
		};

	return (
		{ Encounter: Encounter
		, Action: Action
		, setFrame: setFrame
		});
}());
