var combat = (function (undefined) {
	"use strict";

	var pc, npc, combat;
	combat = {
		selected: "",
		enemies: [],
		enemyActors: [],
		allies: []
	};
	var sides = {};
	sides.ALLY = 0;
	sides.ENEMY = 1;
	function allAllies () {
		if (npc !== undefined) {
			return [pc, npc];
		} else {
			return [pc];
		}
	}
	function allEnemies () {
		return combat.enemyActors;
	}
	function pcTarget () {
		return [combat.selectedActor];
	}
	function randomOnSide (side) {
		var ts;
		switch (side) {
			case sides.ALLY:
				ts = allAllies();
				break;
			case sides.ENEMY:
			default:
				ts = allEnemies();
				break;
		}
		return [ts[Math.floor (Math.random() * ts.length)]];
	}

	function setPc (newPc) {
		pc = newPc;
	}
	function setNpc (newNpc) {
		npc = newNpc;
	}

	// other values: sprite(s)??
	// FIXME: there is nearly no interaction between jQuery DOM objects and this, which means that things like dealing damage aren't reflected in altering the actor's hp. this is a problem when it comes time to end the combat and update the player's real hp w/ the damage incurred by the fight, since that only exists in a DOM object that has no clue about "players" and "stats"
	function Actor (name, stats, actions) {
		this.name = name;
		this.stats = stats;
		this.actions = actions;
		this.id = "";
		this.domColumn = null;
	}
	Actor.prototype =
		{ combatDom: function (id) {
				var
					column,
					selector,
					this_ = this;
				column = document.createElement ('div');
				column.classList.add ("column");
				if (isAlly (this)) {
					column.classList.add ("self");
				} else {
					column.classList.add ("monstro");
				}
				column.id = id;

				selector = document.createElement ('div');
				selector.classList.add ("selector");
				selector.appendChild (document.createElement ('img'));
				selector.appendChild (spanLabel ("name", this.name, "div"));
				selector.appendChild (hpDisplay (this.stats.hp.cur, this.stats.hp.max, function () {
					this_.killed();
				}));
				column.appendChild (selector);
				// fixme: uh we only want actionmeters for the pc; all other actors just get a display bar w/ no clicking
				_.map (this.actions, function (a) {
					column.appendChild (a.meter (this_));
				});
				// FOR INVOLVED UX REASONS YOU DON'T WANT TO BE ABLE TO SELECT YR OWN UNITS (AND THEN ACCIDENTALLY HIT YRSELF IN THE FACE W/ A QUEUED ATTACK, IS THE POINT)
				if (!isAlly (this)) {
					selector.classList.add ("selectable");
					selector.addEventListener ('click', function () {
						selectColumn (this); // yes, `this` and not `this_` is correct
					});
				}

				this.domColumn = column; // so we can pass around Actors and have them tied to their dom values
				return column;
			}
		, killed: function() {
				if (isPc (this)) {
					// todo: freeze all actions, active damage, and controls
					// gameover();
					if (combat.loseCallback) {
						combat.loseCallback ();
					}
				} else if (isAlly (this)) {
					// autorevive(actor);
					// also make sure to reset the meter's callback somehow since they can die multiple times :/
				} else {
					// todo: call the actor's killscript
					// if the killscript doesn't exist or returns false, run the default kill
					victoryWhenEverythingIsDead (this);
				}
			}
		, getHit: function (fromActor, fromMeter, damage) {
				var loc = $(fromMeter).position();
				var targetHealth = this.domColumn.querySelector (".hpdisplay");
				var targetDisplay = this.domColumn.querySelector (".hpdisplay .cur");
				var targetLoc = $(targetDisplay).position();
				var damageNumbers;
				var this_ = this;
				damageNumbers = document.createElement ("div");
				damageNumbers.classList.add ("hitdisplay");
				damageNumbers.style.position = "absolute";
				damageNumbers.style.top = loc.top + "px";
				damageNumbers.style.left = loc.left + "px";
				if (damage.hp >= 0) {
					damageNumbers.classList.add ("damage");
					damageNumbers.appendChild (document.createTextNode (damage.hp));
				} else {
					damageNumbers.classList.add ("heal");
					damageNumbers.appendChild (document.createTextNode (-damage.hp));
				}
				combat.frame.appendChild (damageNumbers);
				$(damageNumbers).animate ({top: targetLoc.top + 22, left: targetLoc.left},
					{ duration: 600
					, easing: "swing"
					, complete: function () {
							alterValue (targetHealth, damage.hp, this_); // deal damage to target hp
							alterValueUnsafe (this, damage.hp); // decrement damage numbers
							$(this).animate ({opacity: 0},
								{ duration: 900
								, easing: "swing"
								, complete: function () {
										this.parentNode.removeChild(this);
									}
								});
						}
					});
			}
		};

	function Action (name, baseCharge, baseCooldown, damfunc, hitfunc) {
		this.name = name;
		this.baseCharge = baseCharge;
		this.baseCooldown = baseCooldown;
		this.hitfunc = hitfunc === undefined ? null : hitfunc;
		this.damfunc = damfunc;
		this.used = 0;
	}
	Action.prototype =
	// | todo: make these values depend on the actor's stats (mostly charge?)
	// | also todo: have a display meter for enemies, rather than a skill meter (since you can click skill meters). this should maybe be based on their stats? like if you have Insight 1 you can see their available actions, and if you have Insight 2 you can see their charge/cooldown? or completely separate skills, one that shows charge indicators but not names, and another that shows names/effects. and leveling them up lets you see more?
		{ meter: function (actor) {
			var this_ = this;
			return skillMeter (this.name, this.baseCharge, this.baseCooldown, this.hitfunc, function (u, t) {
					this_.used += 1;
					return this_.damfunc(u, t);
				}, actor);
			}
		};

	function isAlly (actor) {
		return (actor.id === "ally1" || actor.id === "ally2");
	}

	function isPc (actor) {
		return (actor.id === "ally1");
	}

	function selectColumn (sel) {
		var i, cols = combat.frame.querySelectorAll (".column .selector");
		_.map (cols, function (c) {
			c.classList.remove ("selected");
		});
		sel.classList.add ("selected");
		combat.selected = sel.parentNode.id;
		combat.selectedActor = null;
		for (i = 0; i < combat.enemyActors.length; i++) {
			if (combat.enemyActors[i].domColumn === sel.parentNode) {
				combat.selectedActor = combat.enemyActors[i];
			}
		}
		if (combat.selectedActor === null) {
			console.log ("ERR: no actor selected on target \"" + combat.selected + "\"");
		}
	}

	function hpDisplay (cur, max, zeroCallback) {
		var display;
		display = document.createElement ('div');
		display.classList.add ("hpdisplay");
		display.appendChild (spanLabel ("cur", cur));
		display.appendChild (spanLabel ("max", max));
		display.dataset.cur = cur;
		display.dataset.max = max;
		display.zeroCallback = zeroCallback;
		return display;
	}

	// | note that a cooldown adds an additional 200ms delay, due to animation stuff. if you want to have two actions that take the same time, one of which has a cooldown and the other of which doesn't, subtract 200 from the cooldown to make them equal (or as equal as possible w/ js)
	function skillMeter (name, castTime, cooldownTime, hitfunc, damfunc, user) {
		var
			skill,
			inner;
		inner = document.createElement ('span');
		inner.classList.add ("inner");
		inner.appendChild (spanLabel ("text", name));
		skill = document.createElement ('button');
		skill.classList.add ("skillmeter");
		skill.appendChild (inner);
		skill.appendChild (spanLabel ("text", name));
		skill.addEventListener ('click', function () {
			var chargeFunc, cooldownFunc, this_;
			this.disabled = true;
			this_ = this;
			if (castTime === 0) {
				chargeFunc = function (f) {
					inner.style.transition = "0";
					inner.style.width = "100%";
					f();
				}.bind(skill);
			} else {
				chargeFunc = function (f) {
					$(inner).animate ({width: "100%"},
						{ duration: castTime
						, easing: "linear"
						, complete: function () {
								f();
							}.bind(this)
						});
				}.bind(skill);
			}
			if (cooldownTime === 0) {
				cooldownFunc = function () {
					this_.disabled = false;
					$(inner).animate ({opacity: "0"},
						{ duration: 200
						, easing: "swing"
						, complete: function () {
								inner.style.width = 0;
								inner.style.opacity = 1;
							}
						});
				};
			} else {
				cooldownFunc = function () {
					inner.style.transition = "0.2s";
					this_.classList.add ("cooldown");
					setTimeout (function() {
						inner.style.transition = "none";
						$(inner).animate ({width: "0%"},
							{ duration: cooldownTime
							, easing: "linear"
							, complete: function () {
									this_.classList.remove ("cooldown");
									this_.disabled = false;
								}
							});
					}, 200);
				};
			}
			chargeFunc (function () {
				var meter = this;
				var targets = hitfunc ();
				_.map (targets, function (target) {
					target.getHit (user, meter, damfunc (user.stats, target.stats));
				});
				cooldownFunc ();
			}.bind(this));
		});
		return skill;
	}

	function victoryWhenEverythingIsDead (actor) {
		var column;
		if (combat.enemies.length === 1 && combat.enemies[0] === actor.id) {
			finishCombat ();
			if (combat.winCallback) {
				combat.winCallback (pc);
			}
		}
		// remove this from the enemies list
		combat.enemies.splice (combat.enemies.indexOf (actor.id), 1);
		if (actor.id === combat.selected && combat.enemies.length !== 0) {
			selectColumn (combat.frame.querySelector ("#" + combat.enemies[0] + " .selector"));
		}
		// destroy its div -- we don't actually _destroy_ it, because that would reorganize the other enemy columns. we just let it keep being transparent. we should probably also disable all its actions (if applicable), since otherwise we can still click them :V
		column = combat.frame.querySelector ("#" + actor.id);
		$(column).animate ({opacity: 0},
			{ duration: 800
			, easing: "swing"
			, complete: function () {
				this.style.visibility = "hidden";
			}
		});
		// todo: THIS SHOULD ALSO STOP ALL OF THE ACTOR'S ACTION COUNTERS, EVEN THOSE CURRENTLY DEALING DAMAGE OR W/E.
	}

	function finishCombat () {
		setTimeout (function () {
				while (combat.frame.firstChild) {
					combat.frame.removeChild (combat.frame.firstChild);
				}
				combat.selected = "";
				combat.selectedActor = null;
			}, 800);
	}

	function init (frameId, enemies, winCallback, loseCallback) {
		combat.frame = document.querySelector (frameId);

		// just in case, wipe the visible combat state
		while (combat.frame.firstChild) {
			combat.frame.removeChild (combat.frame.firstChild);
		}

		combat.winCallback = winCallback;
		combat.loseCallback = loseCallback;

		combat.allies = [];
		pc.id = "ally1";
		combat.frame.appendChild (pc.combatDom ("ally1"));
		combat.allies.push ("ally1");
		if (npc) {
			npc.id = "ally2";
			combat.frame.appendChild (npc.combatDom ("ally2"));
			combat.allies.push ("ally2");
		}

		combat.enemyActors = enemies;
		combat.enemies = [];
		var i = 1;
		_.map (enemies, function (e) {
			var name = "enemy" + i;
			combat.enemies.push (name);
			e.id = name;
			combat.frame.appendChild (e.combatDom (name));
			i+=1;
		});
		selectColumn (combat.frame.querySelector ("#enemy1 .selector"));
	}

	// aka "just change some numbers instead of worrying about death callbacks"; this is used to decrement the damage number displays
	function alterValueUnsafe (display, amount) {
		var
			cur = $(display).html(),
			set = 0,
			increment = 1,
			delay = 750 / amount;
		while (delay < 25) {
			delay *= 2;
			increment *= 2;
		}

		function repeat () {
			if (set + increment > amount) {
				cur -= (amount - set);
				set += (amount - set);
				$(display).html(cur);
			} else {
				cur -= increment;
				set += increment;
				$(display).html(cur);
			}
			if (set < amount) {
				setTimeout (repeat, delay);
			}
		}
		repeat ();
	}

	function alterValue (display, amount, target) {
		var
			cur = display.querySelector (".cur"),
			set = 0,
			increment = 1,
			delay = 750 / amount;
		while (delay < 25) {
			delay *= 2;
			increment *= 2;
		}
		function repeat () {
			if (display.dataset.cur - increment < 0) {
				$(cur).html(0);
				if (target !== undefined) {
					target.stats.hp.cur = 0;
				}
				if (display.zeroCallback !== undefined) {
					display.zeroCallback();
					display.zeroCallback = undefined;
				}
				return;
			}
			if (display.dataset.cur - increment > display.dataset.max) {
				$(cur).html(display.dataset.max);
				if (target !== undefined) {
					target.stats.hp.cur = display.dataset.max;
				}
				return;
			}
			if (set + increment > amount) {
				display.dataset.cur -= (amount - set);
				set += (amount - set);
			} else {
				display.dataset.cur -= increment;
				set += increment;
			}
			$(cur).html(display.dataset.cur);
			if (target !== undefined) {
				target.stats.hp.cur = display.dataset.cur;
			}
			// if we die on the last tick.
			if (display.dataset.cur - 0 == 0) {
				if (display.zeroCallback !== undefined) {
					display.zeroCallback();
					display.zeroCallback = undefined;
				}
			}
			if (set < amount) {
				setTimeout (repeat, delay);
			}
		}
		repeat();
	}

	return (
		{ Actor: Actor
		, Action: Action
		, Sides: sides
		, hitfuncs: (
			{ allAllies: allAllies
			, allEnemies: allEnemies
			, pcTarget: pcTarget
			, randomOnSide: randomOnSide
			})
		, setPc: setPc
		, setNpc: setNpc
		, init: init
		});
}());
