
"use strict";

// Production steps of ECMA-262, Edition 5, 15.4.4.14
// Reference: http://es5.github.io/#x15.4.4.14
if (!Array.prototype.indexOf) {
  Array.prototype.indexOf = function(searchElement, fromIndex) {

    var k;

    // 1. Let O be the result of calling ToObject passing
    //    the this value as the argument.
    if (this == null) {
      throw new TypeError('"this" is null or not defined');
    }

    var O = Object(this);

    // 2. Let lenValue be the result of calling the Get
    //    internal method of O with the argument "length".
    // 3. Let len be ToUint32(lenValue).
    var len = O.length >>> 0;

    // 4. If len is 0, return -1.
    if (len === 0) {
      return -1;
    }

    // 5. If argument fromIndex was passed let n be
    //    ToInteger(fromIndex); else let n be 0.
    var n = +fromIndex || 0;

    if (Math.abs(n) === Infinity) {
      n = 0;
    }

    // 6. If n >= len, return -1.
    if (n >= len) {
      return -1;
    }

    // 7. If n >= 0, then Let k be n.
    // 8. Else, n<0, Let k be len - abs(n).
    //    If k is less than 0, then let k be 0.
    k = Math.max(n >= 0 ? n : len - Math.abs(n), 0);

    // 9. Repeat, while k < len
    while (k < len) {
      var kValue;
      // a. Let Pk be ToString(k).
      //   This is implicit for LHS operands of the in operator
      // b. Let kPresent be the result of calling the
      //    HasProperty internal method of O with argument Pk.
      //   This step can be combined with c
      // c. If kPresent is true, then
      //    i.  Let elementK be the result of calling the Get
      //        internal method of O with the argument ToString(k).
      //   ii.  Let same be the result of applying the
      //        Strict Equality Comparison Algorithm to
      //        searchElement and elementK.
      //  iii.  If same is true, return k.
      if (k in O && O[k] === searchElement) {
        return k;
      }
      k++;
    }
    return -1;
  };
}

function removeFirstMatch (arr, val) {
	var i = arr.indexOf (val);
	if (i !== -1) {
		arr.splice (i, 1);
	}
}

// removes and returns the first list value matching the predicate, or null
function findExtract (list, pred) {
	var i = 0;
	var r;
	while (i < list.length) {
		if (pred (list[i])) {
			r = list.splice (i, 1);
			return r[0];
		}
		i++;
	}
	return null;
}

/* i REALLY wanted an enum form that
 *   1. let me refer to things as `enumname.thing` and
 *   2. loudly errored if i was using a name that wasn't actually in the enum.
 * javascript will happily let you read undefined properties:
 *   var foo = {};
 *   var bar = foo.asdf;
 * is entirely okay, and any error will only happen once you use bar for
 * something. this was leading to any typos (or uses of outdated items) in my
 * item enum generating errors, not where i made the typo, but deep in the guts
 * of the item code, without any reference whatsoever to the typo. but
 * javascript is a lot more strict about calling things that aren't functions;
 * those will immediately error. hence: this.
 */
function Enum () {
	// javascript closures are wild (b/c `i` is altered, it's equal to
	// the-highest-enum-value-plus-one at the end of the loop, and consequently
	// when we enter the closure by calling any of the enum functions, `i` is
	// equal to its last values -- if we just went
	//   this[arguments[i]] = function () { return i; }
	// that would always return the same number for every enum function. so we
	// need to have an intermediate function to re-scope/maintain the value as it
	// existed at the point the enum value was constructed.)
	function scope (i) {
		return function () {
			return i;
		}
	}
	for (var i = 0; i < arguments.length; i++) {
		this[arguments[i]] = scope (i);
	}
	Object.freeze(this);
}

// note that if an object's constructor isn't pure, then the objects cloned won't necessarily be the same. ALSO MORE IMPORTANTLY THIS DOESN'T SEEM TO WORK ON ANY NON-ANONYMOUS OBJECTS, SO, (b/c the constructor takes args and thus explodes when it's given nothing but undefined)
function clone(obj) {
	var
		copy, attr;
	if (null === obj || "object" !== typeof obj) {
		return obj;
	}
	copy = obj.constructor();
	for (attr in obj) {
		if (obj.hasOwnProperty(attr)) {
			copy[attr] = obj[attr];
		}
	}
	return copy;
}
// the `clone` is because by default javascript passing objects around by reference, meaning without this we'd get an array of n objects that all change when any one is updated.
function replicate (n, val) {
	return Array.apply(null, new Array(n)).map(function () {
		return clone (val);
	});
}

// this will copy objects by reference rather than trying to clone them
function replicatePointers (n, val) {
	return Array.apply(null, new Array(n)).map(function () {
		return val;
	});
}

function nub (arr) {
	function index (v) {
		var i = 0;
		while (i < arr.length) {
			if (arr[i] === v) {
				return i;
			}
			i++;
		}
		return -1;
	}
	return arr.filter (function (v, i) {
		return index (v) === i;
	});
}

function clearDOM (where) {
	while (where.firstChild) {
		where.removeChild (where.firstChild);
	}
}

function element (element, text) {
	var e = document.createElement (element);
	if (text !== undefined) {
		e.appendChild (document.createTextNode (text));
	}
	return e;
}

function setText (where, text) {
	clearDOM (where);
	where.appendChild (document.createTextNode (text));
}

function spanLabel (label, contents, element) {
	var s;
	if (element === undefined) {
		s = document.createElement ("span");
	} else {
		s = document.createElement (element);
	}
	if (label) {
		s.classList.add (label);
	}
	s.appendChild (document.createTextNode (contents));
	return s;
}

function pad (filler, width, arr) {
	var
		toAdd = _.max ([0, width - arr.length]);
	if (toAdd === 0) {
		return arr;
	}
	return arr.concat (replicate (toAdd, filler));
}
function fromMaybe (def, val) {
	if (val === undefined || val === null) {
		return def;
	}
	return val;
}
