"use strict";
var Editor = (function () {

	var body = null;
	var ngender = null;
	var pgender = null;
	var stub = "orb";

	var storedParts = [];

	function updateDescription (descriptionFrame) {
		var obj_str = JSON.stringify (body.asNecessary());
		descriptionFrame.classList.add ("dirty");
		jQuery.getJSON ("/hell/description", {data: obj_str, ngender: ngender, pgender: pgender}, function (desc, textStatus, jqHXR) {
			descriptionFrame.classList.remove ("dirty");
			clearDOM (descriptionFrame);
			_.map (desc.errors, function (e) {
				var p = document.createElement ("li");
				p.appendChild (document.createTextNode (e));
				descriptionFrame.appendChild (p);
			});
			_.map (desc.description, function (paragraph) {
				var p = document.createElement ("p");
				p.appendChild (document.createTextNode (paragraph));
				descriptionFrame.appendChild (p);
			});
			stub = desc.stub ? desc.stub : "???";
		});
	}

	var selectedNode = null;

	function addStats (part) {
		var hoverInfo = document.querySelector ("#hoverStats");
		var o = $(this).offset();
		clearDOM (hoverInfo);
		hoverInfo.appendChild (statsDOM(part));
		hoverInfo.style.top = Math.floor (o.top + 36) + "px";
		hoverInfo.style.left = Math.floor (o.left + 48) + "px";
		hoverInfo.classList.remove ("rem");
	}
	function remStats (part) {
		var hoverInfo = document.querySelector ("#hoverStats");
		hoverInfo.classList.add ("rem");
	}
	function statsDOM (part) {
		return Stats.partStats(part.value).asModifier();
	}

	function nodeParentList (tree) {
		var ns = [];
		while (tree != null) {
			ns.push (tree);
			tree = tree.parent;
		}
		return ns.reverse();
	}
	function collateChildren (tree) {
		var cs = {};
		tree.children.map (function (ch) {
			if (!cs[ch.type]) {
				cs[ch.type] = [];
			}
			cs[ch.type].push (ch);
		});
		return cs;
	}
	function collateAllowedChildren (tree) {
		var cs = {};
		tree.children.map (function (ch) {
			if (!cs[ch.type]) {
				cs[ch.type] = [];
			}
			cs[ch.type].push (ch);
		});
		// this is kind of janky i know
		tree.childrenTemplate.map (function (template) {
			if (!cs[template.n.type]) {
				cs[template.n.type] = [];
			}
			if (cs[template.n.type].length < template.limit) {
				cs[template.n.type].push (template);
			}
		});
		return cs;
	}

	function exclusiveSetClass (domain, what, class_) {
		exclusiveOp (domain, what, class_, function (p) {
			p.classList.remove (class_);
		}, function (p) {
			p.classList.add (class_);
		});
	}
	function exclusiveOp (domain, what, class_, failure, success) {
		_.map (domain.querySelectorAll ("." + class_), function (p) {
			failure (p);
		});
		if (what) {
			success (what);
		}
	}

	function selectivelyShowParts (partsDOM, showFilter, enableFilter) {
		_.map (partsDOM.children, function (p) {
			if (showFilter (p)) {
				p.style.display = "inline";
			} else {
				p.style.display = "none";
			}
			if (enableFilter (p)) {
				p.disabled = false;
			} else {
				p.disabled = true;
			}
		});
	}

	function fillerNodeAt (node, name) {
		var parents = [];
		var n;
		var p;

		p = node.parent;
		while (p) {
			parents.push (p);
			p = p.parent;
		}
		parents.reverse();
		n = new Body.Part (name, parents);
		if (name === "") {
			n.clear();
		}
		return n;
	}

	function nodeAndAllKids (n) {
		var o = [];
		o.push (n.value);
		n.children.map (function (c) {
			o = o.concat (nodeAndAllKids (c));
		});
		return o;
	}

	function overlapsWith (node, newNode) {
		var duplicate = false;
		if (node.parent) {
			_.map (node.parent.children, function (sibling) {
				if (sibling.value !== node.value &&
						sibling.overlaps (newNode)) {
					duplicate = true;
				}
			});
		}
		return (
			!node.permits (newNode) ? 1
			: duplicate ? 2
			: 0);
	}

	function renderBodyTree (bodyTree, callback) {
		var container;
		var subs;
		if (bodyTree.value === "") {
			return null;
		}
		container = document.createElement ("li");
		container.classList.add ("nodec");
		if (callback) {
			container.appendChild (bodyTree.asButton(callback, null, null));
		} else {
			container.appendChild (bodyTree.asStatic(false));
		}
		subs = document.createElement ("ul");
		bodyTree.children.filter (function (ch) {
			return ch.value !== "";
		}).map (function (ch) {
			subs.appendChild (renderBodyTree (ch, callback));
		});
		if (subs.children.length > 0) {
			container.appendChild (subs);
		}
		return container;
	}

	function bindSelectedNode (node) {
		selectedNode = node;
		//console.log ("(in bindSelectedNode) selectedNode set to:");
		//console.log (selectedNode);
	}

	function renderBodyBreadcrumb (bodyTree, partsList, addPredicate, addCallback, rerenderCallback) {

		var container = document.createElement ("div");
		var breadcrumbs = document.createElement ("div");
		var nodeInfo = document.createElement ("div");
		breadcrumbs.classList.add ("breadcrumbs");
		container.appendChild (breadcrumbs);

		storedParts = partsList;

		if (selectedNode === null) {
			selectedNode = bodyTree;
			//console.log ("(in renderBodyBreadcrumb) selectedNode set to:");
			//console.log (selectedNode);
		}

		nodeParentList(selectedNode).map(function (node) {
			return node.asButton (function (e) {
				selectedNode = node;
				//console.log ("(in breadcrumb callback) selectedNode set to:");
				//console.log (selectedNode);
				rerenderCallback (renderBodyBreadcrumb (bodyTree, partsList, addPredicate, addCallback, rerenderCallback));
			}, null, null);
		}).reduce(function (acc, el) {
			acc.push (document.createTextNode ("\u2192"));
			acc.push (el);
			return acc;
		}, []).map (function (dom, i) {
			if (i == 0) {
				return;
			}
			breadcrumbs.appendChild (dom);
		});
		breadcrumbs.appendChild (document.createTextNode ("\u21B4"));

		breadcrumbs.appendChild (statsDOM(selectedNode));

		//console.log (collateAllowedChildren (selectedNode));
		_.map (collateAllowedChildren (selectedNode), (function (opt) {
			var set = document.createElement ("fieldset");
			var title = document.createElement ("legend");
			var typeclass = opt[0].n ? opt[0].n.typeclass() : opt[0].typeclass();
			set.classList.add ("nodeset");
			title.appendChild (document.createTextNode (typeclass));
			title.classList.add (typeclass);
			title.classList.add ("typeclass");
			set.appendChild (title);
			opt.map (function (o) {
				// ... place node option here
				var part;
				var li = document.createElement ("li");
				var duplicates;
				var disallowedInLocation;
				var optDOM;
				var addMoreButton;
				var reqDOM;
				//console.log (o);

				if (o.n) {
					let req = null;
					let reqID = null;
					switch (o.n.type) {
						case 2: // limb
							req = "Old Stone";
							reqID = 32;
							break;
						case 3: // shape
							req = "Chaos Crystal";
							reqID = 18;
							break;
						case 4: // animal
							req = "Howling Bottle";
							reqID = 33;
							break;
						case 5: // sex
							req = "Budding Branch";
							reqID = 9;
							break;
						case 6: // part
							req = "Black Goo";
							reqID = 34;
							break;
						case 7: // element
							req = "Shimmering Mirror";
							reqID = 35;
							break;
						case 8: // gloss
							req = "Spine Pod";
							reqID = 36;
							break;
						default:
							break;
					}
					addMoreButton = element ("button", "+");
					addMoreButton.addEventListener ('click', function (e) {
						var n = new Body.Part ("", []);
						n.type = o.n.type;
						n.permitted = o.n;
						addCallback (reqID);
						if (!addPredicate (reqID)) {
							this.disabled = true;
						}
						selectedNode.addChild (n);
						rerenderCallback (renderBodyBreadcrumb (bodyTree, partsList, addPredicate, addCallback, rerenderCallback));
					});
					if (!addPredicate (reqID)) {
						addMoreButton.disabled = true;
						li.classList.add ("failed");
					}
					li.appendChild (addMoreButton);
					reqDOM = element ("span", req);
					reqDOM.classList.add ("item");
					li.appendChild (document.createTextNode (" 1x "));
					li.appendChild (reqDOM);
					set.appendChild (li);
					return;
				}

				if (o.type == 9) { // detail
					li.appendChild (o.permitted.title());
				}
				optDOM = document.createElement ("select");
				//console.log (o);
				optDOM.addEventListener ('change', function (e) {
					var replacing;
					// TODO: change part inventory -- remove equipped node, add any unequipped node (including recursively)
					// update body
					replacing = fillerNodeAt (o, optDOM.children[optDOM.selectedIndex].value);

					if (replacing.value !== "") {
						// remove a replacing value from partsList
						removeFirstMatch (partsList, replacing.value);
					}
					if (o.value !== "") {
						// add o.value and all its kids, recursively, to partsList
						partsList = partsList.concat (nodeAndAllKids (o));
					}
					console.log (partsList);
					console.log ("node change: have first (" + o.value + "), replacing with second (" + replacing.value + ")");
					console.log (o);
					console.log (replacing);
					o.changeValue (replacing);
					// rebuild allowed nodes (<select> children) for other nodes; show/hide full inspect button
					rerenderCallback (renderBodyBreadcrumb (bodyTree, partsList, addPredicate, addCallback, rerenderCallback));

					// update body description
					updateDescription (document.querySelector ("#descriptionText"));
				});
				part = document.createElement ("option");
				part.appendChild (document.createTextNode (o.value === ""
					? "\u00abempty\u00bb"
					: o.nameVal()));
				part.value = o.value;
				optDOM.appendChild (part);
				duplicates = [];
				disallowedInLocation = [];
				partsList.filter (function (sparePart) {
					return Body.typeOf (sparePart) === o.type;
				}).map (function (sparePart) {
					var switch_ = document.createElement ("option");
					var n = new Body.Part (sparePart, []).nameVal();
					switch_.appendChild (document.createTextNode (n));
					switch_.value = sparePart;

					switch (overlapsWith (o, sparePart)) {
						case 1:
							disallowedInLocation.push (switch_);
							break;
						case 2:
							duplicates.push (switch_);
							break;
						case 0:
						default:
							optDOM.appendChild (switch_);
							break;
					}
				});
				li.appendChild (optDOM);
				if (o.value !== "") {
					var promoteButton;
					// let people unequip nodes
					var empty = document.createElement ("option");
					empty.appendChild (document.createTextNode ("\u00abempty\u00bb"));
					empty.value = "";
					optDOM.appendChild (empty);

					//console.log ("promotion:");
					//console.log (o);
					// and let them look at the equipped node's details (if there are details)
					if (o.children.length > 0 || !o.atMaxChildren()) {
						promoteButton = document.createElement ("button");
						promoteButton.appendChild (document.createTextNode ("\u21B8"));
						promoteButton.classList.add ("promote");
						promoteButton.addEventListener ('click', function (e) {
							selectedNode = o;
							//console.log ("(in promote button callback) selectedNode set to:");
							//console.log (selectedNode);
							rerenderCallback (renderBodyBreadcrumb (bodyTree, partsList, addPredicate, addCallback, rerenderCallback));
						});
						li.appendChild (promoteButton);
					}
				}
				var optgroup;
				if (duplicates.length > 0) {
					optgroup = document.createElement ("optgroup");
					optgroup.label = "duplicate";
					duplicates.map (function (d) {
						d.disabled = true;
						optgroup.appendChild (d);
					});
					optDOM.appendChild (optgroup);
				}
				if (disallowedInLocation.length > 0) {
					optgroup = document.createElement ("optgroup");
					optgroup.label = "not allowed here";
					disallowedInLocation.map (function (d) {
						d.disabled = true;
						optgroup.appendChild (d);
					});
					optDOM.appendChild (optgroup);
				}
				set.appendChild (li);
			});
			nodeInfo.appendChild (set);

			//console.log (opt);
		}));

		nodeInfo.classList.add ("nodeinfo");
		container.appendChild (nodeInfo);

		return container;
	}

	function renderNodeCollection (target, nodes, asEditor) {
		var controlFrame = document.querySelector (target);
		clearDOM (controlFrame);
		_.map (nodes, function (n) {
			var p;
			if (asEditor) {
				// FIXME: make clicking a node focus that on the breadcrumb display. not sure what kind of callback would need to be provided.
				p = new Body.Part (n, []).asButton (undefined /*switchCallback*/, addStats, remStats);
				p.disabled = true;
			} else {
				p = new Body.Part (n, []).asStatic (undefined, addStats, remStats);
			}
			controlFrame.appendChild (p);
		});
	}

	function setBody (body_, pgender_, ngender_) {
		if (body_ !== null) {
			body = body_;
		}
		if (pgender_ !== null) {
			pgender = pgender_;
		}
		if (ngender_ !== null) {
			ngender = ngender_;
		}
	}

	function getBody () {
		return body;
	}
	function getNodes () {
		return storedParts;
		/*
		var parts = document.querySelector ("#parts");
		var nodes = [];
		var p = parts.firstChild;
		console.log ("LEFTOVER NODES:");
		while (p) {
			console.log (p);
			if (p.dataset.node !== "") {
				nodes.push (p.dataset.node);
			}
			p = p.nextSibling;
		}
		console.log (nodes);
		return nodes;
		*/
	}
	function getStub () {
		return stub;
	}

	function clearState () {
		body = null;
		ngender = null;
		pgender = null;
		stub = "orb";

		storedParts = [];
		selectedNode = null;
	}

	return (
		{ updateDescription: updateDescription
		, renderBodyBreadcrumb: renderBodyBreadcrumb
		, renderBodyTree: renderBodyTree
		, renderNodeCollection: renderNodeCollection
		, setBody: setBody
		, getBody: getBody
		, getNodes: getNodes
		, getStub: getStub
		, clearState: clearState
		, bindSelectedNode: bindSelectedNode
		});
})();
