var Event = (function (){
	/*
	things to look at:

		ferrying state around is kind of a labyrinthine mess. partly b/c of `choices`; it has async link triggers, so the chunk functions can't be pure AND `Chunk` needs to pass in all the information we need to continue running events.

	*/
	function EventPage (init, evs, static_) {
		this.vars = {};
		this.evs = evs;
		this.init = init;
		this.static_ = !(static_ === false);
		this.closeCallback = null;
	}
	// TODO: we need this to properly handle the case where there's no event to even run until that event api request returns ("loading" text or w/e)
	EventPage.prototype =
		{ start: function (whereTitle, where, player, encounter) {
				var bodyDOM;
				var bodyUL;
				this.init (this);
				setText (whereTitle, this.getVar ("_title"));
				if (this.body) {
					console.log ("have event body:");
					console.log (this.body);
					bodyDOM = document.createElement ("section");
					bodyDOM.classList.add ("body");
					bodyDOM.classList.add ("fauxbody");
					bodyDOM.appendChild (element ("header", "body"));
					bodyUL = document.createElement ("ul");
					bodyUL.appendChild (Editor.renderBodyTree (this.body));
					bodyDOM.appendChild (bodyUL);
					where.appendChild (bodyDOM);
				}
				this.runLocal (where, "_activate", player, encounter);
			}
		, runLocal: function (where, name, player, encounter) {
				if (this.evs[name]) {
					this.evs[name].run (where, this, player, encounter);
				}
			}
		, runLocalFrom: function (where, name, offset, player, encounter) {
				if (this.evs[name]) {
					this.evs[name].runFrom (where, this, offset, player, encounter);
				}
			}
		, pushEncounterData: function (key, value) {
				if (!(key in this.vars)) {
					this.vars[key] = [];
				} else if (!Array.isArray (this.vars[key])) {
					return false;
				}
				this.vars[key].push (value);
				return true;
			}
		, setVar: function (key, value) {
				if (value === null) {
					delete this.vars[key];
					return;
				}
				if (typeof value === "object" && "mod" in value) {
					if (!this.vars[key]) { this.vars[key] = 0; }
					switch (value.mod) {
						case "inc":
							this.vars[key]++;
							break;
						case "dec":
							this.vars[key]--;
							break;
						default:
							console.error ("can't parse modifier \"" + value.mod + "\"");
							break;
					}
				} else {
					this.vars[key] = value;
				}
			}
		, getVar: function (key) {
				return this.vars[key];
			}
		, dumpVars: function () {
				return this.vars;
			}
		, loadVars: function (vars) {
				if (vars) {
					this.vars = vars;
				}
			}
		, isStatic: function () {
				return this.static_;
			}
		, onCloseCallback: function (f) {
				this.closeCallback = f;
			}
		};

	function Event () {
		var i;
		this.chunks = [];
		for (i = 0; i < arguments.length; i++) {
			this.chunks.push (arguments[i]);
		}
	}
	Event.prototype =
		{ run: function (where, page, player, encounter, quietFinish) {
				this.runFrom (where, page, 0, player, encounter, quietFinish);
			}
		, runFrom: function (where, page, offset, player, encounter, quietFinish) {
				var i;
				var finished = true;
				var topAnchor;
				topAnchor = where.lastChild ? where.lastChild : where;
				// this might be incorrect; this is just an edit of `$(topAnchor).offset()` to remove the sole jQuery reference
				var localPos = {top: topAnchor.offsetTop, width: topAnchor.offsetWidth };
				for (i = offset; i < this.chunks.length; i++) {
					finished = this.chunks[i].run (where, page, player, encounter);
					// this is used in events that suspend and resume event flow -- fauxChoice and challenge. (`choices` returns false, but it doesn't really do anything, because it's expected it'll be the last event in any given run anyway, and it doesn't 'resume' event running, it just jumps to a new event.)
					if (finished === -1) {
						break;
					}
				}
				if (finished > 0 && !quietFinish) {
					// add a close event too
					//console.log ("DETECTED EVENT HAS RUN ITS COURSE; ADDING CLOSE BUTTON");
					close ().run (where, page, player, encounter);
				}
				// see above re: jQuery
				//$('html,body').scrollTop (localPos.top|0);
				window.scroll (0, localPos.top|0);
			}
		};

	function Chunk (rfunc) {
		this.rfunc = rfunc;
	}
	Chunk.prototype =
		{ run: function (where, page, player, encounter) {
				return this.rfunc (where, page, player, encounter);
			}
		};

	function canProvide (item, inventory, nodeInventory) {
		if (item instanceof Item.Item) {
			return (inventory.hasItem (item.item, item.count));
		} else if (typeof item === "string") {
			return (nodeInventory.indexOf (item) === -1) ? false : true;
		} else {
			console.error ("canProvide: not sure what to do with \"" + typeof item + "/" + item + "\"");
		}
	}
	function saleItemDom (item) {
		if (item instanceof Item.Item) {
			return item.dom ();
		} else if (typeof item === "string") {
			return new Body.Part (item).asStatic ();
		} else {
			console.error ("saleItemDom: not sure what to do with \"" + typeof item + "/" + item + "\"");
		}
	}
	function missingCosts (costs, inv, nodeInv) {
		var missing = [];
		_.map (costs, function (c, i) {
			if (!canProvide (c, inv, nodeInv)) {
				missing.push (i);
			}
		});
		return missing;
	}
	function removeItemOrNode (thing, inv, nodeInv) {
		if (thing instanceof Item.Item) {
			inv.giveItem (thing.item, thing.count * -1);
		} else if (typeof thing === "string") {
			removeFirstMatch (nodeInv, thing);
		} else {
			console.error ("removeItemOrNode: not sure what to do with \"" + typeof thing + "/" + thing + "\"");
		}
	}
	function giveItemOrNode (thing, inv, nodeInv) {
		if (thing instanceof Item.Item) {
			inv.giveItem (thing.item, thing.count);
		} else if (typeof thing == "string") {
			nodeInv.push (thing);
		} else {
			console.error ("giveItemOrNode: not sure what to do with \"" + typeof thing + "/" + thing + "\"");
			//console.log (thing);
		}
	}

	function Alchemy (items, node) {
		this.cost = items;
		this.sale = node;
	}
	Alchemy.prototype =
		{ dom: function (inventory, nodeInventory, buyCallback) {
				var container, costDom, saleDom, canBuy, saleButton;
				costDom = document.createElement ("div");
				saleDom = document.createElement ("div");
				costDom.classList.add ("cost");
				saleDom.classList.add ("sale");
				var missing = missingCosts (this.cost, inventory, nodeInventory);
				canBuy = missing.length === 0;
				_.map (this.cost, function (item, i) {
					var iDom, cont;
					cont = document.createElement ("span");
					iDom = saleItemDom (item);
					if (_.indexOf(missing, i) !== -1) {
						iDom.classList.add ("insufficient");
					}
					cont.appendChild (iDom);
					costDom.appendChild (cont);
				});
				if (this.sale instanceof Item.Item) {
					saleButton = document.createElement ("button");
					saleButton.appendChild (this.sale.dom());
					if (canBuy && buyCallback !== undefined) {
						saleButton.addEventListener ('click', buyCallback);
					} else {
						saleButton.disabled = true;
					}
				} else if (typeof this.sale === "string") {
					saleButton = new Body.Part (this.sale).asButton (canBuy ? buyCallback : undefined);
					if (!canBuy) {
						saleButton.disabled = true;
					}
				} else {
					console.error ("Alchemy.dom: not sure what to do with \"" + typeof i + "/" + i + "\" (in sale)");
				}
				saleDom.appendChild (saleButton);
				container = document.createElement ("li");
				container.classList.add ("alchemy");
				container.appendChild (saleDom);
				container.appendChild (costDom);
				return container;
			}
		};


	function dialogFrame (class_, img, name, els) {
		var i, bloc, icon, speech, namedom;
		bloc = document.createElement ("div");
		icon = document.createElement ("img");
		speech = document.createElement ("div");
		bloc.classList.add (class_);
		icon.src = img;
		icon.classList.add ("icon");
		speech.classList.add ("speech");
		if (els !== undefined && els.length) {
			for (i = 0; i < els.length; i++) {
				speech.appendChild (els[i]);
			}
		}
		bloc.appendChild (icon);
		if (name !== undefined) {
			namedom = document.createElement ("div");
			namedom.classList.add ("name");
			if (typeof name === "string") {
				namedom.appendChild (document.createTextNode (name));
			} else {
				namedom.appendChild (name);
			}
			bloc.appendChild (namedom);
		}
		bloc.appendChild (speech);
		return bloc;
	}


function nameBox () {
	var name;
	name = document.createElement ("input");
	name.type = "text";
	name.id = "nameBox";
	return [document.createTextNode ("name: "), name];
}

function nextRespawnTime (now) {
	var RESPAWN_INTERVAL = 1000;
	return Math.ceil ((now+1) / RESPAWN_INTERVAL) * RESPAWN_INTERVAL;
}

// given a list of lists of mixed strings and markup objects, return a list of dom p nodes with correct formatting
function stringmarkup (page, player, texts) {
	return _.map (texts, function (ts) {
		var p = document.createElement ("p");
		var inline;
		_.map (ts, function (t) {
			_.map (constructNode (page, player, t), function (n) { p.appendChild (n);});
		});
		return p;
	});
}

function constructNode (page, player, t) {
	var inline;
	var output = [];
	if (typeof t === "string") {
		// hax
		switch (t) {
			case "__nameboxprompt":
				var ns = nameBox();
				output = [ns[0], ns[1]];
				break;
			default:
				output = [document.createTextNode (t)];
				break;
		}
	} else if (t.cond) { // varbranch
		if (condPass (t.cond, page, player)) {
			_.map (t.t, function (t_) { output = output.concat (constructNode (page, player, t_)); });
		} else {
			_.map (t.f, function (t_) { output = output.concat (constructNode (page, player, t_)); });
		}
	} else {
		if (t.t) { // markup
			inline = document.createElement (
				t.t === "i" ? "em" :
				t.t === "b" ? "strong" :
				t.t === "br" ? "br" :
				t.t === "q" ? "blockquote" :
				"");
			if (t.s) {
				if (t.t === "q") {
					var ts = stringmarkup (page, player, t.s);
					_.map (ts, function (t) { inline.appendChild (t); });
				} else {
					inline.appendChild (document.createTextNode (t.s));
				}
			}
		} else if (t.l) { // lookup
			inline = document.createTextNode (page.getVar (t.l));
		} else {
			// let's assume it's a raw dom element
			inline = t;
		}
		output = [inline];
	}
	return output;
}

function p (text) {
	return [document.createTextNode (text)];
}

function randomAI(opts) {
	return (opts[Math.floor (Math.random() * opts.length)]);
}

/* okay so the idea here is that certain events (mostly the ones involving a
 * lot of body polymorphism) aren't feasible to calculate in javascript,
 * just because... javascript. maybe that's a reasonable idea to have, maybe
 * not.
 *
 * so instead we have an ~asynchronous json api~ that takes body data + mob
 * data and spits out some preparsed events. this is the function that takes a
 * bunch of JSON and transforms it into the correct kind of closures and
 * javascript objects that the event code actually runs.
 *
 * the event page that this returns is incomplete (it doesn't have an init
 * function) but it's meant to basically have its events extracted and
 * attached to a "real" event page.
 */
// both "key" and "i" are only needed for fauxChoice & challenge, which needs to halt event execution and then resume it later. this is a huge mess, b/c it requires wayyy more context to execute than anything else
	function parseJSONEvent (key, ev, i) {
		switch (ev.type) {
			case "narration":
				return narration (ev.texts);
			case "dialog":
				return dialog (ev.img, ev.name, ev.texts);
			case "itemModify":
				return itemModify (_.map (ev.items, function (v) {
					return new Item.Item (v.item, v.count);
				}));
			case "nodeGive":
				return itemModify ([ev.node]);
			case "pushNode":
				return pushNode (ev.action, ev.path);
			case "choices":
				return choices (_.map (ev.choices, function (v) {
					// FIXME: make this work for more complex conditions (i.e., var x set to 0 AND __hasitems abc). this would maybe be a list of lists of items? each individual list is one set of requirements. so an AND would concat, and an OR would return [c, d], except with a condition like `a OR b OR c` we'd want [a,b,c] back, not [a,[b,c]].
					function getItemReq (cond) {
						if (cond.items !== undefined) {
							return _.map (cond.items, function (i) {
								return new Item.Item (i.item, i.count);
							});
						} else if (cond.flag) {
							return [];
						} else if (cond.a && cond.b) { // and
							return getItemReq (cond.a).concat (getItemReq (cond.b));
						} else if (cond.c && cond.d) { // or
							return getItemReq (cond.c).concat (getItemReq (cond.d));
						}
						return [];
					}
					var itemReq = [];
					if (v.cond) {
						itemReq = getItemReq (v.cond);
					}
					//console.log (v);
					return {t: p (v.t), e: v.v, cond: v.cond ? _.partial (condPass, v.cond) : null, items: itemReq};
				}), undefined);
			case "fauxChoice":
				return fauxChoice (ev.choice, key, i + 1);
			case "setFlag":
				if (isEncounterData (ev.val)) {
					return setEncData (ev.flag, ev.val);
				} else {
					return setVal (ev.flag, ev.val);
				}
			case "setEncounterData":
				return setEncounterData (ev.enc);
			case "peekLoadEncounterData":
				return setEncounterDataFrom (ev.flag);
			case "loadAllSubData":
				return loadAllSubData (ev.flag, ev.allowed);
			case "popToSubData":
				return popToSubdata (ev.flag, ev.allowed);
			case "inlineBranch":
				var pass = new Event();
				var fail = new Event();
				// FIXME: because "_inline" doesn't actually exist, an inline branch will break if it contains a fauxchoice (since it can't resume execution)
				pass.chunks = _.map (ev.pass, _.partial (parseJSONEvent, "_inline"));
				fail.chunks = _.map (ev.fail, _.partial (parseJSONEvent, "_inline"));
				return inlineBranch (_.partial (condPass, ev.cond), pass, fail);
			case "jump":
				return jump (ev.target);
			case "challenge":
				//console.log (ev.opponent);
				ev.opponent.stats = new Stats.Stats (ev.opponent.stats);
				switch (ev.opponent.ai.type) {
					case "random":
						ev.opponent.ai = _.partial (randomAI, ev.opponent.ai.actions);
						break;
					default:
						console.error ("can't parse opponent ai type of \"" + ev.opponent.ai.type + "\"");
				}
				return challenge (ev.opponent, ev.drops, ev.opponent.kind, key, i + 1);
			case "branch":
				return branch (ev.conds, ev.default_);
			case "shop":
				return shop (ev.verb, _.map (ev.sales, function (v) {
					var sale;
					if (typeof v.sale === "string") {
						sale = v.sale;
					} else {
						sale = new Item.Item (v.sale.item, v.sale.count);
					}
					return new Alchemy (_.map (v.cost, function (c) {
						if (typeof c === "string") {
							return c;
						}
						return new Item.Item (c.item, c.count);
					}), sale);
				}));
			case "harvestNode":
				return harvestNode (ev.flag, ev.text, ev.items);
			case "close":
				return close ();
			case "deactivateSelf":
				return deactivate ();
			case "bodyEditor":
				return bodyEditor ();
		}
		console.error ("couldn't parse event named \"" + ev.type + "\"");
		return 0;
	}

	function condPass (cond, page, player) {
		return parseCond (cond) (page, player);
	}



	function parseCond (cond) {
		//console.log ("running condition:");
		//console.log (cond);
		if (cond.flag == "__hasitem") { // item condition
			return function (page, player) {
				var items = _.map (cond.items, function (v) {
					return new Item.Item (v.item, v.count);
				});
				// this isn't the most efficient traversal, but given the n involved it should be fine.
				return items.reduce (function (i, acc) {
					return acc && player.inventory.hasItem (i.item, i.count);
				}, true);
			}
		} else if (cond.flag) { // variable condition
			return function (page, player) {
					var pflag = parseFlag (cond.flag, page, player);
					//console.log ("flag currently set to: " + pflag + "; condition result: " + parseOp (cond.cond, pflag));
					return parseOp (cond.cond, pflag);
				}
		} else if (cond.a && cond.b) { // and
			return function (page, player) {
				//console.log ("parsed value: " + (parseCond (cond.a)(page, player) && parseCond (cond.b)(page, player)));
				return parseCond (cond.a)(page, player) && parseCond (cond.b)(page, player);
			}
		} else if (cond.c && cond.d) { // or
			return function (page, player) {
				//console.log ("parsed value: " + (parseCond (cond.a)(page, player) || parseCond (cond.b)(page, player)));
				return parseCond (cond.c)(page, player) || parseCond (cond.d)(page, player);
			}
		}
		return undefined;
	}
	function parseFlag (flag, page, player) {
		if (flag[0] === "&") {
			return player.getVar (flag.slice(1));
		} else {
			return page.getVar (flag);
		}
	}
	function parseOp (op, val) {
		switch (op.op) {
			case "set":
				return (val !== undefined);
			case "notSet":
				return (val === undefined);
			case "eq":
				//console.log ("condition: EQ, val:" + op.val);
				return (val === op.val);
			case "neq":
				return (val !== op.val);
			case "lt":
				//console.log ("condition: LT, val:" + op.val);
				return (val|0 < op.val|0);
			case "gt":
				//console.log ("condition: GT, val:" + op.val);
				return (val|0 > op.val|0);
			// does this value contain encounter data, and is the final encounter data a match for the requested encounter type
			case "peek":
				return (Array.isArray (val)
					&& val.length > 0
					&& val[val.length-1].name === op.val);
		}
	}

	function parseJSONEventPage (json) {
		var eventIndex = {};
		var page;
		if (json.error) {
			console.error ("event failed parsing, \"" + json.error + "\"");
			// this isn't as slick as the actual server error pages but it'll have to do :V
			eventIndex["_activate"] = new Event (
				narration (["ERROR: \"" + json.error + "\"."]));
			page = new EventPage (function () {}, eventIndex, false);
			page.setVar ("_title", "!! ERROR !!");
			return page;
		}
		_.map (json.events, function (ev) {
			var parsedEvents = [];
			var key = ev[0];
			var evs = ev[1];
			// we're manually constructing this event b/c it turns out it's rather difficult to call a variadic function w/ an array of items to use as args when that function is an object constructor.
			var eventChunk = new Event();
			eventChunk.chunks = _.map (evs, _.partial (parseJSONEvent, key));
			eventIndex[key] = eventChunk;
		});
		/*
		console.log ("constructed event index:");
		console.log (eventIndex);
		console.log ("title: " + json.title);
		*/
		page = new EventPage (function () {}, eventIndex, false);
		page.setVar ("_title", json.title);
		var body = null;
		if (json.body) {
			function buildFauxBody (node, chs) {
				var b = new Body.Part (node, []);
				chs.map (function (c) {
					b.addChild(buildFauxBody (c[0], c[1]));
				});
				return b;
			};
			console.log ("got json body data:");
			console.log (json.body);
			body = buildFauxBody (json.body[0], json.body[1]);
		}
		page.body = body;
		return page;
	}

	function narration (texts) {
		return new Chunk (function (where, page, player, encounter) {
			_.map (stringmarkup (page, player, texts), function (t) {
				where.appendChild (t);
			});
			return true;
		});
	}
	function dialog (img, name, texts) {
		return new Chunk (function (where, page, player, encounter) {
			var namestr, textElements;
			// ah yes javascript polymorphism
			// wait is namestr actually used anywhere???
			if (typeof name === "function") {
				namestr = name(page) + "";
			} else if (typeof name === "string") {
				namestr = name;
			} else {
				namestr = stringmarkup (page, player, [name])[0];
			}
			textElements = stringmarkup (page, player, texts);
			where.appendChild (dialogFrame ("monstro", img, namestr, textElements));
			return true;
		});
	}
	function itemModify (is) {
		return new Chunk (function (where, page, player, encounter) {
			var i, p, cont;
			for (i = 0; i < is.length; i++) {
				//giveItemOrNode (is[i], player.inventory, player.nodeInventory);
				player.pending.push (is[i]);
				p = document.createElement ("p");
				p.classList.add ("itemModify");
				cont = document.createElement ("span");
				if (is[i] instanceof Item.Item) {
					if (is[i].count < 0) {
						p.appendChild (document.createTextNode ("You lost "));
						is[i].count *= -1;
					} else {
						p.appendChild (document.createTextNode ("You got "));
					}
					cont.appendChild (is[i].dom());
					p.appendChild (cont);
				} else if (typeof is[i] == "string") {
					p.appendChild (document.createTextNode ("You got "));
					cont.classList.add ("node");
					cont.appendChild (document.createTextNode ("[" + is[i].toUpperCase() + "]"));
					p.appendChild (cont);
				}
				where.appendChild (p);
			}
			return true;
		});
	}
	function pushNode (action, path) {
		return new Chunk (function (where, page, player, encounter) {
			var p;
			var immediateParent;
			var pathDOM;
			var removed;
			p = document.createElement ("p");
			p.classList.add ("pushNode");

			pathDOM = path.map (function (node) {
				return new Body.Part (node, []).asStatic();
			});

			if (path.length > 0 && path[0] === "animum") {
				// the node followPath functions expect a list of children, so remove the animum at the top.
				path = path.slice(1);
			}

			if (action === "createPath") {
				player.body.followOrAddPath (path);

				pathDOM.map (function (d) { p.appendChild (d); });
				p.appendChild (document.createTextNode (" was added to your body."));
			} else {
				immediateParent = player.body.followPath (path.slice (0, -1));
				if (immediateParent !== null && action.overwrite) {
					removed = immediateParent.replaceChild (action.overwrite, path[path.length-1]);

					if (removed) {
						p.appendChild (document.createTextNode ("Your "));
						pathDOM.slice(0, -1).map (function (d) { p.appendChild (d); });
						p.appendChild (removed.asStatic());
						p.appendChild (document.createTextNode (" was transmuted into "));
						pathDOM.slice(-1).map (function (d) { p.appendChild (d); });
					} else {
						p.appendChild (document.createTextNode ("?! no such node in body to overwrite ("
							+ path
								.map(function (n) {return n.value;})
								.reduce (function (a, b) {return a + ", " + b;})
							+ ")"));
					}
				} else if (immediateParent !== null && action.replace) {
					removed = immediateParent.replaceChild (action.replace, path[path.length-1]);

					if (removed) {
						player.nodeInventory.push (removed.value);

						p.appendChild (document.createTextNode ("Your "));
						pathDOM.slice(0, -1).map (function (d) { p.appendChild (d); });
						p.appendChild (removed.asStatic());
						p.appendChild (document.createTextNode (" was replaced with "));
						pathDOM.slice(-1).map (function (d) { p.appendChild (d); });
						p.appendChild (document.createTextNode (" ("));
						p.appendChild (removed.asStatic());
						p.appendChild (document.createTextNode (" unequipped)"));
					} else {
						p.appendChild (document.createTextNode ("?! no such node in body to replace ("
							+ path
								.map(function (n) {return n.value;})
								.reduce (function (a, b) {return a + ", " + b;})
							+ ")"));
					}
				}
			}

			where.appendChild (p);
			return true;
		});
	}
	function nodeModify (n, state) {
		return new Chunk (function (where, page, player, encounter) {
			var p, i;
			p = document.createElement ("p");
			p.classList.add ("nodeModify");
			if (state === "give") {
				player.nodeInventory.push (n);
				p.appendChild (document.createTextNode ("You received "));
				p.appendChild (new Body.Part (n, []).asStatic());
			} else if (state === "take") {
				removeFirstMatch (player.nodeInventory, n);
				p.appendChild (document.createTextNode ("You lost "));
				p.appendChild (new Body.Part (n, []).asStatic());
			} else {
				console.error ("nodeModify: don't know what's the deal with \"" + state + "\".");
			}
			where.appendChild (p);
			return true;
		});
	}
	function choices (cs, pickCallback) {
		return new Chunk (function (where, page, player, encounter) {
			// needs to be named b/c we need to remove it :(
			function clickTrigger () {
				_.map (document.querySelectorAll (".activeChoice"), function (ch) {
					ch.classList.remove ("activeChoice");
					ch.classList.add ("closedChoice");
					ch.removeEventListener ('click', clickTrigger);
				});
				this.classList.add ("selectedChoice");
				if (pickCallback !== undefined) {
					pickCallback ((this.dataset.choice|0), page, player, encounter);
				}
				page.runLocal (where, this.dataset.evName, player, encounter);
			}
			var i, j, list, item, a, domBlock;
			list = document.createElement ("ul");
			// not mapping here b/c we need the choice index (for picking the right callback later)
			for (i = 0; i < cs.length; i++) {
				// TODO: insert visible item req dom box here, if the choice has an item requirement. also, if the condition fails but contains an item requirement, show the choice, disabled, with a strikethrough and the item requirement in red.
				if (!cs[i].cond || cs[i].cond(page, player)) { // no req or passed req
					item = document.createElement ("li");
					a = document.createElement ("a");
					for (j = 0; j < cs[i].t.length; j++) {
						// we need to clone it, b/c otherwise if we redisplay a choice we'll grab the nodes from higher up on the event reel. (since we precomputed the DOM blocks and are just shoving them into the page from the same stored list over and over as choices are presented.)
						a.appendChild (cs[i].t[j].cloneNode(true));
					}
					a.classList.add ("activeChoice");
					a.dataset.evName = cs[i].e;
					a.dataset.choice = i;
					a.addEventListener ('click', clickTrigger);
					item.appendChild (a);
					list.appendChild (item);
				}
			}
			where.appendChild (dialogFrame ("self", "/static/img/pcicon.png", undefined, [list]));
			return false;
		});
	}
	function fauxChoice (text, target, index, pickCallback) {
		// also hax
		if (text === "__nameboxcallback") {
			text = "==>";
			pickCallback = function (selected, page, player, encounter) {
				var name = document.getElementById ("nameBox");
				if (name.value.trim() === "") {
					name.value = "No-name";
				}
				player.name = name.value;
				name.disabled = true;
			};
		}
		return new Chunk (function (where, page, player, encounter) {
			function clickTrigger () {
				this.classList.remove ("activeChoice");
				this.classList.add ("closedChoice");
				this.removeEventListener ('click', clickTrigger);
				this.classList.add ("selectedChoice");
				if (pickCallback !== undefined) {
					pickCallback (0, page, player);
				}
				page.runLocalFrom (where, target, index, player, encounter);
			}
			var list, item, a;
			list = document.createElement ("ul");
			item = document.createElement ("li");
			a = document.createElement ("a");
			a.appendChild (document.createTextNode (text));
			a.classList.add ("activeChoice");
			a.addEventListener ('click', clickTrigger);
			item.appendChild (a);
			list.appendChild (item);
			where.appendChild (dialogFrame ("self", "/static/img/pcicon.png", undefined, [list]));
			return -1;
		});
	}
	function challenge (opponent, drops, kind, target, index) {
		function challengeButton () {
			var button = document.createElement ("button");
			button.classList.add ("challenge");
			button.appendChild (document.createTextNode (kind));
			return button;
		}
		return new Chunk (function (where, page, player, encounter) {
			function initCombat () {
				this.disabled = true;
				var challenge = new Challenge.Encounter();
				challenge.setOpponent (opponent);
				// copy/clone the stat block so it doesn't end up modifying the pc's real hp until we want it to
				challenge.setOwnStats (player.name, JSON.parse (JSON.stringify (player.stats)));
				challenge.setPcActions (player.skills.map (function (skill) { return skill.forChallenge (); }));

				/* we also need to figure out:
					* the victory drops screen (show drops, add drops to inventory, have a close button that takes the pc to the event screen, resume running whatever event)
						(`drops` is a list of lists of items, & yr expected to pick one of those lists and give them to the pc)
					* a way to undo event vars & shop changes
						* potentially just cache those changes somewhere and commit them when the event closes
				*/
				// just resume the event silently and without note
				challenge.victoryCallback (function (challengeData) {
					var victoryDom = document.querySelector ("#victory");
					var victoryContents = victoryDom.querySelector (".contents");
					//console.log (challengeData);
					player.stats.hp.cur = challengeData.pcStats.hp.cur;
					player.skills.map (function (skill) {
						var timesUsed = challengeData.pcActionsTaken[skill.name] ? challengeData.pcActionsTaken[skill.name] : 0;
						return timesUsed ? skill.gainXp (timesUsed * 4) : 0;
					});
					//console.log (challengeData.pcActionsTaken);
					var b = document.createElement ("button");
					clearDOM (victoryContents);
					// TODO:
					// if the encounter has drops: pick a drop, show, add to inventory
					// show skill gained from use, including any level ups. right now maybe just have a "level up skill" button to spend a useless skill level point :V
					b.appendChild (document.createTextNode ("continue"));
					victoryContents.appendChild (b);
					b.addEventListener ("click", function () {
						wipeReveal (document.querySelector ("#scene"), function () {
							page.runLocalFrom (where, target, index, player, encounter);
						});
					});
					wipeReveal (victoryDom, function () {
						// ...
					});
				});

				challenge.failureCallback (function (challengeData) {
					// FIXME: these don't reset encounter vars to their starting values. this could cause weird divergent events when resetting an encounter. also, if there are multiple combat events in a row, skills are committed at the end of each combat and thus rewinding at the 2nd or later combat event would mean 'free' skill growth
					function makeFailureList () {
						var list = document.createElement ("ul");
						var li, a;
						list.classList.add ("failure");
						li = document.createElement ("li");
						a = document.createElement ("a");
						a.appendChild (document.createTextNode ("rewind to combat start"));
						a.addEventListener ("click", function () {
							initCombat();
						});
						li.appendChild (a);
						list.appendChild (li);

						li = document.createElement ("li");
						a = document.createElement ("a");
						a.appendChild (document.createTextNode ("rewind to encounter start"));
						// FIXME: this retains event state changes and shop purchases made to-this-point in this event. the shop thing isn't a big deal, but the event change is.
						a.addEventListener ("click", function () {
							player.wipePendingItems ();
							runEvent (page);
						});
						li.appendChild (a);
						list.appendChild (li);

						li = document.createElement ("li");
						a = document.createElement ("a");
						a.appendChild (document.createTextNode ("rewind to map"));
						// FIXME: this also retains event state changes and shop purchases made to-this-point in this event! here, since the entire event is supposed to be undone, the shop changes ARE a big deal.
						a.addEventListener ("click", function () {
							player.wipePendingItems ();
							closeFunc (page);
						});
						li.appendChild (a);
						list.appendChild (li);
						// TODO: linebreak and then "load game"

						return list;
					}
					var resultsScreen = document.querySelector ("#victory");
					var resultsText = resultsScreen.querySelector (".contents");
					var resultHeader = resultsScreen.querySelector (".header h2");
					clearDOM (resultsText);
					clearDOM (resultHeader);
					resultHeader.appendChild (document.createTextNode ("FAILURE"));

					resultsText.appendChild (makeFailureList ());

					wipeReveal (resultsScreen);
				});
				challenge.begin ();
				wipeReveal (document.querySelector ("#challenge"));
			}

			var challengeDom = challengeButton ();
			challengeDom.addEventListener ('click', function () {
				this.disabled = true;
				initCombat();
			});
			where.appendChild (challengeDom);
			return -1;
		});
	}
	function jump (target) {
		return new Chunk (function (where, page, player, encounter) {
			return page.runLocal (where, target, player, encounter);
		});
	}
	function inlineBranch (cond, pass, fail) {
		return new Chunk (function (where, page, player, encounter) {
			if (cond (page, player)) {
				return pass.run (where, page, player, encounter, true);
			} else {
				return fail.run (where, page, player, encounter, true);
			}
		});
	}
	function isEncounterData (value) {
		return (typeof value === "object") && ("name" in value) && ("data" in value);
	}
	function setEncData (var_, value) {
		return new Chunk (function (where, page, player, encounter) {
			var success = false;
			if (var_[0] === "&") {
				success = player.pushEncounterData (var_.slice(1), value);
			} else {
				success = page.pushEncounterData (var_, value);
			}
			if (!success) {
				runtimeError (where, "wanted to push encounter data (for \"" + value.name + "\") into value " + var_ + " but it was already in use by something else.");
			}
		});
	}
	function setVal (var_, value) {
		return new Chunk (function (where, page, player, encounter) {
			if (var_[0] === "&") {
				player.setVar (var_.slice(1), value);
			} else {
				page.setVar (var_, value);
			}
			return true;
		});
	}
	// FIXME: this can't be rewound if the event is cancelled (via a later challenge failure or w/e); that means there might be encounter weirdness sometimes.
	function setEncounterData (enc) {
		return new Chunk (function (where, page, player, encounter) {
			if (encounter.name !== enc.name) {
				console.log (encounter);
				console.log (page);
				runtimeError (where, "got '" + enc.name + "' encounter data when we were expecting '" + encounter.name + "'");
			} else {
				encounter.data = enc.data;
				encounter.subdata = enc.subdata;
			}
		});
	}
	function setEncounterDataFrom (flag) {
		return new Chunk (function (where, page, player, encounter) {
			var pval = parseFlag (flag, page, player);
			if (Array.isArray(pval) && pval.length > 0) {
				setEncounterData(pval[pval.length-1]).run(where, page, player, encounter);
			} else {
				runtimeError (where, "got invalid stored encounter data from flag \"" + flag + "\" when we were expecting");
			}
		});
	}
	function loadAllSubData (flag, allowed) {
		return new Chunk (function (where, page, player, encounter) {
			var pval = parseFlag (flag, page, player);
			if (pval === null || pval === undefined) {
				console.error ("can't load subdata from flag \"" + flag + "\"; not set.");
				return;
			}
			if (!("subdata" in encounter)) {
				encounter.subdata = [];
			}
			if (Array.isArray(pval)) {
				pval.forEach (function (enc) {
					if (allowed.indexOf (enc.name) !== -1) {
						encounter.subdata.push (enc);
					} else {
						runtimeError (where, "(in " + flag + ") got encounter data for '" + enc.name + "' when we could only use" + allowed);
					}
				});
			} else {
				runtimeError (where, "got bad value (" + pval + ") when trying to fetch subdata from \"" + flag + "\"");
			}
		});
	}
	function popToSubdata (flag, allowed) {
		return new Chunk (function (where, page, player, encounter) {
			var pval = parseFlag (flag, page, player);
			if (pval === null || pval === undefined) {
				console.error ("can't pop subdata from flag \"" + flag + "\"; not set.");
				return;
			}
			if (Array.isArray(pval) && pval.length > 0) {
				if (allowed.indexOf (pval[pval.length-1].name) !== -1) {
					var newSub = pval.pop();
					if (!("subdata" in encounter)) {
						encounter.subdata = [];
					}
					encounter.subdata.push (newSub);
				} else {
					runtimeError (where, "(in " + flag + ") got encounter data for '" + enc.name + "' when we could only use" + allowed);
				}
			} else {
				runtimeError (where, "got bad value (" + pval + ") when trying to fetch subdata from \"" + flag + "\"");
			}
		});
	}
	function runtimeError (where, errormsg) {
		console.error ("RUNTIME ERROR!!! " + errormsg);
		narration([errormsg + "; plz tell xax about this (and give them a copy of yr save data / tell them how this happened)"]).run(where, null, null, null);
	}
	function branch (conds, default_) {
		return new Chunk (function (where, page, player, encounter) {
			var scene = "";
			for (var i = 0; i < conds.length; i++) {
				if (parseCond (conds[i].cond) (page, player)) {
					scene = conds[i].branch;
					break;
				}
			}
			if (scene === "") { scene = default_; }
			return page.runLocal (where, scene, player, encounter);
		});
	}
	function shop (verb, sales) {
		return new Chunk (function (where, page, player, encounter) {
			var s, t, sc;
			function initSales (saleContainer) {
				clearDOM (saleContainer);
				// we need to make an index of these functions because otherwise we can't remove them from the sale slots, because `removeEventListener` needs the exact same function object as the one used in the corresponding `addEventListener` call
				var buyCallbacks = _.map (sales, function (s) {
					return _.partial (buyItem, s);
				});
				function resetSale (dom, i) {
					var alch = sales[i];
					var saleDom = dom.querySelector (".sale");
					var costDom = dom.querySelector (".cost");
					var missing = missingCosts (alch.cost, player.inventory, player.nodeInventory);
					var canBuy = missing.length === 0;
					var costs = dom.querySelectorAll (".cost > span > span");
					var saleButton = saleDom.querySelector ("button");
					var buyCallback = buyCallbacks[i];
					_.map (costs, function (costDom, i) {
						if (!canProvide (alch.cost[i], player.inventory, player.nodeInventory)) {
							costDom.classList.add ("insufficient");
						} else {
							costDom.classList.remove ("insufficient");
						}
					});
					// always remove, because if it's already set and we're gonna readd it we really don't want to double-up on listeners
					saleButton.removeEventListener ('click', buyCallback);
					saleButton.disabled = true;
					if (canBuy) {
						saleButton.addEventListener ('click', buyCallback);
						saleButton.disabled = false;
					}
				}
				function buyItem (alch) {
					_.map (alch.cost, function (sale) {
						removeItemOrNode (sale, player.inventory, player.nodeInventory);
					});
					giveItemOrNode (alch.sale, player.inventory, player.nodeInventory);
					// update the sale data (& yr inventory display) now that you've bought a thing
					// we could just rerun initSales here, but that would destroy the css transition we just started by clicking on an active buy button
					_.map (saleContainer.children, resetSale);
				}

				_.map (sales, function (alch, index) {
					var alchDom = alch.dom (player.inventory, player.nodeInventory);
					resetSale (alchDom, index);
					saleContainer.appendChild (alchDom);
				});
			}
			s = document.createElement ("section");
			s.classList.add ("shop");
			t = document.createElement ("header");
			t.appendChild (document.createTextNode (verb));
			s.appendChild (t);
			sc = document.createElement ("ul");
			initSales (sc);
			s.appendChild (sc);
			where.appendChild (s);
			return true;
		});
	}
	function closeFunc (page) {
		var mapPane = document.querySelector ("#map");
		if (page.closeCallback) {
			page.closeCallback(page);
		}
		wipeReveal (mapPane, null);
	}
	function harvestNode (controlVar, blurb, itemsDeck) {
		return new Chunk (function (where, page, player, encounter) {
			// make blurb text and harvest button, and put them in wrapper
			var blurbCont = document.createElement ("span");
			var harvestButton = document.createElement ("button");
			var wrapper = document.createElement ("div");
			var ready;
			var now;
			var i;
			var respawnNotice;
			wrapper.classList.add ("harvestNode");
			blurbCont.appendChild (document.createTextNode (blurb));
			harvestButton.appendChild (document.createTextNode ("harvest"));
			wrapper.appendChild (blurbCont);
			wrapper.appendChild (harvestButton);

			// get control var.
			ready = parseFlag (controlVar, page, player);
			now = player.currentTurnCount();
			// if unset or set to a value < current time (by checking player.currentTurnCount)
			if (ready === undefined || ready < now) {
				// then set callback to harvest button.
				harvestButton.addEventListener ('click', function (e) {
					// disable button
					harvestButton.disabled = true;
					// pick a random set of items from itemsDeck; give those to the pc
					i = Math.floor (Math.random() * itemsDeck.length);
					console.log ("adding harvested items:");
					console.log (itemsDeck[i]);
					itemModify (itemsDeck[i].map (function (v) {
						return new Item.Item (v.item, v.count);
					})).run (where, page, player, encounter);
					// set encounter's local variable to the next respawn time
					setVal (controlVar, nextRespawnTime(now)).run (where, page, player, encounter);
				});
			} else {
				// otherwise, set harvest button disabled
				harvestButton.disabled = true;
				// and tell how many turns to respawn
				respawnNotice = document.createElement ("span");
				respawnNotice.classList.add ("respawn");
				respawnNotice.appendChild (document.createTextNode ("(respawn in " + (ready - now) + " turns)"));
				wrapper.appendChild (respawnNotice);
			}
			// attach wrapper to where
			where.appendChild (wrapper);
			return true;
		});
	}
	function close () {
		return new Chunk (function (where, page, player, encounter) {
			var closeButton;
			document.querySelector ("#scene .toMap").style.display = "block";
			closeButton = document.createElement ("button");
			closeButton.classList.add ("toMap");
			closeButton.appendChild (document.createTextNode ("CLOSE"));
			closeButton.addEventListener ('click', function (e) { closeFunc (page); });
			where.appendChild (closeButton);
			return true;
		});
	}
	function deactivate () {
		return new Chunk (function (where, page, player, encounter) {
			page.destroy = true;
			return true;
		});
	}
	function bodyEditor () {
		return new Chunk (function (where, page, player, encounter) {
			var promptButton, editPane;
			promptButton = document.createElement ("button");
			promptButton.classList.add ("toEditor");
			promptButton.appendChild (document.createTextNode ("EDIT BODY"));
			editPane = document.querySelector ("#editor");
			promptButton.addEventListener ('click', _.partial (wipeReveal, editPane, function () {
				// steal the desc dom and place it into the editor (just so we don't have to duplicate all its update/pronoun/etc stuff & also make all them parametric on which description display we're updating, & keep the two synced, etc, etc
				// the close button for the editor should re-grab the description and shuffle it back over to the stats page.
				var desc = document.querySelector ("#description");
				var descriptionFrame = document.querySelector ("#descriptionText");
				var editorTarget = document.querySelector ("#descEditorMarker");
				editorTarget.parentNode.insertBefore (desc, editorTarget);

				// now actually set up the editing interface:
				var builder = document.querySelector ("#builder");
				var tree = document.querySelector ("#treeBuilder");
				// i can call updateDescription here b/c javascript scoping is wild. but it's probably a bad idea.
				Editor.updateDescription (descriptionFrame);
				clearDOM (builder);
				let inventoryCheck = function (itemName) {
					console.log (itemName);
					console.log (player.inventory);
					return player.inventory.hasItem (itemName, 1);
				}
				let inventorySpend = function (itemName) {
					player.inventory.giveItem (itemName, -1);
				}
				var rerender = function (thing) {
						clearDOM (builder);
						builder.appendChild (thing);
						clearDOM (tree);
						tree.appendChild (Editor.renderBodyTree (player.body, function (node) {
							Editor.bindSelectedNode (node);
							rerender (Editor.renderBodyBreadcrumb (player.body, player.nodeInventory, inventoryCheck, inventorySpend, rerender));
						}));
					};
				builder.appendChild (Editor.renderBodyBreadcrumb (player.body, player.nodeInventory, inventoryCheck, inventorySpend, rerender));
				clearDOM (tree);
				tree.appendChild (Editor.renderBodyTree (player.body, function (node) {
						Editor.bindSelectedNode (node);
						rerender (Editor.renderBodyBreadcrumb (player.body, player.nodeInventory, inventoryCheck, inventorySpend, rerender));
				}));

				/*
				// now actually set up the editing interface:
				var builder = document.querySelector ("#builder");
				// i can call updateDescription here b/c javascript scoping is wild. but it's probably a bad idea.
				Editor.updateDescription (descriptionFrame);
				clearDOM (builder);
				builder.appendChild (Editor.renderBodyTree (player.body, "#parts"));
				//Editor.renderNodeCollection ("#parts", player.nodeInventory, true);
				*/
			}));
			where.appendChild (promptButton);
			return true;
		});
	}

	return (
		{ EventPage: EventPage
		, Event: Event
		, Alchemy: Alchemy
		, parseJSON: parseJSONEventPage
		, parseJSONEvent: parseJSONEvent
		, narration: narration
		, dialog: dialog
		, shop: shop
		, itemModify: itemModify
		, nodeModify: nodeModify
		, pushEncounterData: setEncData
		, loadAllSubData: loadAllSubData
		, popToSubData: popToSubdata
		, setVal: setVal
		, choices: choices
		, fauxChoice: fauxChoice
		, branch: branch
		, jump: jump
		, close: close
		, bodyEditor: bodyEditor
		});
})();

