var Hubur_A = (function() {
	function generatorInit () {
	}

	function generateRandomSection (sectionDeck, fieldDeck) {
		var s = {};
		s.flags = [];
		s.type = sectionDeck.pull();

		s.fieldType = s.type === "shallow" ? "solid"
			: s.type === "empty" ? "islands"
			: fieldDeck.pull();
		s.rCount = randomIntInRange (2, 7);
		if (s.fieldType === "solid") {
			s.rCount += 3;
		}
		if (s.type === "badland") {
			s.rCount += 1;
		}
		if (s.type === "empty") {
			s.rCount = 0;
		}

		if (s.type !== "empty") {
			s.flags.push (s.type);
		}
		s.flags.push (s.fieldType);

		s.edges = [];
		return s;
	}

	function sectionGenerator (constants) {
		var sectionDeck = new Deck (["field", "field", "badland", "badland", "factory", "shallow", "swamp", "glade"/*, "empty"*/]);
		var fieldDeck = new Deck (["islands", "islands", "solid", "solid", "solid"]);

		var MAX_ZONE_SIZE = 6;
		var i = 0;
		var coord;
		var prevLeftId;
		var prevLeft = null;
		var prevUpId;
		var prevUp = null;
		var r = 0;

		var zoneGraph;
		var zones;
		var chunks;

		var arr = Array (constants.SECTION_SIZE);
		fillArray (arr, null);

		var v = makeChunkZones (constants.SECTION_DIMENSION, MAX_ZONE_SIZE);

		zones = v.zones;
		chunks = v.chunks;
		zoneGraph = v.graph;

		zones.map (function (zone, i) {
			var section = generateRandomSection (sectionDeck, fieldDeck);
			zone.calcSpec (constants.SECTION_DIMENSION);
			//console.log (i + ": " + chunks[i].subs);
			//console.log ("cloned:");
			zone.subs.map (function (j) {
				arr[j] = clone (section);
				arr[j].edges = []; // lol more spooky action at a distance
				arr[j].root = zone;
				arr[j].id = j;
				if (zone.spec === j) {
					switch (section.type) {
						case "glade":
						case "badland":
							arr[j].spec = "standing_stones";
							break;
						case "swamp":
							arr[j].spec = "skull";
							break;
						case "shallow":
							arr[j].spec = "vortex";
							break;
						case "field":
							arr[j].spec = "farm";
							break;
						default:
							arr[j].spec = null;
							break;
					}
				}
			});
		});

		// since object-placing is so dumb currently there's still no guarantee that the gate will be placed.
		var hasGate = randomFrom (arr.filter(function (s, i) {
			return s.type === "badland" && s.root.spec !== i;
		}));
		if (!hasGate) {
			console.error ("hasGate didn't find a match?!");
			console.error ("arr looks like:");
			console.error (arr);
		} else {
			hasGate.spec = "gate";
		}

		function getAdj (x, y) {
			return (x >= 0 && x < constants.SECTION_DIMENSION
					&& y >= 0 && y < constants.SECTION_DIMENSION
				? arr[linearize (constants.SECTION_DIMENSION, x, y)]
				: null);
		}
		arr.map (function (s, i) {
			var c = unlinearize (constants.SECTION_DIMENSION, i);
			var adjDownLeftId = linearize (constants.SECTION_DIMENSION, c.x, c.y+1);
			var adjDownLeft = getAdj (c.x, c.y+1);
			var adjDownRightId = linearize (constants.SECTION_DIMENSION, c.x+1, c.y);
			var adjDownRight = getAdj (c.x+1, c.y);
			var MIN_SIZE = 3;
			var EDGE_SIZE = 3;
			var BRIDGE_LENGTH = 1;
			var start;
			var length;
			var edge;
			var type;
			var makeIslands;

			function zoneEdgeHere (n, m, a, b) {
				return zoneGraph.edgesBetween (zoneGraph.getNodeId (n), zoneGraph.getNodeId (m)).filter (function (e) {
					return (e.label.a === a && e.label.b === b) || (e.label.a === b && e.label.b === a);
				}).length > 0;
			}

			if (adjDownLeft && (adjDownLeft.root.id === s.root.id
						|| zoneEdgeHere (s.root, adjDownLeft.root, i, adjDownLeftId))) {
				if (s.fieldType === "solid" && adjDownLeft.fieldType === "solid") {
					start = randomIntInRange (0, constants.DIMENSION - MIN_SIZE - constants.BUFFER) + constants.BUFFER;
					length = randomIntInRange (MIN_SIZE, constants.DIMENSION - start);
					type = (["factory", "swamp"].indexOf (s.type) !== -1) ? s.type : adjDownLeft.type;
					edge =
						{ start: new Point2d (start, constants.FULL_DIMENSION - EDGE_SIZE)
						, size: new Point2d (length, EDGE_SIZE)
						, type: type
						};
					s.edges.push (edge);
					type = (["factory", "swamp"].indexOf (adjDownLeft.type) !== -1) ? adjDownLeft.type : s.type;
					edge =
						{ start: new Point2d (start, 0)
						, size: new Point2d (length, EDGE_SIZE)
						, type: type
						}
					adjDownLeft.edges.push (edge);
				} else {
					type = adjDownLeft.root.id === s.root.id ? s.type
						: s.type === "shallow" || adjDownLeft.type === "shallow" ? "shallow_connection"
						: s.type === "factory" && adjDownLeft.type === "factory" ? "factory_bridge"
						: "bridge";
					makeIslands = adjDownLeft.root.id === s.root.id;
					start = randomIntInRange (0, constants.DIMENSION - 2) + constants.BUFFER + 1;
					edge =
						{ start: new Point2d (start, constants.FULL_DIMENSION - BRIDGE_LENGTH)
						, size: new Point2d (makeIslands ? 2 : 1, BRIDGE_LENGTH+1)
						, type: type
						};
					s.edges.push (edge);
					edge =
						{ start: new Point2d (start, -1)
						, size: new Point2d (makeIslands ? 2 : 1, BRIDGE_LENGTH+1)
						, type: type
						};
					adjDownLeft.edges.push (edge);
				}
			}
			if (adjDownRight && (adjDownRight.root.id === s.root.id
						|| zoneEdgeHere (s.root, adjDownRight.root, i, adjDownRightId))) {
				if (s.fieldType === "solid" && adjDownRight.fieldType === "solid") {
					start = randomIntInRange (0, constants.DIMENSION - MIN_SIZE - constants.BUFFER) + constants.BUFFER;
					length = randomIntInRange (MIN_SIZE, constants.DIMENSION - start);
					type = (["factory", "swamp"].indexOf (s.type) !== -1) ? s.type : adjDownRight.type;
					edge =
						{ start: new Point2d (constants.FULL_DIMENSION - EDGE_SIZE, start)
						, size: new Point2d (EDGE_SIZE, length)
						, type: type
						};
					s.edges.push (edge);
					type = (["factory", "swamp"].indexOf (adjDownRight.type) !== -1) ? adjDownRight.type : s.type;
					edge =
						{ start: new Point2d (0, start)
						, size: new Point2d (EDGE_SIZE, length)
						, type: type
						}
					adjDownRight.edges.push (edge);
				} else {
					type = adjDownRight.root.id === s.root.id ? s.type
						: s.type === "shallow" || adjDownRight.type === "shallow" ? "shallow_connection"
						: s.type === "factory" && adjDownRight.type === "factory" ? "factory_bridge"
						: "bridge";
					makeIslands = adjDownRight.root.id === s.root.id;
					start = randomIntInRange (0, constants.DIMENSION - 2) + constants.BUFFER + 1;
					edge =
						{ start: new Point2d (constants.FULL_DIMENSION - BRIDGE_LENGTH, start)
						, size: new Point2d (BRIDGE_LENGTH+1, makeIslands ? 2 : 1)
						, type: type
						};
					s.edges.push (edge);
					edge =
						{ start: new Point2d (-1, start)
						, size: new Point2d (BRIDGE_LENGTH+1, makeIslands ? 2 : 1)
						, type: type
						};
					adjDownRight.edges.push (edge);
				}
			}
		});
		return {array: arr, graph: zoneGraph};
	}

	function findShortestEdge (as, bs) {
		var edges = [];
		var shortest;
		for (var i = 0; i < as.length; i++) {
			for (var j = 0; j < bs.length; j++) {
				edges.push ({i: i, j: j, e: as[i].closestPoint (bs[j])});
			}
		}
		return edges.reduce (function (minE, curE) {
			// check vs. length (shorter wins)
			//console.log (min);
			//console.log (cur);
			var min = minE.e;
			var cur = curE.e;
			if (cur.result === "intersecting" || min.result === "intersecting") {
				console.log (as);
				console.log (bs);
				throw new Error (":( bad bucketing");
			}
			if (min.distance.magnitude("manhattan") > cur.distance.magnitude("manhattan")
				|| (min.distance.magnitude("manhattan") === cur.distance.magnitude("manhattan")
					&& cur.result == "edge"
					&& min.result !== "edge")) {
				return curE;
			}
			return minE;
		});
	}

	function maxConnection (a, b) {
		function reduceLabel (val) {
			switch (val) {
				case undefined:
					throw new Error ("got undefined label");
					break;
				case "swamp":
				case "shallow":
				case "field":
				case "glade":
				case "badland":
				case "empty":
					val = "internal";
					break;
				case "factory":
					val = "factory_bridge";
					break;
				default:
					break;
			}
			return val;
		}
		var types = ["factory_bridge", "internal", "bridge", "shallow_connection"];
		var aLabel;
		var bLabel;
		aLabel = reduceLabel (a);
		bLabel = reduceLabel (b);
		if (types.indexOf (aLabel) === -1) {
			throw new Error ("bad label of \"" + aLabel + "\" (a)");
		}
		if (types.indexOf (bLabel) === -1) {
			throw new Error ("bad label of \"" + bLabel + "\" (b)");
		}
		return types.indexOf (aLabel) > types.indexOf (bLabel) ? aLabel : bLabel;
	}

	function makeBridge (connection, connectionType) {
		var corner;
		var edgeMid;
		var basePt;
		var endPt;
		var cornerPivot;
		var rs = [];
		var r;
		function adjustDimensions (rect) {
			if (rect.wh.x === 0) {
				rect.wh.x = 1;
				rect.wh.y += 1;
			}
			if (rect.wh.y === 0) {
				rect.wh.x += 1;
				rect.wh.y = 1;
			}
		}
		//console.log (connection);
		corner = connection.distance.x !== 0 && connection.distance.y !== 0;
		switch (connection.result) {
			case "point":
				basePt = connection.point;
				break;
			case "edge":
				edgeMid = connection.edge;
				if (edgeMid.wh.x === 0) { // vert edge
					edgeMid = new Point2d (edgeMid.xy.x, edgeMid.xy.y + Math.floor (edgeMid.wh.y / 2));
				} else { // horiz edge
					edgeMid = new Point2d (edgeMid.xy.x + Math.floor (edgeMid.wh.x / 2), edgeMid.xy.y);
				}
				basePt = edgeMid;
				break;
			default:
				break;
		}
		// FIXME: current code: seems like it's making endpoints one too short, so they don't actually cover a tile on the other rect. or maybe this also sometimes applies to start points also? leading to a zero-size rect, which is definitely not good.
		endPt = basePt.add (connection.distance);
		if (corner) {
			cornerPivot = new Point2d (basePt.x, endPt.y);
			r = Rect.prototype.fromPoints (basePt, cornerPivot);
			adjustDimensions (r);
			if (connectionType !== undefined) {
				r.label = connectionType;
			}
			rs.push (r);
			r = Rect.prototype.fromPoints (cornerPivot, endPt);
			adjustDimensions (r);
			if (connectionType !== undefined) {
				r.label = connectionType;
			}
			rs.push (r);
		} else {
			r = Rect.prototype.fromPoints (basePt, endPt);
			adjustDimensions (r);
			if (connectionType !== undefined) {
				r.label = connectionType;
			}
			rs.push (r);
		}
		//console.log (rs);
		return rs;
	}


	var landTiles =
		{ "glade": ["ground_grass1", "ground_grass2", "ground_bushes"/*, "ground_trees"*/]
		, "field": ["ground_grass1", "ground_grass2"]
		, "badland": ["ground_mud", "ground_rock", "ground_rock", "ground_rock"]
		, "factory": ["factory_panel", "factory_grating", "factory_quads", "factory_tatami_l", "factory_tatami_r"]
		, "shallow": ["shallow_water"]
		, "beach": ["ground_sand"]
		, "swamp": ["swamp_water"]
		, "swampEdge": ["ground_grass1", "ground_grass2", "ground_mud", "ground_mud"]
		, "gladeEdge": ["ground_trees"]
		, "badlandEdge": ["water", "ground_mud"]
		, "fieldEdge": ["water", "ground_mud"]
		};
	function tileGenerator (constants, section) {
		//console.log (section);
		var s = section.raw;
		var sectionType = section.flags[0];
		var sectionField = section.flags[1];
		var currentType;
		var tiles = [];

		var MIN_RECT_SIZE = 2;
		var MAX_RECT_SIZE = 7;

		var dimension = constants.DIMENSION;
		var buffer = constants.BUFFER;
		// glades & swamps use the border tiles so they should generate smaller rects, and they have problems w/ generating at the edge and using the buffers, so they should also have dimension -= 1 and buffer += 1 (actually i guess the bigger rects are fine)
		if (sectionType === "glade" || sectionType === "swamp") {
			//MIN_RECT_SIZE -= 1;
			//MAX_RECT_SIZE -= 2;
			//dimension -= 1;
			//buffer += 1;
		} else if (sectionType === "factory") {
			MIN_RECT_SIZE = 3;
			MAX_RECT_SIZE = 8;
		} else if (sectionType === "badland") {
			MIN_RECT_SIZE = 4;
		} else if (sectionType === "shallow") {
			dimension -= 2;
			buffer += 1;
		}
		//console.log (s);

		var rs = [];
		var r;
		var bonus = [];
		var i = 0;
		var x;
		var y;
		var w;
		var h;
		for (i = 0; i < s.rCount; i++) {
			w = randomIntInRange (MIN_RECT_SIZE, MAX_RECT_SIZE);
			h = randomIntInRange (MIN_RECT_SIZE, MAX_RECT_SIZE);

			x = randomIntInRange (0, dimension - w) + buffer;
			y = randomIntInRange (0, dimension - h) + buffer;
			r = new Rect (new Point2d (x, y), new Point2d (w, h));
			r.label = sectionType;
			rs.push (r);
		}
		if (sectionField === "solid") {
			x = Math.floor (constants.FULL_DIMENSION / 4);
			y = Math.floor (constants.FULL_DIMENSION / 4);
			w = Math.floor (constants.FULL_DIMENSION / 2);
			y = Math.floor (constants.FULL_DIMENSION / 2);
			r = new Rect (new Point2d (x, y), new Point2d (w, h));
			r.label = sectionType;
			rs.push (r);
		}
		s.edges.map (function (edge) {
			var r = new Rect (edge.start, edge.size);
			r.label = edge.type;
			if (r.label === "bridge" || r.label === "shallow_connection" || r.label === "factory_bridge") {
				bonus.push (r);
			} else {
				rs.push (r);
			}
		});

		var landscape = rs.concat(bonus);

		// maybe just make this 'touches' rather than 'overlaps'. but the issue with 'touches' is that it also counts diagonal touches, so,
		var pred = function (r1, r2) {
			return r1.overlaps(r2);
		}
		var buckets = landscape.reduce (function (buckets, rect) {
			var part = partition (function (b) {
				return b.fits (rect);
			}, buckets);
			var fit = part.yes;
			var rest = part.no;
			var b = null;
			switch (fit.length) {
				case 0:
					b = new Bucket(pred);
					b.add (rect);
					break;
				case 1:
					fit[0].add (rect);
					break;
				default:
					b = fit[0].merge (fit.slice(1));
					b.add (rect);
					buckets = rest;
					break;
			}
			if (b) {
				buckets.push (b);
			}
			return buckets;
		}, []);
		if (buckets.length > 1) {
			//console.log ("chunk ... not fully connected (" + buckets.length + ")");

			var connectivity = new Graph (buckets, []);
			for (var i = 0; i < buckets.length; i++) {
				for (var j = i+1; j < buckets.length; j++) {
					var shortest = findShortestEdge (buckets[i].contents(), buckets[j].contents());
					var finalConnection = maxConnection
						( buckets[i].contents()[shortest.i].label
						, buckets[j].contents()[shortest.j].label
						);
					connectivity.addEdge (i, j, {connection: finalConnection, shortest: shortest.e});

				}
			}
			var bridgeChunks = connectivity.minimumSpanningTree (function (label) {
				return label.shortest.distance.magnitude("manhattan");
			}).map(function (edge) {
				return makeBridge (edge.label.shortest, edge.label.connection);
			});
			bridgeChunks = [].concat.apply([], bridgeChunks);
			bridgeChunks.map (function (rect) {
				// console.log ("bridge:")
				// console.log (rect);
				//rect.label = "bridge";
				return rect;
			});
			bonus = bonus.concat(bridgeChunks);
			//console.log (bonus);
		}

		var inRect;
		var justOutside;
		var p;
		var h;
		var t;
		var inForeignRect = null;
		var alongForeignRect = null;
		var specUsed = false;
		for (i = 0; i < constants.SIZE; i++) {
			y = (i / constants.FULL_DIMENSION)|0;
			x = i % constants.FULL_DIMENSION;
			currentType = s.type;
			inRect = 0;
			justOutside = false;
			inForeignRect = null;
			p = new Point2d (x, y);
			rs.map (function (r) {
				if (r.in (p)) {
					inRect += 1;
					if (r.label && r.label !== "shallow") {
						if (currentType === "shallow") {
							inForeignRect = "beach";
							currentType = r.label === "badland" ? "badland" : "field";
						} else {
							inForeignRect = r.label;
						}
					}
				}
				if (r.alongEdge (p)) {
					justOutside = true;
					if (r.label !== s.type) {
						alongForeignRect = r.label;
					}
				}
			});
			if (inRect > 0) {
				if (inForeignRect && (inForeignRect === "beach" || inForeignRect === "swamp" || randomInRange (0, 1) < 0.5)) {
					currentType = inForeignRect;
				}
				h = currentType === "factory" ? 0 //Math.floor (inRect / 3)
					: currentType === "shallow" ? -1
					: currentType === "beach" ? -1
					: currentType === "swamp" ? -1
					: currentType === "bridge" ? -1
					: inForeignRect === "beach" ? -1 // Math.floor (Math.random() * 2) - 1
					: 0; // Math.floor (inRect / 3) + Math.floor (Math.sqrt (Math.random() * 6) - 1);
				if (landTiles[currentType]) {
					t = randomFrom (landTiles[currentType]);
				} else {
					throw new Error ("bad section type of \"" + currentType + "\"")
				}
				tiles[i] = new Tile (new Height ("flat", h), t, null);
				if (tiles[i].t === "factory_grating" && randomInRange (0, 1) < 0.25) {
					tiles[i].t = "factory_pit";
				}
			} else if (justOutside) {
				switch (currentType) {
					case "swamp":
						tiles[i] = new Tile
							( new Height ("flat", 0)
							, randomFrom (landTiles["swampEdge"])
							, null //randomFrom ([null, null, "dead_tree"])
							);
						break;
					case "glade":
						tiles[i] = new Tile
							( new Height ("flat", 0) // Math.floor (Math.max (0, Math.sqrt (Math.random() * 9) - 1))
							, randomFrom (landTiles["gladeEdge"])
							, null //randomFrom ([null, null, "dead_tree"])
							);
						break;
					case "field":
					case "badland":
						tiles[i] = new Tile
							( new Height ("flat", -1)
							, randomFrom (landTiles[currentType+"Edge"])
							, null
							);
						break;
					default:
						tiles[i] = new Tile
							( null
							, null
							, null
							);
						break;
				}
				if (alongForeignRect === "swamp" && currentType !== "shallow") {
					t = currentType === "swamp" ? "ground_mud"
						: currentType === "glade" ? "ground_trees"
						: randomFrom (landTiles[currentType]);
					tiles[i] = new Tile
						( new Height ("flat", 0)
						, t
						, null //randomFrom ([null, null, "dead_tree"])
						);
				}
			} else {
				tiles[i] = new Tile
					( null
					, null
					, null
					);
			}

			var objRate = tiles[i].t === "water" && currentType !== "shallow" ? 0.05
				: currentType === "factory" ? 0.33
				: currentType === "swamp" && tiles[i].t === "swamp_water" ? 0.75
				: s.spec === "gate" && tiles[i].t === "ground_rock" ? 0.50
				: 0.125;
			if (randomInRange (0, 1) < objRate) {
				switch (tiles[i].t) {
					case "factory_grating":
					case "factory_pit":
						break;
					case "factory_panel":
					case "factory_floor":
					case "factory_quads":
					case "factory_tatami_l":
					case "factory_tatami_r":
						tiles[i].o = randomFrom (["factory_building_barracks", "factory_building_flare_small", "factory_building_machine_shop"]);
						//tiles[i].o = randomFrom (["factory_building1", "factory_building2", "factory_building3", "factory_building4"]);
						//tiles[i].o = randomFrom (["minotaur", "flame_demon", "gargoyle", "hamar_tabal", "acid_slime_purple", "ganzer_worker"]);
						break;
					case "ground_mud":
						tiles[i].o = randomFrom (["treasure", "mudman"]);
						break;
					case "ground_rock":
						if (s.spec === "gate") {
							if (!specUsed) {
								tiles[i].o = randomFrom (["gate", "hellhound", "hellhound", "hellhound"]);
								if (tiles[i].o === "gate") {
									specUsed = true;
								}
							} else {
								tiles[i].o = randomFrom ([null, "hellhound"]);
							}
						} else if (s.spec === "standing_stones" && !specUsed) {
							tiles[i].o = randomFrom (["standing_stones", "obelisk", "cave", "hellhound"]);
							if (tiles[i].o === "standing_stones") {
								specUsed = true;
							}
						} else {
							tiles[i].o = randomFrom (["obelisk", "cave", "hellhound"]);
						}
						break;
					case "shallow_water":
						if (s.spec === "vortex" && !specUsed) {
							tiles[i].o = randomFrom (["vortex", "bubbles", "water_rocks"]);
							if (tiles[i].o === "vortex") {
								specUsed = true;
							}
						} else {
							tiles[i].o = randomFrom (["bubbles", "bubbles", "water_rocks"]);
						}
						break;
					case "swamp_water":
						tiles[i].d = randomFrom ([null, "cattails"]);
						if (tiles[i].d === null) {
							if (s.spec === "skull" && !specUsed) {
								tiles[i].o = randomFrom (["skull"]);
								if (tiles[i].o === "skull") {
									specUsed = true;
								}
							} else {
								tiles[i].d = randomFrom (["lilypad", "lilypads", "dead_tree", "dead_tree"]);
							}
						}
						break;
					case "ground_sand":
						tiles[i].o = "treasure";
						break;
					case "water":
						tiles[i].d = randomFrom (["lilypad", "lilypads"]);
						break;
					case "ground_grass1":
					case "ground_grass2":
						if (currentType === "field") {
							if (s.spec === "farm" && !specUsed) {
								tiles[i].o = randomFrom (["tree", "generic_mob", "farm"]);
								if (tiles[i].o === "farm") {
									specUsed = true;
								}
							} else {
								tiles[i].o = randomFrom (["tree", "generic_mob"]);
							}
							break;
						}
					case "ground_bushes":
						if (currentType === "glade") {
							if (s.spec === "standing_stones" && !specUsed) {
								tiles[i].o = randomFrom (["standing_stones", "cedar_tree", "cedar_tree", "generic_mob"]);
								if (tiles[i].o === "standing_stones") {
									specUsed = true;
								}
							} else {
								tiles[i].o = randomFrom (["cedar_tree", "cedar_tree", "generic_mob"]);
							}
						}
						break;
					case "ground_trees":
						break;
					case -1:
					case null:
					case "ground_debug1":
					case "ground_debug2":
					case "ground_debug3":
					case "ground_debug4":
						break;
					default:
						tiles[i].o = randomFrom (["obelisk", "tree", "cedar_tree"]);
						break;
				}
			}

			inRect = 0;
			var rectsIn = [];
			var baseHeight;
			// "bonus" rects are currently only bridge/shallows anchor points.
			bonus.map (function (b) {
				if (b.in (p)) {
					inRect += 1;
					inForeignRect = b.label;
					rectsIn.push (b);
				}
			});
			if (inRect > 0) {
				if (tiles[i].h === null) {
					tiles[i] = new Tile (new Height ("flat", -1), "water", null);
				}
				tiles[i].o = null;
				if (true/*[-1, "water"].indexOf (tiles[i].h) !== -1*/) {

					if (s.type === "glade" && tiles[i].h.baseHeight() !== -1) {
						tiles[i].t = randomFrom (["ground_grass1", "ground_grass2", "ground_bushes"]);
					} else if (s.type === "field" && tiles[i].h.baseHeight() !== -1) {
						tiles[i].t = randomFrom (["ground_grass1", "ground_grass2"]);
					}
					switch (inForeignRect) {
						case "bridge":
							if (tiles[i].h.baseHeight() === -1 && tiles[i].t !== "swamp_water") {
								if (rectsIn[0].wh.x === 1) {
									tiles[i].d = "bridge_tall_left";
								} else {
									tiles[i].d = "bridge_tall_right";
								}
								//tiles[i].t = "water";
							} else {
								switch (s.type) {
									case "badland":
										tiles[i].t = "ground_rock";
										break;
									default:
										break;
								}
							}
							//tiles[i].t = "ground_debug1";
							break;
						case "factory_bridge":
							if (tiles[i].h.baseHeight() === -1) {
								if (rectsIn[0].wh.x === 1) {
									tiles[i].d = y % 2 ? "factory_bridge_span_left" : "factory_bridge_pillar_left";
								} else {
									tiles[i].d = x % 2 ? "factory_bridge_span_right" : "factory_bridge_pillar_right";
								}
							}
							//tiles[i].t = "ground_debug3";
							break;
						case "shallow_connection":
							if (tiles[i].h.baseHeight() === -1 && tiles[i].t !== "swamp_water") {
								tiles[i].t = "shallow_water";
							} else {
								//tiles[i].h = -1;
							}
							//tiles[i].t = "ground_debug2";
							break;
						case "internal":
							if (tiles[i].h.baseHeight() !== -1) {
								// surface
								switch (s.type) {
									case "badland":
										tiles[i].t = "ground_rock";

										baseHeight = tiles[i].h.baseHeight();
										if (rectsIn.length > 1 || rectsIn[0].wh.x > 2 || rectsIn[0].wh.y > 2) {
											tiles[i].t = "ground_mud";
											if (rectsIn[0].wh.x === 1) {
												if (rectsIn[0].xy.equal (new Point2d (x, y))) {
													tiles[i].h = new Height ("uneven", -1, 0, 0, -1);
												} else {
													tiles[i].h = new Height ("uneven", 0, -1, -1, 0);
												}
											} else {
												if (rectsIn[0].xy.equal (new Point2d (x, y))) {
													tiles[i].h = new Height ("uneven", 0, 0, -1, -1);
												} else {
													tiles[i].h = new Height ("uneven", -1, -1, 0, 0);
												}
											}
										}
										break;
									case "factory":
										tiles[i].t = randomFrom(["factory_panel", "factory_grating", "factory_quads", "factory_tatami_l", "factory_tatami_r"]);
										break;
									case "swamp":
										tiles[i] = new Tile (new Height ("flat", -1), "swamp_water", null)
										break;
									default:
										break;
								}
							} else {
								// water
								switch (s.type) {
									case "swamp":
										tiles[i].t = "swamp_water";
										break;
									case "shallow":
										tiles[i].t = "shallow_water";
										break;
									default:
										break;
								}
							}
							//tiles[i].t = "ground_debug4";
							if (["shallow", "swamp"].indexOf (s.type) === -1 && tiles[i].h.baseHeight() === -1) {
								if (s.type === "badland") {
									if (tiles[i].t === "water") {
										tiles[i].d = "water_rocks";
									} // (else it's mud or rocks and walkable)
								} else {
									if (rectsIn[0].wh.x === 1) {
										tiles[i].d = "bridge_tall_left";
									} else {
										tiles[i].d = "bridge_tall_right";
									}
								}
							}
							break;
						default:
							console.error ("bad bonus connection type of \"" + inForeignRect + "\"");
							break;
					}
				}
			}

		}

		var t;
		var j;
		var adj1;
		var adj2;
		var back;
		for (i = 0; i < constants.SIZE; i++) {
			y = (i / constants.FULL_DIMENSION)|0;
			x = i % constants.FULL_DIMENSION;
			t = tiles[i];
			j = 0;
		}
		return tiles;
	}

	function tileFinisher (constants, tiles) {
		var adjTiles;
		var adjacentTileNonEmpty = false;
		var i = 0;
		var c;
		var updates = [];
		console.log ("in tileFinisher");
		// add a one-wide water buffer around every tile
		for (i = 0; i < constants.FULL_MAP_SIZE; i++) {
			c = unlinearize (constants.FULL_MAP_DIMENSION, i);
			if (tiles[i].t !== null) {
				continue;
			}
			//console.log (tiles[i]);
			//console.log (c);
			adjacentTileNonEmpty = adjacentCoords (c).map (function (c) {
				return c.inBounds (0, constants.FULL_MAP_DIMENSION) ? tiles[linearize (constants.FULL_MAP_DIMENSION, c.x, c.y)] : null;
			}).filter (function (t) {
				// not sure why t would ever end up `undefined` but it is sometimes
				return t !== null && t !== undefined;
			}).reduce (function (acc, t) {
				if (acc) {
					return acc;
				}
				return (t.t !== null);
			}, false);
			if (adjacentTileNonEmpty) {
				updates.push (i);
			}
		}
		updates.map (function (i) {
			tiles[i].t = "water";
			tiles[i].h = new Height ("flat", -1);
			if (randomInRange (0, 1) < 0.05) {
				tiles[i].d = randomFrom (["lilypad", "lilypads"]);
			}
		});
	}

	var spritesheet = new Spritesheet (null, new Point2d (64, 64),
		[ { "name": "ground_grass1", "dim": new Point2d (0, 0)}
		, { "name": "ground_grass2", "dim": new Point2d (1, 0)}
		, { "name": "ground_bushes", "dim": new Point2d (2, 2)}
		, { "name": "ground_rock", "dim": new Point2d (3, 0)}
		, { "name": "ground_trees", "dim": new Point2d (2, 1)}
		, { "name": "ground_wall_top_left", "dim": new Point2d (0, 1)}
		, { "name": "ground_wall_top_right", "dim": new Point2d (0, 2)}
		, { "name": "ground_wall_top_center", "dim": new Point2d (0, 3)}
		, { "name": "ground_wall_left", "dim": new Point2d (1, 1)}
		, { "name": "ground_wall_right", "dim": new Point2d (1, 2)}
		, { "name": "ground_wall_center", "dim": new Point2d (1, 3)}

		, { "name": "ground_grass1_top_flat", "dim": new Point2d (8, 0)}
		, { "name": "ground_grass1_bottom_flat", "dim": new Point2d (9, 0)}
		, { "name": "ground_grass1_left_flat", "dim": new Point2d (10, 0)}
		, { "name": "ground_grass1_right_flat", "dim": new Point2d (11, 0)}
		, { "name": "ground_grass1_fullslope_topleft", "dim": new Point2d (8, 1)}
		, { "name": "ground_grass1_fullslope_topright", "dim": new Point2d (9, 1)}
		, { "name": "ground_grass1_fullslope_bottomright", "dim": new Point2d (10, 1)}
		, { "name": "ground_grass1_fullslope_bottomleft", "dim": new Point2d (11, 1)}
		, { "name": "ground_grass1_top_slope_up", "dim": new Point2d (8, 2)}
		, { "name": "ground_grass1_right_slope_up", "dim": new Point2d (9, 2)}
		, { "name": "ground_grass1_bottom_slope_up", "dim": new Point2d (10, 2)}
		, { "name": "ground_grass1_left_slope_up", "dim": new Point2d (11, 2)}
		, { "name": "ground_grass1_bottom_slope_down", "dim": new Point2d (8, 3)}
		, { "name": "ground_grass1_left_slope_down", "dim": new Point2d (9, 3)}
		, { "name": "ground_grass1_top_slope_down", "dim": new Point2d (10, 3)}
		, { "name": "ground_grass1_right_slope_down", "dim": new Point2d (11, 3)}

		, { "name": "ground_mud", "dim": new Point2d (2, 0)}
		, { "name": "ground_mud_fullslope_topleft", "dim": new Point2d (12, 1)}
		, { "name": "ground_mud_fullslope_topright", "dim": new Point2d (13, 1)}
		, { "name": "ground_mud_fullslope_bottomright", "dim": new Point2d (14, 1)}
		, { "name": "ground_mud_fullslope_bottomleft", "dim": new Point2d (15, 1)}

		, { "name": "ground_sand", "dim": new Point2d (4, 0)}
		, { "name": "ground_sand_fullslope_topleft", "dim": new Point2d (12, 0)}
		, { "name": "ground_sand_fullslope_topright", "dim": new Point2d (13, 0)}
		, { "name": "ground_sand_fullslope_bottomright", "dim": new Point2d (14, 0)}
		, { "name": "ground_sand_fullslope_bottomleft", "dim": new Point2d (15, 0)}

		, { "name": "water", "dim": new Point2d (4, 3)}
		, { "name": "shallow_water", "dim": new Point2d (4, 2)}
		, { "name": "swamp_water", "dim": new Point2d (4, 1)}

		, { "name": "factory_floor", "dim": new Point2d (4, 0)}
		, { "name": "factory_panel", "dim": new Point2d (5, 0)}
		, { "name": "factory_grating", "dim": new Point2d (6, 0)}
		, { "name": "factory_pit", "dim": new Point2d (7, 0)}
		, { "name": "factory_quads", "dim": new Point2d (5, 1)}
		, { "name": "factory_tatami_l", "dim": new Point2d (6, 1)}
		, { "name": "factory_tatami_r", "dim": new Point2d (7, 1)}
		, { "name": "factory_wall_left", "dim": new Point2d (3, 1)}
		, { "name": "factory_wall_right", "dim": new Point2d (3, 2)}
		, { "name": "factory_wall_center", "dim": new Point2d (3, 3)}
		, { "name": "factory_bridge_span_right", "dim": new Point2d (3, 6)}
		, { "name": "factory_bridge_pillar_right", "dim": new Point2d (4, 6)}
		, { "name": "factory_bridge_span_left", "dim": new Point2d (5, 6)}
		, { "name": "factory_bridge_pillar_left", "dim": new Point2d (6, 6)}

		, { "name": "factory_building_barracks", "dim": new Point2d (8, 7)}
		, { "name": "factory_building_flare_small", "dim": new Point2d (9, 7)}
		, { "name": "factory_building_machine_shop", "dim": new Point2d (10, 7)}
		, { "name": "factory_building4", "dim": new Point2d (7, 7)}

		, { "name": "bridge_left", "dim": new Point2d (1, 7)}
		, { "name": "bridge_right", "dim": new Point2d (1, 6)}
		, { "name": "bridge_tall_left", "dim": new Point2d (2, 7)}
		, { "name": "bridge_tall_right", "dim": new Point2d (2, 6)}

		, { "name": "obelisk", "dim": new Point2d (6, 2)}
		, { "name": "tree", "dim": new Point2d (5, 4)}
		, { "name": "cedar_tree", "dim": new Point2d (4, 4)}
		, { "name": "treasure", "dim": new Point2d (6, 3)}
		, { "name": "cave", "dim": new Point2d (5, 3)}
		, { "name": "gate", "dim": new Point2d (7, 3)}
		, { "name": "standing_stones", "dim": new Point2d (6, 4)}
		, { "name": "skull", "dim": new Point2d (4, 5)}
		, { "name": "farm", "dim": new Point2d (7, 4)}

		, { "name": "mudman", "dim": new Point2d (1, 15)}
		, { "name": "hellhound", "dim": new Point2d (2, 15)}
		, { "name": "generic_mob", "dim": new Point2d (0, 15)}
		, { "name": "minotaur", "dim": new Point2d (3, 15)}
		, { "name": "gargoyle", "dim": new Point2d (4, 15)}
		, { "name": "flame_demon", "dim": new Point2d (5, 15)}
		, { "name": "hamar_tabal", "dim": new Point2d (6, 15)}
		, { "name": "acid_slime_purple", "dim": new Point2d (7, 15)}
		, { "name": "ganzer_worker", "dim": new Point2d (8, 15)}

		, { "name": "lilypad", "dim": new Point2d (0, 4)}
		, { "name": "lilypads", "dim": new Point2d (1, 4)}
		, { "name": "dead_tree", "dim": new Point2d (2, 4)}
		, { "name": "cattails", "dim": new Point2d (3, 4)}
		, { "name": "bubbles", "dim": new Point2d (3, 5)}
		, { "name": "vortex", "dim": new Point2d (2, 5)}
		, { "name": "water_rocks", "dim": new Point2d (1, 5)}

		, { "name": "outline_surface", "dim": new Point2d (15, 15)}
		, { "name": "outline_surface_fullslope_topleft", "dim": new Point2d (12, 14)}
		, { "name": "outline_surface_fullslope_topright", "dim": new Point2d (13, 14)}
		, { "name": "outline_surface_fullslope_bottomright", "dim": new Point2d (14, 14)}
		, { "name": "outline_surface_fullslope_bottomleft", "dim": new Point2d (15, 14)}

		, { "name": "cursor", "dim": new Point2d (13, 15)}

		, { "name": "ground_debug1", "dim": new Point2d (15, 6)}
		, { "name": "ground_debug2", "dim": new Point2d (14, 6)}
		, { "name": "ground_debug3", "dim": new Point2d (13, 6)}
		, { "name": "ground_debug4", "dim": new Point2d (12, 6)}
		]);

	var lands =
		{ "field":
			{ "solid": "rgb(0,192,0)"
			, "island": "rgb(0,128,0)"
			}
		, "badland":
			{ "solid": "rgb(192,128,0)"
			, "island": "rgb(128,96,0)"
			}
		, "factory":
			{ "solid": "rgb(96,96,96)"
			, "island": "rgb(64,64,64)"
			}
		, "shallow":
			{ "solid": "rgb(0,192,192)"
			, "island": "rgb(0,128,128)"
			}
		, "swamp":
			{ "solid": "rgb(0,255,128)"
			, "island": "rgb(0,192,96)"
			}
		, "glade":
			{ "solid": "rgb(0,128,0)"
			, "island": "rgb(0,64,0)"
			}
		};

	var specs =
		{ "vortex": "rgb(0,63,127)"
		, "standing_stones": "rgb(191,191,191)"
		, "gate": "rgb(63,15,127)"
		, "skull": "rgb(255,231,191)"
		, "farm": "rgb(163,231,127)"
		};

	return (
		{ generatorInit: generatorInit
		, sectionGenerator: sectionGenerator
		, tileGenerator: tileGenerator
		, tileFinisher: tileFinisher
		, spritesheet: spritesheet
		, spritesheetImage: "./tiles5.png"
		, legend: {lands: lands, specs: specs}
		});
}());
