var Hubur_B = (function (){
	function generatorInit () {
	}

	function between (v, lo, hi) {
		return v >= lo && v < hi;
	}

	function pathString (list) {
		return list.map (ptString
		).reduce (function (a, b) {
			return a + ", " + b;
		});
	}

	function ptString (pt) {
		return "{" + pt.x + "," + pt.y + "}";
	}


	function Tunneler (start, ai) {
		this.pos = start;
		this.last = null;
		this.ai = ai;
		this.stopped = false;
	}
	Tunneler.prototype =
	{ halt: function () {
			this.stopped = true;
		}
	, advance: function () {
			if (this.stopped) {
				this.last = this.pos;
				return this.pos;
			}
			var loc = this.pos;
			var adjs =
				[ new Point2d (loc.x + 1, loc.y)
				, new Point2d (loc.x - 1, loc.y)
				, new Point2d (loc.x, loc.y + 1)
				, new Point2d (loc.x, loc.y - 1)
				].filter (function (p) {
					return this.last === null || !p.equal (this.last); // not the last move
				}.bind (this));
			// copy the pos so the ai can't just mutate it to teleport
			var move = this.ai (new Point2d (this.pos.x, this.pos.y), adjs);
			if (move !== null && adjs.indexOf (move) === -1) {
				throw new Error ("Tunneler: bad move. got " + move.x + "," + move.y + "; only provided " + pathString (adjs));
			}
			this.last = this.pos;
			if (move !== null) {
				this.pos = move;
			}
			//console.log ("move: " + this.pos.x + "," + this.pos.y);
			return this.pos;
		}
	}

	function random () {
		return function (loc, open) {
			if (open.length === 0) {
				return null;
			}
			return randomFrom (open);
		}
	}

	function straightLine (dir) {
		var moves = new Point2d (0, 0);
		var coefficient = new Point2d (Math.cos (dir), Math.sin (dir)).normalize();
		//console.log ("coefficient: " + coefficient.x + "," + coefficient.y);
		return function (loc, open) {
			var choice;
			if (open.length === 0) {
				throw new Error ("straightLine: can't move; boxed in at " + loc.x + "," + loc.y);
			} else if (open.length === 1) {
				return open[0];
			}
			choice = open.reduce (function (a, b) {
				var adiff = a.subtract (loc);
				var bdiff = b.subtract (loc);
				var ma = moves.add (adiff).normalize();
				var mb = moves.add (bdiff).normalize();
				var ad = coefficient.dot (ma);
				var bd = coefficient.dot (mb);
				return ad > bd ? a : b;
			});
			moves = moves.add (choice.subtract (loc));
			return choice;
		};
	}
	function towards (pt) {
		return function (loc, open) {
			var choice;
			// if we're as close as we can get
			if (loc.subtract(pt).magnitude() <= 1) {
				return null;
			}
			if (open.length === 0) {
				throw new Error ("towards(" + pt.x + "," + pt.y + "): can't move; boxed in at " + loc.x + "," + loc.y);
			} else if (open.length === 1) {
				return open[0];
			}
			choice = open.reduce (function (a, b) {
				var adiff = a.subtract (pt);
				var bdiff = b.subtract (pt);
				var amag = adiff.magnitude ();
				var bmag = bdiff.magnitude ();
				return amag < bmag ? a : b;
			});
			return choice;
		}
	}

	function Board (dim) {
		this.dim = dim;
		this.tunnelers = [];
		this.usedSpaces = [];
		this.regionTiles = {};
	}
	Board.prototype =
		{ newTunneler: function (start, ai, regionName) {
				// wrap the ai in a function that does board bounds-checking
				var t = new Tunneler (start, function (loc, open) {
						// trim open values before passing them to the tunneller:
						var trimmedOpen = open.filter (function (pj) {
							// bounds check
							return between (pj.x, 0, this.dim) && between (pj.y, 0, this.dim) &&
								// not already used
								find (this.usedSpaces, function (u) { return u.equal (pj); }) === undefined
							}.bind (this));
						var r = ai (loc, trimmedOpen);
						/*
						if (r !== null) {
							console.log ("trimmedOpen: " + pathString (trimmedOpen));
						} else {
							console.log ("didn't move");
						}
						*/
						return r;
					}.bind (this));
				//console.log ("pushing " + this.tunnelers.length + " @ {" + start.x + "," + start.y + "}");
				this.tunnelers.push ({tunneler: t, region: regionName, path: [start]});
				this.initRegion (regionName);
				//console.log ("made new tunneller; started at {" + start.x + "," + start.y + "}");
				if (find (this.usedSpaces, function (p) { return p.equal (start); }) === undefined) {
					//console.log ("... and pushed to usedSpaces.");
					this.usedSpaces.push (start);
					this.regionTiles[regionName].push (start);
				}
				return t;
			}
		, branchAtClosestPoint: function (originRegion, ai, regionName, target) {
				var availablePoints = this.regionTiles[originRegion];
				var chosen;
				if (availablePoints === undefined || availablePoints.length == 0) {
					throw new Error ("can't place new tunneller; no tiles of region type \"" + originRegion + "\"");
				}
				chosen = availablePoints.reduce (function (a, b) {
					return a.distance(target) < b.distance (target) ? a : b;
				});
				return this.newTunneler (chosen, ai, regionName);
			}
		, branchFrom: function (originRegion, ai, regionName) {
				var availablePoints = this.regionTiles[originRegion];
				var chosen;
				if (availablePoints === undefined || availablePoints.length == 0) {
					throw new Error ("can't place new tunneller; no tiles of region type \"" + originRegion + "\"");
				}
				chosen = randomFrom (availablePoints);
				return this.newTunneler (chosen, ai, regionName);
			}
		, initRegion: function (regionName) {
				if (!this.regionTiles[regionName]) {
					this.regionTiles[regionName] = [];
				}
			}
		, regionTileList: function (regionName) {
				if (!this.regionTiles[regionName]) {
					return null;
				}
				return this.regionTiles[regionName];
			}
		, whatRegionTile: function (pt) {
				return Object.keys(this.regionTiles).map (function (k) {
					var inRegion = find (this.regionTiles[k], function (pt_) { return pt_.equal(pt); });
					return { k: k, m: inRegion };
				}.bind (this)).reduce (function (acc, match) {
					if (match.m !== undefined) {
						if (acc !== undefined) {
							throw new Error ("point " + ptString (pt) + " in multiple board regions(" + acc + " & " + match.k + ")");
						}
						return match.k;
					}
					return acc;
				}, undefined);
			}
		, getData: function (tunneller) {
				var m = find (this.tunnelers, function (t) {
					return t.tunneler == tunneller;
				});
				if (m === undefined) {
					throw new Error ("no such tunneller on board");
				}
				return m;
			}
		, getPath: function (tunneller) {
				return this.getData (tunneller).path;
			}
		, advance: function (n) {
				if (n === undefined || n < 0) {
					n = 1;
				}
				while (n > 0) {
					this.tunnelers.map (function (t, i) {
						var step = t.tunneler.advance();
						if (step === null || step.equal (t.tunneler.last)) {
							return; // stopped or error
						}
						t.path.push (step);
						this.regionTiles[t.region].push (step);
						//console.log ("advance(" + i + "): pushed " + ptString (step) + "; last was " + ptString (t.tunneler.last));
						this.usedSpaces.push (step);
					}.bind (this));
					n--;
				}
			}
		};

	// generate the large-scale map structure
	function sectionGenerator (constants) {

		var arr = Array (constants.SECTION_SIZE);
		fillArray (arr, null);

		function connectPoints (graph, start, end) {
			var zero = linearize (constants.SECTION_DIMENSION, start.x, start.y);
			var one = linearize (constants.SECTION_DIMENSION, end.x, end.y);

			graph.addEdge
				( graph.getNodeId (arr[zero].root.id)
				, graph.getNodeId (arr[one].root.id)
				, { a: zero , b: one }
				);
			graph.addEdge
				( graph.getNodeId (arr[one].root.id)
				, graph.getNodeId (arr[zero].root.id)
				, { a: one , b: zero }
				);
			return graph;
		}
		function setPathRegion (path, region) {
			path.map (function (pt) {
				var l = linearize (constants.SECTION_DIMENSION, pt.x, pt.y);
				arr[l].type = region;
			});
		}

		var graph = new Graph ();

		var zoneCount = 0;
		function segmentRoute (spacing, list, callback) {
			var prevZone = null;
			var prevI = null;
			var active = null;
			function segmentChunk (pos, x) {
				var i = linearize (constants.SECTION_DIMENSION, pos.x, pos.y);
				//console.log (i);
				if (x % spacing === 0) {
					prevZone = active;
					active = new Zone (zoneCount++);
					graph.addNode (active.id);
					if (prevZone !== null && prevI !== null) {
						graph.addEdge (graph.getNodeId (active.id), graph.getNodeId (prevZone.id)
							, {a: i, b: prevI});
						graph.addEdge (graph.getNodeId (prevZone.id), graph.getNodeId (active.id)
							, {a: prevI, b: i});
					}
				}
				arr[i] = {id: i, root: active};
				active.add (i);
				if (callback !== undefined) {
					callback (arr[i], x, i);
				}
				prevI = i;
			}
			list.map (segmentChunk);
		}

		var pi = 3.1415926;

		var pt = new Point2d (constants.SECTION_DIMENSION / 2, constants.SECTION_DIMENSION / 2);

		var board = new Board (constants.SECTION_DIMENSION);

		var origin = new Point2d
			( pt.x + randomIntInRange (0, Math.floor (constants.SECTION_DIMENSION / 6))
			, pt.y + randomIntInRange (0, Math.floor (constants.SECTION_DIMENSION / 6))
			);

		// place origin swamp glob
		var origin;
		origin = board.newTunneler (origin, random(), "swamp");
		board.advance(6);
		origin.halt();
		var originPath = board.getPath (origin);
		segmentRoute (8, originPath);
		setPathRegion (originPath, "swamp");
		//console.log ("swamp chunks: " + pathString (originPath));

		// select factory seed points
		var spin = randomInRange (0, pi * 2);
		var factoryRoots = [];
		factoryRoots.push (new Point2d (0, constants.SECTION_DIMENSION / 4).rotate (spin));
		factoryRoots.push (factoryRoots[0].rotate ((120 + randomInRange(-30,30)) * pi/180));
		factoryRoots.push (factoryRoots[0].rotate ((240 + randomInRange(-30,30)) * pi/180));
		function toMapCoord (a) { return Math.round (a + constants.SECTION_DIMENSION / 2); }
		factoryRoots = factoryRoots.map (function (pt) {
			var p = new Point2d (toMapCoord (pt.x), toMapCoord (pt.y));
			var i = 0;
			// if a factory root overlaps with the swamp chunk above, generation fails (since then the factory tunneller can't place any chunks, and further tunnellers branching from the root fail also). so we try to bump roots out of place.
			while (find (originPath, function (o) { return o.equal (p); }) !== undefined) {
				i++;
				if (Math.abs (pt.x) < Math.abs (pt.y)) {
					p = new Point2d (toMapCoord (pt.x + (pt.x > 0 ? i : i * -1)), toMapCoord (pt.y));
				} else {
					p = new Point2d (toMapCoord (pt.x), toMapCoord (pt.y + (pt.y > 0 ? i : i * -1)));
				}
			}
			//console.log ("factory root: " + ptString (p));
			return p;
		});

		// make swamp tendrils out to the factory points
		// this HAS to be more complex, b/c it's still possible for tendrils to get boxed in or cut off
		var swamps = [];
		swamps = factoryRoots.map (function (pt) {
			return board.branchAtClosestPoint ("swamp", towards (pt), "swamp", pt);
		});
		board.advance (16);

		var swampDeck = new Deck (["swamp", "field", "badland", "glade", "shallows"]);
		var swampZones = [];
		swamps.map (function (s) {
			var glob;
			s.halt();
			glob = board.getData(s);
			// drop the start (in origin)
			segmentRoute (randomIntInRange (2, 4), glob.path.slice (1));
			// add unique regions to the zone list
			swampZones = swampZones.concat (glob.path.slice (1).map (function (pt) {
					var l = linearize (constants.SECTION_DIMENSION, pt.x, pt.y);
					return arr[l].root;
				}).filter (function (sz) {
					return find (swampZones, function (sz_) {
						return sz_.id === sz.id;
					}) === undefined;
				})
			);

			if (glob.path.length < 2) {
				// so far the only times i've had this trigger is when the origin swamp is directly adjacent to the target. in which case the code works fine: it just connects the tendrils final value (also its first value, a chunk in the origin swamp) to the factory root. so we can just do nothing.
				//throw new Error ("bad tendril ({" + glob.path[0].x + "," + glob.path[0].y + "})");
				return;
			}
			// connect the tendril up with the origin
			connectPoints (graph, glob.path[0], glob.path[1]);
		});
		// this should ignore size-0 zones :V
		swampZones.map (function (sz) {
			if (sz.size <= 0) {
				return;
			}
			var t = swampDeck.pull();
			sz.mapSubs (function (chunk) {
				arr[chunk].type = t;
			});
		});

		// start generating factory chunks
		var factories = new Deck (["factory-research", "factory-quarry", "factory-processing"]);
		var extend =
			{ "factory-research":
				{"outer": "factory-stacks"
				,"court": "factory-pulpit"
				}
			, "factory-quarry":
				{"outer": "factory-tunnels"
				,"court": "factory-tarpit"
				}
			, "factory-processing":
				{"outer": "factory-plant"
				,"court": "factory-grotto"
				}
			};

		var ts = [];

		var factoryZones = factoryRoots.map (function (root) {
			var factory = factories.pull();
			var main;
			switch (factory) {
				case "factory-quarry":
					main = board.newTunneler (root, random(), factory);
					break;
				default:
					main = board.newTunneler (root, straightLine (randomInRange (0, pi * 2)), factory);
					break;
			}
			var depths = board.branchFrom (factory, random(), extend[factory].outer);
			return {main: main, depths: depths, factory: factory, root: root};
		});

		board.advance (6);

		factoryZones.map (function (zone) {
			zone.main.halt();
			zone.depths.halt();
		});

		var offshoots = [];
		factoryZones.map (function (zone) {
			var offshootCount = randomIntInRange (4, 10);
			var offshootType = null;
			var core;
			switch (zone.factory) {
				case "factory-quarry":
					// tarpits have to be attached to quarry
					core = board.branchFrom (zone.factory, random(), extend[zone.factory].court);
					break;
				default:
					core = board.branchFrom (randomFrom ([zone.factory, extend[zone.factory].outer]), random(), extend[zone.factory].court);
					break;
			}
			zone.core = core;
			offshoots.push (zone.depths);
			offshoots.push (core);
			zone.alcoves = [];
			var offshoot;
			while (offshootCount > 0) {
				offshootType = offshootCount % 2 === 1 ? zone.factory : extend[zone.factory].outer;
				if (offshootType === extend[zone.factory].outer) {
					offshoot = board.branchFrom (randomFrom ([zone.factory, extend[zone.factory].outer]), random(), offshootType);
				} else {
					offshoot = board.branchFrom (offshootType, random(), offshootType);
				}
				offshoots.push (offshoot);
				zone.alcoves.push (offshoot);
				offshootCount--;
			}
		});
		board.advance();
		// i uh don't know how this'll work wrt multiple factory sections (apparently it means sometimes it'll break things, i guess) (b/c the cores are also in offshoots. except the whole point of the lower map is to keep advancing until we get a good core. so, idk.)
		offshoots.map (function (t) { t.halt(); });

		// -1954183071 infinite loop???
		factoryZones.map (function (zone) {
			// if the core hasn't generated, keep picking random tiles to advance from until something works.
			while (board.getPath (zone.core).length < 2) {
				zone.core.halt(); // the whole problem is core can't advance, so this shouldn't matter, but let's just make sure here
				switch (zone.factory) {
					case "factory-quarry":
						// tarpits have to be attached to quarry
						zone.core = board.branchFrom (zone.factory, random(), extend[zone.factory].court);
						break;
					default:
						zone.core = board.branchFrom (randomFrom ([zone.factory, extend[zone.factory].outer]), random(), extend[zone.factory].court);
						break;
				}
				offshoots.push (zone.core); // if the replacement cores aren't pushed then when we link segments the core will be left unlinked
				board.advance();
			}
			zone.core.halt();
		});

		factoryZones.map (function (zone) {
			var glob = board.getData (zone.main);
			//console.log (zone.factory + " path:");
			//console.log (pathString (glob.path));

			var depthsGlob = board.getData (zone.depths);
			//console.log ("depths path:");
			//console.log (pathString (depthsGlob.path));

			var coreGlob = board.getData (zone.core);
			//console.log ("core:")
			//console.log (coreGlob);
			//console.log (pathString (coreGlob.path));

			switch (zone.factory) {
				case "factory-quarry":
					segmentRoute (12, glob.path);
					break;
				default:
					segmentRoute (3, glob.path);
					break;
			}
			setPathRegion (glob.path, glob.region);
		});

		// now that the factory zones are generated, connect the swamp to it
		swamps.map (function (s, i) {
			var glob = board.getData (s);
			// connect the tendril end up to the corresponding factory
			connectPoints (graph, glob.path[glob.path.length-1], factoryZones[i].root);
		});

		//console.log ("used spaces:");
		//console.log (pathString (board.usedSpaces));

		var swampOffshoots = [];
		var swampOffshootCount = randomIntInRange (24, 48);
		var advanceStep = Math.ceil(swampOffshootCount / 6);
		var i = 0;
		var j = 0;
		for (j = 0; j < 7; j++) {
			for (i = 0; i < advanceStep; i++) {
				swampOffshoots.push (board.branchFrom ("swamp", random (), "swamp"));
			}
			board.advance (1);
		}
		swampOffshoots.map (function (so) {
			var glob = board.getData (so);
			so.halt();
			if (glob.path.length < 2) {
				return;
			}
			segmentRoute (3, glob.path.slice(1));
			setPathRegion (glob.path.slice(1), swampDeck.pull());
			connectPoints (graph, glob.path[0], glob.path[1]);
		});

		// this has the side-effect of making zones (via the segmentChunk call). all zones need to be created before we can add graph edges (since otherwise we might be trying to link to a thing w/ no zone id), so we map twice: first making the zones, and then linking them.
		factoryZones.map (function (zone) {
			var depthsGlob = board.getData (zone.depths);
			segmentRoute (3, depthsGlob.path.slice(1), function (chunk, i, arrPos) {
				chunk.depth = i + 1;
			});
			setPathRegion (depthsGlob.path.slice(1), depthsGlob.region);

			var coreGlob = board.getData (zone.core);
			segmentRoute (3, coreGlob.path.slice(1));
			setPathRegion (coreGlob.path.slice(1), coreGlob.region);

			// now that the main depths are set, we can construct offshoots and properly set their depths
			zone.alcoves.map (function (alcove) {
				var glob = board.getData (alcove);
				if (glob.path.length < 2) {
					//console.log ("generated zero-length offshoot (at " + ptString (glob.path[0]) + "); ignoring");
					return;
				}
				var rootPt = linearize (constants.SECTION_DIMENSION, glob.path[0].x, glob.path[0].y);
				var startDepth = arr[rootPt].depth !== undefined ? arr[rootPt].depth : null;
				if (startDepth === null
						&& ["factory-stacks", "factory-tunnels", "factory-plant"].indexOf (glob.region) !== -1) {
					startDepth = 0;
				}
				if (startDepth !== null) {
					segmentRoute (3, glob.path.slice(1), function (chunk, i) {
						chunk.depth = startDepth + 1 + i;
					});
				} else {
					segmentRoute (3, glob.path.slice(1));
				}
				setPathRegion (glob.path.slice(1), glob.region);
			});
		});

		// offshoots includes depths and core tunnelers
		offshoots.map (function (offshoot) {
			var glob = board.getData (offshoot);
			var path = glob.path;
			//console.log ("linking offshoot");
			//console.log (offshoot);
			if (path.length < 2) {
				//console.log ("early exit (" + path[0].toString() + ")");
				return;
			}
			//console.log ("linking " + path[0].toString() + " to " + path[1].toString());
			// set the zone connection b/t the root tile and the rest of the offshoot
			connectPoints (graph, path[0], path[1]);
		});

		var zones = arr
			.map (function (chunk) { return chunk ? chunk.root : null; })
			.filter (function (z) { return z !== null; })
			.filter (function (z, i, self) { return self.indexOf (z) === i; });
		//console.log ("ALL ZONES:");
		//console.log (zones);
		var swampSubtypeDeck = new Deck (["zombie", "sobek"]);
		var swampZones = zones
			.filter (function (z) { return arr[z.subs[0]].type === "swamp"; })
			.map (function (z) {
				var subtype;
				if (z.subs.length >= 6) {
				} else {
					subtype = swampSubtypeDeck.pull ();
					z.mapSubs (function (chunk) {
						arr[chunk].subtype = subtype;
					});
				}
			});


		var loops = 0;
		var tries = 0;

		var swampChunks = new Deck (board.regionTileList ("swamp"));
		var tile;
		var tileI;
		var picked;
		while (loops < 15 && tries < 45) {
			tries++;
			tile = swampChunks.pull(); // lookup random swamp point.
			tileI = linearize (constants.SECTION_DIMENSION, tile.x, tile.y);
			//console.log ("graph: init flow from " + graph.getNodeId (arr[tileI].root.id));
			graph.dijkstras (graph.getNodeId (arr[tileI].root.id));
			adjs = tile.adjacent().filter (function (pt) {
				if (!(between (pt.x, 0, constants.SECTION_DIMENSION) && between (pt.y, 0, constants.SECTION_DIMENSION))) { // out of bounds
					//console.log (ptString (pt) + ": early exit; out of bounds");
					return false;
				}
				var adjI = linearize (constants.SECTION_DIMENSION, pt.x, pt.y);
				if (arr[adjI] === null) { // adjacent tile unused
					//console.log (ptString (pt) + ": early exit; unused chunk");
					return false;
				}
				var adjId = graph.getNodeId (arr[adjI].root.id);
				var inSeparateZone = false;
				var pathDistance = 0;
				// if tile is in a separate swamp region or in a factory entrance
				switch (board.whatRegionTile (pt)) {
					// these are board regions, not actual zone types; all non-factory regions are just 'swamp'
					case "swamp":
						if (arr[tileI].root.id !== arr[adjI].root.id) {
							inSeparateZone = true;
						}
						break;
					case "factory-quarry":
					case "factory-processing":
					case "factory-research":
						inSeparateZone = true;
						break;
					default:
						break;
				}
				/*
				console.log ("looking for path b/t "
					+ tile.toString() + " (" + graph.getNodeId (arr[tileI].root.id) + ") and"
					+ pt.toString() + " (" + adjId + ")");
				*/
				pathDistance = graph.pathTo (adjId).length; // minimum distance is 2
				//console.log (ptString (pt) + ": " + inSeparateZone + " (" + board.whatRegionTile(pt) + ") & " + pathDistance);
				return inSeparateZone && pathDistance >= 5;
			});
			if (adjs.length === 0) {
				//console.log ("skipping; " + ptString (tile) + " has no loop potential");
				continue;
			}
			loops++;
			picked = randomFrom (adjs)
			//console.log ("added loop b/t " + ptString (tile) + " and " + ptString (picked));
			connectPoints (graph, tile, picked);
		}

		var leftovers = new Zone (999);
		arr = arr.map (function (v, i) {
			if (v !== null) {
				return v;
			}
			leftovers.add (i);
			return {id: i, root: leftovers, type: "empty"};
		});
		graph.addNode (999);

		arr.map (function (v, i) {
			// set local adjacencies
			var pt = unlinearize (constants.SECTION_DIMENSION, i);
			var north = pt.y <= 0 ? null
				: linearize (constants.SECTION_DIMENSION, pt.x, pt.y-1);
			var south = pt.y >= constants.SECTION_DIMENSION - 1 ? null
				: linearize (constants.SECTION_DIMENSION, pt.x, pt.y+1);
			var west = pt.x <= 0 ? null
				: linearize (constants.SECTION_DIMENSION, pt.x-1, pt.y);
			var east = pt.x >= constants.SECTION_DIMENSION - 1 ? null
				: linearize (constants.SECTION_DIMENSION, pt.x+1, pt.y);

			v.adjs =
				{ north: null
				, south: null
				, east: null
				, west: null
				};
			[ {i: north, l: "north"}
			, {i: south, l: "south"}
			, {i: west, l: "west"}
			, {i: east, l: "east"}
			].map (function (o) {
				if (o.i === null) {
					return;
				}
				if (arr[o.i].root.id === v.root.id) { // same zone
					v.adjs[o.l] = {type: "local", to: arr[o.i].type};
				} else if (graph.edgesMatching
					( graph.getNodeId (arr[o.i].root.id)
					, graph.getNodeId (v.root.id)
					, function (e) {
						return (e.label.a === i && e.label.b === o.i) ||
							(e.label.a === o.i && e.label.b === i);
						}).length > 0) { // edge between zones
					v.adjs[o.l] = {type: "door", to: arr[o.i].type};
				}
			});
		});

		/*
		this is precisely the stuff that shouldn't be in the clientside generator. instead of having decks, we should call `population`, and use that to get locations and encounters.

		this also implies that we should split 'swamp' in two: swamp-zombie and swamp-sobek.

		then, given the population reels, first dump locations into chunks, and then scatter encounters.
		(i guess randomize the list of chunks, and then loop through it repeatedly, placing locations and then encounters at one-per-chunk until we run out of events.)

swamp encounters:
	sobek, zombie, will-o-wisp
	clay man (on clay edges)

		*/
		/*
		var specDecks =
			{ "field": new Deck (["farm"])
			, "shallows": new Deck (["vortex"])
			, "glade": new Deck (["standing_stones"])
			, "swamp":
				{ "zombie": new Deck (["skull", "hive"])
				, "sobek": new Deck (["nest"])
				}
			, "badland": new Deck (["obelisk", "cave"])

			, "factory-quarry": new Deck (["DUMMY"])
			, "factory-tunnels": new Deck (["barracks"])
			, "factory-tarpit": new Deck (["DUMMY"])

			, "factory-processing": new Deck (["flare_stack", "machine_shop"])
			, "factory-plant": new Deck (["DUMMY"])
			, "factory-grotto": new Deck (["DUMMY"])

			, "factory-research": new Deck (["alchemy_lab"])
			, "factory-stacks": new Deck (["DUMMY"])
			, "factory-pulpit": new Deck (["DUMMY"])
			};
*/
		arr.map (function (v, i) {
			if (v.type === "empty") {
				return;
			}
			v.flags = [];

			v.flags.push (v.type);
			var usedAdjs = [v.adjs.north, v.adjs.south, v.adjs.east, v.adjs.west].filter (function (a) { return a !== null; });
			if (usedAdjs.length === 1) {
				switch (usedAdjs[0].type) {
					case "local":
						v.flags.push ("deadend");
						break;
					case "door":
						v.flags.push ("alcove");
						break;
					default:
						break;
				}
			}
			if (v.depth !== undefined) {
				v.flags.push ("depth-"+v.depth);
			}
			if (v.subtype !== undefined) {
				v.flags.push (v.subtype);
			}
			/*
			// set specs
			if (!v.spec) {
				switch (v.type) {
					case "factory-tunnels":
						// FIXME: the tunnels will not always have a 6-depth location
						if (v.depth === 6) {
							v.spec = "void_hole";
						}
						break;
					case "swamp":
						if (randomInRange (0, 1) < 0.5) {
							if (v.flags.indexOf ("zombie") !== -1) {
								v.spec = specDecks["swamp"].zombie.pull ();
							} else if (v.flags.indexOf ("sobek") !== -1) {
								v.spec = specDecks["swamp"].sobek.pull ();
							}
						}
						break;
					case "factory-pulpit":
					case "factory-tarpit":
					case "factory-grotto":
						break;
					default:
						if (usedAdjs.length === 1 || randomInRange (0, 1) < 0.25) {
							v.spec = specDecks[v.type].pull();
						}
						break;
				}
			}
			*/
			//console.log (v);
		});

		arr.map (function (section) {
			sectionDetailGenerator (constants, section);
		});


		//console.log (graph);
		return {array: arr, graph: graph};
	}


	var tileOptions =
		{ "factory-tarpit": ["tar"]
		, "factory-tunnels": ["ground_underground"]
		, "factory-quarry": ["quarry_floor"]

		, "factory-pulpit": ["ground_grass1"]
		, "factory-stacks": ["library_boards"]
		, "factory-research": ["factory_floatingcrystal", "factory_crystalbed", "factory_diagram", "factory_diagram", "factory_diagram", "factory_diagram"]

		, "factory-grotto": ["ground_grass1"]
		, "factory-plant": ["plant_floor"]
		, "factory-processing":
			[ "factory_panel", "factory_panel", "factory_quads", "factory_quads"
			, "factory_tatami_l", "factory_tatami_l", "factory_tatami_r", "factory_tatami_r"
			, "factory_grating"
			]
		, "swamp": ["swamp_water"]
		, "field": ["ground_grass1", "ground_grass2"]
		, "badland": ["ground_rock", "ground_rock", "ground_rock", "ground_rock", "ground_rock", "ground_rock", "ground_rock"
			, "ground_mud"
			, "ground_rock2"
			, "ground_crater"]
		, "shallows": ["water"]
		, "glade":
			[ "ground_grass1", "ground_grass2", "ground_bushes", "ground_bushes"
			]
		, "sand": ["ground_sand"]
		, "mudfield": ["ground_mud"]
		, undefined: ["void"]
		};

	var painters =
		{ "waterWrap": tilePainter (staticPainter ("water"), staticPainter (-1), weighedPainter (
			[ {w: 0.90, v: null}
			, {w: 0.05, v: "lilypad"}
			, {w: 0.05, v: "lilypads"}
			]))

		//, "swamp": tilePainter (randomPainter (tileOptions["swamp"]), staticPainter (-1))
		, "swamp": tilePainter (randomPainter (tileOptions["swamp"]), staticPainter (-1), weighedPainter (
			[ {w: 0.50, v: null}
			, {w: 0.45, v: "cattails"}
			, {w: 0.05, v: "dead_tree"}
			]))
		, "swamp_pods": tilePainter (randomPainter (tileOptions["swamp"]), staticPainter (-1), weighedPainter (
			[ {w: 0.50, v: null}
			, {w: 0.45, v: "cattails"}
			, {w: 0.05, v: "alien_tree_pods"}
			]))
		, "swamp_fringe": tilePainter (randomPainter (["ground_mud", "ground_mud", "ground_grass1", "ground_grass2"]), staticPainter (0))
		, "shallows": tilePainter (randomPainter (tileOptions["shallows"]), staticPainter (-1))
		, "field": tilePainter (randomPainter (tileOptions["field"]), staticPainter (0))
		, "badland": tilePainter (randomPainter (tileOptions["badland"]), staticPainter (0), weighedPainter (
			[ {w: 0.90, v: null}
			, {w: 0.05, v: "broken_obelisk"}
			, {w: 0.025, v: "alien_tree_mound"}
			, {w: 0.025, v: "alien_tree_holes"}
			]))
		, "badlandT": tilePainter (randomPainter (tileOptions["badland"]), staticPainter (1), weighedPainter (
			[ {w: 0.90, v: null}
			, {w: 0.05, v: "broken_obelisk"}
			, {w: 0.025, v: "alien_tree_mound"}
			, {w: 0.025, v: "alien_tree_holes"}
			]))
		, "badland-clearing": tilePainter (staticPainter ("ground_barerock"), staticPainter (0))
		, "badland-clearingT": tilePainter (staticPainter ("ground_barerock"), staticPainter (1))
		, "glade": tilePainter (randomPainter (tileOptions["glade"]), staticPainter (0), weighedPainter (
			[ {w: 0.95, v: null}
			, {w: 0.05, v: "alien_tree_truffula"}
			]))
		, "glade_trees": tilePainter (randomPainter (["ground_trees"]), staticPainter (0))
		, "glade_blue": tilePainter (randomPainter (["ground_grass_blue"]), staticPainter (0), weighedPainter (
			[ {w: 0.90, v: null}
			, {w: 0.05, v: "alien_tree_blue"}
			, {w: 0.05, v: "alien_flowers"}
			]))

		, "sand": tilePainter (randomPainter (tileOptions["sand"]), staticPainter (-1))
		, "mudfield_low": tilePainter (randomPainter (tileOptions["mudfield"]), staticPainter (-1))
		, "mudfield_high": tilePainter (randomPainter (tileOptions["mudfield"]), staticPainter (0))

		, "factory-tunnels": tilePainter (randomPainter (tileOptions["factory-tunnels"]), staticPainter (0))
		, "factory-stacks": tilePainter (randomPainter (tileOptions["factory-stacks"]), staticPainter (0))
		, "factory-shelves": tilePainter (randomPainter (tileOptions["factory-stacks"]), staticPainter (1))
		, "factory-quarry": tilePainter (randomPainter (tileOptions["factory-quarry"]), staticPainter (0)) // these are placed on doors into/out of quarries
		, "factory-quarry_high": tilePainter (randomPainter (tileOptions["factory-quarry"]), staticPainter (0))
		, "factory-quarry_low": tilePainter (randomPainter (tileOptions["factory-quarry"]), staticPainter (-1))
		, "factory-pulpit": tilePainter (randomPainter (tileOptions["factory-pulpit"]), staticPainter (0))
		, "factory-research": tilePainter (randomPainter (tileOptions["factory-research"]), staticPainter (0))
		, "factory-grotto": tilePainter (randomPainter (tileOptions["factory-grotto"]), staticPainter (0))
		, "factory-tarpit": tilePainter (randomPainter (tileOptions["factory-tarpit"]), staticPainter (-1))
		, "factory-plant": tilePainter (randomPainter (tileOptions["factory-plant"]), staticPainter (0))
		, "factory-processing": tilePainter (randomPainter (tileOptions["factory-processing"]), staticPainter (0))


	//	, tilePainter (randomPainter (tileOptions[section.type]), staticPainter (height))
		};


	function curriedTile (tile) {
		return function (height) {
			if (height instanceof Height) {
				return new Tile (height, tile, null);
			} else {
				return new Tile (new Height ("flat", height), tile, null);
			}
		}
	}

	tunnelBuilder = function (painter, region) {
		return function (constants, section, dir, baseRect) {
			var bridge;
			var midpoint = constants.FULL_DIMENSION % 2
				? Math.floor (constants.FULL_DIMENSION / 2)
				: (constants.FULL_DIMENSION / 2) - 1;
			var lengthToCenter = Math.ceil (constants.FULL_DIMENSION / 2);
			switch (dir) {
				case "north":
					bridge = new Rect
						( new Point2d (midpoint, 0)
						, new Point2d (1, lengthToCenter)
						);
					break;
				case "south":
					bridge = new Rect
						( new Point2d (midpoint, midpoint)
						, new Point2d (1, lengthToCenter + 1)
						);
					break;
				case "east":
					bridge = new Rect
						( new Point2d (midpoint, midpoint)
						, new Point2d (lengthToCenter + 1, 1)
						);
					break;
				case "west":
					bridge = new Rect
						( new Point2d (0, midpoint)
						, new Point2d (lengthToCenter, 1)
						);
					break;
				default:
					throw new Error ("bad direction: \"" + dir + "\"");
			}
			baseRect.push ({rect: bridge
				, data: {type: region}
				, painter: painter
			});
			return;
		}
	}
	tunnelRoomFinalizer = function (constants, section, landscape) {
		if (landscape.length > 2) {
			return;
		}
		landscape[0].rect.xy.x -= 3;
		landscape[0].rect.xy.y -= 3;

		landscape[0].rect.wh.x += 6;
		landscape[0].rect.wh.y += 6;
	}

	centerBuilder = function (painter, size) {
		return function (constants, section) {
			var halfSize = Math.floor (size / 2);
			var midpoint = constants.FULL_DIMENSION % 2
				? Math.floor (constants.FULL_DIMENSION / 2)
				: (constants.FULL_DIMENSION / 2) - 1;
			return [ {rect: new Rect
					( new Point2d (midpoint - halfSize, midpoint - halfSize)
					, new Point2d (size, size)
					)
				, data: null
				, painter: painter
				}];
		};
	};

	defaultInitBuilder = function (painter) {
		return function (constants, section) {
			return [ {rect: new Rect
					( new Point2d (constants.BUFFER, constants.BUFFER)
					, new Point2d (constants.DIMENSION, constants.DIMENSION)
					)
				, data: null
				, painter: painter
				}];
			};
	};
	defaultFinalizer = function (constants, section, landscape) {
		return;
	}
	function edgeBlobsFinalizer (lo, hi, blobFunc) {
		return function (constants, section, landscape) {
			var dir;
			var c = randomIntInRange (lo, hi);
			var dirs = ["north", "south", "east", "west"].filter (function (dir) {
				return section.adjs[dir] === null || section.adjs[dir].type === "door";
			});
			if (dirs.length === 0) {
				return;
			}
			while (c > 0) {
				// this should probably be limited to directions that there isn't a local connection on, which we could manage by making this a finisher that looks up adjacent info to filter the direction list
				landscape.push (blobFunc (constants, section, dirs));
				c--;
			}
		};
	}
	waterWrapFinalizer = function (constants, section, landscape) {
		var ws = landscape.map (function (r) {
			if (r.data.type.indexOf ("shallows") !== -1) {
				return null;
			}
			return (
				{ rect: genericRect (r.rect.xy.x - 1, r.rect.xy.y - 1
						, r.rect.wh.x + 2, r.rect.wh.y + 2)
				, data: {type: []}
				, painter: "waterWrap"
				, priority: 0.01
				});
		});
		ws.map (function (w) { if (w !== null) { landscape.push (w); } });
		return;
	}

	defaultDoorConnection = function (painter, depth, width) {
		return function (constants, section, dir, baseRects) {
			// zone connection. add new rect.
			var bridge;
			var midpoint = constants.FULL_DIMENSION % 2
				? Math.floor (constants.FULL_DIMENSION / 2) - (Math.floor (width / 2) - 1)
				: (constants.FULL_DIMENSION / 2) - (Math.floor (width / 2));
			switch (dir) {
				case "north":
					bridge = new Rect
						( new Point2d (midpoint, 0)
						, new Point2d (width, depth)
						);
					break;
				case "south":
					bridge = new Rect
						( new Point2d (midpoint, constants.FULL_DIMENSION - depth)
						, new Point2d (width, depth)
						);
					break;
				case "east":
					bridge = new Rect
						( new Point2d (constants.FULL_DIMENSION - depth, midpoint)
						, new Point2d (depth, width)
						);
					break;
				case "west":
					bridge = new Rect
						( new Point2d (0, midpoint)
						, new Point2d (depth, width)
						);
					break;
				default:
					throw new Error ("bad direction: \"" + dir + "\"");
			}
			baseRects.push ({rect: bridge
				, data: {type: section.adjs[dir].to}
				, painter: painter
				, priority: 6
			});
		}
	};
	function defaultLocalConnection (n) {
		function toN (l, f) {
			var i = 0;
			while (i <= n) {
				f (l[i]);
				i++;
			}
		}
		return function (constants, section, dir, baseRects) {
			// shared zone. extend base rect
			switch (dir) {
				case "north":
					toN (baseRects, function (ln) {
						ln.rect.xy.y -= constants.BUFFER;
						ln.rect.wh.y += constants.BUFFER;
					});
					break;
				case "south":
					toN (baseRects, function (ln) {
						ln.rect.wh.y += constants.BUFFER;
					});
					break;
				case "west":
					toN (baseRects, function (ln) {
						ln.rect.xy.x -= constants.BUFFER;
						ln.rect.wh.x += constants.BUFFER;
					});
					break;
				case "east":
					toN (baseRects, function (ln) {
						ln.rect.wh.x += constants.BUFFER;
					});
					break;
				default:
					throw new Error ("bad direction: \"" + dir + "\"");
			}
		}
	}
	function genEdgeRect (constants) {
		return function (dir, offset, width, depth, buffer) {
			var r = null;
			var edgeBuffer = buffer !== undefined ? buffer : 0;
			switch (dir) {
				case "north":
					r = genericRect (offset, edgeBuffer, width, depth);
					break;
				case "south":
					r = genericRect (offset, constants.FULL_DIMENSION - (depth + edgeBuffer), width, depth);
					break;
				case "west":
					r = genericRect (edgeBuffer, offset, depth, width);
					break;
				case "east":
					r = genericRect (constants.FULL_DIMENSION - (depth + edgeBuffer), offset, depth, width);
					break;
				default:
					throw new Error ("bad direction: \"" + dir + "\"");
			}
			return r;
		};
	}
	function defaultWaterDoor (constants, section, dir, landscape) { // nonlocal edge
		var r;
		var offset;
		var width;
		var sectionType;
		var doorPainter;
		var edgeRect = genEdgeRect (constants);
		switch (section.adjs[dir].to) {
			case "swamp":
			case "shallows":
				width = 2;
				sectionType = section.type;
				break;
			default:
				sectionType = "sand";
				width = 4;
				break;
		}
		offset = constants.BUFFER + Math.floor ((constants.DIMENSION - width) / 2);
		r = edgeRect (dir, offset, width, constants.BUFFER + 1);
		landscape.push (
			{ rect: r
			, data: {type: [sectionType, "encounter", "edge"]}
			, painter: sectionType
			, priority: 5.5
			}
		);
	}

	many = function (builders) {
		return function (constants, section, dir, landscapes) {
			builders.map (function (f) {
				f (constants, section, dir, landscapes);
			});
		};
	};

	function newTunnelRoomFinalizer (constants, section, landscape) {
		if (landscape.filter(function (l) {
			return l.data.type.indexOf ("edge") !== -1;
		}).length > 1) {
			return;
		}
		landscape[0].rect.xy.x -= 3;
		landscape[0].rect.xy.y -= 3;
		landscape[0].rect.wh.x += 5;
		landscape[0].rect.wh.y += 5;

		landscape[1].rect.xy.x -= 1;
		landscape[1].rect.xy.y -= 1;
		landscape[1].rect.wh.x += 3;
		landscape[1].rect.wh.y += 3;
	}

	function genericRect (x, y, w, h) {
		return new Rect
			( new Point2d (x, y)
			, new Point2d (w, h)
			);
	}
	function basicLandscape (size) {
		return function (constants, section) {
			function centeredRect (size) {
				var halfSize = Math.floor (size / 2);
				var midpoint = constants.FULL_DIMENSION % 2
					? Math.floor (constants.FULL_DIMENSION / 2)
					: (constants.FULL_DIMENSION / 2) - 1;
				return new Rect
					( new Point2d (midpoint - halfSize + 1, midpoint - halfSize + 1)
					, new Point2d (size, size)
					);
			}
			var ls = [];
			var r;
			r = centeredRect (size);
			ls.push ( // ground
				{ rect: r
				, data: {type: [section.type, "encounter"]}
				, painter: section.type
				, priority: 5
				}
			);
			r = centeredRect (Math.max (size - 4, 3));
			ls.push ( // encounter swath i guess
				{ rect: r
				, data: {type: [section.type, "location"]}
				, painter: null
				, priority: -1
				}
			);
			return ls;
		};
	}

	function basicDoor (width, depth, encounterTags) {
		if (encounterTags === undefined) {
			encounterTags = ["encounter", "edge"];
		}
		return function (constants, section, dir, landscape) { // nonlocal edge
			var edgeRect = genEdgeRect (constants);
			var r;
			var offset = constants.BUFFER + Math.ceil ((constants.DIMENSION - width) / 2);
			var sectionType;
			var doorPainter;
			r = edgeRect (dir, offset, width, depth);
			switch (section.adjs[dir].to) {
				case "shallows":
				case "swamp":
					sectionType = "sand"; // section.adjs[dir].to;
					doorPainter = sectionType; // tilePainter (randomPainter (tileOptions[sectionType]), staticPainter (-1));
					break;
				default:
					sectionType = section.type;
					doorPainter = section.type; //tilePainter (randomPainter (tileOptions[section.type]), staticPainter (0));
					break;
			}
			landscape.push (
				{ rect: r
				, data: {type: encounterTags.concat ([sectionType])}
				, painter: doorPainter
				, priority: 6
				}
			);
		};
	}
	// generate an individual section, absent any information about any larger map structure -- given only the corresponding data in the `array` value in the `sectionGenerator` return value
/*

so either on the section-level or the tile-level, the generator should make a list of LANDSCAPE STRUCTURES: these are just rectangles of land that are in any particular order, and that each basically set a nxm section to a given height/type.

right now there are like two hardcoded bundles of landscape structures: 'bridge' and 'landscape'. this should be generalized into just a single big 'landscapes' list.

so like a glade could generate the following landscapes for its central bit:
	8x8 grass
	10x10 forest
	12x12 deep water
and then its bridges could generate like,
	2x4 forest
	2x2 grass
for its 'bridge' connection to another external glade, or
	8x1 grass
for its 'bridge' connection to an internal glade

(landscapes would have priority levels, so like it could be 1 deep water, 3 forest, 5 grass; + 4 bridge forest, 6 bridge grass)

but the POINT IS this should permit some more flexible landscape generation, with less worry about overwriting things or w/e.

(and then secondarily adding a 'regions' list that's used for spawning things and that have tags attached. right now landscapes are basically landscape-tagged, and bridges are basically bridge-tagged; there's no understanding of a non-tagged landscape, or of any tags more intricate than landscapes and bridges)

*/
	function sectionDetailGenerator (constants, section) {
		function centeredRect (size) {
			var halfSize = Math.floor (size / 2);
			var midpoint = constants.FULL_DIMENSION % 2
				? Math.floor (constants.FULL_DIMENSION / 2)
				: (constants.FULL_DIMENSION / 2) - 1;
			return new Rect
				( new Point2d (midpoint - halfSize + 1, midpoint - halfSize + 1)
				, new Point2d (size, size)
				);
		}
		var edgeRect = genEdgeRect (constants);
		var pt;
		//console.log (section);
		var landscapeRects = [];

		var landscapeBuilder;
		var localBuilder;
		var doorBuilder;
		var finalizer;
		switch (section.type) {
			case "factory-quarry":
				landscapeBuilder = function (constants, section) {
					var ls = [];
					var r;
					r = centeredRect (constants.DIMENSION);
					ls.push (
						{ rect: r
						, data: {type: [section.type, "encounter"]}
						, painter: "factory-quarry_high"
						, priority: 3
						}
					);
					r = centeredRect (constants.DIMENSION - 2);
					ls.push (
						{ rect: r
						, data: {type: []}
						, painter: "factory-quarry_low"
						, priority: 5
						}
					);
					r = centeredRect (constants.DIMENSION - 4);
					ls.push (
						{ rect: r
						, data: {type: [section.type, "location"]}
						, painter: null
						, priority: -1
						}
					);
					return ls;
				}
				localBuilder = function (constants, section, dir, landscape) { // local edge
					var r;
					var offset = constants.BUFFER+1;
					var width = constants.DIMENSION-2;
					var depth = constants.BUFFER+1;
					r = edgeRect (dir, offset, width, depth);
					landscape.push ( // edge recess
						{ rect: r
						, data: {type: []}
						, painter: "factory-quarry_low" //tilePainter (randomPainter (tileOptions[section.type]), staticPainter (-1))
						, priority: 5.5
						}
					);
					r = edgeRect (dir, offset-1, width+2, depth);
					landscape.push ( // edge wall
						{ rect: r
						, data: {type: []}
						, painter: "factory-quarry_high" //tilePainter (randomPainter (tileOptions[section.type]), staticPainter (0))
						, priority: 5.25
						}
					);
					r = edgeRect (dir, offset-1, width+2, depth-1);
					landscape.push (
						{ rect: r
						, data: {type: [section.type, "encounter", "edge"]}
						, painter: null
						, priority: -1
						}
					);
				}
				doorBuilder = basicDoor (2, constants.BUFFER);
				finalizer = waterWrapFinalizer;
				break;
			case "factory-tunnels":
				landscapeBuilder = function (constants, section) {
					var ls = [];
					var size = 5;
					r = centeredRect (size + 2);
					ls.push (
						{ rect: r
						, data: {type: [section.type]}
						, painter: section.type
						, priority: 5
						}
					);
					return basicLandscape (size) (constants, section).concat (ls);
				}
				localBuilder = basicDoor (1, Math.floor ((constants.FULL_DIMENSION - 2) / 2), ["edge"]);
				doorBuilder = basicDoor (1, Math.floor ((constants.FULL_DIMENSION - 2) / 2), ["edge"]);
				finalizer = newTunnelRoomFinalizer;
				break;
			case "factory-stacks":
				landscapeBuilder = function (constants, section) {
					var ls = [];
					var size = 5;
					var t = randomIntInRange (0, 2);

					var shelfLength = 5;
					var shelfDepth = 1;

					var halfSize = Math.floor (shelfLength / 2);
					var midpoint = constants.FULL_DIMENSION % 2
						? Math.floor (constants.FULL_DIMENSION / 2)
						: (constants.FULL_DIMENSION / 2) - 1;
					var shelfInit = midpoint - halfSize + 1;
					var shelfMax = shelfInit + shelfLength;
					var i;
					
					r = centeredRect (size + 2);
					ls.push (
						{ rect: r
						, data: {type: [section.type]}
						, painter: section.type
						, priority: 5
						}
					);
					switch (t) {
						case 0:
							i = shelfInit;
							while (i < shelfMax) {
								ls.push (
									{ rect: new Rect
										( new Point2d (shelfInit, i)
										, new Point2d (shelfLength, shelfDepth)
										)
									, data: {type: [section.type]}
									, painter: "factory-shelves"
									, priority: 7
									}
								);
								i += shelfDepth + 1;
							}
							break;
						case 1:
							i = shelfInit;
							while (i < shelfMax) {
								ls.push (
									{ rect: new Rect
										( new Point2d (i, shelfInit)
										, new Point2d (shelfDepth, shelfLength)
										)
									, data: {type: [section.type]}
									, painter: "factory-shelves"
									, priority: 7
									}
								);
								i += shelfDepth + 1;
							}
							break;
						case 2:
						default:
							break;
					}
					return basicLandscape (size) (constants, section).concat (ls);
				}
				localBuilder = basicDoor (1, Math.floor ((constants.FULL_DIMENSION - 2) / 2), ["edge"]);
				doorBuilder = basicDoor (1, Math.floor ((constants.FULL_DIMENSION - 2) / 2), ["edge"]);
				finalizer = newTunnelRoomFinalizer;
				break;
			case "factory-tarpit":
				landscapeBuilder = function (constants, section) {
					var ls = [];
					var r;
					r = centeredRect (constants.DIMENSION);
					ls.push ( // trees
						{ rect: r
						, data: {type: []}
						, painter: "mudfield_high" //tilePainter (staticPainter ("ground_mud"), staticPainter (0))
						, priority: 3
						}
					);
					r = centeredRect (constants.DIMENSION - 2);
					ls.push ( // ground
						{ rect: r
						, data: {type: [section.type, "encounter"]}
						, painter: "factory-tarpit" // tilePainter (randomPainter (tileOptions[section.type]), staticPainter (-1))
						, priority: 5
						}
					);
					r = centeredRect (constants.DIMENSION - 4);
					ls.push (
						{ rect: r
						, data: {type: [section.type, "location"]}
						, painter: null
						, priority: -1
						}
					);
					return ls;
				};
				localBuilder = many (
					[ defaultDoorConnection
						( randomPainter (tileOptions[section.type])
						, 1
						, 10
						)
					, defaultLocalConnection (0)
					]);
				doorBuilder = function (constants, section, dir, landscape) { // nonlocal edge
					var r;
					var width = 2;
					var offset = constants.BUFFER + Math.floor ((constants.DIMENSION - width) / 2);
					var sectionType;
					var doorPainter;
					r = edgeRect (dir, offset, width, constants.BUFFER + 1);
					landscape.push (
						{ rect: r
						, data: {type: ["mudfield", "encounter", "edge"]}
						, painter: "mudfield_high" // tilePainter (staticPainter ("ground_mud"), staticPainter (0))
						, priority: 6
						}
					);
				}
				finalizer = defaultFinalizer;
				break;
			case "glade":
				landscapeBuilder = function (constants, section) {
					var ls = [];
					var r;
					if (randomInRange (0, 1) < 0.33) {
						r = centeredRect (constants.DIMENSION - 2);
						ls.push ( // ground
							{ rect: r
							, data: {type: ["glade", "encounter"]}
							, painter: "glade_blue"
								// tilePainter (randomPainter (tileOptions[section.type]), staticPainter (0))
							, priority: 5
							}
						);
					} else {
						r = centeredRect (constants.DIMENSION);
						ls.push ( // trees
							{ rect: r
							, data: {type: []}
							, painter: "glade_trees" //tilePainter (staticPainter ("ground_trees"), staticPainter (0))
							, priority: 3
							}
						);
						r = centeredRect (constants.DIMENSION - 2);
						ls.push ( // ground
							{ rect: r
							, data: {type: ["glade", "encounter"]}
							, painter: "glade"
								// tilePainter (randomPainter (tileOptions[section.type]), staticPainter (0))
							, priority: 5
							}
						);
					}
					/*
					if (randomInRange (0, 1) < 0.5) {
						var i = 0;
						var w;
						var h;
						var x;
						var y;
						while (i < 3) {
							w = randomIntInRange (3, 8);
							h = randomIntInRange (3, 8);
							x = randomIntInRange (0, constants.DIMENSION - w);
							y = randomIntInRange (0, constants.DIMENSION - h);
							r = new Rect
								( new Point2d (x + constants.BUFFER, y + constants.BUFFER)
								, new Point2d (w, h)
								);
							ls.push ( // ground
								{ rect: r
								, data: {type: ["glade"]}
								, painter: "glade_blue" // tilePainter (randomPainter (tileOptions[section.type]), staticPainter (0))
								, priority: 7
								}
							);
							i++;
						}
					}
					*/
					r = centeredRect (constants.DIMENSION - 4);
					ls.push ( // encounter swath i guess
						{ rect: r
						, data: {type: ["glade", "location"]}
						, painter: null
						, priority: -1
						}
					);
					return ls;
				};
				localBuilder = function (constants, section, dir, landscape) { // local edge
					var r;
					var offset = constants.BUFFER;
					var width = constants.DIMENSION;
					r = edgeRect (dir, offset, width, constants.BUFFER + 1);
					landscape.push (
						{ rect: r
						, data: {type: []}
						, painter: "glade_trees" //tilePainter (staticPainter ("ground_trees"), staticPainter (0))
						, priority: 4
						}
					);
					r = edgeRect (dir, offset + 1, width - 2, constants.BUFFER + 1);
					landscape.push (
						{ rect: r
						, data: {type: ["glade", "encounter", "edge"]}
						, painter: "glade" //tilePainter (randomPainter (tileOptions[section.type]), staticPainter (0))
						, priority: 6
						}
					);
				}
				doorBuilder = function (constants, section, dir, landscape) { // nonlocal edge
					var r;
					var offset = constants.BUFFER + Math.floor ((constants.DIMENSION - 4) / 2);
					var width = 4;
					var sectionType;
					var doorPainter;
					r = edgeRect (dir, offset, width, constants.BUFFER + 1);
					landscape.push (
						{ rect: r
						, data: {type: []}
						, painter: "glade_trees" //tilePainter (staticPainter ("ground_trees"), staticPainter (0))
						, priority: 4
						}
					);
					r = edgeRect (dir, offset + 1, width - 2, constants.BUFFER + 1);
					switch (section.adjs[dir].to) {
						case "shallows":
						case "swamp":
							sectionType = "sand"; //section.adjs[dir].to;
							doorPainter = sectionType; //tilePainter (randomPainter (tileOptions[sectionType]), staticPainter (-1));
							break;
						default:
							sectionType = "glade";
							doorPainter = sectionType; //tilePainter (randomPainter (tileOptions[section.type]), staticPainter (0));
							break;
					}
					landscape.push (
						{ rect: r
						, data: {type: [sectionType, "encounter", "edge"]}
						, painter: doorPainter
						, priority: 6
						}
					);
				}
				finalizer = many (
					[edgeBlobsFinalizer (1, 4, function (constants, section, dirs) {
						var dir = randomFrom (dirs);
						var width = 2; // randomIntInRange (2, 3);
						var offset = randomIntInRange (0, constants.FULL_DIMENSION - width);
						var depth = randomFrom ([1,2,3]);
						var edgeOffset = randomFrom ([1,2]);
						var r = edgeRect (dir, offset, width, depth, depth - edgeOffset);
						return (
							{ rect: r
							, data: {type: ["noencounter", "nolocation"]}
							, painter: "glade_trees" //tilePainter (staticPainter ("ground_trees"), staticPainter (0))
							, priority: 5.5
							}
						);
					})
					,waterWrapFinalizer
				]);
				break;
			case "swamp":
				landscapeBuilder = function (constants, section) {
					var ls = [];
					var r;
					r = centeredRect (constants.DIMENSION);
					ls.push ( // swamp rim
						{ rect: r
						, data: {type: []}
						, painter: "swamp_fringe" //tilePainter (randomPainter (["ground_mud", "ground_mud", "ground_grass1", "ground_grass2"]), staticPainter (0))
						, priority: 3
						}
					);
					r = centeredRect (constants.DIMENSION - 2);
					ls.push ( // swamp
						{ rect: r
						, data: {type: [section.type, "encounter"]}
						, painter: randomInRange (0, 1) < 0.5 ? "swamp" : "swamp_pods"
						, priority: 5
						}
					);
					r = centeredRect (constants.DIMENSION - 4);
					ls.push (
						{ rect: r
						, data: {type: [section.type, "location"]}
						, painter: null
						, priority: -1
						}
					);
					return ls;
				};
				localBuilder = function (constants, section, dir, baseRects) {
					switch (dir) {
						case "north":
							baseRects[0].rect.xy.y -= constants.BUFFER;
							baseRects[0].rect.wh.y += constants.BUFFER;
							baseRects[1].rect.xy.y -= constants.BUFFER+1;
							baseRects[1].rect.wh.y += constants.BUFFER+1;
							break;
						case "south":
							baseRects[0].rect.wh.y += constants.BUFFER;
							baseRects[1].rect.wh.y += constants.BUFFER+1;
							break;
						case "west":
							baseRects[0].rect.xy.x -= constants.BUFFER;
							baseRects[0].rect.wh.x += constants.BUFFER;
							baseRects[1].rect.xy.x -= constants.BUFFER+1;
							baseRects[1].rect.wh.x += constants.BUFFER+1;
							break;
						case "east":
							baseRects[0].rect.wh.x += constants.BUFFER;
							baseRects[1].rect.wh.x += constants.BUFFER+1;
							break;
						default:
							throw new Error ("bad direction: \"" + dir + "\"");
					}
				};
				doorBuilder = defaultWaterDoor;
				finalizer = many (
					[edgeBlobsFinalizer (3, 7, function (constants, section, dirs) {
						var dir = randomFrom (dirs);
						var width = randomIntInRange (2, 3);
						var offset = randomIntInRange (0, constants.FULL_DIMENSION - width);
						var depth = randomFrom ([2,2,3]);
						var r = edgeRect (dir, offset, width, depth, depth - 1);
						var t = randomFrom (["mud", "edge", "edge"]);
						var painter;
						var data;
						switch (t) {
							case "mud":
								data = {type: ["mudfield", "encounter"]};
								painter = "mudfield_low" //tilePainter (randomPainter (["ground_mud"]), staticPainter (-1));
								break;
							case "edge":
								data = {type: ["noencounter", "nolocation"]};
								painter = "swamp_fringe" //tilePainter (randomPainter (["ground_mud", "ground_mud", "ground_grass1", "ground_grass2"]), staticPainter (0));
								break;
						}
						return (
							{ rect: r
							, data: data
							, painter: painter
							, priority: 5.5
							}
						);
					})
					,waterWrapFinalizer
					]);
				break;
			case "shallows":
				landscapeBuilder = basicLandscape (constants.DIMENSION);
				localBuilder = defaultLocalConnection (0);
				doorBuilder = defaultWaterDoor;
				finalizer = waterWrapFinalizer;
				break;
			case "field":
				landscapeBuilder = basicLandscape (constants.DIMENSION);
				localBuilder = defaultLocalConnection (0);
				doorBuilder = basicDoor (2, constants.BUFFER);
				finalizer = many (
					[edgeBlobsFinalizer (1, 4, function (constants, section, dirs) {
						var dir = randomFrom (dirs);
						var width = randomIntInRange (3, 5);
						var offset = randomIntInRange (0, constants.FULL_DIMENSION - width);
						var depth = randomFrom ([2,2,3]);
						var r = edgeRect (dir, offset, width, depth, depth - 1);
						return (
							{ rect: r
							, data: {type: ["mudfield", "encounter"]}
							, painter: "mudfield_low" //tilePainter (staticPainter ("ground_mud"), staticPainter (-1))
							, priority: 5.5
							}
						);
					})
					,waterWrapFinalizer
					]);
				break;
			case "badland":
				landscapeBuilder = function (c, s) {
					function centeredRect (size) {
						var halfSize = Math.floor (size / 2);
						var midpoint = constants.FULL_DIMENSION % 2
							? Math.floor (constants.FULL_DIMENSION / 2)
							: (constants.FULL_DIMENSION / 2) - 1;
						return new Rect
							( new Point2d (midpoint - halfSize + 1, midpoint - halfSize + 1)
							, new Point2d (size, size)
							);
					}
					/*
					make a central landscape chunk:
						make one random 4x4 landscape square offset by uhh
						make two random 3x3 landscape squares
						make three random 2x2 landscape squares
					*/
					function randomSizedRect (size, dim) {
						var variance = dim - size + 1;
						var x = randomIntInRange (0, variance);
						var y = randomIntInRange (0, variance);
						return new Rect
							( new Point2d (x, y)
							, new Point2d (size, size)
							);
					}
					// make buckets to attach all the edges + all the 3-size rects? (but maybe as a finalizer since the edges don't exist yet)
					// also make specific painters with specific things? so only grass/hole trees/caves/obelisks/etc show up in certain locations
					var sizes = [6,5,4,4];
					sizes = sizes.concat (randomFrom([[3,3],[3,3,3],[3,3,3],[3,3,3,3]]));
					sizes = sizes.concat ([2,2,2,2]);
					function flatMap (arr, func) {
						var o = [];
						arr.map (function (v) {
							o = o.concat (func (v));
						});
						return o;
					}
					var ls = flatMap (sizes, function (s) {
						var r = randomSizedRect (s, constants.DIMENSION)
							.offset (constants.BUFFER, constants.BUFFER);
						var ts;
						var w = 5;
						var painter = "badland";
						switch (s) {
							case 2:
								ts = ["badland", "encounter"];
								break;
							case 3:
								ts = ["badland"];
								break;
							default:
								ts = ["badland", "encounter"];
								break;
						}
						if (s === 2 && randomInRange (0, 1) < 0.5) {
							painter += "T";
							w = 5.2;
						}
						if (s === 3) {
							painter = "badland-clearing";
							if (randomInRange (0, 1) < 0.33) {
								painter += "T";
							}
							w = 5.3;
							return (
								[ { rect: r
									, data: {type: ts}
									, painter: painter
									, priority: w
									}
								, { rect: new Rect (new Point2d (r.xy.x+1, r.xy.y+1), new Point2d (1,1))
									, data: {type: ["badland", "location"]}
									, painter: null
									, priority: 0
									}
								]);
						}
						return [
							{ rect: r
							, data: {type: ts}
							, painter: painter
							, priority: w
							}];
					});
					return ls;
				};
				localBuilder = function (c, s, dir, lands) {
					// place one rect in the buffer space, centered, w/ a random width b/t 4-7
					defaultDoorConnection ("badland", c.BUFFER, randomIntInRange (4, 7)) (c, s, dir, lands);
					return;
				}
				doorBuilder = function (c, s, dir, lands) {
					// place one rect in the buffer space, centered, w/ a random width b/t 1-3
					defaultDoorConnection ("badland", c.BUFFER, randomIntInRange (1, 3)) (c, s, dir, lands);
					return;
				}
				finalizer = function (constants, section, landscape) {
					// bucket the landscape and add bridges/rocks/sand/land/etc
					waterWrapFinalizer (constants, section, landscape);
				}
				break;
			default:
				landscapeBuilder = basicLandscape (constants.DIMENSION);
				localBuilder = defaultLocalConnection (0);
				doorBuilder = basicDoor (2, constants.BUFFER);
				finalizer = waterWrapFinalizer;
				break;
		}
		landscapeRects = landscapeBuilder (constants, section);

		var tileType;
		["north", "south", "east", "west"].map (function (dir) {
			if (section.adjs[dir] === null) {
				return;
			}
			switch (section.adjs[dir].type) {
				case "local":
					localBuilder (constants, section, dir, landscapeRects);
					break;
				case "door":
					doorBuilder (constants, section, dir, landscapeRects);
					break;
				case null:
				default:
					break;
			}
		});
		finalizer (constants, section, landscapeRects);

		// the goal is "any priority before no priority, and higher priority before lower priority"
		landscapeRects.sort (function (a, b) {
			if (a.priority === undefined && b.priority === undefined) {
				return 0;
			} else if (a.priority !== undefined && b.priority === undefined) {
				return -1;
			} else if (a.priority === undefined && b.priority === undefined) {
				return 1;
			} else {
				return b.priority - a.priority;
			}
		});

		section.landscape = { main: landscapeRects };

	}

	function tileGenerator (constants, section) {
		var pt;
		var inLandscape;
		var t;

		var tiles = [];
		var i = 0;
		var landscapeRects;

		landscapeRects = section.landscape;
		//console.log (landscapeRects);
		landscapeRects.reverse(); // this should not need to be here; the sorting above in `sectionDetailGenerator` should leave the list sorted by descending priority, but instead it seems to be sorted again somewhere else? since regardless of sort they end up being in ascending priority by this point.
		for (i = 0; i < constants.SIZE; i++) {
			pt = unlinearize (constants.FULL_DIMENSION, i);
			inLandscape = any (landscapeRects, function (landscape) {
				if (landscape.rect.in (pt)) {
					return landscape;
				}
				return false;
			});
			if (inLandscape && inLandscape.painter && inLandscape.painter.paint !== null) { // in main bulk
				if (painters[inLandscape.painter.paint] === undefined) {
					console.log (inLandscape.painter);
					console.error ("no painter for \"" + inLandscape.painter.paint + "\"");
				}
				t = painters[inLandscape.painter.paint](pt, inLandscape.rect);
				//t = inLandscape.painter (pt, inLandscape.rect);
				if (t instanceof Tile) {
					tiles[i] = t;
				} else {
					throw new Error ("bad tile: " + t);
				}
			} else { // outside region
				tiles[i] = null;
			}
		}
		return tiles;
	}

	// run across all tiles, after they've been merged together into a single giant map
	function tileFinisher (constants, tiles) {
	}

	var spritesheet = new Spritesheet (null, new Point2d (64, 64),
		[ { "name": "ground_grass1", "dim": new Point2d (0, 0)}
		, { "name": "ground_grass2", "dim": new Point2d (1, 0)}
		, { "name": "ground_sand", "dim": new Point2d (4, 0)}
		, { "name": "ground_rock", "dim": new Point2d (3, 0)}
		, { "name": "ground_mud", "dim": new Point2d (2, 0)}
		, { "name": "ground_bushes", "dim": new Point2d (2, 2)}
		, { "name": "ground_trees", "dim": new Point2d (2, 1)}

		, { "name": "factory_panel", "dim": new Point2d (5, 0)}
		, { "name": "factory_grating", "dim": new Point2d (6, 0)}
		, { "name": "factory_pit", "dim": new Point2d (7, 0)}
		, { "name": "factory_quads", "dim": new Point2d (5, 1)}
		, { "name": "factory_tatami_l", "dim": new Point2d (6, 1)}
		, { "name": "factory_tatami_r", "dim": new Point2d (7, 1)}

		, { "name": "ground_underground", "dim": new Point2d (5, 2)}
		, { "name": "quarry_floor", "dim": new Point2d (13, 6)}
		, { "name": "plant_floor", "dim": new Point2d (14, 6)}

		, { "name": "quarry_wall_left", "dim": new Point2d (13, 7)}
		, { "name": "quarry_wall_right", "dim": new Point2d (13, 8)}
		, { "name": "quarry_wall_center", "dim": new Point2d (13, 9)}

		, { "name": "ground_wall_top_left", "dim": new Point2d (0, 1)}
		, { "name": "ground_wall_top_right", "dim": new Point2d (0, 2)}
		, { "name": "ground_wall_top_center", "dim": new Point2d (0, 3)}

		, { "name": "ground_wall_left", "dim": new Point2d (1, 1)}
		, { "name": "ground_wall_right", "dim": new Point2d (1, 2)}
		, { "name": "ground_wall_center", "dim": new Point2d (1, 3)}

		, { "name": "factory_wall_left", "dim": new Point2d (3, 1)}
		, { "name": "factory_wall_right", "dim": new Point2d (3, 2)}
		, { "name": "factory_wall_center", "dim": new Point2d (3, 3)}

		, { "name": "library_boards", "dim": new Point2d (12, 2)
			//, "wall": {"top": "library_wall_top", "base": "library_wall"}
			}

		, { "name": "tar", "dim": new Point2d (13, 2)}

		, { "name": "factory_diagram", "dim": new Point2d (12, 4)}
		, { "name": "factory_floatingcrystal", "dim": new Point2d (13, 4)}
		, { "name": "factory_crystalbed", "dim": new Point2d (13, 3)}

		, { "name": "library_wall_top_corner", "dim": new Point2d (12, 3)}
		, { "name": "library_wall_top_left", "dim": new Point2d (14, 2)}
		, { "name": "library_wall_top_right", "dim": new Point2d (14, 3)}
		, { "name": "library_wall_top_center", "dim": new Point2d (14, 4)}

		, { "name": "library_wall_left", "dim": new Point2d (15, 2)}
		, { "name": "library_wall_right", "dim": new Point2d (15, 3)}
		, { "name": "library_wall_center", "dim": new Point2d (15, 4)}

		, { "name": "water", "dim": new Point2d (4, 3)}
		, { "name": "swamp_water", "dim": new Point2d (4, 1)}

		, { "name": "lilypad", "dim": new Point2d (0, 4)}
		, { "name": "lilypads", "dim": new Point2d (1, 4)}
		, { "name": "dead_tree", "dim": new Point2d (2, 4)}
		, { "name": "cattails", "dim": new Point2d (3, 4)}

		, { "name": "ground_barerock", "dim": new Point2d (4, 8)}
		, { "name": "ground_crater", "dim": new Point2d (5, 8)}
		, { "name": "ground_rock2", "dim": new Point2d (6, 8)}
		, { "name": "ground_grass_blue", "dim": new Point2d (0, 6)}

		, { "name": "ground_alien_left", "dim": new Point2d (0, 7)}
		, { "name": "ground_alien_right", "dim": new Point2d (0, 8)}
		, { "name": "ground_alien_center", "dim": new Point2d (0, 9)}

		, { "name": "broken_obelisk", "dim": new Point2d (4, 9)}
		, { "name": "alien_tree_mound", "dim": new Point2d (5, 9)}
		, { "name": "alien_tree_pods", "dim": new Point2d (6, 9)}
		, { "name": "alien_tree_blue", "dim": new Point2d (7, 9)}
		, { "name": "alien_tree_holes", "dim": new Point2d (8, 9)}
		, { "name": "alien_tree_truffula", "dim": new Point2d (9, 9)}

		, { "name": "alien_flowers", "dim": new Point2d (7, 10)}

		, { "name": "void", "dim": new Point2d (15, 7)}

		, { "name": "dryad", "dim": new Point2d (0, 15)}
		, { "name": "zombie", "dim": new Point2d (0, 15)}
		, { "name": "paperrook", "dim": new Point2d (0, 15)}
		, { "name": "zombiehierophant", "dim": new Point2d (0, 15)}
		, { "name": "shadowmage", "dim": new Point2d (0, 15)}
		, { "name": "sahagin", "dim": new Point2d (0, 15)}
		, { "name": "crystalgolem", "dim": new Point2d (0, 15)}
		, { "name": "tarpitdemon", "dim": new Point2d (0, 15)}
		, { "name": "humbaba", "dim": new Point2d (0, 15)}
		, { "name": "umu-demon", "dim": new Point2d (0, 15)}
		, { "name": "nature spirit", "dim": new Point2d (0, 15)}
		, { "name": "asakku", "dim": new Point2d (0, 15)}
		, { "name": "rock demon", "dim": new Point2d (0, 15)}

		, { "name": "amduscias", "dim": new Point2d (10, 14)}
		, { "name": "willowisp", "dim": new Point2d (9, 14)}
		, { "name": "feralhound", "dim": new Point2d (8, 14)}
		, { "name": "nightmare", "dim": new Point2d (2, 14)}
		, { "name": "shadow", "dim": new Point2d (7, 14)}
		, { "name": "sobek", "dim": new Point2d (1, 14)}
		, { "name": "voidworm", "dim": new Point2d (3, 14)}
		, { "name": "crystaldragonoid", "dim": new Point2d (4, 14)}
		, { "name": "unicorn", "dim": new Point2d (6, 14)}
		, { "name": "suiteddemon", "dim": new Point2d (5, 14)}

		, { "name": "clayman", "dim": new Point2d (1, 15)}
		, { "name": "houndknight", "dim": new Point2d (2, 15)}
		, { "name": "lesser hellhound", "dim": new Point2d (2, 15)}
		, { "name": "minotaurminer", "dim": new Point2d (3, 15)}
		, { "name": "gargoyle", "dim": new Point2d (4, 15)}
		, { "name": "flamedemon", "dim": new Point2d (5, 15)}
		, { "name": "goat flamedemon", "dim": new Point2d (5, 15)}
		, { "name": "son of hamar-tabal", "dim": new Point2d (6, 15)}
		, { "name": "slimeguy", "dim": new Point2d (7, 15)}
		, { "name": "ganzer", "dim": new Point2d (8, 15)}

		, { "name": "miningbarracks", "dim": new Point2d (8, 7)}
		, { "name": "mechanic", "dim": new Point2d (10, 7)}
		, { "name": "flarestack", "dim": new Point2d (9, 7)}

		, { "name": "cave", "dim": new Point2d (5, 3)}
		, { "name": "farmer", "dim": new Point2d (7, 4)}
		, { "name": "obelisk", "dim": new Point2d (6, 2)}
		, { "name": "hellgate", "dim": new Point2d (7, 3)}
		, { "name": "skull", "dim": new Point2d (4, 5)}
		, { "name": "cedartree", "dim": new Point2d (4, 4)}
		, { "name": "standing stones", "dim": new Point2d (6, 4)}
		, { "name": "thicket", "dim": new Point2d (6, 5)}

		, { "name": "bubbles", "dim": new Point2d (3, 5)}
		, { "name": "pile", "dim": new Point2d (6, 3)}
		, { "name": "oldbones", "dim": new Point2d (5, 5)}

		, { "name": "demoShop", "dim": new Point2d (6, 2)}

		, { "name": "outline_surface", "dim": new Point2d (15, 15)}
		, { "name": "pc", "dim": new Point2d (0, 14)}
		]);

	var lands =
		{ "factory-research":
			{ "solid": "rgb(207,191,95)"
			}
		, "factory-stacks":
			{ "solid": "rgb(223,223,191)"
			}
		, "factory-pulpit":
			{ "solid": "rgb(255,127,15)"
			}
		, "factory-quarry":
			{ "solid": "rgb(63,127,91)"
			}
		, "factory-tunnels":
			{ "solid": "rgb(15,63,91)"
			}
		, "factory-tarpit":
			{ "solid": "rgb(1,7,3)"
			}
		, "factory-processing":
			{ "solid": "rgb(191,255,0)"
			}
		, "factory-plant":
			{ "solid": "rgb(191,255,127)"
			}
		, "factory-grotto":
			{ "solid": "rgb(15, 63, 7)"
			}

		, "field":
			{ "solid": "rgb(161,191,127)"
			}
		, "badland":
			{ "solid": "rgb(191,95,63)"
			}
		, "shallows":
			{ "solid": "rgb(7,127,127)"
			}
		, "swamp":
			{ "solid": "rgb(31,191,95)"
			}
		, "glade":
			{ "solid": "rgb(63,127,63)"
			}
		};
	var specs =
		{ "vortex": "rgb(0,63,127)" // shallows
		, "standing_stones": "rgb(191,191,191)" // badland, glade
		, "gate": "rgb(63,15,127)" // badland
		, "obelisk": "rgb(91,31,161)" // badland
		, "cave": "rgb(127,31,127)" // badland
		, "skull": "rgb(255,231,191)" // swamp (zombie)
		, "hive": "rgb(0,31,63)" // swamp (zombie)
		, "nest": "rgb(191,31,191)" // swamp (sobek)
		, "farm": "rgb(163,231,127)" // field

		, "flare_stack": "rgb(255, 11, 31)" // factory-processing
		, "machine_shop": "rgb(127, 127, 127)" // factory-processing
		, "void_hole": "rgb(0,0,0)" // factory-tunnels (at the end)
		, "barracks": "rgb(127,191,63)" // factory-tunnels
		, "alchemy_lab": "rgb(221, 191, 191)" // factory-research
		};

	return (
		{ generatorInit: generatorInit
		, sectionGenerator: sectionGenerator
		, tileGenerator: tileGenerator
		, tileFinisher: tileFinisher
		, spritesheet: spritesheet
		, spritesheetImage: "/static/img/tiles5.png"
		, legend: {lands: lands, specs: specs}
		});
}());
