
function Rect (xy, wh) {
	this.xy = xy;
	this.wh = wh;
}
Rect.prototype =
	{ fromPoints: function (p1, p2) {
			var xy;
			var wh;
			if (p1.x <= p2.x && p1.y <= p2.y) {
				xy = new Point2d (p1.x, p1.y);
				wh = p2.subtract (p1);
			} else if (p1.x > p2.x && p1.y > p2.y) {
				xy = new Point2d (p2.x, p2.y);
				wh = p1.subtract (p2);
			} else if (p1.x <= p2.x) { // x1 <= x2 && y1 > y2
				xy = new Point2d (p1.x, p2.y);
				wh = new Point2d (p2.x - p1.x, p1.y - p2.y);
			} else { // x1 > x2 && y1 <= y2
				xy = new Point2d (p2.x, p1.y);
				wh = new Point2d (p1.x - p2.x, p2.y - p1.y);
			}
			return new Rect (xy, wh);
		}
	, offset: function (x, y) {
			return new Rect (new Point2d (this.xy.x + x, this.xy.y + y)
				, new Point2d (this.wh.x, this.wh.y));
		}
	, in: function (pt) {
			return (pt.x >= this.xy.x && pt.x < this.xy.x + this.wh.x) &&
				(pt.y >= this.xy.y && pt.y < this.xy.y + this.wh.y);
		}
	, onEdge: function (pt) {
			return pt.x == this.xy.x || pt.x == this.xy.x + this.wh.x - 1 ||
				pt.y == this.xy.y || pt.y == this.xy.y + this.wh.y - 1;
		}
	, alongEdge: function (pt) {
			return ((pt.x == this.xy.x-1 || pt.x == (this.xy.x + this.wh.x)) &&
				(pt.y+1 >= this.xy.y && pt.y <= this.xy.y + this.wh.y)) ||
				((pt.y == this.xy.y-1 || pt.y == (this.xy.y + this.wh.y)) &&
					(pt.x+1 >= this.xy.x && pt.x <= this.xy.x + this.wh.x));
		}
	// i'm guessing at the north/south/west/east directions here; they're probably wrong
	, edgeLabels: function (pt) {
			function within (lo, hi, v) {
				return v >= lo && v <= hi;
			}
			var o = [];
			var xlo = this.xy.x;
			var xhi = this.xy.x + this.wh.x - 1;
			var ylo = this.xy.y;
			var yhi = this.xy.y + this.wh.y - 1;
			if (pt.x == xlo && within (ylo, yhi, pt.y)) {
				o.push ("west");
			}
			if (pt.x == xhi && within (ylo, yhi, pt.y)) {
				o.push ("east");
			}
			if (pt.y == ylo && within (xlo, xhi, pt.x)) {
				o.push ("north");
			}
			if (pt.y == yhi && within (xlo, xhi, pt.x)) {
				o.push ("south");
			}
			return o;
		}
	, distanceFromEdge: function (pt) {
			var xLo = Math.abs (pt.x - this.xy.x);
			var xHi = Math.abs (pt.x - (this.wh.x + this.xy.x));
			var xDim = Math.min (xLo, xHi);
			var yLo = Math.abs (pt.y - this.xy.y);
			var yHi = Math.abs (pt.y - (this.wh.y + this.xy.y));
			var yDim = Math.min (yLo, yHi);

			return Math.min (xDim, yDim);
		}
	// haven't tested these (overlaps & closestPoint) so i don't know if they actually work yet
	// they've been tested a LITTLE now
	, overlaps: function (rect) {
			function dimCollide (offset1, length1, offset2, length2) {
				return offset1 < offset2 + length2 &&
					offset1 + length1 > offset2;
			}
			return dimCollide (this.xy.x, this.wh.x, rect.xy.x, rect.wh.x) &&
				dimCollide (this.xy.y, this.wh.y, rect.xy.y, rect.wh.y);
		}
	// this one hasn't been tho
	, touches: function (rect) {
			function dimCollide (offset1, length1, offset2, length2) {
				return offset1 <= offset2 + length2 &&
					offset1 + length1 >= offset2;
			}
			return dimCollide (this.xy.x, this.wh.x, rect.xy.x, rect.wh.x) &&
				dimCollide (this.xy.y, this.wh.y, rect.xy.y, rect.wh.y);
		}
	, closestPoint: function (rect) {
			var xCheck = checkSpacing (this.xy.x, this.wh.x, rect.xy.x, rect.wh.x);
			var yCheck = checkSpacing (this.xy.y, this.wh.y, rect.xy.y, rect.wh.y);
			if (xCheck.kind === "overlap" && yCheck.kind === "overlap") {
				return {result: "intersecting"};
			} else if (xCheck.kind === "none" && yCheck.kind === "none") {
				return {result: "point"
					, point: new Point2d (xCheck.offset, yCheck.offset)
					, distance: new Point2d (xCheck.gap, yCheck.gap)
					};
			}
			var edge = xCheck.kind === "overlap"
				? new Rect (new Point2d (xCheck.offset, yCheck.offset), new Point2d (xCheck.length, 0))
				: new Rect (new Point2d (xCheck.offset, yCheck.offset), new Point2d (0, yCheck.length));
			var distance = xCheck.kind === "overlap"
				? new Point2d (0, yCheck.gap)
				: new Point2d (xCheck.gap, 0);
			return {result: "edge"
				, edge: edge
				, distance: distance
				};
		}
	};
