
function Height (val, h1, h2, h3, h4) {
	var error = new Error ("bad slope values: [" + h1 + ", " + h2 + ", " + h3 + ", " + h4 + "]");
	this.type = val;
	switch (val) {
		case "flat":
			this.base = h1;
			this.heightCode = "000";
			this.slopes = [{n: ""}];
			break;
		case "uneven":
			this.base = h1;
			this.corners = [h2 - h1, h3 - h1, h4 - h1];
			// the four corners of a tile can each move at most ±1 around (b/c if it's more than that we don't have the sprites to render it), so if we have height values that don't conform to that then we have to break
			this.corners.reduce (function (last, c) {
				if (Math.abs (last - c) > 1) {
					throw error;
				}
				return c;
			}, 0);
			if (Math.abs (this.corners[2]) > 1) {
				throw error;
			}
			this.lowest = this.base + this.corners.reduce (function (a, b) {
				return a < b ? a : b;
			}, 0);
			// listen i couldn't think of a good way to integer-encode this, so this is what you get. sorry.
			this.heightCode = [2,0,1][this.corners[0]+1] + "" +
				[2,0,1][this.corners[2]+1] + "" +
				[4,2,0,1,3][this.corners[1]+2];
			//console.log (h1 + " " + h2 + " " + h3 + " " + h4 + ";" + this.corners + ";" + this.heightCode);
			// this switch SHOULD be turned into an array index or object names or smth, though
			switch (this.heightCode) {
				case "001":
					this.slopes = [{n: "_left_flat"}, {n: "_right_slope_up", o: 1}];
					break;
				case "002":
					this.slopes = [{n: "_left_flat"}, {n: "_right_slope_down"}];
					break;
				case "010":
					this.slopes = [{n: "_top_flat"}, {n: "_bottom_slope_up"}];
					break;
				case "011":
					this.slopes = [{n: "_fullslope_bottomright", o: 1}];
					break;
				case "020":
					this.slopes = [{n: "_top_flat"}, {n: "_bottom_slope_down"}];
					break;
				case "022":
					this.slopes = [{n: "_fullslope_topleft", o: 1}];
					break;
				case "100":
					this.slopes = [{n: "_top_slope_up", o: 1}, {n: "_bottom_flat"}];
					break;
				case "101":
					this.slopes = [{n: "_fullslope_topright", o: 1}];
					break;
				case "110":
					this.slopes = [{n: "_top_slope_up", o: 1}, {n: "_bottom_slope_up"}];
					break;
				case "111":
					this.slopes = [{n: "_left_slope_down", o: 1}, {n: "_right_flat", o: 1}];
					break;
				case "113":
					this.slopes = [{n: "_left_slope_down", o: 1}, {n: "_right_slope_up", o: 1}];
					break;
				case "120":
					this.slopes = [{n: "_top_slope_up", o: 1}, {n: "_bottom_slope_down"}];
					break;
				case "200":
					this.slopes = [{n: "_top_slope_down"}, {n: "_bottom_flat"}];
					break;
				case "202":
					this.slopes = [{n: "_fullslope_bottomleft", o: 1}];
					break;
				case "210":
					this.slopes = [{n: "_top_slope_down"}, {n: "_bottom_slope_up"}];
					break;
				case "220":
					this.slopes = [{n: "_left_slope_up"}, {n: "_right_slope_up"}];
					break;
				case "222":
					this.slopes = [{n: "_left_slope_up"}, {n: "_right_flat", o: -1}];
					break;
				case "224":
					this.slopes = [{n: "_left_slope_up"}, {n: "_right_slope_down", o: -1}];
					break;
				default:
					throw new error ("bad height code \"" + this.heightCode + "\"");
					break;
			}
			break;
		default:
			throw new Error ("bad height type \"" + val + "\"");
	}
}
Height.prototype =
	{ edge: function (edge) {
			if (this.type === "flat") {
				return [this.base, this.base];
			}
			switch (edge) {
				case "upperLeft":
					return [this.base, this.base + this.corners[0]];
				case "upperRight":
					return [this.base + this.corners[0], this.base + this.corners[1]];
				case "lowerRight":
					return [this.base + this.corners[1], this.base + this.corners[2]];
				case "lowerLeft":
					return [this.base + this.corners[2], this.base];
					break;
				default:
					throw new Error ("bad height query on \"" + edge + "\"");
			}
		}
	, edgeMax: function (edge) {
			return this.edge (edge).reduce (function (a, b) {
				return a < b ? b : a;
			}, -9999);
		}
	, edgeMin: function (edge) {
			return this.edge (edge).reduce (function (a, b) {
				return a < b ? a : b;
			}, 9999);
		}
	, baseHeight: function () {
			if (this.type === "flat") {
				return this.base;
			} else {
				return this.lowest;
			}
		}
	, slopeNames: function () {
			return this.slopes;
		}
	};

function Tile (height, type, encounter, decoration) {
	this.h = height;
	this.t = type;
	this.o = encounter ? encounter : null;
	this.d = decoration ? decoration : null
}
Tile.prototype =
	{ edge: function (edge) {
			return this.h.edge(edge);
		}
	, surfaceSprites: function () {
			var this_ = this;
			//console.log (this);
			return this.h.slopeNames().map (function (n) {
				return {n: this_.t + n.n, o: n.o ? n.o : 0};
			});
		}
	, cursorSprites: function () {
			var this_ = this;
			//console.log (this);
			if (this.h === null) {
				return [{n: "outline_surface", o: 0}];
			} else {
				return this.h.slopeNames().map (function (n) {
					return {n: "outline_surface" + n.n, o: n.o ? n.o : 0};
				});
			}
		}
	};


// maybe extract out this + the various coordinate functions from `map` into some static coordinate-mapping object b/c this is getting thorny given the amount of times i have to pass `renderer` or `mapData` around
function getCursorSlices (renderer, px, camera) {
	var cursorTile = renderer.fromPx (px.x, px.y);
	var cursorTilePx = renderer.tile (cursorTile.x, cursorTile.y);
	var cursorTileScreenOffset = camera.add (cursorTilePx);
	cursorTileScreenOffset.y += 32; // the sprites are 64x64 images; the tiles only occupy the bottom half, so we should add half the tile height if we want the top-left corner of the visible tile
	var tilePx = px.subtract (cursorTileScreenOffset);
	var slices =
		{ x: cursorTile.x, y: cursorTile.y
		, vert: tilePx.x <= 32 ? "left" : "right"
		, xSplit: (1 - (tilePx.x / 64)) > (tilePx.y / 32) ? "top" : "bottom"
		, ySplit: (tilePx.x / 64) > (tilePx.y / 32) ? "top" : "bottom"
		};
	return slices;
}

function Map (generator, canvas, renderer, dim) {
	var MAP_DIMENSION = dim.MAP_DIMENSION;
	var MAP_BUFFER = dim.MAP_BUFFER;
	var SECTION_COUNT = dim.SECTION_COUNT;

	this.data = new MapData
		( SECTION_COUNT
		, generator.sectionGenerator
		, MAP_DIMENSION
		, MAP_BUFFER
		, generator.tileGenerator
		, generator.generatorInit
		, generator.tileFinisher
		);
	this.renderer = new MapRenderer (renderer, canvas, canvas.getContext("2d"));

	this.cursorFlash = null;
	this.spritesheet = null;
	this.spritesheetImage = generator.spritesheetImage;
}
Map.prototype =
	{ generateMap: function () {
			return this.data.generateMap();
		}
	, centerCameraOnTile: function (x, y) {
			return this.renderer.centerCameraOnTile (x, y);
		}
	, renderCanvas: function () {
			return this.renderer.renderCanvas (this.data.data(), this.spritesheet);
		}
	, cursorTileResolve: function (cursorPx) {
			return this.data.cursorTileResolve (getCursorSlices (this.renderer, cursorPx, this.renderer.camera));
		}
	, smoothCameraToTile: function (x, y, ms, cursorPx, callback) {
			return this.renderer.smoothCameraToTile (x, y, 400, this.renderer, this.data, this.data.data(), this.spritesheet, cursorPx, this.cursorFlash, callback);
		}
	, destroy: function () {
			// removes dangling event listeners from the canvas
			this.renderer.destroy();
		}
	, createFlash: function () {
			this.cursorFlash = new StoredDraw
				( null
				, this.spritesheet.get ("outline_surface").dim
				, this.spritesheet.size
				, new Point2d (0, 0)
				, this.spritesheet.size
				);
		}
	, setSpritesheet: function (spritesheet) {
			var image = new Image();
			var this_ = this;
			this.spritesheet = spritesheet;

			image.onload = function () {
				this_.spritesheet.image = this;
				this_.cursorFlash.image = this;

				this_.renderCanvas();
			};
			image.src = this.spritesheetImage;
			//console.log ("setting map spritesheet image to \"" + this.spritesheetImage + "\".");
		}
	, reflash: function (cursorCoords) {
			// redraw map
			this.renderer.reflash("map", {coord: cursorCoords, flash: this.cursorFlash});
			// draw semi-transparent cursor overlay
			this.renderer.context.globalAlpha = 0.33;
			this.cursorFlash.draw (this.renderer.context, this.renderer.camera);
			this.renderer.context.globalAlpha = 1.0;
		}
	, setCamera: function (pt) {
			this.renderer.centerCameraOnTile (pt.x, pt.y);
		}
	, clearObject: function (coord) {
			this.setObject (coord, null);
		}
	, setObject: function (coord, obj) {
			var t = this.data.tile (coord.x, coord.y);
			if (t === null) {
				return;
			}
			t.o = obj;
		}
	, setDecoration: function (coord, decor) {
			var t = this.data.tile (coord.x, coord.y);
			if (t === null) {
				return;
			}
			t.d = decor;
		}
	, getTile: function (coord) {
		return this.data.tile (coord.x, coord.y);
	}
	, getObject: function (coord) {
			var t = this.data.tile (coord.x, coord.y);
			if (t === null) {
				return undefined;
			}
			return t.o;
		}
	, getDecoration: function (coord) {
			var t = this.data.tile (coord.x, coord.y);
			if (t === null) {
				return undefined;
			}
			return t.d;
		}
	, serializeMap: function () {
			return (
				{ spawns: this.data.mapSpawns
				});
		}
	};


function canBePlaced (enctype, reqflags) {
	return function (chunk) {
		return suitableRects (chunk, enctype, reqflags).length > 0;
	};
}
function suitableRects (chunk, enctype, reqflags) {
	return chunk.landscape.filter (function (landscape) {
			// has enctype
		return (landscape.flags.indexOf (enctype) !== -1) &&
			// has at least one zone flag
			any (landscape.flags, function (flag) { return reqflags.indexOf (flag) !== -1; });
	});
}
function pickFromLandscapes (landscapes) {
	var l = randomFrom (landscapes);
	return pickFromRect (l.rect);
}
function pickFromRect (rect) {
	var x = randomIntInRange (rect.xy.x, rect.xy.x + rect.wh.x);
	var y = randomIntInRange (rect.xy.y, rect.xy.y + rect.wh.y);
	return new Point2d (x, y);
}


function MapData (sectionDim, sectionGenerator, dimension, buffer, tileGenerator, generatorInit, tileFinisher) {
	this.dimension = dimension; // generate inside this range
	this.buffer = buffer;
	this.fullDimension = dimension + (buffer * 2); // but calculate tiles within this range;
	this.sectionDimension = sectionDim;

	this.sectionGenerator = sectionGenerator;

	this.generator = tileGenerator;
	this.generatorInit = generatorInit;

	if (tileFinisher) {
		this.tileFinisher = tileFinisher;
	}
	this.paths = {};

	this.sideLength = this.fullDimension * this.sectionDimension;
	this.tileCount = this.sideLength * this.sideLength;

	this.mapTiles = Array(this.tileCount);
	fillArray (this.mapTiles, null);
}
MapData.prototype =
	{ generateMap: function () {
			var tries = 5;
			var attempts = 0;
			this.sectionData = null;
			while (this.sectionData === null) {
				try {
					this.sectionData = this.sectionGenerator (this.constants());
				} catch (e) {
					attempts++;
					if (attempts < tries) {
						console.error (e);
						console.warn ("section data failed to generate; retrying");
					} else {
						throw e;
					}
				}
			}

			this.mapSpawns = this.sectionData.array.filter (function (section) {
				return section.type !== "empty";
			}).map (function (section) {
				return (
					{ id: section.id // used for unlinearizing to get coord offset
					, flags: section.flags
					, landscape: section.landscape
						? section.landscape.main.map (function (land) {
								return (
									{ rect: land.rect
									, flags: land.data.type
									, painter:
										{ priority: land.priority
										, paint: land.painter
										}
									});
							}).sort (function (a, b) {
								return a.painter.priority - b.painter.priority;
							})
						: []
					, raw: section
					});
			});

			this.generatorInit ();
			this.generateTiles ();
			if (this.tileFinisher) {
				this.tileFinisher (this.constants(), this.mapTiles);
			}

			console.log ("MAP GENERATED");
			//console.log (this.sectionData);
			//console.log (this.mapTiles);
			//console.log ("SPAWN DATA:");
			//console.log (this.mapSpawns);
		}
	, generateTiles: function () {
			this.mapSpawns.map (function (section) {
				var rawTiles = this.generator (this.constants(), section);
				var c = unlinearize (this.sectionDimension, section.id);
				this.setSectionTiles (c, rawTiles);
			}.bind(this));
		}
	, setSectionTiles: function (coords, tiles) {
			var sectionOffset = new Point2d (coords.x * this.fullDimension, coords.y * this.fullDimension);
			var i = 0;
			var localCoord;
			var l;
			//console.log ("setSectionTiles: length: " + tiles.length);
			for (i = 0; i < tiles.length; i++) {
				localCoord = unlinearize (this.fullDimension, i);
				l = linearize(this.sideLength, sectionOffset.x + localCoord.x, sectionOffset.y + localCoord.y);
				this.mapTiles[l] = tiles[i];
			}
		}
	, getSectionData: function () {
			return this.sectionData;
		}
	, getSection: function (pt) {
			return this.sectionData.array[linearize (this.sectionDimension, pt.x, pt.y)];
		}
	, constants: function () {
			return {DIMENSION: this.dimension
				, BUFFER: this.buffer
				, FULL_DIMENSION: this.fullDimension
				, SIZE: this.fullDimension * this.fullDimension
				, SECTION_DIMENSION: this.sectionDimension
				, SECTION_SIZE: this.sectionDimension * this.sectionDimension
				, FULL_MAP_DIMENSION: this.sectionDimension * this.fullDimension
				, FULL_MAP_SIZE: this.sectionDimension * this.sectionDimension * this.fullDimension * this.fullDimension
				};
		}
	, tile: function (x, y) {
			var i = this.tileIndex (x, y);
			if (i < 0 || i >= this.sideLength * this.sideLength) {
				return null;
			}
			return this.mapTiles[i];
		}
	, tileIndex: function (x, y) {
			var rx;
			var ry;
			if (typeof x === "object") {
				rx = x.x;
				ry = x.y;
			} else {
				rx = x;
				ry = y;
			}
			return linearize (this.sideLength, rx, ry);
		}
	, data: function () {
			return {edgeLength: this.sideLength, tiles: this.mapTiles};
		}
	, cursorTileResolve: function (slices) {
			/* var slices =
				{ x: cursorTile.x, y: cursorTile.y
				, vert: tilePx.x <= 32 ? "left" : "right"
				, xSplit: (1 - (tilePx.x / 64)) > (tilePx.y / 32) ? "top" : "bottom"
				, ySplit: (tilePx.x / 64) > (tilePx.y / 32) ? "top" : "bottom"
				}; */
			var base = new Point2d (slices.x, slices.y);
			var match = base;
			var adjLeft;
			var adjRight;
			var adjBelow;
			var tile;

			//console.log ("cursorTileResolve:");
			//console.log (slices);

			/* bear in mind the default height match is -1!! not 0!
			*/
			var i = 1;
			var requiredHeight = 0;
			var baseHeight;
			while (i < 5) {
				requiredHeight = (i - 1) * 2;
				if (slices.vert == "left") {
					adjLeft = new Point2d (base.x + (i - 1), base.y + i);
					tile = this.tile (adjLeft);
					baseHeight = tile && tile.h !== null ? tile.h.baseHeight() : -1;
					// .edge
					if (tile && tile.h !== null
							&& (baseHeight > requiredHeight
								|| (baseHeight == requiredHeight && slices.ySplit == "bottom"))) {
						match = adjLeft;
						//console.log ("left(" + i + ") (" + adjLeft.x + "," + adjLeft.y + ") looks good");
						//console.log ("tile: {h:" + tile.h + " ... }");
					}
				}
				if (slices.vert == "right") {
					adjRight = new Point2d (base.x + i, base.y + (i - 1));
					tile = this.tile (adjRight);
					baseHeight = tile && tile.h !== null ? tile.h.baseHeight() : -1;
					if (tile && tile.h !== null
						&& (baseHeight > requiredHeight
							|| (baseHeight == requiredHeight && slices.xSplit == "bottom"))) {
						match = adjRight;
						//console.log ("right(" + i + ") (" + adjRight.x + "," + adjRight.y + ") looks good");
						//console.log ("tile: {h:" + tile.h + " ... }");
					}
				}
				adjBelow = new Point2d (base.x + i, base.y + i);
				tile = this.tile (adjBelow);
				baseHeight = tile && tile.h !== null ? tile.h.baseHeight() : -1;
				if (tile && tile.h !== null
					&& (baseHeight > requiredHeight
						|| (baseHeight == requiredHeight && slices.xSplit == "bottom" && slices.ySplit == "bottom"))) {
					match = adjBelow;
					//console.log ("below(" + i + ") (" + adjBelow.x + "," + adjBelow.y + ") looks good");
					//console.log ("tile: {h:" + tile.h + " ... }");
				}
				i++;
			}

			//console.log ("final tile: " + match.x + "," + match.y);
			return match;
		}
	, updatePathingFrom: function (start, range, passFunc) {
			var
				open = [new Point2d (start.x, start.y)],
				info = {},
				activeInfo;
			if (this.tile(start.x, start.y) === null) {
				console.error ("updatePathingFrom: tile @ " + start.x + "," + start.y + " is null.");
				this.paths = info;
				return info;
			}
			info[this.tileIndex(start.x, start.y)] =
				{ x: start.x
				, y: start.y
				, h: this.tile(start.x, start.y).h
				, length: 0
				, parent: null
				};
			var this_ = this;
			function adjacent (pt) {
				var neighbors = [];
				var tile = this_.tile (pt.x, pt.y);
				//console.log (tile);
				if (tile === undefined || tile.t === null) {
					//console.log ("no such tile " + pt.x + "," + pt.y);
					return [];
				}
				// this ordering is important: this prefers straight-line moves over diagonal moves, so that when there are distance ties the prettier-looking path is picked.
				neighbors.push (new Point2d (pt.x    , pt.y + 1));
				neighbors.push (new Point2d (pt.x + 1, pt.y));
				neighbors.push (new Point2d (pt.x - 1, pt.y));
				neighbors.push (new Point2d (pt.x    , pt.y - 1));

				neighbors.push (new Point2d (pt.x + 1, pt.y + 1));
				neighbors.push (new Point2d (pt.x - 1, pt.y + 1));
				neighbors.push (new Point2d (pt.x + 1, pt.y - 1));
				neighbors.push (new Point2d (pt.x - 1, pt.y - 1));
				neighbors = neighbors.filter (function (n) {
					var ntile = this_.tile (n.x, n.y);
					//console.log ("PASS(" + nt + " @ " + n.x + "," + n.y + ") = " + passFunc (tile, nt));
					return passFunc (tile, pt, ntile, n);
				});
				return neighbors;
			}
			while (active = open.shift()) {
				//console.log ("checking " + active.x + "," + active.y);
				//console.log (active);
				activeInfo = info[this.tileIndex (active.x, active.y)];
				if (activeInfo.length >= range) {
					break;
				}
				neighbors = adjacent (active);
				//console.log (neighbors);
				neighbors.map (function (neighbor) {
					var neighborInfo = info[this_.tileIndex (neighbor.x, neighbor.y)];
					var n;
					if (neighborInfo === undefined) {
						n = this_.tile (neighbor.x, neighbor.y);
						if (n !== null) {
							neighborInfo =
								{ x: neighbor.x
								, y: neighbor.y
								// set height value, which we need way later when rendering these paths
								, h: n.h
								, length: activeInfo.length + 1
								, parent: activeInfo
								};
							info[this_.tileIndex (neighbor.x, neighbor.y)] = neighborInfo;
							//console.log ("adding " + neighbor.x + "," + neighbor.y);
							open.push (new Point2d (neighbor.x, neighbor.y));
						}
					} else if (neighborInfo.length > activeInfo.length + 1) {
						//console.log ("rerouting " + neighbor.x + "," + neighbor.y);
						neighborInfo.length = activeInfo.length + 1;
						neighborInfo.parent = activeInfo;
					}
				});
			}
			this.paths = info;
			return info;
		}
	, getPathTo: function (t) {
			var o = [];
			var m = find (this.paths, function (t_) {
				return t.equal(t_);
			});
			if (m === undefined) {
				return o;
			}
			while (m !== null) {
				o.push (m);
				m = m.parent;
			}
			return o;
		}
	, runPopulationQuery: function (seed, callback) {
			var this_ = this;
			console.log ("sending population query:");
			this.queryState = "PENDING";
			// TODO: manually add in pc start location data, and place a 'pc event' manually
			// maybe?? should pc state be included here??
			jQuery.getJSON ("/hell/pop", {seed: seed, map: "hubur"}, function (result, textStatus, jqHXR) {
				var chunks;
				console.log ("got population response");
				//console.log (result.loc);
				//console.log (result.pop);
				//console.log (result.zoneTemplate);
				chunks = randomShuffle (this_.mapSpawns);
				//console.log ("chunks:");
				//console.log (chunks);

				function placeEncounter (reqtype) {
					return function (event) {
						var chunk;
						var places;
						var chunkOffset;
						var coord;
						var tile;
						var tries = 0;
						var r = null;
						var enc = event.event;
						if (!result.zoneTemplate[enc.name]) {
							console.warn ("no zone template for event \"" + enc.name + "\"");
							return null;
						}
						// find a region that contains reqtype + one of zones
						var chunk = findExtract (chunks, canBePlaced (reqtype, result.zoneTemplate[enc.name].zones));
						if (chunk === null) {
							//console.warn ("no appropriate location for event \"" + enc.name + "\"");
							return null;
						}
						places = suitableRects (chunk, reqtype, result.zoneTemplate[enc.name].zones);
						chunkOffset = unlinearize (this_.sectionDimension, chunk.id);
						chunkOffset.x *= (this_.dimension + this_.buffer * 2);
						chunkOffset.y *= (this_.dimension + this_.buffer * 2);

						coord = pickFromLandscapes (places).add (chunkOffset);
						tile = this_.mapTiles[linearize (this_.fullDimension * this_.sectionDimension, coord.x, coord.y)];
						while (tile.o !== null && tries < 20) {
							coord = pickFromLandscapes (places).add (chunkOffset);
							tile = this_.mapTiles[linearize (this_.fullDimension * this_.sectionDimension, coord.x, coord.y)];
							tries++;
						}
						if (tile.o === null) {
							// encode the encounter info later, elsewhere, i guess
							//console.log ("adding \"" + enc.name + "\" to chunk " + chunk.id + " @ {" + coord.x + "," + coord.y + "}");
							//console.log (tile);
							// IDEALLY we wouldn't be mutating the map here, but right now we need to do this to prevent encounters from spawning on top of each other
							if (enc.name !== "pcLocation") { // this is a bit of a hack, but whatever
								tile.o = enc.name;
							}
							r = {loc: new Point2d (coord.x, coord.y), ev: event};
						} else {
							console.log ("skipping placing encounter \"" + enc.name + "\"; couldn't find free space");
						}
						chunks.push (chunk);
						return r;
					}
				}

				var placedEncounters = [];
				placedEncounters = placedEncounters.concat (
					result.loc
						.map (placeEncounter ("location"))
						.filter (function (v) { return v !== null; })
					);
				placedEncounters = placedEncounters.concat (
					result.pop
						.map (placeEncounter ("encounter"))
						.filter (function (v) { return v !== null; })
					);
				var playerSpawn = placeEncounter ("location") ({event: {name: "pcLocation"}, id: -1}).loc;

				this_.placedEncounters = placedEncounters;
				//console.log ("PLACED ENCOUNTERS:");
				//console.log (placedEncounters);

				this_.queryState = "RECEIVED";

				if (callback !== undefined) {
					callback (this_, playerSpawn);
				}
			}) .fail (function () {
				console.log ("population query failed!!");
				this_.queryState = "FAILED";
			});
		}
	};

function MapRenderer (tileRenderer, canvas, context, camera) {
	this.canvas = canvas;
	this.context = context;
	this.context.fillStyle = "black";
	if (camera !== undefined) {
		this.camera = camera;
	} else {
		this.camera = new Point2d (Math.floor (this.canvas.width / 2), Math.floor (this.canvas.height / 2));
	}
	this.focusedTile = this.fromPx (Math.floor (this.canvas.width / 2), Math.floor (this.canvas.height / 2));
	this.renderTiles = tileRenderer;
	this.recalculateBounding();
	this.cache =
		{ active: false
		, store: {}
		};
	this.animation = null;

	this.callbacks =
		{ moveCallback: null
		, clickCallback: null
		};
}
MapRenderer.prototype =
	{ drawSpriteRaw: function (spritesheet, sprite, x, y, bounds, coords) {
			var ux;
			var uy;
			if (typeof x === "object") {
				ux = x.x;
				uy = x.y;
			} else {
				ux = x;
				uy = y;
			}
			//console.log (spritesheet.image);
			this.context.drawImage (spritesheet.image
				, sprite.dim.x
				, sprite.dim.y
				, spritesheet.size.x
				, spritesheet.size.y
				, this.camera.x + ux
				, this.camera.y + uy
				, spritesheet.size.x
				, spritesheet.size.y);
			if (this.cache.active) {
				this.cache.store[this.cache.active].push (
					{ bounds: bounds
					, coords: coords
					, draw: new StoredDraw (spritesheet.image, sprite.dim, spritesheet.size, new Point2d (ux, uy), spritesheet.size)
					});
			}
		}
	, drawSprite: function (spritesheet, sprite, type, x, y, height) {
			if (!this.inBounds (x, y, height)) {
				return;
			}
			//console.log ("sprite in bounds:");
			//console.log (sprite);
			var px = type === "tile" ? this.tile (x, y, height) : this.obj (x, y, height);
			this.drawSpriteRaw (spritesheet, sprite, px.x, px.y, this.calcBounds (x, y, height), {x: x, y: y, height: height});
		}
	, beginDraw: function (layer) {
			if (this.cache.active !== false) {
				console.error ("MAP RUNNING \"" + layer + "\" when already running \"" + this.cache.active + "\"!!");
			}
			this.cache.active = layer;
			this.cache.store[layer] = [];
		}
	, doneDraw: function () {
			this.cache.active = false;
		}
	, drawOver: function (f) {
			return f (this.context, this.camera);
		}
	, reflash: function (layer, cursor) {
			var this_ = this;
			var cursorDrawn = false;
			this.context.fillRect (0, 0, this.canvas.width, this.canvas.height);
			if (this.cache.store[layer] === undefined) {
				console.error ("NO SUCH LAYER \"" + layer + "\" in renderer!");
			}
			//console.log ("in reflash(\"" + layer + "\")");
			//console.log (this.cache.store[layer]);
			this.cache.store[layer].map (function (s) {
				s.draw.draw (this_.context, this_.camera);
				if (!cursorDrawn) {
					// note that this means the cursor flash will only be drawn when the cursor is actually over a tile, and not in the void. hence the else if below
					if (s.coords.x == cursor.coord.x && s.coords.y == cursor.coord.y) {
						//console.log ("drawing cursor flash (@ " + cursor.coord.x + "," + cursor.coord.y + "; h: " + s.coords.height + ")");
						cursor.flash.placement = cursor.flash.placement.plus (this_.obj (cursor.coord.x, cursor.coord.y, s.coords.height));
						cursor.flash.draw (this_.context, this_.camera);
						cursorDrawn = true;
					} else if (s.coords.y > cursor.coord.y) { // not sure if this ifcheck is 100% sound but it seems to work in all cases that i know of
						// okay no it breaks on the bottom left edge of the map
						//console.log ("drawing cursor flash (@ " + cursor.coord.x + "," + cursor.coord.y + "; h: -1) (over void)");
						cursor.flash.placement = this_.obj (cursor.coord.x, cursor.coord.y, -1);
						cursor.flash.draw (this_.context, this_.camera);
						cursorDrawn = true;
					}
				}
			});
			if (!cursorDrawn) {
				cursor.flash.placement = this_.obj (cursor.coord.x, cursor.coord.y, -1);
				cursor.flash.draw (this_.context, this_.camera);
				cursorDrawn = true;
			}
		}
	, centerCameraOnTile: function (x, y) {
			var offset = this.tile (x, y, 0);
			this.camera.x = Math.floor (this.canvas.width / 2) - offset.x;
			this.camera.y = Math.floor (this.canvas.height / 2) - offset.y;

			this.focusedTile = new Point2d (x, y);

			this.recalculateBounding ();
		}
	, smoothCameraToTile: function (x, y, ms, renderer, mapData, data, spritesheet, cursorPx, cursorFlash, callback) {
			function sine (f) {
				return Math.sin (f * Math.PI / 2);
			}
			var this_ = this;
			var startTile = this.fromPx (this.canvas.width / 2, this.canvas.height / 2);
			var start = this.tile (startTile.x, startTile.y, 0);
			// i don't really know why this offset is needed but i assume it's the same reason that we need an offset with fromPx
			start.x += 32;
			start.y += 48;
			var finish = this.tile (x, y, 0);
			var diff = new Point2d (start.x - finish.x, start.y - finish.y);
			var cursorCoords = this.fromPx (cursorPx.x, cursorPx.y);
			this.recalculateBounding (new Point2d (x, y));
			this.renderCanvas (data, spritesheet);
			var startT = new Date().getTime();
			var finishT = startT + ms;
			if (this.animation !== null) {
				clearInterval (this.animation);
			}
			//console.log ("start: " + start.x + "," + start.y);
			//console.log ("finish: " + finish.x + "," + finish.y);
			//console.log ("camera at start: " + this.camera.x + "," + this.camera.y);
			this.animation = setInterval(function () {
				var now = new Date().getTime();
				var elapsed = (now - startT) / ms;
				if (elapsed >= 1) {
					clearInterval (this_.animation);
					this_.animation = null;
					cursorCoords = mapData.cursorTileResolve (getCursorSlices (renderer, cursorPx, renderer.camera));
					this_.centerCameraOnTile (x, y);

					this_.renderCanvas(data, spritesheet);

					this_.reflash("map", {coord: cursorCoords, flash: cursorFlash});
					this_.context.globalAlpha = 0.33;
					cursorFlash.draw (this_.context, this_.camera);
					this_.context.globalAlpha = 1.0;

					this_.focusedTile = new Point2d (x, y);
					//updateCoordDisplay(cursorCoords);
					if (callback) {
						callback();
					}
					return;
				}
				elapsed = sine (elapsed);
				//console.log (diff.x + "," + diff.y + " / elapsed:" + elapsed);
				this_.camera.x = Math.floor (this_.canvas.width / 2) - Math.floor (start.x - (diff.x * elapsed));
				this_.camera.y = Math.floor (this_.canvas.height / 2) - Math.floor (start.y - (diff.y * elapsed));

				cursorCoords = mapData.cursorTileResolve (getCursorSlices (renderer, cursorPx, renderer.camera));

				this_.reflash("map", {coord: cursorCoords, flash: cursorFlash});
				this_.context.globalAlpha = 0.33;
				cursorFlash.draw (this_.context, this_.camera);
				this_.context.globalAlpha = 1.0;
				//updateCoordDisplay(cursorCoords);
			}, 17);
		}
	, recalculateBounding: function (target) {
			var TILE_WIDTH = 64;
			var TILE_HEIGHT = 32;
			var centers = [];
			var xSpan;
			var ySpan;

			xSpan = Math.floor (this.canvas.width / TILE_WIDTH / 2);
			ySpan = Math.floor (this.canvas.height / TILE_HEIGHT / 2);

			centers.push (this.fromPx (Math.floor (this.canvas.width / 2), Math.floor (this.canvas.height / 2))); // tile at center of viewport
			if (target) {
				centers.push (target);
			}
			var leftEdge = centers.reduce (function (acc, t) {
				var v = new Point2d (t.x - xSpan, t.y + (xSpan + 2));
				var edge = v.x - v.y;
				return edge < acc ? edge : acc;
			}, 999);
			var rightEdge = centers.reduce (function (acc, t) {
				var v = new Point2d (t.x + (xSpan + 1), t.y - (xSpan + 2));
				var edge = v.x - v.y;
				return edge > acc ? edge : acc;
			}, -999);
			var topEdge = centers.reduce (function (acc, t) {
				var v = new Point2d (t.x - ySpan, t.y - (ySpan + 3)); // and then also +3 offset since a lot of tiles are at height -1 :V
				var edge = v.x + v.y;
				return edge < acc ? edge : acc;
			}, 999);
			var bottomEdge = centers.reduce (function (acc, t) {
				var v = new Point2d (t.x + (ySpan + 1), t.y + (ySpan + 3)); // +3 offset since the tops of tiles could be sticking up
				var edge = v.x + v.y;
				return edge > acc ? edge : acc;
			}, -999);

			// these values would be the difference b/t the x and the y components, since that's what defines a vertical line in this coordinate system. e.g., 0 is the line going down along 0,0 1,1 2,2 3,3 etc. this goes down to the left and up to the right.
			this.leftEdge = leftEdge;
			this.rightEdge = rightEdge;

			// these values would be the sum of the x and y components, since that's what defines a horizontal line. 0 would be the line going across along 1,-1 0,0 -1,1 -2,2 etc. this goes down going up and up going down.
			this.topEdge = topEdge;
			this.bottomEdge = bottomEdge;
			/*
			console.log ("recalculateBounding: center @ " + centers[0].x + "," + centers[0].y);
			if (centers.length > 1) {
				console.log ("  other center @ " + centers[1].x + "," + centers[1].y);
			}
			console.log ("xSpan: " + xSpan + "; ySpan: " + ySpan);
			console.log ("left: " + this.leftEdge + "; right: " + this.rightEdge);
			console.log ("top: " + this.topEdge + "; bottom: " + this.bottomEdge);
			*/
		}
	, inBounds: function (x, y, h) {
			var bounds = this.calcBounds (x, y, h);
			if (bounds.horiz >= this.leftEdge && bounds.horiz < this.rightEdge &&
					bounds.vert >= this.topEdge && bounds.vert < this.bottomEdge) {
				return true;
			}
			return false;
		}
	, calcBounds: function (x, y, h) {
			return {horiz: x - y, vert: x + y - h};
		}
	, tile: function (x, y, h) {
			if (h === undefined) {
				h = 0;
			}
			return new Point2d ((x - y) * 32, (x + y) * 16 - h * 16);
		}
	, obj: function (x, y, h) {
			var i = this.tile (x, y, h);
			i.y -= 16;
			return i;
		}
	, fromPxRaw: function (x, y) {
			/* equation is
				X = (x - y) * 32 = (x * 32 + y * -32)
				Y = (x + y) * 16 = (x * 16 + y *  16)

					[ 32 -32 | X ]
					[ 16  16 | Y ]

				subtract Y*2 from X to get Y':

				(x * 32 + y *  32) = Y * 2
				(x *  0 + y * -64) = X - (Y * 2) = Y'

				Y' is equal to y * -64, so just divide it by -64...

				y = Y' / -64

				and now we can solve X to get x

				x = (X + (y * 32)) / 32
			*/
			function gaussian (screenX, screenY) {
				var y2 = screenY * 2;
				var y_ = screenX - y2;
				var coordY = y_ / -64;
				var coordX = (screenX + (coordY * 32)) / 32;
				return (new Point2d (coordX, coordY));
			}
			return gaussian (x - this.camera.x, y - this.camera.y);
		}
	, fromPx: function (x, y) {
			var coords = this.fromPxRaw (x, y);
			coords.x = Math.round(coords.x) - 2;
			coords.y = Math.round(coords.y) - 1;
			return coords;
		}
	, renderCanvas: function (data, spritesheet) {
			console.log ("RENDERING CANVAS");
			this.context.fillRect (0, 0, this.canvas.width, this.canvas.height);

			this.beginDraw ("map");
			this.renderTiles (this, spritesheet, data.edgeLength, data.tiles);
			this.doneDraw ();
		}
	, setCallbacks: function (callbacks) {
			if (this.callbacks.moveCallback !== null || this.callbacks.clickCallback !== null) {
				throw new Error ("attempted to set callbacks twice");
			}
			this.callbacks = callbacks;
			this.canvas.addEventListener ("mousemove", this.callbacks.moveCallback, false);
			this.canvas.addEventListener ("click", this.callbacks.clickCallback, false);
		}
	, destroy: function () {
			this.canvas.removeEventListener ("mousemove", this.callbacks.moveCallback, false);
			this.canvas.removeEventListener ("click", this.callbacks.clickCallback, false);
		}
	};



function Zone (id) {
	this.id = id;
	this.size = 0;
	this.subs = [];
	this.adj = [];
	this.spec = null;
}
Zone.prototype =
	{ add: function (sub) {
			this.size++;
			this.subs.push (sub);
		}
	, mapSubs: function (f) {
			return this.subs.map (f);
		}
	, addAdjacency: function (zoneId, where, to) {
			this.adj.push ({adjZone: zoneId, on: where, to: to});
		}
	, mapAdjacencies: function (f) {
			return this.adj.map (f);
		}
	, disconnect: function (zoneId, where, to) {
			this.adj = this.adj.filter (function (adj) {
				return !(adj.adjZone === zoneId && adj.on === where && adj.to === to);
			});
		}
	, contains: function (sub) {
			return this.subs.indexOf(sub) !== -1;
		}
	// touchesZone will return the FIRST connection; there may be more
	, touchesZone: function (zoneId) {
			var m = find (this.adj, function (a) {
				return a.adjZone === zoneId;
			});
			if (m) {
				return m.on;
			}
			return null;
		}
	// isConnectedTo will return the FIRST connection; there may be more
	, isConnectedTo: function (sub) {
			var m = find (this.adj, function (a) {
				return a.on === sub;
			});
			if (m) {
				return m.to;
			}
			return null;
		}
	, calcSpec: function (dim) {
			var potentials = this.deadEnds(dim);
			this.spec = potentials.length > 0 ? randomFrom (potentials) : null;
		}
	, deadEnds: function (dim) {
			var this_ = this;
			var pos;
			if (this.subs.length === 1 && this.adj.length === 1) {
				return [this.subs[0]];
			}
			pos = this.subs.filter (function (s) {
				return this_.isConnectedTo (s) === null; // subs that 1. don't have a connection outside the zone
			})/*.filter (function (s) {
				// FIXME: index overrun for top/left sectors
				var coord = unlinearize (dim, s);
				var adjs =
					[ linearize (dim, coord.x+1, coord.y)
					, linearize (dim, coord.x, coord.y+1)
					, linearize (dim, coord.x-1, coord.y)
					, linearize (dim, coord.x, coord.y-1)
					].filter (function (adjI) {
						return find (this_.subs, function (t) {
							return t === adjI;
						}) !== undefined;
					});
				return adjs.length <= 1; // and 2. only have one adjacent internal sub
			});*/
			return pos;
		}
	, merge: function (zone) {
			var this_ = this;
			this.subs = this.subs.concat (zone.subs);
			this.adj = this.adj.concat (zone.subs).filter(function (adj) {
				return this_.id !== adj.adjZone && zone.id !== adj.adjZone; // filter out connections that are to the merged zone
			});
			this.size = this.subs.length;
			zone.destroy();
		}
	};

// 'chunk': a section region; there are dim*dim in a map
// 'zone': a set of connected chunks
function makeChunkZones (dimension, maxZoneSize) {
	var i;
	var coord;
	var prevLeftId;
	var prevLeft;
	var prevUpId;
	var prevUp;

	var size = dimension * dimension;

	var zones = [];
	var graph = new Graph ();
	var chunks = Array (size);
	fillArray (chunks, null);

	for (i = 0; i < size; i++) {
		coord = unlinearize (dimension, i);
		prevLeftId = linearize (dimension, coord.x-1, coord.y);
		prevLeft = coord.x <= 0 ? null : chunks[prevLeftId];
		prevUpId = linearize (dimension, coord.x, coord.y-1);
		prevUp = coord.y <= 0 ? null : chunks[prevUpId];
		
		r = randomInRange (0, 1);
		if (prevLeft !== null && prevLeft.root.size < maxZoneSize && r < 0.5) {
			chunks[i] = {id: i, root: prevLeft.root};
		} else if (prevUp !== null && prevUp.root.size < maxZoneSize && r >= 0.5) {
			chunks[i] = {id: i, root: prevUp.root};
		} else {
			chunks[i] = new Zone(i);
			chunks[i].root = chunks[i];
			graph.addNode (chunks[i]);
			zones.push (chunks[i]);
		}
		if (prevLeft !== null && chunks[i].root.id !== prevLeft.root.id) {
			graph.addEdge (graph.getNodeId(chunks[i].root), graph.getNodeId(prevLeft.root), { a: prevLeftId, b: i});
			//prevLeft.root.addAdjacency (chunks[i].root.id, i, prevLeftId);
			//chunks[i].root.addAdjacency (prevLeft.root.id, prevLeftId, i);
		}
		if (prevUp !== null && chunks[i].root.id !== prevUp.root.id) {
			graph.addEdge (graph.getNodeId(chunks[i].root), graph.getNodeId(prevUp.root), {a: prevUpId, b: i});
			//prevUp.root.addAdjacency (chunks[i].root.id, i, prevUpId);
			//chunks[i].root.addAdjacency (prevUp.root.id, prevUpId, i);
		}
		chunks[i].root.add(i);
	}

	// pruning chunk edges / setting graph edges
	var edgeWeights = function (l) {
		return 1; // all edges have the same weight
	}

	var es = graph.nodes.filter (function (n) {
		return n.subs.length === maxZoneSize;
	});
	es = randomShuffle (es).slice (0, 2);
	es = es.map (function (n) {
		return graph.minimumSpanningTree (edgeWeights, graph.getNodeId(n));
	});
	es = [].concat.apply ([], es); // join
	es = es.filter (function (v, i, self) {
		return self.indexOf(v) === i;
	});
	graph.resetEdges (es);

	graph.edges.map (function (e) {
		var n = graph.nodes[e.n];
		var m = graph.nodes[e.m];
		n.addAdjacency (m.id, e.label.b, e.label.a);
		m.addAdjacency (n.id, e.label.a, e.label.b);
	});

	return { zones: zones, chunks: chunks, graph: graph };
}
