/* from http://www.graviness.com/js/mt19937ar.c.js.phtml @ june 3 2017
 */
var Random = (function() {
	/* from
		https://stackoverflow.com/questions/7616461/generate-a-hash-from-string-in-javascript-jquery
		http://werxltd.com/wp/2010/05/13/javascript-implementation-of-javas-string-hashcode-method/
			@ june 25 2017
	*/
	function hashString (string) {
		var hash = 0, i, chr;
		if (string.length === 0) return hash;
		for (i = 0; i < string.length; i++) {
			chr   = string.charCodeAt(i);
			hash  = ((hash << 5) - hash) + chr;
			hash |= 0; // Convert to 32bit integer
		}
		return hash;
	}

	MATRIX_A = 0x9908b0df;   /* constant vector a */
	UPPER_MASK = 0x80000000; /* most significant w-r bits */
	LOWER_MASK = 0x7fffffff; /* least significant r bits */
	function RandomGenerator (seed) {
		//console.log ("internal seed (a): " + seed);
		if ((seed|0) === 0 && !(seed === 0)) {
			//console.log ("rehashing, b/c |0 = " + (seed|0) + "; eq 0: " + ((seed|0) === 0) + "; neq 0: " + (!(seed === 0)));
			seed = hashString (seed);
		}
		//console.log ("internal seed (b): " + seed);
		this.N = 624;
		this.M = 397;

		this.mt = new Array(this.N); /* the array for the state vector */
		this.mti = this.N+1; /* mti==N+1 means mt[N] is not initialized */

		this.mt[0]= seed >>> 0;
		for (this.mti = 1; this.mti < this.N; this.mti++) {
			var s = this.mt[this.mti-1] ^ (this.mt[this.mti-1] >>> 30);
			this.mt[this.mti] = (((((s & 0xffff0000) >>> 16) * 1812433253) << 16) + (s & 0x0000ffff) * 1812433253)
				+ this.mti;
			// See Knuth TAOCP Vol2. 3rd Ed. P.106 for multiplier. 
			// In the previous versions, MSBs of the seed affect   
			// only MSBs of the array mt[].                        
			// 2002/01/09 modified by Makoto Matsumoto             
			this.mt[this.mti] >>>= 0;
			// for >32 bit machines
		}
	}
	RandomGenerator.prototype =
		{ next_i32: function () {
				var y;
				var mag01 = new Array(0x0, MATRIX_A);
				// mag01[x] = x * MATRIX_A  for x=0,1 

				if (this.mti >= this.N) { // generate N words at one time
					var kk;
					if (this.mti == this.N+1)   // if init_genrand() has not been called, 
						init_genrand(5489); // a default initial seed is used 

					for (kk=0; kk<this.N-this.M; kk++) {
						y = (this.mt[kk]&UPPER_MASK)|(this.mt[kk+1]&LOWER_MASK);
						this.mt[kk] = this.mt[kk+this.M] ^ (y >>> 1) ^ mag01[y & 0x1];
					}
					for (; kk<this.N-1; kk++) {
						y = (this.mt[kk]&UPPER_MASK)|(this.mt[kk+1]&LOWER_MASK);
						this.mt[kk] = this.mt[kk+(this.M-this.N)] ^ (y >>> 1) ^ mag01[y & 0x1];
					}
					y = (this.mt[this.N-1]&UPPER_MASK)|(this.mt[0]&LOWER_MASK);
					this.mt[this.N-1] = this.mt[this.M-1] ^ (y >>> 1) ^ mag01[y & 0x1];

					this.mti = 0;
				}

				y = this.mt[this.mti++];

				// Tempering
				y ^= (y >>> 11);
				y ^= (y << 7) & 0x9d2c5680;
				y ^= (y << 15) & 0xefc60000;
				y ^= (y >>> 18);

				return y >>> 0;
			}
		, next: function () {
				return this.next_i32() * (1.0/4294967296.0);
				/* divided by 2^32 */
			}
		};

	return { Generator: RandomGenerator };
	}());
