var Skills = (function () {
	"use strict";
	/*
	attrs that skills would need:
		* what 'domain' do they run in (fighting, tracking, searching, thinking, etc)
		* whats their damage formula (or what stats does their formula use)
			* maybe skills don't have damage formula, but blocs do. so "hit" uses some phys damage formula that might happen to include "skill level"
		* skill description flavor text
		* what level are they

	skill-related information:
		* how is it gained or lost (what body sequences are required; what are forbidden)


uh fundamental domains that i can think of right now:
	fighting
	searching
	thinking
	talking??
	*/



	function and (vs) {
		return vs.reduce (function (memo, n) { return memo && n; }, true);
	}
	function or (vs) {
		return vs.reduce (function (memo, n) { return memo || n; }, false);
	}


	function Skill (name, description, domain, actionSequence, xp, requires, forbids) {
		this.name = name;
		this.description = description;
		this.domain = domain;

		this.requires = requires;
		this.forbids = forbids;
		//console.log ("for \"" + name + "\"; requires: " + this.requires + " // forbids: " + this.forbids);

		if (xp) {
			this.level = xp.level;
			this.xp = xp.xp;
		} else {
			this.level = 1;
			this.xp = 0;
		}

		this.actionSequence = actionSequence;
	}
	Skill.prototype =
		{ printDOM: function () {
				var c, i, lvlBar, lvlNode, lvlNumber;
				var fromStr;
				var actContainer;
				c = document.createElement ("li");
				c.classList.add ("skill");

				// background xp shadow
				lvlBar = document.createElement ("span");
				lvlBar.classList.add ("xpbar");
				lvlBar.style.width = Math.round ((this.xp / this.xpTNL()) * 100) + "%";
				c.appendChild (lvlBar);

				// skill name
				c.appendChild (spanLabel ("name", this.name));

				// big level number
				lvlNode = document.createElement ("span");
				lvlNode.classList.add ("level");
				lvlNode.appendChild (document.createTextNode ("lv"));
				lvlNumber = spanLabel ("number", this.level);
				lvlNumber.classList.add ("lvl" + this.level);
				lvlNode.appendChild (lvlNumber);
				c.appendChild (lvlNode);

				// skill blocs (hit hit wait, etc)
				actContainer = document.createElement ("ol");
				actContainer.classList.add ("acts");
				actContainer.classList.add ("bloc");
				this.actionSequence.map(function (act) {
					var b = document.createElement ("li");
					b.classList.add (act);
					b.appendChild (document.createTextNode (act));
					actContainer.appendChild(b);
				});
				c.appendChild (actContainer);

				// what nodes grant this skill
				//console.log (this.requires);

				fromStr = (function (req) {
					return req.split (" ").filter (function (c) {
							return c !== ">";
						}).map (function (c) {
							if (c === "+") {
								return c;
							} else {
								return new Body.Part (c, []).nameVal();
							}
						}).join (" ");
				})(this.matchedRequires);
				c.appendChild (spanLabel ("from", fromStr));
				// skill description
				c.appendChild (spanLabel ("desc", this.description));
				return c;
			}
		, xpTNL: function () {
				// NOTE: FILLER XP/LEVEL EQUATION
				return (this.level + 2) * (this.level + 2);
			}
		, gainXp: function (xp) {
				var levelsGained = 0;
				this.xp += xp;
				while (this.xp > this.xpTNL()) {
					this.xp -= this.xpTNL();
					this.level += 1;
					levelsGained++;
				}
				return levelsGained;
			}
		, forChallenge: function () {
				var importantProps =
					{ name: this.name
					, level: this.level
					};
				return new Challenge.Action (importantProps, this.actionSequence);
			}
		, fitsOnBody: function (body) {
			/* expects trees in the form of `{value: v, children: [x,y,z]}`
			 * returns true if the tree contains a sequential path of the values in `path`
			 * or false otherwise.
			 */
			/*
			has a css-style selector. supports " " (any-length descent), ">" (immediate descent) and "+" (adjacent siblings). chunks are space separated (e.g., "foo>bar" will not work; "foo > bar" will)
			returns true if the selector can fully match any part of the tree and false otherwise

			in practice we can make our descent a little simpler than is necessary, since we can be sure there are never any duplicate parent/child nodes. so no node sequences like "a > b" that might show up in the tree like "a a b". basically once we hit a node, we can split off that part of the path and be sure we're not shooting ourselves in the foot. we ALSO know there aren't gonna be duplicate nodes, so if we have `a + b` and we test that against a list of children and have two matches, we know we got one match for `a` and one for `b`.
			*/
			function hasSelector (tree, sel) {
				function matchChunks (tree, cs) {
					var c;
					var mchs;
					if (cs.length === 0) {
						return true;
					}
					c = cs[0];
					//console.log ("tree: " + JSON.stringify (tree) + ";  cs:" + JSON.stringify (cs));
					switch (c.spec) {
						case ">":
							return or (tree.children.filter (function (ch) {
								return ch.value === c.n;
							}).map (function (ch) {
								var next = cs.slice();
								next.shift();
								return matchChunks (ch, next);
							}));
						case "n":
							return or (tree.children.filter (function (ch) {
								return ch.value === c.n;
							}).map (function (ch) {
								var next = cs.slice();
								next.shift();
								return matchChunks (ch, next);
							}).concat (tree.children.map (function (ch) {
								return matchChunks (ch, cs);
							})));
						case "+":
						case "+n":
							mchs = tree.children.filter (function (ch) {
								return c.n.indexOf(ch.value) !== -1;
							});
							if (mchs.length === c.n.length) {
								var next = cs.slice();
								next.shift();
								return or (mchs.map (function (ch) {
									return matchChunks (ch, next);
								}));
							} else {
								return or (tree.children.map (function (ch) {
									return matchChunks (ch, cs);
								}));
							}
							return false;
					}
				}
				var chunks = sel.split(" ").reduce (function (memo, n) {
					switch (n) {
						case ">":
							memo.push ({ spec: ">", n: null});
							break;
						case "+":
							memo[memo.length-1].spec = "+";
							break;
						default:
							if (memo.length === 0) {
								memo.push ({ spec: "n", n: n});
							} else {
								switch (memo[memo.length-1].spec) {
									case "+":
										memo[memo.length-1].n = [].concat(memo[memo.length-1].n, n);
										memo[memo.length-1].spec = "+n";
										break;
									case ">":
										memo[memo.length-1].n = n;
										break;
									default:
										memo.push ({ spec: "n", n: n});
										break;
								}
							}
							break;
					}
					return memo;
				}, []);
				//console.log ("got \"" + JSON.stringify(chunks) + "\" from \"" + sel + "\"");
				return matchChunks ({value: "__root__", children: [tree]}, chunks);
			}
			var matchedReqs = this.requires.filter (function (path) { return hasSelector (body, path); } );
			this.matchedRequires = matchedReqs.length === 0 ? null : matchedReqs[0];
			return matchedReqs.length !== 0 &&
				and (this.forbids.map (function (path) {return !hasSelector (body, path); }));
			}
		};

	var anyLimb =
		[ "animum arms"
		, "animum legs"
		, "animum tail"
		, "animum head"
		, "animum > wings"
		];

	var skills =
		[ new Skill ("pulse", "a heartbeat from the soul"
			, "fighting", ["wait", "wait", "cast"]
			, null
			, ["animum"]
			, anyLimb
			)
		, new Skill ("smack", "ectoplasmic collision"
			, "fighting", ["wait", "hit", "wait"]
			, null
			, ["animum"]
			, anyLimb
			)
		, new Skill ("bleat", "[goat noises]"
			, "fighting", ["wait", "wait", "cast"]
			, null
			, ["head goat", "head + goat"]
			, [] // should be: "head animal!goat"
			)
		, new Skill ("hiss", "ssssssssss"
			, "fighting", ["wait", "cast", "wait"]
			, null
			, ["head snake", "head + snake"]
			, [] // see above
			)
		, new Skill ("tear", "bite down hard"
			, "fighting", ["charge", "hit", "dazed"]
			, null
			, ["head wolf", "head + wolf"]
			, [] // see above
			)
		, new Skill ("gore", "a stab with your horn(s)"
			, "fighting", ["wait", "charge", "hit"]
			, null
			, ["head horns"]
			, []
			)
		, new Skill ("dazzle", "light reflecting off your horn"
			, "fighting", ["reflect", "parry", "wait"]
			, null
			, ["unicorn + head horns", "head unicorn + horns"]
			, []
			)
		, new Skill ("dream", "..."
			, "fighting", ["dazed", "charge", "cast"]
			, null
			, ["head"]
			, ["arms", "legs"]
			)
		, new Skill ("lash", "..."
			, "fighting", ["hit", "wait", "wait"]
			, null
			, ["tail snake", "tail + snake"]
			, []
			)
		, new Skill ("constrict", "..."
			, "fighting", ["hit", "block", "block"]
			, null
			, ["tail snake", "tail + snake"]
			, ["legs"]
			)
		, new Skill ("kick", "..."
			, "fighting", ["wait", "wait", "hit"]
			, null
			, ["legs"]
			, []
			)
		, new Skill ("punch", "..."
			, "fighting", ["wait", "hit", "wait"]
			, null
			, ["arms"]
			, []
			)
		, new Skill ("claw", "..."
			, "fighting", ["wait", "hit", "wait"]
			, null
			, ["arms wolf", "legs wolf", "arms + wolf", "legs + wolf"]
			, []
			)
		, new Skill ("stomp", "a stomp w/ yr hooves"
			, "fighting", ["wait", "wait", "hit", "hit", "dazed"]
			, null
			, ["unicorn + legs more", "bull + legs more", "goat + legs more"
				,"legs unicorn + more", "legs bull + more", "legs goat + more"
				]
			, []
			)
		, new Skill ("dbl punch", "..."
			, "fighting", ["wait", "hit", "hit", "dazed", "dazed"]
			, null
			, ["arms more"]
			, []
			)
		, new Skill ("whip", "..."
			, "fighting", ["parry", "hit", "wait"]
			, null
			, ["chain"]
			, []
			)
		];

	function getSkill (name) {
		var f = _.filter (skills, function (s) { return s.name === name; });
		if (f.length > 0) { return f[0]; }
		return null;
	}

	// returns a list of skills
	function availableSkillsByBody (body) {
		//console.log ("filtering skills using body:");
		//console.log (body);
		return skills.filter (function (skill) { return skill.fitsOnBody (body); });
	}


	return (
		{ Skill: Skill
		, allSkills: skills
		, get: getSkill
		, availableSkillsByBody: availableSkillsByBody
		});
})();
