var Stats = (function() {

	function Stats (rawStats) {
		this.str = rawStats.str;
		this.vit = rawStats.vit;
		this.def = rawStats.def;
		this.spd = rawStats.spd;

		this.int_ = rawStats.int_;
		this.atn = rawStats.atn;
		this.res = rawStats.res;
		this.cha = rawStats.cha;
		this.hp = {cur: 0, max: 0};
		this.mp = {cur: 0, max: 0};

		this.recalculateSecondaries(true);
	}
	Stats.prototype =
		{ mconcat: function (ss, heal) {
				var i;
				for (i = 0; i < ss.length; i++) {
					this.str += ss[i].str;
					this.vit += ss[i].vit;
					this.def += ss[i].def;
					this.spd += ss[i].spd;
					this.int_ += ss[i].int_;
					this.atn += ss[i].atn;
					this.res += ss[i].res;
					this.cha += ss[i].cha;
				}
				this.recalculateSecondaries(heal === undefined ? false : heal);
			}
		, mempty: function () {
				return new Stats ({str: 0, vit: 0, def: 0, spd: 0, int_: 0, atn: 0, res: 0, cha: 0});
			}
		, recalculateSecondaries: function (heal) {
				this.hp.max = Math.floor (this.vit * Math.sqrt(this.vit) * 3);
				this.mp.max = Math.floor (this.atn * Math.sqrt(this.atn) * 3);
				if (this.hp.cur > this.hp.max || heal) {
					this.hp.cur = this.hp.max;
				}
				if (this.mp.cur > this.mp.max || heal) {
					this.mp.cur = this.mp.max;
				}
			}
		, valueSpan: function (f, id, class_) {
				var v;
				v = document.createElement ("span");
				if (id !== undefined) {
					v.id = id;
				}
				if (class_ !== undefined) {
					v.classList.add (class_);
				}
				v.classList.add ("v");
				v.appendChild (document.createTextNode (f(this)));
				return v;
			}
		, domStat: function (stat) {
			var i = document.createElement ("li");
				i.appendChild (spanLabel ("n", stat));
				i.appendChild (this.valueSpan (function (s) {return stat === "int" ? s.int_ : s[stat];}));
				return i;
		}
		, asModifier: function () {
				var i = document.createElement ("ul");
				var l, s;
				var this_ = this;
				i.classList.add ("statModifier");
				["str", "def", "vit", "spd", "int_", "res", "atn", "cha"].map (function (attr) {
					var pattr = attr === "int_" ? "int" : attr
					if (this_[attr] !== 0) {
						l = document.createElement ("li");
						l.classList.add (pattr);
						if (this_[attr] < 0) {
							l.classList.add ("neg");
							s = document.createElement ("span");
							s.appendChild (document.createTextNode (this_[attr]));
							l.appendChild (s);
							l.appendChild (document.createTextNode (pattr));
						} else {
							l.classList.add ("pos");
							s = document.createElement ("span");
							s.appendChild (document.createTextNode ("+" + this_[attr]));
							l.appendChild (s);
							l.appendChild (document.createTextNode (pattr));
						}
						i.appendChild (l);
					}
				});
				return i;
			}
		, domBlock: function () {
				var c, item, s, v, bar;
				c = document.createElement ("ul");
				c.id = "statlist";
				item = document.createElement ("li");
				item.id = "shp";
				item.appendChild (spanLabel ("n", "soul hp "));
				v = document.createElement ("span");
				v.appendChild (document.createTextNode ("("));
				v.appendChild (this.valueSpan (function (s) {return s.hp.cur;}, "chp"));
				v.appendChild (document.createTextNode ("/"));
				v.appendChild (this.valueSpan (function (s) {return s.hp.max;}, "mhp"));
				v.appendChild (document.createTextNode (")"));
				item.appendChild (v);

				fullwidth = Math.max (this.hp.max, this.mp.max);
				chpWidth = this.hp.cur / this.hp.max;
				mhpWidth = this.hp.max / fullwidth;
				cmpWidth = this.mp.cur / this.mp.max;
				mmpWidth = this.mp.max / fullwidth;
				bar = document.createElement ("span");
				bar.id = "shpBar";
				bar.classList.add ("bar");
				bar.style.width = Math.round (mhpWidth * 100) + "%";
				barInner = document.createElement ("span");
				barInner.classList.add ("innerBar");
				barInner.style.width = Math.round (chpWidth * 100) + "%";
				bar.appendChild (barInner);
				item.appendChild (bar);

				c.appendChild (item);

				item = document.createElement ("li");
				item.id = "smp";

				bar = document.createElement ("span");
				bar.id = "smpBar";
				bar.classList.add ("bar");
				bar.style.width = Math.round (mmpWidth * 100) + "%";
				barInner = document.createElement ("span");
				barInner.classList.add ("innerBar");
				barInner.style.width = Math.round (cmpWidth * 100) + "%";
				bar.appendChild (barInner);
				item.appendChild (bar);

				v = document.createElement ("span");
				v.appendChild (document.createTextNode ("("));
				v.appendChild (this.valueSpan (function (s) {return s.mp.cur;}, "cmp"));
				v.appendChild (document.createTextNode ("/"));
				v.appendChild (this.valueSpan (function (s) {return s.mp.max;}, "mmp"));
				v.appendChild (document.createTextNode (")"));
				item.appendChild (v);
				item.appendChild (spanLabel ("n", " soul mp"));
				c.appendChild (item);

				s = document.createElement ("section");
				s.classList.add ("phys");
				s.appendChild (this.domStat ("str"));
				s.appendChild (this.domStat ("spd"));
				s.appendChild (this.domStat ("def"));
				s.appendChild (this.domStat ("vit"));
				c.appendChild (s);

				s = document.createElement ("section");
				s.classList.add ("mag");
				s.appendChild (this.domStat ("int"));
				s.appendChild (this.domStat ("cha"));
				s.appendChild (this.domStat ("res"));
				s.appendChild (this.domStat ("atn"));
				c.appendChild (s);

				return c;
			}
		};

	function partStats (part) {
		switch (part) {
			case "animum":
				return (new Stats ({str: 3, vit: 3, def: 3, spd: 3, int_: 3, atn: 3, res: 3, cha: 3}));

			case "legs":
				return (new Stats ({str: 0, vit: 2, def: 0, spd: 2, int_: 0, atn: 0, res: 0, cha: 0}));
			case "arms":
				return (new Stats ({str: 2, vit: 0, def: 0, spd: 2, int_: 0, atn: 0, res: 0, cha: 0}));
			case "head":
				return (new Stats ({str: 0, vit: 0, def: 0, spd: 0, int_: 3, atn: 2, res: 1, cha: 1}));
			case "horns":
				return (new Stats ({str: 1, vit: 0, def: 0, spd: 0, int_: 0, atn: 1, res: 0, cha: 1}));
			case "tail":
				return (new Stats ({str: 0, vit: 0, def: 1, spd: 1, int_: 0, atn: 0, res: 1, cha: 1}));

			case "more":
				return (new Stats ({str: 1, vit: 1, def: 0, spd: 0, int_: 1, atn: 1, res: 0, cha: 0}));

			case "chain":
				return (new Stats ({str: 4, vit: 0, def: 0, spd: 0, int_: 1, atn: 0, res: 0, cha: 0}));
			case "sword":
				return (new Stats ({str: 5, vit: 0, def: 0, spd: 0, int_: 0, atn: 0, res: 0, cha: 0}));
			case "bow":
				return (new Stats ({str: 3, vit: 0, def: 0, spd: 2, int_: 0, atn: 0, res: 0, cha: 0}));

			case "cock":
				return (new Stats ({str: 0, vit: 1, def: 0, spd: 0, int_: 0, atn: 0, res: 0, cha: 1}));
			case "balls":
				return (new Stats ({str: 0, vit: 1, def: 0, spd: 0, int_: 0, atn: 0, res: 0, cha: 1}));
			case "knot":
				return (new Stats ({str: 0, vit: 1, def: 0, spd: 0, int_: 0, atn: 0, res: 0, cha: 1}));

			case "goat":
				return (new Stats ({str: 0, vit: 2, def: 0, spd: 0, int_: 0, atn: 3, res: 0, cha: 0}));
			case "snake":
				return (new Stats ({str: 0, vit: 0, def: 0, spd: 2, int_: 3, atn: 0, res: 0, cha: 0}));
			case "human":
				return (new Stats ({str: 0, vit: 1, def: 0, spd: 0, int_: 2, atn: 0, res: 0, cha: 2}));
			case "bull":
				return (new Stats ({str: 0, vit: 3, def: 2, spd: 0, int_: 0, atn: 0, res: 0, cha: 0}));
			case "unicorn":
				return (new Stats ({str: 0, vit: 0, def: 0, spd: 0, int_: 1, atn: 2, res: 2, cha: 0}));
			case "wolf":
				return (new Stats ({str: 3, vit: 2, def: 0, spd: 0, int_: 0, atn: 0, res: 0, cha: 0}));
			default:
				return Stats.prototype.mempty();
		}
	}
	function bstats (b) {
		var i, r, stats = [];
		stats.push (partStats (b.value));
		for (i = 0; i < b.children.length; i++) {
			stats = stats.concat (bstats (b.children[i]));
		}
		r = Stats.prototype.mempty();
		r.mconcat (stats, true);
		return r;
	}

	// TODO: this might also need targeting bits (all allies, one enemy, all enemies??) targeting in general is complex; b/c the combat is based off of the dark room interface the process of selecting 'one ally' actions is complex (we can say there's a target reticule that 'one enemy' uses, and all allies/all enemies actions ignore it and just autotarget. if yr 'one enemy' and 'one ally' targets share a reticle that's a problem, but it's really awkward to not do that. so: no single-target heals/buffs, & no equivalent to "cast cure on zombies to hurt them")
	// the main issue i guess is actually looking up the reticule: if we give pcs, npcs, and enemies the same attack functions, which calculate a target, they need a ~dependency injection~ to be able to find their target. (this damfunction assumes a target has already been found; the real code being used in combat.js does not, and returns an object with a target: value)
	// anyway the point is we basically have four/five hitfuncs: "all allies", "all enemies", "whatever the target reticule is on", and "one random ally/enemy"
	function Skill (name, description, from, damfunc, charge, cooldown, available, powered, damages, flags) {
		this.name = name;
		this.description = description;
		this.from = from;
		this.damfunc = damfunc;
		this.charge = charge;
		this.cooldown = cooldown;
		this.available = available;
		this.powered = powered;
		this.damages = damages;
		this.flags = flags;
		this.level = {level: 1, xp: 0};
	}
	Skill.prototype =
		{ domBlock: function () {
				var c, i, lvlBar, lvlNode, lvlNumber;
				c = document.createElement ("li");
				c.classList.add ("skill");
				lvlBar = document.createElement ("span");
				lvlBar.classList.add ("xpbar");
				lvlBar.style.width = Math.round ((this.level.xp / this.xpNextLevel()) * 100) + "%";
				c.appendChild (lvlBar);

				c.appendChild (spanLabel ("name", this.name));
				lvlNode = document.createElement ("span");
				lvlNode.classList.add ("level");
				lvlNode.appendChild (document.createTextNode ("lv"));
				lvlNumber = spanLabel ("number", this.level.level);
				lvlNumber.classList.add ("lvl" + this.level.level);
				lvlNode.appendChild (lvlNumber);
				c.appendChild (lvlNode);
				s = document.createElement ("span");
				s.classList.add ("typesection");
				for (i = 0; i < this.powered.length; i++) {
					s.appendChild (typeLabel (this.powered[i]));
				}
				s.appendChild (document.createTextNode (" -> "));
				for (i = 0; i < this.damages.length; i++) {
					s.appendChild (typeLabel (this.damages[i]));
				}
				c.appendChild (s);

				if (this.charge !== 0) {
					c.appendChild (spanLabel ("charge", "ch " + (this.charge / 1000) + "s"));
				}
				if (this.cooldown !== 0) {
					c.appendChild (spanLabel ("cooldown", "cld " + (this.cooldown / 1000) + "s"));
				}
				c.appendChild (spanLabel ("from", this.from));
				c.appendChild (spanLabel ("desc", this.description));
				return c;
			}
		, xpNextLevel: function () {
				// NOTE: FILLER XP/LEVEL EQUATION
				return (this.level.level + 2) * (this.level.level + 2);
			}
		, gainXp: function (xp) {
				this.level.xp += xp;
				while (this.level.xp > this.xpNextLevel()) {
					this.level.xp -= this.xpNextLevel();
					this.level.level += 1;
				}
			}
		};
	function typeLabel (t) {
		var label = document.createElement ("span");
		label.classList.add ("type");
		label.classList.add (slabel[t]);
		label.appendChild (document.createTextNode (slabel[t]));
		return label;
	}

	var StatTypes = (
		{ UNKNOWN: 0
		, STR: 1
		, SPD: 2
		, DEF: 3
		, VIT: 4
		, INT: 5
		, CHA: 6
		, RES: 7
		, ATN: 8
		, HP: 9
		, MP: 10
		});
	var slabel = {};
	slabel[StatTypes.STR] = "str";
	slabel[StatTypes.SPD] = "spd";
	slabel[StatTypes.DEF] = "def";
	slabel[StatTypes.VIT] = "vit";
	slabel[StatTypes.INT] = "int";
	slabel[StatTypes.CHA] = "cha";
	slabel[StatTypes.RES] = "res";
	slabel[StatTypes.ATN] = "atn";
	slabel[StatTypes.HP] = "hp";
	slabel[StatTypes.MP] = "mp";
	var Skillflags = (
		{
		});

	return (
		{ statsFromBody: bstats
		, partStats: partStats
		, Stats: Stats
		, Skill: Skill
		, Types: StatTypes
		});
}());